#include "abstract_supporting_log_p.h"
#include "const_return_codes.h"

ASupportingLogPAlg::ASupportingLogPAlg() {
	mpz_init(Modulus);
	mpz_init(Order);
	gmp_randinit_mt(RandomState);
}

ASupportingLogPAlg::~ASupportingLogPAlg() {
	mpz_clear(Modulus);
	mpz_clear(Order);
	gmp_randclear(RandomState);
}
/**
  Sets ASupportingLogPAlg::Modulus and clears ASupportingLogPAlg::Order. New Order should be supplied by SetOrder() or computed using ComputeOrder()
 */
int ASupportingLogPAlg::SetModulus(mpz_t aModulus) {
	mpz_set(Modulus,aModulus);
	mpz_set_ui(Order,0); 
}

int ASupportingLogPAlg::SetOrder(mpz_t aOrder) {
	mpz_set(Order,aOrder);
};

int ASupportingLogPAlg::ComputeOrder() {
	if (0 != mpz_probab_prime_p(Modulus,25)) {
		mpz_sub_ui(Order,Modulus,1);
		return ConstRC::Ok;
	} else {
		return ConstRC::NotSupported;
	};
};
