#ifndef __ABSTRACT_SUPPORTING_LOG_P_H_
#define __ABSTRACT_SUPPORTING_LOG_P_H_

#include "gmp.h"
#include "enums_common.h"
/*!
 \file abstract_supporting_factor_alg.h header file for common ancestor of basic DLP algorithms
 */


/*!
\brief An abstract class representing a supporting discrete logarithm algorithm for Z_p
*/

class ASupportingLogPAlg
{

public:
	ASupportingLogPAlg();
	virtual ~ASupportingLogPAlg();
	int SetModulus(mpz_t aModulus); ///< Sets modulus \f$n\f$ of \f$Z_n\f$
	int SetOrder(mpz_t aModulus); ///< Sets order of \f$Z_n^*\f$ so we do not have to compute it if known
	int ComputeOrder(); ///< in case the order is not known, we can try to compute it
	virtual int LogP(mpz_t aBase, mpz_t aValue, mpz_t aExponent) = 0; ///< Actual computation

protected:
	
	mpz_t Modulus; ///< We keep modulus as set so we can profit from computing several DL in the same group
	mpz_t Order; ///< We keep order as set so we can profit from computing several DL in the same group
	gmp_randstate_t RandomState; ///< state of random number generator
	
};

#endif // __ABSTRACT_SUPPORTING_LOG_P_H_
