#include "logarithm_trial.h"

#include "const_return_codes.h"
#include "common_utils.h"
#include <stdlib.h>
#include <iostream>

TrialLogP::TrialLogP() :ASupportingLogPAlg ()
{
	mpz_init(aux);
	max_tries = 10000;
}

TrialLogP::~TrialLogP() {
	mpz_clear(aux);
}

/*!
	computation
*/
int TrialLogP::LogP( mpz_t aBase, mpz_t aValue, mpz_t aExponent) {
	mpz_set(aux,aBase);
	for (unsigned long exp = 2; exp < max_tries; exp++) {
		mpz_mul(aux,aux,aBase);
		mpz_mod(aux,aux,Modulus);
		if (!mpz_cmp(aux,aValue)) {
			mpz_set_ui(aExponent,exp);
			return ConstRC::Ok;
		};
	};
	return ConstRC::NotFactorized;
}

/*!
	test functionality of TrialLogP	
*/
int TrialLogP::Test() {
	mpz_t modulo, base, exponent, value, result, limit, check;
	mpz_init(modulo);
	mpz_init(base);
	mpz_init(exponent);
	mpz_init(value);
	mpz_init(result);
	mpz_init(check);
	mpz_init(limit);
	mpz_set_str(limit,"10000",10);
	int tests = 100;
	int success = 0;
	int failure = 0;
	int error = 0;
	for(int i = 1;i<=tests; i++) {
		mpz_urandomm(modulo,RandomState,limit);
		mpz_nextprime(modulo,modulo);
		SetModulus(modulo);
		mpz_sub_ui(base,modulo,1);
		SetOrder(base);
		mpz_urandomm(exponent,RandomState,base);
		mpz_urandomm(base,RandomState,modulo);
		mpz_powm(value,base,exponent,modulo);
		int rvalue = LogP(base, value, result);
		if (rvalue == ConstRC::Ok) {
			mpz_powm(check,base,result, modulo);
			if (mpz_cmp(value,check)) {
				//gmp_printf("\n%Zd^%Zd %% %Zd = %Zd, ... %Zd ??",base,exponent,modulo,value,result);
//				gmp_fprintf(stderr,"E");
				error++;
			} else {
				success++;
//				gmp_fprintf(stderr,"S");
			};
		} else {
			failure++;
//			gmp_fprintf(stderr,"F");
		};
	};	
	std::cout << "\nTests: " << tests << "   Success: " << success << "    Failure: "<<failure << "    Error: "<< error;
	mpz_clear(modulo);
	mpz_clear(base);
	mpz_clear(exponent);
	mpz_clear(value);
	mpz_clear(result);
	mpz_clear(limit);
	mpz_clear(check);
};
