#ifndef __POLLARD_RHO_LOG_P_H_
#define __POLLARD_RHO_LOG_P_H_

#include "abstract_supporting_log_p.h"

#define DEFAULT_POLLARD_ENTRIES 100
/*!
	A basic type for representation of the intermediate values of iterations.
*/
typedef struct {
	mpz_t value;
	mpz_t basecount;
	mpz_t valuecount;
	int   inited;
} pollard_entry;


/*!
          \brief This class implements the Pollard Rho algorithm for computing DLP.
	  
	This class implements the Pollard Rho (a.k.a. Monte Carlo) factoring
algorithm for discrete logarithm problem over \f$Z_n\f$.
*/
class PollardRhoLogP : public ASupportingLogPAlg
{
public:
	PollardRhoLogP();
	~PollardRhoLogP();
	int LogP(mpz_t aBase, mpz_t aValue, mpz_t aExponent); ///< actual computation


	int Test(); ///< Test functionality of the DLP Pollard rho algorithm

	unsigned long max_restarts; ///< Limits number of restarts due to the order not being coprime to the difference of value exponents

private:
	pollard_entry*  pollard; ///< 	This array is used for storage of necessary iterated f(x) values.

	int  		pollard_maximal_index;	///< This variable contains length-related information for the main array.

	mpz_t   fx1; ///< Auxiliary variable for the value of the function
	mpz_t   bx1; ///< Auxiliary variable for the base count
	mpz_t   vx1; ///< Auxiliary variable for the value count
	mpz_t   fx2; ///< Auxiliary variable for the value of the function
	mpz_t   bx2; ///< Auxiliary variable for the base count
	mpz_t   vx2; ///< Auxiliary variable for the value count

	int  start_from;///< auxiliary variable - n from which iteration to start checks again (see Brent).

	unsigned long restarts; ///< Overall number of restarts for statistical purposes
};

#endif // __POLLARD_RHO_LOG_P_H_

