#ifndef __DEFINITIONS_H__
#define __DEFINITIONS_H__

#include <climits>
#include "common_definitions.h"
/*!
  \file 

  This file contains definitions of symbolic constants.
*/

/*! \brief Undef this to disable XML serialization and deserialization
 By undefing this constant, you will get a lightweight, no-XML version of MPQS/SIQS.
 No loading/saving of states really possible.*/
#define WITH_XML_SERIALIZATOR

/*! \def PATH_SEPARATOR
 \brief Path separator (OS dependent)
*/
#ifndef WIN32
#include <inttypes.h>
#include <unistd.h>
#define PATH_SEPARATOR '/'
#else
#define PATH_SEPARATOR '\\'
#endif

//! This must be defined in order to have time messages in QS (after each X polynomials a message concerning time expensiveness of tasks).
#define TIME_MESSAGE 

//! This must be defined in order to have time messages after block Lanczos run, concerning phases of Lanczos computation
#define LANCZOS_TIME_MESSAGE 
//! This must be defined in order to collect time expensiveness of matrix (BitMatrix and SparseMatrix) operations.
#define MATRIX_OPERATIONS_TIME_MESSAGE

#define DEFAULT_INTERMEDIATE_ROOTS  200

// delimiters for MPQS/SIQS command line options
#define MAX_POLY 	  1000000000
#define MAX_OMIT 	  1000
#define IMPLICIT_FACTORBASE 2000
#define IMPLICIT_SIEVE_INT  10000
#define IMPLICIT_TMESS 	    100
#define IMPLICIT_POLY       1000000000
#define MAXIMAL_LINE_SIEVE_HALFLINE 100000000
#define MAXIMAL_FACTOR_BASE_UPPER_BOUND 100000000


#define INCONSISTENT_STATE -10

//#define CHECK_SPARSE_MATRICES
#define RANK_COMPUTATIONS

//! QS version number
#define QS_VERSION     		2.99

#define TRUE        		1
#define FALSE       		0
#define _LOG_FACTOR     	0x1
#define DEFAULT_LOG_BASE        4
#define SIEVE_ROUNDOFF_ERROR 	0x8
#define TEST_VALUE  		0x80
#define CHECK_LEVEL_STEP	0x04

// For Carrier-Wagstaff method of 'a' generation.
#define UNSUITABLE_FOR_SIQS     1
#define DEFAULT_MACHINE_BOUND_START	20
#define DEFAULT_MACHINE_BOUND_END   	55
#define CW_EVEN_INDEX_START	56
#define CW_EVEN_INDEX_SIZE	100
//#define CW_ODD_INDEX_START      57
//#define CW_ODD_INDEX_SIZE       60
#define TRIPLE_SIZE      	4
#define TRIPLES_LOG_FACTOR 	131072

// For sieving with prime powers (since 1.15)
#define DEFAULT_PRIME_POWERS_ALLOCATED 1000
#define DEFAULT_LIMIT_MULTIPLIER 10

// For Pollard Rho factoring method
#define DEFAULT_POLLARD_BLOCK_SIZE 50

// For Double Large Prime Variation
// 2^23 entries
//#define DEFAULT_CYCLE_TABLE_SIZE 8388608
//#define DEFAULT_CYCLE_TABLE_SIZE 524288*2
//#define DEFAULT_CYCLE_TABLE_SIZE 32768
//#define DEFAULT_CYCLE_TABLE_SIZE 1024
				  

// Masks for row_element.inited
#define ARG_INITED 	0x1
#define FAC_INITED 	0x2
#define REM_INITED 	0x4
#define LSV_INITED 	0x8
#define RSV_INITED 	0x10

// Other constants for row_element
#define NO_POLY_USED    -1

// Masks for abstractsieve and quadsieve.inited
#define N_INITED  	0x1000
#define kN_INITED 	0x2000
#define M_INITED  	0x4000
#define ROOT_INITED     0x10000
#define MPF_INITED      0x40000
#define F1_INITED       0x80000
#define F2_INITED       0x100000
#define STIME_MAIN_INITED 0x400000
#define STIME_L1_INITED   0x800000
#define STIME_L2_INITED   0x1000000
#define IDEAL_A_VALUE_INITED 0x2000000
#define MIN_C_VALUE_INITED   0x4000000
#define MAX_C_VALUE_INITED   0x8000000

// Masks for quadsieve.open
#define PARTIALS_OPEN   	0x1
#define SMOOTHS_OPEN    	0x2
#define ALL_PARTIALS_OPEN 	0x4
#define USABLE_PARTIALS_OPEN    0x8
#define TEMPORARY_OPEN 		0x10
#define JOB_FILE_OPEN		0x20
#define LAST_STATE_FILE_OPEN    0x40
#define CURRENT_STATE_FILE_OPEN 0x80

// Masks for quad_polynomial.inited
#define A_INITED        0x1
#define B_INITED        0x2
#define C_INITED        0x4
#define D_INITED        0x8
#define D2_INITED       0x10

// Masks for variation.inited
#define VAR_INITED      0x1

// Masks for auxiliary inited
#define AUX_1_INITED    0x1
#define AUX_2_INITED    0x2
#define AUX_3_INITED    0x4
#define AUX_4_INITED    0x8
#define AUX_5_INITED    0x10
#define AUX_A_INITED    0x20
#define AUX_B_INITED    0x40
#define AUX_C_INITED    0x80
#define AUX_D_INITED    0x100
#define	AUX_P_INITED    0x200
#define AUX_P_EXP_INITED 0x400
#define AUX_A_INV_INITED 0x800
#define AUX_K_INITED     0x1000
#define AUX_L_INITED     0x2000

// Masks for auxiliary - Shanks-Tonelli inited
#define ST_AUX_A_INITED           0x1
#define ST_AUX_A_INVERSE_INITED   0x2
#define ST_AUX_B_INITED           0x4
#define ST_AUX_R_INITED           0x8
#define ST_AUX_ROOT_INITED        0x10
#define ST_AUX_N_INITED           0x20
#define ST_AUX_P_MINUS_1_INITED   0x40
#define ST_AUX_P_INITED           0x80
#define ST_AUX_1_INITED           0x100
#define ST_AUX_B_EXP_2I_INITED    0x200
#define ST_AUX_NON_INITED         0x400

// Masks for WIB auxiliary variables
#define WIB_DIFF_INITED       0x1
#define WIB_RATIO_INITED      0x2
#define WIB_MUL_P_INITED      0x4
#define WIB_ARG_INITED        0x8
#define WIB_ARG_2_INITED      0x10
#define WIB_ARG_PLUS_INITED   0x20

// Error return codes - general
#define NOT_ENOUGH_MEMORY    -1
#define COULD_NOT_OPEN_FILE  -2
#define COULD_NOT_CLOSE_FILE -3
#define COULD_NOT_READ_FROM_NULL_STREAM -4
#define END_OF_FILE          -5
#define BAD_ARGUMENT         -6

// Further error codes can be found in connectivity_framework_definitions.h
// They are -500 and lower

// Modes of file access
//! File mode constant for read
#define READ                  0
//! File mode constant for write
#define WRITE                 1
//! File mode constant for append
#define APPEND                2

// Primes smaller than this constant will be generated if primefile not found. NormalMessage("   Reading primes from file ...",NO_NUMBER);
#define DEFAULT_PRIMEFILE_BOUND 10000000

// Error return codes - init environment etc.
#define CALLED_TOO_EARLY     -101
#define NOT_SUPPORTED        -102

// Quadratic sieve defaults
#define DEFAULT_EXTENT          1
#define DEFAULT_MARGIN          10
#define DEFAULT_VALUE_M         10000
#define DEFAULT_FB_BOUND        10000
#define DEFAULT_NR_POLYNOMIALS  1
#define DEFAULT_DIV_ALLOCATE    16
#define INIT_VALUE_FACTOR       0
#define INIT_VALUE_N		-1
#define INIT_VALUE_k		0


// Return values of RunMPQS()
//! Return value of RunMPQS
#define FACTORIZED              1
//! Return value of RunMPQS
#define PRIME_NUMBER            2
//! Return value of RunMPQS
#define DIVISIBLE_BY_SMALL_PRIME 3
//! Return value of RunMPQS
#define NOT_FACTORIZED          -2
//! Return value of RunMPQS
#define NOT_ENOUGH_SMOOTHS      -3
//! Return value of RunMPQS
#define ERROR_INIT_ENVIRONMENT  -4
//! Return value of RunMPQS
#define ERROR_SIEVING           -5
//! Return value of RunMPQS
#define ERROR_LINALG            -6
//! Return value of RunMPQS
#define TOO_SMALL_FACTORBASE    -7
//! Return value of RunMPQS
#define NO_DEPENDENCIES         -8
//! Return value of RunMPQS
#define ERROR_PROCESSING_RELATIONS -9
//! Return value of RunMPQS
#define EXCESSIVE_FACTOR_BASE   -10

// Return values of RunTTR()
#define TTR_OK                  0
#define TTR_NOT_OK              -1

// Checks for primality - number of rounds in Miller-Rabin test
#define MILLER_RABIN_ROUNDS_NORMAL 	15
#define MILLER_RABIN_ROUNDS_STRICT 	40

// Default file names
//! Default file name for saving partial relations
#define DEFAULT_PARTIAL_FILE_NAME "partials"
//! Default file name for saving smooth relations
#define DEFAULT_SMOOTH_FILE_NAME  "smooths"
//! Default file name for file with primes
#define DEFAULT_PRIMEFILE_NAME    "primefil"

// Since 2.00
#define DEFAULT_TEMPORARY_FILE_NAME "temp"
#define DEFAULT_ALL_PARTIALS_FILE_NAME "all_partials"
#define DEFAULT_USABLE_PARTIALS_FILE_NAME "usable_partials"
// Since 2.02
#define DEFAULT_JOB_FILE_NAME "job"
#define DEFAULT_LAST_STATE_FILE_NAME "last_state"
#define DEFAULT_CURRENT_STATE_FILE_NAME "current_state"
#define DEFAULT_CYCLE_ARRAY_ENTRIES 15
#define DEFAULT_ROOT_LIST           20

// Default reallocation steps
#define SMOOTH_REALLOCATION_STEP  	100
#define USELESS_PRIME_REALLOCATION_STEP 200
#define USELESS_ROW_REALLOCATION_STEP   200

// SIQS defaults (added in 1.09)
#define DEFAULT_S		8
#define DEFAULT_Q_SHIFT         2
#define DEFAULT_LEAST_SIQS_DIVISOR	1024

// Variation-related constants
#define VAR_INITED                0x1
#define NO_INDEX                  -1
#define VARIATION_FACTOR          128

// for normal message
#define NO_NUMBER                 -1

// For allocating polynomials
#define DEFAULT_POLYNOMIALS_ALLOCATED   128
#define DEFAULT_TIME_MESSAGE_INTERVAL   100

// Rough mode settings - from this prime rough mode runs
#define ROUGH_MODE_LOWER_BOUND    100
#define DEFAULT_ROUGH_INDEX	  15

// Sieving matrix - how much divisors are allocated at the beginning
#define DIV_ALLOCATE    	  8

// Value ENOUGH is returned if enough smooth values have been generated
#define NOT_ENOUGH                -1
#define ENOUGH                    1
#define ENOUGH_BUT_WARNING	  2

// Quadratic equation pathologies
#define TRIVIAL_EQUATION          1
#define NOT_SUITABLE_MODULUS      -1
#define INSOLUBLE_EQUATION        -2

// Solutions backup
#define THERE                      1
#define BACK                       2

// Levels of MPQS execution
#define MAIN_LEVEL                 0
#define LEVEL_1                    1
#define LEVEL_2                    2

// Constants connected with variations
#define NONE                       0
#define LEFT                       1
#define RIGHT                      2
#define VAR_INITED                 0x1

// Constant connected with hashes
#define HASH_INITED                0x1000

// Return values of CheckNulity
#define ZERO           		   0
#define NONZERO                    1

// Return value of FindVariationInFactorBase, meaning failure
#define NOT_FOUND                  -10
#define FOUND                      10

// Masks for integer solutions, added in 1.04
#define ALLOCATED_1_SOLS           0x1
#define ALLOCATED_2_SOLS           0x2
#define ALLOCATED_USABLE_SOLS      0x4

#define VALID_1                    0x1
#define VALID_2                    0x2

// Bit Matrix - some return values
#define EXCESSIVE_ROW_INDEX        -10
#define EXCESSIVE_COLUMN_INDEX     -20
#define NEGATIVE_INDEX             -25
#define NULL_POINTER_SUPPLIED      -30
#define SIZE_MISMATCH              -40
#define GENERAL_ERROR              0xffffffff

// Sparse Matrix - some macros
#define SPARSE_REALLOCATION_STEP   10
#define ALLOCATED(x) SPARSE_REALLOCATION_STEP*((((x)+2)/SPARSE_REALLOCATION_STEP)+1)

// Large Prime Variation - reallocation in array of line numbers
#define LPV_LINE_REALLOCATION_STEP  4
#define LPV_ALLOCATED(x) LPV_LINE_REALLOCATION_STEP*((((x)+2)/LPV_LINE_REALLOCATION_STEP)+1)

// Large Prime Variation - reallocation in arrays
#define DEFAULT_INTERMEDIATE_ARRAY_ALLOCATE 100
#define DEFAULT_LPS_ARRAY_ALLOCATE_SINGLE   2
#define DEFAULT_LPS_ARRAY_ALLOCATE_DOUBLE   40

#ifdef _INTTYPES_H // getting typedefs from inttypes.h
typedef int64_t INT64;
#endif

#ifdef WIN32       // windows specific
typedef __int64 INT64;
#endif

#if defined(_UNISTD_H) || defined(__CYGWIN__)
#define SLEEP(x)	sleep((x))
#endif

//#ifdef WIN32
//#define SLEEP(x)	Sleep((x))
//#endif

/*! abnormal exit
	 = exit(x) for normal exit
	 = abort() for exit with coredump
*/
#if defined(_UNISTD_H) || defined(__CYGWIN__)
	#define ABNORMAL_EXIT(x) abort()
#else
	#define ABNORMAL_EXIT(x) exit(x)
#endif

// #define HASH_MASK                  0x7ffff
#define NONSINGLETON_MASK       0x80000000

// For Large Singleton Removal
// #define SINGLETON               0
// #define NONSINGLETON            1
// #define FRESH_SINGLETON         2

// For finding cycles
// #define ROOT_FLAG               0x7fffffff
// For combination of relation
#define SMOOTH_REL                  0
#define CYCLE                   1
// #define FINISHED                0xffffffff
#define PARITY_MASK		0x80000000

// For factorization of 2-partials
#define FACTOR_TOO_LARGE        -4

// Checkauto - constants
#define CHECKAUTO_MAXINDEX	3
#define PARTIAL_1_WEIGHT	5
#define PARTIAL_2_WEIGHT	1
#define RUNNING                 2
#define DEFAULT_INTERVAL_STEP   3500

// Value distribution
#define DISTRIBUTION_STEP       1000
#define MAX_DISTANCE            100
#define SMOOTH_VALUE		1
#define PARTIAL_1_VALUE		2
#define PARTIAL_2_VALUE		4

// CFRAC
#define DEFAULT_CFRAC_BASE_SIZE 250

// For factorization algorithms generally
#define OPTIMIZATION_LEVEL_0     0
#define OPTIMIZATION_LEVEL_1     1
#define OPTIMIZATION_LEVEL_2     2

#define QS_LINE_SIEVING
//#define GNFS_LINE_SIEVING

// An internal constant for the line siever.
#define MAX_UPDATES 500
#define MINIMAL_PRIME_TO_SIEVE_WITH 128

#define POSITIVE  1
#define NEGATIVE -1
#define CACHE_UPDATES 64
#define CHECK_MASK 0x80808080
#define UNLIMITED_SIEVING 0

// Text messages and their indices
#define GEN_NEW_POLY   10
#define GEN_NEW_POLY_TEXT "      Overall time spent in  GenerateNewPoly  is"

#define QUAD_MACHINE   20
#define QUAD_MACHINE_TEXT "      Overall time spent in  QuadraticMachine is"

#define PERF_SIEVING   30
#define PERF_SIEVING_TEXT "      Overall time spent in  PerformSievingPo is"

#define SIEVING_LOOP   40
#define SIEVING_LOOP_TEXT "                         Where  SievingLoopLogar"

#define REFILL_MAINM   50
#define REFILL_MAINM_TEXT "                         Where  RefillMainMatrix"

#define SORT_SMOOTHS   60
#define SORT_SMOOTHS_TEXT "                         Where  SortSmoothValues"

#define DIVISORS_OFA   70
#define DIVISORS_OFA_TEXT "                              Where Divisor Of A"

#define SMALL_FACTOR   80
#define SMALL_FACTOR_TEXT "                              Where Small Factor"

#define COUNT_CYCLES   90
#define COUNT_CYCLES_TEXT "                              Where Count Cycles"

#define BLOCK_SIEVES  100
#define BLOCK_SIEVES_TEXT "      Overall time spent in   Block Set Sieve is"

#define FILL_HSTABLE  110
#define FILL_HSTABLE_TEXT "      Overall time spent in   Fill Hashtables is"

#define FILL_HST_NEG  120
#define FILL_HST_NEG_TEXT "                          Where FillHasht-NEG is"

#define ROOTS_UPDATE  130
#define ROOTS_UPDATE_TEXT "      Overall time spent in   Root Update/New is"

#define FIND_FACTORZ  140
#define FIND_FACTORZ_TEXT "                          Where Find&FactorCa is"

#define UNKNOWN_PART_TEXT "      ! Unknown part ! "

#define FIRST_ROOT_VALID 0x10
#define SECOND_ROOT_VALID 0x20
#define ROOT_INVALID 0xffffffff

#define EQUAL 0
#define NOT_EQUAL 1

#define LIST_LOAD_FLAG 0x800000

#define COMPRESSED_FILE_EXTENSION ".z"
#define TERMINATED 100

// Relation processing flags
#define RELATIONS_READY_FLAG    		  0x1
#define LARGE_SINGLETONS_REMOVED_FLAG  		  0x2
#define CYCLES_CONSTRUCTED_FLAG	  		  0x4
#define RELATION_COLLECTION_FINISHED_FLAG 	  0x8
#define SIEVING_RUNNING_FLAG			  0x10

#define AUTOSAVE_INTERVAL_IN_SECONDS		30


#endif
