/*

   Hlavickovy soubor pro ruzne testy prvociselnosti.
   ================================================
   M. Kechlibar, unor 2003


   Plnokrevna verze je bezna C funkce, ktera si dava pozor na chyby typu "nelze otevrit
   soubor", stara se o vypisy eventualnich hlaseni apod. Pro ucely dalsiho pouziti v programovani
   je proto vhodnejsi, avsak pro studium algoritmu zcela nevhodna - kod osetrujici mozne chyby
   zpusobuje jeji necitelnost.

   Minimalisticka verze je nejuspornejsi mozny kod provadejici prislusny algoritmus. Je dobre
   citelna, avsak z hlediska absence vstupnich/vystupnich hlaseni a neosetrenych chyb, vyjimek ap.
   nepouzitelna pro serioznejsi programovani. Je urcena hlavne studentum, kteri si chteji
   osvojit algoritmus skryty za testem.

   Parametr "vypis": je-li zadan jako 1, nahlasi test svuj vysledek, napriklad v teto forme:
     "Miller-Rabin: testovane cislo je slozene a zjistili jsme to po 1 iteraci."
   Je-li "vypis" roven 0, jsou vsechna hlaseni na obrazovku s vyjimkou chybovych potlacena.
   Nastaveni 0 vyrazne setri pamet a urychluje beh programu.

   Parametr "initialize_random": je-li zadan jako 1, kazdy prubeh daneho testu si znovu
   inicializuje generator nahodnych cisel a po svem ukonceni alokovanou pamet uvolni. Tento
   stav je doporucen, pokud testujeme jedno velke cislo. Je-li zadan jako 0, spoleha se dany
   test na to, ze uz byla inicializovana globalni promenna "state". Tento stav je doporucen,
   kdyz pocitame s mnoha prubehy daneho testu, dejme tomu s milionem prubehu: pak totiz
   odpadavajici nutnost inicializovat generator usetri nejaky (nezanedbatelny) cas, ovsem
   na ukor toho, ze pred startem testu musime inicializovat globalni promennou.

 */
#ifndef __GMP_TESTS_HISTORIC_H__
#define __GMP_TESTS_HISTORIC_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gmp.h"
#include "time.h"
#include "gmp_messages.h"


/// globalni promenna pomahajici nastartovat nahodny generator 
extern gmp_randstate_t state;

/// pro testednum-1=r2^s, kde r liche, vraci s
unsigned long maximalni_mocnina_dvou(mpz_t testednum);

/// minimalisticke testy, k pedagogickym ucelum 
int pseudoprime_test_minimalistic(mpz_t testednum, int exactness);


int miller_rabin_test(mpz_t testednum, int exactness, int vypis, int initialize_random);
int solovay_strassen_test(mpz_t testednum,int exactness, int vypis, int initialize_random);
int trial_division_with_file(mpz_t testednum,char *name,unsigned long bound, int vypis, int initialize_random);
int trial_division_with_memory(mpz_t testednum,unsigned long bound, int vypis);


/* polokrevne testy, optimalizovane pro testovani cisel proslych nekolika */
/* koly zkusmeho deleni. Neobsahuji testovani vlozeneho cisla na koncovou */
/* cislici, nebot predpokladaji, ze toto jiz vyridilo zkusme deleni */
int pseudoprime_test_pure(mpz_t testednum, int exactness, int vypis, int initialize_random);
int miller_rabin_test_pure(mpz_t testednum, int exactness, int vypis, int initialize_random);
int solovay_strassen_test_pure(mpz_t testednum,int exactness, int vypis, int initialize_random);

/* vykona urcite mnozstvi zkusmych deleni a pak preda praci */
/* mez urcuje maximalni prvocislo, pro nez pripustime zkusme deleni */
/* prvni a druha funkce se lisi nasledovne: predpokladame-li, ze nebudeme testovat cisla
   vetsi nez dejme tomu 5-nasobek kvadratu cisla mez, je rychlejsi prvni, nebot vykonava
   test na to, zda testovane cislo neni pod kvadratem meze. pro obecne vetsi testovane hodnoty
   je lepsi druha funkce, ktera se s timto porovnanim nezatezuje */
int optimal_primality_test(mpz_t testednum,int exactness, int vypis, int initialize_random, int mez);
int optimal_primality_test_2(mpz_t testednum,int exactness, int vypis, int initialize_random, int mez);

/* memory-based trial division test, vraci v parametru delitel prvniho nalezeneho delitele */
int trial_division_return(long delitel, mpz_t testednum,unsigned long bound, int vypis);





#endif__GMP_TESTS_HISTORIC_H__
