#include "line_siever.h"

#ifdef QS_LINE_SIEVING
#include "mulm.h"
#endif

#ifdef GNFS_LINE_SIEVING
#include "integral_domain.h"
#endif

#include "log.h"

extern "C" {
#include "gmp_factor.h"
}

// Debugging constants; define to get debugging tests and messages;
//#define ONE_PRIME_DEBUG
// #define CACHE_FLUSHER_DEBUG
// #define BLOCK_SIEVE_DEBUG
// #define BLOCK_DEBUG
// #define SIQS_SHIFT_DEBUG
//#define FACTOR_DEBUG
//#define COUNT_HITS
// Let this value uncommented and set it to a problematic prime (demonstrating strange behavior) if needed.
#define CRITICAL_PRIME 0

#ifdef ONE_PRIME_DEBUG
bool isThisPrimeControlled(main_sieving_type aType)
{
	if (aType == 44687) return TRUE;
	/*if (aType == 44777) return TRUE;
	if (aType == 44809) return TRUE;*/
	return FALSE;
}
#endif

LineSiever::LineSiever()
{
	this->iLineSieveSize = ESizeUndefined;
	this->iBlockSize = 0;
	this->iFactorBaseUpperBound = 0;
	this->iBlock = NULL;
	this->iLineLowerBound = 0;
	this->iLineUpperBound = 0;
	this->iPositiveBlocks = 0;
	this->iNegativeBlocks = 0;
	this->iBlockSetSize = 0; /// Size of the set of blocks determined by upper 
	this->iAllocatedHashtableSize = 0;
	this->iInternalSieveState = EBeforeInit;
	this->iHashtable = NULL;
	this->iCaches = NULL;
	this->iMaxLines = UNLIMITED_SIEVING;
#ifdef GNFS_LINE_SIEVING	
	this->iAlgebraicHashtable = NULL; 
	this->iAlgebraicCaches = NULL;
	this->iNfsAlgebraicFactorBase = NULL;
	this->iNfsIntegralFactorBase = NULL;
	this->iNfsRelation = NULL;
	this->iAlgebraicBlock = NULL;
	this->iIntegralBase = NULL;
	mpz_init(this->iM);
	this->iNfsCompressEvenIntervals = FALSE;
	this->iNfsOddToEven = FALSE;
	this->iNfsEvenToOdd = FALSE;
	this->iIsCompressedNow = FALSE;
	this->iShiftingAlgebraic = FALSE;
#endif
	this->iIntermediateIntegers = 0;
	this->iIntermediateAlgebraic = 0;	
#ifdef QS_LINE_SIEVING
	this->iQsInstance = NULL;
	iQsCurrentPolynomial.inited = 0;
	iQsNextPolynomial.inited = 0;
	iQsRelation = NULL;
	iQsUseRootShort = FALSE;
#endif
	this->iAssertionMode = FALSE;
	this->iCheckAllMode = FALSE;
	this->iInitIntegralBlockValue = 0;
	mpz_init(this->aux_1);
	mpz_init(this->aux_2);
	mpz_init(this->aux_3);
	mpz_init(this->aux_4);
	mpz_init(this->aux_5);
	this->inited = AUX_1_INITED | AUX_2_INITED | AUX_3_INITED | AUX_4_INITED | AUX_5_INITED;
#ifdef QS_LINE_SIEVING
	this->iQsMajorPolyChange = FALSE;
	this->iQsCurrentDivisorsOfA = NULL;
	this->iQsNextDivisorsOfA = NULL;
	this->iQsSiqsSValue = 0;
#endif
	this->iSievingTimeStart = 0;
	this->iSievingTimeEnd = 0;
	this->iSievingClockDiff = 0;
	this->iSievingClockTotal = 0;
	mpz_init(iFactoredValue); // kN
	this->iFVInited = TRUE;
	this->iUseHashingForMiddlePrimes = FALSE;
	this->iUseMirrorTableForLargishPrimes = FALSE;
	this->iBlockSieveTotal = 0;
	this->iFillHashtableTotal = 0;
	this->iFillHashtableNeg = 0;
	this->iRootsUpdateTotal = 0;
	this->iFindAndFactorizeTotal = 0;
	this->iTimeMessage = 0;
	this->iAccepted = 0;
	this->iRejected = 0;
	this->iTotalDivrem = 0;
	this->iSievingAlgebraic = FALSE;
	this->iSievingIntegral = FALSE;
}

LineSiever::~LineSiever()
{
	if (iFVInited)
	{
		mpz_clear(iFactoredValue);
		iFVInited = FALSE;
	}
	this->iInternalSieveState = EBeingDestroyed;
	if (iBlock != NULL) 
	{
		free(iBlock);	
		iBlock = NULL;
	}
	DeleteHashtable(this->iHashtable,this->iAllocatedHashtableSize-1); // max index
	this->iHashtable = NULL;
	if (iCaches) {
		free(iCaches);
		iCaches = NULL;
	}
#ifdef GNFS_LINE_SIEVING	
	DeleteHashtable(this->iAlgebraicHashtable,this->iAllocatedHashtableSize-1); // max index
	this->iAlgebraicHashtable = NULL; 
	if (iAlgebraicBlock != NULL)
	{
		free (iAlgebraicBlock);
		iAlgebraicBlock = NULL;
	}	
	if (iAlgebraicCaches)
	{
		free(iAlgebraicCaches);
		iAlgebraicCaches = NULL;
	}
	if (iNfsInstance)
	{
		if (iNfsRelation)
		{
			iNfsInstance->ClearRelation(iNfsRelation);
			free(iNfsRelation);
			iNfsRelation = NULL;
		}
	}
	mpz_clear(this->iM);
#endif
#ifdef QS_LINE_SIEVING
	if (iQsInstance)
	{
		iQsInstance->DestroyPolynomial(&iQsCurrentPolynomial);
		iQsInstance->DestroyPolynomial(&iQsNextPolynomial);
		if (iQsRelation)
		{
			iQsInstance->ClearRelation(iQsRelation);
			free(iQsRelation);
			iQsRelation = NULL;
		}
	}
	
#endif
	if (this->inited & AUX_1_INITED)
	{
		mpz_clear(aux_1);
		this->inited &= ~(AUX_1_INITED);
	}
	if (this->inited & AUX_2_INITED)
	{
		mpz_clear(aux_2);
		this->inited &= ~(AUX_2_INITED);
	}
	if (this->inited & AUX_3_INITED)
	{
		mpz_clear(aux_3);
		this->inited &= ~(AUX_3_INITED);	
	}
	if (this->inited & AUX_4_INITED)
	{
		mpz_clear(aux_4);
		this->inited &= ~(AUX_4_INITED);
	}
	if (this->inited & AUX_5_INITED)
	{
		mpz_clear(aux_5);
		this->inited &= ~(AUX_5_INITED);
	}
#ifdef QS_LINE_SIEVING
	free(this->iQsCurrentDivisorsOfA);
	free(this->iQsNextDivisorsOfA);
#endif
#ifdef GNFS_LINE_SIEVING
	delete iIntegralBase;	
#endif
}

#ifdef GNFS_LINE_SIEVING
void LineSiever::SetNfsCompressEvenIntervals(bool aValue)
{
	this->iNfsCompressEvenIntervals = aValue;
}

inline void LineSiever::UpdateNfsRoot(nfs_fb_type* aEntry)
{
	register main_sieving_type p;
	p = aEntry->p;
//#define UPDATE_NFS_ROOT_DEBUG
#ifdef UPDATE_NFS_ROOT_DEBUG
	bool res = (p == 131) || (p ==65537);
	if (res) {
		printf("Update Nfs Root: old information ");
		PrintNfsElementInfo(*aEntry);		
	}
#endif
	if (iNfsCompressEvenIntervals)
	{
		// Return from a compressed interval to a normal interval.
		if (iNfsEvenToOdd)
		{
			if (p != 2) {
#ifdef UPDATE_NFS_ROOT_DEBUG
				if (res) {
					printf("Update Nfs Root: (de)compression from even to odd. Old value of a root for %d is %d \n",p,aEntry->root_1);
				}	
#endif
				aEntry->root_1 <<= 1;
#ifdef UPDATE_NFS_ROOT_DEBUG
				if (res) {
					printf("Update Nfs Root: (de)compression from even to odd. After multiplication by 2 is the root for %d equal to %d \n",p,aEntry->root_1);
				}	
#endif
				if (aEntry->root_1 >= p)
				{
					aEntry->root_1 -= p;
				}
#ifdef UPDATE_NFS_ROOT_DEBUG
				if (res) {
					printf("Update Nfs Root: (de)compression from even to odd. After possible reduction by p is the root for %d equal to %d \n",p,aEntry->root_1);
				}	
#endif
			}
		}
#ifdef UPDATE_NFS_ROOT_DEBUG
		if (res) {
			printf("Update Nfs Root: old value of a root is %d \n",aEntry->root_1);
		}	
#endif
	}
	aEntry->root_1 += aEntry->auxVal;
	// auxVal can be negative
	if (aEntry->root_1 < 0)
	{
		aEntry->root_1 += p;
	}
	if (aEntry->root_1 >= p)	
	{
		aEntry->root_1 -= p;
	}
	if (iNfsCompressEvenIntervals)
	{
		if (iNfsOddToEven)
		{
			if (p != 2)
			{
#ifdef UPDATE_NFS_ROOT_DEBUG
				if (res) {
					printf("Update Nfs Root: compression from odd to even. Old value of a root for %d is %d \n",p,aEntry->root_1);
				}	
#endif
				// We will check whether the result is even. If no, we will add p. 
				// Then, we will divide the result by 2.
				// ? What about algebraic prime 2?
				if (aEntry->root_1 % 2)
				{
					aEntry->root_1 += p;

#ifdef UPDATE_NFS_ROOT_DEBUG
					if (res) {
						printf("Update Nfs Root: compression from odd to even. Root must be expanded by p, result is %d \n",aEntry->root_1);
					}	
#endif

				}
				aEntry->root_1 >>= 1; // division by 2
#ifdef UPDATE_NFS_ROOT_DEBUG
				if (res) {
					printf("Update Nfs Root: compression from odd to even. After division by 2, the root for %d is %d \n",p,aEntry->root_1);
				}	
#endif	
			}
		}
	}
#ifdef UPDATE_NFS_ROOT_DEBUG
	if (res)
	{
		printf("Update Nfs Root: new information ");
		PrintNfsElementInfo(*aEntry);
	}
#endif
	
}
#endif

#ifdef QS_LINE_SIEVING
inline bool LineSiever::IsDivisorOfA(main_sieving_type& aPrime)
{	
 	if ((aPrime < iLeastCurrentDivisorOfA) || (aPrime > iGreatestCurrentDivisorOfA))
	{
		return FALSE;
	}
	int j;	
	for (j=0; j < iQsSiqsSValue; j++)
	{
		if (aPrime == iQsCurrentDivisorsOfA[j]) return TRUE;
	}
	return FALSE;
}


inline void LineSiever::UpdateSiqsRoots(qs_fb_type* aEntry)
{
	register main_sieving_type p;
	p = aEntry->p;
#ifdef SIQS_SHIFT_DEBUG
	printf("Update siqs roots: value p = %7d, subtract %d, Nu = %d.\n",p,iQsSubtract,iQsNu);
#endif
	if ((this->iQsMode == ESiqs) && IsDivisorOfA(p))
	{
#ifdef SIQS_SHIFT_DEBUG
		printf("Update siqs roots: %d divides 'a': ",p);
#endif		
		main_sieving_type sol = iQsInstance->CalculateSolutionForADivisor(&iQsNextPolynomial,p);
		// We must calculate root normally
#ifdef SIQS_SHIFT_DEBUG
		printf(" sol = %d\n ",sol);
#endif
		if (sol != -1) 
		{
			main_sieving_type after = iQsInstance->Evaluate(&iQsNextPolynomial,sol,p);
			if (after)
			{
				printf("Update Siqs Roots: error determining root for %d ... a divisor of 'a'\n",p);
				iQsInstance->EvaluateWithPrint(&iQsNextPolynomial,sol,p);	
			} else
			{
				if ((p == 3313) || (p == 3163))
				{
					printf("LineSiever::Update SIQS Roots. Determined root for %d as %d.\n",p,sol);
					after = iQsInstance->EvaluateWithPrint(&iQsNextPolynomial,sol,p);
				}

			}
			aEntry->root_1 = sol - 1; // -1 for offset calculation
			if ((aEntry->root_1) < 0)
			{
				aEntry->root_1 += p;
			}
		} else
		{
			aEntry->root_1 = -1;
			printf("Update Siqs Roots: Root 1 is -1 for p = %d divisor of 'a'\n",p);
		}
		return;
	}
	if (!iQsSubtract){
		if (aEntry->root_1 > -1)
		{
			
			//printf("Update siqs roots: %d does not divide 'a': ",p);
#ifdef SIQS_SHIFT_DEBUG
			printf("  USQ root 1 sub: current root is %7d, ba_inv   is %d.\n",aEntry->root_1,aEntry->ba_inv[iQsNu]);
#endif
			if ((aEntry->root_1 -= aEntry->ba_inv[iQsNu]) < 0)
			{
					aEntry->root_1 += p;
			}
#ifdef SIQS_SHIFT_DEBUG
			printf("  USQ root 1 sub: new root is %7d.\n",aEntry->root_1);
			
#endif			
		}
		if (aEntry->root_2 > -1)
		{
#ifdef SIQS_SHIFT_DEBUG
			printf("  USQ root 2 sub: current root is %7d, ba_inv   is %d.\n",aEntry->root_2,aEntry->ba_inv[iQsNu]);
#endif
			if ((aEntry->root_2 -= aEntry->ba_inv[iQsNu]) < 0)
			{
				aEntry->root_2 += p;
			}
#ifdef SIQS_SHIFT_DEBUG
			printf("  USQ root 2 sub: new root is %7d.\n",aEntry->root_2);		
#endif
		}
	}
	else { // case of adding Bainv_i to solutions
		if (aEntry->root_1 > -1)
		{
#ifdef SIQS_SHIFT_DEBUG
			printf("  USQ root 1 add: current root is %7d, ba_inv   is %d.\n",aEntry->root_1,aEntry->ba_inv[iQsNu]);
#endif
			if ((aEntry->root_1 += aEntry->ba_inv[iQsNu]) >= p)
			{
				aEntry->root_1 -= p;
			}
#ifdef SIQS_SHIFT_DEBUG
			printf("  USQ root 1 add: new root is %7d.\n",aEntry->root_1);
#endif
		}
		if (aEntry->root_2 > -1)
		{
#ifdef SIQS_SHIFT_DEBUG
			printf("  USQ root 2 add: current root is %7d, ba_inv   is%d.\n",aEntry->root_2,aEntry->ba_inv[iQsNu]);
#endif
			if ((aEntry->root_2 += aEntry->ba_inv[iQsNu]) >= p)
			{
				aEntry->root_2 -= p;
			}
#ifdef SIQS_SHIFT_DEBUG
			printf("  USQ root 2 adds: new root is %7d.\n",aEntry->root_2);		
#endif
		}
	}

}
#endif


#ifdef QS_LINE_SIEVING
inline void LineSiever::CalculateNewRoots(qs_fb_type* aEntry, main_sieving_type inverse_a)
{
	main_sieving_type b;
	register main_sieving_type p1 = aEntry->p;
	if (!inverse_a)
	{
#ifdef SIQS_SHIFT_DEBUG
		printf("Calculate new roots: we need to calculate solution for %d ... a divisor of 'a'\n",p1);
#endif
		// Not a true quadratic equation
		main_sieving_type minus_c = mpz_fdiv_ui(iQsNextPolynomial.c,p1);		
		minus_c = p1 - minus_c; // -c;
		mpz_set_ui(aux_1,p1);
		mpz_invert(aux_2,iQsNextPolynomial.b,aux_1);
		main_sieving_type inv_b = mpz_get_ui(aux_2);
		if (inv_b) {
			aEntry->root_1 = Mulm(minus_c,inv_b,p1) - 1; // -1 for offset calculation
			if (aEntry->root_1 < 0) 
			{
				aEntry->root_1 += p1;
			}
			main_sieving_type after = iQsInstance->Evaluate(&iQsNextPolynomial,aEntry->root_1 + 1,p1);
			if (after)
			{
				printf("Calculate new roots: error determining root for %d ... a divisor of 'a'\n",p1);
				iQsInstance->PrintPolynomial(&iQsNextPolynomial);
			} else
			{
				if ((p1 == 3313) || (p1 == 3163))
				{
					printf("LineSiever::CalculateNewRoots. Determined root for %d as %d.\n",p1,aEntry->root_1);
					after = iQsInstance->EvaluateWithPrint(&iQsNextPolynomial,aEntry->root_1 + 1,p1);
				}

			}
			aEntry->root_2 = -1;
		}
		else {
			// Case of p | b. Pathological case.
			aEntry->root_1 = -1;
			aEntry->root_2 = -1;
		}	
	} 
	else {
		// A true quadratic equation
		b  = mpz_fdiv_ui(iQsNextPolynomial.b,p1);
		aEntry->root_1 = (aEntry->sqrt_kn - b) % p1;
		aEntry->root_1 = Mulm((int) aEntry->root_1,(int) inverse_a,p1) - 1; // -1 ... for offset (see also QuadraticSieve::InitAdvanced...
		if (aEntry->root_1 < 0)
		{
			aEntry->root_1 += p1;
		}
		aEntry->root_2  = ((-1)*aEntry->sqrt_kn - b) % p1;
		aEntry->root_2 = Mulm((int) aEntry->root_2,(int) inverse_a,p1) - 1; // -1 ... for offset (see also QuadraticSieve::InitAdvanced...
		if (aEntry->root_2 < 0)
		{
			aEntry->root_2 += p1;
		}
	}	
}


void LineSiever::SetQsUseRootShort(bool aValue)
{
	this->iQsUseRootShort = aValue;
}
#endif

void LineSiever::DeleteHashtable(hashtable_t* aHashtable, int aMax)
{
	if (aHashtable)
	{
		hashtable_t* h1;
		for (int i=0; i<= aMax; i++) {
			h1 = &(aHashtable[i]);
			if (h1) {
				ks_script_t* ptr = h1->script;
				ks_script_t* next;
				while (ptr)
				{
					next = ptr->next;
					free(ptr);
					ptr = next;
				}
			}
		}
	}
	free(aHashtable);
}

int  LineSiever::SetupHashtable(hashtable_t** aHashtable, ks_cache_t** aCache)
{
	//printf("Setup hashtable: allocated size %d.\n",iAllocatedHashtableSize);
	*aHashtable = (hashtable_t*) AbstractSieve::AllocationMachine((void**) aHashtable,this->iAllocatedHashtableSize*sizeof(hashtable_t));
	if (*aHashtable == NULL)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup hashtable",0,this->iAllocatedHashtableSize*sizeof(hashtable_t),"Could not allocate space for the sieving hashtable");
		return NOT_ENOUGH_MEMORY;
	} /*else {
		printf("Allocated hashtable: %d blocks.",this->iAllocatedHashtableSize);
		fflush(stdout);
	}*/
	memset(*aHashtable,0,this->iAllocatedHashtableSize*sizeof(hashtable_t));
	*aCache = (ks_cache_t*) AbstractSieve::AllocationMachine((void**) aCache,this->iAllocatedHashtableSize*sizeof(ks_cache_t));
	if (*aCache == NULL)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup hashtable",0,this->iAllocatedHashtableSize*sizeof(ks_cache_t),"Could not allocate space for the auxiliary cache");
		return NOT_ENOUGH_MEMORY;
	} else {
		//printf("Cache construction: size %d.\n",this->iAllocatedHashtableSize*sizeof(ks_cache_t));
	}
	memset(*aCache,0,this->iAllocatedHashtableSize*sizeof(ks_cache_t));
	int i, max = this->iAllocatedHashtableSize;
	hashtable_t* t1;
	for (i=0; i< max; i++)
	{
		//printf("i = %d; ",i);
		//fflush(stdout);
		t1 = &((*aHashtable)[i]);
		//printf("pointer = %p; ",t1);
		//fflush(stdout);
		t1->script = (ks_script_t*) malloc(sizeof(ks_script_t));
		if (t1->script == NULL)
		{
			CLog::LogError(WHEREARG,"Line siever","Setup hashtable",i,sizeof(ks_script_t),"Could not allocate space for the script inside a hashtable");
			return NOT_ENOUGH_MEMORY;
		}
		t1->script->next = NULL;
		t1->script->num_used = 0;
		(*aCache)[i].used = 0;
	}
	return ConstRC::Ok;
}

#ifdef QS_LINE_SIEVING
void LineSiever::SetQsFactorBase(qs_fb_type* aQsFactorBase)
{
	if (this->iInternalSieveState != EWaitingForFBSetup)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup qs factor base",0,this->iInternalSieveState,"The line siever is in suspicious state.");
	
	}
	if (aQsFactorBase == NULL)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup qs factor base",0,0,"The supplied factor base is NULL, aborting.");
		return;
	}
	this->iQsFactorBase = aQsFactorBase;
	if ((this->iInternalSieveState == EWaitingForFBSetup) && (aQsFactorBase != NULL))
	{
		this->iInternalSieveState = EReadyForSieving;
	}
	int i = 0;
	iQsFirstMiddlePrime  = NULL;
	iQsFirstLargishPrime = NULL;
	main_sieving_type p;
	//printf("Line Siever: set qs factor base before while\n");
	while ((p = this->iQsFactorBase[i].p) != 0)
	{
		if ((!iQsFirstMiddlePrime) && (p > MINIMAL_PRIME_TO_SIEVE_WITH))
		{
			iQsFirstMiddlePrime = this->iQsFactorBase+i;
		}
		// Increments continually to the largest available 'middle-sized prime'
		if (p <= iBlockSize)
		{
			iQsLastMiddlePrime = this->iQsFactorBase+i;
		}
		if ((!iQsFirstLargishPrime) && (p > iBlockSize))
		{
	
			iQsFirstLargishPrime = this->iQsFactorBase+i;
			break;
		}
		i++;
	}	
	if (!iQsFirstMiddlePrime)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup qs factor base",0,0,"Qs first middle prime pointer is NULL - inconsistent state!");
	}
	if (!iQsFirstLargishPrime)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup qs factor base",0,0,"Qs first largish prime pointer is NULL - only middle-sized primes possible!");
	}
}

void LineSiever::SetQsInstance(QuadraticSieve* aQsInstance)
{
	if (aQsInstance == NULL)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup qs instancee",0,0,"The qs instance is NULL.");
	
	} else {
		this->iQsMode = aQsInstance->Get_QSMode();
		aQsInstance->Get_kN(iFactoredValue);
	}
	this->iQsInstance = aQsInstance;
	if ((!iQsRelation) && (iQsInstance))
	{
		iQsRelation = iQsInstance->InitRelation();
		if (!iQsRelation)
		{
			CLog::LogError(WHEREARG,"Line siever","Reset blocks",0,sizeof(qs_relation),"Could not allocate space for the qs relation");
			return;
		}
	}
	if (iQsInstance)
	{
		this->iTimeMessage = iQsInstance->Get_TimeMessageInterval();
		this->iCheckAllMode = (iQsInstance->Get_CheckMode() == ECheckAll);		
	}
	
}
#endif

#ifdef GNFS_LINE_SIEVING
void LineSiever::SetNfsInstance(NumberFieldSieve* aNfsInstance)
{
	if (aNfsInstance == NULL)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup nfs instance",0,0,"The nfs instance is NULL.");
	
	} else {		
		aNfsInstance->Get_N(iFactoredValue);
	}
	this->iNfsInstance = aNfsInstance;
	if ((!iNfsRelation) && (iNfsInstance))
	{
		iNfsRelation = iNfsInstance->InitRelation( FOR_DOUBLE_PP);
		if (!iNfsRelation)
		{
			CLog::LogError(WHEREARG,"Line siever","Setup NFS instance",0,sizeof(nfs_relation),"Could not allocate space for the nfs relation");
			return;
		}
	}
	if (iNfsInstance)
	{
		this->iTimeMessage = iNfsInstance->Get_TimeMessageInterval();
		this->iCheckAllMode = (iNfsInstance->GetCheckMode() == ECheckAll);
	}
	if (iIntegralBase)
	{
		delete iIntegralBase;
		iIntegralBase = NULL;
	}
 	iIntegralBase = new HermiteMatrix(iNfsInstance->GetPolynomialDegree(),iNfsInstance->GetPolynomialDegree());		
	iIntegralBase->Allocate();
	iIntegralBase->Zeroize();
	
}

void LineSiever::SetNfsIntegralFactorBase(nfs_fb_type* aNfsIntegralFactorBase)
{
	if (this->iInternalSieveState != EWaitingForFBSetup)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup nfs integral factor base",0,this->iInternalSieveState,"The line siever is in suspicious state.");
	
	}
	if (aNfsIntegralFactorBase == NULL)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup nfs integral factor base",0,0,"The supplied factor base is NULL, aborting.");
		return;
	}
	this->iNfsIntegralFactorBase = aNfsIntegralFactorBase;
	if ((this->iInternalSieveState == EWaitingForFBSetup) && (aNfsIntegralFactorBase != NULL) && (this->iNfsAlgebraicFactorBase != NULL))
	{
		this->iInternalSieveState = EReadyForSieving;
	}
	int i = 0;
	iNfsFirstIntegralMiddlePrime  = NULL;
	iNfsFirstIntegralLargishPrime = NULL;
	main_sieving_type p;
	//printf("Line Siever: set nfs integral factor base before while\n");
	while ((p = this->iNfsIntegralFactorBase[i].p) != 0)
	{
		//printf("p = %d.\n",p);
		if ((!iNfsFirstIntegralMiddlePrime) && (p > MINIMAL_PRIME_TO_SIEVE_WITH))
		{
			iNfsFirstIntegralMiddlePrime = this->iNfsIntegralFactorBase+i;
		}
		// Increments continually to the largest available 'middle-sized prime'
		if (p <= iBlockSize)
		{
			iNfsLastIntegralMiddlePrime = this->iNfsIntegralFactorBase+i;
		}
		if ((!iNfsFirstIntegralLargishPrime) && (p > iBlockSize))
		{
	
			iNfsFirstIntegralLargishPrime = this->iNfsIntegralFactorBase+i;
			break;
		}
		i++;
	}	
	if (!iNfsFirstIntegralMiddlePrime)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup nfs integral factor base",0,0,"Nfs first integral middle prime pointer is NULL - inconsistent state!");
	}
	if (!iNfsFirstIntegralLargishPrime)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup nfs integral factor base",0,0,"Nfs first integral largish prime pointer is NULL - only middle-sized primes possible!");
	}
}

void LineSiever::SetNfsAlgebraicFactorBase(nfs_fb_type* aNfsAlgebraicFactorBase)
{
	if (this->iInternalSieveState != EWaitingForFBSetup)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup nfs algebraic factor base",0,this->iInternalSieveState,"The line siever is in suspicious state.");
	
	}
	if (aNfsAlgebraicFactorBase == NULL)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup nfs algebraic factor base",0,0,"The supplied factor base is NULL, aborting.");
		return;
	}
	this->iNfsAlgebraicFactorBase = aNfsAlgebraicFactorBase;
	if ((this->iInternalSieveState == EWaitingForFBSetup) && (aNfsAlgebraicFactorBase != NULL) && (this->iNfsIntegralFactorBase != NULL))
	{
		this->iInternalSieveState = EReadyForSieving;
	}
	int i = 0;
	iNfsFirstAlgebraicMiddlePrime  = NULL;
	iNfsFirstAlgebraicLargishPrime = NULL;
	main_sieving_type p;
	//printf("Line Siever: set qs factor base before while\n");
	while ((p = this->iNfsAlgebraicFactorBase[i].p) != 0)
	{
		if ((!iNfsFirstAlgebraicMiddlePrime) && (p > MINIMAL_PRIME_TO_SIEVE_WITH))
		{
			iNfsFirstAlgebraicMiddlePrime = this->iNfsAlgebraicFactorBase+i;
		}
		// Increments continually to the largest available 'middle-sized prime'
		if (p <= iBlockSize)
		{
			iNfsLastAlgebraicMiddlePrime = this->iNfsAlgebraicFactorBase+i;
		}
		if ((!iNfsFirstAlgebraicLargishPrime) && (p > iBlockSize))
		{	
			iNfsFirstAlgebraicLargishPrime = this->iNfsAlgebraicFactorBase+i;
			break;
		}
		i++;
	}	
	if (!iNfsFirstAlgebraicMiddlePrime)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup nfs algebraic factor base",0,0,"Nfs first algebraic middle prime pointer is NULL - inconsistent state!");
	}
	if (!iNfsFirstAlgebraicLargishPrime)
	{
		CLog::LogError(WHEREARG,"Line siever","Setup nfs algebraic factor base",0,0,"Nfs first algebraic largish prime pointer is NULL - only middle-sized primes possible!");
	}
}
#endif

void LineSiever::SetMaxLines(int aValue)
{
	this->iMaxLines = aValue;
}

void LineSiever::SetUnlimitedSieving()
{
	this->iMaxLines = UNLIMITED_SIEVING;
}

void LineSiever::SetAssertionMode(bool aValue)
{
	this->iAssertionMode = aValue;
}

void LineSiever::SetInitBlockValue(log_type aValue)
{
	//printf("Line siever: setting init block value: %d.\n",aValue);
	this->iInitIntegralBlockValue = aValue;
}

#ifdef GNFS_LINE_SIEVING
void LineSiever::SetInitAlgebraicBlockValue(log_type aValue)
{
	iInitAlgebraicBlockValue = aValue;
}
#endif // GNFS_LINE_SIEVING

int  LineSiever::PerformSieving()
{
	

	return ConstRC::Ok;
}

int  LineSiever::FillHashtable(hashtable_t* aHashtable, ks_cache_t* aCache, int aDirection, bool aIsAlgebraic)
{
	main_sieving_type p;
	main_sieving_type root_1;
	main_sieving_type root_2;
	main_sieving_type c_p;
	log_type log_p;
	int ret;
	int shiftConst = 2;
#ifdef QS_LINE_SIEVING
	qs_fb_type* start   = iQsFirstLargishPrime;
	qs_fb_type* running;
	main_sieving_type inverse_a;
	mpz_t  two_a;
	mpz_t  invert;
	mpz_t  aux_p;
	if (iQsMajorPolyChange)
	{
		mpz_init(two_a);
		mpz_init(invert);
		mpz_init(aux_p);
		mpz_mul_ui(two_a,iQsNextPolynomial.a,2);
		if (this->iQsMode == ESiqs)
		{			
			
			iQsInstance->CopyADivisors(iQsNextDivisorsOfA);
			iLeastNextDivisorOfA = iQsNextDivisorsOfA[0];
			iGreatestNextDivisorOfA = iQsNextDivisorsOfA[iQsSiqsSValue-1];
		}
	}
#endif // QS_LINE_SIEVING
#ifdef GNFS_LINE_SIEVING
	if (iNfsCompressEvenIntervals && iIsCompressedNow)
	{
		shiftConst = 1;
	}
	nfs_fb_type* start = NULL;
	nfs_fb_type* running;
	if (!aIsAlgebraic)
	{
		start = iNfsFirstIntegralLargishPrime;
	} 
	else {
		start = iNfsFirstAlgebraicLargishPrime;
	}
#endif // GNFS_LINE_SIEVING
	ClearHashtables(aHashtable,aCache);
	running = start;
	if (start == NULL)
	{
		// In case that all primes are middle (no largish exist).
		return ConstRC::Ok;
	}
	//printf(" Fill hashtable running. ...");
	// The upper bound of the FB is denoted by a zero block;
	while (p = running->p)
	{		
 		root_1 = running->root_1;
		log_p  = running->log_p;
		// if root is valid ...
		if (root_1 >= 0)
		{
			//printf("Running.root_1 = %d.\n",root_1);
			if (aDirection == NEGATIVE)	
			{
				// Recalculate the root for the negative sieving
				root_1 = p - root_1 - shiftConst; // shiftConst ... offset move caused by offset policy
				if (root_1 < 0)
				{
					root_1 += p;
				}
			}
			// Add to the update table
			/*if (isThisPrimeControlled(p))
			{
				printf("------------------------------------\n");
				printf("Fill Hashtable: p = %d, alg = %d, current c_p is %d, root is %d.\n",p, aIsAlgebraic,running->c_p,root_1); 			
			}	*/	
			//printf("Fill hashtable: adding to table: %6d (root %d).\n",p,root_1);
			c_p = 0;
#ifdef GNFS_LINE_SIEVING
			if (aIsAlgebraic)
			{
				c_p = running->c_p;
#ifdef ONE_PRIME_DEBUG
				if (p == 191251) {
					printf("\nFill Hashtable Algebraic: Adding to table: p = %d, c_p = 	%d, root_1 = %d.\n",p,c_p,root_1);
					wait();
				}
#endif
			} else 
			{
				//printf("Fill Hashtable integral: Adding to table: p = %d, c_p = %d.\n",p,c_p);
			}
			
#endif			
			if (root_1 >= p) {
				printf("I. Root1 %d > p %d.\n",root_1,p);
			}
			ret = AddToTable(aHashtable, aCache, p, root_1, c_p, log_p);

			
			if (ret != ConstRC::Ok)
			{
				return ret;
			}
			if (aDirection == NEGATIVE)
			{
				// Shift the root for the next polynomial or line. 
				// For MPQS/SIQS this is done in root_2 branch for both roots.
			}
		}    
		// The only case when second root must be calculated is MPQS/SIQS (not NFS).
#ifdef QS_LINE_SIEVING
		root_2 = running->root_2;
		if (root_2 >= 0)
		{
			if (aDirection == NEGATIVE)	
			{
				// Recalculate the root for the negative sieving
				root_2 = p - root_2 - 2;
				if (root_2 < 0)
				{
					root_2 += p;
				}
			}
			// Add to the update table
			ret = AddToTable(aHashtable, aCache, p, root_2, 0, log_p); // c_p = 0, has no meaning in MPQS/SIQS
			if (ret != ConstRC::Ok)
			{
				return ret;
			}			
		}
		// Recalculation of roots must be done for each largish entry.
		if (aDirection == NEGATIVE)
		{
			// Shift the root for the next polynomial or line
			// Identical code for middle primes is at the end of SieveLine method
			if (iQsMajorPolyChange)
			{
				mpz_set_ui(aux_p,p);
				if (mpz_invert(invert,two_a,aux_p))
				{
					inverse_a = mpz_get_ui(invert);
				}
				else
				{
					inverse_a = 0;
				}	
				CalculateNewRoots(running,inverse_a);
			}
			// Shift the root in SIQS, when only 'b' is changed.
			 else 
			{
				UpdateSiqsRoots(running);
			}
		}
#endif
#ifdef GNFS_LINE_SIEVING
		// Recalculation of roots must be done for each largish entry.
		if (aDirection == NEGATIVE)
		{
			UpdateNfsRoot(running);	
		}
#endif
		running++;
	}
	int max = iBlockSetSize;
	// Flush all caches
	if (!aIsAlgebraic)
	{
		for (int i=0; i<= max; i++) {
			FlushUpdateCache(&(iHashtable[i]),&(iCaches[i]));
		}
	}
#ifdef GNFS_LINE_SIEVING
	else {
		for (int i=0; i<= max; i++) {
			FlushUpdateCache(&(iAlgebraicHashtable[i]),&(iAlgebraicCaches[i]));
		}
	}
#endif
	//PrintHashtables(iHashtable);
#ifdef QS_LINE_SIEVING
	if (iQsMajorPolyChange)
	{
		mpz_clear(two_a);
		mpz_clear(invert);
		mpz_clear(aux_p);
	}
#endif
	return ConstRC::Ok;
}

int  LineSiever::AddToTable(hashtable_t* aHashtable, ks_cache_t* aCache, main_sieving_type aPrime, main_sieving_type aRoot, main_sieving_type aCp, log_type aLog, int aHashtableIndex)
{
	ks_cache_t* cache = &(aCache[aHashtableIndex]);
#ifdef ONE_PRIME_DEBUG
	if ((aPrime == 191251) && (aCp)) printf("Add To Table: prime is %d, root is %d, log is %d, c_p is %d, index in caches is %d.\n",aPrime,aRoot,aLog,aCp,index);	

	/*if (isThisPrimeControlled(aPrime))
	{
		printf("------------------------------------\n");
		printf("Add to table: p = %d, current c_p is %d, root is %d. Cache used is %d. Hashtable index is %d\n",aPrime,aCp,aRoot,cache->used,aHashtableIndex); 			
	}*/	
#endif
	/*printf("\n\n\n\n\n\n\nCache %d, used = %d.\n",aHashtableIndex,cache->used);
	printf("Cache pointer: %p.\n\n\n\n\n\n\n",cache);*/
	if (aHashtableIndex >= iAllocatedHashtableSize)
	{
		printf("aHashtableIndex too high: %d for prime %d.\n",aHashtableIndex,aPrime);
		return GENERAL_ERROR;
	}
	
	if (cache->used == CACHE_UPDATES)
	{
		
		hashtable_t* target = &(aHashtable[aHashtableIndex]);
		int ret = FlushUpdateCache(target,cache);
		if (ret != ConstRC::Ok)
		{
			return ret;
		}
	}
	cache->updates[cache->used].p    = aPrime;
	cache->updates[cache->used].h    = aRoot;
	cache->updates[cache->used].logp = aLog;
#ifdef GNFS_LINE_SIEVING
	cache->updates[cache->used].c_p = aCp;
#ifdef ONE_PRIME_DEBUG
	if ((aPrime == 191251) && (aCp))
	{
		printf("AddToTable: 191251.\n");
		nfs_fb_type type;
		type.p = aPrime;
		type.root_1 = aRoot;
		type.log_p = aLog;
		type.c_p = aCp;
		PrintNfsElementInfo(type);
		PrintUpdateInfo(cache->updates[cache->used]);
		wait();
	}
#endif
#endif
	
	cache->used++;
	return ConstRC::Ok;
}

int  LineSiever::AddToTable(hashtable_t* aHashtable, ks_cache_t* aCache, main_sieving_type aPrime, main_sieving_type aRoot, main_sieving_type aCp, log_type aLog)
{
	register int index = aRoot / iBlockSize;
	if (index >= iAllocatedHashtableSize)
	{
		printf("AddToTable: aHashtableIndex too high: %d for prime %d, aRoot = %d, blockSize = %d.\n",index,aPrime,aRoot,iBlockSize);
		return GENERAL_ERROR;
	}

	return AddToTable(aHashtable,aCache,aPrime,aRoot % iBlockSize, aCp, aLog, index);
}


int  LineSiever::FlushUpdateCache(hashtable_t* aHashtable, ks_cache_t* aCache)
{
	int ret;
#ifdef CACHE_FLUSHER_DEBUG
	printf("~~~~~~~ Flushing update cache start ~~~~~~~~\n");
#endif
	ks_script_t* script = aHashtable->script;
	if (!script)
	{
		//printf("Flush update cache: problem, script is NULL.");
		return INCONSISTENT_STATE;
	} /*else {
		printf("Script address is %p, next is %p.\n",script,next);
	}*/
	ks_script_t* next   = script->next;
	int previous = 0;
	while (next)
	{
		if (script->num_used < MAX_UPDATES)
		{
#ifdef CACHE_FLUSHER_DEBUG
			if (next == NULL)
			{
				printf("Flush update cache: the current hashtable is already allocated, but empty.\n");
			}
#endif
			break;
		}
		script = next;
		next = script->next;
		if (next) {
			previous += script->num_used;
		}
	}
	// Copy the part of the list to cache
	PREFETCH(&(script->list[script->num_used]));
	// Now script is the last script.
	if ((next == NULL) && ((script->num_used + aCache->used) > MAX_UPDATES))
	{
		//printf("Flush update cache: script->num used is %d, used is %d, max_upd is %d, will call next ... \n",script->num_used,aCache->used,MAX_UPDATES);
		ret = AddScript(script);
		if (ret != ConstRC::Ok)
		{
			return ret;
		}
		//newScript = TRUE;
	}
	register int i, max = aCache->used;
	register int j = script->num_used;
	for (i=0; i< max; i++)
	{
		if (script->num_used == MAX_UPDATES)
		{
			//printf("script changed: next is %p\n",script->next);
			previous += script->num_used;
			script = script->next;
			j = 0;			
		}
		j = script->num_used;

		/*printf("Doplnuji z cache: p is %d.\n",aCache->updates[i].p);
		printf("Doplnuji z cache: h is %d.\n",aCache->updates[i].h);*/
		/*if (newScript)
		{
			printf("j = %d, p = %d\n",j,aCache->updates[i].p);
		}*/
		script->list[j].p    = aCache->updates[i].p;
		script->list[j].h    = aCache->updates[i].h;
		script->list[j].logp = aCache->updates[i].logp;
#ifdef GNFS_LINE_SIEVING
		script->list[j].c_p  = aCache->updates[i].c_p;
#ifdef ONE_PRIME_DEBUG
		/*if (isThisPrimeControlled(script->list[j].p))
		{
			printf("Flush update cache message:\n");
			PrintUpdateInfo(aCache->updates[i]);
			wait();
		}*/
#endif
		/*if (script->list[j].p == 65537)
		{
			printf("Flush update cache for 65537: c_p is %d.\n",script->list[j].c_p);
		}*/
#endif
#ifdef CACHE_FLUSHER_DEBUG
		printf("Flush update cache, added to index %6d prime %6d with root %6d.\n",j+previous,script->list[j].p,script->list[j].h);
#endif
		script->num_used++;
				
	}
	aCache->used = 0;
#ifdef CACHE_FLUSHER_DEBUG
	printf("~~~~~~~ Flushing update cache end ~~~~~~~~\n");
#endif
	return ConstRC::Ok;
}


int  LineSiever::RunSieve()
{
	int result;
	bool enough = FALSE;
	//printf("\nLine Siever: Run Sieve active. Now calling run sieve...");
#ifdef QS_LINE_SIEVING
	if (!this->iQsInstance)
	{
		CLog::LogError(WHEREARG,"Line siever","Run sieve",0,0,"QS Instance is null!");
		return INCONSISTENT_STATE;
	}
	if (this->iInternalSieveState != EReadyForSieving)
	{
		CLog::LogError(WHEREARG,"Line siever","Run sieve",0,this->iInternalSieveState,"Run sieve called before the siever was ready!");
		return INCONSISTENT_STATE;
	}
	// We will copy the first sieving polynomial to current polynomial variable.
	// The first sieving polynomial has been generated by the calling instance.
	// printf("\nLine Siever: Run Sieve active. Now copying polynomial to member variable...");
	iQsInstance->PolyCopy(&iQsCurrentPolynomial);
	if (iQsUseRootShort)
	{
		iQsInstance->RootCopy(iQsRoot1,iQsRoot2);
		result = SetupRootShort();
		if (result != ConstRC::Ok)
		{
			return result;
		}
	}
	if (this->iQsMode == ESiqs) 
	{
		if ((!this->iQsCurrentDivisorsOfA) || (!this->iQsNextDivisorsOfA))
			{
				this->iQsSiqsSValue = iQsInstance->Get_S();
				this->iQsCurrentDivisorsOfA = (main_sieving_type*) AbstractSieve::AllocationMachine((void**) &this->iQsCurrentDivisorsOfA,sizeof(main_sieving_type)*this->iQsSiqsSValue);
				if (!this->iQsCurrentDivisorsOfA)
				{
					CLog::LogError(WHEREARG,"Line siever","Run sieve ",0,sizeof(main_sieving_type)*this->iQsSiqsSValue,"Could not allocate enough space for current divisors of A");
					return NOT_ENOUGH_MEMORY;
				}
				this->iQsNextDivisorsOfA = (main_sieving_type*) AbstractSieve::AllocationMachine((void**) &this->iQsNextDivisorsOfA,sizeof(main_sieving_type)*this->iQsSiqsSValue);
				if (!this->iQsNextDivisorsOfA)
				{
					CLog::LogError(WHEREARG,"Line siever","Run sieve ",0,sizeof(main_sieving_type)*this->iQsSiqsSValue,"Could not allocate enough space for next divisors of A");
					return NOT_ENOUGH_MEMORY;
				}
			}
			// Now, the iQsDivisorsOfA is securely allocated
			iQsInstance->CopyADivisors(iQsCurrentDivisorsOfA);
			iLeastCurrentDivisorOfA = iQsCurrentDivisorsOfA[0];
			iGreatestCurrentDivisorOfA = iQsCurrentDivisorsOfA[iQsSiqsSValue-1];
			// Exceptional situation for the very first polynomial.
			int x1;
			for (x1 = 0; x1 < iQsSiqsSValue; x1++)
			{
				iQsNextDivisorsOfA[x1] = iQsCurrentDivisorsOfA[x1];
			}
			iLeastNextDivisorOfA = iQsCurrentDivisorsOfA[0];
			iGreatestNextDivisorOfA = iQsCurrentDivisorsOfA[iQsSiqsSValue-1];
	}
#endif	
	int line = 0;
#ifdef QS_LINE_SIEVING
	printf("\n");
	iQsInstance->PrintHeader();
	line = iQsInstance->GetLineNr();
#endif
#ifdef GNFS_LINE_SIEVING
	printf("\n");
	iNfsInstance->PrintHeader();
	iNfsInstance->GetM(this->iM);
	line = iNfsInstance->GetLineNr();
#endif	
	this->iSievingTimeStart = time(NULL);
	iLocalStart = clock();
	do
	{
		//printf("\rRunning sieve line ... %5d, time message = %d, %d\n\n",line++,this->iTimeMessage,(line % this->iTimeMessage) == (this->iTimeMessage-1));
		line++;
#ifdef QS_LINE_SIEVING

			iQsInstance->PrintValues();
			iQsInstance->SetLineNr(line);
#endif
#ifdef GNFS_LINE_SIEVING
			//printf("Line: %d\n",line);
			iNfsInstance->PrintValues();
			iNfsInstance->SetLineNr(line);
#endif
			fflush(stdout);

		if (this->iTimeMessage > 0)
		{
			if ((line % this->iTimeMessage) == (this->iTimeMessage-1))
			{
#ifdef QS_LINE_SIEVING
				AbstractSieve* sieve = iQsInstance;
#endif
#ifdef GNFS_LINE_SIEVING
				AbstractSieve* sieve = iNfsInstance;
#endif
				printf("\nSo far, %d iterations have been run.\n",line);
				sieve->PrintTimeInfo(BLOCK_SIEVES,iBlockSieveTotal,iSievingClockTotal);
				sieve->PrintTimeInfo(FIND_FACTORZ,iFindAndFactorizeTotal,iSievingClockTotal);					
				sieve->PrintTimeInfo(FILL_HSTABLE,iFillHashtableTotal,iSievingClockTotal);
				sieve->PrintTimeInfo(FILL_HST_NEG,iFillHashtableNeg,iSievingClockTotal);
				sieve->PrintTimeInfo(ROOTS_UPDATE,iRootsUpdateTotal,iSievingClockTotal);		

#ifdef QS_LINE_SIEVING
				iQsInstance->PrintEfficiencyInfo(iTotalDivrem, iAccepted, iRejected,line, iSievingClockTotal);
#endif			
				//sieve->PrintEfficiencyInfo(iTotalDivrem,iAccepted,iRejected,line, iSievingClockTotal);

#ifdef GNFS_LINE_SIEVING
				iNfsInstance->PrintEffectiveness(iLocalStart,iIntermediateIntegers,iIntermediateAlgebraic);
				iNfsInstance->ResetAllIntermediates();
				this->iIntermediateIntegers = 0;
				this->iIntermediateAlgebraic = 0;
#endif
				this->iLocalStart = (float) clock();
			}
		}
#ifdef GNFS_LINE_SIEVING
		main_sieving_type now;
		switch (iNfsInstance->GetSievingDirection())
		{
			case EIncrementA:
			case EDecrementA:
				now = iNfsInstance->GetA();
				break;
			case EIncrementB:
			case EDecrementB:
				now = iNfsInstance->GetB();
				break;
		}
		if (!(now % 2))
		{
			// The next value will be odd, current is even
			this->iNfsEvenToOdd = TRUE;
			this->iNfsOddToEven = FALSE;
			this->iIsCompressedNow = TRUE;
		} else 
		{
			// The next value will be even, current is odd
			this->iNfsEvenToOdd = FALSE;
			this->iNfsOddToEven = TRUE;
			this->iIsCompressedNow = FALSE;
		}
#endif
		result = SieveLine();
		if (result != ConstRC::Ok)
		{
			return result;
		}	
		if (result == TERMINATED)
		{
			return ConstRC::Ok;
		}
		if (iMaxLines != UNLIMITED_SIEVING)
		{
			if (line >= iMaxLines) {
				this->iSievingTimeEnd = time(NULL);
// 	AbstractSieve::TimeFormat((iSievingTimeEnd-iSievingTimeStart));
				printf("\n      Total time spent by sieving itself is %12.4f sec.\n",iSievingClockTotal/(double) CLOCKS_PER_SEC);
			 	break; 
			}
		}
#ifdef QS_LINE_SIEVING
		enough = (! iQsInstance->Get_IsDistributed()) &&  (iQsInstance->CheckEnough() == ENOUGH);
		if (!enough)
		{
			log_type complement = 255-iInitIntegralBlockValue;
			if ((line % THRESHOLD_OPTIMIZER_INTERVAL) == 0)
			{
				iQsInstance->GetOptimizer().AnalyzeAndOptimize(complement);
				iInitIntegralBlockValue = 255-complement;
			}
		}
#endif
#ifdef GNFS_LINE_SIEVING
		enough = (iNfsInstance->CheckEnough() == ENOUGH);		
		switch (iNfsInstance->GetSievingDirection())
		{
			case EIncrementA:
				iNfsInstance->SetA(++now);
				break;
			case EDecrementA:
				iNfsInstance->SetA(--now);
				break;
			case EIncrementB:
				iNfsInstance->SetB(++now);
				break;
			case EDecrementB:
				iNfsInstance->SetB(--now);
				break;
		}
#endif		

	} while (!enough);
	// The sieving ended before enough relations have been found
	if (!enough)
	{
		
		return NOT_ENOUGH_SMOOTHS;
	}
	return ConstRC::Ok;
}

/**
	\short This method will perform sieving in a single line and readjust the data for the next line.

	This method takes care of the line sieving. It expects the factor base(s) to be already
	divided into middle and largish primes.

	The method will at first sieve the positive part of the sieving interval: [1,M], and
	then, separately, the negative part [-M,-1]. The value of 0 is ignored.

	The positive and negative parts will be sieved with use of "block sets". The "block set" is a set of blocks big enough to cover the whole factor base (in case of NFS, to cover
	the bigger of the two factor bases). The sieving of each of the block sets is performed
	using the BlockSetSieve method.

	Before both the positive and the negative part of the sieving, a hashtable with largish
	primes is filled in. During the negative hashtable fill, the roots are also adjusted for
	the next sieving line; in case of MPQS/SIQS, this also means need to generate a new
	polynomial in between the positive and the negative part.

*/
int  LineSiever::SieveLine()
{
	AbstractSerializator* serializator = NULL;
	bool interrupt = FALSE;
	int  flag = -1, flag2 = -1, ret;
#ifdef QS_LINE_SIEVING
	interrupt = iQsInstance->Get_InterruptNow();
	if (interrupt)
	{
		serializator = iQsInstance->GetSerializator();
		flag = EQsSieving;
		flag2 = EQsSieverState;
	}
#endif
#ifdef GNFS_LINE_SIEVING
	interrupt = iNfsInstance->Get_InterruptNow();	
	if (interrupt)
	{
		serializator = iNfsInstance->GetSerializator();
		flag = ENfsSieving;
		flag2 = ENfsSieverState;
	}	
#endif
	if (interrupt)
	{
		printf("(Advanced) line siever received request for serialization.\n");
		if (serializator)
		{
			ret = serializator->Serialize(flag);
			if (ret != ConstRC::Ok)
			{
				return ret;
			}
			ret = serializator->Serialize(flag2);
			if (ret != ConstRC::Ok)
			{
				return ret;
			}
		}
		return TERMINATED;
	}
	int result;
	int k;
	int block_max;
#ifdef QS_LINE_SIEVING
	qs_fb_type* start   = iQsFirstMiddlePrime;
	qs_fb_type* running;
	qs_fb_type* end     = iQsFirstLargishPrime;	
	//QuadraticSieve::PrintPolynomial(&iQsCurrentPolynomial);
#endif
#ifdef GNFS_LINE_SIEVING
	nfs_fb_type* start   = iNfsFirstIntegralMiddlePrime;
	nfs_fb_type* running;
	nfs_fb_type* end     = iNfsFirstIntegralLargishPrime;	
#endif
	iSievingClockStart = clock();
	iSievingNegative = FALSE;
	// Fill hashtable
	result = FillHashtable(this->iHashtable,this->iCaches, POSITIVE, FALSE); // FALSE = is not algebraic
	if (result != ConstRC::Ok)
	{
		return result;
	}
#ifdef GNFS_LINE_SIEVING
	result = FillHashtable(this->iAlgebraicHashtable,this->iAlgebraicCaches, POSITIVE, TRUE); // TRUE = is algebraic
	if (result != ConstRC::Ok)
	{
		return result;
	}

	iInitIntegralBlockValue  = 0x80 - iNfsInstance->ComputeExpectedIntegralLog(6);
	iInitAlgebraicBlockValue = 0x80 - iNfsInstance->ComputeExpectedAlgebraicLog(0);
#endif
#ifdef TIME_MESSAGE
	AbstractSieve::AddTime(iFillHashtableTotal,iSievingClockStart);
#endif
	// Positive sieving (from 1 to M)
	iCurrentBlockSetIndex = 0;
	int bStart = 0;
	int bEnd = iPositiveBlocks;
#ifdef QS_LINE_SIEVING
	if (this->iQsUseRootShort)
	{
		bStart = iQsPositiveLowestBlockSetIndex;
		bEnd = iQsPositiveHighestBlockSetIndex+1;
	}
#endif
	for (k=bStart; k< bEnd; k+= iBlockSetSize, iCurrentBlockSetIndex++) {
//#define BLOCK_SIEVE_DEBUG
#ifdef BLOCK_SIEVE_DEBUG
		printf("\n Block set sieve: positive k = %d start.\n",k);
#endif
		block_max = iBlockSetSize;		
		if ((iPositiveBlocks-k) < block_max)
		{
			block_max = iPositiveBlocks-k;
			//printf("Block max lesser than set: %d.\n",block_max);
		}
#ifdef QS_LINE_SIEVING
		if (this->iQsUseRootShort)
		{
			if (k == iQsPositiveHighestBlockSetIndex)
			{
				block_max = iQsPositiveEndSetIndex;
			}
		}
#endif
#ifdef TIME_MESSAGE
		iAuxStart = clock();
#endif
		// The sieving itself
		result = BlockSetSieve(block_max,POSITIVE, (k==0)); 
		if (result != ConstRC::Ok)
		{
			return result;
		}
#ifdef TIME_MESSAGE
		AbstractSieve::AddTime(iBlockSieveTotal,iAuxStart);
#endif
#ifdef BLOCK_SIEVE_DEBUG
		printf("\n Block set sieve: positive k = %d end.\n",k);
#endif
	} // end of for (k=0; k< iPositiveBlocks; k+= iBlockSetSize) cycle
	AbstractSieve::AddTime(iSievingClockTotal,iSievingClockStart);
	// Now, we will detect and save the smooth values in the positive section.
	
	iSievingClockStart = clock();
#ifdef QS_LINE_SIEVING
	// We will generate the new polynomial (for the next sieve line) now, in between
	// the positive and negative sieving, to facilitate the index change.
	iQsInstance->IncrementPolynomialCounter();
	result = iQsInstance->GenerateNewPolynomial();
	if (result != ConstRC::Ok)
	{
		return result;
	}	
	iQsInstance->PolyCopy(&iQsNextPolynomial);
	if (iQsUseRootShort)
	{
		iQsInstance->RootCopy(iQsNextRoot1,iQsNextRoot2);
	}
	if (iQsMode == EMpqs)
	{
		this->iQsMajorPolyChange = TRUE;
		iQsCounterSinceMPC = 0;
	}
	if (iQsMode == ESiqs) {
		if (mpz_cmp(iQsNextPolynomial.a,iQsCurrentPolynomial.a))
		{
			this->iQsMajorPolyChange = TRUE;
			iQsCounterSinceMPC = 0;
		}
		else 
		{
			this->iQsMajorPolyChange = FALSE;
			iQsCounterSinceMPC++;
			this->iQsSubtract = iQsInstance->Get_Subtract();
			this->iQsNu       = iQsInstance->Get_Nu();
		}
	}
#endif
#ifdef TIME_MESSAGE
	iAuxStart = clock();
#endif
	// Negative sieving, from -M to -1.
	result = FillHashtable(this->iHashtable,this->iCaches, NEGATIVE, FALSE); // FALSE = is not algebraic
	if (result != ConstRC::Ok)
	{
		return result;
	}
#ifdef GNFS_LINE_SIEVING
	result = FillHashtable(this->iAlgebraicHashtable,this->iAlgebraicCaches, NEGATIVE, TRUE); // TRUE = is algebraic
	if (result != ConstRC::Ok)
	{
		return result;
	}
#endif
#ifdef TIME_MESSAGE
	AbstractSieve::AddTime(iFillHashtableTotal,iAuxStart);
	AbstractSieve::AddTime(iFillHashtableNeg,iAuxStart);
#endif
	iSievingNegative = TRUE;
	iCurrentBlockSetIndex = 0;
	bStart = 0;
	bEnd = iNegativeBlocks;
#ifdef QS_LINE_SIEVING
	if (this->iQsUseRootShort)
	{
		bStart = iQsNegativeLowestBlockSetIndex;
		bEnd = iQsNegativeHighestBlockSetIndex+1;
	}
#endif
	for (k=bStart; k< bEnd; k+= iBlockSetSize, iCurrentBlockSetIndex++) {
#ifdef BLOCK_SIEVE_DEBUG
		printf("\n Block set sieve: negative k = %d start.\n",k);
#endif
		block_max = iBlockSetSize;		
		if ((iNegativeBlocks-k) < block_max)
		{
			block_max = iNegativeBlocks-k;
		}
#ifdef QS_LINE_SIEVING
		if (this->iQsUseRootShort)
		{
			if (k == iQsNegativeHighestBlockSetIndex)
			{
				block_max = iQsNegativeEndSetIndex;
			}
		}
#endif

#ifdef TIME_MESSAGE
		iAuxStart = clock();
#endif
		result = BlockSetSieve(block_max,NEGATIVE, (k == 0)); // init negative iff k == 0
		if (result != ConstRC::Ok)
		{
			return result;
		}			
#ifdef TIME_MESSAGE
		AbstractSieve::AddTime(iBlockSieveTotal,iAuxStart);
#endif
#ifdef BLOCK_SIEVE_DEBUG
		printf("\n Block set sieve: negative k = %d end.\n",k);
#endif
	} // end of for (k=0; k< iNegativeBlocks; k+= iBlockSetSize) cycle

	// Now, we will detect and save the smooth values in the positive section.
	/*
		Update roots of small and middle primes the next line ... already done for the largish primes
		in the FillHashtable method.
	*/
#ifdef QS_LINE_SIEVING
	running = iQsFactorBase+2; // omitting -1 and 2
	end = iQsLastMiddlePrime;
	main_sieving_type inverse_a;
	if (iQsMajorPolyChange)
	{
		mpz_mul_ui(aux_5,iQsNextPolynomial.a,2);
	}
#endif // QS_LINE_SIEVING
#ifdef GNFS_LINE_SIEVING
	running = iNfsIntegralFactorBase+2; // omitting -1 and 2
	end = iNfsLastIntegralMiddlePrime; 
#endif // GNFS_LINE_SIEVING
#ifdef TIME_MESSAGE
	iAuxStart = clock();
#endif
	// Shift the middle-sized integral elements
	// Identical code is present in FillHashtable for largish primes.
	do {
#ifdef QS_LINE_SIEVING
		if (iQsMajorPolyChange)
			{
				mpz_set_ui(aux_3,running->p);
				if (mpz_invert(aux_4,aux_5,aux_3))
				{
					inverse_a = mpz_get_ui(aux_4);
				}
				else
				{
					inverse_a = 0;
				}
				if (!inverse_a) {
#ifdef SIQS_SHIFT_DEBUG
					printf("Sieve Line: calculate new roots for %d, inverse a is %d.\n",running->p,inverse_a);	
#endif // SIQS_SHIFT_DEBUG
				}
				CalculateNewRoots(running,inverse_a);
			}
			// Shift the root in SIQS, when only 'b' is changed.
			 else 
			{
				//printf("Sieve Line: update SIQS roots for %d.\n",running->p);
				UpdateSiqsRoots(running);
			}
#endif // QS_LINE_SIEVING
#ifdef GNFS_LINE_SIEVING
		UpdateNfsRoot(running);	
#endif	// GNFS_LINE_SIEVING
		if (running == end)
		{
			break;
		}
		running++;
	} while (TRUE); // Will be broken by internal check
#ifdef GNFS_LINE_SIEVING
	// Shift the middle-sized algebraic ideals
	running = iNfsAlgebraicFactorBase; 
	end = iNfsLastAlgebraicMiddlePrime; 
	do {
		UpdateNfsRoot(running);
		if (running == end)
		{
			break;
		}
		running++;
	} while (TRUE); // Will be broken by internal check
#endif // GNFS_LINE_SIEVING
#ifdef TIME_MESSAGE
	AbstractSieve::AddTime(iRootsUpdateTotal,iAuxStart);
#endif
	iSievingClockDiff = clock() - iSievingClockStart;
	if (iSievingClockDiff >= 0)
	{
		iSievingClockTotal += iSievingClockDiff;
	}
#ifdef QS_LINE_SIEVING
	// We will shift the polynomials (next -> current)
	mpz_set(iQsCurrentPolynomial.a,iQsNextPolynomial.a);
	mpz_set(iQsCurrentPolynomial.b,iQsNextPolynomial.b);
	mpz_set(iQsCurrentPolynomial.c,iQsNextPolynomial.c);
	mpz_set(iQsCurrentPolynomial.d,iQsNextPolynomial.d);
	mpz_set(iQsCurrentPolynomial.d2,iQsNextPolynomial.d2);
	if (iQsUseRootShort)
	{
		iQsRoot1 = iQsNextRoot1;
		iQsRoot2 = iQsNextRoot2;
		result = SetupRootShort();
		if (result != ConstRC::Ok)
		{
			return result;
		}
	}
	int x1;
	for (x1 = 0; x1 < this->iQsSiqsSValue; x1++)
	{
		iQsCurrentDivisorsOfA[x1] = iQsNextDivisorsOfA[x1];
	}
	iLeastCurrentDivisorOfA     = iLeastNextDivisorOfA;	
	iGreatestCurrentDivisorOfA  = iGreatestNextDivisorOfA;	
#endif // QS_LINE_SIEVING
#ifdef BLOCK_SIEVE_DEBUG
	wait();
#endif

	return ConstRC::Ok;
}

#ifdef QS_LINE_SIEVING
void  LineSiever::WalkTheQsFactorBase()
{
	qs_fb_type* running = iQsFactorBase;
	while (running->p)
	{	
		printf("p = %6d, log = %4d.\n",running->p,running->log_p);
		running++;
	}
}
#endif

/**

	The hashtable principle:
	
	We have at least (iBlockSetSize+1) hashtables available.

	Of this, at most aBlockSetSize hashtables are meaningful
	at the moment. The last hashtable is used as an auxiliary
	variable.

	The greatest possible jump from current update to next update
	is by iBlockSetSize hashtables. If it lands in the same hashtable,
	we will rather update the last hashtable, and then switch the contents.
	

*/
int  LineSiever::BlockSetSieve(int aBlockMax, int aSign, bool aInit)
{
	int result;
	int i;
	int min = 0;
#ifdef QS_LINE_SIEVING
	qs_fb_type* start   = iQsFirstMiddlePrime; // iQsFactorBase+2
	qs_fb_type* end_middle = iQsLastMiddlePrime;
	if (this->iQsUseRootShort)
	{
		if (aSign == POSITIVE)
		{
			min = iQsPositiveStartSetIndex;
		}
		else if (aSign == NEGATIVE)
		{
			min = iQsNegativeStartSetIndex;
		}
	}
#endif
#ifdef GNFS_LINE_SIEVING
	nfs_fb_type* start = iNfsFirstIntegralMiddlePrime;
	nfs_fb_type* startAlg = iNfsFirstAlgebraicMiddlePrime;
	nfs_fb_type* end = NULL;
	nfs_fb_type* end_middle = iNfsLastIntegralMiddlePrime;
	nfs_fb_type* end_middle_alg = iNfsLastAlgebraicMiddlePrime;
	nfs_fb_type* running;
	hashtable_t* hashtableAlg = iAlgebraicHashtable;
#endif // GNFS_LINE_SIEVING

	for (i=min; i< aBlockMax; i++) 
	{			
		iCurrentBlockIndex = i;
#ifdef BLOCK_SIEVE_DEBUG		
		printf("BlockSetSieve: i = %d. Memset the block to 0x%x, size %d.\n",i,iInitIntegralBlockValue,iBlockSize*sizeof(log_type));
#endif
		
		// At first, we must reset the sieving block for integral sieving
		/*log_walking_type* cast = (log_walking_type*) iBlock;
		memset(cast,0,iBlockSize*sizeof(log_type));*/
		// iInitIntegralBlockValue was calculated in SieveLine by ComputeExpectedIntegralLog		
		if(memset(this->iBlock,iInitIntegralBlockValue,iBlockSize*sizeof(log_type)) == NULL)
		{
			CLog::LogError(WHEREARG,"Line siever","Block set sieve",0,iBlockSize*sizeof(log_type),"Memset failed!");
			return NOT_ENOUGH_MEMORY;
		} /*else {
			printf("iInitIntegralBlockValue is 0x%x.\n",iInitIntegralBlockValue);
		}*/
#ifdef BLOCK_SIEVE_DEBUG	
		
#endif		
		this->iSievingIntegral = TRUE;
		this->iIntermediateIntegersStart = clock();
		// Then, we sieve with the middle-sized primes.
		result = SieveWithMiddlePrimes(start,end_middle,iBlock,aSign, aInit && (i == 0));
		if (result != ConstRC::Ok) { return result; }
#ifdef BLOCK_SIEVE_DEBUG	
		printf("Sieve With Middle Primes Integral Finished. Calling Sieve With Largish Primes\n");
#endif
		// Then, we will sieve with the largish primes	
		result = SieveWithLargishPrimes(iHashtable[i].script, iBlock);
		if (result != ConstRC::Ok) { return result; }
#ifdef BLOCK_SIEVE_DEBUG	
		printf("Sieve With Largish Primes Integral Finished.\n");
#endif

		this->iSievingIntegral = FALSE;
		this->iIntermediateIntegers += (clock() - this->iIntermediateIntegersStart);
// In case of NFS, we must sieve with the algebraic prime ideals too
#ifdef GNFS_LINE_SIEVING
		this->iSievingAlgebraic = TRUE;
		// iInitAlgebraicBlockValue = iNfsInstance->ComputeExpectedAlgebraicLog(0); was called in SieveLine
		if(memset(this->iAlgebraicBlock,iInitAlgebraicBlockValue,iBlockSize*sizeof(log_type)) == NULL)
		{
			CLog::LogError(WHEREARG,"Line siever","Block set sieve",0,iBlockSize*sizeof(log_type),"Memset failed!");
			return NOT_ENOUGH_MEMORY;
		}
#ifdef BLOCK_SIEVE_DEBUG	
		printf("Calling Sieve With Middle Primes Algebraic.\n");
#endif
		this->iIntermediateAlgebraicStart = clock();
		// Then, we sieve with the middle-sized primes.
		result = SieveWithMiddlePrimes(startAlg,end_middle_alg,iAlgebraicBlock,aSign, aInit && (i == 0));
		if (result != ConstRC::Ok) { return result; }
#ifdef BLOCK_SIEVE_DEBUG	
		printf("Sieve With Middle Primes Algebraic Finished. Calling Sieve With Largish Primes\n");
#endif
		// Then, we will sieve with the largish primes	
		result = SieveWithLargishPrimes(iAlgebraicHashtable[i].script, iAlgebraicBlock);
		if (result != ConstRC::Ok) { return result; }
#ifdef BLOCK_SIEVE_DEBUG	
		printf("Sieve With Largish Primes Algebraic Finished.\n");
#endif

		this->iIntermediateAlgebraic += (clock() - this->iIntermediateAlgebraicStart);
		this->iSievingAlgebraic = FALSE;
#endif // GNFS_LINE_SIEVING
		// Now the factorizable candidates are found and factorized
		hashtable_t* rationalTable = &(iHashtable[i]);
		hashtable_t* algebraicTable = NULL;
#ifdef GNFS_LINE_SIEVING
		algebraicTable = &(iAlgebraicHashtable[i]);
#endif
#ifdef TIME_MESSAGE
		int diff = clock();
#endif
		result = FindAndFactorizeCandidates(rationalTable,algebraicTable);
		if (result != ConstRC::Ok) { return result; }
#ifdef TIME_MESSAGE
		AbstractSieve::AddTime(iFindAndFactorizeTotal,diff);
#endif
#ifdef BLOCK_SIEVE_DEBUG	
		printf("Calling Shift Largish Primes Integral.\n");
#endif
		// Now, we must shift the sieving values for the largish primes in the hashtable for the next block. This has already been done for the middle-sized primes.
		// An open question: is it better to do this apart from the sieving, or within the
		// largish sieving cycle?

		result = ShiftLargishPrimes(iHashtable[i].script,iHashtable,iCaches);

		if (result != ConstRC::Ok) { return result; }

#ifdef BLOCK_SIEVE_DEBUG	
		printf("Shift Largish Primes Integral Finished.\n");
#endif
#ifdef GNFS_LINE_SIEVING
#ifdef BLOCK_SIEVE_DEBUG	
		printf("Calling Shift Largish Primes Algebraic.\n");
#endif

		this->iShiftingAlgebraic = TRUE;
		result = ShiftLargishPrimes(iAlgebraicHashtable[i].script,iAlgebraicHashtable,iAlgebraicCaches);
		this->iShiftingAlgebraic = FALSE;
#ifdef BLOCK_SIEVE_DEBUG	
		printf("Shift Largish Primes Algebraic Finished.\n");
#endif

		SwapHashtable(iAlgebraicHashtable,i);
#endif
		// End of the hashtable shift
		SwapHashtable(iHashtable,i);

	} //for (i=0; i< aBlockMax; i++) 
	
	return ConstRC::Ok;
}

int   LineSiever::SieveWithLargishPrimes(ks_script_t* aScript, log_type* aBlock)
{
	ks_script_t* script = aScript;
	int j, max;
	int total = 0;
	int listIndex = 0;
	int runningIndex = 0;
	int errors = 0;
	while (script)
	{	
		max = script->num_used;
		//printf("List Index %d: num used is %d.\n",listIndex,max);
		total += max;
		listIndex++;
		ks_update_t *updates = script->list;
		for (j=0; j< max; j++)
		{
			if (errors >= 10)
			{
				return INCONSISTENT_STATE;
			}
	        	if (j%8==0) {
          			PREFETCH(updates+j+16);
			}
			//printf("\nAdding to bucket ... %5d[%d]",updates[j].p,updates[j].h);				
			if (this->iAssertionMode) {
#ifdef ONE_PRIME_DEBUG
				if (updates[j].p == 191251)
				{
					PrintUpdateInfo(updates[j]);
				}
#endif
#ifdef QS_LINE_SIEVING		
				if (!TestQs(updates[j].h,updates[j].p))
				{
					printf("(%6d): ",runningIndex);
					TestQs(updates[j].h,updates[j].p,TRUE);
					errors++;
				}
#endif
#ifdef GNFS_LINE_SIEVING
				if (updates[j].p >= 0)
				{
					nfs_fb_type running;					
					running.p = updates[j].p;
					running.c_p = updates[j].c_p;
					running.root_1 = updates[j].h;
					running.log_p = updates[j].logp;
					running.flags = 0; // Largish primes do not usually have flags
					bool result = TRUE;
					if (this->iSievingIntegral)
					{
						//printf("Testing NFS Integral ... %d.\n",updates[j].p);
						TestNfsIntegral(running,updates[j].h);
					}
					if (this->iSievingAlgebraic) {
						TestNfsAlgebraic(running,updates[j].h);
					}
				}
#endif
			}
			if (updates[j].h >= iBlockSize)
			{
				printf("\n..... Line siever, sieve with largish primes: index too big ... %5d[%d]",updates[j].p,updates[j].h);
				return INCONSISTENT_STATE;
			} else {
	        		aBlock[updates[j].h] += updates[j].logp;
#ifdef COUNT_HITS
				sum += updates[j].logp;
				hits++;
#endif
			}			
			runningIndex++;
/* 
			// Posun indexu
			printf("\n   The new offset will be %d",updates[j].h+updates[j].p);
			// Novy offset
			newHashtableIndex = (newUpdateH = (updates[j].h + updates[j].p)) / iBlockSize;
			printf("\n   Mezivysledek  index je %d.",newHashtableIndex);
			newHashtableIndex %= iBlockSetSize;
	
			printf("\n   Mezivysledek2 index je %d.",newHashtableIndex);		
			newUpdateH %= iBlockSize;
			if (newHashtableIndex == i)
			{
				printf("\n   Pujdeme do k+1 na indexu %d.",newHashtableIndex, newUpdateH);
				result = AddToTable(iHashtable,iCaches,updates[j].p,newUpdateH,updates[j].log,iBlockSetSize);
				

				if (res != ConstRC::Ok)
				{
					return result;
				}
			} else 
			{
	
				result = AddToTable(iHashtable,iCaches,updates[j].p,newUpdateH,updates[j].log,newHashtableIndex);
		
				if (res != ConstRC::Ok)
				{
					return result;
				}
			}				
*/				
		}
		script = script->next;
	}
#ifdef BLOCK_SIEVE_DEBUG
	if (hits) 
	{
		printf("Sieve largish primes: added total %d logs in %d hits (avg %d).\n",sum,hits,sum/hits);
	} else {
		printf("Sieve largish primes: No largish prime availables\n");
	}
#endif
	return ConstRC::Ok;
}

#ifdef QS_LINE_SIEVING

//#define SETUP_ROOT_SHORT_DEBUG

void   LineSiever::DetermineLoAndHiBlocks(main_sieving_type aRootAbs, int& loBlockIndex, int& hiBlockIndex)
{
	// Now, we will determine the lowest block to use.
	int ratio = aRootAbs / iBlockSize;
	loBlockIndex = ratio;
	hiBlockIndex = ratio;	
	int lo = ratio    *iBlockSize;
	int hi = (ratio+1)*iBlockSize;
	bool goLow = FALSE;
	bool goHigh = FALSE;
#ifdef SETUP_ROOT_SHORT_DEBUG
	printf("The ratio is %d, low = %d, high = %d.\n",ratio,lo,hi);
#endif
	if ((aRootAbs-lo) < (hi-aRootAbs))
	{
#ifdef SETUP_ROOT_SHORT_DEBUG
		printf("  Going lower, if possible.\n");
#endif
		goLow = TRUE;

	} else 
	{
#ifdef SETUP_ROOT_SHORT_DEBUG
		printf("  Going higher, if possible.\n");
#endif

		goHigh = TRUE;
	}
	if (goLow && (loBlockIndex > 0))
	{
		loBlockIndex--;
	}
	if (goHigh && (hiBlockIndex < (iPositiveBlocks-1)))
	{
		hiBlockIndex++;
	}
#ifdef SETUP_ROOT_SHORT_DEBUG
	printf("  We will operate on blocks %d to %d.\n",loBlockIndex,hiBlockIndex);
#endif
	
}

int   LineSiever::SetupRootShort()
{
	// At first, we will determine which two blocks will be used for sieving.
	// Or, perhaps, one block only.
	main_sieving_type positiveRoot = iQsRoot1;
	main_sieving_type negativeRoot = iQsRoot2;
	
	if (positiveRoot < 0)
	{
		positiveRoot = iQsRoot2;
		negativeRoot = iQsRoot1;
	}
	if (positiveRoot < 0)
	{
		printf("LineSiever: SetupRootShort. Both roots seem to be negative. Aborting.\n");
		return GENERAL_ERROR;
	} 
	if (negativeRoot > 0)
	{
		printf("LineSiever: SetupRootShort. Both roots seem to be positive. Aborting.\n");
		return GENERAL_ERROR;
	}
	main_sieving_type absoluteNegativeRoot = (-1)*negativeRoot;
	DetermineLoAndHiBlocks(positiveRoot,this->iQsPositiveLowestBlockIndex,this->iQsPositiveHighestBlockIndex);

	this->iQsPositiveLowestBlockSetIndex = this->iQsPositiveLowestBlockIndex / this->iBlockSetSize;
	this->iQsPositiveStartSetIndex = this->iQsPositiveLowestBlockIndex - this->iQsPositiveLowestBlockSetIndex*this->iBlockSetSize;

	this->iQsPositiveHighestBlockSetIndex = this->iQsPositiveHighestBlockIndex / this->iBlockSetSize;
	this->iQsPositiveEndSetIndex = this->iQsPositiveHighestBlockIndex - this->iQsPositiveHighestBlockSetIndex*this->iBlockSetSize;

	//printf("We will sieve from block %d in set %d to block %d in set %d.\n",this->iQsPositiveStartSetIndex,this->iQsPositiveLowestBlockSetIndex,this->iQsPositiveEndSetIndex,this->iQsPositiveHighestBlockSetIndex);
	
	DetermineLoAndHiBlocks(absoluteNegativeRoot,this->iQsNegativeLowestBlockIndex,this->iQsNegativeHighestBlockIndex);

	this->iQsNegativeLowestBlockSetIndex = this->iQsNegativeLowestBlockIndex / this->iBlockSetSize;
	this->iQsNegativeStartSetIndex = this->iQsNegativeLowestBlockIndex - this->iQsNegativeLowestBlockSetIndex*this->iBlockSetSize;

	this->iQsNegativeHighestBlockSetIndex = this->iQsNegativeHighestBlockIndex / this->iBlockSetSize;
	this->iQsNegativeEndSetIndex = this->iQsNegativeHighestBlockIndex - this->iQsNegativeHighestBlockSetIndex*this->iBlockSetSize;

	return ConstRC::Ok;
}

int   LineSiever::SieveWithMiddlePrimes(qs_fb_type* aStart, qs_fb_type* aEnd, log_type* aBlock, int aSign, bool aInit)
#endif
#ifdef GNFS_LINE_SIEVING
int   LineSiever::SieveWithMiddlePrimes(nfs_fb_type* aStart, nfs_fb_type* aEnd, log_type* aBlock, int aSign, bool aInit) 
#endif
{
	register main_sieving_type p;
	register main_sieving_type offset;
	register log_type          log;
	int shiftConst = 2;
#ifdef QS_LINE_SIEVING
	qs_fb_type* running;
#endif
#ifdef GNFS_LINE_SIEVING
	nfs_fb_type* running;
	if (iNfsCompressEvenIntervals && iIsCompressedNow)
	{
		shiftConst = 1;
	}
#endif
	running = aStart;
	
	do
	{
		p = running->p;		
		log    = running->log_p;
#ifdef BLOCK_SIEVE_DEBUG
		if (!log)
		{
			printf("Log is zero for p = %d, strange.\n",p);
		}
#endif
		if (running->root_1 >= 0) {				
#ifdef BLOCK_SIEVE_DEBUG
			//printf("SievingWithMiddlePrimes: p = %d, root = %d, curr = %p, end = %p.\n",p,running->root_1,running, aEnd);
#endif
			if (aInit)
			{
				if (aSign == NEGATIVE)
				{

					offset = (running->next_1 = p - running->root_1-shiftConst); // shiftConst is to adjust values and offsets for the negative sieve.
					if (offset < 0) {
						offset = (running->next_1 += p);
					}
				} else {
					offset = (running->next_1 = running->root_1);
				}
			} else {
				offset = running->next_1;
			}
			if (iAssertionMode)
			{	
#ifdef GNFS_LINE_SIEVING
				if (iSievingIntegral) 
				{
					//printf("Testing integral with offset %d.\n",offset);
					TestNfsIntegral(*running,offset);
				}
#endif
			}
			//printf("Block Sieve middle: the offset of %d should be %d (root is %d).\n",p,offset,running->root_1);
				/*if (!(iCurrentBlockIndex | iCurrentBlockSetIndex))
				{
					main_sieving_type mid = offset+1;
					if (aSign == NEGATIVE)
					{
						mid*=(-1);
						printf("Expected negative init value is %d\n",mid);
					}
					//printf("Block Sieve middle: Therefore, the root should be %d.\n",mid);
					mid = iQsInstance->Evaluate(&iQsCurrentPolynomial,mid,p);
					printf("   The result of mod is %d.\n",mid);
				}*/
			if (this->iAssertionMode) 
			{
#ifdef QS_LINE_SIEVING			
				if (running->root_2 == -1) 
				{
#ifdef SIQS_SHIFT_DEBUG
					printf("TestQs ");
					if (aSign == NEGATIVE)
					{
						printf("(NEGATIVE)");
					} else 
					{
						printf("(POSITIVE)");
					}
					printf(" in SieveWithMiddlePrimes for p=%d \n",p);
#endif
				}
				TestQs(offset,p);
#endif // QS_LINE_SIEVING
#ifdef GNFS_LINE_SIEVING
				if (offset >= 0)
				{
					
					bool result = TRUE;
					if (this->iSievingIntegral)
					{
						TestNfsIntegral(*running,offset);
					}
					if (this->iSievingAlgebraic) {
						TestNfsAlgebraic(*running,offset);
					}
				}
#endif
			}
			do {					
				*(aBlock+offset) += log;
#ifdef COUNT_HITS
				sum += log;
				hits++;
#endif
				offset += p;
			} while (offset < iBlockSize);
			// The new index for the next block
			running->next_1 = offset - iBlockSize;
			//printf(" The next mod %d would be %d, diff is %d ... test ...\n",p,offset, running->next_1);
			
		} // else the root is invalid
#ifdef QS_LINE_SIEVING			
		if (running->root_2 >= 0) {
			if (aInit)
			{
				if (aSign == NEGATIVE)
				{
					offset = (running->next_2 = p - running->root_2-2);
					if (offset < 0) {
						offset = (running->next_2 += p);
					}
				} else {
					offset = (running->next_2 = running->root_2);
				}
			} else {
				offset = running->next_2;
			}
			if (this->iAssertionMode)
			{
				TestQs(offset,p);
			}
			do {						
				aBlock[offset] += log;
#ifdef COUNT_HITS
				sum += log;
				//printf("%7d (%7d)",sum,log);
				hits++;
#endif
				offset += p;
			} while (offset < iBlockSize);
			running->next_2 = offset - iBlockSize;
		}
#endif
		// Finish sieving with middle primes.
		if (running  == aEnd)
		{
			break;
		}
		running++;
	} while (TRUE); // Will be broken by internal check.
#ifdef BLOCK_SIEVE_DEBUG
	printf("Sieve mid primes: added total %d logs in %d hits (%d on avg).\n",sum,hits,sum/hits);
#endif
	return ConstRC::Ok;
}


int   LineSiever::ShiftLargishPrimes(ks_script_t* aScript, hashtable_t* aTableList, ks_cache_t* aCacheList)
{
	main_sieving_type newHashtableIndex;
	main_sieving_type newUpdateH;
	main_sieving_type newCp;
	int j, max, result;
	int errors  = 0;
	ks_script_t* script = aScript;
#ifdef BLOCK_SIEVE_DEBUG
 	printf("Shift largish primes: current block index is %d.\n",iCurrentBlockIndex);
#endif
	while (script)
	{
		max = script->num_used;
		//printf("List Index %d: num used is %d.\n",listIndex,max);
		//total += max;
		//listIndex++;
		ks_update_t *updates = script->list;
		for (j=0; j< max; j++)
		{
			if (errors >= 10)
			{
				return INCONSISTENT_STATE;
			}
	       		if (j%8==0) {
          			PREFETCH(updates+j+16);
			}
			
			newHashtableIndex = iCurrentBlockIndex + (newUpdateH = (updates[j].h + updates[j].p)) / iBlockSize;
			/*if (updates[j].p == 65537)
			{
				printf("Shift largish primes: current h is %d, new h is %d.\n",updates[j].h,newUpdateH);	
				PrintUpdateInfo(updates[j]);		
			}*/
			newHashtableIndex %= iBlockSetSize;
			/*if (updates[j].p == 65537)
			{
				printf("Mod block is %d.\n",newHashtableIndex);			
			}*/
#ifdef ONE_PRIME_DEBUG
			/*if (isThisPrimeControlled(updates[j].p))
			{
				printf("------------------------------------\n");
				printf("Shift largish primes: p = %d, alg = %d, current h is %d, new h is %d.\n",updates[j].p, iSievingAlgebraic, updates[j].h,newUpdateH);	
				PrintUpdateInfo(updates[j]);		
				printf("Mod block is %d.\n",newHashtableIndex);	
			}		*/
#endif

			if (this->iAssertionMode)
			{
#ifdef QS_LINE_SIEVING		
				if ((!TestQs(updates[j].h,updates[j].p)) ||(updates[j].p == CRITICAL_PRIME))			
				{
					TestQs(updates[j].h,updates[j].p,TRUE);
					errors++;
				}	
				if ((!TestQs(newUpdateH,updates[j].p)) || (updates[j].p == CRITICAL_PRIME))
				{
					//TestQs(updates[j].h,updates[j].p,TRUE);
					printf("+: ");
					TestQs(newUpdateH,updates[j].p,TRUE);
					errors++;
				}
#endif
#ifdef GNFS_LINE_SIEVING
				/*if ((!TestNfs(updates[j].h,updates[j].p,FALSE)))
				{
					errors++;
				}*/
#endif
			}
				/*if (updates[j].p == 25073) {
					printf("New update index    je %6d.\n",newUpdateH);
					printf("New hashtable index je %6d.\n",newHashtableIndex);
					printf("Block set size je rove %6d.\n",iBlockSetSize);
				}*/
				//printf("\n   Mezivysledek  index je %d.",newHashtableIndex);				newHashtableIndex %= iBlockSetSize;
			//printf("\n   Mezivysledek2 index je %d.",newHashtableIndex);
			newUpdateH %= iBlockSize;
				/*if (updates[j].p == 25073) {
					printf("~~After mod:.\n");
					printf("New hashtable index je %6d.\n",newHashtableIndex);
					printf("New update h je roven  %6d.\n",newUpdateH);
				}*/
			if (newHashtableIndex == iCurrentBlockIndex)
			{
#ifdef BLOCK_SIEVE_DEBUG
				if (updates[j].p == CRITICAL_PRIME) {
						printf("\n   We will go to k+1 with prime %d at index %d.\n",updates[j].p, newUpdateH);
					}
#endif
#ifdef GNFS_LINE_SIEVING
				newCp = updates[j].c_p;
#endif
				if (newUpdateH >= updates[j].p) {
					printf("II. Root1 %d > p %d.\n",newUpdateH,updates[j].p);
				}
				
				result = AddToTable(aTableList,aCacheList,updates[j].p,newUpdateH,newCp,updates[j].logp,iBlockSetSize);
				if (result != ConstRC::Ok)
				{
					return result;
				}
			} else 
			{
#ifdef BLOCK_SIEVE_DEBUG
				if (updates[j].p == CRITICAL_PRIME) {
						printf("We will go to %d with prime %d at index %d. Block set size is %d. \n\n",newHashtableIndex,updates[j].p, newUpdateH,iBlockSetSize);
					}
#endif
#ifdef GNFS_LINE_SIEVING
				newCp = updates[j].c_p;
#endif
				result = AddToTable(aTableList,aCacheList,updates[j].p,newUpdateH,newCp,updates[j].logp,newHashtableIndex);
				if (result != ConstRC::Ok)
				{
					return result;
				}
			}
		}
		script = script->next;
	}
	// Flush all caches for all hashtables.
	int m, mx1 = this->iBlockSetSize;
	for (m=0; m<= mx1; m++) {
		FlushUpdateCache(&(aTableList[m]),&(aCacheList[m]));
	}
	return ConstRC::Ok;
}

void  LineSiever::WalkTheBlock()
{
	int i;
	for (i=0; i< iBlockSize; i++)
	{
		if (this->iBlock[i] == 0x80) { continue; }
	}
}

//#define BLOCK_DEBUG

int   LineSiever::FindAndFactorizeCandidates(hashtable_t* aRationalHashtable, hashtable_t* aAlgebraicHashtable)
{
	log_walking_type* cast  = (log_walking_type*) this->iBlock;
	log_walking_type  mask  = CHECK_MASK;
	int size = sizeof(log_walking_type);
	if (size == 8)
	{
		mask <<= ((size/2)*8);
		mask += CHECK_MASK;
	}
#ifdef BLOCK_DEBUG
	printf("Mask is %lx.\n",mask);
#endif
	int iters = iBlockSize / size;
	int offset = 0;
	int i, j;
	int mask_hits  = 0;
	int candidates = 0;
	int sizes[10];
	int result;
	memset(sizes,0,10*sizeof(int));
#ifdef BLOCK_DEBUG
	for (i=0; i< iBlockSize; i++)
	{
		if (this->iBlock[i] == 0x80) { continue; }
	}
#endif
	for (i=0; i< iters; i++)
	{
#ifdef BLOCK_DEBUG
		offset = size*i;	
		for (j=0; j< size; j++)
		{
			sum += iBlock[j+offset];
			aux = iBlock[j+offset] / 16;
			if ((aux >=0) && (aux <= 9))
			{
				sizes[aux]++;
			}
			if (iBlock[j+offset] & 0x80) 
			{
				//printf("Candidate 0x%2x at offset d.\n",iBlock[j+offset],offset+j);
				candidates++;
			}
		} 
#else
		if ((cast[i] & mask)  || (iCheckAllMode)) { 
			mask_hits++;
			offset = size*i;	
			for (j=0; j< size; j++)
			{
				
				if ((iBlock[offset+j] & 0x80) || (iCheckAllMode))
				{					
					if ((offset+j) >= iBlockSize)
					{
						printf("Size: %d, i = %d, offset = %d, j = %d, offset+j = %d, pointer = 0x%d, iters = %d.\n",size,i,offset,j,offset+j,iBlock[offset+j],iters);
						wait_for_Enter();
					}
#ifdef GNFS_LINE_SIEVING
					if ((iAlgebraicBlock[offset+j] & 0x80) || (iCheckAllMode))
#endif
					{
					// Try factorization
					/*if (iCheckAllMode)
					{
						printf("Check all mode, factor %d\n",offset+j);
					}*/
					result = Factor(offset+j,aRationalHashtable, aAlgebraicHashtable);			
					candidates++;		
					}
				} else 
				{
					if ((offset+j) >= iBlockSize)
					{
						printf("Size: %d, i = %d, offset = %d, j = %d, offset+j = %d, pointer = 0x%d, iters = %d.\n",size,i,offset,j,offset+j,iBlock[offset+j],iters);
						wait_for_Enter();
					}
				}							
			}
		} 
#endif
	}
#ifdef BLOCK_SIEVE_DEBUG
 	printf("Had %d mask hits and %d candidates for factorization. Size statistics:\n",mask_hits,candidates);
#endif
#ifdef BLOCK_DEBUG
	int total = 0;
	for (i=9; i>=0; i--)
	{
		printf("      to 0x%d0: %d.\n",i,sizes[i]);
		total += sizes[i];
	}
	printf("      total   : %7d.\n",total);
	printf("      sum     : %7d.\n",sum);
	printf("      default : %7d.\n",(iInitIntegralBlockValue)*iBlockSize);
	printf("      added   : %7d.\n",sum - (iInitIntegralBlockValue)*iBlockSize);
	if (1) return ConstRC::Ok;
#endif
/*	for (i=0; i< iters; i++)
	{
		printf("0x%16lx (%4d / %4d)\n",cast[i],i,iters);
		//printf("Cast[%d]: %lx, and: %lx.\n",i,cast[i],cast[i] & mask);
		if (cast[i] & mask)
		{
			mask_hits++;
			//printf("0x%lx ",cast[i]);	
			offset = size*i;
			for (j=0; j< size; j++)
			{
				if (iBlock[j+offset] & 0x80) 
				{
					//printf("Candidate 0x%2x at offset %d.\n",iBlock[j+offset],offset+j);
					candidates++;
				}
			} 
		}
		
	}
	printf("Had %d candidates in %d mask hits for factorization.\n",candidates,mask_hits);
	wait();*/
	return ConstRC::Ok;
}

void LineSiever::ResetRelation()
{
#ifdef GNFS_LINE_SIEVING	
	iNfsRelation->maximal_sieving_element_index  = -1;
	iNfsRelation->maximal_prime_ideal_index  = -1;					
	iNfsRelation->max_lp_index	= -1;
	iNfsRelation->max_lpideal_index	= -1;	
	iNfsRelation->type = 0;
	iNfsRelation->a = 0;
	iNfsRelation->b = 0;
	mpz_set_si(iNfsRelation->integralRemaining,0);
	mpz_set_si(iNfsRelation->algebraicNormRemaining,0);
#endif
}

void LineSiever::SetupRelation(int aX)
{
#ifdef QS_LINE_SIEVING
	// We will init the iQsRelation data
	mpz_set_si(iQsRelation->argument,aX); 
	iQsRelation->lprime1 = 1;	// Setting all x-partial-specific values to 1			
	iQsRelation->lprime2 = 1;					
	// The number to factor ...
	mpz_mul(iQsRelation->remaining,iQsRelation->argument,iQsCurrentPolynomial.a);	// ax
	mpz_mul_2exp (iQsRelation->remaining,iQsRelation->remaining,1); // 2ax
	mpz_add(iQsRelation->remaining,iQsRelation->remaining,iQsCurrentPolynomial.b); // 2ax+b
	mpz_set(iQsRelation->left_side_value,iQsRelation->remaining); // square root on the left side
	mpz_mul(iQsRelation->remaining,iQsRelation->remaining,iQsRelation->remaining); // (2ax+b)^2
	mpz_sub(iQsRelation->remaining,iQsRelation->remaining,iFactoredValue); // (2ax+b)^2 - kN; factorized value on the right side
	if (iAssertionMode){
		if (!mpz_divisible_p(iQsRelation->remaining,iQsCurrentPolynomial.a)) {
			printf("LineSiever.SetupRelation: Assertion failed:  a does not divide Q(x)!\n");
		}
	}
	// The factorized value (2ax+b)^2 - kN is always divisible by 'a'. We factor only the quotient.
	mpz_fdiv_q(iQsRelation->remaining,iQsRelation->remaining,iQsCurrentPolynomial.a); //  ((2ax+b)^2 mod kN)/a		
	if (iQsMode == EMpqs) 
	{ 						
		mpz_set(iQsRelation->right_side_value,iQsCurrentPolynomial.d); // MPQS: a square factor on the right side.		
	}
	 else 
	{
		mpz_set_ui(iQsRelation->right_side_value,1); // Siqs: no square factor on the right side. Factors of a will be among small divisors of the right side.
	}
	mpz_set(iQsRelation->factorized_value,iQsRelation->remaining);
#endif
#ifdef GNFS_LINE_SIEVING
	iNfsRelation->maximal_sieving_element_index  = -1;
	iNfsRelation->maximal_prime_ideal_index  = -1;					
	iNfsRelation->max_lp_index	= -1;
	iNfsRelation->max_lpideal_index	= -1;	
	switch (iNfsInstance->GetSievingDirection()) 
	{
		case EIncrementA:
		case EDecrementA:							
			iNfsRelation->a = iNfsInstance->GetA();
			iNfsRelation->b = aX;
			break;
		case EIncrementB:
		case EDecrementB:
			iNfsRelation->a = aX;
			iNfsRelation->b = iNfsInstance->GetB();
			break;			
	} // end of switch
#ifdef FACTOR_DEBUG
	printf("Setup relation: a = %d, b = %d.\n",iNfsRelation->a,iNfsRelation->b);
#endif
	iNfsRelation->polynomial->Set(0,iNfsRelation->a);				
	iNfsRelation->polynomial->Set(1,iNfsRelation->b);
	iNfsInstance->GetM(iNfsRelation->integralPart);	
	mpz_mul_si(iNfsRelation->integralPart,iNfsRelation->integralPart,iNfsRelation->b);
	if (iNfsRelation->a > 0) 
	{
		mpz_add_ui(iNfsRelation->integralPart,iNfsRelation->integralPart,iNfsRelation->a);
	} else 
	{
		mpz_sub_ui(iNfsRelation->integralPart,iNfsRelation->integralPart,abs(iNfsRelation->a));
	}			
	mpz_set(iNfsRelation->integralRemaining,iNfsRelation->integralPart);
#endif	
}

//#define FACTOR_DEBUG

/**
	This function will perform complete factorization of a QS relation, or of the integral
	part of a NFS relation. The algebraic part of a NFS relation will be factored separately.

*/
int  LineSiever::Factor(int aOffset,hashtable_t* aRationalHashtable, hashtable_t* aAlgebraicHashtable)
{
	int shiftConst = 2;

#ifdef GNFS_LINE_SIEVING	
	ResetRelation();
	if (iNfsCompressEvenIntervals && iIsCompressedNow)
	{
		shiftConst = 1;
	}
	sieving_direction dir = iNfsInstance->GetSievingDirection();
	bool direction_a = ((dir = EIncrementA) || (dir == EDecrementA));	
#endif	
	int last_written_factor = -1;
	int result;
	int exponent;
	main_sieving_type prime;
	main_sieving_type rem;	
	main_sieving_type root_1;
	main_sieving_type root_2;
	// This will be comparable (via mod) to the root ...
	main_sieving_type auxiliaryOffset = iCurrentBlockSetIndex*iBlockSize*iBlockSetSize + iCurrentBlockIndex*iBlockSize + aOffset;
	// At first, we will calculate the relation-related values
	int x = iCurrentBlockSetIndex*iBlockSize*iBlockSetSize + iCurrentBlockIndex*iBlockSize + aOffset;
#ifdef GNFS_LINE_SIEVING	
	if (iNfsCompressEvenIntervals && iIsCompressedNow)
	{
		x *= 2;
	}
#endif
	x += 1;
	if (iSievingNegative)
	{
		x *= -1;				
#ifdef FACTOR_DEBUG
		printf("\n");
		printf("======= NEGATIVE ======= ");
#endif
	} 
	else {
#ifdef FACTOR_DEBUG
		printf("\n");
		printf("======= POSITIVE ======= ");
#endif
	}
#ifdef FACTOR_DEBUG
	printf("Factor: x is equal to %d.\n",x);
#endif

#ifdef QS_LINE_SIEVING
	qs_fb_type* running = iQsFactorBase+2;
	qs_fb_type* moddivision_end = iQsLastMiddlePrime;

	qs_fb_type* assertionStart = iQsFirstLargishPrime;
	SetupRelation(x);
	mpz_t* remaining = &iQsRelation->remaining;	
#endif // QS_LINE_SIEVING

#ifdef GNFS_LINE_SIEVING
	
	nfs_fb_type* running = iNfsIntegralFactorBase+2;
	nfs_fb_type* moddivision_end = iNfsLastIntegralMiddlePrime;	
	nfs_fb_type* assertionStart = iNfsFirstIntegralLargishPrime;
	SetupRelation(x);
	iNfsInstance->IncreaseTested();
#ifdef FACTOR_DEBUG
	printf("Factor: [a,b] = [%d,%d].\n",iNfsRelation->a,iNfsRelation->b);
#endif
	
	if (!TestCoprimality())
	{
		iNfsInstance->IncreaseRejected();
		iNfsInstance->IncreaseNotCoprime();
		return ConstRC::Ok;
	}
	mpz_t* remaining = &iNfsRelation->integralRemaining;
#endif
	// The integral part, common for NFS and MPQS/SIQS
#ifdef FACTOR_DEBUG
	gmp_printf("Factor: remaining is %Zd.\n",remaining);
#endif

#ifdef QS_LINE_SIEVING
	if (this->iQsMode == ESiqs)
	{
		// We will add all divisors of 'a' to the resulting array.
		int i, max = this->iQsSiqsSValue;
		for (i=0; i< max; i++)
		{
			prime = this->iQsCurrentDivisorsOfA[i];
			// The exponent may be larger than 1; in this case,
			// we divide out (p^(exp-1)). The last remaining power
			// of p is contained in the 'a' value.
			exponent = 1;
			/*while( ! mpz_fdiv_qr_ui(aux_1,aux_2,*remaining,prime) )
			{
				exponent++;
				iTotalDivrem++;
				mpz_swap(*remaining,aux_1);
			} */
			result = AddIntegralDivisor(prime, exponent, last_written_factor);
			if (result != ConstRC::Ok) { return result; }   
#ifdef FACTOR_DEBUG           		
			printf("Factor: added SIQS 'a' divisor: %5d\n",this->iQsCurrentDivisorsOfA[i]);
#endif
		}	
	}
#endif // QS_LINE_SIEVING
	// At first, we will take care of -1.
	if (mpz_cmp_si(*remaining,0) < 0) 
	{
		prime = -1;
		exponent = 1;
		result = AddIntegralDivisor(prime,exponent,last_written_factor);
		if (result != ConstRC::Ok) { return result; }              		
		mpz_mul_si(*remaining,*remaining,-1);
#ifdef FACTOR_DEBUG
		gmp_printf("Factor: was negative, added -1 to list.\n");
#endif
	}
	exponent = mpz_scan1(*remaining,0);
	if (exponent)
	{
		prime = 2;
		result = AddIntegralDivisor(prime,exponent,last_written_factor);
		if (result != ConstRC::Ok) { return result; }              		
#ifdef FACTOR_DEBUG
#ifdef QS_LINE_SIEVING
		gmp_printf("Factor: Reduced %Zd by 2^%d.\n",iQsRelation->remaining,exponent);
		//wait();
#endif
#ifdef GNFS_LINE_SIEVING
		gmp_printf("Factor: Reduced %Zd by 2^%d.\n",iNfsRelation->integralRemaining,exponent);
		//wait();
#endif
#endif // FACTOR_DEBUG
		mpz_fdiv_q_2exp(*remaining,*remaining,exponent);				
	}
	
#ifdef QS_LINE_SIEVING
	if (iAssertionMode)
	{
		if (exponent < 2)
		{
			printf("Line Siever, factor: assertion failed: Q(x) not divisible by 4!\n");		
		}
	}
	
#endif
	// Classic sieve factorization vs. hashing sieve factorization for tiny and middle primes
	if (!iUseHashingForMiddlePrimes) {
		/* Now, the tiny primes and middle primes*/
		while (running <= moddivision_end)
		{
			// Compare the required 
			prime = running->p;
#ifdef GNFS_LINE_SIEVING
			// We must specially solve the p's for which p | m.			
			if (direction_a)
			{
				if (running->flags & DIVIDES_M_MINI_FLAG)
				{
					//printf("Factor: prime %d divides M.\n",prime);
					if (!(iNfsRelation->a % prime))
					{
						exponent = 0;			
						while( ! mpz_fdiv_qr_ui(aux_1,aux_2,*remaining,prime) ){
							exponent++;
							mpz_swap(*remaining,aux_1);
							iTotalDivrem++;
						} 
						if (exponent) 	
						{
							result = AddIntegralDivisor(prime,exponent,last_written_factor);
							if (result != ConstRC::Ok) { 
								return result; 
							}      
						} else {
							
							gmp_printf("\nLine: WHEREARG: FactorError: reduction of %Zd by %d | m impossible: exponent %d, x = %d, aOffset = %d, auxiliaryOffset = %d, rem = %d, root_1 = %d, running_root1 = %d, condition1 = %d\n",*remaining,prime,exponent,x,aOffset,auxiliaryOffset,rem,root_1,running->root_1,(rem == root_1));
						}
					}
					running++;
					continue;
				}
			}
#endif
			if (this->iSievingNegative) {
				rem = auxiliaryOffset % prime;
				// if (valid root 1)
				if (running->root_1 >= 0) 
				{
					root_1 = prime - running->root_1 - shiftConst;
					if (root_1 < 0) {
						root_1 += prime;
					}
				} else 
				{
					root_1 = -1;
				}
#ifdef QS_LINE_SIEVING
				// if (valid root 2)
				if (running->root_2 >= 0) 
				{
					root_2 = prime - running->root_2 - shiftConst;
					if (root_2 < 0) {
						root_2 += prime;
					}
				} else 
				{
					root_2 = -1;
				}
#endif
			} else {
				rem = auxiliaryOffset % prime;
				root_1 = running->root_1;
#ifdef QS_LINE_SIEVING
				root_2 = running->root_2;
#endif
 			}
			if (rem < 0) { rem += prime; }
			/*if ((prime == 173) && (aOffset < 1000))
			{
				printf("P = %d, rem = %d, root_1 = %d\n",prime,rem,root_1);
			}*/

#ifdef QS_LINE_SIEVING
			if ((rem == root_1) || (rem == root_2))
#endif
#ifdef GNFS_LINE_SIEVING	
			if (rem == root_1)
#endif			
			{
				exponent = 0;
			
				while( ! mpz_fdiv_qr_ui(aux_1,aux_2,*remaining,prime) ){
					exponent++;
					iTotalDivrem++;
					mpz_swap(*remaining,aux_1);
				} 
				if (exponent) 	
				{
#ifdef FACTOR_DEBUG
					gmp_printf("Factor: reducing %Zd by %d^%d.\n",*remaining,prime,exponent);
#endif
					bool wasAdded = FALSE;
#ifdef QS_LINE_SIEVING
					if (this->iQsMode == ESiqs)
					{
						// Check whether this divisor is a SIQS a-divisor, if so, increase the exponent only.
						wasAdded =  InsertSiqsDivisor(prime, exponent);
					}
#endif
					if (!wasAdded) {
						result = AddIntegralDivisor(prime,exponent,last_written_factor);
						if (result != ConstRC::Ok) { return result; }            
					} else 	
					{
#ifdef FACTOR_DEBUG
						//if ((prime == 173) || (prime == 157) || (prime == 163))
						{

							printf("Prime %d was already present as SIQS 'a' divisor\n. Root1 = %d, Root2 = %d. x = %d, aOff= %d, aAuxOff = %d. Poly #%d\n",prime, root_1, root_2, x,aOffset,auxiliaryOffset,iQsRelation->index_of_used_polynomial);
							
						}
#endif
					}
				} 
				else 
				{
#ifdef QS_LINE_SIEVING
					if (root_2 == -1)
					{
						printf("\nFactorError: prime %d probably divides 'a'",prime);
						PrintDivisorsOfA();
						
					} else 
					{
						printf("\nFactorError: prime %d probably does not divide 'a'\n",prime);					
					}
					iQsInstance->PrintPolynomial(&iQsCurrentPolynomial);
					printf("Iters since last major polynomial change: %d",iQsCounterSinceMPC);
					main_sieving_type after = iQsInstance->EvaluateWithPrint(&iQsCurrentPolynomial,running->root_1 + 1,prime);
					if (after)
					{
						printf("FactorError: error determining root 1 for %d ... result is %d\n",prime, after);
					}
					else
					{
						printf("Oddly enough, the root seems to be correct.\n");
					}
					main_sieving_type after2 = iQsInstance->EvaluateWithPrint(&iQsCurrentPolynomial,x,prime);
					if (after2)
					{
						printf("FactorError: for x = %d, polynomial is not divisible by for %d ... result is %d\n",x, prime, after);
					} else
					{
						printf("Oddly enough, the polynomial value seems to be correct.\n");
					}
					gmp_printf("\nLine: WHEREARG: FactorError: reduction of %Zd by %d impossible: exponent %d, x = %d, aOffset = %d, auxiliaryOffset = %d, rem = %d, root_1 = %d, running_root1 = %d, root_2 = %d, running_root2 = %d, condition1 = %d, condition2 = %d, total condition = %d\n",*remaining,prime,exponent,x,aOffset,auxiliaryOffset,rem,root_1,running->root_1,root_2,running->root_2,(rem == root_1), (rem == root_2),(rem == root_1) || (rem == root_2));
					exit(1);
#endif
#ifdef GNFS_LINE_SIEVING
					gmp_printf("\nLine: WHEREARG: FactorError: reduction of %Zd by %d impossible: exponent %d, x = %d, aOffset = %d, auxiliaryOffset = %d, rem = %d, root_1 = %d, running_root1 = %d, condition1 = %d\n",*remaining,prime,exponent,x,aOffset,auxiliaryOffset,rem,root_1,running->root_1,(rem == root_1));
#endif
				}			
			}
			// Not divisible by p
			else if (iAssertionMode)
			{

				rem = mpz_mod_ui(aux_1,*remaining,prime);
				if (!rem)
				{
#ifdef QS_LINE_SIEVING
					gmp_printf("FactorError: the value %Zd should be divisible by %d, but offset indicates otherwise. aOffset = %d, x = %d. Auxiliary offset is %d, rem is %d, root1 is %d, root2 is %d \n",*remaining,prime,aOffset,x,auxiliaryOffset,rem,running->root_1,running->root_2);
#endif
#ifdef GNFS_LINE_SIEVING
					gmp_printf("FactorError: the value %Zd should be divisible by %d, but offset indicates otherwise. aOffset = %d, x = %d. Auxiliary offset is %d, rem is %d, root1 is %d, c_p is %d \n",*remaining,prime,aOffset,x,auxiliaryOffset,rem,running->root_1,running->c_p);
#endif
				}
 
			}
			running++;
		}
	}
	// Tiny and middle primes by second-level hashing. 
	else 
	{
	}
	// Now, the largish primes. We will walk through the current update table ...
	if (!iUseMirrorTableForLargishPrimes)
	{
		if (aRationalHashtable)
		{
			ks_script_t* script = aRationalHashtable->script;
			while (script)
			{
				int k, max = script->num_used;
				ks_update_t *updates = script->list;
				for (k=0; k< max; k++)
				{
					if (updates[k].h == aOffset)
					{
						prime = updates[k].p;
						//gmp_printf("Factor: %Zd should be divisible by %d.\n",*remaining,updates[k].p);
						exponent = 0;			
						while( ! mpz_fdiv_qr_ui(aux_1,aux_2,*remaining,prime) ){
							exponent++;
							iTotalDivrem++;
							mpz_swap(*remaining,aux_1);
						} 
						if (exponent) 	
						{
#ifdef FACTOR_DEBUG
							gmp_printf("Factor: reducing %Zd by %d^%d.\n",*remaining,prime,exponent);
#endif
							bool wasAdded = FALSE;
#ifdef QS_LINE_SIEVING
							if (this->iQsMode == ESiqs)
							{
								// Check whether this divisor is a SIQS a-divisor, if so, increase the exponent only.
								wasAdded =  InsertSiqsDivisor(prime, exponent);
							}
#endif
							if (!wasAdded) {
								result = AddIntegralDivisor(prime,exponent,last_written_factor);
								if (result != ConstRC::Ok) { return result; }            
							} else 	
							{
#ifdef FACTOR_DEBUG
								printf("Prime %d was already present as SIQS 'a' divisor.",prime);
#endif
							}
						} 
						else 
						{
							gmp_printf("\nLine WHEREARG:  FactorError: reduction of %Zd by %d impossible: exponent %d, x = %d, aOffset = %d, auxiliaryOffset = %d, updates[k]= %d.\n",*remaining,prime,exponent,x,aOffset,auxiliaryOffset,updates[k].h);
						}			
					} else 
					{

					}
				}
				script = script->next;
			}
		}
	} else 
	{
		// Mirror table for large primes
	}
	if (iAssertionMode)
	{
		//gmp_printf("Factor: assertion for large primes. Remaining %Zd, ass start %p\n",*remaining,assertionStart);
		if (mpz_cmp_ui(*remaining,1) && (assertionStart))
		{
			
			// We will try all remaining large primes
			running = assertionStart;
			while (running != NULL)
			{
				prime = running->p;
				//printf("%7d",prime);	fflush(stdout);
				if (!prime) {
					break;
				}
				if (mpz_mod_ui(aux_1,*remaining,prime) == 0)
				{
					gmp_printf("\nFactorIntegralError: remaining %Zd is divisible by largish prime %d, but this prime hasn't been foung during the process.\n",*remaining,prime);
					printf("Now searching for position of the prime in hashtables ....\n");
					// Search
					wait_for_Enter();
				}
				running++;
			}
		}
	}
	// Now, we have factored the relation with all rational factor base primes. But we
	// need to find potential large primes too.

#ifdef GNFS_LINE_SIEVING
	result = FactorAlgebraic(aOffset,auxiliaryOffset,aAlgebraicHashtable);
	if (result != ConstRC::Ok)
	{
		return result;
	}
	bool one = FALSE;
	if (!mpz_cmp_ui(iNfsRelation->integralRemaining,1))
	{
		one = TRUE;
	}
	//iNfsInstance->PrintNFSRelation(iNfsRelation);
	bool success = iNfsInstance->DetermineTypeAndSaveRelation(iNfsRelation, one, iNfsRelation->a, iNfsRelation->b);
	if (success)
	{
		
	} else 
	{
		iNfsInstance->IncreaseRejected();
	}
	

#endif	
#ifdef QS_LINE_SIEVING
	result = iQsInstance->SaveCandidate(iQsRelation,iAccepted,iRejected);
	if (result != ConstRC::Ok)
	{
		return result;
	}
#endif
	return ConstRC::Ok;
}
#ifdef QS_LINE_SIEVING
void  LineSiever::PrintDivisorsOfA()
{
	int j;
	printf("\nDivisors of 'a': ");	
	for (j=0; j < iQsSiqsSValue; j++)
	{
		printf(" %8d,",iQsCurrentDivisorsOfA[j]);		
	}
}
#endif

#ifdef GNFS_LINE_SIEVING
int   LineSiever::FactorAlgebraic(main_sieving_type aOffset, main_sieving_type aAuxiliaryOffset, hashtable_t* aAlgebraicHashtable)
{
	int shiftConst = 2;
	if (iNfsCompressEvenIntervals && iIsCompressedNow)
	{
		shiftConst = 1;
	}
	int last_written_factor = -1;
	int result;
	sieving_direction dir = iNfsInstance->GetSievingDirection();
	IntegralDomain* domain = iNfsInstance->GetDomain();
	domain->ABaseOfPrincipal(iIntegralBase,iNfsRelation->a,iNfsRelation->b);
	domain->Norm(iNfsRelation->algebraicNorm,iNfsRelation->a,iNfsRelation->b);
	mpz_abs(iNfsRelation->algebraicNorm,iNfsRelation->algebraicNorm);
	mpz_set(iNfsRelation->algebraicNormRemaining,iNfsRelation->algebraicNorm);
#ifdef FACTOR_DEBUG
	gmp_printf("Algebraic norm is equal to: %Zd, a = %d, b = %d.\n",iNfsRelation->algebraicNorm,iNfsRelation->a,iNfsRelation->b);
#endif
	nfs_fb_type* running = iNfsAlgebraicFactorBase;
	nfs_fb_type* moddivision_end = iNfsLastAlgebraicMiddlePrime;
	nfs_fb_type* assertionStart = iNfsFirstAlgebraicLargishPrime;
	bool direction_a = ((dir = EIncrementA) || (dir == EDecrementA));	
	main_sieving_type exponent = 0, prime, rem, root_1;
	mpz_t* remaining = &iNfsRelation->algebraicNormRemaining;
	int special_index = 0;
	if (!iUseHashingForMiddlePrimes) {
		/* Now, the tiny primes and middle primes*/
		while (running <= moddivision_end)
		{
			prime = running->p;
			exponent = 0;
			//printf("Prime %d.\n",prime);
			if (!(running->flags & SPECIAL_PRIME_MINI_FLAG)) 
			{
				//printf("Not a special prime. \n");
				if (direction_a && (running->flags & DIVIDES_C_P_MINI_FLAG))
				{
					//printf("Prime %d divides c_p. \n",prime);
					// In this case, prime divides norm iff prime divides current 'a'.
					main_sieving_type a = iNfsRelation->a;
					if (! (a % prime))
					{
									
						while( ! mpz_fdiv_qr_ui(aux_1,aux_2,*remaining,prime) )
						{
							exponent++;
							iTotalDivrem++;
							mpz_swap(*remaining,aux_1);
						}
						// Some serious error happened in the algorithm 
						if (!exponent)
						{
							// TODO: elegantneji
							if ((shiftConst != 1) || (prime !=  2))
							gmp_printf("\nFactorAlgebraic Error: reduction of %Zd by %d impossible: exponent %d, aAuxiliaryOffset = %d. c_p divides case.\n",*remaining,prime,exponent,aAuxiliaryOffset);
						} else {
							//printf("Added exponent %d for %d.\n",exponent,prime);
						}
					}
				} 
				else // does not divide c_p. Normal test
				{
					//if (prime == 3) printf("3 does not divide c_p,\n");
					if (this->iSievingNegative) 
					{
						rem = aAuxiliaryOffset % prime;
						// if (valid root 1)
						if (running->root_1 >= 0) 
						{
							root_1 = prime - running->root_1 - shiftConst;
							if (root_1 < 0) {
								root_1 += prime;
							}
						} else 
						{
							root_1 = -1;
						}
					} else { // sieving positive
						rem = aAuxiliaryOffset % prime;
						root_1 = running->root_1;
		 			}		
					if (rem < 0) { rem += prime; }
					//printf("Rem = %d.\n",rem);
					/*if (prime == 13)
					{
						printf("for a=%d, b=%d:\n",iNfsRelation->a,iNfsRelation->b);
						printf("prime = %d\n",prime);
						printf("auxOf = %d\n",aAuxiliaryOffset);
						printf("root1 = %d\n",root_1);
						printf("rem   = %d\n",rem);
					}*/
					if (rem == root_1)
					{
						//printf(" rem = %d = root_1.\n",rem);
									
						while( ! mpz_fdiv_qr_ui(aux_1,aux_2,*remaining,prime) ){
							exponent++;
							iTotalDivrem++;
							mpz_swap(*remaining,aux_1);
						} 
						if (exponent) 	
						{
#ifdef FACTOR_DEBUG
							gmp_printf("FactorAlgebraic: reducing %Zd by %d^%d.\n",*remaining,prime,exponent);
#endif
						} 
						else  // some problem has occurred
						{
							// TODO: elegantneji
							if ((shiftConst != 1) || (prime!=2) )
							gmp_printf("\nFactorAlgebraic Error: reduction of %Zd by %d impossible: exponent %d, aAuxiliaryOffset = %d. c_p divides case.\n",*remaining,prime,exponent,aAuxiliaryOffset);

						}	
					} // if rem == root_1
				}
				// Save the divisor if appropriate
				if (exponent)
				{
					AddAlgebraicDivisor(prime, exponent, running->c_p, 0,last_written_factor);
					exponent = 0;
				}
			} // normal prime ideal
			else if (running->flags & SPECIAL_PRIME_MINI_FLAG)
			{
				//printf("Now valuating special prime %d.\n",prime);
				long valuation;
				int ret = domain->ValuationOfPrime(&valuation,iNfsInstance->GetSpecialPrimeIdeal(special_index), iIntegralBase);
				if (ret != ConstRC::Ok)
				{
					return ret;
				}
				//printf("Valuation of ideal with prime %d is %ld.\n",prime,valuation);
				exponent = valuation;
				if (exponent > 0) {
					//printf("Is special prime. \n");
						
#ifdef FACTOR_DEBUG
					gmp_printf("FactorAlgebraic: reducing %Zd by a special prime %d^%d.\n",*remaining,prime,exponent);
					//wait();
#endif
						 
					while( ! mpz_fdiv_qr_ui(aux_1,aux_2,*remaining,prime) ){						
						mpz_swap(*remaining,aux_1);
						iTotalDivrem++;
					} 			
					AddAlgebraicDivisor(prime, exponent, running->c_p, SPECIAL_FLAG ,last_written_factor);
					
				}
				// We rely on the fact that special primes are sorted.
				special_index++;
			} // special prime ideal
			running++;
		} // while (running <= moddivison_end
	} // end of !iUseHashingForMiddlePrimes
	// Now, the largish primes. We will walk through the current update table ...
	if (!iUseMirrorTableForLargishPrimes)
	{
		if (aAlgebraicHashtable)
		{
			ks_script_t* script = aAlgebraicHashtable->script;
			while (script)
			{
				int k, max = script->num_used;
				ks_update_t *updates = script->list;
				for (k=0; k< max; k++)
				{
#ifdef ONE_PRIME_DEBUG
					if (updates[k].p == 191251){
						printf("Factor algebraic script message:\n");						
						PrintUpdateInfo(updates[k]);
						wait();
					}
#endif
					if (updates[k].h == aOffset)
					{
						prime = updates[k].p;
						//gmp_printf("FactorAlgebraic: %Zd should be divisible by %d.\n",*remaining,updates[k].p);
						exponent = 0;			
						while( ! mpz_fdiv_qr_ui(aux_1,aux_2,*remaining,prime) ){
							exponent++;
							iTotalDivrem++;
							mpz_swap(*remaining,aux_1);
						} 
						if (exponent) 	
						{
#ifdef FACTOR_DEBUG
							gmp_printf("FactorAlgebraic: reducing %Zd by %d^%d.\n",*remaining,prime,exponent);
#endif
							result = AddAlgebraicDivisor(prime, exponent, updates[k].c_p, 0,last_written_factor);
							exponent = 0;
							if (result != ConstRC::Ok) { 
								return result; 
							}            		
						} 
						else 
						{
							
							gmp_printf("\nFactorAlgebraicError: reduction of %Zd by %d impossible: exponent %d, auxiliaryOffset = %d, updates[k]= %d.\n",*remaining,prime,exponent,aAuxiliaryOffset,updates[k].h);
							iNfsInstance->PrintNFSRelation(iNfsRelation);
							wait();
						}			
					} else 
					{

					}
				}
				script = script->next;
			}
		}
	} else 
	{
		// Mirror table for large primes
	}
	if (iAssertionMode)
	{
		//gmp_printf("Factor: assertion for large prime ideals. Remaining %Zd, ass start %p\n",*remaining,assertionStart);
		if (mpz_cmp_ui(*remaining,1) && (assertionStart))
		{
			// We will try all remaining large primes
			running = assertionStart;
			while (running != NULL)
			{
				prime = running->p;
				//printf("%7d",prime);				
				if (!prime) {
					break;
				}
				if (mpz_mod_ui(aux_1,*remaining,prime) == 0)
				{
					gmp_printf("\nFactorAlgebraicError: remaining %Zd is divisible by largish prime %d, but this prime hasn't been found during the process.\nOffset for this element is %d.\n",*remaining,prime,aOffset);
					printf("Now searching for position of the prime in hashtables ....\n");
					// Search					
					iNfsInstance->PrintNFSRelation(iNfsRelation);
					SearchForPrimeInHashtable(iAlgebraicHashtable, prime, running->c_p);
					wait();
				}
				running++;
			}
			
		}
	}
	return ConstRC::Ok;
}	
#endif

void  LineSiever::SearchForPrimeInHashtable(hashtable_t* aHashtable, main_sieving_type aPrime, main_sieving_type aCp)
{
	int i, max = this->iBlockSetSize;
	int total = 0;
	for (i=0; i<= max; i++)
	{
		printf("Searching in hashtable %d ... ",i);
		fflush(stdout);
		ks_script_t* script = aHashtable[i].script;
		bool found = FALSE;
		while (script)
		{
			int j, max2 = script->num_used;
			//printf("List Index %d: num used is %d.\n",listIndex,max);
			ks_update_t *updates = script->list;
			for (j=0; j< max2; j++)
			{
				total++;
				if (updates[j].p == aPrime)
				{
#ifdef GNFS_LINE_SIEVING
					if (updates[j].c_p == aCp) 
				
#endif	
					{
						printf("FOUND!\n");
						found = TRUE;
						PrintUpdateInfo(updates[j]);
					}
				}
			}
			script = script->next;
		}
		if (!found) {
			printf("NOT FOUND (listed total %d).\n",total);
		}
	}
}

#ifdef QS_LINE_SIEVING
bool  LineSiever::InsertSiqsDivisor(main_sieving_type& aPrime, int& aExponent)
{	
	int i, max = this->iQsSiqsSValue;
	for (i=0; i< max; i++)
	{
		sieving_element* element = &(iQsRelation->elements[i]);
		if (element->p == aPrime)
		{	
			element->exponent += aExponent;
			return TRUE;
		}
		
	}
	return FALSE;
}
#endif

#ifdef GNFS_LINE_SIEVING
int   LineSiever::AddAlgebraicDivisor(main_sieving_type& aDivisor, main_sieving_type& aExponent, main_sieving_type& aCp,int aFlags, int& aLastWrittenIndex)
{	
	if (this->iAssertionMode)
	{
		if ((aLastWrittenIndex >= 0) && (!(aFlags & SPECIAL_FLAG)))
		{
			main_sieving_type primeBefore = iNfsRelation->prime_ideals[aLastWrittenIndex].prime;
			if (primeBefore == aDivisor)
			{
				printf("Add Algebraic Divisor: assertion failed. Two distinct nonspecial prime ideals are dividing a single relation. p = %d.\n",aDivisor);
			}
		}
	}
	if ((aExponent > 25) || ((aDivisor > 2) && (aExponent > 15))) {
		printf("Suspicious: prime = %d, exponent = %d.\n",aDivisor,aExponent);
	}
	if ((++aLastWrittenIndex) == iNfsRelation->currently_assigned_prime_ideals) {
		// realokace
		iNfsRelation->prime_ideals = (sieving_element*) AbstractSieve::AllocationMachine((void**) &iNfsRelation->prime_ideals,(iNfsRelation->currently_assigned_prime_ideals+DEFAULT_ADD_PRIME_IDEALS)*sizeof(sieving_element));
		if (iNfsRelation->prime_ideals == NULL) {
			return NOT_ENOUGH_MEMORY;
		}
		iNfsRelation->currently_assigned_prime_ideals += DEFAULT_ADD_PRIME_IDEALS;
	}
	(iNfsRelation->prime_ideals[aLastWrittenIndex]).prime = aDivisor;
	(iNfsRelation->prime_ideals[aLastWrittenIndex]).exponent = aExponent;
	(iNfsRelation->prime_ideals[aLastWrittenIndex]).c_p = aCp;
	(iNfsRelation->prime_ideals[aLastWrittenIndex]).flags = aFlags;
	iNfsRelation->maximal_prime_ideal_index = aLastWrittenIndex;
	return ConstRC::Ok;
}
#endif

int   LineSiever::AddIntegralDivisor(main_sieving_type& aDivisor, int& aExponent, int& aLastWrittenFactor)
{
#ifdef QS_LINE_SIEVING
	// Ensure space
	if ((++aLastWrittenFactor) > (iQsRelation->currently_assigned_sieving_elements-1))
	{
		iQsRelation->currently_assigned_sieving_elements += DEFAULT_DIV_ALLOCATE;
		iQsRelation->elements = (sieving_element*) AbstractSieve::AllocationMachine((void**) &(iQsRelation->elements),iQsRelation->currently_assigned_sieving_elements*sizeof(sieving_element));
		if (!(iQsRelation->elements))
		{
			CLog::LogError(WHEREARG,"Line siever","Add integral divisor",0,iQsRelation->currently_assigned_sieving_elements*sizeof(sieving_element),"Not enough memory to allocate new divisor field for QS relation!");
			return NOT_ENOUGH_MEMORY;
		}
        }
	// Now the space is ensured and the last written factor is shifted to an empty place
	// Note that unlike in old MPQS/SIQS, we write down primes rather than their indices in the factor base. This will be compensated by the ReadRelation method in QS.
	iQsRelation->elements[aLastWrittenFactor].p = aDivisor;
	iQsRelation->elements[aLastWrittenFactor].exponent = aExponent;
	iQsRelation->maximal_sieving_element_index = aLastWrittenFactor;
#endif
#ifdef GNFS_LINE_SIEVING
	// Ensure space
	if ((++aLastWrittenFactor) > (iNfsRelation->currently_assigned_sieving_elements-1))
	{
		iNfsRelation->currently_assigned_sieving_elements += DEFAULT_DIV_ALLOCATE;
		iNfsRelation->elements = (sieving_element*) AbstractSieve::AllocationMachine((void**) &(iNfsRelation->elements),iNfsRelation->currently_assigned_sieving_elements*sizeof(sieving_element));
		if (!(iNfsRelation->elements))
		{
			CLog::LogError(WHEREARG,"Line siever","Add integral divisor",0,iNfsRelation->currently_assigned_sieving_elements*sizeof(sieving_element),"Not enough memory to allocate new divisor field for QS relation!");
			return NOT_ENOUGH_MEMORY;
		}
        }
	// Now the space is ensured and the last written factor is shifted to an empty place
	// Note that unlike in old MPQS/SIQS, we write down primes rather than their indices in the factor base. This will be compensated by the ReadRelation method in QS.
	iNfsRelation->elements[aLastWrittenFactor].prime = aDivisor;
	iNfsRelation->elements[aLastWrittenFactor].exponent = aExponent;
	iNfsRelation->maximal_sieving_element_index = aLastWrittenFactor;
#endif
	return ConstRC::Ok;
}

void  LineSiever::ZeroizeLastHashtable(hashtable_t* aTableList)
{
	hashtable_t* last = &(aTableList[iBlockSetSize]);
	ks_script_t* script = last->script;
	while (script)
	{
		script->num_used = 0;
		script = script->next;
	}
}

void  LineSiever::SwapHashtable(hashtable_t* aTableList, int aIndex)
{
	if ((aIndex < 0) || (aIndex >= iBlockSetSize))
	{
		return;
	}
	ks_script_t* ptr;
	ptr = aTableList[aIndex].script;
	aTableList[aIndex].script = aTableList[iBlockSetSize].script;
	aTableList[iBlockSetSize].script = ptr;
	ZeroizeLastHashtable(aTableList);
}

bool  LineSiever::TestNfs(main_sieving_type aUpdateIndex, main_sieving_type aPrime, bool aPrintOk)
{
#ifdef GNFS_LINE_SIEVING	
	int shiftConst = 2;
	ResetRelation();
	if (iNfsCompressEvenIntervals && iIsCompressedNow)
	{
		shiftConst = 1;
	}
	// At first, we will calculate the relation-related values
	int x = iCurrentBlockSetIndex*iBlockSize*iBlockSetSize + iCurrentBlockIndex*iBlockSize + aUpdateIndex;
	if (iNfsCompressEvenIntervals && iIsCompressedNow)
	{
		x *= 2;
	}
	x += 1;
	if (iSievingNegative)
	{
		x *= -1;				
	}
	SetupRelation(x);
	IntegralDomain* domain = iNfsInstance->GetDomain();
	domain->ABaseOfPrincipal(iIntegralBase,iNfsRelation->a,iNfsRelation->b);
	domain->Norm(iNfsRelation->algebraicNorm,iNfsRelation->a,iNfsRelation->b);
	// Norm of the  
	mpz_t aux;
	mpz_init(aux);
	if (iShiftingAlgebraic)
	{
		
		int res = mpz_mod_ui(aux,iNfsRelation->algebraicNorm,aPrime);
		if (res)
		{
			printf("Test NFS: algebraic check failed. Prime = %d.\n",aPrime);
			gmp_printf("  The norm %Zd is not divisible by %d.\n",iNfsRelation->algebraicNorm,aPrime);
			wait();
		}
	} 
	else 
	{
		int res = mpz_mod_ui(aux,iNfsRelation->integralRemaining,aPrime);
		if (res)
		{
			printf("Test NFS: integral check failed. Prime = %d.\n",aPrime);
			gmp_printf("  The value %Zd is not divisible by %d.\n",iNfsRelation->integralRemaining,aPrime);
			wait();
		}
	}
	mpz_clear(aux);
#endif // GNFS_LINE_SIEVING
	return FALSE;
}

bool  LineSiever::TestQs(main_sieving_type aUpdateIndex, main_sieving_type aPrime, bool aPrintOK)
{
	main_sieving_type currentArgument = iCurrentBlockSetIndex*iBlockSize*iBlockSetSize + iCurrentBlockIndex*iBlockSize + aUpdateIndex + 1;
	if (iSievingNegative)
	{
		currentArgument *= (-1);
	}
	 
#ifdef QS_LINE_SIEVING
	main_sieving_type result = iQsInstance->Evaluate(&iQsCurrentPolynomial,currentArgument,aPrime);
	if (!result) {
		bool isSiqsDivisor = FALSE;
		//printf(" 0, Ok\n");
		//aPrintOK = TRUE;
		if (this->iQsMode == ESiqs) {
			int i, max = this->iQsSiqsSValue;
			for (i=0; i< max; i++)
			{
				if (aPrime == iQsCurrentDivisorsOfA[i])
				{
					//printf("Test: Prime %d is divisor of 'a'\n",aPrime);
					//aPrintOK = TRUE;
					isSiqsDivisor = TRUE;
					break;
				}
			}
		}
		if (isSiqsDivisor)
		{
			// We must evaluate mod p^2, since p divides every Q(x) trivially.
			main_sieving_type square = aPrime*aPrime;
			result = iQsInstance->Evaluate(&iQsCurrentPolynomial,currentArgument,square);
			if ((aPrintOK) && (!result))
			{
				printf("Test - siqs div: bs %2d, b %2d, u %5d, val = %5d ... ",iCurrentBlockSetIndex,iCurrentBlockIndex,aUpdateIndex, currentArgument);
				printf(" %d mod %d (= %d^2), Ok\n",result,square,aPrime);
			} else if (result)
			{
				printf("Test - siqs div: bs %2d, b %2d, u %5d, val = %5d ... ",iCurrentBlockSetIndex,iCurrentBlockIndex,aUpdateIndex, currentArgument);
				printf(" %d mod %d (= %d^2), ERROR\n",result,square,aPrime);
			}
		}
		if (aPrintOK)
		{
		printf("Test: bs %2d, b %2d, u %5d, val = %5d ... ",iCurrentBlockSetIndex,iCurrentBlockIndex,aUpdateIndex, currentArgument);
		printf(" %d mod %d, Ok\n",result,aPrime);
		}
		return TRUE;
	} else {
		printf("Test: bs %2d, b %2d, u %5d, val = %5d ... ",iCurrentBlockSetIndex,iCurrentBlockIndex,aUpdateIndex, currentArgument);
		printf(" %d mod %d, ERROR\n",result,aPrime);
	}
#endif	
	return FALSE;
}
bool  LineSiever::TestQs(main_sieving_type aUpdateIndex, main_sieving_type aPrime)
{
	return TestQs(aUpdateIndex,aPrime,FALSE);
}


void  LineSiever::PrintUpdateInfo(ks_update_t aInfo)
{
	printf("\nUpdate Type: \n");
	printf("     Prime   : %d\n",aInfo.p);
#ifdef GNFS_LINE_SIEVING
	printf("     C_P     : %d\n",aInfo.c_p);
#endif
	printf("     Log     : %d\n",aInfo.logp);
	printf("     H       : %d\n",aInfo.h);
}

#ifdef GNFS_LINE_SIEVING
void  LineSiever::PrintNfsElementInfo(nfs_fb_type aInfo)
{
	printf("\nNFS FB Type: \n");
	printf("     Prime   : %d\n",aInfo.p);
	printf("     C_P     : %d\n",aInfo.c_p);
	printf("     Log     : %d\n",aInfo.log_p);
	printf("     Root1   : %d\n",aInfo.root_1);
	printf("     Next1   : %d\n",aInfo.next_1);
	printf("     Inv     : %d\n",aInfo.auxVal);
}


bool  LineSiever::TestNfsIntegral(nfs_fb_type aInfo, main_sieving_type aUpdateIndex)
{
	bool result = FALSE;
	mpz_t aux, aux2;
	mpz_init(aux);
	mpz_init(aux2);
	main_sieving_type p = aInfo.p;
	sieving_direction dir = iNfsInstance->GetSievingDirection();
	main_sieving_type currentArgument = iCurrentBlockSetIndex*iBlockSize*iBlockSetSize + iCurrentBlockIndex*iBlockSize;	
//#define NFS_INTEGRAL_DEBUG
#ifdef NFS_INTEGRAL_DEBUG
	PrintNfsElementInfo(aInfo);
#endif
	if (aUpdateIndex != NO_UPDATE_INDEX) 
	{
		currentArgument += aUpdateIndex;
	} 
	else 
	{
		currentArgument += aInfo.root_1;
	}
	if (iNfsCompressEvenIntervals && iIsCompressedNow)
	{
		currentArgument <<= 1;
	}
	currentArgument += 1;
	if (iSievingNegative)
	{
		currentArgument *= (-1);
	}
	if ((dir == EIncrementB) || (dir == EDecrementB)) 
	{
		mpz_mul_si(aux2,this->iM,iNfsInstance->GetB());
#ifdef NFS_INTEGRAL_DEBUG
		gmp_printf("LineSiever: Test Integral. MB = %Zd.\n",aux2);
#endif		
		mpz_set_si(aux,currentArgument);		
#ifdef NFS_INTEGRAL_DEBUG
		printf("LineSiever: Test Integral. curr arg = %d.\n",currentArgument);		
#endif
	}
	if ((dir == EIncrementA) || (dir == EDecrementA)) 
	{
		mpz_set_si(aux,iNfsInstance->GetA());		
#ifdef NFS_INTEGRAL_DEBUG
		gmp_printf("LineSiever: Test Integral. A = %Zd.\n",aux);
#endif		
		mpz_mul_si(aux2,this->iM,currentArgument);
#ifdef NFS_INTEGRAL_DEBUG
		gmp_printf("LineSiever: Test Integral. MB = %Zd.\n",aux2);		
#endif
	}
	mpz_add(aux,aux,aux2);
	if (mpz_mod_ui(aux,aux,p) != 0) {
		gmp_printf("Integral sieving error 1: at %d mod %d: %Zd, is compressed %d\n",currentArgument,p,aux,iIsCompressedNow);		
		wait();
	} else {
		
	}
	mpz_clear(aux);
	mpz_clear(aux2);
	return result;
}


bool  LineSiever::TestNfsAlgebraic(nfs_fb_type aInfo, main_sieving_type aUpdateIndex)
{
	bool result = FALSE;
	main_sieving_type plus;
	mpz_t axxx;
	mpz_init(axxx);	
	sieving_direction dir = iNfsInstance->GetSievingDirection();
	main_sieving_type currentArgument = iCurrentBlockSetIndex*iBlockSize*iBlockSetSize + iCurrentBlockIndex*iBlockSize;	
//#define NFS_ALGEBRAIC_DEBUG
#ifdef NFS_ALGEBRAIC_DEBUG
	PrintNfsElementInfo(aInfo);
#endif
	/*if (aInfo.p == 65537) {
		printf("\ntest NFS algebraic: Printing for control ...\n");
		PrintNfsElementInfo(aInfo);
		printf("\nEnd of printing for control ...\n");
	}*/
	if (aUpdateIndex != NO_UPDATE_INDEX) 
	{
		currentArgument += aUpdateIndex;
	} 
	else 
	{
		currentArgument += aInfo.root_1;
	}
	if (iNfsCompressEvenIntervals && iIsCompressedNow)
	{
		currentArgument <<= 1;
	}
	currentArgument += 1;
	if (iSievingNegative)
	{
		currentArgument *= (-1);
	}
	if ((dir == EIncrementB) || (dir == EDecrementB)) 
	{
		mpz_set_si(axxx,iNfsInstance->GetB());
#ifdef NFS_ALGEBRAIC_DEBUG
		printf("LineSiever: Test Algebraic. B = %d.\n",iNfsInstance->GetB());
#endif
		mpz_mul_si(axxx,axxx,aInfo.c_p);
#ifdef NFS_ALGEBRAIC_DEBUG
		printf("LineSiever: Test Algebraic. c_p = %d.\n",aInfo.c_p);		
#endif
		plus = currentArgument;
	}
	if ((dir == EIncrementA) || (dir == EDecrementA)) 
	{
		mpz_set_si(axxx,currentArgument);
		mpz_mul_si(axxx,axxx,aInfo.c_p);
#ifdef NFS_ALGEBRAIC_DEBUG
		printf("LineSiever: Test Algebraic. c_p = %d.\n",aInfo.c_p);
#endif
		plus = iNfsInstance->GetA();
#ifdef NFS_ALGEBRAIC_DEBUG
		
		printf("LineSiever: Test Algebraic. A = %d.\n",iNfsInstance->GetA());
#endif
	}
	if ((!(aInfo.flags & SPECIAL_PRIME_MINI_FLAG)) && (!(aInfo.flags & DIVIDES_C_P_MINI_FLAG))) {
		int res1 = mpz_mod_ui(axxx,axxx,aInfo.p);
		int res2 = res1 + plus;					 
		res2 %= aInfo.p;
		if (res2) {
			PrintNfsElementInfo(aInfo);
			gmp_printf("Line Siever: Test Algebraic. For prime %d: current root is %d, which means argument %d, mod is %d. .\n",aInfo.p,aInfo.root_1,currentArgument,res2);
			wait();
			result = FALSE;
		} else {
			result = TRUE;
		}
		
	} else 
	{
		result = TRUE;
	}
	mpz_clear(axxx);
	return result;
}
#endif // GNFS_LINE_SIEVING


void  LineSiever::ClearHashtables(hashtable_t* aTableList, ks_cache_t* aCacheList)
{
	int i, max = iBlockSetSize;
	hashtable_t* target1;
	ks_cache_t*     cache1;
	for (i=0; i<= max; i++)
	{
		target1 = &(aTableList[i]);
		cache1  = &(aCacheList[i]);
		ks_script_t* script = target1->script;
		while (script)
		{
			script->num_used = 0;
			script = script->next;
		}
		cache1->used = 0;
	}
}

void  LineSiever::PrintHashtables(hashtable_t* aTableList)
{
	int i, j, max;
	printf("\n\nPrint Hashtables start\n\n");
	for (i=0; i<= iBlockSetSize; i++)
	{
		printf("\n\n HASHTABLE # %d: \n\n",i);
		ks_script_t* script = iHashtable[i].script;
		int counter = 0;
		while (script)
		{
			max = script->num_used;
			ks_update_t *updates = script->list;
			for (j=0; j< max; j++) {
				printf("(%6d:) Prime %6d, h = %6d.\n",counter, updates[j].p,updates[j].h);
				counter++;
				
			}
			printf("----- list boundary -----\n");
			script = script->next;
		}
		printf("  This hashtable had %d entries.\n",counter);		
	}
	printf("\n\nPrint Hashtables end\n\n");
}

int  LineSiever::AddScript(ks_script_t* aLast)
{
	if (!aLast)
	{
		CLog::LogError(WHEREARG,"Line siever","Add script",0,0,"Add script found that last script is null!");
		return INCONSISTENT_STATE;
	}
	// Next script already allocated
	if (aLast->next)
	{
		return ConstRC::Ok;
	}
	ks_script_t* next = (ks_script_t*) malloc(sizeof(ks_script_t));
	if (!next)
	{
		CLog::LogError(WHEREARG,"Line siever","Add script",0,sizeof(ks_script_t),"Not enough memory to allocate new script!");
		return NOT_ENOUGH_MEMORY;
	}
	memset(next,0,sizeof(ks_script_t));
	aLast->next = next;
	// We will prefetch a bit of memory, because we add scripts in the 
	PREFETCH(next->list);
	return ConstRC::Ok;
}

line_sieve_size	LineSiever::GetLineSieveSize() const
{
	return this->iLineSieveSize;
}

int LineSiever::GetLineLowerBound() const 
{
	return this->iLineLowerBound;
}

int LineSiever::GetLineUpperBound() const
{	
	return this->iLineUpperBound;
}

internal_sieve_state LineSiever::GetSieveState() const
{
	return this->iInternalSieveState;
}

int LineSiever::Init(line_sieve_size aLineSieveSize, int aLineLowerBound, int aLineUpperBound, int aFactorBaseUpperBound)
{
	if (this->iInternalSieveState != EBeforeInit)
	{
		CLog::LogError(WHEREARG,"Line siever","Init",0,this->iInternalSieveState,"Init called in an inappropriate moment. Should be called after constructor and only once!");
		return INCONSISTENT_STATE;
	}
	if (aLineSieveSize == ESizeUndefined)
	{
		// We do not want to reallocate the block to zero size :-)
		CLog::LogError(WHEREARG,"Line siever","Init",0,0,"Cannot create Block with undefined size.");			
		return INCONSISTENT_STATE;
	}
	this->iInternalSieveState = EBeingInited;
	this->iLineSieveSize = aLineSieveSize;
	this->iBlockSize     = (int) iLineSieveSize;
	if (this->iBlockSize <= 0)
	{
		CLog::LogError(WHEREARG,"Line siever","Init",0,this->iBlockSize,"Value of block size must be positive!");
		return INCONSISTENT_STATE;
	}
	if (aLineLowerBound >= 0) 
	{
		CLog::LogError(WHEREARG,"Line siever","Init",0,aLineLowerBound,"Value of line lower bound must be negative!");
		return INCONSISTENT_STATE;
	}
	if (aLineLowerBound*(-1) > MAXIMAL_LINE_SIEVE_HALFLINE)
	{
		CLog::LogError(WHEREARG,"Line siever","Init",(-1)*aLineLowerBound,MAXIMAL_LINE_SIEVE_HALFLINE,"Value of line lower bound too big - must not exceed a particular value!");
		return INCONSISTENT_STATE;
	}
	this->iLineLowerBound = aLineLowerBound;
	if (aLineUpperBound <= 0) 
	{
		CLog::LogError(WHEREARG,"Line siever","Init",0,aLineUpperBound,"Value of line upper bound must be positive!");
		return INCONSISTENT_STATE;
	}
	if (aLineUpperBound > MAXIMAL_LINE_SIEVE_HALFLINE)
	{
		CLog::LogError(WHEREARG,"Line siever","Init",aLineUpperBound,MAXIMAL_LINE_SIEVE_HALFLINE,"Value of line upper bound too big - must not exceed a particular value!");
		return INCONSISTENT_STATE;
	}
	this->iLineUpperBound = aLineUpperBound;
	if (aFactorBaseUpperBound <= 0)
	{
		CLog::LogError(WHEREARG,"Line siever","Init",0,aFactorBaseUpperBound,"Value of factor base upper bound must be positive.");
		return INCONSISTENT_STATE;
	}
	if (aFactorBaseUpperBound > MAXIMAL_FACTOR_BASE_UPPER_BOUND)
	{
		CLog::LogError(WHEREARG,"Line siever","Init",aFactorBaseUpperBound,MAXIMAL_FACTOR_BASE_UPPER_BOUND,"Value of factor base upper bound too big - must not exceed a particular value!");
		return INCONSISTENT_STATE;
	}
	this->iFactorBaseUpperBound = aFactorBaseUpperBound;
	int result = InternalReset();
	if (result != ConstRC::Ok)
	{
		this->iInternalSieveState = ENotReady;
		CLog::LogError(WHEREARG,"Line siever","Init",0,result,"Internal reset failed.");			
		return result;
	}	
	this->iInternalSieveState = EWaitingForFBSetup;
	return ConstRC::Ok;
}

int LineSiever::ResetBlocks(line_sieve_size aNewLineSieveSize)
{
	switch (this->iInternalSieveState)
	{
		case EBeforeInit:
		case EKSSieving:
		case ENotReady:
		case EBeingDestroyed:
			// Not the right time to reset the sieving structures. Will spit out a complaint and exit without doing anything.
			CLog::LogError(WHEREARG,"Line siever","Reset blocks",0,this->iInternalSieveState,"ResetBlocks called in an inappropriate moment");
			return INCONSISTENT_STATE;
		default:
			// We can go on.
			break;
	}
	// No need to change anything.
	if (this->iLineSieveSize == aNewLineSieveSize)
	{
		return ConstRC::Ok;
	}
	if (aNewLineSieveSize == ESizeUndefined)
	{
		// We do not want to reallocate the block to zero size :-)
		CLog::LogError(WHEREARG,"Line siever","Reset blocks",0,0,"Cannot Reset Block to an undefined size.");			
		return INCONSISTENT_STATE;
	}
	internal_sieve_state oldState = this->iInternalSieveState;
	int result = InternalReset();
	if (result != ConstRC::Ok)
	{
		this->iInternalSieveState = ENotReady;
		CLog::LogError(WHEREARG,"Line siever","Reset blocks",0,result,"Internal reset failed.");			
		return result;
	}
	this->iInternalSieveState = oldState; 	
	return ConstRC::Ok;
}

int LineSiever::InternalReset()
{
	// We will reset the block
	if (this->iBlock != NULL)
	{
		free(this->iBlock);
		iBlock = NULL;
	}

	if (iBlockSize <= 0)
	{
		CLog::LogError(WHEREARG,"Line siever","Internal reset",0,0,"Cannot Reset Block to a nonpositive size.");			
		return INCONSISTENT_STATE;
	}
	this->iBlock = (log_type*) AbstractSieve::AllocationMachine((void**) &this->iBlock,(iBlockSize)*sizeof(log_type));
	if (this->iBlock == NULL)
	{
		CLog::LogError(WHEREARG,"Line siever","Reset blocks",0,iBlockSize*sizeof(log_type),"Could not allocate space for internal sieving block");
		return NOT_ENOUGH_MEMORY;
	} else {
		printf(" block is allocated to %u bytes.",(iBlockSize)*sizeof(log_type));
	}
	memset(this->iBlock,0,(iBlockSize)*sizeof(log_type));
#ifdef GNFS_LINE_SIEVING
	this->iAlgebraicBlock = (log_type*) AbstractSieve::AllocationMachine((void**) &this->iAlgebraicBlock,(iBlockSize)*sizeof(log_type));
	if (this->iAlgebraicBlock == NULL)
	{
		CLog::LogError(WHEREARG,"Line siever","Reset blocks",0,iBlockSize*sizeof(log_type),"Could not allocate space for internal algebraic sieving block");
		return NOT_ENOUGH_MEMORY;
	} else {
		printf(" Algebraic block is allocated to %d bytes.",(iBlockSize)*sizeof(log_type));
	}
	memset(this->iAlgebraicBlock,0,(iBlockSize)*sizeof(log_type));

#endif
	// Just for testing
	int i; for (i=0; i< iBlockSize; i++) { if (iBlock[i]) continue; }

	this->iPositiveBlocks = iLineUpperBound / iBlockSize; 
	if (iLineUpperBound % iBlockSize)
	{
		this->iPositiveBlocks++;
	}
	this->iNegativeBlocks = (-1)*iLineLowerBound / iBlockSize;
	if (((-1)*iLineLowerBound) % iBlockSize) 
	{
		this->iNegativeBlocks++;
	}
	this->iBlockSetSize = iFactorBaseUpperBound / iBlockSize;
	if (iFactorBaseUpperBound % iBlockSize)
	{
		this->iBlockSetSize++;
	}
	// The security margin
	iAllocatedHashtableSize = this->iBlockSetSize + 2;
	// The hashtable is being set
	DeleteHashtable(this->iHashtable,iAllocatedHashtableSize-1);
	this->iHashtable = NULL;
	int result = SetupHashtable(&this->iHashtable,&this->iCaches);
	if (result != ConstRC::Ok)	
	{
		return result;
	}
#ifdef GNFS_LINE_SIEVING
	// The hashtable is being set
	DeleteHashtable(this->iAlgebraicHashtable,iAlgebraicAllocatedHashtableSize-1);
	this->iAlgebraicHashtable = NULL;
	result = SetupHashtable(&this->iAlgebraicHashtable,&this->iAlgebraicCaches);
	if (result != ConstRC::Ok)	
	{
		return result;
	}
#endif	
	return ConstRC::Ok;
}

#ifdef GNFS_LINE_SIEVING
bool	LineSiever::TestCoprimality()
{
	main_sieving_type a = iNfsRelation->a;
	main_sieving_type b = iNfsRelation->b;
	main_sieving_type result = 0;
	iNfsInstance->GreatestCommonDivisor(abs(a),abs(b),result);
	if (result == 1) {
		return TRUE;
	}
	return FALSE;
}
#endif

