/*! \file 
 *  \brief main() function, taking care of parameters, -h output
 *
*/
// MPQS version 0.99, 10.1.2004
// MPQS version 1.03, 23.2.2004
// MPQS version 1.09, 12.6.2004
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include "gmp.h"
#include "time.h"
#include "abstract_sieve.h"
#include "quadratic_sieve.h"
#include "qs_parameters.h"
#include "definitions.h"
#include "line_siever.h"
#include "parameter_test.h"
#include "filesystem_connector.h"
#include "xml_qs_serializator.h"
#include "node.h"
#include "center.h"
#include "types.h"
#include "center_info.h"
#include "url.h"
#include <string>

int mpqs_mode = 0;
int siqs_mode = 0;
int test_mode = 0;
int list_mode = 0;
int load_mode = 0;
int node_mode = 0;
int center_mode = 0;
int demo_create_batch_mode = 0;

using namespace std;

#ifdef WIN32
static int strcasecmp ( char* s1, char* s2 );
#endif

int isDecimalDigit(char ch);

void help(void);

QuadraticSieve* qs_instance = NULL;
Node*		no_instance = NULL;
Center*		ce_instance = NULL;
XmlQsSerializator*  no_seri = NULL;
XmlQsSerializator*  ce_seri = NULL;

void SolveSignal(int aParameter)
{
	//printf("\n Received %d, pointer to sieve is %p.\n",aParameter,qs_instance);
	if (qs_instance)
	{
		bool canInterrupt = qs_instance->Get_Interruptable();
		if (canInterrupt)
		{
			printf("\nSieve interrupted. Please stand by while the state is being saved ...\n");
			qs_instance->Set_InterruptNow(TRUE);
		} else
		{
			printf("\nSorry, sieve cannot be interrupted now! You can only kill it by SIGKILL.");
		}
	}
	if (no_instance)
	{
		no_instance->Break();
	}
	if (ce_instance)
	{
		ce_instance->Break();
	}
	
}

int TestParameterConsistency(int aLoadFlags)
{
	int ret = ConstRC::Ok;	
	if (aLoadFlags & FB_LOAD_FLAG)
	{
		if (!(aLoadFlags & JOB_LOAD_FLAG))
		{
			printf("Inconsistent load parameters: you cannot load factor bases without loading job as well.\n");			
			ret = GENERAL_ERROR;	
		}
	}
	if (aLoadFlags & SIEVING_LOAD_FLAG)
	{
		if (!(aLoadFlags & FB_LOAD_FLAG))
		{
			printf("Inconsistent load parameters: you cannot load sieving without loading factor bases as well.\n");			
			ret = GENERAL_ERROR;	
		}
		if (!(aLoadFlags & JOB_LOAD_FLAG))
		{
			printf("Inconsistent load parameters: you cannot load sieving without loading job as well.\n");			
			ret = GENERAL_ERROR;	
		}
	}
	return ret;
}


/*!
 * \brief Just takes care of parameters, constructs and calls instance of Sieve.
 * @param argc - number of arguments
 * @param argv - array of arguments
 */
int main(int argc, char *argv[]){	

	FilesystemConnector::Dir("/home/marian/MFF");
	int i;	
	QSParameters params;

	void (*originalTermAddress)(int) = NULL;	
	void (*originalQuitAddress)(int) = NULL;	
	void (*originalIntAddress)(int) = NULL;	

	char* algorithm = NULL;		
	char* url = NULL;
	char* machinename = NULL;
	char* thresholdoptimizer = NULL;
	mpz_t argument;
	mpz_t sieve_int;
	mpz_t fb_size;
	mpz_t n_poly_mpz;
	int n_poly_uint = 1;
	bool keep_messages = FALSE;

	mpz_init(argument);
	mpz_init(sieve_int);
	mpz_init(fb_size);
	mpz_init(n_poly_mpz);	

	vector<const char*> centerinfos;

	// initialize xml parser
	xmlInitParser();

	/* Reading the arguments */
	for (i=1;i<argc;i++){
		if (!strcasecmp(argv[i],"-a")){
			// Assertion Mode
			params.Set_AssertionMode(TRUE);
			continue;
		}
		if (!strcasecmp(argv[i],"-t")){
			qs_instance = new QuadraticSieve();
			qs_instance->RunTTR(ENormal);
			goto final_dealloc;
		}
		if (!strcasecmp(argv[i],"-tfast")){
			qs_instance = new QuadraticSieve();
			qs_instance->RunTTR(EFast);
			goto final_dealloc;
		}
		if (!strcasecmp(argv[i],"-tstrict")){
			qs_instance = new QuadraticSieve();
			qs_instance->RunTTR(EStrict);
			goto final_dealloc;
		}
		if (!strcasecmp(argv[i],"-h")){
			help();
			goto final_dealloc;
		}
		if (!strcasecmp(argv[i],"-q")){
			params.Set_QSMode(EOldMpqs);
			mpqs_mode = 1;			
		}
		if (!strcasecmp(argv[i],"-nq")){
			params.Set_QSMode(EMpqs);
			mpqs_mode = 1;			
			continue;
		}
		if (!strcasecmp(argv[i],"-oq")) 
		{
			params.Set_QSMode(EOldMpqs);
			mpqs_mode = 1;			
			continue;
		}
		if (!strcasecmp(argv[i],"-i")){
			// Info Mode - extended info on screen
			params.Set_InfoMode(TRUE);			
			continue;
		}		
		if (!strcasecmp(argv[i],"-checkauto")){
			params.Set_CheckMode(ECheckAuto);
			continue;
		}

		if (!strcasecmp(argv[i],"-checkall")){
			params.Set_CheckMode(ECheckAll);			
			continue;
		}		
		if(!strcasecmp(argv[i],"-ltime")){
			params.Set_LtMode(TRUE);			
			continue;
		}		
		if(!strcasecmp(argv[i],"-mflex")){
			params.Set_SievingIntervalMode(EFlexible);			
			continue;
		}
		
		if (strstr(argv[i],"-check") && !(strstr(argv[i],"-checkall"))){ // a checkmode has been given
			// we turn the rest of the parameter to an integer
			int checklevel = atoi(argv[i]+6);
			//printf("Checklevel found as %d",checklevel);
			params.Set_CheckLevel(checklevel);			
			continue;
		}
		if (strstr(argv[i],"-memblock"))
		{
			int memsize = atoi(argv[i]+9);
			params.Set_MemblockSize(memsize);
			continue;
		}	
		if (strstr(argv[i],"-tmessage")){ // a timemessage has been given
			// we turn the rest of the parameter to an integer
			int tmess = atoi(argv[i]+9);
			//if (!tmess) tmess = IMPLICIT_TMESS;
			params.Set_TimeMessageInterval(tmess);			
			continue;
		}
		// Statistics mode will be a standard part of info mode.
		if (!strcasecmp(argv[i],"-de")){
 			params.Set_DiMode(TRUE);			
			continue;
		}
		if (!strcasecmp(argv[i],"-bit")){
 			params.Set_LinalgType(EBitMatrix);			
			continue;
		}
		if (!strcasecmp(argv[i],"-r")){
			printf("\nSorry, rough division mode not supported in version %.2f.\n",QS_VERSION);
			goto final_dealloc;
			//qs_instance->Set_SievingMode(ERoughDivisionMode);
		}
		if (!strcasecmp(argv[i],"-x")){
			printf("\nSorry, exact division mode not supported in version %.2f.\n",QS_VERSION);
			goto final_dealloc;
			//qs_instance->Set_SievingMode(EExactDivisionMode);
		}
		if (!strcasecmp(argv[i],"-w")){
			params.Set_WpList(TRUE);			
			continue;
		}
		if (!strcasecmp(argv[i],"-v")){
			params.Set_UsingVariations(ELargePrimeV);			
			continue;
		}
		if (!strcasecmp(argv[i],"-debug")){
			params.Set_InfoMode(TRUE);
			params.Set_DebugMode(TRUE);
			continue;
		}
		if (!strcasecmp(argv[i],"-ranks")){
			// If set, calculates ranks of A and B.
			params.Set_RankCalculationMode(TRUE);			
			continue;
		}
		if (!strcasecmp(argv[i],"-valdist")){
			params.Set_ValueDistributionMode(TRUE);
			continue;
		}
		if (!strcmp(argv[i],"-printsort")){
			params.Set_PrintSort(EPrintSort);			
			continue;
		}
		if (!strcasecmp(argv[i],"-s")){
			params.Set_QSMode(EOldSiqs);
			siqs_mode = 1;			
			continue;
		}
		if (!strcasecmp(argv[i],"-ns")){
			params.Set_QSMode(ESiqs);
			siqs_mode = 1;			
			continue;
		}
		if (!strcasecmp(argv[i],"-batchdemo"))
		{
			demo_create_batch_mode = 1;
			continue;
		}
		if (!strcasecmp(argv[i],"-os")) 
		{
			params.Set_QSMode(EOldSiqs);
			siqs_mode = 1;			
			continue;
		}
		if (!strcasecmp(argv[i],"-extk")) {
			params.Set_ExtendedK(TRUE);
			continue;
		}
		if (!strcasecmp(argv[i],"-nodelsingles")){
			params.Set_SingletonMode(EDoNotDeleteSingletons);						
			continue;
		}		
		if (!strcasecmp(argv[i],"-power")){
			params.Set_PowerMode(ESieveWithPrimePowers);						
			continue;
		}
		if (strstr(argv[i],"-load"))
		{			
			if (strstr(argv[i],"job"))
			{
				params.Set_Flag(JOB_LOAD_FLAG);			
				load_mode = TRUE;
			}
			if (strstr(argv[i],"fb"))
			{
				params.Set_Flag(FB_LOAD_FLAG);		
			}
			if (strstr(argv[i],"list"))
			{
				params.Set_Flag(LIST_LOAD_FLAG);		
				list_mode = TRUE;
			}
			if (strstr(argv[i],"sieving"))
			{
				params.Set_Flag(SIEVING_LOAD_FLAG);						
			}
			continue;
		}
		if (!strcasecmp(argv[i],"-list"))
		{
			list_mode = TRUE;
			continue;
		}
		if (!strcasecmp(argv[i],"-dir"))
		{
			if (i != (argc-1))
			{
				printf("MPQS.CPP: setting DIR name as %s.\n",argv[i+1]);
				params.Set_DirectoryName(argv[i+1]);
				int len = strlen(argv[i+1])+5;
				char* local = (char*) malloc(len*sizeof(char));
				memset(local,0,len*sizeof(char));		
				//printf("Dir arg: %s.\n",argv[i+1]);		
				strcat(local,argv[i+1]);
				//printf("Local: %s.\n",local);
				int pos = strlen(argv[i+1])-1;
				//printf("Pos: %d.\n",pos);
				if (local[pos] != PATH_SEPARATOR)
				{
					local[pos+1] = PATH_SEPARATOR; 
					//printf("New local: %s.\n",local);
				}
				bool result = AbstractSieve::TestDirectoryAccess(local);
				free(local);
				if (!result)
				{
					printf("Could not access the directory '%s' for writing. Please correct the situation manually and restart the job. Aborting.\n",local);
					goto final_dealloc;
				}
				++i;
				continue;
			} else {
				printf("The directory must be specified explicitely after -dir. Aborting.\n");
				goto final_dealloc;
			}
		}
		if (!strcasecmp(argv[i],"-rootshort"))
		{
			params.Set_RootShort(TRUE);
			continue;
		}
		if (!strcasecmp(argv[i],"-gz"))
		{
			params.Set_CompressionLevel(8);
			continue;
		}
		if (!strcasecmp(argv[i],"-v2")){
			params.Set_UsingVariations(EDoubleLargePrimeV);			
			continue;
		}
		// choice of algorithm (in double lpv)
		if (strstr(argv[i],"-alg=")) {
			// we extract the real name of the algorithm
			algorithm = (argv[i]+5);
			printf("Algorithm: %s\n",algorithm);
			if (!strcasecmp(algorithm,"cfrac")) {
				printf("Algorithm set as CFRAC");
				params.Set_FactorAlg(ECFRAC);
			} else if (!strcasecmp(algorithm,"pminus1")) {
				printf("Algorithm set as P minus 1");
				params.Set_FactorAlg(EPMinus1);
			}
			continue;
		}
		if (!strcasecmp(argv[i],"-testblock"))
		{
			mpqs_mode = FALSE;
			siqs_mode = FALSE;
			test_mode = TRUE;
			continue;
		}
		if (!strcasecmp(argv[i],"-center"))
		{
			params.Set_DistributionMachineType(ECenter);
			node_mode   = 0;
			center_mode = 1;
			if (ce_seri == NULL)
			{
			ce_seri = new XmlQsSerializator();
			}
			ce_instance = new Center(*ce_seri,params.Get_DirectoryName());			
			continue;
		}
		if (strstr(argv[i],"-conninfo="))
		{
			if (no_instance || ce_instance)
			{
				char* info = strchr(argv[i],'[');
//  				printf("Info is %s.\n",info);
				if (info)
				{
					ConnectorInfo* ci = ConnectorInfo::Parse(info);
					if (ci)
					{
						if (no_instance)
						{
						no_instance->RegisterReceiveConnection(*ci);
						}
						if (ce_instance)
						{
						ce_instance->RegisterReceiveConnection(*ci);
						}
					}
					delete ci;
				}
			} else
			{
			// Ignored.
			}
			continue;
		}
		if (strstr(argv[i],"-centerinfo="))
		{
// 			printf("Centerinfo detected: %s\n",argv[i]);
			if (no_instance) // Node instance initialized
			{
				// A command-line "centerinfo" for a node.				
				char* info = strchr(argv[i],'[');				
// 				printf("Info is %s.\n",info);
				if (info)
				{
					centerinfos.push_back(info);	
							
				}
			}
			else 
			{
				printf("All -centerinfo specifications must follow the -node par.\n");
			}
			continue;
		}
		if (!strcasecmp(argv[i],"-node"))
		{
			params.Set_DistributionMachineType(ENode);
			node_mode = 1;
			if (no_seri == NULL)
			{
				no_seri = new XmlQsSerializator();
			}
			no_instance = new Node(*no_seri);
			continue;
		}
		if (strstr(argv[i],"-url=")) {
			// we extract the shared URL. 
			url = (argv[i]+5);
			printf("MPQS.CPP: shared URL: %s\n",url);
			params.Set_SharedURL(url);
			continue;
		}		
		if (strstr(argv[i],"-id=")) {
			// we extract the machine name
			machinename = (argv[i]+4);
			//printf("Id: %s\n",machinename);
			params.Set_MachineName(machinename);
			continue;
		}
		if (!strcasecmp(argv[i], "-keep-messages")) {
			keep_messages = TRUE;
			continue;
		}
		if (strstr(argv[i],"-to="))
		{
			int policy = 0;
			thresholdoptimizer = (argv[i]+4);
			if (!strcasecmp(thresholdoptimizer,"off"))
			{
				policy = 0;
			}	
			else
			{
				policy = atoi(thresholdoptimizer);
			}
			params.Set_ThresholdOptimizer_Level(policy);
		}	
		// The arguments with variable length must be processed at the end of this sequence of argument-processing if's
		if (!strncasecmp(argv[i],"-n=",3)){
			if (0 == mpz_set_str(argument,argv[i]+3,10)) {
				params.Set_Modulus(argument);
			}
			continue;
		}
		if (!strncasecmp(argv[i],"-m=",3)){
			if (0 == mpz_set_str(sieve_int,argv[i]+3,10)) {
				params.Set_SievingIntervalLength( mpz_get_ui(sieve_int));
			}
			continue;
		}
		if (!strncasecmp(argv[i],"-f=",3)){
			if (0 == mpz_set_str(fb_size,argv[i]+3,10)) {
				params.Set_UpperIntBound(mpz_get_ui(fb_size));
			}
			continue;
		}
		if (!strncasecmp(argv[i],"-p=",3)){
			if (0 == mpz_set_str(n_poly_mpz,argv[i]+3,10)) {
				n_poly_uint = mpz_get_ui(n_poly_mpz);
				if ((n_poly_uint<1) || (n_poly_uint>MAX_POLY)){
					n_poly_uint = IMPLICIT_POLY;
				}
				params.Set_MaxIter(n_poly_uint);
			}
			continue;
		}
		if (argv[i][1] == '_') {
			printf("\n\n !!! Frontend pipe disabled !!! \n\n");
			goto final_dealloc;
		}
	} /* end of argument processing */
	if (list_mode)
	{
		mpqs_mode = FALSE;
		siqs_mode = FALSE;
		test_mode = FALSE;
		load_mode = FALSE;		
		qs_instance = new QuadraticSieve();
		AbstractSerializator* ser = qs_instance->GetSerializator();
		qs_instance->SetDataCompressionLevel(params.Get_CompressionLevel());
		if (ser)
		{
			ser->Set_DirectoryName(params.Get_DirectoryName().c_str());			
			ser->Deserialize(EQsJob);			
			qs_instance->InitForList(params.Get_DirectoryName().c_str());
			ser->Deserialize(EQsFactorBase);
			ser->Deserialize(EQsSieving);
			if (qs_instance->Get_RelationProcessingFlags() & SIEVING_RUNNING_FLAG)
			{
				ser->Deserialize(EQsSieverState);
			}
		}
	}
	if (load_mode)
	{
		if (TestParameterConsistency(params.Get_LoadFlags()) != ConstRC::Ok)
		{
			goto final_dealloc;
		}
	}
	if (mpqs_mode || siqs_mode || load_mode || demo_create_batch_mode){
		qs_instance = new QuadraticSieve();
		int resLoc = qs_instance->SetupParameters(params);
		if (resLoc == ConstRC::Ok)
		{			
			originalTermAddress = signal(SIGTERM,SolveSignal);
			originalQuitAddress = signal(SIGQUIT,SolveSignal);
			originalIntAddress = signal(SIGINT,SolveSignal);
			if (demo_create_batch_mode)
			{
				qs_instance->DemonstrateBatches();
			}
 			else 
			{
				qs_instance->RunMPQS();
			}
		}
	} else if (node_mode)
	{	
		
		// At first, we must register the sieve instance.
		// Only after the registration of the instance
		// we can process center infos.	
		no_instance->SetMachineName(params.Get_MachineName());
		no_instance->SetKeepMessages(keep_messages);
		qs_instance = new QuadraticSieve();
		//if (qs_instance->SetupParameters(params) == ConstRC::Ok)
		{
			originalTermAddress = signal(SIGTERM,SolveSignal);
			originalQuitAddress = signal(SIGQUIT,SolveSignal);
			originalIntAddress = signal(SIGINT,SolveSignal);
			//qs_instance->WaitForNodeSetup();			
		}
		no_instance->RegisterDistAlg(qs_instance);
		no_seri->RegisterInstance(qs_instance);
		// Process the center infos from command line
		int i, max = centerinfos.size();
		for (i=0; i< max; i++)
		{
			const char* info = centerinfos.at(i);
			CenterInfo* ci = CenterInfo::Parse(info);
			if (ci)
			{
				no_instance->RegisterCenter(ci);
			}				
		}
		// Process the center infos from XML.
		no_instance->LoadCenterList(params.Get_DirectoryName().c_str());
		// Go!	
		no_instance->Run();
	} else if (center_mode)
	{
		string gmn = params.Get_MachineName();
		printf("Current machine name is %s.\n",gmn.c_str());
		ce_instance->SetMachineName(params.Get_MachineName());
		ce_instance->SetKeepMessages(keep_messages);
		originalTermAddress = signal(SIGTERM,SolveSignal);
		originalQuitAddress = signal(SIGQUIT,SolveSignal);
		originalIntAddress = signal(SIGINT,SolveSignal);
		if (params.Complete())
		{			
			ce_instance->ProcessCmdLineJobParameters(params);		
		}
		ce_instance->Run();
	} else if (test_mode)
	{
		ParameterTest* test = new ParameterTest(params);
		test->Run(ETestMemblock);
		test->PrintResults();
		delete test;
	}

final_dealloc:
	delete qs_instance;
	delete no_instance;
	delete ce_instance;
	mpz_clear(argument);
	mpz_clear(fb_size);
	mpz_clear(n_poly_mpz);
	mpz_clear(sieve_int);
	if (BitMatrix::K_SHIFTS) {
		free(BitMatrix::K_SHIFTS);
	}
	if (BCMatrix::K_SHIFTS) {
		free(BCMatrix::K_SHIFTS);
	}
	// clean up xml parser
	xmlCleanupParser();
	// The original handler for signal.
	signal(SIGTERM,originalTermAddress);
	signal(SIGQUIT,originalQuitAddress);
	signal(SIGINT,originalIntAddress);
	return 0;
}


/*! \brief Sole purpose of this function is to print out a brief info on all available parameters.
 *
 *
 *
*/
void help(void){
	printf("\n");
	printf("Multiple polynomial quadratic sieve %.2f, (c) Marian Kechlibar et al., 2003-5.\n",QS_VERSION);
  	printf("Usage: mpqs [-t] [-a] [-h] [-q] [-i] [-n=X] [-m=X] [-f=X] ...\n");
	printf("\n");
	printf("Command-line options:\n -t:         Performs normal Thorough Test Routine.");
	printf("\n -tfast:     Performs fast Thorough Test Routine.");
	printf("\n -tstrict:   Performs extensive Thorough Test Routine.");
	printf("\n -a:         Assertion mode, performs various checks on invariants (slow).");
	printf("\n -h:         This help screen.");
	printf("\n -q:         MPQSieve function. Either this or -s must be used when factoring.");
	printf("\n -s:         SIQSieve function. Either this or -q must be used when factoring.");
	printf("\n -nq:        New line siever MPQSieve function. Is slower than -q.");
	printf("\n -ns:        New line siever SIQSieve function. Is slower than -s.");
	printf("\n -i:         Info mode, displays runtime information on screen. Very useful.");
	printf("\n -debug:     Debug mode, a lot of runtime information.");
	printf("\n -ltime:     Display timing of concrete parts of Lanczos method.*");
	printf("\n -n=X:        Factor number X, given in decimal expression (e.g. -n1026212).");
	printf("\n -m=X:        Use sieving interval [-X,X]. Default is %d.",IMPLICIT_SIEVE_INT);
	printf("\n -f=X:        Use factor base with primes less or equal X. Default is %d.",IMPLICIT_FACTORBASE);
	printf("\n -p=X:        Use X polynomials. (default is %d, maximum %d).",IMPLICIT_POLY,MAX_POLY);
	//printf("\n -w:         Without listing of factorbase primes (useful when f>10000, accelerates startup).");
  	printf("\n -v:         With single large prime variation.");
	printf("\n -v2:        With double large prime variation.");
	printf("\n -bit:       Uses bit matrix representation rather than sparse matrix (slow).");
	printf("\n -de:        Checks and prints density of matrices A and B, very time and mem expensive.");
	printf("\n -checkX:    Changes check level, X anywhere from -5 to 5 (default 0).");	
	printf("\n -checkall:  Checks all generated values for smoothness (very slow!)");
	printf("\n -checkauto: Tries to determine optimal checklevel, recommended.");
	printf("\n -mflex:     Tries to adjust the given sieving interval for optimal speed.");
	printf("\n -valdist:   Checks and prints out distances of accepted values from roots.");
	printf("\n -ranks:     Calculates ranks of A and B (slow, mem expensive, not recommended over size 10^4+).");
	printf("\n -printsort: Prints out results of sort smooth values (accepted/rejected).");	
	printf("\n -tmessageX: Changes intervals between poly info; default 0 = never.*");	
	printf("\n -nodelsingles: Do not delete singletons. For testing only.");
	printf("\n -power:     Sieve with higher powers of small FB primes.");
	printf("\n -_X:        Output progress info to file X (could be used by GUI frontends).");
	printf("\n -extk:      Uses extended modifier 'k', to include 3, 5 and 7 into factor base.");
	printf("\n -testblock:  Test the best size of the new line sieving memblock.");
	printf("\n -memblockXX: Set the size of a memory block for new line sieving.");
	printf("\n -rootshort: Use only proximity of roots for new ls. Experimental.");
	printf("\n -dir XXX:   Use this directory to save/load data from XML. Experimental.");
	printf("\n -load:job,fb: The indicated saved states will be loaded.");
	printf("\n -list:     Printout the saved state in dir, not changing.");
	printf("\n -gz:	      GZIP the saved XML.");
	printf("\n -center:   This instance will be center of a distributed process.");
	printf("\n -node:     This instance will be node of a distributed process.");
	printf("\n -centerinfo=[name,connection,url,timeout]:    The info about a preferred center for the distributed process.");
	printf("\n -conninfo=[connection,url]: The info about a receiving connection.");
	printf("\n -id=XXX:   The machine id for the node.");
	printf("\n -batchdemo: Demonstrate creation of batches of node instructions.");
	printf("\n -keep-messages: archive old messages (in distributed case)");
	printf("\n -to=XXX:   Threshold optimizer. XXX can be either \"off\", or one of integers 0,1,2,3,4,5.");
  	printf("\n --------------------------------------------------------");
	printf("\n * functional only if the program is compiled with specific #constants defined.");
	printf("\n");
	// TODO nasledujici radek neni ve skutecnosti pravdivy
	printf("All options are case and order insensitive.\n");
}

int isDecimalDigit(char ch) {
	if ((ch > '9') || (ch < '0')) return 0;
	return 1;
}

#ifdef WIN32
static int strcasecmp ( char* s1, char* s2 )
{
    unsigned char c1;
    unsigned char c2;
    
    do {
        c1 = tolower(*s1);
        c2 = tolower(*s2);
        if (!c1) {
            break;
        }
        ++s1;
        ++s2;
    } while (c1 == c2);
    return c1 - c2;
}
#endif



