#include "parameter_test.h"

ParameterTest::ParameterTest()
{
	Init();
}

#ifdef GNFS_LINE_SIEVING	
ParameterTest::ParameterTest(nfs_basic_parameters& aParameters)
{
	Init();
	NumberFieldSieve::CopyBasicParameters(this->iParameters,aParameters);
}
#endif

#ifdef QS_LINE_SIEVING	
ParameterTest::ParameterTest(QSParameters& aParameters) : iParameters(aParameters)
{
	Init();	
}
#endif

void ParameterTest::Init()
{
	this->iTestType = ETestUndefined;
	this->iSieve    = NULL;
#ifdef GNFS_LINE_SIEVING
	this->iParameters.iPFNAllocated = FALSE;
	this->iParameters.iNAllocated = FALSE;
	NumberFieldSieve::SetDefaultBasicParameters(this->iParameters);	
#endif
	this->iValues = NULL;
	this->iValuesMaxIndex = -1;
	this->iResults = NULL;
	this->iResultsMaxIndex = -1;
}

int  ParameterTest::Run(test_type aTestType)
{
	this->iTestType = aTestType;
	switch (iTestType)
	{
		case ETestUndefined:
			break;
		case ETestMemblock:
			return TestMemblock();
			break;
		case ETestCheckLevel:
			break;
		case ETestIntegralFBSize:
			break;
		case ETestAlgebraicFBSize:
			break;
		case ETestSievingIntervalSize:
			break;
		default:
			break;
	}
	return ConstRC::Ok;
}

int  ParameterTest::TestMemblock()
{
	int size = 20;
	line_sieve_size* sizes = (line_sieve_size*) malloc(size*sizeof(line_sieve_size));
	sizes[0] =  ESize16k;
	sizes[1] =  ESize32k;
	sizes[2] =  ESize48k;
	sizes[3] =  ESize64k;
	sizes[4] =  ESize96k;
	sizes[5] =  ESize128k;
	sizes[6] =  ESize160k;
	sizes[7] =  ESize192k;
	sizes[8] =  ESize256k;
	sizes[9] =  ESize384k;
	sizes[10] =  ESize512k;
	iValues  = (int*) malloc(size*sizeof(int));
	iResults = (float*) malloc(size*sizeof(float));
	memset(iValues,0, size*sizeof(int));
	memset(iResults,0,size*sizeof(float));
	iValuesMaxIndex = size-1;
	iResultsMaxIndex= size-1;
	int i;
	int max = 11;
	for (i=0; i< max; i++)
	{
#ifdef GNFS_LINE_SIEVING
		this->iSieve = new NumberFieldSieve();
#endif
#ifdef QS_LINE_SIEVING
		this->iSieve = new QuadraticSieve();
#endif
		this->iParameters.Set_MemblockSize(sizes[i] / 1024);
		this->iSieve->SetupParameters(this->iParameters);
#ifdef GNFS_LINE_SIEVING
		this->iSieve->RunNFS();
#endif
#ifdef QS_LINE_SIEVING
		this->iSieve->RunMPQS();	
#endif
		iValues[i]  = this->iParameters.Get_MemblockSize();
		iResults[i] = this->iSieve->GetSecondsInSieving();
		delete this->iSieve;
		this->iSieve = NULL;

		
	}
	free(sizes);
	return ConstRC::Ok;
}

void ParameterTest::PrintResults()
{
	int i, max = iValuesMaxIndex;
	printf("\n\n Parameter Test Results \n");
	switch(iTestType)
	{
		case ETestUndefined:
			printf("   Test parameters undefined.\n");
			break;
		case ETestMemblock:
			printf("   Testing memblock size effectivity.\n");
			break;
		case ETestCheckLevel:
			printf("   Testing checklevel effectivity.\n");
			break;
		case ETestIntegralFBSize:
			break;
		case ETestAlgebraicFBSize:
			break;
		case ETestSievingIntervalSize:
			break;
		default:
			break;
	}
	printf("\n ____________________________________________\n");
	printf("|    SIZE        |      SIEVING TIME        |\n");
	float base = 1000000000;
	for (i=0; i<= max; i++)
	{
		if (iResults[i] >0)
		{
			if (iResults[i] < base)
			{
				base = iResults[i];
			}
		}
	}
	float ratio;
	for (i=0; i<= max; i++)
	{
		printf("|    %4d        |      %6.2f s            | ",iValues[i],iResults[i]);
		ratio = iResults[i] / base;
		ratio *= 20;
		int j;
		
		for (j=0; j< ratio; j++)
		{
			printf("+");
		}
		printf("\n");
	}
	printf(" ____________________________________________\n");
}

ParameterTest::~ParameterTest()
{
#ifdef GNFS_LINE_SIEVING
	NumberFieldSieve::FreeBasicParameters(this->iParameters);
#endif
	delete iSieve;
	delete iValues;	
	delete iResults;
}

