#include "qs_parameters.h"
#include "qs_parameter_names.h"

const unsigned int QSParameters::KMinModulus = 0xffffffffL;
const char*	   QSParameters::KModulusMustBePositive = "Modulus must be positive";
const char*    	   QSParameters::KModulusTooSmall = "Modulus too small";
const unsigned int QSParameters::KMinFBBound = 256;
const char*    	   QSParameters::KFBBoundTooSmall = "Upper FB Bound too small";
const unsigned int QSParameters::KMinSieveInt= 2048;
const char*    	   QSParameters::KSieveIntTooSmall = "Sieving interval too small";
const char*	   QSParameters::KSeparator	= ", ";
const char*	   QSParameters::KFinisher	= ".";


QSParameters::QSParameters()
{	
	InitDefaults();
}

void QSParameters::InitDefaults()
{
	Set_AssertionMode(FALSE);
	Set_QSMode(ESiqs);
	Set_InfoMode(TRUE);
	Set_DebugMode(FALSE);
	Set_CheckMode(ECheckNormal);
	Set_LtMode(FALSE);
	Set_SievingIntervalMode(EFixed);
	Set_CheckLevel(0);
	Set_MemblockSize(64);
	Set_TimeMessageInterval(0);
	Set_DiMode(FALSE);
	Set_LinalgType(ESparseBitMatrix);
	Set_WpList(FALSE);
	Set_UsingVariations(EDoNotUseVariations);
	Set_RankCalculationMode(FALSE);
	Set_ValueDistributionMode(FALSE);
	Set_PrintSort(EDoNotPrintSort);
	Set_ExtendedK(FALSE);
	Set_SingletonMode(EDeleteSingletons);
	Set_PowerMode(EDoNotSieveWithPrimePowers);
	Set_FactorAlg(EPollardRho);
	mpz_init(iN);
	Set_MaxIter(IMPLICIT_POLY);
	Set_SievingIntervalLength(IMPLICIT_SIEVE_INT);	
	Set_UpperIntBound(IMPLICIT_FACTORBASE);
	Set_RootShort(FALSE);
	Set_DirectoryName(".");	
	Set_LoadFlags(0);	
	Set_CompressionLevel(0);

	Set_DistributionMachineType(ENone);
	Set_SharedURL("");	
	Set_MachineName("");	
	Set_CommitInterval(IMPLICIT_COMMIT_INTERVAL);	
	memset(iMachineSpecificDivisors,0,KMaxMSD*sizeof(int));
	Set_Machine_Specific_Divisors_Given(FALSE);		
	Set_MSD_Interval_Given(FALSE);

	Set_ThresholdOptimizer_Level(0);
	Set_MachineSpecificGeneration(MSGTwoDiffer);

}

QSParameters::QSParameters(const QSParameters& aSrc)
{
	InitDefaults();
	Set_AssertionMode(	aSrc.Is_AssertionMode());
	Set_QSMode(		aSrc.Get_QSMode());
	Set_InfoMode(		aSrc.Is_InfoMode());
	Set_DebugMode(		aSrc.Is_DebugMode());
	Set_CheckMode(		aSrc.Get_CheckMode());
	Set_LtMode(		aSrc.Is_LtMode());
	Set_SievingIntervalMode(aSrc.Get_SievingIntervalMode());
	Set_CheckLevel(		aSrc.Get_CheckLevel());
	Set_MemblockSize(	aSrc.Get_MemblockSize());
	Set_TimeMessageInterval(aSrc.Get_TimeMessageInterval());
	Set_DiMode(		aSrc.Is_DiMode());
	Set_LinalgType(		aSrc.Get_LinalgType());
	Set_WpList(		aSrc.Is_WpList());
	Set_UsingVariations(	aSrc.Get_UsingVariations());
	Set_RankCalculationMode(aSrc.Is_RankCalculationMode());
	Set_ValueDistributionMode(aSrc.Is_ValueDistributionMode());
	Set_PrintSort(		aSrc.Get_PrintSort());
	Set_ExtendedK(		aSrc.Is_ExtendedK());
	Set_SingletonMode(	aSrc.Get_SingletonMode());
	Set_PowerMode(		aSrc.Get_PowerMode());
	Set_FactorAlg(		aSrc.Get_FactorAlg());
	aSrc.Get_Modulus(this->iN);
	Set_MaxIter(		aSrc.Get_MaxIter());
	Set_SievingIntervalLength(aSrc.Get_SievingIntervalLength());	
	Set_UpperIntBound(	aSrc.Get_UpperIntBound());
	Set_RootShort(		aSrc.Is_RootShort());
	Set_DirectoryName(	aSrc.Get_DirectoryName());	
	Set_LoadFlags(		aSrc.Get_LoadFlags());	
	Set_CompressionLevel(	aSrc.Get_CompressionLevel());

	Set_DistributionMachineType(aSrc.Get_DistributionMachineType());	
	Set_SharedURL(		aSrc.Get_SharedURL());	
	Set_MachineName(	aSrc.Get_MachineName());	
	Set_CommitInterval(	aSrc.Get_CommitInterval());

	memset(iMachineSpecificDivisors,0,KMaxMSD*sizeof(int));

	Set_Machine_Specific_Divisors_Given(aSrc.Machine_Specific_Divisors_Given());	
	Set_MSD_Interval_Given(	aSrc.MSD_Interval_Given());
	Set_MSD_Lower_Bound(	aSrc.MSD_Lower_Bound());
	Set_MSD_Upper_Bound(	aSrc.MSD_Upper_Bound());
	int i;
	for (i=0; i< aSrc.Machine_Specific_Divisors_Count(); i++)
	{
		Add_MSD(	aSrc.Get_MSD(i));
	}
		Set_ThresholdOptimizer_Level(aSrc.Get_ThresholdOptimizer_Level());
}

QSParameters::QSParameters(const QuadraticSieve& aInstance)
{
	Set_AssertionMode(	aInstance.GetAssertionMode());
	Set_QSMode(		aInstance.Get_QSMode());
	Set_InfoMode(		aInstance.GetInfoMode());
	Set_DebugMode(		aInstance.GetDebugMode());
	Set_CheckMode(		aInstance.Get_CheckMode());
	Set_LtMode(		aInstance.Get_LTMode());
	Set_SievingIntervalMode(aInstance.Get_SievingIntervalMode());
	Set_CheckLevel(		aInstance.Get_CheckLevel());
	Set_MemblockSize(	aInstance.Get_MemblockSize());
	Set_TimeMessageInterval(aInstance.Get_TimeMessageInterval());
	Set_DiMode(		aInstance.Get_DIMode());
	Set_LinalgType(		aInstance.GetMatrixType());
	Set_WpList(		aInstance.Get_WPMode());
	Set_UsingVariations(	aInstance.Get_UsingVariations());
	Set_RankCalculationMode(aInstance.GetRankCalculationMode());
	Set_ValueDistributionMode(aInstance.Get_ValDistMode());
	Set_PrintSort(		aInstance.Get_PrintingSort());
	Set_ExtendedK(		aInstance.Get_ExtK());
	Set_SingletonMode(	aInstance.Get_SingletonMode());
	Set_PowerMode(		aInstance.Get_PowerMode());
	Set_FactorAlg(		aInstance.Get_FactoringAlgorithm());
	mpz_init(this->iN);
	aInstance.Get_N(this->iN);
	Set_MaxIter(		aInstance.Get_NrPolynomials());
	Set_SievingIntervalLength(aInstance.Get_Integer_M());	
	Set_UpperIntBound(	aInstance.Get_UpperFBBound() );
	Set_RootShort(		aInstance.Get_UseRootShort());
	Set_DirectoryName(	aInstance.Get_DirectoryName());			
	Set_CompressionLevel(	aInstance.GetDataCompressionLevel());
	Set_DistributionMachineType(aInstance.Get_DistributionMachineType());	
	Set_SharedURL(		aInstance.Get_SharedURL());	
	Set_MachineName(	aInstance.Get_MachineName());	
	Set_CommitInterval(	aInstance.Get_CommitInterval());
	Set_ThresholdOptimizer_Level(aInstance.GetOptimizationPolicy());
	memset(iMachineSpecificDivisors,0,KMaxMSD*sizeof(int));
}

QSParameters::~QSParameters()
{
	mpz_clear(this->iN);
}

bool	QSParameters::Is_AssertionMode() const
{
return iAssertionMode;
}

int	QSParameters::Get_TimeMessageInterval() const
{
return iTimeMessageInterval;
}


bool	QSParameters::Is_InfoMode() const
{
return iInfoMode;
}

bool	QSParameters::Is_DebugMode() const
{
return iDebugMode;
}


qs_mode	QSParameters::Get_QSMode() const
{
return iQsMode;
}

check_mode	QSParameters::Get_CheckMode() const
{
return iCheckMode;
}

bool	QSParameters::Is_LtMode() const
{
return iLtMode;
}

bool	QSParameters::Is_RootShort() const
{
return iUseRootShort;
}

siev_i_mode	QSParameters::Get_SievingIntervalMode() const
{
return iSievingIntervalMode;
}

int		QSParameters::Get_CheckLevel() const
{
return iCheckLevel;
}

int	QSParameters::Get_MemblockSize() const
{
return iMemblockSize;
}

bool	QSParameters::Is_DiMode() const
{
return iDiMode;
}

linalg_type	QSParameters::Get_LinalgType() const
{
return iLinalgType;
}

bool	QSParameters::Is_WpList() const
{
return iWpList;
}

variations_types	QSParameters::Get_UsingVariations() const
{
return iUseVariations;
}

bool	QSParameters::Is_RankCalculationMode() const
{
return iRankCalculationMode;
}

bool	QSParameters::Is_ValueDistributionMode() const
{
return iValueDistributionMode;
}

print_sort	QSParameters::Get_PrintSort() const
{
return iPrintingSort;
}

bool	QSParameters::Is_ExtendedK() const
{
return iUseExtendedK;
}

sgt_mode QSParameters::Get_SingletonMode() const
{
return iSingletonMode;
}

power_mode QSParameters::Get_PowerMode() const
{
return iPowerMode;
}

factor_alg QSParameters::Get_FactorAlg() const
{
return iFactorAlg;
}

void	QSParameters::Get_Modulus(mpz_t aTarget) const
{
mpz_set(aTarget,iN);
}

int	QSParameters::Get_MaxIter() 	   const
{
return iMaxIter;
}

int	QSParameters::Get_SievingIntervalLength() const
{
return iSievingIntervalLength;
}

int	QSParameters::Get_UpperIntBound() const
{
return iUpperIntBound;
}

string	QSParameters::Get_DirectoryName() const
{
return iDirectoryName;
}

int	QSParameters::Get_LoadFlags() const
{
return iLoadFlags;
}

int	QSParameters::Get_CompressionLevel() const
{
return iCompressionLevel;
}

bool	QSParameters::Machine_Specific_Divisors_Given() const
{
return iMSDGiven;
}

int	QSParameters::Machine_Specific_Divisors_Count() const
{
	int i;
	for (i=0; i< KMaxMSD; i++)
	{
		if (!iMachineSpecificDivisors[i])
		{
			return i;
		}
	}
	return 0;
}

int	QSParameters::Get_MSD(int aIndex) const
{
	int max = Machine_Specific_Divisors_Count();
	if ((aIndex < 0) || (aIndex >= max))
	{
		return 0;
	}
	return iMachineSpecificDivisors[aIndex];
}

bool	QSParameters::MSD_Interval_Given() const
{
	return iMSDIntervalGiven;
}

int	QSParameters::MSD_Lower_Bound() const
{
	return iLowerMSDBound;
}

int	QSParameters::MSD_Upper_Bound() const
{
	return iUpperMSDBound;
}

void	QSParameters::Set_TimeMessageInterval(int aValue)
{
iTimeMessageInterval = aValue;
}

void	QSParameters::Set_InfoMode(bool aValue)
{
iInfoMode = aValue;
}

void	QSParameters::Set_DebugMode(bool aValue)
{
iDebugMode = aValue;
}


void	QSParameters::Set_QSMode(qs_mode aValue)
{
iQsMode = aValue;
}

void	QSParameters::Set_AssertionMode(bool aValue)
{
iAssertionMode = aValue;
}


void	QSParameters::Set_CheckMode(check_mode aValue)
{
iCheckMode = aValue;
}

void 	QSParameters::Set_LtMode(bool aValue)
{
iLtMode = aValue;
}

void 	QSParameters::Set_SievingIntervalMode(siev_i_mode aMode)
{
iSievingIntervalMode = aMode;
}

void	QSParameters::Set_CheckLevel(int aValue)
{
iCheckLevel = aValue;
}

void	QSParameters::Set_MemblockSize(int aValue)
{
iMemblockSize = aValue;
}

void	QSParameters::Set_DiMode(bool aValue)
{
iDiMode = aValue;
}

void	QSParameters::Set_LinalgType(linalg_type aValue)
{
iLinalgType = aValue;
}

void 	QSParameters::Set_WpList(bool aValue)
{
iWpList = aValue;
}

void	QSParameters::Set_UsingVariations(variations_types aValue)
{
iUseVariations = aValue;
}

void	QSParameters::Set_RankCalculationMode(bool aValue)
{
iRankCalculationMode = aValue;
}

void	QSParameters::Set_ValueDistributionMode(bool aValue)
{
iValueDistributionMode = aValue;
}

void	QSParameters::Set_PrintSort(print_sort aValue)
{
iPrintingSort = aValue;
}

void	QSParameters::Set_ExtendedK(bool aValue)
{
iUseExtendedK = aValue;
}

void	QSParameters::Set_SingletonMode(sgt_mode aMode)
{
iSingletonMode = aMode;
}

void	QSParameters::Set_PowerMode(power_mode aMode)
{
iPowerMode = aMode;
}

void	QSParameters::Set_FactorAlg(factor_alg aAlg)
{
iFactorAlg = aAlg;
}

void	QSParameters::Set_Modulus(const mpz_t aValue)
{
mpz_set(iN,aValue);
}

void	QSParameters::Set_MaxIter(int aValue)
{
iMaxIter = aValue;
}

void	QSParameters::Set_SievingIntervalLength(int aValue)
{
iSievingIntervalLength = aValue;
}

void	QSParameters::Set_UpperIntBound(int aValue)
{
iUpperIntBound = aValue;
}

void	QSParameters::Set_DirectoryName(const string& aValue)
{
iDirectoryName.assign(aValue);
}

void	QSParameters::Set_DirectoryName(const char*  aValue)
{
	if (aValue)
	{
	iDirectoryName.assign(aValue);
	}
	else
	{
	// The default value: this directory.
	iDirectoryName.assign(".");
	}
}

void	QSParameters::Set_LoadFlags(int aValue)
{
iLoadFlags = aValue;
}
void	QSParameters::Set_Flag(int aFlag)
{
iLoadFlags |= aFlag;
}

void	QSParameters::Set_CompressionLevel(int aValue)
{
iCompressionLevel = aValue;
}

void	QSParameters::Set_RootShort(bool aValue)
{
iUseRootShort = aValue;
}



void    QSParameters::Set_Machine_Specific_Divisors_Given(bool aState)
{
	iMSDGiven = aState;
	if (!aState)
	{
		memset(iMachineSpecificDivisors, 0, KMaxMSD*sizeof(int));
	}
}

void	QSParameters::Add_MSD(int aMachineSpecificDivisor)
{
	if (!iMSDGiven)
	{
		return;
	}
	int i;
	for (i=0; i < KMaxMSD; i++)
	{
		if (!iMachineSpecificDivisors[i])
		{
// 			printf("Add_MSD: Added %d to index %d\n",aMachineSpecificDivisor,i);
			iMachineSpecificDivisors[i] = aMachineSpecificDivisor;
			return;
		}
	}
}

void	QSParameters::Set_MSD_Interval_Given(bool aState)
{
	iMSDIntervalGiven = aState;
	if (!aState)
	{
		Set_MSD_Lower_Bound(0);
		Set_MSD_Upper_Bound(0);
	}
}

void 	QSParameters::Set_MSD_Lower_Bound(int aLower)
{
	iLowerMSDBound = aLower;
}

void 	QSParameters::Set_MSD_Upper_Bound(int aUpper)
{
	iUpperMSDBound = aUpper;
}

const string  QSParameters::GetStringParameter(const char* aName) const
{
	if (aName != NULL)
	{
		if (!strcmp(aName,KDirectoryName))
		{
			return Get_DirectoryName();
		}
		if (!strcmp(aName,KSharedUrl))
		{
			return Get_SharedURL();
		}
		if (!strcmp(aName,KMachineName))
		{
			return Get_MachineName();
		}
	}
	// Either aName == NULL, or it has not been caught by previous code (is unknown)
	return KStringParameterUndefined;	
}

int      QSParameters::GetIntegerParameter(const char* aName) const
{
	if (aName != NULL)
	{
		if (!strcmp(aName,KQsMode))
		{
			return Get_QSMode();
		}
		if (!strcmp(aName,KCheckMode))
		{
			return Get_CheckMode();
		}
		if (!strcmp(aName,KSieveIntMode))
		{
			return Get_SievingIntervalMode();
		}
		if (!strcmp(aName,KCheckLevel))
		{
			return Get_CheckLevel();
		}
		if (!strcmp(aName,KMemblockSize))
		{
			return Get_MemblockSize();
		}
		if (!strcmp(aName,KTimeMessage))
		{
			return Get_TimeMessageInterval();
		}
		if (!strcmp(aName,KLinalgType))
		{
			return Get_LinalgType();
		}
		if (!strcmp(aName,KUseVariations))
		{
			return Get_UsingVariations();
		}
		if (!strcmp(aName,KPrintSort))
		{
			return Get_PrintSort();
		}
		if (!strcmp(aName,KSingletonMode))
		{
			return Get_SingletonMode();
		}
		if (!strcmp(aName,KPowerMode))
		{
			return Get_PowerMode();
		}
		if (!strcmp(aName,KFactorAlg))
		{
			return Get_FactorAlg();
		}
		if (!strcmp(aName,KMaxIter))
		{
			return Get_MaxIter();
		}		
		if (!strcmp(aName,KSILength))
		{
			return Get_SievingIntervalLength();
		}
		if (!strcmp(aName,KUIBound))
		{
			return Get_UpperIntBound();
		}
		if (!strcmp(aName,KLoadFlags))
		{
			return Get_LoadFlags();
		}
		if (!strcmp(aName,KCompLevel))
		{
			return Get_CompressionLevel();
		}
		if (!strcmp(aName,KDistMachType))
		{
			return Get_DistributionMachineType();
		}
		if (!strcmp(aName,KCommitInt))
		{
			return Get_CommitInterval();
		}
	}
	// Either aName == NULL, or it has not been caught by previous code (is unknown)
	return KIntegerParameterUndefined;	
}

double   QSParameters::GetDoubleParameter(const char* aName) const
{
	if (aName != NULL)
	{
		// No Double parameters yet.
	}
	// Either aName == NULL, or it has not been caught by previous code (is unknown)
	return KDoubleParameterUndefined;	
}

bool     QSParameters::GetBoolParameter(const char* aName) const
{
	if (aName != NULL)
	{
		if (!strcmp(aName,KAssertionMode))
		{
			return Is_AssertionMode();
		}
		if (!strcmp(aName,KInfoMode))
		{
			return Is_InfoMode();
		}
		if (!strcmp(aName,KLtMode))
		{
			return Is_LtMode();
		}
		if (!strcmp(aName,KDiMode))
		{
			return Is_DiMode();
		}
		if (!strcmp(aName,KWpList))
		{
			return Is_WpList();
		}
		if (!strcmp(aName,KRankCalcMode))
		{
			return Is_RankCalculationMode();
		}
		if (!strcmp(aName,KValDistMode))
		{
			return Is_ValueDistributionMode();
		}
		if (!strcmp(aName,KExtendedKMode))
		{
			return Is_ExtendedK();
		}
		if (!strcmp(aName,KRootShort))
		{
			return Is_RootShort();
		}

	}
	// Either aName == NULL, or it has not been caught by previous code (is unknown)
	return FALSE;	
}

int     QSParameters::GetMpzParameter(const char* aName, mpz_t aResult) const
{
	if (aName != NULL)
	{
		if (!strcmp(aName,KModulus))
		{
			mpz_set(aResult,iN);	
			return ConstRC::Ok;
		}		
	}

	// Either aName == NULL, or it has not been caught by previous code (is unknown)
	mpz_set_si(aResult,KIntegerParameterUndefined);

	return ConstRC::GeneralError;
}

bool	 QSParameters::Complete() const
{
	if (mpz_cmp_ui(iN,0) <= 0)
	{
		return FALSE;
	}
	if (mpz_cmp_ui(iN,KMinModulus) < 0)
	{
		return FALSE;
	}
	if (iUpperIntBound < KMinFBBound)
	{
		return FALSE;
	}
	if (iSievingIntervalLength < KMinSieveInt)
	{
		return FALSE;
	}
	return TRUE;
}

string	 QSParameters::GetFailureReason() const
{
	string result;
	if (mpz_cmp_ui(iN,0) < 0)
	{
		if (!result.empty())
		{
		result += KSeparator;
		}
		result += KModulusMustBePositive;
	}
	if (mpz_cmp_ui(iN,KMinModulus) < 0)
	{
		if (!result.empty())
		{
		result += KSeparator;
		}
		result += KModulusTooSmall;
	}
	if (iUpperIntBound < KMinFBBound)
	{
		if (!result.empty())
		{
		result += KSeparator;
		}
		result += KFBBoundTooSmall;
	}
	if (iSievingIntervalLength < KMinSieveInt)
	{		
		if (!result.empty())
		{
		result += KSeparator;
		}
		result += KSieveIntTooSmall;
	}
	if (!result.empty())
		{
		result += KFinisher;
		}
	return result;
}

QSParameters*	QSParameters::CreateCopy() const
{
	QSParameters* result = new QSParameters(*this);
	return result;
}

void QSParameters::Print() const
{
	printf("QSParameters\n");
	printf("Lower MSD bound: %d\n",iLowerMSDBound);
	printf("Upper MSD bound: %d\n",iUpperMSDBound);	
	printf("MSD given      : %d\n",iMSDGiven);
	if (iMSDGiven)
	{
		for (int i=0; i< KMaxMSD; i++)
		{
			if (iMachineSpecificDivisors[i])
			{
				printf("  MSD[%d] = %d\n",i,iMachineSpecificDivisors[i]);
			}
		}
	}
}

void QSParameters::PrintInBatch(const char *aPrefix) const {

	return;
}

void QSParameters::Set_ThresholdOptimizer_Level(int aValue)
{
	iThresholdOptimizerLevel = aValue;
}

int QSParameters::Get_ThresholdOptimizer_Level() const
{
	return iThresholdOptimizerLevel;
}

void QSParameters::Set_FrontendPipe(const char* aValue) {
	iFrontendPipe = aValue;
}

string QSParameters::Get_FrontendPipe() const {
	return iFrontendPipe;
} 

void QSParameters::Set_MachineSpecificGeneration(machine_specific_generation_type aValue)
{
	iMachineSpecificGeneration = aValue;
}

machine_specific_generation_type QSParameters::Get_MachineSpecificGeneration() const
{
	return iMachineSpecificGeneration;
}
