#ifndef __XML_QS_SERIALIZATOR__
#define __XML_QS_SERIALIZATOR__

#include "abstract_serializator.h"
#include "quadratic_sieve.h"
#include <libxml/encoding.h>
#include <libxml/xmlwriter.h>
#include <libxml/xmlreader.h>
#include "common_xml_helper.h"
#include "xml_message_definitions.h"
#include "qs_parameters.h"
#include "algorithm.h"

// Forward declaration
class QuadraticSieve;


/*!
 * \brief Serializator for QS
 */
class XmlQsSerializator : public AbstractSerializator
{
	public:
		XmlQsSerializator();
		XmlQsSerializator(const char* aDirectoryName, int aCompressionLevel);
		XmlQsSerializator(AbstractSieve* aSieveInstance, const char* aDirectoryName, int aCompressionLevel);
		virtual ~XmlQsSerializator();
		virtual int Serialize(int aAction);
		virtual int Deserialize(int aAction);	

		virtual int SerializeJob(xmlTextWriterPtr& aWriter, const JobParameters& aParameters);
		virtual int DeserializeJob(xmlTextReaderPtr& aReader, JobParameters& aParameters);		
		virtual int DeserializeJob(const char* aPath, JobParameters& aParameters);

		virtual int SerializeResult();
		virtual int SerializeJob(xmlTextWriterPtr& aWriter, const QSParameters& aParameters);
		virtual int DeserializeJob(xmlTextReaderPtr& aReader, QSParameters& aParameters);		
		virtual int DeserializeJob(JobParameters& aParameters);
		virtual int DeserializeJob(QSParameters& aParameters);
		virtual int SerializeData(xmlTextWriterPtr& aWriter); ///< Used in parallelization to send fresh data from Node to Center.

		virtual int DeserializeData(const char* aPath);

		virtual Algorithm* CreateInstance(const JobParameters& aParameters) const;
		virtual Algorithm* CreateInstance(const QSParameters& aParameters) const;
		
	//  From Algorithm
		virtual const char* 	AlgorithmName()       const;
		virtual JobParameters*  CreateNewParameters() const;
		// Has no real meaning in XML QS Serializator
		virtual int		SetupParameters(const JobParameters& aParameters);

		virtual void 	RegisterInstance(Algorithm*	aAlgorithm);

		virtual void	RegisterInstance(AbstractSieve* aSieveInstance);

		virtual void	PrintInstance() const;

	protected:
		virtual int SerializeJob();
		virtual int DeserializeJob();
		virtual int SerializeSieving();
		virtual int DeserializeSieving();
		virtual int SerializeFactorBase();
		virtual int DeserializeFactorBase();
		virtual int SerializeSieverState();
		virtual int DeserializeSieverState();
		//virtual int WriteSerializatorData(xmlTextWriterPtr& aWriter);	
		virtual int ReadSerializatorData(xmlTextReaderPtr& aReader, const char* aMessage);
		virtual int DeserializePolynomial(xmlTextReaderPtr& aReader, quad_polynomial* aArray,int aMaxIndex);

		int	DeserializeNodeInfo(xmlTextReaderPtr& aReader, QSParameters* aParameters = NULL);
		int	SerializeNodeInfo(xmlTextWriterPtr& aWriter, const QSParameters* aParameters = NULL);
	private:
		// This pointer does not belong to serializator and will not be deleted
		// on destruction time.
		QuadraticSieve* iQsInstance;					
};

#endif

