/*
ECM.cpp : Defines the entry point for the console application.
2014 - 2015
Adla Hankov
*/

#include <stdlib.h>

#include "iostream" 
#ifdef VLD
#include <vld.h> //Visual Leak Detector for Visual C++
#endif

//includes for NFS
#ifdef INCLUDED_ECM
#include "const_return_codes.h"
#include "common_utils.h"
#include "integer_class.h"
#endif

//other includes
#include <stdint.h>
#include <cstring> 
#include <time.h>       /* clock_t, clock, CLOCKS_PER_SEC */

#include "gmp.h"
#include "ECM.h"
#include "ECM_parameters.h"

using namespace std;

#ifdef EXTERN_FILE
FILE * myfile; 
FILE * numbers;
#endif

/*!
The default constructor.
*/
#ifdef INCLUDED_ECM
ECM::ECM(long aUpperFBBound, int aNumCurves) : ASupportingFactorAlg()
#else
ECM::ECM(long aUpperFBBound, int aNumCurves)
#endif
{
	averagetime = 0;
	test = false;

	N = NULL;
	d = NULL;
	x = NULL;
	y = NULL;
	oneMon = NULL;
	N_dashed = NULL;
	output = NULL;

	kernel = NULL;

	//Set_AlgorithmType(EFAPollardRho); //TODO
	setupCL();
	setup(aUpperFBBound, aNumCurves);

	/* TODO?
	if ((inited & AUX_1_INITED) == 0){
		mpz_init(aux_1);
		inited |= AUX_1_INITED;
	}
	if ((inited & AUX_2_INITED) == 0){
		mpz_init(aux_2);
		inited |= AUX_2_INITED;
	}
	if ((inited & LAST_INITED) == 0) {
		mpz_init(last);
		inited |= LAST_INITED;
	}
	if ((inited & FACT_1_INITED) == 0){
		mpz_init(fact_1);
		inited |= FACT_1_INITED;
	}
	if ((inited & FACT_2_INITED) == 0){
		mpz_init(fact_2);
		inited |= FACT_2_INITED;
	}*/
}

ECM::~ECM()
{
	if (N != NULL) delete[] N;
	if (d != NULL) delete d;
	if (x != NULL) delete x;
	if (y != NULL) delete y;
	if (oneMon != NULL) delete oneMon;
	if (N_dashed != NULL) delete N_dashed;
	if (output != NULL) delete output;

	if (kernel != NULL) cleanup();
}


#ifdef INCLUDED_ECM
/*!
* An overloaded method from ASupportingFactorAlg, providing interface for factorization.
*
* \return Returns either
* - ConstRC::Ok - computation successful
* - ConstRC::NotFactorized - computation unsuccessful
*/
int ECM::Factor(mpz_t aModulus, mpz_t aFactor1, mpz_t aFactor2) {
	mpz_t * Factor1 = new mpz_t[1];
	mpz_init(Factor1[0]);
	mpz_t * Factor2 = new mpz_t[1];
	mpz_init(Factor2[0]);
	mpz_t * Modulus = new mpz_t[1];
	mpz_init(Modulus[0]);
	mpz_set(Modulus[0], aModulus);

	FactorArray(1, Modulus, Factor1, Factor2);

	mpz_set(aFactor1, Factor1[0]);
	mpz_set(aFactor2, Factor2[0]);

	//TODO je to sprvn?
	mpz_clear(Factor1[0]);
	mpz_clear(Factor2[0]);
	mpz_clear(Modulus[0]);
	delete[] Factor1;
	delete[] Factor2;
	delete[] Modulus;
	//return ConstRC::NotFactorized;
	return 0;
}

/*!
* Factorizes a whole array of numbers. All input arrays are considered
* allocated and all of their mpz_t already initialized.
*
* \param aCount - amount of numbers for factorization
* \param aComposite - numbers for factorization
* \param aFactors1 - first array of factors
* \param aFactors2 - second array of factors
* \return Count of successful factorizations
*/
int ECM::FactorArray(int aCount, mpz_t * aComposite, mpz_t * aFactors1, mpz_t * aFactors2) {
	int success = 0;

	//TODO
	/*for (int i = 0; i < aCount; i++) {
		if (ConstRC::Ok == Factor(aComposite[i], aFactors1[i], aFactors2[i]))
			success++;
	};*/
	N = aComposite;
	quantity = aCount;
	Factors1 = aFactors1;
	Factors2 = aFactors2;
	success = runKernels();
	return success;
}
#endif

/*!
Cleanup the opencl memory allocations
*/
int ECM::cleanup()
{
	/*Step 12: Clean the resources. */
	cl_int status;
	status = clReleaseKernel(kernel);				//Release kernel.
	status = clReleaseProgram(program);				//Release the program object.
	status = clReleaseMemObject(inputBuffer0);		//Release mem objects.
	status = clReleaseMemObject(inputBuffer1);
	status = clReleaseMemObject(inputBuffer2);
	status = clReleaseMemObject(inputBuffer3);
	status = clReleaseMemObject(inputBuffer4);
	status = clReleaseMemObject(inputBuffer5);
	status = clReleaseMemObject(inputBuffer6);
	status = clReleaseMemObject(inputBuffer7);
	status = clReleaseMemObject(inputBuffer8);
	status = clReleaseMemObject(inputBuffer9);
	status = clReleaseMemObject(outputBuffer);
	status = clReleaseCommandQueue(commandQueue);	//Release  Command queue.
	status = clReleaseContext(context);				//Release context.

	if (output != NULL)
	{
		free(output);
		output = NULL;
	}

	if (devices != NULL)
	{
		free(devices);
		devices = NULL;
	}
	return SUCCESS;
}

/*!
Converts results from my_mpz to mpz_t and computes GCD of results and the given number
(last step of the ECM algorithm)
Function will find and write only the first divisor
*/
int ECM::getResults(mpz_t aFactor)
{
	mpz_t result, power, base;
	mpz_init_set_ui(base, MAX); //base is 2^32
	mpz_add_ui(base, base, 1);
	mpz_init(result);
	mpz_init(power);
	for (int i = 0; i < NumCurves; i++)
	{
		mpz_set_ui(result, 0);
		// convert from my_mpz to mpz_t
		for (int j = 0; j < DELKA; j++)
		{
			mpz_pow_ui(power, base, j);
			mpz_mul_ui(power, power, output[i].coeff[j]);
			mpz_add(result, result, power);
		}
		//fromMontgomery(result, &result);
		mpz_gcd(power, result, actualN); //I don't need power anymore, so I will use it to store the gcd
		if (mpz_cmp_si(power, 1) != 0 && mpz_cmp(power, actualN) != 0) //nontrivial divisor found
		{
			mpz_set(aFactor, power);
			mpz_clear(result);
			mpz_clear(power);
			mpz_clear(base);
			return 0;
		}
	}
	//divisor not found
	mpz_set_ui(aFactor, 1);
	mpz_clear(result);
	mpz_clear(power);
	mpz_clear(base);
	return 0;
}


/*!
Create buffers, set values for kernels' arguments, enqueue calls to the kernels
on to the command queue, wait till end of kernel execution.
Copy the results back.
Get kernel start and end time if timing is enabled
*/
int ECM::runKernels()
{
	int status;
#ifdef TIME_INFO
	clock_t run_time_cpu;
	float run_time_gpu;
	float one_curve_time;
#endif
	//getPrimes();
	if (test)
		generateN();

#ifdef EXTERN_FILE
	/* write info */
	char buffer[16] = { 0 };
	sprintf_s(buffer, "Curves: %u\n", NumCurves);
	fputs(buffer, myfile);
	sprintf_s(buffer, "B1: %u\n", B1);
	fputs(buffer, myfile);
	sprintf_s(buffer, "Quantity: %u\n", quantity);
	fputs(buffer, myfile);
	fputs("-----------------------------------------------------------------------------------------\n ", myfile);
#endif

	for (int i = 0; i < quantity; i++)
	{
#ifdef TIME_INFO
		run_time_cpu = clock();
#endif
		mpz_init_set(actualN, N[i]);
		r = mpz_size(actualN);

#ifdef EXTERN_FILE
		mpz_out_str(myfile, 10, actualN);
		fputs("\n", myfile);
		mpz_out_str(numbers, 10, actualN);
		fputs("\n", numbers);
#endif

		mpz_t rm;
		mpz_init(rm);
		mpz_init_set_ui(rm, 1);
		mpz_mul_2exp(rm, rm, r * 32);
		//cout << "\nR = ";
		//mpz_out_str(NULL, 10, rm);
		mpz_clear(rm);
		generateParameters();
		mpzToMyMpz(N[i], &myN);

		status = sizeof(my_mpz);
		status = sizeof(int);
		status = sizeof(char);
		status = sizeof(cl_int);

		//output = (my_mpz*)malloc(NumCurves * sizeof(my_mpz)); 
		output = new my_mpz[NumCurves];

		outputBuffer = clCreateBuffer(context, CL_MEM_WRITE_ONLY, NumCurves * sizeof(my_mpz), NULL, NULL); //ten posledn NULL je na chybu
		inputBuffer0 = clCreateBuffer(context, CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, sizeof(my_mpz), (void *)&myN, NULL);
		inputBuffer1 = clCreateBuffer(context, CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, sizeof(unsigned int), (void *)&B1, NULL);
		inputBuffer2 = clCreateBuffer(context, CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, NumCurves * sizeof(my_mpz), (void *)d, NULL);
		inputBuffer3 = clCreateBuffer(context, CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, NumCurves * sizeof(my_mpz), (void *)x, NULL);
		inputBuffer4 = clCreateBuffer(context, CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, NumCurves * sizeof(my_mpz), (void *)y, NULL);
		inputBuffer5 = clCreateBuffer(context, CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, primesLength * sizeof(unsigned int), (void *)primes, NULL);
		inputBuffer6 = clCreateBuffer(context, CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, sizeof(int), (void *)&primesLength, NULL);
		inputBuffer7 = clCreateBuffer(context, CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, sizeof(my_mpz), (void *)oneMon, NULL);
		inputBuffer8 = clCreateBuffer(context, CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, sizeof(unsigned int), (void *)&r, NULL);
		inputBuffer9 = clCreateBuffer(context, CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, sizeof(my_mpz), (void *)N_dashed, NULL);

		/*Step 8: Create kernel object */
		kernel = clCreateKernel(program, "factorize", &status);
		//cout << "Create kernel status: " << status << endl;
		if (status < 0)
		{
			cout << "Couldn't create kernel" << endl;
			exit(1);
		}
		/*Step 9: Sets Kernel arguments.*/
		status = clSetKernelArg(kernel, 0, sizeof(cl_mem), (void *)&outputBuffer);
		//cout << "Set kernel arg 0 status: " << status << "\n";
		status = clSetKernelArg(kernel, 1, sizeof(cl_mem), (void *)&inputBuffer0); 
		//cout << "Set kernel arg 1 status: " << status << "\n";
		status = clSetKernelArg(kernel, 2, sizeof(cl_mem), (void *)&inputBuffer1);
		//cout << "Set kernel arg 2 status: " << status << "\n";
		status = clSetKernelArg(kernel, 3, sizeof(cl_mem), (void *)&inputBuffer2);
		//cout << "Set kernel arg 3 status: " << status << "\n";
		status = clSetKernelArg(kernel, 4, sizeof(cl_mem), (void *)&inputBuffer3);
		//cout << "Set kernel arg 4 status: " << status << "\n";
		status = clSetKernelArg(kernel, 5, sizeof(cl_mem), (void *)&inputBuffer4);
		//cout << "Set kernel arg 5 status: " << status << "\n";
		status = clSetKernelArg(kernel, 6, sizeof(cl_mem), (void *)&inputBuffer5);
		//cout << "Set kernel arg 6 status: " << status << "\n";
		status = clSetKernelArg(kernel, 7, sizeof(cl_mem), (void *)&inputBuffer6);
		//cout << "Set kernel arg 7 status: " << status << "\n";
		status = clSetKernelArg(kernel, 8, sizeof(cl_mem), (void *)&inputBuffer7);
		//cout << "Set kernel arg 8 status: " << status << "\n";
		status = clSetKernelArg(kernel, 9, sizeof(cl_mem), (void *)&inputBuffer8);
		//cout << "Set kernel arg 9 status: " << status << "\n";
		status = clSetKernelArg(kernel, 10, sizeof(cl_mem), (void *)&inputBuffer9);
		//cout << "Set kernel arg 10 status: " << status << "\n";

#ifdef TIME_INFO
		run_time_cpu = clock() - run_time_cpu;
		//cout << "Preparation time: " << (float)run_time_cpu / CLOCKS_PER_SEC << " s" << endl;
		one_curve_time = MILIARDA *(float)run_time_cpu / CLOCKS_PER_SEC;
		//averagetime += MILIARDA *(float)run_time_cpu / CLOCKS_PER_SEC;
#endif // TIME_INFO

#ifdef GET_KERNEL_INFO /* Get and print kernel workgroup info */
		size_t vlaken;
		status = clGetKernelWorkGroupInfo(kernel, *devices, CL_KERNEL_PREFERRED_WORK_GROUP_SIZE_MULTIPLE, sizeof(size_t), &vlaken, NULL);
		cout << "Preferred work group size multiple: " << vlaken << endl;
		cl_ulong vel;
		status = clGetKernelWorkGroupInfo(kernel, *devices, CL_KERNEL_LOCAL_MEM_SIZE, sizeof(cl_ulong), &vel, NULL);
		cout << "Kernel local mem size " << vel << std::endl;
		status = clGetKernelWorkGroupInfo(kernel, *devices, CL_KERNEL_PRIVATE_MEM_SIZE, sizeof(cl_ulong), &vel, NULL);
		cout << "Kernel private mem size " << vel << std::endl;
#endif

		/*Step 10: Running the kernel.*/
		size_t global_work_size[1] = { NumCurves }; //TODO nastav poet vlken !!

#ifdef TIME_INFO
		cl_event prof_event;
		status = clEnqueueNDRangeKernel(commandQueue, kernel, 1, NULL, global_work_size, NULL, 0, NULL, &prof_event);
#else
		status = clEnqueueNDRangeKernel(commandQueue, kernel, 1, NULL, global_work_size, NULL, 0, NULL, NULL);
#endif

		//cout << "Enqueue kernel status: " << status << "\n";
		if (status < 0)
		{
			cout << "Couldn't enqueue kernel.";
			exit(1);
		}

		status = clFinish(commandQueue); //TODO - chci j tady pro ely men, ale jinak mon ne?
		//cout << "Finish status: " << status << "\n";

#ifdef TIME_INFO
		cl_ulong ev_start_time = (cl_ulong)0;
		cl_ulong ev_end_time = (cl_ulong)0;

		status |= clGetEventProfilingInfo(prof_event, CL_PROFILING_COMMAND_QUEUED, sizeof(cl_ulong), &ev_start_time, NULL);
		status |= clGetEventProfilingInfo(prof_event, CL_PROFILING_COMMAND_END, sizeof(cl_ulong), &ev_end_time, NULL);
		//the resolution of the events is 1e-09 sec - nanoseconds
		run_time_gpu = (float)(ev_end_time - ev_start_time);
		one_curve_time += run_time_gpu;
		//averagetime += run_time_gpu;
		run_time_gpu /= MILIARDA;
		//cout << "Elapsed time: " << run_time_gpu << " s " << endl;
#endif

		/*Step 11: Read the cout put back to host memory.*/
#ifdef TIME_INFO
		status = clEnqueueReadBuffer(commandQueue, outputBuffer, CL_TRUE, 0, NumCurves * sizeof(my_mpz), output, 0, NULL, &prof_event);
#else
		status = clEnqueueReadBuffer(commandQueue, outputBuffer, CL_TRUE, 0, NumCurves * sizeof(my_mpz), output, 0, NULL, NULL);
#endif
		if (status < 0) 
		{
			cout << "Couldn't enqueue read buffer";
			exit(1);
		}
#ifdef TIME_INFO
		status |= clGetEventProfilingInfo(prof_event, CL_PROFILING_COMMAND_QUEUED, sizeof(cl_ulong), &ev_start_time, NULL);
		status |= clGetEventProfilingInfo(prof_event, CL_PROFILING_COMMAND_END, sizeof(cl_ulong), &ev_end_time, NULL);
		run_time_gpu = (float)(ev_end_time - ev_start_time);
		//averagetime += run_time_gpu;
		one_curve_time += run_time_gpu;
		run_time_gpu /= MILIARDA;
		//cout << "Read buffer time: "  << run_time_gpu << " s" << endl;

		clReleaseEvent(prof_event);
		run_time_cpu = clock();
#endif
		getResults(Factors1[i]);
		mpz_div(Factors2[i], actualN, Factors1[i]);

		delete[] output;
		output = NULL;
#ifdef TIME_INFO
		run_time_cpu = clock() - run_time_cpu;
		//cout << "Get results time: " << (float)run_time_cpu / CLOCKS_PER_SEC << " s" << endl;
		one_curve_time += MILIARDA *(float)run_time_cpu / CLOCKS_PER_SEC;
		//averagetime += MILIARDA *(float)run_time_cpu / CLOCKS_PER_SEC;
		//cout << "Time spent on this number: " << one_curve_time/MILIARDA << " s" << endl;
#ifdef EXTERN_FILE
		fputs("Time spent on this number: " , myfile);
		char buffer[16] = { 0 };
		sprintf_s(buffer, "%f\n", one_curve_time / MILIARDA);
		fputs(buffer, myfile);
#endif
		averagetime += one_curve_time;
#endif
		//cout << "Number done.\n";
#ifdef EXTERN_FILE
		fputs("-----------------------------------------------------------------------------------------\n ", myfile);
#endif
	}
#ifdef TIME_INFO
	averagetime = averagetime / quantity;
#endif
	return SUCCESS;
}

#ifndef INCLUDED_ECM
int main(int argc, char* argv[])
{
	ECM myECM(5000,2);

	if (argc > 1)
	{
		ECMparameters param = ECMparameters();
		int error = param.ParseInputParameters(argc, argv);
		if (error)
		{
			return FAILURE;
		}
		error = param.check();
		if (error)
		{
			return FAILURE;
		}
		myECM.setup(&param);
	}
	else
	{
		mpz_t cislo;
		mpz_init(cislo);
		int vlaken;
		unsigned int aB1;
		//run program without
		//TODO oetit proti patnm vstupm
		cout << "Enter the number for factorization: ";
		mpz_inp_str(cislo, NULL, 10); //nacti cislo ze vstupu

		cout << "Enter the number of threads: ";
		cin >> vlaken;

		cout << "Enter the bound for primes (B1): ";
		cin >> aB1;

		myECM.setup(cislo, aB1, vlaken); //!
	}
	
#ifdef EXTERN_FILE
	/* Create the file, named with date and time */
	time_t rawtime;
	struct tm timeinfo;
	char buffer[80];

	time(&rawtime);
	localtime_s(&timeinfo, &rawtime);

	strftime(buffer, 80, "%d_%m__%H_%M_%S.txt", &timeinfo);
	int err = fopen_s(&myfile, buffer, "w");
	strftime(buffer, 80, "%d_%m__%H_%M_%S_numbers.txt", &timeinfo);
	err = fopen_s(&numbers, buffer, "w");
#endif
	myECM.setupCL();
	myECM.runKernels();
	//myECM.getResults();
	cout << endl << "Average time: " << myECM.averagetime / MILIARDA << " s" << endl;
	myECM.cleanup();

	cout << endl << "Passed!";
#ifdef EXTERN_FILE
	fputs("Average time: ", myfile);
	sprintf_s(buffer, "%f\n", myECM.averagetime / MILIARDA);
	fputs(buffer, myfile);
	fclose(myfile);
#endif
#ifdef VLD
	_CrtDumpMemoryLeaks();
#endif
	return SUCCESS;
}
#endif 
