#ifndef ECM_H_
#define ECM_H_

//#include "gmp.h"
#include "CL/cl.h"
#include "ECM_parameters.h"
#include "ECM_const.h"



#define SUCCESS 0
#define FAILURE 1

#define DELKA 8
#define MAX 4294967295 //UINT MAX
#define MILIARDA 1000000000


#ifdef INCLUDED_ECM
#include "abstract_supporting_factor_alg.h"
#endif

/*!
 * \brief long integers for OpenCL
 */
typedef struct
{
	unsigned int coeff[DELKA];
	//char numCoeff;
}my_mpz;


/*!
 * \brief ECM implementation using OpenCL
 */
#ifdef INCLUDED_ECM
class ECM : public ASupportingFactorAlg
#else
class ECM 
#endif
{
public:

	ECM(long aUpperFBBound, int numCurves);
	~ECM();

#ifdef INCLUDED_ECM
	int Factor(mpz_t aModulus, mpz_t aFactor1, mpz_t aFactor2);
	int FactorArray(int aCount, mpz_t * aComposite, mpz_t * aFactors1, mpz_t * aFactors2);
#endif
	int setup(int aB1, int aNumCurves);

	int setup(ECMparameters * parameters);
	int setup(mpz_t aN, int aB1, int aNumCurves);
	int setupCL();
	int runKernels();
	int cleanup();

#ifdef TIME_INFO
	float averagetime;
#endif

private:
	//****** VARIABLES *******//
	
	cl_context context;             /**< CL context */
	cl_device_id *devices;          /**< CL device list */
	cl_uint	numDevices;

	cl_mem inputBuffer0;			// N
	cl_mem inputBuffer1;			//B1
	cl_mem inputBuffer2;			// * a
	cl_mem inputBuffer3;            // * x
	cl_mem inputBuffer4;            // * z
	cl_mem inputBuffer5;            // * primes
	cl_mem inputBuffer6;            // length
	cl_mem inputBuffer7;		    // invers of 4 mod N
	cl_mem inputBuffer8;		    // r
	cl_mem inputBuffer9;		    // N_dashed
	cl_mem outputBuffer;

	//cl_mem dataBuf;                 /**< CL memory buffer for data */
	//cl_mem midDeviceBinBuf;         /**< CL memory buffer for intermittent device bin */

	cl_command_queue commandQueue;      /**< CL command queue */
	cl_program program;                 /**< CL program  */
	cl_kernel kernel;                   /**< CL kernel */
	unsigned int r;                     /* r needed for montgomery algorithm, this will be (2^32)^r...so r is number of limbs */

	my_mpz * output;

	mpz_t * N;            /* Numbers for factorizing */
	mpz_t * Factors1;
	mpz_t * Factors2;
	mpz_t actualN;
	int quantity;		  //amount of numbers
	my_mpz myN;			  /* Number for factorizing in my_mpz format*/
	my_mpz * d;           /* Array of coefficients of curve */
	my_mpz * x;           /* Array of x coordinates of points P0 */
	my_mpz * y;           /* Array of z coordinates of points P0 */
	my_mpz * oneMon;	  /* number one in Montgomery representation */
	my_mpz * N_dashed;      /* = -N^(-1) mod r */
	unsigned int B1;              /* First bound for ECM algorithm */
	unsigned int B2;       /*second bound for ECM algorithm*/
	unsigned int * primes; //array of small primes
	unsigned int * differences; //array of differences for phase 2
	unsigned int primesLength; //length of the array with primes
	unsigned int differencesLength; //length of the array with differences
	int NumCurves;                     /* Number of curves for one number */

	/* Testing */
	bool test;
	int size;  //size of numbers for testing - only in test mode
	int generateN(); //generate random numbers for factorization if testing 

	//****** FUNCTIONS *******//
	int generateParameters();
	int computeNdashed();
	int naivni(gmp_randstate_t * state, int poradi);
	void getPrimes();
	int convertToString(const char *filename, std::string& s);
	int mpzToMyMpz(mpz_t input, my_mpz * output);
	int toMontgomery(mpz_t input, mpz_t * output);
	int fromMontgomery(mpz_t input, mpz_t * output);
	int getResults(mpz_t aFactor);
};

#endif
