/* 
Kernel will try to factorize number N using given Edwards curve and a point on it
Edwards curve is given by parameter d, a = 1, point by x and z
Extended points are used, (X:Y:Z:T) is obtained from affine point (x,y) as follows (x:y:1:x*y)
Montgomery representation for big numbers is useds
*/


#define DELKA 8
#define HALF 4 //maximal length of N
#define MAX 4294967295U
#define BASE 4294967296UL
#define BITSPERDIGIT 32

//typedef uint8 mpz;

//TODO  sjednotit pojmenovn (intmul,  Monmul)
//TODO  podn vyeit globln a lokln pam, ne si vechno peukldat, jak se to hod...

typedef struct{
	unsigned int coeff[DELKA];
	//char numCoeff;
} mpz;

/*!
ell_point represents point on Edwards curve in extended coordinates
*/
typedef struct{
	mpz x;
	mpz y;
	mpz z;
	mpz t;
} extended_EllPoint;

typedef struct{
	mpz x;
	mpz y;
	mpz z;
} projective_EllPoint;

/* print function for debugging 
void mpz_print(mpz * number)
 {
	 for (int i = 0; i < DELKA; i++)
		 printf(" %u ", number->coeff[i]);
	 printf("\n");
 }*/

/* print function for debugging 
void point_print(extended_EllPoint *aP)
 {
	 printf("x-ova souradnice: ");
	 mpz_print(&aP->x);
	 printf("y-ova souradnice: ");
	 mpz_print(&aP->y);
	 printf("z-ova souradnice: ");
	 mpz_print(&aP->z);
	 printf("t-ova souradnice: ");
	 mpz_print(&aP->t);
	 printf("\n");
 }*/

 /*
void mpz_init(mpz * number)
{
	//for (int i = 0; i < DELKA; 
}*/

/*! 
compare two mpz numbers, return 1 if op1 > op2, return 0 if op1 = op2, return -1 if op1 < op2 
op2 is in global memory
*/
int compareGlobal(mpz * op1, __global mpz * op2)
{
	for(int i = DELKA - 1; i >= 0; i--)
	{
		if( (*op1).coeff[i] > (*op2).coeff[i])
			return 1;
		if ( (*op1).coeff[i] < (*op2).coeff[i])
			return -1;
	}
	return 0;
}

/*! 
compare two mpz numbers, return 1 if op1 > op2, return 0 if op1 = op2, return -1 if op1 < op2 
both op1 and op2 are in local memory
*/
int compare(mpz * op1, mpz * op2)
{
	for(int i = DELKA - 1; i >= 0; i--)
	{
		if( op1->coeff[i] > op2->coeff[i])
			return 1;
		if ( op1->coeff[i] < op2->coeff[i])
			return -1;
	}
	return 0;
}

/*! set rop = op1 + op2 */
/* should be right*/
void mpz_add (mpz * rop, mpz * op1, mpz * op2) 
{
	//printf("    Scitam velka cisla \n");
	bool carry = false;
	unsigned int * actual = &rop->coeff[0];
	unsigned int * actualop1 = &op1->coeff[0];
	unsigned int * actualop2 = &op2->coeff[0];
	ulong var; //auxiliary variable
	for (int i = 0; i < DELKA; i++)
	{

		var  = (ulong)*actualop1 + (ulong)*actualop2 + (ulong)carry;
		*actual = (unsigned int)var;
		if((ulong)*actual != var)
			carry = true;
		else
			carry = false;
		actual++; actualop1++; actualop2++;
	}
	//if (carry)
	//printf("Preteklo mi scitani \n");
}

/*! set rop = op1 - op2 mod N*/
/* should be right */
void mpz_sub (mpz * rop, mpz op1, mpz * op2, __global mpz* aN) 
{
	bool carry = false;
	//int carry = 0;
	unsigned int * actual = &rop->coeff[0];
	unsigned int * actualop1 = &op1.coeff[0];
	unsigned int * actualop2 = &op2->coeff[0];
	unsigned long var;
	//if op1 < op2, add N to op1, so we don't get into negative numbers
	if(compare(&op1, op2) < 0)
	{
		//add N, so you will not get negative number
		mpz N = *aN;
		mpz_add(&op1, &op1, &N);
	}
	//carry = false;
	for (int i = 0; i<DELKA; i++)
	{
		var = (unsigned long)*actualop1 - (unsigned long)*actualop2 - (unsigned long)carry;
		*actual = var;
		if((unsigned long)*actual != var)
			carry = true;
		else
			carry = false;
		actual++; actualop1++; actualop2++;
	}
}

/*! set rop = op1*op2*/
void mpz_mul (mpz * rop, mpz *op1, mpz *op2, __global mpz* aN)  //TODO aN se ve skutenosti nepouije...
{
	//printf("    Nasobim velka cisla \n");
	ulong carry = 0;
	uint subresult[DELKA];
	unsigned int * actualop1 = &op1->coeff[0];
	unsigned int * actualop2 = &op2->coeff[0];
	for (int i = 0; i < DELKA; i++) //memset !!
		subresult[i]=0;
	for(char i = 0; i < HALF; ++i, actualop2++) //go throught coefficients of the second number
	{
		for(char j = 0; j < HALF; ++j, actualop1++) //go throught coefficients of the first number
		{
			carry = carry + ((ulong)(*actualop2) * (ulong)(*actualop1)) + (ulong)subresult[i+j]; 
			//actualop1++;
			subresult[i+j] = (uint)carry; //carry % BASE
			carry = carry >> BITSPERDIGIT;
		}
		actualop1 = &op1->coeff[0];
		//actualop2++;
		//subresult[i + HALF] = subresult[i + HALF] + carry; 
		subresult[i + HALF] = (uint) (carry);// >> BITSPERDIGIT); 
		carry = 0;
	}
	for (int i = 0; i < DELKA; i++) 
	{
		rop->coeff[i] = (uint)subresult[i];
	}
} 

/* Montgomery multiplication set rop = op1 * op2 in Montgomery form*/
void mpz_Monmul (mpz * rop, mpz *op1, mpz *op2, __global mpz* aN, __global unsigned int * r, mpz * N_dashed)
{
	mpz t, x; // auxiliary variables
	//printf("Vstup pro nsoben: \n");
	//mpz_print(op1);
	//mpz_print(op2);
	mpz_mul(&t, op1, op2, aN); //t = op1 * op2
	//mpz_print(&t);
	mpz_mul(&x, &t, N_dashed, aN); // x = t * N'
	//mpz_print(&x);
	//TODO kolik toho potebuju doopravdy vynsobit, kdy to pak budu modulit? 
	for (int i = *r; i < DELKA; i++) //TODO: tady bylo i<8 i pi DELKA =10 a fungovalo to...
		x.coeff[i] = 0; //x mod 2^(32*r)
	//mpz_print(&x);
	mpz N = *aN;
	mpz_mul(&x, &x, &N, aN); //x = x * N //TODO tohle nsoben u mus bt pesn
	//mpz_print(&x);
	mpz_add(&x, &x, &t); // x = x + t
	//mpz_print(&x);

	//x div 2^(32*r)
	for (int i = *r; i < DELKA; i++) //TODO: tady bylo i<8 i pi DELKA =10 a fungovalo to...
		x.coeff[i - *r] = x.coeff[i]; 
	for (int i = 8 - (*r); i < DELKA; i++) //TODO: tady bylo i<8 i pi DELKA =10 a fungovalo to...
		x.coeff[i] = 0;
	//mpz_print(&x);

	if( compareGlobal(&x, aN) == 1)
		mpz_sub(rop, x, &N, aN);
	else *rop = x;
	//printf("vysledek: \n");
	//mpz_print(rop);
}

/* set rop = op1 * op2, where op1 is mpz and op2 is unsigned integer 
void mpz_intmul (mpz * rop, mpz op1, unsigned int op2)
{
	ulong carry = 0;
	ulong base = MAX +1;
	for (int i = 0; i < DELKA; i++) //memset !!
		rop->coeff[i]=0;
	for(int i = 0; i < DELKA-1; i++) //go throught coefficients of the number
	{
			carry = carry + ((ulong)op1.coeff[i] * (ulong)op2); 
			rop->coeff[i] = (carry % base);
			carry = carry/base;
	}
	if (carry > 0)
	{
		rop->coeff[DELKA-1] = carry; 
	}
}*/

/*set rop to op1 * B^op2 
void mpz_mulbase (mpz * rop, mpz * op1, unsigned int op2)
{
	//ulong carry = 0;
	for (unsigned int i = DELKA - 1; i >= op2; i--) 
		rop->coeff[i]= op1->coeff[i - op2];
	for (int i = 0; i < op2; i++) 
		rop->coeff[i]=0;
} */


/* just substract N if op1 > N */
void mpz_onemod(mpz * rop, mpz op1, __global mpz* aN)
{
	if( compareGlobal(&op1, aN) == 1)
	{
		mpz N = *aN;
		mpz_sub(rop, op1, &N, aN);
	}
	else *rop = op1;
}

/* Great Common Divisor of u and *N, u in local memory, *N in global memory 
void mpz_gcd(mpz * vysl, mpz u, __global mpz * aN) 
{
    mpz t;
	mpz v = *aN;
	mpz nula; 
	for (int i = 0; i < DELKA; i++) 
		nula.coeff[i]=0;
    while ( compare(&v, &nula) != 0) {
    t = u; 
    u = v; 
    mpz_modgcd(&v, t, v, aN); //v = t % v;
  }
	*vysl = u;
}

bool isZero(extended_EllPoint *aP)
{
	for (int i = 0; i < DELKA; i++)
		if (aP->x.coeff[i] != 0) 
			return false;
	return true;
}*/

//conversion of projective point into extended
extended_EllPoint projectiveToExtended(projective_EllPoint *P, __global mpz * aN, __global mpz* aD, __global unsigned int * r, mpz * N_dashed)
{ 
	extended_EllPoint result;
	//Given (X: Y : Z) in projective passing to extended can be performed in 3M+1S by computing (XZ, YZ, XY, Z^2).
	//because the auxiliary coordinate T has the property T = XY/Z
	mpz_Monmul(&result.x, &P->x, &P->z, aN, r, N_dashed);
	mpz_Monmul(&result.y, &P->y, &P->z, aN, r, N_dashed);
	mpz_Monmul(&result.z, &P->z, &P->z, aN, r, N_dashed);
	mpz_Monmul(&result.t, &P->x, &P->y, aN, r, N_dashed);
	return result;
}

//take P in projective coordinates and return 2P in extended coordinates
//formulas are in fact independent of d
extended_EllPoint extendedDoubling(projective_EllPoint *P, __global mpz * aN, __global mpz* aD, __global unsigned int * r, mpz * N_dashed)
{
	extended_EllPoint result;
	/* Cost: 4M+ 4S + 1D
	Source: Twisted Edwards Curves Revisited, (7)
	These formulae are independent of the curve constant d. These are essentially the
	same formulae from [1] plus the formula for T3 which increases the number of multiplications ... by 1. 
	A = X^2 ; B = Y^2 ; C = 2*Z^2 ; D = aA; E = (X + Y)^2 - A - B; G = D + B; F = G - C; H =  D - B; 
	X3 = E*F; Y3 = G * H; T3 = E * H; Z3 = F * G: */
	mpz a, b, c, e, f, g, h ;
	mpz_Monmul(&a, &P->x, &P->x, aN, r, N_dashed); //A = X^2
	mpz_Monmul(&b, &P->y, &P->y, aN, r, N_dashed); //B = Y^2
	mpz_Monmul(&c, &P->z, &P->z, aN, r, N_dashed); //C = Z^2
	mpz_add(&c, &c, &c); //C = 2C = 2*z^2
	mpz_onemod(&c, c, aN); //C mod N
	//D = a*A, but a = 1
	mpz_add(&e, &P->x, &P->y); // E = X + Y
	mpz_onemod(&e, e, aN);
	mpz_Monmul(&e, &e, &e, aN, r, N_dashed); // E = (X + Y)^2
	mpz_sub(&e, e, &a, aN); // E = (X + Y)^2 - A
	mpz_sub(&e, e, &b, aN); // E = (X + Y)^2 - A - B
	mpz_add(&g, &a, &b); //G = D + B = A + B
	mpz_onemod(&g, g, aN); 
	mpz_sub(&f, g, &c, aN); //F = G - C
	mpz_sub(&h, a, &b, aN); //H = D - B = A - B
	mpz_Monmul(&result.x, &e, &f, aN, r, N_dashed); // X3 = E * F 
	mpz_Monmul(&result.y, &g, &h, aN, r, N_dashed); // Y3 = G * H
	mpz_Monmul(&result.t, &e, &h, aN, r, N_dashed); // T3 = E * H
	mpz_Monmul(&result.z, &f, &g, aN, r, N_dashed); // Z3 = F * G
	return result;
}

//take P in projective coordinates and return 2P in projective coordinates
//formulas are in fact independent of d
projective_EllPoint projectiveDoubling(projective_EllPoint *P, __global mpz * aN, __global mpz* aD, __global unsigned int * r, mpz * N_dashed)
{
	projective_EllPoint result;
	//because you can transform extended to projective point just by forgetting the T coordinate
	//and extendedDoubling does not take the T coordinate as an input
	//it should be OK to use extendedDoubling and just skip the computing of T3
	//Cost: 3M+ 4S + 1D
	mpz a, b, c, e, f, g, h ;
	mpz_Monmul(&a, &P->x, &P->x, aN, r, N_dashed); //A = X^2
	mpz_Monmul(&b, &P->y, &P->y, aN, r, N_dashed); //B = Y^2
	mpz_Monmul(&c, &P->z, &P->z, aN, r, N_dashed); //C = Z^2
	mpz_add(&c, &c, &c); //C = 2C = 2*z^2
	mpz_onemod(&c, c, aN); //C mod N
	//D = a*A, but a = 1
	mpz_add(&e, &P->x, &P->y); // E = X + Y
	mpz_onemod(&e, e, aN);
	mpz_Monmul(&e, &e, &e, aN, r, N_dashed); // E = (X + Y)^2
	mpz_sub(&e, e, &a, aN); // E = (X + Y)^2 - A
	mpz_sub(&e, e, &b, aN); // E = (X + Y)^2 - A - B
	mpz_add(&g, &a, &b); //G = D + B = A + B
	mpz_onemod(&g, g, aN); 
	mpz_sub(&f, g, &c, aN); //F = G - C
	mpz_sub(&h, a, &b, aN); //H = D - B = A - B
	mpz_Monmul(&result.x, &e, &f, aN, r, N_dashed); // X3 = E * F 
	mpz_Monmul(&result.y, &g, &h, aN, r, N_dashed); // Y3 = G * H
	mpz_Monmul(&result.z, &f, &g, aN, r, N_dashed); // Z3 = F * G
	return result;
}

//P2 = extendedAddition(aP1, P2, aDiff); P2 = P1 + P2
projective_EllPoint extendedAddition(extended_EllPoint *aP, extended_EllPoint *aQ, __global mpz * aN, __global mpz* aD, __global unsigned int * r, mpz * N_dashed)
{
		
	/*
	Cost: 9M + 7add. 
	Source: 2008 HisilWongCarterDawson, http://eprint.iacr.org/2008/522, Section 3.2.
	For two distinct points only
	  A = X1*X2      B = Y1*Y2      C = Z1*T2      D = T1*Z2
      E = D + C      F = (X1-Y1)*(X2+Y2)+B-A       G = B+A      H = D-C
      X3 = E*F      Y3 = G*H      T3 = E*H      Z3 = F*G
	  */

	//printf("x_p: "); mpz_print(&(aP->x));
	//printf("y_p: "); mpz_print(&(aP->y));
	//printf("x_q: "); mpz_print(&(aQ->x));
	//printf("y_q: "); mpz_print(&(aQ->y));
	//printf("    Scitam 2 body ");
	projective_EllPoint result;
	{
		mpz a, b, c, d, e, f, g, h ;
		mpz D = *aD;
		mpz_Monmul(&a, &aP->x, &aQ->x, aN, r, N_dashed); //A = X1*X2
		mpz_Monmul(&b, &aP->y, &aQ->y, aN, r, N_dashed); //B = Y1*Y2
		mpz_Monmul(&c, &aP->z, &aQ->t, aN, r, N_dashed); //C = Z1*T2
		mpz_Monmul(&d, &aP->t, &aQ->z, aN, r, N_dashed); //D = Z1*Z2
		mpz_add(&e, &d, &c); //E = D+C
		mpz_onemod(&e, e, aN); 
		mpz_add(&g, &b, &a); //G = B+A
		mpz_onemod(&g, g, aN); 
		mpz_sub(&h, d, &c, aN); //H = D-C
		//we don't need d and c anymore - use them as auxiliary variables
		mpz_add(&c, &aQ->x, &aQ->y); //C = X2 + Y2
		mpz_onemod(&c, c, aN);  // C = (X1+Y1) mod N
		mpz_sub(&d, aP->x, &aP->y, aN); //D = X1 - Y1
		mpz_Monmul(&f, &c, &d, aN, r, N_dashed); // F = (X1-Y1)*(X2+Y2)
		mpz_sub(&f, f, &a, aN); // F = (X1-Y1)*(X2+Y2) - A
		mpz_add(&f, &f, &b); // F = (X1+Y1)*(X2+Y2) - A + B
		mpz_onemod(&f, f, aN); // F = (X1+Y1)*(X2+Y2) - A + B mod N
		mpz_Monmul(&result.x, &e, &f, aN, r, N_dashed); // X3 = E*F 
		mpz_Monmul(&result.y, &g, &h, aN, r, N_dashed); // Y3 = G*H
		//mpz_Monmul(&result.t, &e, &h, aN, r, N_dashed); // T3 = E*H
		mpz_Monmul(&result.z, &f, &g, aN, r, N_dashed); // Z3 = F*G
	}
	return result;
}

projective_EllPoint doubleAndAdd(extended_EllPoint *Pext, projective_EllPoint *Pproj, __global mpz * aN, __global mpz* aD, __global unsigned int * r, mpz * N_dashed)
{
	extended_EllPoint helper;
	projective_EllPoint result;
	helper = extendedDoubling(Pproj, aN, aD, r, N_dashed);
	result = extendedAddition(Pext, &helper, aN, aD, r, N_dashed);
	return result;
}
/* TODO aD se vbec nepouije? */
//P2 = mulPoint(P1, Scalar,...); P2 = Scalar * P1
extended_EllPoint mulPoint (extended_EllPoint aP1, unsigned int aScalar, __global mpz* aN, __global mpz* aD, __global unsigned int * r, mpz * N_dashed, __global mpz * oneMon)
{
	//printf("    Nasobim 2 body \n");
	projective_EllPoint P2; /* P2 = 0 (neutral element) */
	for (int i = 0; i < DELKA; i++)
	{
		P2.x.coeff[i] = 0;
		P2.z.coeff[i] = 0;
	}
	P2.y = *oneMon;
	P2.z = P2.y;
	//P2 = extendedAddition(&P2, &P2, aDiff, aN, inv4, Nmultiples, multiplesLength);  //add_points(P2, P2, P2, NULL); P2 = 2A
	unsigned int mask = 2147483648; //2^(31)
	while(mask > aScalar)
		mask = mask >> 1;
	while (mask > 0)
	{
		if (aScalar & mask)
		{
			//P2 = extendedAddition(&P2, &P2, aN, aD, r, N_dashed); //P2 = P2 + P2
			//P2 = extendedAddition(&aP1, &P2, aN, aD, r, N_dashed); //P2 = P1 + P2
			P2 = doubleAndAdd(&aP1, &P2, aN, aD, r, N_dashed); //P2 = 2*P2 + P1
		}
		else
			//P2 = extendedAddition(&P2, &P2, aN, aD, r, N_dashed); //P2 = P2 + P2
			P2 = projectiveDoubling(&P2, aN, aD, r, N_dashed); //P2 = P2 + P2
		mask = mask >> 1;
	}
	extended_EllPoint result;
	result = projectiveToExtended(&P2, aN, aD, r, N_dashed);
	return result; 
}

__kernel void factorize(__global mpz* out, __global mpz* aN, __global unsigned int * aB1, __global mpz* aD, __global mpz* aX, __global mpz* aY, __global unsigned int * primes,
						__global int *Length,  __global mpz * oneMon, __global unsigned int * r, __global mpz * aN_dashed) 
{
	uint globalID = get_global_id(0);
	//printf("I am thread number %d, my A is: %u %u %u %u \n", globalID, aA[globalID].s0, aA[globalID].s1, aA[globalID].s2, aA[globalID].s3); 
	__private extended_EllPoint Q;
	mpz N_dashed = *aN_dashed;
	Q.x = aX[globalID];
	Q.y = aY[globalID];
	Q.z = *oneMon;
	mpz_Monmul(&Q.t, &Q.x, &Q.y, aN, r, &N_dashed);

	
	// real algorithm
	//uint k;
	mpz gcd;
	mpz N = *aN;

	//Phase 1
	for(uint i = 0; i < *Length; i++) //number of primes
	{
		Q = mulPoint(Q, primes[i], aN,  &aD[globalID], r, &N_dashed, oneMon);
		/*
		mpz_gcd(&gcd, Q.z, aN);
		if ( compare(&gcd, &jedna) > 0)
		{
			break;
		}*/
	}
	/*
	if (existsGCD <= 0) //Then Phase 2
	{
		extended_EllPoint P0, pQ;
		P0 = Q;
		pQ = Q;
		for(uint i = 0; i < *diffLength; i++)
		{ //P0 = diff[i]Q, Q' = Q' + P0
			P0 = mulPoint(Q, differences[i], &Q, aN, inv4); 
		}
	} */
	//point_print(&Q);
	//printf("I am thread number %d, my Q is: \n x = %u \n y = %u \n z= %u \n t= %u \n", globalID, Q.x.coeff[0], Q.y.coeff[0], Q.z.coeff[0], Q.t.coeff[0]); 
	out[globalID] = Q.x;
}