#include "CL/cl.h"
#include "gmp.h"
#include "ECM.h"
#include "ECM_parameters.h"
#include "ECM_const.h"
#include <time.h>       /* clock_t, clock, CLOCKS_PER_SEC */

#define SUCCESS 0
#define FAILURE 1

using namespace std;

/*!
Generates random parameters of curves.
*/
int ECM::naivni(gmp_randstate_t * state, int poradi)
{
	/*
	Choose x and y randomly, a = 1, and compute d that fits x^2 + y^2 = 1 + d x^2 y^2
	extended coordinates will be used: (x,y) -> (X, Y, Z, T), X=x, Y=y, Z=1, T= x*y
	*/
	mpz_t varx, vary;
	mpz_init(varx); mpz_init(vary);
	mpz_urandomm(varx, *state, actualN);
	mpz_urandomm(vary, *state, actualN);
	mpz_t var1, var2, vard;
	mpz_init(var1); mpz_init(var2); mpz_init(vard);
	mpz_mul(var1, varx, varx); //var1 = x^2
	mpz_mul(var2, vary, vary); //var2 = y^2
	mpz_add(vard, var1, var2); //vard = x^2 + y^2
	mpz_sub_ui(vard, vard, 1); //vard = x^2 + y^2 - 1
	mpz_mul(var1, var1, var2); //var1 = x^2 * y^2
	int inv;
	inv = mpz_invert(var1, var1, actualN); //var1 = (x^2 * y^2)^(-1) mod N
	mpz_mul(vard, var1, vard); //vard = vard * ((x^2 * y^2)^(-1) mod N)
	mpz_mod(vard, vard, actualN); //vard = vard mod N
	/*cout << "X = ";
	mpz_out_str(NULL, 10, varx);
	cout << "\nY = ";
	mpz_out_str(NULL, 10, vary);
	cout << "\nD = ";
	mpz_out_str(NULL, 10, vard);
	cout << "\n";*/
	toMontgomery(varx, &varx);
	toMontgomery(vary, &vary);
	mpzToMyMpz(varx, &x[poradi]);
	mpzToMyMpz(vary, &y[poradi]);
	toMontgomery(vard, &vard);
	mpzToMyMpz(vard, &d[poradi]);

	mpz_clear(varx);
	mpz_clear(vary);
	mpz_clear(var1); mpz_clear(var2); mpz_clear(vard);
	return SUCCESS;
}

/*!
Generates parameters for kernel - parameters of curves
*/
int ECM::generateParameters()
{
	gmp_randstate_t state; //TODO taky jako promnn ECM?
	gmp_randinit_default(state);
#ifdef RANDOM_C
	long seed = time(NULL);
	gmp_randseed_ui(state, seed);
#endif
	d = new my_mpz[NumCurves];
	x = new my_mpz[NumCurves];
	y = new my_mpz[NumCurves];

	for (int i = 0; i < NumCurves; i++)
	{
		naivni(&state, i);
	}

	computeNdashed();

	//now get number one in the Montgomery representation
	oneMon = new my_mpz;
	//mpz_t x, d, four;
	mpz_t one;
	mpz_init_set_ui(one, 1);
	mpz_mul_2exp(one, one, r * 32);
	mpz_mod(one, one, actualN);
	mpzToMyMpz(one, oneMon);
	/*cout << "One in Montgomery: ";
	mpz_out_str(NULL, 10, one);
	cout << "\n";*/
	mpz_clear(one);
	gmp_randclear(state);

	return SUCCESS;
}