#include <string>
#include <cstring>
#include <cstdlib>
#include <iostream>
#include "gmp.h"
#include "ECM_parameters.h"


using namespace std;

ECMparameters::ECMparameters()
{
	test = false;
	N = NULL;
	quantity = 0;
	B1 = 0;
	curves = 0;
	size = 0;
	//TODO: co mpz * ?
}

ECMparameters::~ECMparameters()
{
	//delete[] N;
}

/*!
Check if we have all parameters we need for running the algorithm.
If B1 or curves were not set, do it.
*/
int ECMparameters::check()
{
	if ((!test & quantity == 0) | (!test & N == NULL))
	{
		cout << "Error occured during checking parameters. Run with -h for help.";
		return -1;
	}
	if (B1 == 0)
	{
		B1 = 10000; //TODO: asi nastavovat a v ECM a hlavn njak chyteji? Nebo se bude eit extern?
	}
	if (curves == 0)
	{
		curves = 100;
	}
	return 0;
}

void ECMparameters::PrintHelp()
{
	cout << "Run without parameters to set them during the run. " << endl;
	cout << "-h                         for help " << endl;
	cout << "For factorizing 1 number set obligatory parameter: " << endl;
	cout << "-n NUMBER	                where NUMBER is number for factorizing " << endl;
	cout << "and optional parameters: " << endl;
	cout << "-c CURVES                  number of curves c" << endl;
	cout << "-b1 BOUND1                 bound for the first stage" << endl;
	cout << "For factorizing more numbers set obligatory parameter: " << endl;
	cout << "-muln COUNT NUMBERS        where counts is quantity followed by numbers separated by space" << endl;
	cout << "and optional parameters: " << endl;
	cout << "-c CURVES                  number of curves c" << endl;
	cout << "-b1 BOUND1                 bound for the first stage" << endl;
}

/*!
Parsing of input parameters.
*/
int ECMparameters::ParseInputParameters(int argc, char *argv[])
{
	// first parameter is path of program
	// TODO probably an error can ocure when bad parameters are given
	if (!strcmp(argv[1], "-h"))
	{
		PrintHelp();
		return -1; //the program should not continue
	}
	int i = 1;
	int error = 0;
	while (i < argc - 1 )
	{
		string str(argv[i]);
		if (str.length() == 0 || str[0] != '-')
		{
			error = -1;
		}
		if (str == "-n")
		{
			quantity = 1; //we will have just one number for factorization
			N = new mpz_t[1]; //TODO to je divn...podvat se na to.
			mpz_init_set_str(*N, argv[i+1], 10);
			i += 2;
			continue;
		}
		if (str == "-c")
		{
			curves = atoi(argv[i + 1]);
			i += 2;
			continue;
		}
		if (str == "-b1")
		{
			B1 = atoi(argv[i + 1]);
			i += 2;
			continue;
		}
		if (str == "-t") //for testing set -b1 -c (optional) and -s size 
		{
			test = true;
			quantity = atoi(argv[i + 1]);
			i += 2;
			continue;
		}
		if (str == "-s")
		{
			size = atoi(argv[i + 1]);
			i += 2;
			continue;
		}
		if (str == "-muln")
		{
			quantity = atoi(argv[i + 1]);
			N = new mpz_t[quantity]; //TODO ohldat?
			if (i + quantity + 2 > argc)
			{
				error = -1;
			}
			for (int j = 0; j < quantity; j++)
			{
				error = mpz_init_set_str(N[j], argv[i + 2 + j], 10);
				if (error) break;
			}
			i += quantity + 2;
		}
		if (error)
		{
			cout << "Error occured during parsing parameters. Run with -h for help.";
			exit(1);
		}
	}
	return 0;
}
