﻿//#include "stdafx.h"
#include "ECMproc.h"
#include "gmp.h"
//#include <math.h>       /* sqrt */
#include <iostream>
#include <cstring>
#include <cmath>

ell_point::ell_point()
{
	mpz_init(x);
	mpz_init(y);
	mpz_init(z);
}

//creates point (0:1:0)
void ell_point::init(ell_curve_m * aCurve) 
{
	mpz_set_ui(x, 1);
	mpz_set_ui(y, 0);
	mpz_set_ui(z, 0);
	curve = aCurve;
} 

//creates point (x:y:z)
void ell_point::init(ell_curve_m * aCurve, mpz_t aX, mpz_t aY, mpz_t aZ)
{
	mpz_set(x, aX);
	mpz_set(y, aY);
	mpz_set(z, aZ);
	curve = aCurve;
}

//creates point (x::z)
void ell_point::init(ell_curve_m * aCurve, mpz_t aX, mpz_t aZ)
{
	mpz_set(x, aX);
	mpz_set(z, aZ);
	curve = aCurve;
}

//set parameters of the point the same as parametres of the point aPoint
void ell_point::init(ell_point * aPoint)
{
	mpz_set(x, aPoint->x);
	mpz_set(y, aPoint->y);
	mpz_set(z, aPoint->z);
	curve = aPoint->curve;
}


ell_point::~ell_point()
{
	//delete curve; ??
	mpz_clear(x);
	mpz_clear(y);
	mpz_clear(z);
}

ell_curve_m::ell_curve_m()
{
	mpz_init(a);
	mpz_init(b);
}

ell_curve_m::~ell_curve_m()
{
	mpz_clear(a);
	mpz_clear(b);
	N = NULL;
}

int ell_curve_m::init(mpz_t * aN)
{
	N = aN;
	P = new ell_point;
	P->init(this);
	gmp_randstate_t state;
	gmp_randinit_default(state);
	mpz_urandomm(a , state, *N);
	//random a and x with z=1 suffice, there allways exist b and y that the equality of the curve holds
	return 0;
}

void ell_curve_m::Suyama(mpz_t * aN, gmp_randstate_t aState)
{
	N = aN;
	//random a random state musí být asi mimo
	mpz_t random;
	mpz_init_set_ui(random, 1);
	//gmp_randstate_t state;
	//gmp_randinit_default(state);
	//choose a random integer > 5
	do
	mpz_urandomb(random, aState, 40); // !!! ?
	while(mpz_cmp_ui(random,5) <= 0);
	//then compute: u = random^2 - 5, v = 4*random, x_0 = u^3, z_0 = v^3
	mpz_t u, v;
	mpz_init(u);
	mpz_init(v);
	mpz_powm_ui(u, random, 2, *aN);
	mpz_sub_ui(u, u, 5);
	mpz_mul_ui(v, random, 4);
	mpz_clear(random); //I don't need random anymore...maybe I should remember it?
	mpz_powm_ui(u, u, 3, *aN); // x_0 = u^3
	mpz_powm_ui(v, v, 3, *aN); // z_0 = v^3
	P = new ell_point;
	P->init(this, u, v);
	std::cout << "Random parameter : X ";
	mpz_out_str(NULL, 10, u);
	std::cout << "  Z: ";
	mpz_out_str(NULL, 10, v);
	std::cout << "\n";
	// a = (v-u)^3 (3u+v)/(4u^3 v) - 2
	mpz_t temp1, temp2, temp3;
	mpz_init(temp1);
	mpz_init(temp2);
	mpz_init(temp3);
	mpz_sub(temp1, u, v);
	mpz_powm_ui(temp1, temp1, 3, *aN); //(v-u)^3 mod N
	mpz_mul_ui(temp2, u, 3);
	mpz_add(temp2, temp2, v); //3u+v
	mpz_powm_ui(temp3, u, 3, *aN);
	mpz_mul(temp3, temp3, v);
	mpz_mul_ui(temp3, temp3, 4); // 4 u^3 v
	mpz_mod(temp3, temp3, *aN);
	if(mpz_cmp_ui(temp3, 0) == 0)
	{
		//??
	}
	mpz_t gcd;
	mpz_init(gcd);
	mpz_gcdext(gcd, temp3, NULL, temp3, *aN); // (4 u^3 v)^(-1)
	if(mpz_cmp_ui(gcd,1) != 0)
	{
		//we found a divisor of N
	}
	mpz_clear(gcd);
	mpz_mul(temp3, temp3, temp2);
	mpz_mul(temp3, temp3, temp1);
	mpz_sub_ui(temp3, temp3, 2);
	mpz_mod(temp3, temp3, *aN);
	mpz_set(this->a, temp3);
	mpz_clear(temp1);
	mpz_clear(temp2);
	mpz_clear(temp3);
	mpz_clear(u);
	mpz_clear(v);
}


ECM::ECM(mpz_t * aModulus, int aCurves, int aB1)
{
        //choose B_1, B_2, the number of curves numCurves
        B_1 = aB1;
        B_2 = 5000;
		N = aModulus;
		//find primes and generate random curves
		get_primes();
		numCurves = aCurves;
		mpz_t one;
		mpz_init_set_ui(one,1);
		gmp_randstate_t state;
	    gmp_randinit_default(state);
		//generate random curves with random points
		curves = new ell_curve_m[numCurves];
		for (int i = 0; i < numCurves; i++)
		{
			curves[i].Suyama(N, state);
		}		
}

ECM::~ECM()
{
	delete [] primes;
	delete [] curves;
}

//Returns true if the points are equal
bool ell_point::equals(ell_point * aPoint){
	if(mpz_cmp(x, aPoint->x)||mpz_cmp(y,aPoint->y)||mpz_cmp(z,aPoint->z) )
	{
		return false;
	}
	else{
		return true;
	}
}

//add two points
int ell_curve_m::add_points(ell_point *aResult, ell_point *aP, ell_point *aQ, ell_point *aDiff){
	
	if(aP->equals(aQ))
	{
		//duplication of point
		//needs inverse of 4 mod N
		//void mpz_gcdext (mpz_t g, mpz_t s, mpz_t t, const mpz_t a, const mpz_t b) gcd(a,b) a*s + b*t = g
		mpz_t d; //inverse of 4 mod N
		mpz_t u,v,t;
		mpz_t x,z;
		mpz_t four;
		mpz_init_set_ui(four, 4);
		mpz_init(x);
		mpz_init(z);
		mpz_init(u);
		mpz_init(v);
		mpz_init(t);
		mpz_init(d);
		mpz_gcdext(x, d, z, four, *N); //set d to inverse of 4 mod N, EXTENDED GCD
		mpz_add(u, aP->x, aP->z); // u = x_p + z_p
		mpz_mul(u, u, u); //u = u^2
		mpz_mod(u, u, *aP->curve->N); //u mod N
		mpz_sub(v, aP->x, aP->z);
		mpz_mul(v, v, v); //v = v^2
		mpz_mod(v, v, *aP->curve->N); //v mod N  !!!!!!!!!!!!!!!!
		mpz_sub(t, u, v); //t = u-v
		mpz_mul(t, t, d); //t = t*d 
		mpz_add(t, t, v); //t = t+v
		mpz_mul(x, u, v); //x = u*v
		mpz_mod(x, x, *aP->curve->N);
		mpz_sub(u, u, v); // u = u-v ...u is just an auxiliary variable now 
		mpz_mul(z, u, t); //z = (u-v)t
		mpz_mod(z, z, *aP->curve->N);
		mpz_set(aResult->x, x);
		mpz_set(aResult->z, z);
		mpz_clear(four);
		mpz_clear(x);
		mpz_clear(z);
		mpz_clear(u);
		mpz_clear(v);
		mpz_clear(t);
		mpz_clear(d);
		return 0;
	}
	else
	{
		/*add two distinct points, all mod n
		P = (x_p :: z_p)     Q = (x_q :: z_q)   aDiff = (x_(p-q) :: z_(p-q))
	    u = (x_p + z_p)*(x_q - z_q)  v = (x_p - z_p)*(x_q + z_q)
	    w = (u+v)^2                  t = (u-v)^2
	    x_(p+q) = z_(p-q) * w        z_(p+q) = x_(p-q) * t  */
		mpz_t u, v, w, t, x, z ;
		mpz_init(x);
		mpz_init(z);
		mpz_init(u);
		mpz_init(v);
		mpz_init(w);
		mpz_init(t);
		mpz_set_ui(w, 2);
		//mpz_add (mpz_t rop, const mpz_t op1, const mpz_t op2)   Set rop to op1 + op2.
		//mpz_sub (mpz_t rop, const mpz_t op1, const mpz_t op2)   Set rop to op1 − op2.
		//mpz_mul (mpz_t rop, const mpz_t op1, const mpz_t op2)   Set rop to op1 times op2.
		//mpz_mod (mpz_t r, const mpz_t n, const mpz_t d)         Set r to n mod d. The sign of the divisor is ignored; the result is always non-negative.
		//w,t as auxiliary variable
		mpz_add(w, (aP->x), (aP->z)); //(x_p + z_p)
		mpz_sub(t, aQ->x, aQ->z); //(x_q - z_q)
		mpz_mul(u, w, t); //u
		mpz_mod(u, u, *aP->curve->N); //u mod N     ?? proc *
 		mpz_add(w, aQ->x, aQ->z); //(x_q + z_q)
		mpz_sub(t, aP->x, aP->z); //(x_p - z_p)
		mpz_mul(v, w, t); //v
		mpz_mod(v, v, *aP->curve->N); //v mod N 
		mpz_add(w,u,v); //w = u+v
		mpz_sub(t,u,v); //t = u-v
		mpz_powm_ui(w, w, 2, *aP->curve->N); //w = w^2 mod N
		mpz_powm_ui(t, t, 2, *aP->curve->N); //t = t^2 mod N 
		//x_(p+q) = z_(p-q) * w        z_(p+q) = x_(p-q) * t  
		mpz_mul(x, aDiff->z, w);
		mpz_mul(z, aDiff->x, t);
		mpz_mod(x, x, *aP->curve->N);
		mpz_mod(z, z, *aP->curve->N);
		mpz_set(aResult->x, x);
		mpz_set(aResult->z, z);
		mpz_clear(x);
		mpz_clear(z);
		mpz_clear(u);
		mpz_clear(v);
		mpz_clear(w);
		mpz_clear(t);
	}

		return 0;
	}

//multiple point by a scalar
int ell_curve_m::mul_point(ell_point *aResult, ell_point *aA, unsigned int aScalar)
	{
		//Scalar multiplication using Montgomery ladder
		ell_point *P1, *P2;
		P1 = new ell_point;
		P2 = new ell_point;
		P1->init(aA); //P1 = A
		P2->init(aA->curve);
		ell_curve_m curve;
		curve.init(&(*aA->curve->N));
		//curve.add_points( P2, P2, P2, NULL); //P2 = 2*A
		//aScalar = aScalar / 2; //shift posun????
		while (aScalar != 0)
		{
			if (aScalar % 2 != 0) //if nth bit == 0
			{
				curve.add_points(P2, P1, P2, aA); //P2 = P1 + P2
				curve.add_points(P1, P1, P1, NULL); //P1 = 2*P1
			}
			else
			{
				curve.add_points(P1, P1, P2, aA); //P1 = P1 + P2
				curve.add_points(P2, P2, P2, NULL); //P2 = 2*P2
			}
			//aScalar /= 2; 
			aScalar = aScalar >> 1;
		}
		aResult->init(P1); 
		//aResult = P1; //?? jak je to s ukazatelama? Delete P1?
		delete P2,P1;
		return 0;
	}

void ECM::get_primes()
{
	//find all primes less than B_1
	//sieve of Erastothenes
	bool * isComposite = new bool[B_1 + 1];  
    memset(isComposite, 0, sizeof(bool) * (B_1 + 1)); //get false into the array
	length = B_1 - 1 ;
    for (int m = 2; m <= sqrt(double(B_1)) ; m++) 
	{
            if (!isComposite[m]) //then it is prime number
			{
				//do something, because it is prime number?
                for (int k = m + m; k <= B_1; k += m)
				{
					if (!(isComposite[k]))
						length--;
					//set all his multiples as composite numbers
                    isComposite[k] = true;			 
				}				
            }
    }
	primes = new unsigned int [length]; 
	unsigned int i = 0;
    for (int m = 2; m < B_1; m++)
		//copy only prime numbers into array primes
        if (!isComposite[m])
			{
				primes[i] = m;
                i++;
			}   
    delete [] isComposite;
}

int ECM::stage1(ell_point * aP, ell_point * aResult, ell_curve_m * aCurve)
{
        //stage 1 fails if gcd(n,z_q) = 1
	    ell_point Q;
        Q.init(aP);
		unsigned int k;		
        for(unsigned int i = 0; i < length; i++)
        {
			//find k, such that p^k <= B_1 < p^(k+1)
			k = log(double(B_1))/log((double(primes[i]))) ;
			for (unsigned int j = 1; j <= k; j++) 
			{
				aCurve->mul_point(&Q, &Q, primes[i]);
			}
        }
		aResult->init(&Q); //??
	    return 0;
}

int ECM::factor(unsigned int * aResults)
{
	//uloží seznam dělitelů v aResults, co by tak ideálně měla vracet?
	ell_point * results = new ell_point[numCurves];
	mpz_t gcd;
	mpz_init(gcd);
	for (int k = 0; k < numCurves; k++)
	{
		results[k].init(&curves[k]);
		stage1(curves[k].P, &results[k], &curves[k]);
		//compute gcd, if gcd != 1, stage 1 was succesfull
		std::cout << "Vysledek " << k << ": ";
		mpz_out_str(NULL, 10, results[k].z);
		std::cout << "\n";
		mpz_gcd(gcd, *N, results[k].z);
		aResults[k] = mpz_get_ui (gcd);
	}
	delete [] results; 
	mpz_clear(gcd);
	return 0;
}
