//Classes implementing ECM for processors
//Adla Zajkov
//9.10.2013
#ifndef ECMproc_H_
#define ECMproc_H_

#include "gmp.h"

class ell_curve_m;

//point of an elliptic curve
class ell_point{

public:
	//constructor
	ell_point();
	//destructor
	~ell_point();

	//VARIABLES
	//homogenous coordinates !vn public?
	mpz_t x;
	mpz_t y; //bude poteba?
	mpz_t z;
	//elliptic curve to which the point belongs
	ell_curve_m * curve; //co ostatn formy??

	//FUNCTIONS
	//init by mlo bt sp set... !
	void init(ell_curve_m * aCurve); //initialize point (0,1,0) 
	void init(ell_curve_m * aCurve, mpz_t aX, mpz_t aY, mpz_t aZ);
	void init(ell_curve_m * aCurve, mpz_t aX, mpz_t aZ);
	void init(ell_point * aPoint);
	//equality of points
	bool equals(ell_point * aPoint);
};

//elliptic curve in Montgomery projective form: byz^2 = x^3+azx^2+xz^2
class ell_curve_m{
public:
	//constructor
	//ell_curve_m(mpz_t * aN);
	ell_curve_m();
	//destructor
	~ell_curve_m();

	//VARIABLES
	ell_point * P; //random point for computing
	//coefficients
	mpz_t a;
	mpz_t b;
	//modulus = number we want to factorize = N
	mpz_t * N;

	//FUNCTIONS
	//initialization, creates random curve with random point P_0, computing mod aN
	int init(mpz_t * aN);
	//sum of points A+B
	int add_points(ell_point *aResult, ell_point *aP, ell_point *aQ, ell_point *aDiff);
	//scalar multiple of point
	int mul_point(ell_point *aResult, ell_point *aA, unsigned int aScalar);
	//Suyama's parametrization
	void Suyama(mpz_t * aN, gmp_randstate_t aState);
};

//class implementing the algorithm ECM
class ECM{
public:
	//consructor
	ECM(mpz_t * aModulus, int aCurves, int aB1);
	//destrctor
	~ECM();
	//main function
	int factor(unsigned int * aResults);
private:
	mpz_t * N;
	//stage 1 gets the point (??) and the curve it will use
	int stage1(ell_point * aP, ell_point * aResult, ell_curve_m * aCurve);
	int numCurves;
	unsigned int B_1; 
	unsigned int B_2; //nebo int b_1/2?
	ell_curve_m * curves; //all curves that will be used for factorization
    void get_primes(); //find primes <= B_1
    unsigned int * primes; //array of small primes
	int length; //length of the array with primes
};
#endif