/*
 * abstract_integer_matrix.h
 *
 *  Created on: 7.11.2013
 *      Author: jero
 */

#ifndef ABSTRACT_INTEGER_MATRIX_H_
#define ABSTRACT_INTEGER_MATRIX_H_

#include "abstract_matrix.h"

typedef long integer_matrix_type;
/*!
		\brief Common ancestor for integer matrices (both with fixed and arbitrary precision)

	This class introduces integer matrices and operations, mostly abstract. It can host values of binary matrices and it is therefore descended from AbstractMatrix. It has few member variables on its own, because most inner variables of its descendants depend on the fixed/arbitrary precision setting.
*/


class AbstractIntegerMatrix: public AbstractMatrix {
public:
	AbstractIntegerMatrix();
	virtual ~AbstractIntegerMatrix();

	virtual AbstractIntegerMatrix * clone() = 0;
	virtual int Copy(AbstractIntegerMatrix * aSource); ///< Copy matrix
	virtual int PartialCopy(AbstractIntegerMatrix * aSource, long aMaxRow, long aMaxColumn); ///< Copy matrix up to given row and column

	virtual int SetModulo(int aValue) = 0; ///< Sets internal modulo arithmetic
	virtual int SetModulo(mpz_t aValue) = 0; ///< Sets internal modulo arithmetic
	virtual int GetModulo() = 0; ///< Returns internat modulo arithmetic
	virtual int ApplyModulo() = 0; ///< Applies current modulo to the matrix.

	virtual bool IsRowZero(long aRow, long aLimit); ///< Check give row for existence of nonzero value
	virtual bool IsInvertible(long aRow, long aColumn) = 0;

	virtual int PutValue(long aRow, long aColumn, integer_matrix_type aValue) = 0; ///< Put integer value in the matrix
	virtual int GetValue(long aRow, long aColumn, integer_matrix_type & aValue) = 0; ///< Read integer value from the matrix
	virtual long IterateRow(long aRow,long & aRowIndex, long aColumnIndex, long & aColumn, integer_matrix_type & aValue); ///< Iterate through nonzero items on row

	virtual void PrintToScreen(); ///< Print matrix to the standard output

	virtual int Gauss(long aLimit, long aRowIndices[], long & aRank, bool aPartial); ///< Gaussian or Gauss-Jordan elimination for first aLimit columns
	long ChoosePivotRow(long aRow, long aColumn); ///< Choose row with nonzero value in given column
	virtual int GaussColumn(long aRow, long aColumn, bool aPartial) = 0; ///< clear column given pivot row and column

	int ReorderRows(long aLimit, long aRowIndices[]); ///< ReorderRows given desired permutation
	int GetRegularSubmatrixModQ(long aLimit, long aModulo, long & aRank); ///< Find regular submatrix modulo given number

	virtual int MultiplyRow(long aRow, integer_matrix_type aMul) =0 ; ///< Multiply row by constant
	virtual int MultiplyRow(long aRow, mpz_t aMul) =0; ///< Multiply row by constant

	virtual int SubtractRowMultiple(long aTargetRow, long aSourceRow, integer_matrix_type aMul) = 0;
	virtual int SubtractRowMultiple(long aTargetRow, long aSourceRow, mpz_t aMul) = 0;

	int RowProduct(long & aResult, long aRow1, long aRow2);
	int LLL();

protected:
	mpz_t tempmul; ///< auxiliary arbitrary precision integer for multiplication
	mpz_t tempadd; ///< auxiliary arbitrary precision integer for addition
	mpz_t tempval;
	mpz_t tempval2;
};

#endif /* ABSTRACT_INTEGER_MATRIX_H_ */
