#include "abstract_matrix.h"
#include "common_definitions.h"
#include <stdio.h>
#include <iostream>
#include "const_return_codes.h"
#include "log.h"

AbstractMatrix::AbstractMatrix() : MSerializable(FILE_NAME_MATRIX)
{
	this->maximal_row_index = -1;
	this->maximal_column_index = -1;
	this->species = SAbstractMatrix;
	this->info_mode = 0;
	this->assertion_mode = 0;
}

AbstractMatrix::~AbstractMatrix(){	
}

// GETTERS AND SETTERS
species_of_matrices AbstractMatrix::GetSpecies() const {
	return this->species;
}

long AbstractMatrix::GetMaximalRowIndex() const {
	return this->maximal_row_index;
}

long AbstractMatrix::GetMaximalColumnIndex() const {
	return this->maximal_column_index;
}

void AbstractMatrix::SetMaximalRowIndex(long aArg){
	this->maximal_row_index = aArg;
}

void AbstractMatrix::SetMaximalColumnIndex(long aArg){
	this->maximal_column_index = aArg;
}

void AbstractMatrix::SetInfoMode(int aMode){
	this->info_mode = aMode;
}

void AbstractMatrix::SetAssertionMode(int aMode){
	this->assertion_mode = aMode;
}

int  AbstractMatrix::GetInfoMode() const {
	return info_mode;
}

int  AbstractMatrix::GetAssertionMode() const {
	return assertion_mode;
}

int AbstractMatrix::IsZero()
{
  int i,j;
  for(i=0;i<=maximal_row_index;i++)
  for(j=0;j<=maximal_column_index;j++)
    if(!IsZero(i,j)) return FALSE;
  return TRUE;
}

int  AbstractMatrix::PerformColumnMask(AbstractMatrix* aTarget, AbstractMatrix* aOperand, matrix_type aMask){
	return ConstRC::NotSupported;
}

int  AbstractMatrix::Copy(AbstractMatrix* aSource){
#ifdef MATRIX_OPERATIONS_TIME_MESSAGE
        zero_start = clock();
#endif
        if (this->maximal_row_index != aSource->maximal_row_index)              return SIZE_MISMATCH;
        if (this->maximal_column_index != aSource->maximal_column_index)        return SIZE_MISMATCH;
        int ret;
        ret = this->Allocate();
        if (ret != ConstRC::Ok){
                return ret;
        }
	this->Zeroize();
        long i,j;
        for (i=0; i<= this->GetMaximalRowIndex(); i++)
        for (j=0; j<= this->GetMaximalColumnIndex(); j++){
                if(aSource->IsOne(i,j)) this->PutOne(i,j);
        }
#ifdef MATRIX_OPERATIONS_TIME_MESSAGE
        diff = clock() - zero_start;
        if (diff >=0){ // in this way we avoid problems with integer overflow, causing addition of negative values
                total_copy += (double) diff;
        }
#endif
        return ConstRC::Ok;
}

int  AbstractMatrix::Compare(AbstractMatrix* aOperand)
{
        if (!aOperand)                                                   return NULL_POINTER_SUPPLIED;
        if (aOperand->GetMaximalRowIndex() != this->maximal_column_index)   return SIZE_MISMATCH;
        if (aOperand->GetMaximalColumnIndex() != this->maximal_row_index)   return SIZE_MISMATCH;
	int i, max = aOperand->GetMaximalRowIndex();
	int j, max2= aOperand->GetMaximalColumnIndex();
	for (i = 0; i<= max; i++)
	{
		for (j=0; j<= max2; j++)
		{
			if (aOperand->IsOne(i,j) && (this->IsZero(i,j)))
			{
				printf("Abstract Matrix, Compare. There is a mismatch between A and B at indices [%d,%d] (0|1)\n",i,j);
				return NOT_EQUAL;
			}
		}
	}
	return EQUAL;
}

int  AbstractMatrix::Transpose(AbstractMatrix* aTarget){
#ifdef MATRIX_OPERATIONS_TIME_MESSAGE
        start = clock();
#endif
        if (!aTarget)                                                   return NULL_POINTER_SUPPLIED;
        if (aTarget->GetMaximalRowIndex() != this->maximal_column_index)   return SIZE_MISMATCH;
        if (aTarget->GetMaximalColumnIndex() != this->maximal_row_index)   return SIZE_MISMATCH;
        int ret = aTarget->Allocate();
        if (ret != ConstRC::Ok)          return ret;
	aTarget->Zeroize();
        long i, j;
        for (i=0; i<=this->maximal_column_index; i++)
        {
                for (j=0; j<=this->maximal_row_index; j++){
                        if (this->IsOne(j,i)){
                                aTarget->PutOne(i,j);
                        }
                }
        }
#ifdef MATRIX_OPERATIONS_TIME_MESSAGE
        diff = clock() - start;
        if (diff >=0){ // in this way we avoid problems with integer overflow, causing addition of negative values
                total_transpose += (double) diff;
        }
#endif
        return ConstRC::Ok;
}

int AbstractMatrix::Add(AbstractMatrix* aTarget, AbstractMatrix* aOperand2){
#ifdef MATRIX_OPERATIONS_TIME_MESSAGE
        start = clock();
#endif
        if (!aTarget)                                                           return NULL_POINTER_SUPPLIED;
        if (aTarget->maximal_row_index != this->maximal_row_index)              return SIZE_MISMATCH;
        if (aTarget->maximal_column_index != this->maximal_column_index)        return SIZE_MISMATCH;
        if (!aOperand2)                                                         return NULL_POINTER_SUPPLIED;
        if (aOperand2->maximal_row_index != this->maximal_row_index)            return SIZE_MISMATCH;
        if (aOperand2->maximal_column_index != this->maximal_column_index)      return SIZE_MISMATCH;
        int ret;
        long i;
        ret = aTarget->Allocate();
        if (ret != ConstRC::Ok)          return ret;
        ret = aOperand2->Allocate();
        if (ret != ConstRC::Ok)          return ret;
	if (aOperand2->species != this->species){
		for (i=0; i<=aTarget->maximal_row_index;i++)
		for (long j=0; j<=aTarget->maximal_column_index;j++)
			if (this->IsOne(i,j) != aOperand2->IsOne(i,j)) aTarget->PutOne(i,j);
			else aTarget->PutZero(i,j);
	} else {
		aTarget->Copy(this);
	        for (i=0; i<=aTarget->maximal_row_index;i++){
        	        aTarget->AddToRow(i, aOperand2,i);
                	}
	}
#ifdef MATRIX_OPERATIONS_TIME_MESSAGE
        diff = clock() - start;
        if (diff >=0){ // in this way we avoid problems with integer overflow, causing addition of negative values
                total_add += (double) diff;
        }
#endif
        return ConstRC::Ok;
}

int  AbstractMatrix::MultiplyInternal(AbstractMatrix* aTarget, AbstractMatrix* aOperand1, AbstractMatrix* aOperand2){
#ifdef MATRIX_OPERATIONS_TIME_MESSAGE
        start = clock();
#endif
        if (!aOperand1 || !aOperand2 || !aTarget)                               return NULL_POINTER_SUPPLIED;
        if (aTarget->maximal_row_index != aOperand1->maximal_row_index)         return SIZE_MISMATCH;
        if (aTarget->maximal_column_index != aOperand2->maximal_column_index)   return SIZE_MISMATCH;
        if (aOperand1->maximal_column_index != aOperand2->maximal_row_index)    return SIZE_MISMATCH;
        int ret;
        long i,j;
        ret = aTarget->Allocate();
        if (ret != ConstRC::Ok)          return ret;
        ret = aOperand1->Allocate();
        if (ret != ConstRC::Ok)          return ret;
        ret = aOperand2->Allocate();
        if (ret != ConstRC::Ok)          return ret;
	if (aOperand2->species != aTarget->species){	
	        for(i=0; i<=aOperand1->maximal_row_index; i++)
        	        for(j=0; j<=aOperand2->maximal_column_index; j++){
				bool b=FALSE;
				for(long k=0; k<=aOperand1->maximal_column_index; k++)
					if(aOperand1->IsOne(i,k)&&aOperand2->IsOne(k,j)) b = !b;
		//			printf("i-%d j-%d k-%d b-%d\n",i,j,k,b);}
				if (b) aTarget->PutOne(i,j); else aTarget->PutZero(i,j);
			}
	} else {
	        aTarget->Zeroize();
        	for(i=0; i<=aOperand1->maximal_row_index; i++)
                	for(j=0; j<=aOperand1->maximal_column_index; j++)
                        	if (aOperand1->IsOne(i,j))
                                	aTarget->AddToRow(i,aOperand2,j);
	}
#ifdef MATRIX_OPERATIONS_TIME_MESSAGE
        if (diff >=0){ // in this way we avoid problems with integer overflow, causing addition of negative values
                total_word_multiply += (double) diff;
                //printf("Pridavame do t.w.m cislo %d, coz je %6.2f sekund, vysledek je %12.4f, coz je %6.2f sekund.\n",diff,diff/(double) CLOCKS_PER_SEC,total_word_multiply,total_word_multiply/(double) CLOCKS_PER_SEC);
        }
#endif
        return ConstRC::Ok;
}

int  AbstractMatrix::MultiplyInternalTransposed(AbstractMatrix* aTarget, AbstractMatrix* aOperand1, AbstractMatrix* aOperand2){
#ifdef MATRIX_OPERATIONS_TIME_MESSAGE
        start = clock();
#endif
        if (!aOperand1 || !aOperand2 || !aTarget)                               return NULL_POINTER_SUPPLIED;
        if (aTarget->maximal_row_index != aOperand1->maximal_column_index)      return SIZE_MISMATCH;
        if (aTarget->maximal_column_index != aOperand2->maximal_column_index)   return SIZE_MISMATCH;
        if (aOperand1->maximal_row_index != aOperand2->maximal_row_index)       return SIZE_MISMATCH;
        int ret;
        long i,j,k;
	bool b;
        ret = aTarget->Allocate();
        if (ret != ConstRC::Ok)          return ret;
        ret = aOperand1->Allocate();
        if (ret != ConstRC::Ok)          return ret;
        ret = aOperand2->Allocate();
        if (ret != ConstRC::Ok)          return ret;
	if (aOperand2->species != aTarget->species){	
	        for(i=0; i<=aOperand1->maximal_column_index; i++)
        	        for(j=0; j<=aOperand2->maximal_column_index; j++){
				b=FALSE;
				for(k=0; k<=aOperand1->maximal_row_index; k++)
					if(aOperand1->IsOne(k,i)&&aOperand2->IsOne(k,j)) b = !b;
				if (b) aTarget->PutOne(i,j); else aTarget->PutZero(i,j);
			}
	} else {
	        aTarget->Zeroize();
        	for(i=0; i<=aOperand1->maximal_row_index; i++)
                	for(j=0; j<=aOperand1->maximal_column_index; j++)
                        	if (aOperand1->IsOne(i,j))
                                	aTarget->AddToRow(j,aOperand2,i);
	}
#ifdef MATRIX_OPERATIONS_TIME_MESSAGE
        if (diff >=0){ // in this way we avoid problems with integer overflow, causing addition of negative values
                total_word_transpose_multiply += (double) diff;
                //printf("Pridavame do t.w.m cislo %d, coz je %6.2f sekund, vysledek je %12.4f, coz je %6.2f sekund.\n",diff,diff/(double) CLOCKS_PER_SEC,total_word_multiply,total_word_multiply/(double) CLOCKS_PER_SEC);
	}
#endif
        return ConstRC::Ok;
}

int  AbstractMatrix::PerformRowMask(AbstractMatrix* aTarget, AbstractMatrix* aOperand, matrix_type aMask)
{
	return ConstRC::NotSupported;
}



#ifdef MATRIX_OPERATIONS_TIME_MESSAGE

// Time measurement only

int AbstractMatrix::start = 0;
int AbstractMatrix::zero_start = 0;
int AbstractMatrix::diff = 0;
double AbstractMatrix::total_big_multiply = 0;
double AbstractMatrix::total_word_multiply = 0;
double AbstractMatrix::total_word_transpose_multiply = 0;
double AbstractMatrix::total_transpose = 0;
double AbstractMatrix::total_add = 0;
double AbstractMatrix::total_zeroize = 0;
double AbstractMatrix::total_copy = 0;
double AbstractMatrix::total_iszero = 0;

#endif

int AbstractMatrix::Serialize(const CBaseParameters &aParam) const
{
	return MSerializable::Serialize(aParam);
}

int AbstractMatrix::Serialize(const CBaseParameters &aParam, xmlTextWriterPtr &aWriter) const
{
	std::string mainTagName;	
	aParam.GetStringParameter(PARAM_SERIALIZATION_ELEMENT_NAME, XML_SERIAL_MATRIX_NAME, mainTagName);

	int rc = xmlTextWriterStartElement(aWriter, BAD_CAST mainTagName.c_str());
    if (rc < 0) 
	{
        CLog::LogError(WHEREARG,"AbstractMatrix","Serialize",rc,0,"Could not write the parameter tag.");
    	return ConstRC::GeneralError;
    }
	rc = CommonXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = WriteHeader(aWriter);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = WriteBody(aWriter);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = xmlTextWriterEndElement(aWriter); // mainTagName
    if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"AbstractMatrix","Serialize",rc,0,"Could not end the parameter tag.");
        return ConstRC::GeneralError;
    }
	rc = CommonXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	return ConstRC::Ok;
}

long AbstractMatrix::CountNonZeroItems() {
	long count = 0;
	for (long row = 0; row <=GetMaximalRowIndex(); row++)
		for (long column = 0; column <= GetMaximalColumnIndex(); column++)
			if (!IsZero(row, column)) count++;
	return count;
}

long AbstractMatrix::MaxNonZeroItemsPerRow() {
	long count = 0;
	for (long row = 0; row <=GetMaximalRowIndex(); row++) {
		long row_count = 0;
		for (long column = 0; column <= GetMaximalColumnIndex(); column++)
			if (!IsZero(row, column)) row_count++;
		if (row_count > count) count = row_count;
	}
	return count;
}

int AbstractMatrix::Deserialize(const CBaseParameters &aParam)
{
	return MSerializable::Deserialize(aParam);
}

int AbstractMatrix::Deserialize(const CBaseParameters &aParam, xmlTextReaderPtr &aReader)
{
	std::string mainTagName;	
	aParam.GetStringParameter(PARAM_SERIALIZATION_ELEMENT_NAME, XML_SERIAL_MATRIX_NAME, mainTagName);

	int rc;
	xmlChar* name = xmlTextReaderName(aReader);

	if (name == NULL)
	{
		CLog::LogError(WHEREARG,"AbstractMatrix","Deserialize",0,0,"Deserialization main element name not found.");
		return ConstRC::GeneralError;  
	}

	if (xmlStrEqual(name,BAD_CAST mainTagName.c_str()) == 0)
	{
		CLog::LogError(WHEREARG,"AbstractMatrix","Deserialize",0,0,"Main element name is wrong.");
		xmlFree(name);
		return ConstRC::GeneralError; 
	}
	xmlFree(name);
		
	rc = CommonXMLHelper::MoveToNextNode(aReader);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = ReadHeader(aReader);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CommonXMLHelper::MoveToNextNode(aReader);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	return ReadBody(aReader);
}


int AbstractMatrix::WriteHeader(xmlTextWriterPtr aWriter) const
{
	int rc = xmlTextWriterStartElement(aWriter, BAD_CAST XML_SERIAL_HEADER_NAME);
    if (rc < 0) 
	{
        CLog::LogError(WHEREARG,"AbstractMatrix","WriteHeader",rc,0,"Could not write the header parameter tag.");
    	return ConstRC::GeneralError;
    }
	rc = CommonXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CommonXMLHelper::SerializeSignedLong(aWriter, XML_SERIAL_MAX_ROW_INDEX_NAME, this->maximal_row_index);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CommonXMLHelper::SerializeSignedLong(aWriter, XML_SERIAL_MAX_COLUMN_INDEX_NAME, this->maximal_column_index);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CommonXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_MATRIX_TYPE_NAME, (int)this->species);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CommonXMLHelper::SerializeSignedInt(aWriter, PARAM_INFO_MODE, this->info_mode);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CommonXMLHelper::SerializeSignedInt(aWriter, PARAM_ASSERTION_MODE, this->assertion_mode);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = xmlTextWriterEndElement(aWriter); // XML_SERIAL_HEADER_NAME
    if (rc < 0) 
	{
        CLog::LogError(WHEREARG,"AbstractMatrix","WriteHeader",rc,0,"Could not end the header parameter tag.");
    	return ConstRC::GeneralError;
    }

	return CommonXMLHelper::NewLine(aWriter,FALSE);
}

int AbstractMatrix::ReadHeader(xmlTextReaderPtr aReader)
{
	int rc;
	xmlChar * name = xmlTextReaderName(aReader);
	if (name == NULL)
	{
		CLog::LogError(WHEREARG,"AbstractMatrix","ReadHeader",0,0,"Could not find the header parameter tag.");
		return ConstRC::GeneralError;  
	}

	if (xmlStrEqual(name,BAD_CAST XML_SERIAL_HEADER_NAME) == 0)
	{
		CLog::LogError(WHEREARG,"AbstractMatrix","ReadHeader",0,0,"Could not find the header parameter tag.");
		xmlFree(name);
		return ConstRC::GeneralError;
	}

	int depth = xmlTextReaderDepth(aReader);
	int auxInt;
	while ((rc = CommonXMLHelper::MoveToNextNode(aReader, depth)) == ConstRC::Ok)
	{
		if (NULL != name) xmlFree(name);
		name = xmlTextReaderName(aReader);
	
		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_MAX_ROW_INDEX_NAME))
		{
			rc = CommonXMLHelper::DeserializeSignedLong(aReader, "Deserialize max row index...", this->maximal_row_index, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_MAX_COLUMN_INDEX_NAME))
		{
			rc = CommonXMLHelper::DeserializeSignedLong(aReader, "Deserialize max column index...", this->maximal_column_index, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_MATRIX_TYPE_NAME))
		{
			rc = CommonXMLHelper::DeserializeSignedInt(aReader, "Deserialize matrix type...", auxInt, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}

			if (this->species != (species_of_matrices)auxInt)
			{
				CLog::LogError(WHEREARG,"AbstractMatrix","ReadHeader",0,0,"Wrong Matrix type.");
				rc = ConstRC::GeneralError;
				break;
			}
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST PARAM_INFO_MODE))
		{
			rc = CommonXMLHelper::DeserializeSignedInt(aReader, "Deserialize info mode...", this->info_mode, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST PARAM_ASSERTION_MODE))
		{
			rc = CommonXMLHelper::DeserializeSignedInt(aReader, "Deserialize assertion mode...", this->assertion_mode, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			continue;
		}
	}
	if (NULL != name) xmlFree(name);
	if (rc == ConstRC::GeneralError || rc == ConstRC::NotEnoughMemory)
	{
		return rc;
	}

	return ConstRC::Ok;
}
int AbstractMatrix::WriteBody(xmlTextWriterPtr aWriter) const
{
	int rc = xmlTextWriterStartElement(aWriter, BAD_CAST XML_SERIAL_BODY_NAME);
    if (rc < 0) 
	{
        CLog::LogError(WHEREARG,"AbstractMatrix","WriteBody",rc,0,"Could not write the body tag.");
    	return ConstRC::GeneralError;
    }
	rc = CommonXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = WriteData(aWriter);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = xmlTextWriterEndElement(aWriter); // XML_SERIAL_BODY_NAME
    if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"AbstractMatrix","WriteBody",rc,0,"Could not end the body tag.");
        return ConstRC::GeneralError;
    }
	return CommonXMLHelper::NewLine(aWriter,FALSE);
}

int AbstractMatrix::ZeroizeRow(long aRow) {
	for (long index = 0; index <= GetMaximalColumnIndex();index ++)
		PutZero(aRow, index);
}

int AbstractMatrix::ZeroizeRow(long * aRowList, long aListMaxIndex) {
	if (!aRowList) return ConstRC::GeneralError;
	int rc = ConstRC::Ok;
	for (long index = 0; index <= aListMaxIndex; index ++) {
		int rowrc = ZeroizeRow(aRowList[index]);
		if (ConstRC::Ok != rowrc) rc = rowrc;
	}
	return rc;
}

int AbstractMatrix::ReadBody(xmlTextReaderPtr aReader)
{
	int rc;
	xmlChar * name = xmlTextReaderName(aReader);
	if (name == NULL)
	{
		CLog::LogError(WHEREARG,"AbstractMatrix","ReadBody",0,0,"Could not find the body parameter tag.");
		return ConstRC::GeneralError;  
	}

	if (xmlStrEqual(name,BAD_CAST XML_SERIAL_BODY_NAME) == 0)
	{
		CLog::LogError(WHEREARG,"AbstractMatrix","ReadBody",0,0,"Could not find the body parameter tag.");
		xmlFree(name);
		return ConstRC::GeneralError;
	}
	xmlFree(name);

	rc = CommonXMLHelper::MoveToNextNode(aReader);
	if (rc != ConstRC::Ok)
	{
		if (rc == ConstRC::EndOfFile)
		{
			return ConstRC::Ok;
		}
		return rc;
	}

	return ReadData(aReader);
}
