#ifndef __ABSTRACT_MATRIX_SOLVER_H__
#define __ABSTRACT_MATRIX_SOLVER_H__

#include "bit_matrix_class.h"
#include "enums_common.h"


///Abstract class for algorithm solving linear systems
class AMatrixSolver
{
protected:
// Protected fields
	///Info mode turn on/off
	bool iInfoMode;
	///Assertion mode turn on/off
	bool iAssertionMode;
	///Will ranks of A and B be calculated and displayed?
	bool iRankCalculationMode;
	///Will density of A and B be calculated and displayed?
	bool iDensityMode;
	///Will the list of phases and time spent in them be displayed?
	bool iTimeMessageMode;
	///Will finally B and result be checked to give zero matrix?
	bool iCheckFinalResult;
	///Type of matrix to solve
	species_of_matrices iMatrixType;
	///Specific type of algorithm
	matrix_solver_type iType;

public:
// Public methods
	///Solve linear system
	virtual BitMatrix * Compute(AbstractMatrix * aMatrix) = 0;

	///Virtual destructor for later use
	virtual ~AMatrixSolver() {}

// Getters and Setters

	bool Get_InfoMode() const { return this->iInfoMode; }
	void Set_InfoMode(bool aInfoMode) { this->iInfoMode = aInfoMode; }

	bool Get_AssertionMode() const { return this->iAssertionMode; }
	void Set_AssertionMode(bool aAssertionMode) { this->iAssertionMode = aAssertionMode; }

	bool Get_RankCalculationMode() const { return this->iRankCalculationMode; }
	void Set_RankCalculationMode(const bool aRankCalculationMode) { this->iRankCalculationMode = aRankCalculationMode; }

	bool Get_DensityMode() const { return this->iDensityMode; }
	void Set_DensityMode(bool aDensityMode) { this->iDensityMode = aDensityMode; }

	bool Get_TimeMessageMode() const { return this->iTimeMessageMode; }
	void Set_TimeMessageMode(bool aTimeMessageMode) { this->iTimeMessageMode = aTimeMessageMode; }

	bool Get_CheckFinalResult() const { return this->iCheckFinalResult; }
	void Set_CheckFinalResult(bool aCheckFinalResult) { this->iCheckFinalResult = aCheckFinalResult; }
	
	species_of_matrices Get_MatrixType() const { return this->iMatrixType; }
	void Set_MatrixType(species_of_matrices aMatrixType) { this->iMatrixType = aMatrixType; }

	matrix_solver_type Get_MatrixSolverType() const { return this->iType; }

};


#endif // __ABSTRACT_MATRIX_SOLVER_H__
