/*
 * abstract_mpz_matrix.h
 *
 *      Author: jero
 */

#ifndef ABSTRACT_MPZ_MATRIX_H_
#define ABSTRACT_MPZ_MATRIX_H_

#include "abstract_integer_matrix.h"

/*! \brief Abstract common ancestor for matrices with mpz_t items
 *
 *  AbstractMpzMatrix is a common ancestor class which is (theoretically) capable of
 *  of anything that is possible with bit or integer matrices, therefore it is a descendant
 *  of AbstractIntegerMatrix
 */
class AbstractMpzMatrix: public AbstractIntegerMatrix {
public:
	AbstractMpzMatrix();
	virtual ~AbstractMpzMatrix();

	virtual void PrintToScreen(); ///< This method will be used for display of the matrix

	int SetModulo(mpz_t aValue);
	int SetModulo(int aValue);
	virtual int GetModulo();

	virtual int  Randomize(); ///< Initialize matrix with random elements

	virtual int SwapRows(long aRow1, long aRow2); ///< swap two rows
	virtual int MultiplyRow(long aRow, mpz_t aValue); ///< multiply row by value
	virtual int MultiplyRow(long aRow, integer_matrix_type aValue); ///< multiply row by value

	virtual int SubtractRowMultiple(long aTargetRow, long aSourceRow, integer_matrix_type aMul);
	virtual int SubtractRowMultiple(long aTargetRow, long aSourceRow, mpz_t aMultiple);
	bool IsInvertible(long aRow, long aColumn);

	virtual int PutValueMpz(long aRow, long aColumn, mpz_t aValue) = 0; ///< Put arbitrary long value in the matrix
	virtual int GetValueMpz(long aRow, long aColumn, mpz_t aValue) = 0; ///< Get arbitrary value from the matrix
	virtual mpz_t * GetValueDirect(long aRow, long aColumn) = 0; ///< Get pointer to the arbitrary longvalue in the matrix


	virtual int PutValue(long aRow, long aColumn, integer_matrix_type aValue) = 0; ///< Put integer value in the matrix
	virtual int GetValue(long aRow, long aColumn, integer_matrix_type & aValue) =0; ///< Get integer value form the matrix

	virtual int GaussColumn(long aRow, long aColumn, bool aPartial); ///< Clear column with given pivot

	int RowProduct(mpz_t & aResult, long aRow1, long aRow2);
	int LLL();

	int Test();

protected:
	bool modular;	///< Is modulo set?
	mpz_t modulo;  ///< Internal modular arithmetic

};

#endif /* ABSTRACT_MPZ_MATRIX_H_ */
