#include "abstract_serializator.h"
#include "algorithm.h"
#include <string.h>
#include "log.h"

/// constructors
AbstractSerializator::AbstractSerializator() {
	iAlgorithmInstance = this;
	iDirectoryName.assign(".");
	iCompressionLevel = 0;

	return;
}

AbstractSerializator::AbstractSerializator(Algorithm *aAlgorithmInstance, const char* aDirectoryName, int aCompressionLevel) : iAlgorithmInstance(aAlgorithmInstance), iCompressionLevel(aCompressionLevel), iInfoMode(TRUE)
{
	iDirectoryName.assign(".");		
}

AbstractSerializator::~AbstractSerializator() 
{
}

void	AbstractSerializator::RegisterInstance(Algorithm *aAlgorithmInstance)
{
	iAlgorithmInstance = aAlgorithmInstance;
}

int	AbstractSerializator::Get_CompressionLevel() const 
{ 
	return this->iCompressionLevel; 
}

void	AbstractSerializator::Set_CompressionLevel(int aCompressionLevel) 
{ 
	iCompressionLevel = aCompressionLevel; 
}

void  	AbstractSerializator::Set_DirectoryName(const string& aDirectoryName) 
{
	iDirectoryName = aDirectoryName;		
}
	
void	AbstractSerializator::AppendPathSeparator(string& aPath)
{
	if (aPath.length() > 0)
	{
		char last = aPath[aPath.length()-1];
		// PATH_SEPARATOR is defined in definitions.h
		if (last != PATH_SEPARATOR)
		{				
			aPath += PATH_SEPARATOR;				
		}
	} 
}
     
/**
	This function does not return reference, but a copy of 
	the string. This is because we do not want the user
	to mess with internal member of this class.
*/
string    AbstractSerializator::CreateFileNamePlain(const char* aFile)
{
	string str;
	str.append(iDirectoryName);
	AppendPathSeparator(str);
	if (aFile)
	{
		str.append(aFile);
	}
	//printf("AbstractSer: Create fn plain: %s",str.c_str());
	iAuxName = str;
	return iAuxName;
}	

void    AbstractSerializator::CreateFileNameCompressed(const char* aFile)
{
	CreateFileNamePlain(aFile);
	iAuxName.append(COMPRESSED_FILE_EXTENSION);
	//printf("AbstractSer: Create fn compressed: %s",iAuxName.c_str());
}
	
void	AbstractSerializator::CreateFileNameAuto(const char* aFile)
{
	if (this->iCompressionLevel)
	{
		CreateFileNameCompressed(aFile);
	} else {
		CreateFileNamePlain(aFile);
	}
	//printf("XML Qs Serializator: filename created as %s\n",iAuxChar);
}

	// Removes both XXX.xml and XXX.xml.z from the current directory
void    AbstractSerializator::Remove(const char* aFile)
{
	if (aFile == NULL) return;
	// .xml file
	CreateFileNamePlain(aFile);
	//printf("Abstract Serializator: removing file %s\n",iAuxName.c_str());
	remove(iAuxName.c_str());
	// .xml.z file
	CreateFileNameCompressed(aFile);
	remove(iAuxName.c_str());
}

FILE*	AbstractSerializator::OpenFile(const char* aMode)
{
	FILE* result = fopen(iAuxName.c_str(),aMode);
	if (!result)
	{
		printf("Abstract Serializator - Open File, Could not open file.\n");
	}
	return result;
}

void	AbstractSerializator::PrintInstance() const
{
	printf("*************** AbstractSerializator: %p, instance %p.\n",this,iAlgorithmInstance);
}

int	AbstractSerializator::CopyTextFileToWriter(xmlTextWriterPtr& aWriter, FILE* aFile)
{
	char buffer[513];
	bool isEof = FALSE;
	int rc = 0;
	int read;
	int totalRead = 0;
	do
	{
		memset(buffer,0,513);
		read = fread(buffer, 1, 512, aFile);
		if (read)	
		{
			rc = xmlTextWriterWriteString	(aWriter, BAD_CAST buffer);
    			if (rc < 0) 
			{
	        		CLog::LogError(WHEREARG,"AbstractSerializator","CopyTextFileToWriter",rc,0,"Could not write a data string.");
        			return totalRead;
    			}		
			totalRead += read;
		}
		
		isEof = feof(aFile);
	} while (!isEof);
	return totalRead;
}

