#ifndef _ALGORITHM_H_
#define _ALGORITHM_H_

class JobParameters;

/*!
 * \brief Abstract ancestor of sieving factorization algorithms
 */
class Algorithm
{
public:
	// Pure virtual destructor.
	virtual ~Algorithm() = 0; 

	
	// Returns a non-NULL name of this algorithm.
	virtual const char* 	AlgorithmName()       const = 0;
	// Is intended to create a fresh, empty (or with default values)
	// instance of JobParameters appropriate for this algorithm.
	// This instance will probably be a direct subclass of JobParameters.
	virtual JobParameters*  CreateNewParameters() const = 0;
	// Is intended to prepare the algorithm for run. Not all algorithms use this
	// meaningfully.
	virtual int		SetupParameters(const JobParameters& aParameters) = 0;
};

#endif

