/*
 * base_int_matrix.h
 *      Author: jero
 */

#ifndef BASE_INT_MATRIX_H_
#define BASE_INT_MATRIX_H_

#include "abstract_integer_matrix.h"
#include "gmp.h"

/*!
 * \brief Common ancestor for all int-typed matrices
 */
class BaseIntMatrix: public AbstractIntegerMatrix {
public:
	BaseIntMatrix();
	virtual ~BaseIntMatrix();

	int  Randomize(); ///< Fill matrix with random numbers

	int SetModulo(int aValue); ///< Set internal modulo
	int SetModulo(mpz_t aValue); ///< Set internal modulo
	virtual int GetModulo(); ///< Get internal modulo
	int ApplyModulo(); ///< Apply modulo on matrix members

	virtual int MultiplyRow(long aRow, integer_matrix_type aMul); ///< Multiply row by integer
	virtual int MultiplyRow(long aRow, mpz_t aMul); ///< Multiply row by mpz_t

	virtual int SubtractRowMultiple(long aTargetRow, long aSourceRow, integer_matrix_type aMul);
	virtual int SubtractRowMultiple(long aTargetRow, long aSourceRow, mpz_t aMul);

	virtual int GaussColumn(long aRow, long aColumn, bool aPartial); ///< (optionally partially) clear other values in this column
	virtual bool IsInvertible(long aRow, long aColumn);

	bool Equals(BaseIntMatrix * aMatrix); ///< check equality of matrices

protected:
	integer_matrix_type modulo; ///< value of modulo for modular arithmetic
};

#endif /* BASE_INT_MATRIX_H_ */
