#ifndef _BASIC_HASHTABLE_
#define _BASIC_HASHTABLE_

/*
	Here we introduce basic hashtable class for large primes.
	It should just keep in memory which large primes have been met.
	We treat collision resolution and hashtable enlargement.
*/

#include "const_hashtables.h"

/*! 
 * \brief Basic entry for hashtables supporting value and mask
 */
class CBasicHashEntry {
protected:
	large_prime_type value;	
public:	
	void SetMask() 	{value = ConstHT::FULL_MASK_NFS;};
	bool IsMasked() {return value == ConstHT::FULL_MASK_NFS;};

	large_prime_type GetValue() const {return value;};
	void SetValue(large_prime_type aValue) {value = aValue;};
};

/*!
 * \brief Basic hashtable supporting value and mask
 */

class CBasicHashtable {
protected:
	unsigned long filled;
public:
	///Hashtable data type
	CBasicHashEntry *Hashtable;
	///Number of the allocated hashtable entries
	long AllocatedSize;
	unsigned long	Mask;

	CBasicHashtable();
	~CBasicHashtable();

	void DeleteHashtable();
	int ClearHashtable();
	int SetupHashtable(long aAmount);

	int AddToTable(large_prime_type aPrime);
	int FindPrimeInHashtable(large_prime_type aPrime, bool & aPresent, long & aIndex, bool & aEnlarged) ;

	void Added() { filled++;};

	unsigned long GetFilled() {return filled;};

	void Print();
private:
	int CollisionResolution(long aHash, large_prime_type aPrime, long & aIndex) const;
	
	int EnlargeHashtable();
};

#endif // _BASIC_HASHTABLE_

