#ifndef __BUFFERED_FILE_WRITER_H__
#define __BUFFERED_FILE_WRITER_H__

# include "buffered_file_writer_item.h"
# include <pthread.h>
# include <stdio.h>

/*!
 * \brief Support structure for CBufferedFileWriter
 */
struct WrittingThreadArgs {
	FILE *file;
	CBufferedFileWriterItem **itemsBuffer;
	int filled;
};

/*!
 * \brief Buffered write in separate thread.
 *
 * This class implements buffered write of items (relations) to file.  Items
 * are written to file since items buffer is filled. Write runs in a separate
 * thread.  When file is being closed, buffered items are flushed to file.
 * Close waits for writting thread.
*/
class CBufferedFileWriter {

private:

	FILE *iFile;

	int iItemsBufferSize;
	int iItemsBufferFilled;
	CBufferedFileWriterItem **iItemsBuffer;

	char *iFileBuffer;

	pthread_t iWrittingThread;

public:

	CBufferedFileWriter();
	~CBufferedFileWriter();

	int Open(const char *aFilename, bool aAppend, int aItemsBufferSize, int aFileBufferSize);
	int Close();

	int Write(CBufferedFileWriterItem &aItem);
	int Flush();

};

#endif // __BUFFERED_FILE_WRITER_H__

