#ifndef __COMMON_DEFINITIONS_H__
#define __COMMON_DEFINITIONS_H__

/*!
  \file 

  This file contains definitions of symbolic constants.
*/

/*! \brief Undef this to disable XML serialization and deserialization
 By undefing this constant, you will get a lightweight, no-XML version of MPQS/SIQS.
 No loading/saving of states really possible.*/
#define WITH_XML_SERIALIZATOR

/*! \def PATH_SEPARATOR
 \brief Path separator (OS dependent)
*/
#ifndef WIN32
#include <inttypes.h>
#include <unistd.h>
#define PATH_SEPARATOR '/'
#else
#define PATH_SEPARATOR '\\'
#endif

#ifdef WIN32
    #ifndef NAN
        static const unsigned long __nan[2] = {0xffffffff, 0x7fffffff};
        #define NAN (*(const float *) __nan)
    #endif
#endif

//! This must be defined in order to have time messages in QS (after each X polynomials a message concerning time expensiveness of tasks).
#define TIME_MESSAGE 

//! This must be defined in order to have time messages after block Lanczos run, concerning phases of Lanczos computation
#define LANCZOS_TIME_MESSAGE 
//! This must be defined in order to collect time expensiveness of matrix (BitMatrix and SparseMatrix) operations.
#define MATRIX_OPERATIONS_TIME_MESSAGE

#define DEFAULT_INTERMEDIATE_ROOTS  200

// delimiters for MPQS/SIQS command line options
#define MAX_POLY 	  1000000000
#define MAX_OMIT 	  1000
#define IMPLICIT_FACTORBASE 2000
#define IMPLICIT_SIEVE_INT  10000
#define IMPLICIT_TMESS 	    100
#define MAXIMAL_LINE_SIEVE_HALFLINE 100000000
#define MAXIMAL_FACTOR_BASE_UPPER_BOUND 100000000


#define INCONSISTENT_STATE -10

//#define CHECK_SPARSE_MATRICES
#define RANK_COMPUTATIONS

//! QS version number
#define QS_VERSION     		2.99

#define TRUE        		1
#define FALSE       		0
#define _LOG_FACTOR     	0x1
#define DEFAULT_LOG_BASE        4
#define SIEVE_ROUNDOFF_ERROR 	0x8
#define TEST_VALUE  		0x80
#define CHECK_LEVEL_STEP	0x04

// For Carrier-Wagstaff method of 'a' generation.
#define UNSUITABLE_FOR_SIQS     1
#define DEFAULT_MACHINE_BOUND_START	20
#define DEFAULT_MACHINE_BOUND_END   	55
#define CW_EVEN_INDEX_START	56
#define CW_EVEN_INDEX_SIZE	100
//#define CW_ODD_INDEX_START      57
//#define CW_ODD_INDEX_SIZE       60
#define TRIPLE_SIZE      	4
#define TRIPLES_LOG_FACTOR 	131072

// For sieving with prime powers (since 1.15)
#define DEFAULT_PRIME_POWERS_ALLOCATED 1000
#define DEFAULT_LIMIT_MULTIPLIER 10

// For Pollard Rho factoring method
#define DEFAULT_POLLARD_BLOCK_SIZE 50

// For Double Large Prime Variation
// 2^23 entries
#define DEFAULT_CYCLE_TABLE_SIZE  1024
//#define DEFAULT_CYCLE_TABLE_SIZE 131072
//#define DEFAULT_CYCLE_TABLE_SIZE 8388608
//#define DEFAULT_CYCLE_TABLE_SIZE 524288*2
//#define DEFAULT_CYCLE_TABLE_SIZE 32768

// Masks for row_element.inited
#define ARG_INITED 	0x1
#define FAC_INITED 	0x2
#define REM_INITED 	0x4
#define LSV_INITED 	0x8
#define RSV_INITED 	0x10

// Other constants for row_element
#define NO_POLY_USED    -1

// Masks for abstractsieve and quadsieve.inited
#define N_INITED  	0x1000
#define kN_INITED 	0x2000
#define M_INITED  	0x4000
#define ROOT_INITED     0x10000
#define MPF_INITED      0x40000
#define F1_INITED       0x80000
#define F2_INITED       0x100000
#define STIME_MAIN_INITED 0x400000
#define STIME_L1_INITED   0x800000
#define STIME_L2_INITED   0x1000000
#define IDEAL_A_VALUE_INITED 0x2000000
#define MIN_C_VALUE_INITED   0x4000000
#define MAX_C_VALUE_INITED   0x8000000

// Masks for quadsieve.open
#define PARTIALS_OPEN   	0x1
#define SMOOTHS_OPEN    	0x2
#define ALL_PARTIALS_OPEN 	0x4
#define USABLE_PARTIALS_OPEN    0x8
#define TEMPORARY_OPEN 		0x10
#define JOB_FILE_OPEN		0x20
#define LAST_STATE_FILE_OPEN    0x40
#define CURRENT_STATE_FILE_OPEN 0x80

// Masks for quad_polynomial.inited
#define A_INITED        0x1
#define B_INITED        0x2
#define C_INITED        0x4
#define D_INITED        0x8
#define D2_INITED       0x10

// Masks for variation.inited
#define VAR_INITED      0x1

// Masks for auxiliary inited
#define AUX_1_INITED    0x1
#define AUX_2_INITED    0x2
#define AUX_3_INITED    0x4
#define AUX_4_INITED    0x8
#define AUX_5_INITED    0x10
#define AUX_A_INITED    0x20
#define AUX_B_INITED    0x40
#define AUX_C_INITED    0x80
#define AUX_D_INITED    0x100
#define	AUX_P_INITED    0x200
#define AUX_P_EXP_INITED 0x400
#define AUX_A_INV_INITED 0x800
#define AUX_K_INITED     0x1000
#define AUX_L_INITED     0x2000

// Masks for auxiliary - Shanks-Tonelli inited
#define ST_AUX_A_INITED           0x1
#define ST_AUX_A_INVERSE_INITED   0x2
#define ST_AUX_B_INITED           0x4
#define ST_AUX_R_INITED           0x8
#define ST_AUX_ROOT_INITED        0x10
#define ST_AUX_N_INITED           0x20
#define ST_AUX_P_MINUS_1_INITED   0x40
#define ST_AUX_P_INITED           0x80
#define ST_AUX_1_INITED           0x100
#define ST_AUX_B_EXP_2I_INITED    0x200
#define ST_AUX_NON_INITED         0x400

// Masks for WIB auxiliary variables
#define WIB_DIFF_INITED       0x1
#define WIB_RATIO_INITED      0x2
#define WIB_MUL_P_INITED      0x4
#define WIB_ARG_INITED        0x8
#define WIB_ARG_2_INITED      0x10
#define WIB_ARG_PLUS_INITED   0x20

// Error return codes - general
#define NOT_ENOUGH_MEMORY    -1
#define COULD_NOT_OPEN_FILE  -2
#define COULD_NOT_CLOSE_FILE -3
#define COULD_NOT_READ_FROM_NULL_STREAM -4
#define END_OF_FILE          -5
#define BAD_ARGUMENT         -6

// Further error codes can be found in connectivity_framework_definitions.h
// They are -500 and lower

// Modes of file access
//! File mode constant for read
#define READ                  0
//! File mode constant for write
#define WRITE                 1
//! File mode constant for append
#define APPEND                2

// Primes smaller than this constant will be generated if primefile not found. NormalMessage("   Reading primes from file ...",NO_NUMBER);
#define DEFAULT_PRIMEFILE_BOUND 10000000

// Error return codes - init environment etc.
#define CALLED_TOO_EARLY     -101
#define NOT_SUPPORTED        -102

// Quadratic sieve defaults
#define DEFAULT_EXTENT          1
#define DEFAULT_MARGIN          10
#define DEFAULT_VALUE_M         10000
#define DEFAULT_FB_BOUND        10000
#define DEFAULT_NR_POLYNOMIALS  1
#define DEFAULT_DIV_ALLOCATE    16
#define INIT_VALUE_FACTOR       0
#define INIT_VALUE_N		-1
#define INIT_VALUE_k		0


// Return values of RunMPQS()
//! Return value of RunMPQS
#define PRIME_NUMBER            2
//! Return value of RunMPQS
#define DIVISIBLE_BY_SMALL_PRIME 3
//! Return value of RunMPQS
#define NOT_FACTORIZED          -2
//! Return value of RunMPQS
#define NOT_ENOUGH_SMOOTHS      -3
//! Return value of RunMPQS
#define ERROR_INIT_ENVIRONMENT  -4
//! Return value of RunMPQS
#define ERROR_SIEVING           -5
//! Return value of RunMPQS
#define ERROR_LINALG            -6
//! Return value of RunMPQS
#define TOO_SMALL_FACTORBASE    -7
//! Return value of RunMPQS
#define NO_DEPENDENCIES         -8
//! Return value of RunMPQS
#define ERROR_PROCESSING_RELATIONS -9
//! Return value of RunMPQS
#define EXCESSIVE_FACTOR_BASE   -10

// Return values of RunTTR()
#define TTR_OK                  0
#define TTR_NOT_OK              -1

// Checks for primality - number of rounds in Miller-Rabin test
#define MILLER_RABIN_ROUNDS_NORMAL 	15
#define MILLER_RABIN_ROUNDS_STRICT 	40

// Default file names
//! Default file name for saving partial relations
#define DEFAULT_PARTIAL_FILE_NAME "partials"
//! Default file name for saving smooth relations
#define DEFAULT_SMOOTH_FILE_NAME  "smooths"
//! Default file name for file with primes
#define DEFAULT_PRIMEFILE_NAME    "primefil"

// Since 2.00
#define DEFAULT_TEMPORARY_FILE_NAME "temp"
#define DEFAULT_ALL_PARTIALS_FILE_NAME "all_partials"
#define DEFAULT_USABLE_PARTIALS_FILE_NAME "usable_partials"
// Since 2.02
#define DEFAULT_JOB_FILE_NAME "job"
#define DEFAULT_LAST_STATE_FILE_NAME "last_state"
#define DEFAULT_CURRENT_STATE_FILE_NAME "current_state"
#define DEFAULT_CYCLE_ARRAY_ENTRIES 15
#define DEFAULT_ROOT_LIST           20

// Default reallocation steps
#define SMOOTH_REALLOCATION_STEP  	100
#define USELESS_PRIME_REALLOCATION_STEP 200
#define USELESS_ROW_REALLOCATION_STEP   200

// SIQS defaults (added in 1.09)
#define DEFAULT_S		8
#define DEFAULT_Q_SHIFT         2
#define DEFAULT_LEAST_SIQS_DIVISOR	1024

// Variation-related constants
#define VAR_INITED                0x1
#define NO_INDEX                  -1
#define VARIATION_FACTOR          128

// for normal message
#define NO_NUMBER                 -1

// For allocating polynomials
#define DEFAULT_POLYNOMIALS_ALLOCATED   128
#define DEFAULT_TIME_MESSAGE_INTERVAL   100

// Rough mode settings - from this prime rough mode runs
#define ROUGH_MODE_LOWER_BOUND    100
#define DEFAULT_ROUGH_INDEX	  15

// Sieving matrix - how much divisors are allocated at the beginning
#define DIV_ALLOCATE    	  8

// Value ENOUGH is returned if enough smooth values have been generated
#define NOT_ENOUGH                -1
#define ENOUGH                    1
#define ENOUGH_BUT_WARNING	  2

// Quadratic equation pathologies
#define TRIVIAL_EQUATION          1
#define NOT_SUITABLE_MODULUS      -1
#define INSOLUBLE_EQUATION        -2

// Solutions backup
#define THERE                      1
#define BACK                       2

// Levels of MPQS execution
#define MAIN_LEVEL                 0
#define LEVEL_1                    1
#define LEVEL_2                    2

// Constants connected with variations
#define NONE                       0
#define LEFT                       1
#define RIGHT                      2
#define VAR_INITED                 0x1

// Constant connected with hashes
#define HASH_INITED                0x1000

// Return values of CheckNulity
#define ZERO           		   0
#define NONZERO                    1

// Return value of FindVariationInFactorBase, meaning failure
#define NOT_FOUND                  -10
#define FOUND                      10

// Masks for integer solutions, added in 1.04
#define ALLOCATED_1_SOLS           0x1
#define ALLOCATED_2_SOLS           0x2
#define ALLOCATED_USABLE_SOLS      0x4

#define VALID_1                    0x1
#define VALID_2                    0x2

// Bit Matrix - some return values
#define EXCESSIVE_ROW_INDEX        -10
#define EXCESSIVE_COLUMN_INDEX     -20
#define NEGATIVE_INDEX             -25
#define NULL_POINTER_SUPPLIED      -30
#define SIZE_MISMATCH              -40

#define GENERAL_ERROR              0xffffffff

// Sparse Matrix - some macros
#define SPARSE_REALLOCATION_STEP   10
#define ALLOCATED(x) SPARSE_REALLOCATION_STEP*((((x)+2)/SPARSE_REALLOCATION_STEP)+1)
#define INTEGER_ALLOCATED(x) SPARSE_REALLOCATION_STEP*((((x)*2+2)/SPARSE_REALLOCATION_STEP)+1)

// Large Prime Variation - reallocation in array of line numbers
#define LPV_LINE_REALLOCATION_STEP  4
#define LPV_ALLOCATED(x) LPV_LINE_REALLOCATION_STEP*((((x)+2)/LPV_LINE_REALLOCATION_STEP)+1)

// Large Prime Variation - reallocation in arrays
#define DEFAULT_INTERMEDIATE_ARRAY_ALLOCATE 100
#define DEFAULT_LPS_ARRAY_ALLOCATE_SINGLE   2
#define DEFAULT_LPS_ARRAY_ALLOCATE_DOUBLE   40

#ifdef _INTTYPES_H // getting typedefs from inttypes_common.h
typedef int64_t INT64;
#endif

#ifdef WIN32       // windows specific
typedef __int64 INT64;
#endif

#if defined(_UNISTD_H) || defined(__CYGWIN__)
#define SLEEP(x)	sleep((x))
#endif

//#ifdef WIN32
//#define SLEEP(x)	Sleep((x))
//#endif

/*! abnormal exit
	 = exit(x) for normal exit
	 = abort() for exit with coredump
*/
#if defined(_UNISTD_H) || defined(__CYGWIN__)
	#define ABNORMAL_EXIT(x) abort()
#else
	#define ABNORMAL_EXIT(x) exit(x)
#endif

#define HASH_MASK                  0x7ffff

// #define FULL_MASK_LONG 		ULONG_MAX
// #define FULL_MASK 		UINT_MAX
#ifdef LONG_LARGE_PRIME 
	#define FULL_MASK ULONG_MAX
#else
	#define FULL_MASK UINT_MAX
#endif
	

#define FACTOR_TOO_LARGE        -4

// Checkauto - constants
#define CHECKAUTO_MAXINDEX	3
#define PARTIAL_1_WEIGHT	5
#define PARTIAL_2_WEIGHT	1
#define RUNNING                 2
#define DEFAULT_INTERVAL_STEP   3500

// Value distribution
#define DISTRIBUTION_STEP       1000
#define MAX_DISTANCE            100
#define SMOOTH_VALUE		1
#define PARTIAL_1_VALUE		2
#define PARTIAL_2_VALUE		4

// CFRAC
#define DEFAULT_CFRAC_BASE_SIZE 250

// For factorization algorithms generally
#define OPTIMIZATION_LEVEL_0     0
#define OPTIMIZATION_LEVEL_1     1
#define OPTIMIZATION_LEVEL_2     2

#define QS_LINE_SIEVING
//#define GNFS_LINE_SIEVING

// An internal constant for the line siever.
#define MAX_UPDATES 500
#define MINIMAL_PRIME_TO_SIEVE_WITH 128

#define POSITIVE  1
#define NEGATIVE -1
#define CACHE_UPDATES 64
#define CHECK_MASK 0x80808080
#define UNLIMITED_SIEVING 0

// Text messages and their indices
#define GEN_NEW_POLY   10
#define GEN_NEW_POLY_TEXT "      Overall time spent in  GenerateNewPoly  is"

#define QUAD_MACHINE   20
#define QUAD_MACHINE_TEXT "      Overall time spent in  QuadraticMachine is"

#define PERF_SIEVING   30
#define PERF_SIEVING_TEXT "      Overall time spent in  PerformSievingPo is"

#define SIEVING_LOOP   40
#define SIEVING_LOOP_TEXT "                         Where  SievingLoopLogar"

#define REFILL_MAINM   50
#define REFILL_MAINM_TEXT "                         Where  RefillMainMatrix"

#define SORT_SMOOTHS   60
#define SORT_SMOOTHS_TEXT "                         Where  SortSmoothValues"

#define DIVISORS_OFA   70
#define DIVISORS_OFA_TEXT "                              Where Divisor Of A"

#define SMALL_FACTOR   80
#define SMALL_FACTOR_TEXT "                              Where Small Factor"

#define COUNT_CYCLES   90
#define COUNT_CYCLES_TEXT "                              Where Count Cycles"

#define BLOCK_SIEVES  100
#define BLOCK_SIEVES_TEXT "      Overall time spent in   Block Set Sieve is"

#define FILL_HSTABLE  110
#define FILL_HSTABLE_TEXT "      Overall time spent in   Fill Hashtables is"

#define FILL_HST_NEG  120
#define FILL_HST_NEG_TEXT "                          Where FillHasht-NEG is"

#define ROOTS_UPDATE  130
#define ROOTS_UPDATE_TEXT "      Overall time spent in   Root Update/New is"

#define FIND_FACTORZ  140
#define FIND_FACTORZ_TEXT "                          Where Find&FactorCa is"

#define UNKNOWN_PART_TEXT "      ! Unknown part ! "

#define FIRST_ROOT_VALID 0x10
#define SECOND_ROOT_VALID 0x20
#define ROOT_INVALID 0xffffffff

#define EQUAL 0
#define NOT_EQUAL 1

#define JOB_LOAD_FLAG 1
#define POLY_LOAD_FLAG  2
#define FB_LOAD_FLAG  4
#define SIEVING_LOAD_FLAG 8
#define LIST_LOAD_FLAG 0x800000

#define COMPRESSED_FILE_EXTENSION ".z"
#define TERMINATED 100

// Relation processing flags
#define RELATIONS_READY_FLAG    		  0x1
#define LARGE_SINGLETONS_REMOVED_FLAG  		  0x2
#define CYCLES_CONSTRUCTED_FLAG	  		  0x4
#define RELATION_COLLECTION_FINISHED_FLAG 	  0x8
#define SIEVING_RUNNING_FLAG			  0x10

#define AUTOSAVE_INTERVAL_IN_SECONDS		30

#define DEFAULT_NUMBER_BASE 10
//
// Parameters - ISerializable
#define PARAM_COMPRESSION_LEVEL "compression-level"
#define PARAM_SERIALIZATION_DIRECTORY "serialization-directory"
#define PARAM_SERIALIZATION_IDENTIFIER "serialization-identifier"
#define PARAM_SERIALIZATION_ELEMENT_NAME "serialization-element-name"
#define PARAM_SERIALIZATION_MAIN_ELEMENT_NAME "serialization-main-element-name"
#define PARAM_SERIALIZATION_FILE "serialization-file"
#define PARAM_SERIALIZATION_FULL_FILENAME "serialization-full-filename"

#define SERIALIZATION_IDENTIFIER_DATETIME_LENGTH 14 // YYYYmmddHHMMSS
#define SERIALIZATION_IDENTIFIER_DATETIME_FORMAT "%Y%m%d%H%M%S"

// XML Serialization - node names

#define XML_SERIAL_PARAM_ELEMENT_NAME "base-parameters"
#define XML_SERIAL_NFS_PARAM_ELEMENT_NAME "nfs-parameters"
#define XML_SERIAL_FACTOR_ALG_PARAM_NAME "factor-alg-parameters"
#define XML_SERIAL_NFS_PARAM_MAIN_NAME "main-parameters"
#define XML_SERIAL_NFS_PARAM_POLY_NAME "poly-selection-parameters"
#define XML_SERIAL_NFS_PARAM_SIEVE_NAME "sieving-parameters"
#define XML_SERIAL_NFS_PARAM_REL_NAME "rel-processing-parameters"
#define XML_SERIAL_NFS_PARAM_LINEAR_NAME "linear-parameters"
#define XML_SERIAL_NFS_PARAM_SQUARE_NAME "square-root-parameters"
#define XML_SERIAL_PRIMES_NAME "primes"
#define XML_SERIAL_PRIME_NAME "prime"
#define XML_SERIAL_MAX_BOUND_NAME "max-bound"
#define XML_SERIAL_HEADER_NAME "header"
#define XML_SERIAL_BODY_NAME "body"
#define XML_SERIAL_POLYNOMIAL_NAME "polynomial"
#define XML_SERIAL_MAX_ROW_INDEX_NAME "max-row-index"
#define XML_SERIAL_MAX_COLUMN_INDEX_NAME "max-column-index"
#define XML_SERIAL_MATRIX_TYPE_NAME "matrix-type"
#define XML_SERIAL_MATRIX_NAME "matrix"
#define XML_SERIAL_MATRIX_ROW_NAME "matrix-row"
#define XML_SERIAL_RELATION_PART_NAME "relation-part"
#define XML_SERIAL_FILTERING_RELATION_PART_NAME "frelation-part"
#define XML_SERIAL_DIVISORS_NAME "divisors"
#define XML_SERIAL_DESCRIPTION_NAME "description"
#define XML_SERIAL_MAX_ELEMENT_INDEX_NAME "max-element-index"
#define XML_SERIAL_ASSIGNED_ELEMENTS_NAME "assigned-elements"
#define XML_SERIAL_ELEMENTS_NAME "elements"
#define XML_SERIAL_ELEMENT_NAME "element"
#define XML_SERIAL_PARTS_NAME "parts"
#define XML_SERIAL_NFS_RELATION_NAME "nfs-relation"
#define XML_SERIAL_FILTERING_NFS_RELATION_NAME "fnfs-relation"
#define XML_SERIAL_ALG_RELATION_NAME "alg-relation"
#define XML_SERIAL_INTEGRAL_FB_NAME "integral-factor-base"
#define XML_SERIAL_ALGEBRAIC_FB_NAME "algebraic-factor-base"
#define XML_SERIAL_FACTOR_BASE_NAME "factor-base"
#define XML_SERIAL_MAX_INDEX_NAME "max-index"
#define XML_SERIAL_UPPER_BOUND_NAME "upper-bound"
#define XML_SERIAL_COUNT_NAME "count"
#define XML_SERIAL_QUADRATIC_CHARACTERS_NAME "quadratic-characters"
#define XML_SERIAL_TIME_CONTAINER_NAME "time-container"
#define XML_SERIAL_STATISTIC_CONTAINER_NAME "statistic-container"
#define XML_SERIAL_CONFIRM_TOTAL_TIMES_NAME "total-times"
#define XML_SERIAL_CONFIRM_TOTALS_NAME "confirm-totals"
#define XML_SERIAL_INTERMEDIATES_NAME "intermediates"
#define XML_SERIAL_POLY_COEFFICIENTS_NAME "poly-coefficients"
#define XML_SERIAL_DEGREE_NAME "degree"
#define XML_SERIAL_PARAMETER_ENTRY_NAME "parameter-entry"
#define XML_SERIAL_ROOT_LIST_NAME "root-list"
#define XML_SERIAL_LARGE_PRIMEIDEALS_NAME "large-primeideals"
#define XML_SERIAL_PRIMEIDEALS_NAME "primeideals"

#define XML_SERIAL_A_NAME "a"
#define XML_SERIAL_B_NAME "b"
#define XML_SERIAL_INTEGRAL_PART_NAME "integral-part"
#define XML_SERIAL_INTEGRAL_REMAINING_NAME "integral-remaining"
#define XML_SERIAL_ALGEBRAIC_NORM_NAME "algebraic-norm"
#define XML_SERIAL_ALGEBRAIC_NORM_REMAINING_NAME "algebraic-norm-remaining"
#define XML_SERIAL_INDEX_NAME "index"
#define XML_SERIAL_TYPE_NAME "type"
#define XML_SERIAL_ALGEBRAIC1_NORM_NAME "algebraic1-norm"
#define XML_SERIAL_ALGEBRAIC1_NORM_REMAINING_NAME "algebraic1-norm-remaining"
#define XML_SERIAL_ALGEBRAIC2_NORM_NAME "algebraic2-norm"
#define XML_SERIAL_ALGEBRAIC2_NORM_REMAINING_NAME "algebraic2-norm-remaining"
#define XML_SERIAL_NBR_OF_RELATIONS_NAME	"number-of-relations"
#define XML_SERIAL_INT_THRESHOLD_NAME		"integral-threshold"
#define XML_SERIAL_ALG_THRESHOLD_NAME		"algebraic-threshold"
#define XML_SERIAL_ALG1_THRESHOLD_NAME		"algebraic1-threshold"
#define XML_SERIAL_ALG2_THRESHOLD_NAME		"algebraic2-threshold"
#define XML_SERIAL_THRESHOLD_REFERENCE_NAME	"threshold_reference"
#define XML_SERIAL_THRESHOLD_CHANGE_NAME	"threshold_change"
#define XML_SERIAL_NUMBER_PRIME_IDEALS_MULTI_NAME		"number-of-prime-ideals-multi"
#define XML_SERIAL_NUMBER_LARGE_PRIME_IDEALS_MULTI_NAME	"number-of-large-prime-ideals-multi"

#define XML_SERIAL_POLY_SEL_PHASE_NAME		"poly-selection-phase"
#define XML_SERIAL_SIEVING_PHASE_NAME		"sieving-phase"
#define XML_SERIAL_REL_PRO_PHASE_NAME		"rel-processing-phase"
#define XML_SERIAL_LINEAR_PHASE_NAME		"linear-phase"
#define XML_SERIAL_SQUARE_ROOT_PHASE_NAME	"square-root-phase"
#define XML_SERIAL_MONTGOMERY_POLY_SEL_PHASE_NAME	"montgomery-poly-selection-phase"
#define XML_SERIAL_KLEINJUNG_POLY_SEL_PHASE_NAME	"kleinjung-poly-selection-phase"

#define XML_SERIAL_INNER_STATE_NAME "inner-state"
#define XML_SERIAL_ITERATION_NAME	"iteration"
#define XML_SERIAL_INDEPENDENT_CYCLE_NAME	"independent-cycle"
#define XML_SERIAL_EXPECTED_VERTICES_NAME	"expected-vertices"
#define XML_SERIAL_DEPENDENCY_INDEX_NAME	"dependency-index"
#define XML_SERIAL_FACTOR_1_NAME	"factor-1"
#define XML_SERIAL_FACTOR_2_NAME	"factor-2"
#define XML_SERIAL_SPECIAL_Q_NAME	"special-q"
#define XML_SERIAL_ROOT_INDEX_NAME	"root-index"

#define XML_SERIAL_ROOT_DATA_NAME		"root-data"
#define XML_SERIAL_PRIMITIVE_ROOTS_NAME	"primitive-roots"
#define XML_SERIAL_POWERS_NAME			"powers"
#define XML_SERIAL_PRIMITIVE_ROOT_NAME	"primitive-root"
#define XML_SERIAL_POWER_NAME			"power"

#define XML_SERIAL_FOUND_CAND_POLY_NAME		"found-candidate-poly"
#define XML_SERIAL_RATED_CAND_POLY_NAME		"rated-candidate-poly"
#define XML_SERIAL_C_PRIMES_NAME			"c-primes"
#define XML_SERIAL_C_PRIMES_COUNT_NAME		"c-primes-count"
#define XML_SERIAL_CANDIDATE_POLYS_NAME		"candidate-polynomials"
#define XML_SERIAL_ALPHA_NAME				"alpha"
#define XML_SERIAL_ROOT_NAME				"root"
#define XML_SERIAL_RATING_NAME				"rating"
#define XML_SERIAL_SKEWNESS_NAME			"skewness"
#define XML_SERIAL_CANDIDATE_POLY_PAIRS_NAME	"candidate-polynomial-pairs"
#define XML_SERIAL_FIRST_POLY_NAME			"first-poly"
#define XML_SERIAL_SECOND_POLY_NAME			"second-poly"
#define XML_SERIAL_ROUND_NAME				"round"
#define XML_SERIAL_MONIC_M_NAME				"monic-m"
#define XML_SERIAL_RUNNING_AD				"running-ad"

#define XML_SERIAL_PRIME_ATTR "prime"
#define XML_SERIAL_C_P_ATTR "c_p"
#define XML_SERIAL_FLAGS_ATTR "flags"
#define XML_SERIAL_EXPONENT_ATTR "exponent"
#define XML_SERIAL_PARTS_COUNT_ATTR "parts-count"
 
#define XML_SERIAL_ROOT_ATTR "root"
#define XML_SERIAL_INVERSION_OF_ROOT_ATTR "inversion-of-root"
#define XML_SERIAL_M_ATTR "m"
#define XML_SERIAL_INVERSION_OF_M_ATTR "inversion-of-m"
#define XML_SERIAL_SIEVING_INDEX_ATTR "sieving-index"
#define XML_SERIAL_LOG_ATTR "log"

#define XML_SERIAL_NUMBER_ATTR "root"
#define XML_SERIAL_POWER_ATTR "power"

#define XML_SERIAL_INVERSION_OF_C_P_ATTR "inversion-of-c_p"

#define XML_SERIAL_TYPE_ATTR	"type"
#define XML_SERIAL_COUNT_ATTR	"count"
#define XML_SERIAL_VALUE_ATTR	"value"
#define XML_SERIAL_BASE_ATTR	"base"
#define XML_SERIAL_ANCESTOR_ATTR "ancestor"

#define XML_SERIAL_SIEVING_ELEMENT_TYPE				"sieving-element"
#define XML_SERIAL_INTEGER_FOR_SIEVING_TYPE			"integer-for-sieving"
#define XML_SERIAL_PRIME_IDEAL_FOR_SIEVING_TYPE		"prime-ideal-for-sieving"
#define XML_SERIAL_PRIME_IDEAL_FOR_LEGENDRE_TYPE	"prime-ideal-for-legendre"
#define XML_SERIAL_HASHTABLE_ENTRY_TYPE1_TYPE		"hashtable-entry-type1"
#define XML_SERIAL_MPF_TYPE							"mpf"

// ----- generationC - serialization
#define XML_SERIAL_GENERATION_C_NAME "generation-c"
#define XML_SERIAL_FIRST_BIGGER_NAME "first-bigger"
#define XML_SERIAL_COUNT_BIGGER_NAME "count-bigger"
#define XML_SERIAL_MAX_PRIME_NAME "max-prime"
#define XML_SERIAL_BASE_NAME "base"
#define XML_SERIAL_UPPER_BOUND_NAME "upper-bound"
#define XML_SERIAL_LOWER_BOUND_NAME "lower-bound"
#define XML_SERIAL_SEQUENCE_NAME "sequence"
#define XML_SERIAL_INDEX_NAME "index"
#define XML_SERIAL_NEXT_PRIME_NAME "next-prime"
#define XML_SERIAL_SEQUENCE_B_NAME "sequence-bigger"
#define XML_SERIAL_INDEX_B_NAME "index-bigger"
#define XML_SERIAL_NEXT_PRIME_B_NAME "next-bigger"
#define XML_SERIAL_BEGIN_NAME "is-begin"
#define XML_SERIAL_SMALL_NAME "is-small"
#define XML_SERIAL_RESULT_PRODUCT_NAME "result-product"
//------ end of generationC - serialization


// File names for serialization

#define FILE_NAME_MSERIALIZABLE "serialization"
#define FILE_NAME_BASE_PARAMETERS "base_parameters"
#define FILE_NAME_NFS_PARAMETERS "nfs_parameters"
#define FILE_NAME_POLYNOMIAL "polynomial"
#define FILE_NAME_SPARSE_MATRIX "sparse_matrix"
#define FILE_NAME_INTEGER_SPARSE_MATRIX "integer_sparse_matrix"
#define FILE_NAME_BIT_MATRIX "bit_matrix"
#define FILE_NAME_BC_MATRIX "bc_matrix"
#define FILE_NAME_HERMITE_MATRIX "hermite_matrix"
#define FILE_NAME_INTEGER_MATRIX "integer_matrix"
#define FILE_NAME_NORMAL_MATRIX "normal_matrix"
#define FILE_NAME_POLY_PHASE "poly_phase"
#define FILE_NAME_SIEVING_PHASE "sieving_phase"
#define FILE_NAME_REL_PRO_PHASE "rel_processing_phase"
#define FILE_NAME_LINEAR_PHASE "linear_phase"
#define FILE_NAME_SQUARE_ROOT_PHASE "square_root_phase"
#define FILE_NAME_NFS "nfs"
#define FILE_NAME_FULL_PRIMES "primes.xml"
#define FILE_NAME_FULL_PRIMES_TEMP "primes_temp.xml"
#define FILE_NAME_RELATION_PART "relation_part"
#define FILE_NAME_NFS_RELATION "nfs_relation"
#define FILE_NAME_INTEGRAL_FB "integral_factor_base"
#define FILE_NAME_ALGEBRAIC_FB "algebraic_factor_base"
#define FILE_NAME_QUAD_CHARS "qaud_characters"
#define FILE_NAME_MATRIX "matrix"
#define FILE_NAME_RELATIONS "relations"
#define FILE_NAME_SMOOTH_RELATIONS "smooth_relations"
#define FILE_NAME_SMOOTH_LEG_REL "smooth_legendre_relations"
#define FILE_NAME_USABLE_PART_REL "usable_partial_relations"
#define FILE_NAME_TEMP_FILE "temp_file"
#define FILE_NAME_PARTIAL_REL "partial_relations"
#define FILE_NAME_ALL_PARTIAL_REL "all_partial_relations"
#define FILE_NAME_MATRIX_RESULT "matrix_result"
#define FILE_NAME_RELATION_MATRIX "relation_matrix"
#define FILE_NAME_RELATIONS_RESULT "relations_result"


// Parameters Phases

#define PARAM_NFS_TYPE "nfs-type"
#define PARAM_WORKING_DIRECTORY "working-directory"
#define PARAM_DESERIALIZE "deserialize"
#define PARAM_POLY_PHASE_NUMBER "poly-phase-nbr"
#define PARAM_SIEVING_PHASE_NUMBER "sieving-phase-nbr"
#define PARAM_REL_PROCESSING_PHASE_NUMBER "rel-processing-phase-nbr"
#define PARAM_LINEAR_PHASE_NUMBER "linear-phase-nbr"
#define PARAM_SQUARE_ROOT_PHASE_NUMBER "square-root-phase-nbr"
#define PARAM_POLY_PHASE_FULL_FILENAME "poly-phase-full-filename"
#define PARAM_POLY_PHASE_FILENAME "poly-phase-filename"
#define PARAM_POLY_PHASE_DIRECTORY "poly-phase-directory"
#define PARAM_POLY_PHASE_COMPRESSION "poly-phase-compression"
#define PARAM_SIEVING_PHASE_FULL_FILENAME "sieving-phase-full-filename"
#define PARAM_SIEVING_PHASE_FILENAME "sieving-phase-filename"
#define PARAM_SIEVING_PHASE_DIRECTORY "sieving-phase-directory"
#define PARAM_SIEVING_PHASE_COMPRESSION "sieving-phase-compression"
#define PARAM_REL_PROCESSING_PHASE_FULL_FILENAME "rel-processing-phase-full-filename"
#define PARAM_REL_PROCESSING_PHASE_FILENAME "rel-processing-phase-filename"
#define PARAM_REL_PROCESSING_PHASE_DIRECTORY "rel-processing-phase-directory"
#define PARAM_REL_PROCESSING_PHASE_COMPRESSION "rel-processing-phase-compression"
#define PARAM_LINEAR_PHASE_FULL_FILENAME "linear-phase-full-filename"
#define PARAM_LINEAR_PHASE_FILENAME "linear-phase-filename"
#define PARAM_LINEAR_PHASE_DIRECTORY "linear-phase-directory"
#define PARAM_LINEAR_PHASE_COMPRESSION "linear-phase-compression"
#define PARAM_SQUARE_ROOT_PHASE_FULL_FILENAME "square-root-phase-full-filename"
#define PARAM_SQUARE_ROOT_PHASE_FILENAME "square-root-phase-filename"
#define PARAM_SQUARE_ROOT_PHASE_DIRECTORY "square-root-phase-directory"
#define PARAM_SQUARE_ROOT_PHASE_COMPRESSION "square-root-phase-compression"
#define PARAM_INFO_MODE "info-mode"
#define PARAM_ASSERTION_MODE "assertion-mode"
#define PARAM_MATRIX_RESULT_FULL_FILENAME "matrix-result-full-filename"
#define PARAM_MATRIX_RESULT_FILENAME "matrix-result-filename"
#define PARAM_MATRIX_RESULT_DIRECTORY "matrix-result-directory"
#define PARAM_MATRIX_RESULT_COMPRESSION "matrix-result-compression"
#define PARAM_MATRIX_RESULT_TYPE "matrix-result-type"
#define PARAM_RELATION_MATRIX_FULL_FILENAME "relation-matrix-full-filename"
#define PARAM_RELATION_MATRIX_FILENAME "relation-matrix-filename"
#define PARAM_RELATION_MATRIX_DIRECTORY "relation-matrix-directory"
#define PARAM_RELATION_MATRIX_COMPRESSION "relation-matrix-compression"
#define PARAM_SMOOTH_RELATIONS_FULL_FILENAME "smooth-relations-full-filename"
#define PARAM_SMOOTH_RELATIONS_FILENAME "smooth-relations-filename"
#define PARAM_SMOOTH_RELATIONS_DIRECTORY "smooth-relations-directory"
#define PARAM_SMOOTH_RELATIONS_COMPRESSION "smooth-relations-compression"
#define PARAM_SMOOTH_LEG_REL_FULL_FILENAME "smooth-rel-full-filename"
#define PARAM_SMOOTH_LEG_REL_FILENAME "smooth-rel-filename"
#define PARAM_SMOOTH_LEG_REL_DIRECTORY "smooth-rel-directory"
#define PARAM_SMOOTH_LEG_REL_COMPRESSION "smooth-rel-compression"
#define PARAM_TEMP_FILE_FULL_FILENAME "temp-file-full-filename"
#define PARAM_TEMP_FILE_FILENAME "temp-file-filename"
#define PARAM_TEMP_FILE_DIRECTORY "temp-file-directory"
#define PARAM_TEMP_FILE_COMPRESSION "temp-file-compression"
#define PARAM_PARTIAL_REL_FULL_FILENAME "partial-rel-full-filename"
#define PARAM_PARTIAL_REL_FILENAME "partial-rel-filename"
#define PARAM_PARTIAL_REL_DIRECTORY "partial-rel-directory"
#define PARAM_PARTIAL_REL_COMPRESSION "partial-rel-compression"
#define PARAM_USABLE_PART_REL_FULL_FILENAME "usable-part-rel-full-filename"
#define PARAM_USABLE_PART_REL_FILENAME "usable-part-rel-filename"
#define PARAM_USABLE_PART_REL_DIRECTORY "usable-part-rel-directory"
#define PARAM_USABLE_PART_REL_COMPRESSION "usable-part-rel-compression"
#define PARAM_ALL_PARTIAL_REL_FULL_FILENAME "all-part-relations-full-filename"
#define PARAM_ALL_PARTIAL_REL_FILENAME "all-part-relations-filename"
#define PARAM_ALL_PARTIAL_REL_DIRECTORY "all-part-relations-directory"
#define PARAM_ALL_PARTIAL_REL_COMPRESSION "all-part-relations-compression"
#define PARAM_INTEGRAL_FB_FULL_FILENAME "integral-fb-full-filename"
#define PARAM_INTEGRAL_FB_FILENAME "integral-fb-filename"
#define PARAM_INTEGRAL_FB_DIRECTORY "integral-fb-directory"
#define PARAM_INTEGRAL_FB_COMPRESSION "integral-fb-compression"
#define PARAM_ALGEBRAIC_FB_FULL_FILENAME "algebraic-fb-full-filename"
#define PARAM_ALGEBRAIC_FB_FILENAME "algebraic-fb-filename"
#define PARAM_ALGEBRAIC_FB_DIRECTORY "algebraic-fb-directory"
#define PARAM_ALGEBRAIC_FB_COMPRESSION "algebraic-fb-compression"
#define PARAM_QUAD_CHARS_FULL_FILENAME "qaud-chars-full-filename"
#define PARAM_QUAD_CHARS_FILENAME "qaud-chars-filename"
#define PARAM_QUAD_CHARS_DIRECTORY "qaud-chars-directory"
#define PARAM_QUAD_CHARS_COMPRESSION "qaud-chars-compression"
#define PARAM_QUAD_CHARS_COUNT "quad-charrs-count"
#define PARAM_CHECK_RESULT "check-result"
#define PARAM_RANK_CALC_MODE "rank-calc-mode"
#define PARAM_DENSITY_MODE "density-mode"
#define PARAM_TIME_MESSAGE_MODE "time-message-mode"
#define PARAM_MATRIX_SOLVER_TYPE "matrix-solver-type"
#define PARAM_RELATION_MATRIX_TYPE "relation-matrix-type"
#define PARAM_LANCZOS_AUX_MATRIX_TYPE "lanczos-aux-matrix-type"
#define PARAM_CURRENT_PHASE "current-phase"
#define PARAM_MAX_PHASE_NBR "max-phase-nbr"
#define PARAM_MIN_PHASE_NBR "min-phase-nbr"
#define PARAM_ROOT_M "root-m"
#define PARAM_NUMBER_N "number-n"
#define PARAM_SIEVING_POLY "sieving-poly"
#define PARAM_ROOT_FINDER_TYPE "root-finder-type"
#define PARAM_INTEGRAL_FB_BOUND "integral-fb-bound"
#define PARAM_PARTIAL_REL_GRAPH_VERTICES "partial-rel-graph-vertices"
#define PARAM_PARTIAL_REL_GRAPH_COMPONENTS "partial-rel-graph-components"
#define PARAM_PARTIAL_REL_GRAPH_EDGES "partial-rel-graph-edges"
#define PARAM_SMOOTH_REL_COUNT "smooth-relation-count"
#define PARAM_VARIATIONS_TYPE "variations-type"


#define DESERIALIZE_NFS_PARAMETERS "control-file"

// WriteReadMutex

#define DEFAULT_WRMUTEX_MAX_READERS 5
//
// Serialization - Default values

#define DEFAULT_COMPRESSION_LEVEL 0


#define LIST_LOAD_FLAG 0x800000
#define COMPRESSED_FILE_EXTENSION ".z"


#define DEFAULT_CONVERT_ENDIAN_TYPE ELittleEndian
#define DEFAULT_CONVERT_ENDIAN_TYPE_STRING LITTLE_ENDIAN_TYPE_STRING
//
// Polynomial

#define POLY_SEPARATOR "|"
#define POLY_DEGREE_SEPARATOR_LEFT "("
#define POLY_DEGREE_SEPARATOR_RIGHT ")"


#define DEFAULT_FB_BOUND        10000
// Supporting factor alg
#define DEFAULT_FB_PRIME_BOUND 40000
// CFRAC
#define DEFAULT_CFRAC_BASE_SIZE 250

// distribution machine type
# define DEFAULT_DISTRIBUTION_MACHINE_TYPE (int)ENone

#endif
