#ifndef __UTILS_H__
#define __UTILS_H__

#include "aux_structures.h"
// #include "structures.h"
#include <string.h>
#include <string>
#include <stdlib.h>
#include "gmp.h"
#include "constants.h"

// Forward declaration
class Polynomial;

/*!
 * \brief Various utilities common for NFS and QS
 */
class Utils
{
public:
	static void Wait();

	/// Prints \a aMessage and optionally \a aNumber.
	static void NormalMessage(const char* aMessage, int aNumber = NO_NUMBER);
	/// Prints information about an error
	static void ErrorMessage(error_context* aContext);
	/// Prints information about an error
	static void ErrorMessage(const char* aProcess, const char* aProblem, int aErrorCode, int aLabel, const char* aSourceFile, const char* aExplanation);
	/// Prints information about an error
	static void ErrorMessage(int aLineNumber, const char* aSourceFile, const char* aProcess, const char* aProblem, int aErrorCode, int aLabel, const char* aExplanation);

	static void* AllocationMachine(void** aPointer, long aSize);
	static void* AllocationMachine(void** aPointer, long aSize, error_context* aContext);
	static void TimeFormat(int aSeconds);

	static int CreateDirectory(std::string aPath);

	static std::string GetDirectoryFromPath(const char* aPath);
	static int PrintMpzToFile(FILE* aFile, int base, mpz_t aValue);
	static int WriteDataToFile(std::string aFileName, void * aData, long aCount, int aSize);
	static int ReadDataFromFile(std::string aFileName, void ** aData, long& aCount, int& aSize);

	// Convert stuff
	static std::string ConvertToString(bool aValue);
	static std::string ConvertToString(int aValue);
	static std::string ConvertToString(long aValue);
	static std::string ConvertToString(double aValue);
	static std::string ConvertToString(const mpz_t aValue);


	// math stuff

	static double Ln(mpf_t aArg);
	static double Round(double aArg);
	static int GetClosestPowerOf2(long & aResult, long aValue);
	static int GetClosestBiggerPowerOf2(long &aResult, long aValue);
	static int CompareLongs(const void* aArg1, const void* aArg2);
	static int CompareInts(const void* aArg1, const void* aArg2);
	static int CompareDoubles(const void* aArg1, const void* aArg2);
//	static int IncorrectLegendre(int c_p, int prime, Polynomial* polynomial);
	static int GCD(const main_sieving_type &aX, const main_sieving_type &aY, main_sieving_type &aResult);
	static int LGCD(const long &aX, const long &aY, long &aResult);
	///Like gmp_nextprime
	static int PreviousPrime(mpz_t rop, mpz_t op);
	/*!
		Tonilli-Shanks algorithm for square root modulo p
		more Cohen - A Course in Computational Algebraic Number Theory - algorithm 1.5.1
	*/
	static int TonelliShanks(mpz_t x, mpz_t a, mpz_t p);
	static relation_index_type GetSuitableHashtableSize(relation_index_type aAmount);

	// combinatorics stuff

//=======================================================
// combination.h
// Description : Template class to find combinations
//=======================================================
// Copyright 2003 - 2006 Wong Shao Voon
// No warranty, implied or expressed, is included.
// Author is not liable for any type of loss through 
// the use of this source code. Use it at your own risk!
//=======================================================
	// Non recursive template function
	template <class BidIt>
	static bool NextCombination(BidIt n_begin, BidIt n_end, BidIt r_begin, BidIt r_end)
	{
		bool boolmarked = false;
		BidIt r_marked;

		BidIt n_it1 = n_end;
		--n_it1;


		BidIt tmp_r_end = r_end;
		--tmp_r_end;

		for (BidIt r_it1 = tmp_r_end; r_it1 != r_begin || r_it1 == r_begin; --r_it1, --n_it1)
		{
			if (*r_it1 == *n_it1)
			{
				if (r_it1 != r_begin) //to ensure not at the start of r sequence
				{
					boolmarked = true;
					r_marked = (--r_it1);
					++r_it1; //add it back again 
					continue;
				}
				else // it means it is at the start the sequence, so return false
				{
					return false;
				}
			}
			else //if(*r_it1!=*n_it1 )
			{
				//marked code
				if (boolmarked == true)
				{
					//for loop to find which marked is in the first sequence
					BidIt n_marked;//mark in first sequence
					for (BidIt n_it2 = n_begin; n_it2 != n_end; ++n_it2)
					{
						if (*r_marked == *n_it2) 
						{
							n_marked = n_it2;
							break;
						}
					}

					BidIt n_it3 = ++n_marked;    
					for (BidIt r_it2 = r_marked; r_it2 != r_end; ++r_it2, ++n_it3)
					{
						*r_it2 = *n_it3;
					}
					
					return true;
				}

				for (BidIt n_it4 = n_begin; n_it4 != n_end; ++n_it4)
				{
					if (*r_it1 == *n_it4)
					{
						*r_it1 = *(++n_it4);
						return true;           
					}
				}
			}
		}  

		return true; //will never reach here    
	}
};

#endif

