#include "common_xml_helper.h"
#include <string.h>
#include "abstract_sieve.h"
#include "common_utils.h"
#include "log.h"

const char * CommonXMLHelper::HEX_NUMBERS = "0123456789ABCDEF";

int CommonXMLHelper::MoveToNextNode(xmlTextReaderPtr& aReader, int aMinDepth, int aMaxDepth)
{
	int type = -1;
	int rc;
	//int depth = aMinDepth + 1;
	//int minDepth = depth;
	int depth;
	while (type != 1 || aMaxDepth <= depth)
	{
		rc = xmlTextReaderRead(aReader);
		if (rc != 1)
		{
			if (rc == 0)
			{
				return ConstRC::EndOfFile;
			}
			CLog::LogError(WHEREARG,"XML Helper","MoveToNextNode",rc,0,"Cannot read node.");
			return ConstRC::GeneralError;  
		}
		if ((depth = xmlTextReaderDepth(aReader)) <= aMinDepth)
		{
			return ConstRC::IncorrectDepth;
		}
		//minDepth = min(xmlTextReaderDepth(aReader), depth);
		type = xmlTextReaderNodeType(aReader);
	}

	//if (minDepth <= aMinDepth)
	//{
	//	return ConstRC::IncorrectDepth;
	//}

	return ConstRC::Ok;
}

int CommonXMLHelper::ParseDecimalDouble(xmlChar* aString, double& aTarget)
{
	if (aString == NULL) return ConstRC::GeneralError;
	int len = xmlStrlen(aString);
	if (!len) return ConstRC::GeneralError;
	aTarget = 0;
	bool negative = (aString[0] == '-');
	int i,start = 0;
	if (negative)
	{
		start = 1;
	}
	xmlChar char1;
	//printf("XML Helper, parse decimal float. Input string is %s of length %d, start is %d.\n",aString,len,start);
	int decimal_point_position = -1;
	for (i=start; i< len; i++)
	{
		//printf("For cycle, i = %d.\n",i);
		char1 = aString[i];
		//printf("At index %d is char %c.\n",i,char1);
		if (char1 == '.')
		{
			decimal_point_position = (len-1) - i;
			//printf("Found decimal point, used pos is %d.\n",decimal_point_position);
			continue;
		}
		if (! xmlIsDigit_ch(char1))
		{
			printf("Parse decimal double: char %c on index %d of %s is not a digit!\n",char1,i,aString);
			return ConstRC::GeneralError;
		}
		int diff = char1 - '0';
		aTarget *= 10;
		aTarget += diff;
		//printf("Current intermediate result is %f.\n",aTarget);
	}
	if (negative)
	{
		aTarget *= -1;
	}
	if (decimal_point_position != -1)
	{
		for (i=0; i< decimal_point_position; i++)
		{
			aTarget /= 10;
		}
	}
	//printf("Returning %f.\n",aTarget);
	return ConstRC::Ok;
}

int CommonXMLHelper::ParseDecimalMpz(xmlChar* aString, mpz_t aTarget)
{
	if (aString == NULL) return ConstRC::GeneralError;
	int len = xmlStrlen(aString);
	if (!len) return ConstRC::GeneralError;
	mpz_set_ui(aTarget,0);
	bool negative = (aString[0] == '-');
	int i,start = 0 + 1*(negative == TRUE);
	xmlChar char1;
	//printf("XML Helper, parse decimal mpz. Input string is %s of length %d.\n",aString,len);
	for (i=start; i< len;i++)
	{
		char1 = aString[i];
		//printf("At index %d is char %c.\n",i,char1);
		if (! xmlIsDigit_ch(char1))
		{
			printf("Parse decimal mpz: char %c on index %d of %s is not a digit!\n",char1,i,aString);
			return ConstRC::GeneralError;
		}
		int diff = char1 - '0';
		mpz_mul_ui(aTarget,aTarget,10);
		mpz_add_ui(aTarget,aTarget,diff);
		//gmp_printf("Current intermediate result is %Zd.\n",aTarget);
	}
	if (negative)
	{
		mpz_mul_si(aTarget,aTarget,-1);
	}
	//gmp_printf("Returning %Zd.\n",aTarget);
	return ConstRC::Ok;
}

int CommonXMLHelper::ParseDecimalInt(xmlChar* aString, int& aTarget)
{
	if (aString == NULL) return ConstRC::GeneralError;
	int len = xmlStrlen(aString);
	if (!len) return ConstRC::GeneralError;
	aTarget = 0;
	bool negative = (aString[0] == '-');
	int i,start = 0 + 1*(negative == TRUE);
	xmlChar char1;
// 	printf("XML Helper, parse decimal int. Input string is %s of length %d.\n",aString,len);
	for (i=start; i< len; i++)
	{
		char1 = aString[i];
// 		printf("At index %d is char %c.\n",i,char1);
		if (! xmlIsDigit_ch(char1))
		{
			printf("Parse decimal int: char %c on index %d of %s is not a digit!\n",char1,i,aString);
			return ConstRC::GeneralError;
		}
		int diff = char1 - '0';
		aTarget *= 10;
		aTarget += diff;
// 		printf("Current intermediate result is %d.\n",aTarget);
	}
	if (negative)
	{
		aTarget *= -1;
	}
// 	printf("Returning %d.\n",aTarget);
	return ConstRC::Ok;
}
int CommonXMLHelper::ParseDecimalUnsignedIntOrLong(xmlChar* aString, unsigned long& aTarget)
{
	if (aString == NULL) return ConstRC::GeneralError;
	int len = xmlStrlen(aString);
	if (!len) return ConstRC::GeneralError;
	aTarget = 0;
	int i,start = 0;
	xmlChar char1;
// 	printf("XML Helper, parse decimal long. Input string is %s of length %d.\n",aString,len);
	for (i=start; i< len; i++)
	{
		char1 = aString[i];
// 		printf("At index %d is char %c.\n",i,char1);
		if (! xmlIsDigit_ch(char1))
		{
			printf("Parse decimal long: char %c on index %d of %s is not a digit!\n",char1,i,aString);
			return ConstRC::GeneralError;
		}
		int diff = char1 - '0';
		aTarget *= 10;
		aTarget += diff;
// 		printf("Current intermediate result is %ld.\n",aTarget);
	}
// 	printf("Returning %ld.\n",aTarget);
	return ConstRC::Ok;
}


int CommonXMLHelper::ParseDecimalUnsignedIntOrLong(xmlChar* aString, unsigned int& aTarget)
{
	if (aString == NULL) return ConstRC::GeneralError;
	int len = xmlStrlen(aString);
	if (!len) return ConstRC::GeneralError;
	aTarget = 0;
	int i,start = 0;
	xmlChar char1;
// 	printf("XML Helper, parse decimal int. Input string is %s of length %d.\n",aString,len);
	for (i=start; i< len; i++)
	{
		char1 = aString[i];
// 		printf("At index %d is char %c.\n",i,char1);
		if (! xmlIsDigit_ch(char1))
		{
			printf("Parse decimal unsigned int: char %c on index %d of %s is not a digit!\n",char1,i,aString);
			return ConstRC::GeneralError;
		}
		int diff = char1 - '0';
		aTarget *= 10;
		aTarget += diff;
// 		printf("Current intermediate result is %d.\n",aTarget);
	}
// 	printf("Returning %d.\n",aTarget);
	return ConstRC::Ok;
}

int CommonXMLHelper::ParseDecimalLong(xmlChar* aString, long& aTarget)
{
	if (aString == NULL) return ConstRC::GeneralError;
	int len = xmlStrlen(aString);
	if (!len) return ConstRC::GeneralError;
	aTarget = 0;
	bool negative = (aString[0] == '-');
	int i,start = 0 + 1*(negative == TRUE);
	xmlChar char1;
// 	printf("XML Helper, parse decimal long. Input string is %s of length %d.\n",aString,len);
	for (i=start; i< len; i++)
	{
		char1 = aString[i];
// 		printf("At index %d is char %c.\n",i,char1);
		if (! xmlIsDigit_ch(char1))
		{
			printf("Parse decimal long: char %c on index %d of %s is not a digit!\n",char1,i,aString);
			return ConstRC::GeneralError;
		}
		int diff = char1 - '0';
		aTarget *= 10;
		aTarget += diff;
// 		printf("Current intermediate result is %ld.\n",aTarget);
	}
	if (negative)
	{
		aTarget *= -1;
	}
// 	printf("Returning %ld.\n",aTarget);
	return ConstRC::Ok;
}

int CommonXMLHelper::NewLine(xmlTextWriterPtr& aWriter, bool aIndent)
{
	int rc =  xmlTextWriterWriteString(aWriter, BAD_CAST "\n");
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","New Line",rc,0,"Could not write new line.");
        	return ConstRC::GeneralError;
    	}
	return ConstRC::Ok;
}

int CommonXMLHelper::DeserializeTime(xmlTextReaderPtr& aReader, const char* aMessage, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize time",0,0,"No attributes of time, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* sec = xmlTextReaderGetAttribute(aReader, BAD_CAST SECONDS);
	if (!sec)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize time",0,0,"No seconds attribute, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* out = xmlTextReaderGetAttribute(aReader, BAD_CAST FORMATTED);
	if (!out)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize time",0,0,"No formatted string specified");
		int decimalTime=0;
		int res = ParseDecimalInt(sec,decimalTime);
		if (res == ConstRC::Ok)
		{
			time_t what = (time_t) decimalTime;
			out = BAD_CAST ctime(&what);
		}
	}
	if (aInfoMode)
	{
		printf("%s: Time %s.\n",aMessage,out);
	}
	xmlFree(sec);
	xmlFree(out);
	return ConstRC::Ok;
}

int CommonXMLHelper::SerializeTime(xmlTextWriterPtr& aWriter)
{
	return SerializeTime(aWriter, TIME_NAME, time(NULL));
}

int CommonXMLHelper::SerializeTime(xmlTextWriterPtr& aWriter, const char* aTag, time_t aTime)
{
	char*  formattedTime = ctime(&aTime);
	int len = strlen(formattedTime);
	// At some Unices, formatted time is ended by new line. We do not want this, it looks ugly.
	if (formattedTime[len-1] == '\n')
	{
		formattedTime[len-1] = 0;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTag);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Time",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST SECONDS,"%ld",aTime);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Time",rc,0,"Could not write the serializator name.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST FORMATTED,"%s",BAD_CAST formattedTime);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Time",rc,0,"Could not write the serializator version.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Time",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int CommonXMLHelper::DeserializeMpz(xmlTextReaderPtr& aReader, const char* aMessage, mpz_t aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpz",0,0,"No attributes of mpz, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpz",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!(type = xmlStrcat(type,BAD_CAST MPZ)))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpz",0,0,"Data type not mpz, aborting");
		xmlFree(type);
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpz",0,0,"Value not found, aborting");
		xmlFree(type);
		return ConstRC::GeneralError;
	}
	int base = 10;
	xmlChar* baseChar = xmlTextReaderGetAttribute(aReader, BAD_CAST BASE);
	if (baseChar)
	{
		int rc = ParseDecimalInt(baseChar,base);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpz",0,0,"Could not parse base int");
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;
		}
	}
	if (base == 10)
	{
		int rc = ParseDecimalMpz(value,aTarget);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpz",0,0,"Could not parse value mpz");
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;
		}
	}
	else {
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpz",0,0,"Cannot parse nondecimal mpz's yet");
		xmlFree(type);
		xmlFree(value);
		xmlFree(baseChar);
		return ConstRC::GeneralError;
	}
	if (aInfoMode)
	{
		xmlChar* name = xmlTextReaderName(aReader);
		int decLen = mpz_sizeinbase(aTarget,10);
		int binLen = mpz_sizeinbase(aTarget,2);
		printf("%s: Value of %s is %s (%d dec, %d bin digits).\n",aMessage,name,value,decLen,binLen);
		xmlFree(name);
	}
	xmlFree(type);
	xmlFree(value);
	xmlFree(baseChar);
	return ConstRC::Ok;
}

int CommonXMLHelper::DeserializeSignedInt(xmlTextReaderPtr& aReader, const char* aMessage, int& aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Int",0,0,"No attributes of mpz, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Int",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!(type = xmlStrcat(type,BAD_CAST SIGNED_INT)))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Int",0,0,"Data type not mpz, aborting");
		xmlFree(type);
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Int",0,0,"Value not found, aborting");
		xmlFree(type);
		return ConstRC::GeneralError;
	}
	int base = 10;
	xmlChar* baseChar = xmlTextReaderGetAttribute(aReader, BAD_CAST BASE);
	if (baseChar)
	{
		int rc = ParseDecimalInt(baseChar,base);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Int",0,0,"Could not parse base int");
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;
		}
	}
	if (base == 10)
	{
		int rc = ParseDecimalInt(value,aTarget);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Int",0,0,"Could not parse value");
			xmlFree(value);
			xmlFree(type);
			xmlFree(baseChar);
		return ConstRC::GeneralError;
		}
	}
	else {
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Int",0,0,"Cannot parse nondecimal integers yet");
		xmlFree(value);
		xmlFree(type);
		xmlFree(baseChar);
		return ConstRC::GeneralError;
	}
	if (aInfoMode)
	{
		xmlChar* name = xmlTextReaderName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
		xmlFree(name);
	}
	xmlFree(type);
	xmlFree(value);
	xmlFree(baseChar);
	return ConstRC::Ok;
}

int CommonXMLHelper::DeserializeUnsignedLong(xmlTextReaderPtr& aReader, const char* aMessage, unsigned long& aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Long",0,0,"No attributes of unsigned long, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Long",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!xmlStrEqual(type,BAD_CAST UNSIGNED_LONG))
	{
		xmlFree(type);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Long",0,0,"Data type not unsigned long, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);  
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Long",0,0,"Value not found, aborting");			
		xmlFree(type);
		return ConstRC::GeneralError;	
	}	
	int base = 10;
	xmlChar* baseChar = xmlTextReaderGetAttribute(aReader, BAD_CAST BASE);  
	if (baseChar)
	{
		int rc = ParseDecimalInt(baseChar,base);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Long",0,0,"Could not parse base int");			
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;	
		}
	}
	if (base == 10)
	{
		int rc = ParseDecimalUnsignedIntOrLong(value,aTarget);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Long",0,0,"Could not parse value");			
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;	
		}
	} 
	else 
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Long",0,0,"Cannot parse nondecimal longs yet");			
		xmlFree(type);
		xmlFree(value);
		xmlFree(baseChar);
		return ConstRC::GeneralError;	
	}
	if (aInfoMode)
	{
		const xmlChar* name = xmlTextReaderConstName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
	}
	xmlFree(type);
	xmlFree(value);
	xmlFree(baseChar);
	return ConstRC::Ok;
}
int CommonXMLHelper::SerializeUnsignedLong(xmlTextWriterPtr& aWriter, const char* aTagName, /*const char* aAttributeName,*/ unsigned long aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",0,0,"Insufficient data.");
			return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
		if (rc < 0) {
			CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",rc,0,"Could not write the serializator tag.");
			return ConstRC::GeneralError;
		}	
	/*rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST NAME ,"%s",BAD_CAST aAttributeName);
	if (rc < 0) {
			CLog::LogError(WHEREARG,"XML Helper","Serialize mpz",rc,0,"Could not write the name.");
			return ConstRC::GeneralError;
		}*/
	int base = 10;
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%ld",aValue);
	
	if (rc < 0) {
			CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",rc,0,"Could not write the value.");
			return ConstRC::GeneralError;
		}	
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST BASE,"%d",base);	
	if (rc < 0) {
			CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",rc,0,"Could not write the base.");
			return ConstRC::GeneralError;
		}	
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST UNSIGNED_LONG);
	if (rc < 0) {
			CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",rc,0,"Could not write the type.");
			return ConstRC::GeneralError;
		}
	rc = xmlTextWriterEndElement(aWriter);
		if (rc < 0) {
			CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",rc,0,"Could not end the tag.");
			return ConstRC::GeneralError;
		}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}
int CommonXMLHelper::DeserializeUnsignedInt(xmlTextReaderPtr& aReader, const char* aMessage, unsigned int& aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Int",0,0,"No attributes of mpz, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Int",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!(type = xmlStrcat(type,BAD_CAST UNSIGNED_INT)))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Int",0,0,"Data type not mpz, aborting");
		xmlFree(type);
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Int",0,0,"Value not found, aborting");
		xmlFree(type);
		return ConstRC::GeneralError;
	}
	int base = 10;
	xmlChar* baseChar = xmlTextReaderGetAttribute(aReader, BAD_CAST BASE);
	if (baseChar)
	{
		int rc = ParseDecimalInt(baseChar,base);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Int",0,0,"Could not parse base int");
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;
		}
	}
	if (base == 10)
	{
		int rc = ParseDecimalUnsignedIntOrLong(value,aTarget);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Int",0,0,"Could not parse value");
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;
		}
	}
	else {
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Int",0,0,"Cannot parse nondecimal integers yet");
		xmlFree(type);
		xmlFree(value);
		xmlFree(baseChar);
		return ConstRC::GeneralError;
	}
	if (aInfoMode)
	{
		xmlChar* name = xmlTextReaderName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
	}
	xmlFree(type);
	xmlFree(value);
	xmlFree(baseChar);
	return ConstRC::Ok;
}

int CommonXMLHelper::DeserializeSignedLong(xmlTextReaderPtr& aReader, const char* aMessage, long& aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Long",0,0,"No attributes of mpz, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Long",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!(type = xmlStrcat(type,BAD_CAST SIGNED_INT)))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Long",0,0,"Data type not mpz, aborting");
		xmlFree(type);
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Long",0,0,"Value not found, aborting");
		xmlFree(type);
		return ConstRC::GeneralError;
	}
	int base = 10;
	xmlChar* baseChar = xmlTextReaderGetAttribute(aReader, BAD_CAST BASE);
	if (baseChar)
	{
		int rc = ParseDecimalInt(baseChar,base);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Long",0,0,"Could not parse base int");
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;
		}
	}
	if (base == 10)
	{
		int rc = ParseDecimalLong(value,aTarget);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Long",0,0,"Could not parse value");
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;
		}
	}
	else {
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Long",0,0,"Cannot parse nondecimal longs yet");
		xmlFree(type);
		xmlFree(value);
		xmlFree(baseChar);
		return ConstRC::GeneralError;
	}
	if (aInfoMode)
	{
		xmlChar* name = xmlTextReaderName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
		xmlFree(name);
	}
	xmlFree(type);
	xmlFree(value);
	xmlFree(baseChar);
	return ConstRC::Ok;
}

int CommonXMLHelper::SerializeMpz(xmlTextWriterPtr& aWriter, const char* aTagName, /*const char* aAttributeName,*/ const mpz_t aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Mpz",0,0,"Insufficient data.");
        	return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Mpz",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	/*rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST NAME ,"%s",BAD_CAST aAttributeName);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize mpz",rc,0,"Could not write the name.");
        	return ConstRC::GeneralError;
    	}*/
	char* mpzChar = NULL;
	int base = 10;
	mpzChar = mpz_get_str(mpzChar,base,aValue);
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%s",BAD_CAST mpzChar);
	free(mpzChar);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Mpz",rc,0,"Could not write the value.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST BASE,"%d", base);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Mpz",rc,0,"Could not write the base.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST MPZ);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Mpz",rc,0,"Could not write the type.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Mpz",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int CommonXMLHelper::SerializePolynomialCoefficient(xmlTextWriterPtr& aWriter, mpz_t aValue, int aDegree)
{
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST COEFFICIENT_NAME);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Polynomial Coefficent",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	int base = 10;
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST POWER,"%d",aDegree);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Polynomial Coefficient",rc,0,"Could not write the value.");
        	return ConstRC::GeneralError;
    	}
	char* mpzChar = NULL;
	mpzChar = mpz_get_str(mpzChar,base,aValue);
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%s",BAD_CAST mpzChar);
	free(mpzChar);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Polynomial Coefficient",rc,0,"Could not write the value.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Polynomial Coefficient",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int CommonXMLHelper::DeserializePolynomialCoefficient(xmlTextReaderPtr& aReader, const char* aMessage, mpz_t aTargetValue, int& aTargetDegree, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Polynomial Coefficient",0,0,"No attributes, aborting");
		return ConstRC::GeneralError;
	}
	int base = 10;
	xmlChar* degree = xmlTextReaderGetAttribute(aReader, BAD_CAST POWER);
	if (!degree)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Polynomial Coefficient",0,0,"Degree not specified, aborting");
		return ConstRC::GeneralError;
	}
	int result=0;
	int rc = ParseDecimalInt(degree,result);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Polynomial Coefficient",0,0,"Could not degree int");
		xmlFree(degree);
		return ConstRC::GeneralError;
	}
	aTargetDegree = result;
	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Polynomial Coefficient",0,0,"Value not found, aborting");
		xmlFree(degree);
		return ConstRC::GeneralError;
	}
	if (base == 10)
	{
		int rc = ParseDecimalMpz(value,aTargetValue);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Polynomial Coefficient",0,0,"Could not parse value mpz");
			xmlFree(degree);
			xmlFree(value);
			return ConstRC::GeneralError;
		}
	}
	else 
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Polynomial Coefficient",0,0,"Cannot parse nondecimal mpz's yet");
		xmlFree(degree);
		xmlFree(value);
		return ConstRC::GeneralError;
	}
	if (aInfoMode)
	{
		xmlChar* name = xmlTextReaderName(aReader);
		printf("%s: Value of poly coefficient at %d is %s.\n",aMessage,aTargetDegree,value);
		xmlFree(name);
	}
	xmlFree(degree);
	xmlFree(value);
	return ConstRC::Ok;
}

int CommonXMLHelper::SerializeSignedInt(xmlTextWriterPtr& aWriter, const char* aTagName, /*const char* aAttributeName,*/ int aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",0,0,"Insufficient data.");
        	return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	/*rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST NAME ,"%s",BAD_CAST aAttributeName);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize mpz",rc,0,"Could not write the name.");
        	return ConstRC::GeneralError;
    	}*/
	int base = 10;
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%d",aValue);

	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the value.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST BASE,"%d", base);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the base.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST SIGNED_INT);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the type.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Int",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int CommonXMLHelper::SerializeDouble(xmlTextWriterPtr& aWriter, const char* aTagName, /*const char* aAttributeName,*/ double aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",0,0,"Insufficient data.");
        	return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	/*rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST NAME ,"%s",BAD_CAST aAttributeName);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize mpz",rc,0,"Could not write the name.");
        	return ConstRC::GeneralError;
    	}*/
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%f",aValue);

	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the value.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST DOUBLE);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the type.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Int",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int CommonXMLHelper::SerializeUnsignedInt(xmlTextWriterPtr& aWriter, const char* aTagName, /*const char* aAttributeName,*/ unsigned int aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",0,0,"Insufficient data.");
        	return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	/*rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST NAME ,"%s",BAD_CAST aAttributeName);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize mpz",rc,0,"Could not write the name.");
        	return ConstRC::GeneralError;
    	}*/
	int base = 10;
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%u",aValue);

	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",rc,0,"Could not write the value.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST BASE,"%u", base);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",rc,0,"Could not write the base.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST UNSIGNED_INT);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",rc,0,"Could not write the type.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Int",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int CommonXMLHelper::SerializeSignedLong(xmlTextWriterPtr& aWriter, const char* aTagName, /*const char* aAttributeName,*/ long aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",0,0,"Insufficient data.");
        	return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	/*rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST NAME ,"%s",BAD_CAST aAttributeName);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize mpz",rc,0,"Could not write the name.");
        	return ConstRC::GeneralError;
    	}*/
	int base = 10;
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%ld",aValue);

	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the value.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST BASE,"%d",base);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the base.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST SIGNED_LONG);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the type.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Int",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int CommonXMLHelper::SerializeString(xmlTextWriterPtr &aWriter, const char *aTagName, const std::string& aValue)
{
	if (!aTagName)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize String",0,0,"Insufficient data.");
		return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize String",rc,0,"Could not write the serializator tag.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%s", aValue.c_str());
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize String",rc,0,"Could not write the value.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST STRING);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize String",rc,0,"Could not write the type.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterEndElement(aWriter);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize String",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int CommonXMLHelper::SerializeString(xmlTextWriterPtr &aWriter, const char *aTagName, const char* aValue)
{
	if (!aTagName)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize String",0,0,"Insufficient data.");
		return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize String",rc,0,"Could not write the serializator tag.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%s", aValue);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize String",rc,0,"Could not write the value.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST STRING);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize String",rc,0,"Could not write the type.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterEndElement(aWriter);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize String",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int CommonXMLHelper::DeserializeString(xmlTextReaderPtr &aReader, const char *aMessage, char*& aTarget, bool aInfoMode)
{
/*	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize String",0,0,"No attributes of string, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize String",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!xmlStrEqual(type,BAD_CAST STRING))
	{
		xmlFree(type);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize String",0,0,"Data type not string, aborting");
		return ConstRC::GeneralError;
	} */
	int ret = xmlTextReaderRead(aReader);
	if (false == ret) {
		CLog::LogError(WHEREARG,"XML Helper","Deserialize String",0,0,"Node not found, aborting");			
		return ConstRC::GeneralError;	
	};
	xmlChar* value = xmlTextReaderValue(aReader);  
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize String",0,0,"Value not found, aborting");			
	//	xmlFree(type);
		return ConstRC::GeneralError;	
	}
	
	// really dangerous!!!!!!
	strcpy(aTarget,(char*) value);

	if (aInfoMode)
	{
		xmlChar* name = xmlTextReaderName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
		xmlFree(name);
	}
//	xmlFree(type);
	xmlFree(value);
	return ConstRC::Ok;
}


int CommonXMLHelper::DeserializeString(xmlTextReaderPtr &aReader, const char *aMessage, std::string& aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize String",0,0,"No attributes of unsigned long, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize String",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!xmlStrEqual(type,BAD_CAST STRING))
	{
		xmlFree(type);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize String",0,0,"Data type not string, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);  
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize String",0,0,"Value not found, aborting");			
		xmlFree(type);
		return ConstRC::GeneralError;	
	}
	
	aTarget.assign((char*) value);

	if (aInfoMode)
	{
		xmlChar* name = xmlTextReaderName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
		xmlFree(name);
	}
	xmlFree(type);
	xmlFree(value);
	return ConstRC::Ok;
}
int CommonXMLHelper::WriteSerializatorData(xmlTextWriterPtr& aWriter, const char* aSerializatorNameVal, const char* aSerializatorVersion)
{
	int rc;
	rc = xmlTextWriterStartElement(aWriter, BAD_CAST SERIALIZATOR_NAME);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Write Serializator Data",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST NAME,"%s",BAD_CAST aSerializatorNameVal);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Write Serializator Data",rc,0,"Could not write the serializator name.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VERSION_NAME,"%s",BAD_CAST aSerializatorVersion);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Write Serializator Data",rc,0,"Could not write the serializator version.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Write Serializator Data",rc,0,"Could not end the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Write Serializator Data",rc,0,"Could not write new line.");
        	return ConstRC::GeneralError;
    	}
	return ConstRC::Ok;
}

int CommonXMLHelper::SerializeBinaryData(xmlTextWriterPtr &aWriter, const char *aTagName, const void * aValue, const long& aValueUnitCount, const long& aValueUnitByteLength)
{
	if (!aTagName)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize BinaryData",0,0,"Insufficient data.");
		return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize BinaryData",rc,0,"Could not write the serializator tag.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST BINARY_DATA);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize BinaryData",rc,0,"Could not write the type.");
		return ConstRC::GeneralError;
	}
	
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST UNIT_COUNT,"%ld",aValueUnitCount);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize BinaryData",rc,0,"Could not write the unit count.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST UNIT_LENGTH,"%ld",aValueUnitByteLength);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize BinaryData",rc,0,"Could not write the unit length.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST ENDIAN_TYPE,"%s", DEFAULT_CONVERT_ENDIAN_TYPE_STRING);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize BinaryData",rc,0,"Could not write the endian type.");
		return ConstRC::GeneralError;
	}

	char * value = NULL;
	// length without final zero
	long valueLength = aValueUnitByteLength * aValueUnitCount * HEX_IN_BYTE;
	value = (char*)Utils::AllocationMachine((void**)&value, (1 + valueLength) * sizeof(char));
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize BinaryData",0,0,"Cannot allocate memory for binary data.");
		return ConstRC::NotEnoughMemory;
	}
	// final zero
	value[valueLength] = 0;
	rc = CommonXMLHelper::ConvertToHEX(aValue, DEFAULT_CONVERT_ENDIAN_TYPE, value, aValueUnitCount, aValueUnitByteLength, valueLength);
	if (rc != ConstRC::Ok)
	{
		free(value);
		return rc;
	}

	rc = xmlTextWriterWriteString(aWriter, BAD_CAST value);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize BinaryData ",rc,0,"Could not write the binary value.");
		return ConstRC::GeneralError;
	}

	free(value);

	rc = xmlTextWriterEndElement(aWriter);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize BinaryData",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}

	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}
int CommonXMLHelper::DeserializeBinaryDataOut(xmlTextReaderPtr &aReader, const char *aMessage, void** aTarget, long& aTargetUnitCount, long& aTargetUnitByteLength, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"No attributes of binary data, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!xmlStrEqual(type,BAD_CAST BINARY_DATA))
	{
		xmlFree(type);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"Data type not binary data, aborting");
		return ConstRC::GeneralError;
	}
	xmlFree(type);
	type = NULL;

	xmlChar* unitCount = xmlTextReaderGetAttribute(aReader, BAD_CAST UNIT_COUNT);  
	if (!unitCount)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"Unit count not found, aborting");			
		return ConstRC::GeneralError;	
	}
	
	int rc = CommonXMLHelper::ParseDecimalLong(unitCount, aTargetUnitCount); 
	xmlFree(unitCount);
	unitCount = NULL;
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	xmlChar* unitLength = xmlTextReaderGetAttribute(aReader, BAD_CAST UNIT_LENGTH);  
	if (!unitLength)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"Unit length not found, aborting");			
		return ConstRC::GeneralError;	
	}
	
	rc = CommonXMLHelper::ParseDecimalLong(unitLength, aTargetUnitByteLength); 
	xmlFree(unitLength);
	unitLength = NULL;
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	xmlChar* endian = xmlTextReaderGetAttribute(aReader, BAD_CAST ENDIAN_TYPE);  
	if (!endian)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"Endian type not found, aborting");			
		return ConstRC::GeneralError;	
	}

	endian_types endianType;
	rc = CommonXMLHelper::ParseEndianType(endian, endianType);
	xmlFree(endian);
	endian = NULL;
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	if (aTargetUnitCount <= 0 || aTargetUnitByteLength <= 0)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"Insufficient data length, aborting");			
		return ConstRC::GeneralError;
	}

	*aTarget = Utils::AllocationMachine(aTarget, aTargetUnitByteLength * aTargetUnitCount);
	if (!(*aTarget))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"Cannot allocate memory for binary data.");
		return ConstRC::NotEnoughMemory;
	}
	xmlChar* value = xmlTextReaderReadString(aReader);

	rc = CommonXMLHelper::ConvertFromHEX((char*)value,endianType, *aTarget, xmlStrlen(value), aTargetUnitCount, aTargetUnitByteLength);
	if (rc != ConstRC::Ok)
	{
		xmlFree(value);
		return rc;
	}

	if (aInfoMode)
	{
		const xmlChar* name = xmlTextReaderConstName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
	}
	xmlFree(value);
	return ConstRC::Ok;
}

int CommonXMLHelper::DeserializeBinaryDataIn(xmlTextReaderPtr &aReader, const char *aMessage, void* aTarget, const long& aTargetUnitCount, const long& aTargetUnitByteLength, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"No attributes of binary data, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!xmlStrEqual(type,BAD_CAST BINARY_DATA))
	{
		xmlFree(type);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"Data type not binary data, aborting");
		return ConstRC::GeneralError;
	}
	xmlFree(type);
	type = NULL;

	xmlChar* unitCount = xmlTextReaderGetAttribute(aReader, BAD_CAST UNIT_COUNT);  
	if (!unitCount)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"Unit count not found, aborting");			
		return ConstRC::GeneralError;	
	}
	
	int rc;
	long targetUnitCount = 0;
	rc = CommonXMLHelper::ParseDecimalLong(unitCount, targetUnitCount); 
	xmlFree(unitCount);
	unitCount = NULL;
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	xmlChar* unitByteLength = xmlTextReaderGetAttribute(aReader, BAD_CAST UNIT_LENGTH);  
	if (!unitByteLength)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"Unit length not found, aborting");			
		return ConstRC::GeneralError;	
	}

	long targetUnitByteLength = 0;
	rc = CommonXMLHelper::ParseDecimalLong(unitByteLength, targetUnitByteLength); 
	xmlFree(unitByteLength);
	unitByteLength = NULL;
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	if (targetUnitByteLength != aTargetUnitByteLength || targetUnitCount != aTargetUnitCount)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"Insufficient data length, aborting");			
		return ConstRC::GeneralError;
	}

	xmlChar* endian = xmlTextReaderGetAttribute(aReader, BAD_CAST ENDIAN_TYPE);  
	if (!endian)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize BinaryData",0,0,"Endian type not found, aborting");			
		return ConstRC::GeneralError;	
	}

	endian_types endianType;
	rc = CommonXMLHelper::ParseEndianType(endian, endianType); 
	xmlFree(endian);
	endian = NULL;
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	xmlChar* value = xmlTextReaderReadString(aReader);

	rc = CommonXMLHelper::ConvertFromHEX((char*)value, endianType, aTarget, xmlStrlen(value), aTargetUnitCount, aTargetUnitByteLength);
	if (rc != ConstRC::Ok)
	{
		xmlFree(value);
		return rc;
	}

	if (aInfoMode)
	{
		const xmlChar* name = xmlTextReaderConstName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
	}

	xmlFree(value);
	return ConstRC::Ok;
}

int CommonXMLHelper::ConvertToHEX(const void * aField, const endian_types &aHexEndianType, char * aResult, const long &aUnitCount, const long &aUnitByteLength, const long &aResultCharLength)
{
	long length = aUnitCount * aUnitByteLength;
	if (aField == NULL || aResult == NULL || aResultCharLength != length * HEX_IN_BYTE)
	{
		CLog::LogError(WHEREARG,"XML Helper","ConvertToHEX",0,0,"Insufficient data.");
		return ConstRC::GeneralError;
	}

	byte * field = (byte*)aField;
	byte index;

	if (CommonXMLHelper::iMachineEndianType == aHexEndianType)
	{
		for (long i = 0; i < length; i++)
		{
			index = (*field);
			field++;

			*aResult = CommonXMLHelper::HEX_NUMBERS[index >> 4];
			aResult++;
			*aResult = CommonXMLHelper::HEX_NUMBERS[index & SECOND_HEX_MASK];
			aResult++;
		}
	}
	else
	{
		for (long i = 0; i < aUnitCount; i++)
		{
			for (long j = aUnitByteLength - 1; j > -1; j--)
			{
				index = field[j];

				*aResult = CommonXMLHelper::HEX_NUMBERS[index >> 4];
				aResult++;
				*aResult = CommonXMLHelper::HEX_NUMBERS[index & SECOND_HEX_MASK];
				aResult++;
			}

			field += aUnitByteLength;
		}
	}

	return ConstRC::Ok;
}
int CommonXMLHelper::ConvertFromHEX(const char *aHex, const endian_types &aHexEndianType, void *aResult, const long &aHexCharLength, const long &aResultUnitCount, const long &aResultUnitByteLength)
{
	long length = aResultUnitCount * aResultUnitByteLength;
	if (aHex == NULL || aResult == NULL || aHexCharLength != length * HEX_IN_BYTE)
	{
		CLog::LogError(WHEREARG,"XML Helper","ConvertFromHEX",0,0,"Insufficient data.");
		return ConstRC::GeneralError;
	}

	byte * field = (byte*)aResult;
	byte index1, index2;

	if (CommonXMLHelper::iMachineEndianType == aHexEndianType)
	{
		for (long i = 0; i < length; i++)
		{
			for (index1 = 0; index1 < CommonXMLHelper::HEX_NUMBERS_LENGTH; index1++)
			{
				if (*aHex == CommonXMLHelper::HEX_NUMBERS[index1]) break;
			}
			aHex++;
			for (index2 = 0; index2 < CommonXMLHelper::HEX_NUMBERS_LENGTH; index2++)
			{
				if (*aHex == CommonXMLHelper::HEX_NUMBERS[index2]) break;
			}
			aHex++;

			*field = (byte)((index1 << 4) + index2);
			field++;
		}
	}
	else
	{
		for (long i = 0; i < aResultUnitCount; i++)
		{
			for (long j = aResultUnitByteLength - 1; j > -1; j--)
			{
				for (index1 = 0; index1 < CommonXMLHelper::HEX_NUMBERS_LENGTH; index1++)
				{
					if (*aHex == CommonXMLHelper::HEX_NUMBERS[index1]) break;
				}
				aHex++;
				for (index2 = 0; index2 < CommonXMLHelper::HEX_NUMBERS_LENGTH; index2++)
				{
					if (*aHex == CommonXMLHelper::HEX_NUMBERS[index2]) break;
				}
				aHex++;

				field[j] = (byte)((index1 << 4) + index2);
			}
			field += aResultUnitByteLength;
		}
	}

	return ConstRC::Ok;
}


int CommonXMLHelper::SerializeParameterEntry(xmlTextWriterPtr& aWriter, const char* aTagName, const char* aName, const char* aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize ParameterEntry ",0,0,"Insufficient data.");
			return ConstRC::GeneralError;
	}

	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize ParameterEntry ",rc,0,"Could not write the parameter tag.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST NAME, "%s", BAD_CAST aName);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize ParameterEntry ",rc,0,"Could not write the name.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE, "%s", BAD_CAST aValue);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize ParameterEntry ",rc,0,"Could not write the value.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE, "%s", BAD_CAST PARAM_ENTRY);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize ParameterEntry ",rc,0,"Could not write the type.");
		return ConstRC::GeneralError;
	}
	
	rc = xmlTextWriterEndElement(aWriter);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize ParameterEntry ",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}

	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}
int CommonXMLHelper::DeserializeParameterEntry(xmlTextReaderPtr& aReader, const char* aMessage, char*& aTargetName, char*& aTargetValue, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize ParameterEntry ",0,0,"No attributes of parameter entry, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize ParameterEntry ",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (xmlStrEqual(type,BAD_CAST PARAM_ENTRY) == 0)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize ParameterEntry ",0,0,"Data type not parameter entry, aborting");
		xmlFree(type);
		return ConstRC::GeneralError;
	}

	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);  
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize ParameterEntry ",0,0,"Value not found, aborting");			
		xmlFree(type);
		return ConstRC::GeneralError;	
	}	

	xmlChar* name = xmlTextReaderGetAttribute(aReader, BAD_CAST NAME);  
	if (!name)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize ParameterEntry ",0,0,"Name not found, aborting");			
		xmlFree(type);
		xmlFree(value);
		return ConstRC::GeneralError;	
	}	

	int nameLen = xmlStrlen(name);
	int valueLen = xmlStrlen(value);
	aTargetName = (char*)Utils::AllocationMachine((void**)&aTargetName, sizeof(char)*(1 + nameLen));
	if (aTargetName == NULL)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize ParameterEntry ",0,0,"Cannot allocate memory for result, aborting");			
		xmlFree(type);
		xmlFree(name);
		xmlFree(value);
		return ConstRC::NotEnoughMemory;
	}
	aTargetValue = (char*)Utils::AllocationMachine((void**)&aTargetValue, sizeof(char)*(1 + valueLen));
	if (aTargetValue == NULL)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize ParameterEntry ",0,0,"Cannot allocate memory for result, aborting");			
		xmlFree(type);
		xmlFree(name);
		xmlFree(value);
		return ConstRC::NotEnoughMemory;
	}
	strncpy(aTargetName,(char*) name, nameLen);
	strncpy(aTargetValue,(char*) value, valueLen);

	if (aInfoMode)
	{
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
	}
	xmlFree(type);
	xmlFree(name);
	xmlFree(value);
	return ConstRC::Ok;
}
int CommonXMLHelper::DeserializeParameterEntry(xmlTextReaderPtr& aReader, const char* aMessage, std::string& aTargetName, std::string& aTargetValue, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize ParameterEntry ",0,0,"No attributes of parameter entry, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize ParameterEntry ",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (xmlStrEqual(type,BAD_CAST PARAM_ENTRY) == 0)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize ParameterEntry ",0,0,"Data type not parameter entry, aborting");
		xmlFree(type);
		return ConstRC::GeneralError;
	}

	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);  
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize ParameterEntry ",0,0,"Value not found, aborting");			
		xmlFree(type);
		return ConstRC::GeneralError;	
	}	

	xmlChar* name = xmlTextReaderGetAttribute(aReader, BAD_CAST NAME);  
	if (!name)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize ParameterEntry ",0,0,"Name not found, aborting");			
		xmlFree(type);
		xmlFree(value);
		return ConstRC::GeneralError;	
	}	

	aTargetName.assign((char*) name);
	aTargetValue.assign((char*) value);

	if (aInfoMode)
	{
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
	}

	xmlFree(type);
	xmlFree(name);
	xmlFree(value);
	return ConstRC::Ok;
}


int CommonXMLHelper::DeserializeDouble(xmlTextReaderPtr& aReader, const char* aMessage, double& aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Double",0,0,"No attributes of double, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Double",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!xmlStrEqual(type,BAD_CAST DOUBLE))
	{
		xmlFree(type);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Double",0,0,"Data type not double, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);  
	if (!value)
	{
		xmlFree(type);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Double",0,0,"Value not found, aborting");			
		return ConstRC::GeneralError;	
	}	
	int base = 10;
	xmlChar* baseChar = xmlTextReaderGetAttribute(aReader, BAD_CAST BASE);  
	if (baseChar)
	{
		int rc = ParseDecimalInt(baseChar,base);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Double",0,0,"Could not parse base int");			
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;	
		}
	}
	if (base == 10)
	{
		int rc = ParseDecimalDouble(value,aTarget);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Double",0,0,"Could not parse value");			
			xmlFree(value);
			xmlFree(type);
			xmlFree(baseChar);			
		return ConstRC::GeneralError;	
		}
	} 
	else 
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Double",0,0,"Cannot parse nondecimal doubles yet");			
		xmlFree(value);
		xmlFree(type);
		xmlFree(baseChar);
		return ConstRC::GeneralError;	
	}
	if (aInfoMode)
	{
		const xmlChar* name = xmlTextReaderConstName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
	}
	xmlFree(type);
	xmlFree(value);
	xmlFree(baseChar);
	return ConstRC::Ok;
}



int CommonXMLHelper::ParseEndianType(xmlChar *aString, endian_types &aTarget)
{
	if (aString == NULL) return ConstRC::GeneralError;
	int len = xmlStrlen(aString);
	if (!len) return ConstRC::GeneralError;

	aTarget = xmlStrEqual(aString,BAD_CAST LITTLE_ENDIAN_TYPE_STRING) ? ELittleEndian : EBigEndian;

	return ConstRC::Ok;
}

endian_types CommonXMLHelper::iMachineEndianType = CommonXMLHelper::Endian();

endian_types CommonXMLHelper::Endian()
{
	int i = 1;
	char *p = (char *)&i;

	if (p[0] == 1)
		return ELittleEndian;
	else
		return EBigEndian;
}

int CommonXMLHelper::SerializePrimitiveRoot(xmlTextWriterPtr& aWriter, const char* aTagName, const int& aRoot, const int& aPrime)
{
	if (!aTagName)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Primitive Root",0,0,"Insufficient data.");
		return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Primitive Root",rc,0,"Could not write the serializator tag.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST PRIMITIVE_ROOT);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Primitive Root",rc,0,"Could not write the type.");
		return ConstRC::GeneralError;
	}
	
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_PRIME_ATTR,"%d",aPrime);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Primitive Root",rc,0,"Could not write the prime.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_ROOT_ATTR,"%d",aRoot);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Primitive Root",rc,0,"Could not write the root.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterEndElement(aWriter);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Primitive Root",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}
int CommonXMLHelper::DeserializePrimitiveRoot(xmlTextReaderPtr& aReader, const char* aMessage, int& aRoot, int& aPrime, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Primitive Root",0,0,"No attributes of primitive root, aborting");
		return ConstRC::GeneralError;
	}

	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Primitive Root",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!xmlStrEqual(type, BAD_CAST PRIMITIVE_ROOT))
	{
		xmlFree(type);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Primitive Root",0,0,"Data type not primitive root, aborting");
		return ConstRC::GeneralError;
	}
	xmlFree(type);
	type = NULL;
	int rc;

	xmlChar* attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_PRIME_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Primitive Root",0,0,"Prime not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalInt(attr, aPrime);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Primitive Root",0,0,"Could not parse int");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);

	attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_ROOT_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Primitive Root",0,0,"Root not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalInt(attr, aRoot);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Primitive Root",0,0,"Could not parse int");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);
	attr = NULL;

	if (aInfoMode)
	{
		const xmlChar* name = xmlTextReaderConstName(aReader);
		printf("%s: %s - prime: %d, root: %d\n",	aMessage, name, aPrime, aRoot);
	}

	return ConstRC::Ok;
}
int CommonXMLHelper::SerializePowerModP(xmlTextWriterPtr& aWriter, const char* aTagName, const int& aPrime, const int& aNumber, const int& aPower)
{
	if (!aTagName)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Power Mod P",0,0,"Insufficient data.");
		return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Power Mod P",rc,0,"Could not write the serializator tag.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST POWER_MOD_P);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Power Mod P",rc,0,"Could not write the type.");
		return ConstRC::GeneralError;
	}
	
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_PRIME_ATTR,"%d",aPrime);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Power Mod P",rc,0,"Could not write the prime.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_NUMBER_ATTR,"%d",aNumber);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Power Mod P",rc,0,"Could not write the number.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_POWER_ATTR,"%d",aPower);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Power Mod P",rc,0,"Could not write the power.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterEndElement(aWriter);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Power Mod P",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}
int CommonXMLHelper::DeserializePowerModP(xmlTextReaderPtr& aReader, const char* aMessage, int& aPrime, int& aNumber, int& aPower, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Power Mod P",0,0,"No attributes of power mod p, aborting");
		return ConstRC::GeneralError;
	}

	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Power Mod P",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!xmlStrEqual(type, BAD_CAST POWER_MOD_P))
	{
		xmlFree(type);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Power Mod P",0,0,"Data type not power mod p, aborting");
		return ConstRC::GeneralError;
	}
	xmlFree(type);
	type = NULL;
	int rc;

	xmlChar* attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_PRIME_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Power Mod P",0,0,"Prime not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalInt(attr, aPrime);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Power Mod P",0,0,"Could not parse int");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);

	attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_NUMBER_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Power Mod P",0,0,"Number not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalInt(attr, aNumber);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Power Mod P",0,0,"Could not parse int");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);
	attr = NULL;

	attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_POWER_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Power Mod P",0,0,"Power not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalInt(attr, aPower);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Power Mod P",0,0,"Could not parse int");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);
	attr = NULL;

	if (aInfoMode)
	{
		const xmlChar* name = xmlTextReaderConstName(aReader);
		printf("%s: %s - prime: %d, number: %d, power: %d\n",	aMessage, name, aPrime, aNumber, aPower);
	}

	return ConstRC::Ok;
}
