#ifndef __CONNECTORINFO_H__
#define __CONNECTORINFO_H__


#include "types_common.h"
#include "common_xml_helper.h"
#include <string>
#include <libxml/xmlreader.h>

class AbstractConnector; // fwd declaration
class AbstractMessage; // fwd declaration

using namespace std;

/*!
 *\brief Pairing names to types
 */
struct ConnPairing
{
	string 		formalName;
	connector_type	formalNr;
};

/*!
 * \brief Common connector info ancestor for both center and node
 */
class ConnectorInfo
{
public:
	ConnectorInfo(string& aUrlString, connector_type aConnectorType);
	ConnectorInfo(const char* aUrlString, connector_type aConnectorType);
	ConnectorInfo(const ConnectorInfo& aSrc);
	virtual ~ConnectorInfo();

	void Load(AbstractMessage* aMessage);

	const char*		GetUrlString_Char()	const;
	const string&		GetUrlString() const;
	connector_type		GetConnectorType() const;
	void			SetUrlString(const string& aUrlString);
	void			SetUrlString(const char* aUrlString);
	void			SetConnectorType(connector_type aConnectorType);
	void 			Print(unsigned int indent) const;

	static ConnectorInfo*	Parse(const char* info);
	static connector_type	DetectConnectorTypeFromUrl(const string& aUrl);

	bool			IsEqual(const ConnectorInfo* aOtherInstance) const;

	static bool		ParseConnectorType(const string& aValue, connector_type& aType);
	
	
	static void		PrintAllowedConnTypes();

	int			CreateConnector(const string& aOutgoingIdentification);
	void			SetExternalConnector(AbstractConnector* aConnector);
	bool			HasConnector() const;
	AbstractConnector&	GetConnector();

	const char*		GetFailureReason() const;
	
	void			SetFailureReason(const string& aReason);

protected:

	void			SetFailureReason(int aCode);

	int			Deserialize(xmlTextReaderPtr& aReader);

	static void		AddSchemeToUrlString(string& aTarget, connector_type aType);

	
protected:
	AbstractConnector*	iConnector;
	bool			iOwnsConnector;
	string			iFailureReason;

private:
	connector_type		iConnectorType;
	string			iUrlString;
};
#endif
