#ifndef __CONST_HASHTABLES_H_
#define __CONST_HASHTABLES_H_

#include "types_common.h"
#include "limits.h"
/*!
 * \brief Class of constants used in hashtables
*/
class ConstHT
{
public:
	///For Large Singleton Removal - array for partial relations
	static const detection_type NONSINGLETON = 0;
	static const detection_type SINGLETON = 1;
	static const detection_type FRESH_SINGLETON = 2;

	// Small singleton detection
	static const long NOT_YET =	-10;
	static const long TWICE	= 0xffffffff;

	// Some constants regarding list for deletion
	static const long IMPLICIT_LIST_FOR_DELETION_SIZE = 1000;
	static const long IMPLICIT_LIST_FOR_DELETION_RESIZE = 500;

	///For finding cycles
	static const unsigned long ROOT_FLAG = 0x7fffffff;
	///For combination of relation
	static const unsigned long FINISHED = 0xffffffff;
	static const unsigned long PARITY_MASK_NFS = 0x80000000;

#ifdef LONG_LARGE_PRIME
	///Full mask for the field of hashtable - default value
	static const unsigned long FULL_MASK_NFS = ULONG_MAX;
#else
	///Full mask for the field of hashtable - default value
	static const unsigned long FULL_MASK_NFS = UINT_MAX;
#endif

	///Mask for nonsingleton primeideal - this means that primeideal frequency is more 1
	/// static const unsigned long NONSINGLETON_MASK_NFS = FULL_MASK_NFS-FULL_MASK_NFS/2;
	static const unsigned long NONSINGLETON_MASK_NFS = 0x80000000;
	///Min number of expected entries
	static const int MIN_AMOUNT = 2048;
	///Min size of hashtable
	static const int MIN_SIZE = 4096;
	///Max ratio between size of hashtable and count of entries
	static const float MAX_FULLNESS;
	///Min size of hashtable used in sieving for saving large primes
	static const int MIN_LP_TABLE_SIZE = 131072;
	///Number of bits in byte
	static const int BITS_IN_BYTE = 8;
	///For dividing with number of bits in byte - binary shift
	static const int BITS_IN_BYTE_SHIFT = 3;
	///For modulo with number of bits in byte - binary and
	static const int BITS_IN_BYTE_MOD = 0x07;

	///Used as first bit in detection field entry (one byte value) = 2^7
	static const detection_type FIRST_DETECTION_BIT = 0x80;
	
	///Used as flag if prime ideal is contained in more relations
	static const int	UNDEFINED_RELATION = -1;

	///Number of header lines when CFNFSRelation is print to the file (method PrintToFile)
	static const int RELATION_HEADER_FILE_LINES = 5;
	///Number of lines when CFRelationPart is print to the file (method PrintToFile)
	static const int RELATION_PART_FILE_LINES = 3;


};

#endif // __CONST_HASHTABLES_H_
