#ifndef __CONSTANTS_H__
#define __CONSTANTS_H__

/*!
 * \file 
 * \brief This file contains definitions of global constants.
*/


#ifndef WIN32
#include <inttypes.h>
#include <unistd.h>
#define PATH_SEPARATOR '/'
#define PATH_SEPARATOR_OTHER '\\'
#else
#define PATH_SEPARATOR '\\'
#define PATH_SEPARATOR_OTHER '/'
#endif

#define FILE_NAME_SEPARATOR "_"

#ifdef _INTTYPES_H // getting typedefs from inttypes_common.h
typedef int64_t INT64;
#endif

#ifdef WIN32       // windows specific
typedef __int64 INT64;
#endif

#ifdef _UNISTD_H
#define SLEEP(x)	sleep((x))
#else
#include <windows.h>
#define SLEEP(x)    Sleep((x) * 1000)
#endif

typedef INT64   relation_hash_type;
#define RELATION_HASH_TYPE_SIZE 64

//#define NULL 0

/*! 
 \def WHEREARG
 Expands to two predefined macros separated by comma. Frequently used for Utils::ErrorMessage(int aLineNumber, const char* aSourceFile, const char* aProcess, const char* aProblem, int aErrorCode, int aLabel, const char* aExplanation) calls in place of the first two parameters.
 */
#define WHEREARG __LINE__, __FILE__

// for normal message
#define NO_NUMBER                 -1

// Error return codes - general
#define NOT_ENOUGH_MEMORY    -1
#define COULD_NOT_OPEN_FILE  -2
#define COULD_NOT_CLOSE_FILE -3
#define COULD_NOT_READ_FROM_NULL_STREAM -4
#define END_OF_FILE          -5
#define BAD_ARGUMENT         -6
#define COULD_NOT_WRITE_TO_NULL_STREAM -7
//duplicitní k definici v constants.h 
//#define GENERAL_ERROR         0xff000000

#define IMPOSSIBLE          -11
#define CANNOT_BE_DONE      -12
#define STOP_NOW            -13
#define INCORRECT_DEPTH     -14
#define ALREADY_RUNNING     -15

// CBaseParameters entry values for bool
#define TRUE_STRING "true"
#define FALSE_STRING "false"

// Binary convert
#define LITTLE_ENDIAN_TYPE_STRING "little-endian"
#define BIG_ENDIAN_TYPE_STRING    "big-endian"

// Modes of file access
#define FILE_MODE_READ   "r"
#define FILE_MODE_WRITE	 "w"
#define FILE_MODE_APPEND "a"
#define FILE_MODE_WRITE_BINARY "wb"
#define FILE_MODE_READ_BINARY "rb"
#define FILE_MODE_APPEND_BINARY "ab"


// Value ENOUGH is returned if enough smooth values have been generated
#define NOT_ENOUGH                -1
#define ENOUGH                    1

// Return values of CheckNulity
#define ZERO           		   0
#define NONZERO                    1

#define TRUE        		1
#define FALSE       		0

#define EQUAL 0
#define NOT_EQUAL 1

#define NOT_SPECIFIED				 -1

#define HEX_IN_BYTE 2
#define FIRST_HEX_MASK 0xF0
#define SECOND_HEX_MASK 0x0F

#define SECONDS_IN_MINUTE 60

#define IO_BUFF_SIZE 100000

#define XML_FILE_EXTENSION ".xml"
#define TXT_FILE_EXTENSION ".txt"

#endif
