#include "dirutil.h"
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "log.h"

#include "abstract_sieve.h"


int DirUtil::TestDirectoryAccess(const char* aDirectoryName)
{
	if (!aDirectoryName)
	{
		CLog::LogError(WHEREARG,"Directory utility","Test directory access",0,0,"The directory name was NULL");
		return NULL_POINTER_SUPPLIED;
	}
	FILE* f;
	string aux = aDirectoryName;
	// Append PATH_SEPARATOR if absent
	if (aux.size() > 0)
	{
		if (aux.at(aux.size()-1) != PATH_SEPARATOR)
		{
			aux += PATH_SEPARATOR;
		}
	}
	aux.append("tmpfile");
	f = fopen(aux.c_str(),"w");
	if (!f)
	{
		int err = errno;
		CLog::LogError(WHEREARG,"Directory utility","Test directory access",0,0,"Could not create file in the following path");
		printf("Tested dir: %s.\n",aDirectoryName);
		printf("Name of tested file: %s.\n",aux.c_str());
		if (err == ENOENT)
		{
			CLog::LogError(WHEREARG,"Directory utility","Test directory access",0,0,"The indicated directory does not exist. Please create it first.");
			return DIRECTORY_DOES_NOT_EXIST;
		}
		if (err == EACCES)
		{
			CLog::LogError(WHEREARG,"Directory utility","Test directory access",0,0,"The indicated directory does not have sufficient rights. Please assign sufficient rights for write access");
			return DIRECTORY_ACCESS_DENIED;			
		}
		return GENERAL_FILESYSTEM_ERROR;
	}
	fclose(f);
	remove(aux.c_str());	
	return ConstRC::Ok;
}

int DirUtil::TestDirectoryAccess(const string& aDirectoryName)
{
	return TestDirectoryAccess(aDirectoryName.c_str());
}
