#include "ecm_old.h"
#include "gmp_tests.h"
#include "const_return_codes.h"

// #define DEBUG_ELLIPTIC_DETAILED
/*!
 	The default constructor sets 'mode' to optimization level 1 (using Brent's improvement).
*/
oldECM::oldECM(int aNumCurves) {
	NumCurves = aNumCurves;
	this->Inverse = new parallel_Inverse(aNumCurves);
	this->Points = new ell_point[aNumCurves];
	this->TempPoints= new ell_point[aNumCurves];
	mpz_init(this->Field);
	mpz_init(this->fact_1);
	mpz_init(this->fact_2);
	b_1 = 10000;
	this->exponents = new int[b_1];
	this->max_exponent_index = this->b_1 - 1;
	for(int tempi=0;tempi<b_1;tempi++) exponents[tempi]=0;
	int i;  long prime;
	int logb_1 = BinLog(this->b_1);	
	for (i=0; i<= this->max_exponent_index; i++) {
		prime = small_primes[i];
		if (prime > this->b_1) {
			break;
		} else {
			// We will find the maximal power of this prime which is lower than bound B_1. This can be done by (log b_1)/(log prime).
			exponents[i]=1;
			for(;exponents[i]<b_1;exponents[i]*=small_primes[i]);
			exponents[i]/=small_primes[i];
//			printf(" %d ^ %d, b_1 = %d.\n",prime,exponents[i],this->b_1);
		}
	}
	max_exponent_index = i;
}

oldECM::~oldECM() {
	delete this->Points;
	delete this->Inverse;
	delete this->TempPoints;
	mpz_clear(this->Field);
	mpz_clear(this->fact_1);
	mpz_clear(this->fact_2);
}

int oldECM::Factor(qs_relation* aCandidate) {
	int result = Factor(aCandidate->remaining,fact_1,fact_2);
	if (result == ConstRC::Factorized) {
		if (mpz_fits_uint_p(fact_1)) {
			aCandidate->lprime1 = mpz_get_ui(fact_1);			
		} else {
			return FACTOR_TOO_LARGE;
		}
		if (mpz_fits_uint_p(fact_2)) {
			aCandidate->lprime2 = mpz_get_ui(fact_2);						
		} else {
			return FACTOR_TOO_LARGE;			
		}
	}
	return result;
}

ell_point::ell_point() {
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  ell_point::ell_point()");
	#endif
	mpz_init(x);
	mpz_init(y);
	mpz_init(z);
	mpz_init(t1);
	mpz_init(t2);
	mpz_init(t3);
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  $ell_point::ell_point()");
	#endif
};

ell_point::~ell_point() {
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  ell_point::~ell_point()");
	#endif
	curve = NULL;
	mpz_clear(x);
	mpz_clear(y);
	mpz_clear(z);
	mpz_clear(t1);
	mpz_clear(t2);
	mpz_clear(t3);
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  $ell_point::~ell_point()");
	#endif
};

bool ell_point::equals(ell_point * B) {
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  ell_point::equals()");
	#endif
	if(mpz_cmp(B->x,x)||mpz_cmp(B->y,y)||mpz_cmp(B->z,z))
	{
		#ifdef DEBUG_ELLIPTIC_DETAILED
			fprintf(stderr,"  $ell_point::equals():false");
		#endif
		return false;
	} else {
		#ifdef DEBUG_ELLIPTIC_DETAILED
			fprintf(stderr,"  $ell_point::equals():true");
		#endif
		return true;
	};
};

int ell_point::Print(){
	gmp_fprintf(stderr,"[%Zd,%Zd,%Zd]",x,y,z);
	return ConstRC::Ok;
};

ell_curve_ff::ell_curve_ff(mpz_t afield){
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  ell_curve_ff::ell_curve_ff()");
	#endif
	mpz_init(field);
	mpz_set(field,afield);
	mpz_init(result);
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  $ell_curve_ff::ell_curve_ff()");
	#endif
}

ell_curve_ff::~ell_curve_ff(){
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  ell_curve_ff::~ell_curve()");
	#endif
	mpz_clear(field);
	mpz_clear(result);
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  $ell_curve_ff::~ell_curve()");
	#endif
}

ell_point *ell_curve_ff::create_point(){
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  ell_curve_ff::create_point()");
	#endif
	ell_point *temp =new ell_point();
	mpz_set_ui(temp->x,0);
	mpz_set_ui(temp->y,1);
	mpz_set_ui(temp->z,0);
	temp->curve = this;
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  $ell_curve_ff::create_point()");
	#endif
	return temp;
};

int ell_curve_ff::add_points (ell_point *aResult, ell_point *aA, ell_point *aB){
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  ell_curve_ff::add_points(,,,)");
	#endif
	mpz_t t1,t2,t3;
	mpz_init(t1);
	mpz_init(t2);
	mpz_init(t3);

	if (!(aA->equals(aB))) {
		mpz_sub(t2,aA->x,aB->x);mpz_mod(t2,t2,field);
		mpz_gcdext(t1,t3,NULL,t2,field);
		if (mpz_cmp_ui(t1,1)) gmp_printf("mam: %Zd",t1);
		mpz_sub(t1,aA->y,aB->y);mpz_mod(t1,t1,field);
		mpz_mul(t3,t3,t1);
		mpz_mod(t3,t3,field);;
	} else
	{
		
		mpz_mul_ui(t1,aA->x,3);
		mpz_mod(t1,t1,field);
		mpz_mul(t1,aA->x,t1);
		mpz_mod(t1,t1,field);
		mpz_add_ui(t1,t1,a_4);
		mpz_mod(t1,t1,field);;
		mpz_mul_ui(t2,aA->y,2);
		mpz_mod(t2,t2,field);
		mpz_gcdext(t3,t2,NULL,t2,field);;
		if (mpz_cmp_ui(t3,1)) 
			{
				 gmp_printf("mam: %Zd",t3);
				return 0;
			};
		mpz_mul(t3,t1,t2);
		mpz_mod(t3,t3,field);;
	};
	
	mpz_mul(t1,t3,t3);
	mpz_mod(t1,t1,field);
	mpz_sub(t1,t1,aA->x);mpz_mod(t1,t1,field);
	mpz_sub(t1,t1,aB->x);mpz_mod(t1,t1,field);
	mpz_set(t2,t1);

	mpz_sub(t1,aA->x,t2);mpz_mod(t1,t1,field);
	mpz_mul(t1,t1,t3);
	mpz_mod(t1,t1,field);
	mpz_sub(t1,t1,aA->y);mpz_mod(t1,t1,field);

	mpz_set(aResult->x,t2);
	mpz_set(aResult->y,t1);


	mpz_clear(t1);
	mpz_clear(t2);
	mpz_clear(t3);
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  $ell_curve_ff::add_points(,,,)");
	#endif
	return 0;
}

int ell_curve_ff::mul_point (ell_point *aResult, ell_point *aA, unsigned int mul){
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  ell_curve_ff::mul_point(,,%i)",mul);
	#endif
	ell_point *temp = create_point();
	unsigned int counter = 1<< (sizeof(int)*8-1);
	while(0==(mul&counter)) counter >>= 1;
	copy_point(temp,aA);
	counter >>=1;
	while(counter!=0){
		add_points(temp,temp,temp);
		if(mul&counter) add_points(temp,temp,aA);
		counter >>=1;
	}
	copy_point(aResult,temp);
	delete temp;
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  $ell_curve_ff::mul_point(,,%i)",mul);
	#endif
	return 0;
}

int ell_curve_ff::print_point(ell_point aA){
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  ell_curve_ff::printf_point()");
	#endif
	gmp_printf("[%Zd,%Zd,%Zd]",aA.x,aA.y,aA.z);
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  $ell_curve_ff::printf_point()");
	#endif
	return 0;
};

int ell_curve_ff::copy_point(ell_point *aResult, ell_point *aSource){
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  ell_curve_ff::printf_point()");
	#endif
	mpz_set(aResult->x,aSource->x);
	mpz_set(aResult->y,aSource->y);
	mpz_set(aResult->z,aSource->z);
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  $ell_curve_ff::printf_point()");
	#endif
	return 0;
}

int oldECM::copy_point(ell_point *aResult, ell_point *aSource){
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  ell_curve_ff::printf_point()");
	#endif
	mpz_set(aResult->x,aSource->x);
	mpz_set(aResult->y,aSource->y);
	mpz_set(aResult->z,aSource->z);
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr,"  $ell_curve_ff::printf_point()");
	#endif
	return 0;
}


int ell_curve_ff::SetA(int aA){
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr," ell_curve_ff::SetA(%i)",aA);
	#endif
	a_4=aA;
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr," $ell_curve_ff::SetA(%i)",aA);
	#endif
	return 0;
}


int oldECM::Factor(mpz_t aN, mpz_t aFactor1, mpz_t aFactor2){
	if (ConstRC::Factorized == Factoring(aN)){
		mpz_set(aFactor1,fact_1); 
		mpz_div(aFactor2,aN, fact_1);
		#ifdef DEBUG_ELLIPTIC_DETAILED
			gmp_fprintf(stderr,"Rozlozeno: %Zd %Zd",aFactor1,aFactor2);
		#endif
		return ConstRC::Factorized;
	};
	return ConstRC::NotFactorized;;
};
int oldECM::Factoring(mpz_t aN){
	// set field
//	gmp_printf("\nJdu rozlozit %Zd, NumCurves:%d",aN, NumCurves);
	Inverse->SetN(aN);
	mpz_set(Field,aN);
	for(int tempi=0;tempi<this->NumCurves;tempi++)
		InitialisePoint(&this->Points[tempi]);
////	Points[0].Print();
	int exp_index=0;
	for(exp_index=0;exp_index<max_exponent_index;exp_index++) {
	//	printf("\n$%d$%d$%d", exp_index, max_exponent_index,exponents[exp_index]);
//		Points[0].Print();
		unsigned int counter = 1<< (sizeof(int)*8-1);
		while(0==(exponents[exp_index]&counter)) counter >>= 1;
		for(int par=0;par <NumCurves;par++)
			copy_point(&TempPoints[par],&Points[par]);
		counter >>=1;
		while(counter!=0){
			Inverse->Reset();
			for(int par=0;par <NumCurves;par++){
				AddPoints1(&TempPoints[par],&TempPoints[par],&TempPoints[par],par+2);};
			if (ConstRC::Factorized == Inverse->Compute()){
				Inverse->Output(fact_1,0);
				return ConstRC::Factorized;
			};
			for(int par=0;par <NumCurves;par++)
				AddPoints2(&TempPoints[par],&TempPoints[par],&TempPoints[par]);
			if(exponents[exp_index]&counter) {
				Inverse->Reset();
				for(int par=0;par <NumCurves;par++)
					AddPoints1(&TempPoints[par],&TempPoints[par],&Points[par],par+2);
				if (ConstRC::Factorized == Inverse->Compute()){
					Inverse->Output(fact_1,0);
					return ConstRC::Factorized;
				};
				for(int par=0;par <NumCurves;par++)
					AddPoints2(&TempPoints[par],&TempPoints[par],&Points[par]);
			}
			counter >>=1;
		}
		for(int par=0;par <NumCurves;par++)
		copy_point(&Points[par], &TempPoints[par]);
	};
	return ConstRC::Ok;
};

int oldECM::InitialisePoint(ell_point *aPoint){
	mpz_set_ui(aPoint->x,0);
	mpz_set_ui(aPoint->y,1);
	mpz_set_ui(aPoint->z,0);
	return ConstRC::Ok;
};


int oldECM::AddPoints1 (ell_point *aResult, ell_point *aA, ell_point *aB,unsigned a_4){
	if (!(aA->equals(aB))) {
		mpz_sub(aResult->t2,aA->x,aB->x);
		mpz_mod(aResult->t2,aResult->t2,Field);
	}else
	{
		mpz_mul_ui(aResult->t1,aA->x,3);
		mpz_mod(aResult->t1,aResult->t1,Field);
		mpz_mul(aResult->t1,aA->x,aResult->t1);
		mpz_mod(aResult->t1,aResult->t1,Field);
		mpz_add_ui(aResult->t1,aResult->t1,a_4);
		mpz_mod(aResult->t1,aResult->t1,Field);;
		mpz_mul_ui(aResult->t2,aA->y,2);
		mpz_mod(aResult->t2,aResult->t2,Field);
	};
	Inverse->Put(aResult->t2);	

	return ConstRC::Ok;
};

int oldECM::AddPoints2 (ell_point *aResult, ell_point *aA, ell_point *aB){
	Inverse->Get(aResult->t2);
	if (!(aA->equals(aB))) {
		mpz_sub(aResult->t3,aA->y,aB->y);mpz_mod(aResult->t1,aResult->t1,Field);
		mpz_mul(aResult->t2,aResult->t2,aResult->t3);
		mpz_mod(aResult->t3,aResult->t2,Field);;
	}else{
		mpz_mul(aResult->t3,aResult->t1,aResult->t2);
		mpz_mod(aResult->t3,aResult->t3,Field);;
	};
	mpz_mul(aResult->t1,aResult->t3,aResult->t3);
	mpz_mod(aResult->t1,aResult->t1,Field);
	mpz_sub(aResult->t1,aResult->t1,aA->x);mpz_mod(aResult->t1,aResult->t1,Field);
	mpz_sub(aResult->t1,aResult->t1,aB->x);mpz_mod(aResult->t1,aResult->t1,Field);
	mpz_set(aResult->t2,aResult->t1);

	mpz_sub(aResult->t1,aA->x,aResult->t2);mpz_mod(aResult->t1,aResult->t1,Field);
	mpz_mul(aResult->t1,aResult->t1,aResult->t3);
	mpz_mod(aResult->t1,aResult->t1,Field);
	mpz_sub(aResult->t1,aResult->t1,aA->y);mpz_mod(aResult->t1,aResult->t1,Field);

	mpz_set(aResult->x,aResult->t2);
	mpz_set(aResult->y,aResult->t1);

	return ConstRC::Ok;
};

int split_using_ec(mpz_t  number, mpz_t divisor){
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr," $split_using_ec()");
	#endif
	ell_curve_ff *krivka;
	ell_point *bod;
	mpz_t(temp);
	mpz_init(temp);
	mpz_set(temp,number);
	for(int pin=2;pin <100;pin++){
		krivka =new ell_curve_ff(temp);
		krivka->SetA(pin);
		bod=krivka->create_point();
		for(int tempi=2;tempi<250;tempi++){
			krivka->mul_point(bod,bod,tempi);
			}
		delete(bod);
		delete(krivka);
	};
	mpz_clear(temp);
	return 0;
	#ifdef DEBUG_ELLIPTIC_DETAILED
		fprintf(stderr," $split_using_ec()");
	#endif
};

int parallel_Inverse::Reset() 
{
		stackindex = 0; 
		return ConstRC::Ok; 
};

parallel_Inverse::parallel_Inverse(int newcount){
	mpz_init(N);
	mpz_set_ui(N,0);
	mpz_init(MainInverse);
	count = newcount;
	Temps = new mpz_t[count];
	Interfaces = new mpz_t[count+1];
	for (int tempint = 0;tempint < count;tempint++){
		mpz_init(Temps[tempint]);
	};
	for (int tempint = 0;tempint <= count;tempint++){
		mpz_init(Interfaces[tempint]);
	};
	stackindex = 0;
};

parallel_Inverse::parallel_Inverse(mpz_t modN,int newcount){
	mpz_init(N);
	mpz_set(N,modN);
	mpz_init(MainInverse);
	count = newcount;
	Temps = new mpz_t[count];
	Interfaces = new mpz_t[count+1];
	for (int tempint = 0;tempint < count;tempint++){
		mpz_init(Temps[tempint]);
	};
	for (int tempint = 0;tempint <= count;tempint++){
		mpz_init(Interfaces[tempint]);
	};
	stackindex = 0;
};

parallel_Inverse::~parallel_Inverse(){
	mpz_clear(N);
	mpz_clear(MainInverse);
	for (int tempint = 0;tempint < count;tempint++){
		mpz_clear(Temps[tempint]);
	};
	for (int tempint = 0;tempint <= count;tempint++){
		mpz_clear(Interfaces[tempint]);
	};
	delete [] Temps;
	delete [] Interfaces;
};

int parallel_Inverse::SetCount(int newcount){
	return -1;
};

int parallel_Inverse::SetN(mpz_t newN){
	mpz_set(N, newN);
	return 0;
};

int parallel_Inverse::GetN(mpz_t result){
	mpz_set(result,N);
	return 0;
};

int parallel_Inverse::Input(mpz_t InputA, int index){
	mpz_set(Interfaces[index],InputA);
	return 0;
};

int parallel_Inverse::Put(mpz_t InputA){
	if (stackindex>=count) return -1;
	mpz_set(Interfaces[stackindex++],InputA);
	return 0;
};


int parallel_Inverse::Output(mpz_t OutputB, int index){
	mpz_set(OutputB, Interfaces[index+1]);
	return 0;
};

int parallel_Inverse::Get(mpz_t OutputB){
	if (stackindex>=count) return -1;
	mpz_set(OutputB, Interfaces[stackindex++]);
	return 0;
};


int parallel_Inverse::Compute(){
	stackindex=0;
	mpz_set(Temps[0],Interfaces[0]);
        for(int tempi=1;tempi<count;tempi++){
		mpz_mul(Temps[tempi],Temps[tempi-1],Interfaces[tempi]);
		mpz_mod(Temps[tempi],Temps[tempi],N);
	};
	if (!mpz_invert(MainInverse,Temps[count-1],N)) 
	{
		mpz_gcdext(Interfaces[1], MainInverse, NULL, Temps[count-1],N);
	//	gmp_fprintf(stderr, "Compute:mam>%Zd",Interfaces[1]);
		return ConstRC::Factorized;
	}
	for(int tempi=count-1;tempi>0;tempi--){
		mpz_mul(Interfaces[tempi+1],MainInverse,Temps[tempi-1]);
		mpz_mod(Interfaces[tempi+1],Interfaces[tempi+1],N);
		mpz_mul(MainInverse,MainInverse,Interfaces[tempi]);
		mpz_mod(MainInverse,MainInverse,N);
	};
	mpz_set(Interfaces[1],MainInverse);
	return 0;
};


