#ifndef _FACTOR_ALG_H_
#define _FACTOR_ALG_H_

#include <stdio.h>
#include <stdlib.h>
#include "gmp.h"
#include "types_common.h"

#define FACTOR_1_FITS_IN_UINT 0x1
#define FACTOR_2_FITS_IN_UINT 0x2

/*!
 * \brief Supporting algorithm context
 */
typedef struct {
	mpz_t 	     modulus;
	mpz_t 	     factor_1;
	mpz_t 	     factor_2;
	int          flags;
	unsigned int f_1_int;
	unsigned int f_2_int;
} factoring_environment;

/*!
\brief An abstract class representing a factoring algorithm.
   
  This class is an abstract base class for factoring algorithms. Its main purpose is to serve for factorization
  of products of two (relatively large) primes, which are often encountered in Double Large Prime Variation
  of the MPQS/SIQS.
  For this purpose, the overloaded method 
  
       int Factor(relation* aCandidate)
       
  of each extension class should return one of the three values (defined in definitions.h):
  
       NOT_FACTORIZED if factorization of aCandidate->remaining was unsuccessful
       
       FACTOR_TOO_LARGE if factorization was successful, but at least one of the factors does not fit into unsigned int
       
       FACTORIZED if factorization was successful, and both of the factors fit into unsigned int
  
*/

class FactoringAlgorithm {
public:
	FactoringAlgorithm();
	virtual ~FactoringAlgorithm();
	virtual int Factor(mpz_t aModulus, mpz_t aFactor1, mpz_t aFactor2) = 0;
	virtual int Factor(factoring_environment* aEnv) = 0;	
	virtual int Factor(qs_relation* aCandidate) = 0;
};

#endif

