#ifndef __FILESYSTEM_MESSAGE_H__
#define __FILESYSTEM_MESSAGE_H__

#define ARCHIVE_SUPPORT

#include <stdio.h>
#include <time.h>
#include "types_common.h"

#include <libxml/encoding.h>
#include <libxml/xmlwriter.h>
#include <libxml/xmlreader.h>
#include "abstract_message.h"

// Messages are created with extension .tmp.
#define TEMPORARY_EXTENSION	".tmp"
// When they are finished (flushed, closed), they are renamed to .msg
#define MESSAGE_EXTENSION 	".msg"
#define PROCESSED_EXTENSION	".proc"
#define FINISHED_EXTENSION	".fin"

/*!
 * \brief Message passed using filesystem
 */
class FilesystemMessage : public AbstractMessage
{
// FilesystemConnector needs to invoke "ChangeExtension" and "MoveToSubdir" methods,
// which are otherwise too dangerous to be "public".
friend class FilesystemConnector;
public:
	// Overloaded construction and destruction
	FilesystemMessage();
	FilesystemMessage(const string& aFullPath);
	/**
	 *	Address ... directory
         *
         *
	 */
	FilesystemMessage(const char* aDirectory, const char* aSubject, message_type aType, unsigned int aFlags, unsigned int aCounter, const char* aSenderId, const char* aTargetId, const char* aData, unsigned int aTimeout);
	~FilesystemMessage();

	int	MarkAsProcessed();
	int	MarkAsFinished(bool aKeepMessage);

private:
	// Filesystem-specific method
	void	CreateFileName(const char* aExtension = TEMPORARY_EXTENSION);
	void	ChangeExtension(const char* aNewExtension = MESSAGE_EXTENSION);
	string  DetectExtension();
#ifdef ARCHIVE_SUPPORT
	void	MoveToSubdirectory();
#endif
	void	ConstructBodyPath();

protected:	
	xmlTextWriterPtr	CreateOutputStream();
	xmlTextReaderPtr	CreateInputStream();
	int			FinishDeserialization();
	int			FinishSerialization();

private:
		
	string			iFullPath;
	string			iDirectory;
	
};

#endif
