/*

   Hlavickovy soubor pro ruzne testy prvociselnosti.
   ================================================
   M. Kechlibar, unor 2003
   J.J. Zvanovec - srpen 2012: opraveny drobne chyby a bezne nepouzivane procedury odklizeny do gmp_tests.historic.*


   Plnokrevna verze je bezna C funkce, ktera si dava pozor na chyby typu "nelze otevrit
   soubor", stara se o vypisy eventualnich hlaseni apod. Pro ucely dalsiho pouziti v programovani
   je proto vhodnejsi, avsak pro studium algoritmu zcela nevhodna - kod osetrujici mozne chyby
   zpusobuje jeji necitelnost.

   Minimalisticka verze je nejuspornejsi mozny kod provadejici prislusny algoritmus. Je dobre
   citelna, avsak z hlediska absence vstupnich/vystupnich hlaseni a neosetrenych chyb, vyjimek ap.
   nepouzitelna pro serioznejsi programovani. Je urcena hlavne studentum, kteri si chteji
   osvojit algoritmus skryty za testem.

   Parametr "vypis": je-li zadan jako 1, nahlasi test svuj vysledek, napriklad v teto forme:
     "Miller-Rabin: testovane cislo je slozene a zjistili jsme to po 1 iteraci."
   Je-li "vypis" roven 0, jsou vsechna hlaseni na obrazovku s vyjimkou chybovych potlacena.
   Nastaveni 0 vyrazne setri pamet a urychluje beh programu.

   Parametr "initialize_random": je-li zadan jako 1, kazdy prubeh daneho testu si znovu
   inicializuje generator nahodnych cisel a po svem ukonceni alokovanou pamet uvolni. Tento
   stav je doporucen, pokud testujeme jedno velke cislo. Je-li zadan jako 0, spoleha se dany
   test na to, ze uz byla inicializovana globalni promenna "state". Tento stav je doporucen,
   kdyz pocitame s mnoha prubehy daneho testu, dejme tomu s milionem prubehu: pak totiz
   odpadavajici nutnost inicializovat generator usetri nejaky (nezanedbatelny) cas, ovsem
   na ukor toho, ze pred startem testu musime inicializovat globalni promennou.

 */
#ifndef __GMP_TESTS_H__
#define __GMP_TESTS_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gmp.h"
#include "time.h"
#include "gmp_messages.h"

#ifdef __cplusplus
extern "C" {
#endif
/// globalni promenna pomahajici nastartovat nahodny generator 
extern gmp_randstate_t state;

/* do tohoto dynamickeho pole se kopiruji mala prvocisla pro ucely praktickych testu */
/* promenna pod nim obsahuje pocet polozek v poli small_primes MINUS jedna; tedy maximalni */
/* index v poli prvocisel */
extern long *small_primes;
extern long length_of_small_primes;



/* plnokrevne testy, optimalizovane pro testovani cisel od 2 vyse */
/* prvni tri mimo jine obsahuji testovani vlozeneho cisla na koncovou */
/* cislici a pousteji dale jen cisla koncici 1,3,7,9 */
int pseudoprime_test(mpz_t testednum, int exactness, int vypis, int initialize_random);

/* funkce pomahajici zkusmemu deleni, vygeneruje soubor s malymi prvocisly (od FROM do BOUND) */
/* doporucene pouziti je s konstantou from=2, pro hodnoty from mensi nez 2 dojde k zaseknuti  */
/* pri nastaveni parametru add_or_rewrite na 0 dojde k eventualnimu zapisu dalsich prvocisel  */ 
/* za konec stavajiciho souboru, viz dokumentace */

int generate_file_of_small_primes(const char *name, unsigned long from, unsigned long bound, int vypis, int add_or_rewrite,FILE *frontend);
/* nacte do pameti prvocisla od 2 do bound. name musi odkazovat na existujici soubor */
long copy_file_of_small_primes_into_memory(const char *name,int bound);

/* v praktickem pouziti testu je potreba zvazit, jakym zpusobem chceme generovat */
/* nahodna cisla. Na Athlonu XP 1600+ trva vykonani jedne dvojice prikazu        */
/*         gmp_randinit_default(state);        gmp_randclear(state);             */
/* asi 3x10^-6 sekundy. To je relativne malo, ale pokud testujeme na prvociselnost */
/* dejme tomu 10^10 cisel, je uz celkove zpozdeni zpusobene pouze inicalizaci a  */
/* uvolnovanim rovno skoro 9 hodinam                                             */
/* soucasny pristup je takovy, ze kazdy test si inicializuje random generator po svem */
#ifdef __cplusplus
}
#endif

#endif

