#ifndef __INTEGER_CLASS__
#define __INTEGER_CLASS__

#include <libxml/xmlreader.h>
#include <libxml/xmlwriter.h>
#include "common_definitions.h"
#include "common_xml_helper.h"

/*!
 * \brief Class for basic integer number theoretic operations
 */
class CIntegerClass
{
private:
	static int GenerateNewPrimesFile(const long& aMaxBound, xmlTextReaderPtr& aReader);
	static long GetPrimesMaxBound(xmlTextReaderPtr aReader);
	static int WritePrimesHeader(const long& aMaxBound, xmlTextWriterPtr aWriter);
	static long GetPrimitiveRootsMaxBound(xmlTextReaderPtr aReader);
	static int WritePrimitiveRootsHeader(const long& aMaxBound, xmlTextWriterPtr aWriter);
	static int GenerateNewPrimitiveRootsFile(const long& aMaxBound);
public:
	static int Mulm(INT64 aArg1, INT64 aArg2, int aMod);
	static int Mulm(INT64 aArg1, int aArg2, int aMod);
	static int Mulm(int aArg1, int aArg2, int aMod);
	static int Powm(int aArg, int aExp, int aMod);
	static int IPow(int aBase, int aExp);
	static double Trunc(double aArg);
	static short int Sgn(long aOperand);
	static long Max(long a,long b);
	static long Min(long a,long b);
	static long RandomNumber(long aRange); /// return random integer in interval [0..aRange-1]
	static long ExtendedEuclid(long *aComb1, long *aComb2, long aOperand1, long aOperand2); /// return gcd(aOperand1,aOperand2) = aComb1*aOperand1 + aComb2*aOperand2
	static long LeastCommonMultiple(long aOperand1, long aOperand2);
	static long InverseModN(long aOperand, long aModulus); /// result in interval from -(aModulus/2) to (aModulus/2)
	static long InverseModNPositive(long aOperand, long aModulus); /// result in interval from 0 to aModulus-1
	static long PowerModN(long aOperand, unsigned int aPower, long aModulus);
	static short int JacobiSymbol(long aOperand, long aModulus);
	static long SquareRootModPrime(long aOperand, long aModulus);
	static int GetPrimes(const long& aMinBound, const long& aMaxBound, unsigned long& aCount, long*& aPrimes);
	static int GetPrimesCount(const long& aMinBound, const long& aMaxBound, long& aCount);
	static int GetPrimitiveRootsAndPowers(int * aSetP, int aSetPSize, int aMaxBound, int * aPrimitiveRoots, int ** aPowers);

};

#endif

