#ifndef __INTEGER_MATRIX_CLASS__
#define __INTEGER_MATRIX_CLASS__


#include "normal_matrix_class.h"
#include "nfs_definitions.h"
#include <stdlib.h>

/*!
\brief	Class of matrix with long integer coefficients.

	This matrix serves for matrix computation in circumstances where we are absolutely
	sure that the result fits in the long int type, e.g. when computing modulo a prime.
*/

class IntegerMatrix : public NormalMatrix {
public:
	IntegerMatrix();
	IntegerMatrix(long aRows, long aColumns);

	int  ToHermiteNormalForm(void);
	int  ToHermiteNormalForm(long aModulus);
	int  KernelModPrime(IntegerMatrix *aTarget, long *aDimension, long aModulus, bool aOnMyself);
	int  ImageModPrime(IntegerMatrix *aTarget, long *aDimension, long aModulus);
	int  SupplementModPrime(IntegerMatrix *aTarget, long aDimension, long aModulus);
	int  InverseFromTriangularModPrime(IntegerMatrix *aTarget, long aPrime);
	int  InverseImageOfMatrixModPrime(IntegerMatrix *aTarget, IntegerMatrix *aImage, long aPrime, bool aOnMyself);

        int  MultiplyVector(long *aTarget, long *aSource);                                                                                        
        int  VectorMultiply(long *aTarget, long *aSource);   

protected:                                                                                                                                       
        int WriteData(xmlTextWriterPtr aWriter) const;                                                                                            
        int ReadData(xmlTextReaderPtr aReader);
	// De/Serialization methods
	//int Serialize(const CBaseParameters& aParam) const;
	//int Serialize(const CBaseParameters& aParam, xmlTextWriterPtr& aWriter) const;

	//int Deserialize(const CBaseParameters& aParam);
	//int Deserialize(const CBaseParameters& aParam, xmlTextReaderPtr& aReader);

};

#endif

