#ifndef _INTEGER_SPARSE_H_
#define _INTEGER_SPARSE_H_

#include "common_utils.h"
#include "base_int_matrix.h"
#include "gmp.h"


/*!
 * \brief Sparse matrix with integers as elements
 */ 
class IntegerSparseMatrix : public BaseIntMatrix {
public:
	IntegerSparseMatrix();
	~IntegerSparseMatrix();
	IntegerSparseMatrix(long aRows, long aColumns);
	int Allocate();

	IntegerSparseMatrix * clone();
	virtual int Copy(IntegerSparseMatrix *aSource);
	virtual int Transpose(IntegerSparseMatrix *aSource);
	virtual int Transpose(AbstractMatrix *aSource);

	virtual int PutValue(long aRow, long aColumn, integer_matrix_type aValue);
	virtual int GetValue(long aRow, long aColumn, integer_matrix_type & aValue);

	int SparseNonzeroRows(long & aNonzero);
	int SparseNonzeroColumns(long aRowIndex, long & aRow, long & aNonzero);
	int SparseGetValue(long aRowIndex,long aColumnIndex, long & aColumn, integer_matrix_type & aValue);

	virtual int  Zeroize(); ///< This method will be used for initialization of the matrix by 0s.
	//virtual bool Equals(AbstractMatrix* aMatrix) = 0;
	int ApplyModulo();
	virtual void PrintToScreen(); ///< This method will be used for display of the matrix.
	virtual int  PutOne(long aRow, long aColumn); ///< This method will be used to put number 1 to the entry indexed by aRow and aColumn.
	virtual int  PutZero(long aRow, long aColumn); ///< This method will be used to put number 0 to the entry indexed by aRow and aColumn.
	virtual int  IsOne(long aRow, long aColumn); ///< This method will respond whether there is a 1 at the entry.
	virtual int  IsZero(long aRow, long aColumn); ///< This method will respond whether there is a 0 at the entry.


	virtual long MaxNonZeroItemsPerRow(); ///< Find maximal nonzero iterms per row.
	virtual long CountNonZeroItemsOnRow(long aRow); ///< Count number of nonzero cells in the row
	virtual long IterateRow(long aRow,long & aRowIndex, long aColumnIndex, long & aColumn, integer_matrix_type & aValue); ///< Iterate through nonzero items on row
	virtual long CountNonZeroItems(); ///< Count number of nonzero cells in the matrix.
	virtual long CountNeededMemory(); ///< Count needed memory.



	// Swaps etc.
	virtual int  ZeroizeRow(long aRow);; ///< This method will be used to put 0s into a row given by index aRow.

	virtual int  SwapRows(long aRow1, long aRow2); ///< 			This method will be used to swap rows with indices given by aRow1 and aRow2 parameters.

	virtual int  AddRows(long aTarget, long aSource); ///< 			This method will be used to add row with index aSource to row with index aTarget.
	virtual int  SubtractRowMultiple(long aTargetRow, long aSourceRow, integer_matrix_type aMul);

	virtual int  MultiplyRow(long aRow, integer_matrix_type aMul);

	virtual int  ZeroizeColumn(long aColumn); ///< 			This method will be used to put 0s into a column given by index aColumn.
	virtual int  ZeroizeColumn(long* aColumnList, long aListMaxIndex); ///<  This method will be used to put 0s into columns, whose indices are given by array aColumnList.			The aListMaxIndex parameter gives the final index in aColumnList array, in order to prevent			buffer overflow error.

	virtual int  SwapColumns(long aColumn1, long aColumn2);; ///< This method will be used to swap columns with indices given by aColumn1 and aColumn2 parameters.
	virtual int  AddColumns(long aTarget, long aSource); ///< This method will be used to add column with index aSource to column with index aTarget.

	virtual int  AddToRow(long aRow,AbstractMatrix* aOperand, long aRow2); ///< This method will be used to add rows from two different matrices of the same type.
	virtual int  AddToRow(long aRow,AbstractIntegerMatrix* aOperand, long aRow2); ///< This method will be used to add rows from two different matrices of the same type.

	virtual int  Save(char *aName);
	virtual int  Load(char *aName);

	virtual int WriteData(xmlTextWriterPtr aWriter) const;
	virtual int ReadData(xmlTextReaderPtr aReader);

	int Test();
	int PrintStructure(long aLimit = -1);
	int CheckMon();
private:

	long BinarySearchRow(long aRow); //! Find row index in sparse matrix
	long BinarySearchColumn(long aRowIndex, long aColumn); //! Find column index in sparse matrix

	mpz_t tempSum;
	mpz_t tempMul;
	integer_matrix_type* auxRow;

	long max_allocated_row_index;
	integer_matrix_type** matrix;

};

#endif //  _INTEGER_SPARSE_H_
