#include "job_info.h"
#include "job_parameters.h"
#include <string.h> // strrchr fun
#include <time.h>

JobInfo::~JobInfo()
{
}

JobInfo::JobInfo(JobParameters* aParameters, const string& aFileName, TUint32 aCrc, TPriority aPriority) : iParameters(aParameters), iInfoSource(EFile), iJobStatus(EJobWaiting), iPriority(aPriority), iCrc(aCrc)
{
	iFileNameFull.assign(aFileName);
	const char* ptrStart = aFileName.c_str();
	const char* ptrExten = strrchr(ptrStart,'.');
	if (ptrExten != NULL)
	{
		iFileName_NoExtension.assign(ptrStart,0,ptrExten-ptrStart);
	}	
}


JobInfo::JobInfo(JobParameters* aParameters, const string& aFileName, TInfoSource aSource, TPriority aPriority) : iParameters(aParameters), iInfoSource(aSource), iJobStatus(EJobWaiting), iPriority(aPriority), iCrc(0)
{
	iFileNameFull.assign(aFileName);
	const char* ptrStart = aFileName.c_str();
	const char* ptrExten = strrchr(ptrStart,'.');
	if (ptrExten != NULL)
	{
		iFileName_NoExtension.assign(ptrStart,0,ptrExten-ptrStart);
	}	
}

void	JobInfo::Set_Running()
{
	iJobStatus = EJobRunning;
}

void	JobInfo::Set_StartedNow()
{
	iStartupTime = time(NULL);
}
	
const JobParameters&	JobInfo::Get_Parameters() const
{
	return *iParameters;
}

bool			JobInfo::Is_Running() const
{
	return (iJobStatus == EJobRunning);
}

bool			JobInfo::Is_Waiting() const
{
	return (iJobStatus == EJobWaiting);
}

bool			JobInfo::Is_Finished() const
{
	return (iJobStatus == EJobFinished);
}

bool			JobInfo::Has_AbsolutePriority() const
{
	return (iPriority == EAbsolute);
}

TInfoSource		JobInfo::Get_InfoSource() const
{
	return iInfoSource;
}

bool			JobInfo::Is_FileBased() const
{
	return (iInfoSource == EFile);
}

bool			JobInfo::Is_FromFile(const string& aFileNameFull) const

{
	bool res = (Is_FileBased() && (!iFileNameFull.compare(aFileNameFull)));
// 	printf("JobInfo: is from file: %d, iFileNameFull: %s, aFileNameFull: %s, compare = %d, res = %d.\n",Is_FileBased(), iFileNameFull.c_str(), aFileNameFull.c_str(),iFileNameFull.compare(aFileNameFull),res);
	return res ;
}

void			JobInfo::Set_Parameters(JobParameters* aParameters)
{
	if (iParameters)
	{	
		delete iParameters;
		iParameters = NULL;
	}
	iParameters = aParameters;
}

TJobStatus		JobInfo::Get_JobStatus() const
{
	return iJobStatus;
}

TPriority		JobInfo::Get_Priority() const
{
	return iPriority;
}

time_t			JobInfo::Get_CreationTime() const
{
	return iCreationTime;
}

time_t			JobInfo::Get_StartupTime() const
{
	return iStartupTime;
}

time_t			JobInfo::Get_FinishedTime() const
{
	return iFinishedTime;
}

string			JobInfo::Get_FileNameFull() const
{
	return iFileNameFull;
}

string			JobInfo::Get_FileName_NoExtension() const
{
	return iFileName_NoExtension;
}

TUint32			JobInfo::Get_Crc() const
{
	return iCrc;
}

void			JobInfo::Set_Crc(TUint32 aCrc)
{
	iCrc = aCrc;
}

string			JobInfo::Get_JobId() const
{
	return iJobId;
}

void			JobInfo::Set_JobId(const string& aId)
{
	iJobId.assign(aId);
}

