#ifndef __JOB_INFO_H__
#define __JOB_INFO_H__

#include <string>

using namespace std;

class JobParameters;

enum TInfoSource { ECommandLine, EFile };

enum TJobStatus  { EJobNotAllowed, EJobWaiting, EJobRunning, EJobFinished };

enum TPriority	 { ELow, EStandard, EAbsolute };

typedef unsigned int TUint32 ;

/*!
 * \brief Job information
 */
class JobInfo
{
public:
	JobInfo(JobParameters* aParameters, const string& aFileName, TInfoSource aSource, TPriority aPriority = EStandard);

	// This is only for file-based jobs. CRC means the CRC of the file that the job was loaded from.
	JobInfo(JobParameters* aParameters, const string& aFileName, TUint32 aCrc, TPriority aPriority = EStandard);

	~JobInfo();
	
	const JobParameters&	Get_Parameters() const;
	bool			Is_Running() const;
	bool			Is_Waiting() const;
	bool			Is_Finished() const;
	bool			Has_AbsolutePriority() const;
	TInfoSource		Get_InfoSource() const;
	bool			Is_FileBased() const;
	bool			Is_FromFile(const string& aFileNameFull) const;
	TJobStatus		Get_JobStatus() const;
	TPriority		Get_Priority() const;
	TUint32			Get_Crc() const;
	string			Get_JobId() const;
	

	time_t			Get_CreationTime() const;
	time_t			Get_StartupTime() const;
	time_t			Get_FinishedTime() const;

	string			Get_FileNameFull() const;
	string			Get_FileName_NoExtension() const;

	void			Set_Running();
	void			Set_StartedNow();

	void			Set_Parameters(JobParameters* aParameters);
	void			Set_Crc(TUint32 aCrc);
	void			Set_JobId(const string& aValue);

private:
	JobParameters*		iParameters;
	string			iFileNameFull;
	string			iFileName_NoExtension;
	
	bool			iParametersComplete;
	time_t			iCreationTime;
	time_t			iStartupTime;
	time_t			iFinishedTime;
	
	TInfoSource		iInfoSource;
	TJobStatus		iJobStatus;
	TPriority		iPriority;

	TUint32			iCrc;

	string			iJobId; ///< Generated by the Center
};

#endif
