#ifndef __JOB_PARAMETERS_H__
#define __JOB_PARAMETERS_H__

#include <string>
//#include <cmath>
# include <math.h>
#include <climits>
#include "gmp.h"
#include "types_common.h"

using namespace std;

const int KIntegerParameterUndefined = INT_MIN;

const double KDoubleParameterUndefined = NAN;

const string KStringParameterUndefined = "Parameter Undefined";

/*!
 * \brief Parameter parsing for job management
 */
class JobParameters
{
public:
	virtual ~JobParameters() {};

	virtual const string  GetStringParameter(const char* aName) const = 0;
	virtual const string  GetStringParameter(const string& aName) const;

	virtual int      GetIntegerParameter(const char* aName) const = 0;
	virtual int      GetIntegerParameter(const string& aName) const;

	virtual double   GetDoubleParameter(const char* aName) const = 0;
	virtual double   GetDoubleParameter(const string& aName) const;

	virtual bool     GetBoolParameter(const char* aName) const = 0;
	virtual bool     GetBoolParameter(const string& aName) const;

	virtual int     GetMpzParameter(const char* aName, mpz_t aResult) const = 0;
	virtual int     GetMpzParameter(const string& aName, mpz_t aResult) const;

	virtual bool	 Complete() const = 0;
	virtual string	 GetFailureReason() const = 0;

	virtual JobParameters*	CreateCopy() const = 0;
	

	distribution	Get_DistributionMachineType() const;
	string		Get_SharedURL() const;	
	string		Get_MachineName() const;	
	int		Get_CommitInterval() const;	

	void		Set_DistributionMachineType(distribution aValue);
	void		Set_SharedURL(const string& aValue);	
	void		Set_SharedURL(const char* aValue);	
	void		Set_MachineName(const string& aValue);	
	void		Set_MachineName(const char* aValue);
	void		Set_CommitInterval(int aValue);

	virtual		void Print() const = 0;
	// brief printing of "in batch varying" parameters
	virtual		void PrintInBatch(const char *aPrefix) const = 0;

private:

	bool		iIsDistributed;
	distribution	iDistributionMachineType;
	string		iSharedURL;	
	string		iMachineName;	
	int		iCommitInterval;	

};

#endif
