#include <cstring>

#include "lanczos_class.h"
#include "common_utils.h"
#include "sparse_matrix_class.h"

#include "log.h"

Lanczos::Lanczos()
{
	this->iInfoMode      = false;
	this->iAssertionMode = false;
	this->iRankCalculationMode = false;
	this->iDensityMode = false;
	this->iTimeMessageMode = false;
	this->iCheckFinalResult = false;
	this->iMatrixType    = SBCMatrix;
	this->iMultiType     = ENormalMul;
	this->iAuxiliaryMatricesType = SBitMatrix;
	this->iType = ELanczos;
}

Lanczos::Lanczos(species_of_matrices aType)
{
	this->iInfoMode      = false;
	this->iAssertionMode = false;
	this->iRankCalculationMode = false;
	this->iDensityMode = false;
	this->iTimeMessageMode = false;
	this->iCheckFinalResult = false;
	this->iMatrixType    = aType;
	this->iMultiType     = ENormalMul;
	this->iAuxiliaryMatricesType = SBitMatrix;
	this->iType = ELanczos;
	// addresses of methods for multiplications
}

Lanczos::Lanczos(species_of_matrices aType, multi_type aType2)
{
	this->iInfoMode      = false;
	this->iAssertionMode = false;
	this->iRankCalculationMode = false;
	this->iDensityMode = false;
	this->iMatrixType    = aType;
	this->iMultiType     = aType2;
	this->iAuxiliaryMatricesType = SBitMatrix;
	this->iType = ELanczos;
	// addresses of methods for multiplications
}

Lanczos::~Lanczos(){
}

species_of_matrices Lanczos::GetAuxiliaryMatricesType() const {
	return this->iAuxiliaryMatricesType;
}

multi_type Lanczos::GetMultiType() const {
	return this->iMultiType;
}

void Lanczos::SetAuxiliaryMatricesType(species_of_matrices aType) {
	this->iAuxiliaryMatricesType = aType;
}

void Lanczos::SetMultiType(multi_type aType) {
	this->iMultiType = aType;
}

/*
Puvodni reseni zbytecne obsahovalo double pointer na matici jako parametr. Muze to byt nebezpecne,
nebot neinicializovany pointer nemusi byt roven NULL.
Marian, 27.12.2004
*/
AbstractMatrix*   Lanczos::NewMatrix(long aRow, long aCol){
	if (this->iAuxiliaryMatricesType == SBitMatrix)
		return new BitMatrix(aRow,aCol);
	else if (this->iAuxiliaryMatricesType == SBCMatrix)
		return new BCMatrix(aRow,aCol);
	else if (this->iAuxiliaryMatricesType == SNormalMatrix)
		return new NormalMatrix(aRow,aCol);
	return NULL;
}

BitMatrix* Lanczos::Compute(AbstractMatrix* aMatrixB){
#ifdef LANCZOS_TIME_MESSAGE
	// time measurement only
	AbstractMatrix::start = 0;
	AbstractMatrix::total_big_multiply = 0;
	AbstractMatrix::total_word_multiply = 0;
	AbstractMatrix::total_word_transpose_multiply = 0;
	AbstractMatrix::total_transpose = 0;
	AbstractMatrix::total_add = 0;
	AbstractMatrix::total_zeroize = 0;
	AbstractMatrix::total_copy = 0;
	AbstractMatrix::total_iszero = 0;
#endif
	SparseMatrix* SMatrixB = NULL;
	BitMatrix*    BMatrixB = NULL;
#ifdef CHECK_SPARSE_MATRICES
	BitMatrix*    B1 = NULL;
	NormalMatrix* N1 = NULL;
	NormalMatrix* N3 = NULL;
#endif

	switch (this->iMatrixType){
		//case EBCMatrix:
		case SSparseMatrix:	SMatrixB    = (SparseMatrix*) aMatrixB;
#ifdef CHECK_SPARSE_MATRICES
					N1 = SMatrixB->ToNormalMatrix();
					B1 = new BitMatrix(N1->GetMaximalRowIndex()+1,N1->GetMaximalColumnIndex()+1);
					BMatrixB = BitMatrix::ToBitMatrix(N1);
					N3 = BMatrixB->ToNormalMatrix();
					if (!N1->Equals(N3)){
						printf("\nN1 and N3 are not equal !!\n");
					} else printf("\nN1 and N3 are equal.\n");

					delete B1;
					B1 = NULL;
					delete N1;
					N1 = NULL;
					delete N3;
					N3 = NULL;
#endif
					break;
		case SBitMatrix:	BMatrixB    = (BitMatrix*) aMatrixB;
					break;
		default:
					
					CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,1449,"Erroneous sieving matrix type");
					break;
	}
	int ret;
	int boolean = 1; // int_result = 0; overflow = 0, 
	int rank = 0, last_rank = 0, krylov_subspace = 0; // , shift = 0, rank_2 = 0;
	AbstractMatrix *partial_v[4]; /* staticke pole ukazatelu na mezivysledky. budeme uchovavat 4 */
	AbstractMatrix *inv_partial_w[4], *aux;/**testovaci[5],*testovaci_transpose[5];*/
	AbstractMatrix *y,*x_minus_y,*v_zero;
	BitMatrix *transpose;
	AbstractMatrix *big_temp_1, *big_temp_2, *big_temp_3, *big_temp_4;
	AbstractMatrix *temp1,*temp2,*temp3,*temp4;
	AbstractMatrix *old_vitavi,*old_vitaquadvi, *new_vitavi, *new_vitaquadvi, *avi;
	AbstractMatrix *repository;
	// New variables serving for faster calculation of X-Y
	AbstractMatrix *matrix_d;
	AbstractMatrix *matrix_e;
	AbstractMatrix *matrix_f;
	AbstractMatrix *inner_products[4];
	/*printf("Auxiliary matrices type: ");
	if (iAuxiliaryMatricesType == EBitMatrix)
	{
		printf(" Bit Matrix\n");
	} 
	if (iAuxiliaryMatricesType == EBCMatrix)
	{
		printf(" BC Matrix\n");
	}*/
//#define NORMAL_MATRIX_ASSERTION
#ifdef NORMAL_MATRIX_ASSERTION
	NormalMatrix  *norm_partial_v[4], *norm_inv_partial_w[4], *norm_aux;
	NormalMatrix  *norm_y, *norm_x_minus_y, *norm_v_zero;
	NormalMatrix  *norm_transpose;
	NormalMatrix  *norm_big_temp_1, *norm_big_temp_2, *norm_big_temp_3, *norm_big_temp_4;
	NormalMatrix  *norm_temp1, *norm_temp2, *norm_temp3, *norm_temp4;
	NormalMatrix  *norm_old_vitavi, *norm_old_vitaquadvi, *norm_new_vitavi, *norm_new_vitaquadvi, *norm_avi;
	NormalMatrix  *norm_repository;
	NormalMatrix  *norm_matrix_d;
	NormalMatrix  *norm_matrix_e;
	NormalMatrix  *norm_matrix_f;
	NormalMatrix  *norm_inner_products[4];
#endif
	
	long i, j, N, k, cycle, word;
	long *permutation,*permutation2, *permutation3, *statistics;


	/* Basic constants */
	N    = aMatrixB->GetMaximalRowIndex();
        word = 8*sizeof(matrix_type);

	// long row_number_big = N+1;
	// long col_number_big = word;
	// long row_number_small=word;
	// long col_number_small=word;
	
	/* We will represent matrices S_{i-1} and S_{i} as single machine words with 1's on corresponding columns */	
	matrix_type alternate_s_i_minus_1_representation = 0;	
	matrix_type alternate_s_i_representation = 0;	

	/* Allocations follow */
	statistics = NULL;
	statistics = (long*) Utils::AllocationMachine((void **) &(statistics),(word+1)*sizeof(long));
	if (!statistics){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word+1,303,"There was not enough memory to allocate statistics");
		return NULL;
	}
	memset(statistics,0,(word+1)*sizeof(long));
	for (i=0; i<=3; i++){
		partial_v[i] = this->NewMatrix(N+1, word);
		//partial_v[i]=new BCMatrix(N+1,word);
		if (partial_v[i] == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,304,"There was not enough memory to allocate partial_v[i]");
			return NULL;
		}
		ret = partial_v[i]->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,305,"There was not enough memory to allocate partial_v[i]");
			return NULL;
		}
		inv_partial_w[i] = this->NewMatrix(word,word);
		//inv_partial_w[i] = new BCMatrix(word,word);
		if (inv_partial_w[i] == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,306,"There was not enough memory to allocate inv_partial_w[i]");
			return NULL;
		}
		ret = inv_partial_w[i]->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,307,"There was not enough memory to allocate inv_partial_w[i]");
			return NULL;
		}
		inner_products[i] = this->NewMatrix(word,word);
//		inner_products[i] = new BCMatrix(word,word);
		if (inner_products[i] == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,3106,"There was not enough memory to allocate inner_products[i]");
			return NULL;
		}
		ret = inner_products[i]->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,3107,"There was not enough memory to allocate inner_products[i]");
			return NULL;
		}
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			norm_partial_v[i] = new NormalMatrix(N+1, word);
			//partial_v[i]=new BCMatrix(N+1,word);
			if (norm_partial_v[i] == NULL){
				CLog::LogError(WHEREARG,"Block Lanczos","Block Lanczos method",i,304,"There was not enough memory to allocate partial_v[i]");
				return NULL;
			}
			ret = norm_partial_v[i]->Allocate();
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,305,"There was not enough memory to allocate partial_v[i]");
				return NULL;
			}
			norm_inv_partial_w[i] = new NormalMatrix(word,word);
			//inv_partial_w[i] = new BCMatrix(word,word);
			if (norm_inv_partial_w[i] == NULL){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,306,"There was not enough memory to allocate inv_partial_w[i]");
				return NULL;
			}
			ret = norm_inv_partial_w[i]->Allocate();
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,307,"There was not enough memory to allocate inv_partial_w[i]");
				return NULL;
			}
			norm_inner_products[i] = new NormalMatrix(word,word);
	//		inner_products[i] = new BCMatrix(word,word);
			if (norm_inner_products[i] == NULL){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,3106,"There was not enough memory to allocate inner_products[i]");
				return NULL;
			}
			ret = norm_inner_products[i]->Allocate();
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,3107,"There was not enough memory to allocate inner_products[i]");
				return NULL;
			}
		}
#endif	
	} // end of for(i=0 ...) - allocation of partial_v[i] etc.
	x_minus_y = this->NewMatrix(N+1,word);
//	x_minus_y = new BCMatrix(N+1,word);
	if (x_minus_y == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,308,"There was not enough memory to allocate x_minus_y");
		return NULL;
	}
	ret = x_minus_y->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,309,"There was not enough memory to allocate x_minus_y");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_x_minus_y = new NormalMatrix(N+1,word);
	//	x_minus_y = new BCMatrix(N+1,word);
		if (norm_x_minus_y == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,308,"There was not enough memory to allocate x_minus_y");
			return NULL;
		}
		ret = norm_x_minus_y->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,309,"There was not enough memory to allocate x_minus_y");
			return NULL;
		}
	}
#endif
	v_zero = this->NewMatrix(N+1,word);
//	v_zero = new BCMatrix(N+1,word);
	if (v_zero == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,310,"There was not enough memory to allocate v_zero");
		return NULL;
	}
	ret = v_zero->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,311,"There was not enough memory to allocate v_zero");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_v_zero = new NormalMatrix(N+1,word);
	//	v_zero = new BCMatrix(N+1,word);
		if (norm_v_zero == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,310,"There was not enough memory to allocate v_zero");
			return NULL;
		}
		ret = norm_v_zero->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,311,"There was not enough memory to allocate v_zero");
			return NULL;
		}	
	}
#endif
	transpose = new BitMatrix(word,N+1);
	if (transpose == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,312,"There was not enough memory to allocate transpose");
		return NULL;
	}
	ret = transpose->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,313,"There was not enough memory to allocate transpose");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_transpose = new NormalMatrix(word,N+1);
		if (norm_transpose == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,312,"There was not enough memory to allocate transpose");
			return NULL;
		}
		ret = norm_transpose->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,313,"There was not enough memory to allocate transpose");
			return NULL;
		}
	}
#endif
	y = this->NewMatrix(N+1,word);
//	y = new BCMatrix(N+1,word);
	if (y == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,316,"There was not enough memory to allocate y");
		return NULL;
	}
	ret = y->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,317,"There was not enough memory to allocate y");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_y = new NormalMatrix(N+1,word);
		//	y = new BCMatrix(N+1,word);
		if (norm_y == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos 	method",N+1,316,"There was not enough memory to allocate y");
			return NULL;
		}
		ret = norm_y->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,317,"There was not enough memory to allocate y");
			return NULL;
		}
	}
#endif
	big_temp_1 = this->NewMatrix(N+1,word);
//	big_temp_1 = new BCMatrix(N+1,word);
	if (big_temp_1 == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,318,"There was not enough memory to allocate big_temp_1");
		return NULL;
	}
	ret = big_temp_1->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,319,"There was not enough memory to allocate big_temp_1");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_big_temp_1 = new NormalMatrix(N+1,word);
//	big_temp_1 = new BCMatrix(N+1,word);
		if (norm_big_temp_1 == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,318,"There was not enough memory to allocate big_temp_1");
			return NULL;
		}
		ret = norm_big_temp_1->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,319,"There was not enough memory to allocate big_temp_1");
			return NULL;
		}
	}
#endif
	big_temp_2 = this->NewMatrix(N+1,word);
//	big_temp_2 = new BCMatrix(N+1,word);
	if (big_temp_2 == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,318,"There was not enough memory to allocate big_temp_2");
		return NULL;
	}
	ret = big_temp_2->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,319,"There was not enough memory to allocate big_temp_2");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_big_temp_2 = new NormalMatrix(N+1,word);
	//	big_temp_2 = new BCMatrix(N+1,word);
		if (norm_big_temp_2 == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,318,"There was not enough memory to allocate big_temp_2");
			return NULL;
		}
		ret = norm_big_temp_2->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,319,"There was not enough memory to allocate big_temp_2");
			return NULL;
		}
	}
#endif
	big_temp_3 = this->NewMatrix(N+1,word);
//	big_temp_3 = new BCMatrix(N+1,word);
	if (big_temp_3 == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,320,"There was not enough memory to allocate big_temp_3");
		return NULL;
	}
	ret = big_temp_3->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,321,"There was not enough memory to allocate big_temp_3");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_big_temp_3 = new NormalMatrix(N+1,word);
	//	big_temp_3 = new BCMatrix(N+1,word);
		if (norm_big_temp_3 == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,320,"There was not enough memory to allocate big_temp_3");
			return NULL;
		}
		ret = norm_big_temp_3->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,321,"There was not enough memory to allocate big_temp_3");
			return NULL;
		}
	}
#endif
	big_temp_4 = this->NewMatrix(N+1,word);
//	big_temp_4 = new BCMatrix(N+1,word);
	if (big_temp_4 == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,322,"There was not enough memory to allocate big_temp_4");
		return NULL;
	}
	ret = big_temp_4->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,323,"There was not enough memory to allocate big_temp_4");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_big_temp_4 = new NormalMatrix(N+1,word);
	//	big_temp_4 = new BCMatrix(N+1,word);
		if (norm_big_temp_4 == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,322,"There was not enough memory to allocate big_temp_4");
			return NULL;
		}
		ret = norm_big_temp_4->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,323,"There was not enough memory to allocate big_temp_4");
			return NULL;
		}
	}
#endif
	temp1 = this->NewMatrix(word,word);
//	temp1 = new BCMatrix(word,word);
	if (temp1 == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,328,"There was not enough memory to allocate temp1");
		return NULL;
	}
	ret = temp1->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,329,"There was not enough memory to allocate temp1");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_temp1 = new NormalMatrix(word,word);
	//	temp1 = new BCMatrix(word,word);
		if (norm_temp1 == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,328,"There was not enough memory to allocate temp1");
			return NULL;
		}
		ret = norm_temp1->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,329,"There was not enough memory to allocate temp1");
			return NULL;
		}
	}
#endif
	temp2 = this->NewMatrix(word,word);
//	temp2 = new BCMatrix(word,word);
	if (temp2 == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,330,"There was not enough memory to allocate temp2");
		return NULL;
	}
	ret = temp2->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,331,"There was not enough memory to allocate temp2");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_temp2 = new NormalMatrix(word,word);
	//	temp2 = new BCMatrix(word,word);
		if (norm_temp2 == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,330,"There was not enough memory to allocate temp2");
			return NULL;
		}
		ret = norm_temp2->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,331,"There was not enough memory to allocate temp2");
			return NULL;
		}
	}
#endif
	temp3 = this->NewMatrix(word,word);
//	temp3 = new BCMatrix(word,word);
	if (temp3 == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,332,"There was not enough memory to allocate temp3");
		return NULL;
	}
	ret = temp3->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,333,"There was not enough memory to allocate temp3");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_temp3 = new NormalMatrix(word,word);
	//	temp3 = new BCMatrix(word,word);
		if (norm_temp3 == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,332,"There was not enough memory to allocate temp3");
			return NULL;
		}
		ret = norm_temp3->Allocate();
		if (ret	!= ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,333,"There was not enough memory to allocate temp3");
			return NULL;
		}
	}
#endif
	temp4 = this->NewMatrix(word,word);
//	temp4 = new BCMatrix(word,word);
	if (temp4 == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,334,"There was not enough memory to allocate temp4");
		return NULL;
	}
	ret = temp4->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,335,"There was not enough memory to allocate temp4");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_temp4 = new NormalMatrix(word,word);
	//	temp4 = new BCMatrix(word,word);
		if (norm_temp4 == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,334,"There was not enough memory to allocate temp4");
			return NULL;
		}
		ret = norm_temp4->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,335,"There was not enough memory to allocate temp4");
			return NULL;
		}
	}
#endif
	matrix_d = this->NewMatrix(word,word);
//	matrix_d = new BCMatrix(word,word);
	if (matrix_d == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,3124,"There was not enough memory to allocate matrix_d");
		return NULL;
	}
	ret = matrix_d->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,3125,"There was not enough memory to allocate matrix_d");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_matrix_d = new NormalMatrix(word,word);
	//	matrix_d = new BCMatrix(word,word);
		if (norm_matrix_d == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,3124,"There was not enough memory to allocate matrix_d");
			return NULL;
		}
		ret = norm_matrix_d->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,3125,"There was not enough memory to allocate matrix_d");
			return NULL;
		}
	}
#endif
	matrix_e = this->NewMatrix(word,word);
//	matrix_e = new BCMatrix(word,word);
	if (matrix_e == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,3126,"There was not enough memory to allocate matrix_e");
		return NULL;
	}
	ret = matrix_e->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,3127,"There was not enough memory to allocate matrix_e");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_matrix_e = new NormalMatrix(word,word);
	//	matrix_e = new BCMatrix(word,word);
		if (norm_matrix_e == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,3126,"There was not enough memory to allocate matrix_e");
			return NULL;
		}
		ret = norm_matrix_e->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,3127,"There was not enough memory to allocate matrix_e");
			return NULL;
		}
	}
#endif
	matrix_f = this->NewMatrix(word,word);
//	matrix_f = new BCMatrix(word,word);
	if (matrix_f == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,3128,"There was not enough memory to allocate matrix_f");
		return NULL;
	}
	ret = matrix_f->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,3129,"There was not enough memory to allocate matrix_f");
		return NULL;
	}	
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_matrix_f = new NormalMatrix(word,word);
	//	matrix_f = new BCMatrix(word,word);
		if (norm_matrix_f == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,3128,"There was not enough memory to allocate matrix_f");
			return NULL;
		}
		ret = norm_matrix_f->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,3129,"There was not enough memory to allocate matrix_f");
			return NULL;
		}	
	}
#endif
	old_vitavi = this->NewMatrix(word,word);
//	old_vitavi = new BCMatrix(word,word);
	if (old_vitavi == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,336,"There was not enough memory to allocate old_vitavi");
		return NULL;
	}
	ret = old_vitavi->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,337,"There was not enough memory to allocate old_vitavi");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_old_vitavi = new NormalMatrix(word,word);
		//	old_vitavi = new BCMatrix(word,word);
		if (norm_old_vitavi == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,336,"There was not enough memory to allocate old_vitavi");
			return NULL;
		}
		ret = norm_old_vitavi->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,337,"There was not enough memory to allocate old_vitavi");
			return NULL;
		}
	}
#endif
	old_vitaquadvi = this->NewMatrix(word,word);
//	old_vitaquadvi = new BCMatrix(word,word);
	if (old_vitaquadvi == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,338,"There was not enough memory to allocate old_vitaquadvi");
		return NULL;
	}
	ret = old_vitaquadvi->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,339,"There was not enough memory to allocate old_vitaquadvi");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_old_vitaquadvi = new NormalMatrix(word,word);
	//	old_vitaquadvi = new BCMatrix(word,word);
		if (norm_old_vitaquadvi == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,338,"There was not enough memory to allocate old_vitaquadvi");
			return NULL;
		}
		ret = norm_old_vitaquadvi->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,339,"There was not enough memory to allocate old_vitaquadvi");
			return NULL;
		}	
	}
#endif
	new_vitavi = this->NewMatrix(word,word);
//	new_vitavi = new BCMatrix(word,word);
	if (new_vitavi == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,340,"There was not enough memory to allocate new_vitavi");
		return NULL;
	}
	ret = new_vitavi->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,341,"There was not enough memory to allocate new_vitavi");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_new_vitavi = new NormalMatrix(word,word);
	//	new_vitavi = new BCMatrix(word,word);
		if (norm_new_vitavi == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,340,"There was not enough memory to allocate new_vitavi");
			return NULL;
		}
		ret = norm_new_vitavi->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,341,"There was not enough memory to allocate new_vitavi");
			return NULL;
		}
	}
#endif
	new_vitaquadvi = this->NewMatrix(word,word);
//	new_vitaquadvi = new BCMatrix(word,word);
	if (new_vitaquadvi == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,342,"There was not enough memory to allocate new_vitaquadvi");
		return NULL;
	}
	ret = new_vitaquadvi->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,343,"There was not enough memory to allocate new_vitaquadvi");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_new_vitaquadvi = new NormalMatrix(word,word);
	//	new_vitaquadvi = new BCMatrix(word,word);
		if (norm_new_vitaquadvi == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,342,"There was not enough memory to allocate new_vitaquadvi");
			return NULL;
		}
		ret = norm_new_vitaquadvi->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,343,"There was not enough memory to allocate new_vitaquadvi");
			return NULL;
		}	
	}
#endif
	avi = this->NewMatrix(N+1,word);
//	avi = new BCMatrix(N+1,word);
	if (avi == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,344,"There was not enough memory to allocate avi");
		return NULL;
	}
	ret = avi->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,345,"There was not enough memory to allocate avi");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_avi = new NormalMatrix(N+1,word);
	//	avi = new BCMatrix(N+1,word);
		if (norm_avi == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,344,"There was not enough memory to allocate avi");
			return NULL;
		}
		ret = norm_avi->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",N+1,345,"There was not enough memory to allocate avi");
			return NULL;
		}	
	}
#endif
	repository = this->NewMatrix(aMatrixB->GetMaximalColumnIndex()+1,word);
//	repository = new BCMatrix(aMatrixB->GetMaximalColumnIndex()+1,word);
	if (repository == NULL){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",aMatrixB->GetMaximalColumnIndex()+1,346,"There was not enough memory to allocate repository");
		return NULL;
	}
	ret = repository->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",aMatrixB->GetMaximalColumnIndex()+1,347,"There was not enough memory to allocate repository");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		norm_repository = new NormalMatrix(aMatrixB->GetMaximalColumnIndex()+1,word);
	//	repository = new BCMatrix(aMatrixB->GetMaximalColumnIndex()+1,word);
		if (norm_repository == NULL){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",aMatrixB->GetMaximalColumnIndex()+1,346,"There was not enough memory to allocate repository");
			return NULL;
		}
		ret = norm_repository->Allocate();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",aMatrixB->GetMaximalColumnIndex()+1,347,"There was not enough memory to allocate repository");
			return NULL;
		}
	}
#endif
	permutation = NULL;
	permutation = (long*) Utils::AllocationMachine((void **) &(permutation),word*sizeof(long));
	if (!permutation){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,1347,"There was not enough memory to allocate permutation");
		return NULL;
	}
	memset(permutation,0,word*sizeof(long));
	permutation2 = NULL;
	permutation2 = (long*) Utils::AllocationMachine((void **) &(permutation2),word*sizeof(long));
	if (!permutation2){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,1348,"There was not enough memory to allocate permutation2");
		return NULL;
	}
	memset(permutation2,0,word*sizeof(long));
	permutation3 = NULL;
	permutation3 = (long*) Utils::AllocationMachine((void **) &(permutation3),word*sizeof(long));
	if (!permutation3){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",word,1349,"There was not enough memory to allocate permutation3");
		return NULL;
	}
	memset(permutation3,0,word*sizeof(long));
select:
	// End of allocations, the procedure itself begins //
	if (this->iInfoMode){
		float iter = ((float) aMatrixB->GetMaximalColumnIndex())/ ((float) word - 0.76);
		printf(" %d iterations expected.\n",(int) iter);
		printf("Iteration 0 ...");
	}
	cycle = 0;
	for (k=0; k<=3; k++) {
		ret = partial_v[k]->Zeroize();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,349,"There was a problem zeroizing partial_v");
			return NULL;
		}
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			ret = norm_partial_v[k]->Zeroize();
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,349,"There was a problem zeroizing partial_v");
				return NULL;
			}
		}
#endif
		ret = inv_partial_w[k]->Zeroize();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,350,"There was a problem zeroizing inv_partial_w");
			return NULL;
		}
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			ret = norm_inv_partial_w[k]->Zeroize();
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,349,"There was a problem zeroizing partial_v");
				return NULL;
			}
		}
#endif

		ret = inner_products[k]->Zeroize();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,3150,"There was a problem zeroizing inner_products");
			return NULL;
		}
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			ret = norm_inner_products[k]->Zeroize();
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,349,"There was a problem zeroizing partial_v");
				return NULL;
			}
		}
#endif

	}
/*
	ret = s_i_minus_1->Zeroize();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,350,"There was a problem zeroizing s_i_minus_1");
		return NULL;
	}
	for (k=0; k<= s_i_minus_1->GetMaximalRowIndex(); k++)
	{
		s_i_minus_1->PutOne(k,k);
	}
*/
	alternate_s_i_minus_1_representation = Lanczos::LANCZOS_FULL_MASK;
	//ret = s_i->Zeroize();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,351,"There was a problem zeroizing s_i");
		return NULL;
	}
	//ret = y->Randomize();
	ret=y->Zeroize();
	for(i=0;i<word;i++) y->PutOne(i,i);
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,352,"There was a problem randomizing y");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		ret=norm_y->Zeroize();
		for(i=0;i<word;i++) norm_y->PutOne(i,i);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,352,"There was a problem randomizing y");
			return NULL;
		}
		ret = norm_x_minus_y->Copy(norm_y);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,353,"There was a problem randomizing x_minus_y");
			return NULL;
		}
	}
#endif
#ifdef PRINT_MATRICES	
	printf("New matrix y was generated as:");
	y->PrintToScreen();
#endif
	ret = x_minus_y->Copy(y);

	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,353,"There was a problem randomizing x_minus_y");
		return NULL;
	}
        // Use of late binding - virtual method
        ret = aMatrixB->MultiplyInternalTransposed(repository,aMatrixB,y);
        if (ret != ConstRC::Ok){
                CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,354,"There was a problem multiplying transposed and Y");
                return NULL;
        }
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{

		int i1, max = norm_repository->GetMaximalRowIndex();	
		for (i1=0; i1<= max; i1++)
		{
			int j1, max1 = norm_repository->GetMaximalColumnIndex();
			for (j1 = 0; j1<= max1; j1++)
			{
				if (repository->IsOne(i1,j1))
				{
					norm_repository->PutOne(i1,j1);
				}
				else
				{
					norm_repository->PutZero(i1,j1);
				}
			}
		}
        	ret = aMatrixB->MultiplyInternalTransposed(norm_repository,aMatrixB,norm_y);
        	if (ret != ConstRC::Ok){
                	CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",ret,354,"There was a problem multiplying transposed and Y");
                	return NULL;
        	}
		norm_repository->Modulo(2);
		if (repository->Compare(norm_repository) == NOT_EQUAL)
		{
			repository->PrintToScreen();
			norm_repository->PrintToScreen();
			wait();
		} else {
			printf("\nMultiply Internal Transposed ConstRC::Ok.");
		}
	}
#endif
        // Use of late binding - virtual method
        ret = aMatrixB->MultiplyInternal(partial_v[2],aMatrixB,repository);
        if (ret != ConstRC::Ok){
                CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,355,"There was a problem multiplying B and repository");
                return NULL;
        }
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		ret = aMatrixB->MultiplyInternal(norm_partial_v[2],aMatrixB,norm_repository);
		if (ret != ConstRC::Ok){
                	CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,355,"There was a problem multiplying B and repository");
                	return NULL;
        	}
		norm_partial_v[2]->Modulo(2);
		if (partial_v[2]->Compare(norm_partial_v[2]) == NOT_EQUAL)
		{
			partial_v[2]->PrintToScreen();
			norm_partial_v[2]->PrintToScreen();
			wait();
		} else {
			printf("\nMultiply Internal ConstRC::Ok.");
		}
	}
#endif
	if (partial_v[2]->IsZero()){
		printf("\nStrange. V_0 = AY is zero matrix.\n");
	}

	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,355,"There was a problem multiplying B and repository");
		return NULL;
	}
	ret = v_zero->Copy(partial_v[2]);
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,356,"There was a problem copying partial_v[2] to v_zero");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		ret = norm_v_zero->Copy(norm_partial_v[2]);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,356,"There was a problem copying partial_v[2] to v_zero");
			return NULL;
		}
	}
#endif
#ifdef PRINT_MATRICES	
	printf("Matrix v_zero was calculated as:");
	v_zero->PrintToScreen();
#endif
	// This permutation serves for renumbering of columns of matrix T. See
	//   Montgomery, Fig.1, Pseudocode for selecting S_i and W_i     //
	for (i=0; i<=temp1->GetMaximalRowIndex(); i++) { permutation[i]=i; }
	ret = old_vitavi->Zeroize();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,357,"There was a problem zeroizing old_vitavi");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		ret = norm_old_vitavi->Zeroize();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,357,"There was a problem zeroizing old_vitavi");
			return NULL;
		}
	}
#endif
	ret = old_vitaquadvi->Zeroize();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,358,"There was a problem zeroizing old_vitaquadvi");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		ret = norm_old_vitaquadvi->Zeroize();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,358,"There was a problem zeroizing old_vitaquadvi");
			return NULL;
		}
	}
#endif
	ret = new_vitavi->Zeroize();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,359,"There was a problem zeroizing new_vitavi");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		ret = norm_new_vitavi->Zeroize();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,359,"There was a problem zeroizing new_vitavi");
			return NULL;
		}
	}
#endif
	ret = new_vitaquadvi->Zeroize();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,360,"There was a problem zeroizing new_vitaquadvi");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		ret = norm_new_vitaquadvi->Zeroize();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,360,"There was a problem zeroizing new_vitaquadvi");
			return NULL;
		}
	}
#endif
	ret = matrix_d->Zeroize();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,3600,"There was a problem zeroizing matrix_d");
		return NULL;
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		ret = norm_matrix_d->Zeroize();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,3600,"There was a problem zeroizing new_matrix_d");
			return NULL;
		}
	}
#endif
	ret = matrix_e->Zeroize();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,3600,"There was a problem zeroizing matrix_e");
		return NULL;
	}	

#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		ret = norm_matrix_e->Zeroize();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,3600,"There was a problem zeroizing new_matrix_e");
			return NULL;
		}
	}
#endif
	ret = matrix_f->Zeroize();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,3600,"There was a problem zeroizing matrix_f");
		return NULL;
	}		

#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		ret = norm_matrix_f->Zeroize();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,3600,"There was a problem zeroizing new_matrix_f");
			return NULL;
		}
	}
#endif
	last_rank = word; // promenna urcuje rank matice S_{i-1}. Pri nulte iteraci je to word. //
#ifdef LANCZOS_TIME_MESSAGE
        int ps_start, diff, total_perform = 0, sub_start;
#elif defined  MATRIX_OPERATIONS_TIME_MESSAGE
        int ps_start, diff, total_perform = 0;
#endif
#ifdef LANCZOS_TIME_MESSAGE
        int phase_start, total_f = 0, total_e = 0, total_d = 0, total_s = 0, total_cyclic = 0, total_big = 0;
        int total_init = 0, total_rank = 0, total_xy = 0;
        int total_vitavi = 0, total_vitaquadvi = 0, total_f_end = 0, total_e_end = 0, total_d_end = 0;
        int sub_1 = 0, sub_2 = 0, sub_3 = 0, sub_4 = 0, sub_5 = 0, sub_6 = 0;
#endif
        // The iteration loop itself //
        do{
#ifdef LANCZOS_TIME_MESSAGE
                ps_start = clock();
                phase_start = clock();
#elif defined MATRIX_OPERATIONS_TIME_MESSAGE
                ps_start = clock(); // the total_perform must be collected even if LANCZOS_TIME_MESSAGE is not defined
#endif
	   	if (this->iInfoMode){
		  	printf("\r");
		  	printf("Iteration %ld ...    ", cycle);
      			fflush(stdout);
   	 	}
                // Use of late binding - virtual method
                ret = aMatrixB->MultiplyInternalTransposed(repository,aMatrixB,partial_v[2]);
                if (ret != ConstRC::Ok)
                {
                        CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,361,"There was a problem multiplying transposed and partial_v[2]");
                        return NULL;
                }
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
	                ret = aMatrixB->MultiplyInternalTransposed(norm_repository,aMatrixB,norm_partial_v[2]);
                	if (ret != ConstRC::Ok)
                	{
                        	CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,361,"There was a problem multiplying transposed and partial_v[2]");
                        	return NULL;
                	}
			if (repository->Compare(norm_repository) == NOT_EQUAL)
			{
				repository->PrintToScreen();
				norm_repository->PrintToScreen();
				wait();
			} else {
				printf("\nMultiply Internal Transposed in cycle ConstRC::Ok.");
			}		
		}
#endif
                // Use of late binding - virtual method
                ret = aMatrixB->MultiplyInternal(avi,aMatrixB,repository);
                if (ret != ConstRC::Ok)
                {
                        CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,362,"There was a problem multiplying B and repository");
                        return NULL;
                }
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
	                ret = aMatrixB->MultiplyInternal(norm_avi,aMatrixB,norm_repository);
                	if (ret != ConstRC::Ok)
                	{
                        	CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,361,"There was a problem multiplying transposed and partial_v[2]");
                        	return NULL;
                	}
			if (avi->Compare(norm_avi) == NOT_EQUAL)
			{
				avi->PrintToScreen();
				norm_avi->PrintToScreen();
				wait();
			} else {
				printf("\nMultiply Internal in cycle ConstRC::Ok.");
			}		
		}
#endif

#ifdef LANCZOS_TIME_MESSAGE
		diff = clock() - phase_start;
		if (diff > 0){
			total_big += diff;
		}
		phase_start = clock();
		sub_start = clock();
#endif
		/*if (avi->MultiplyInternal(avi,aSymmetric,partial_v[2]) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,362,"There was a problem multiplying A and V_i");
			return NULL;
		}*/
		/*if (partial_v[2]->Transpose(transpose) != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,363,"There was a problem transposing partial_v[2]");
			return NULL;
		}
		if (new_vitavi->MultiplyInternal(new_vitavi,transpose,avi) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,364,"There was a problem multiplying transpose and avi");
			return NULL;
		}*/
		if (new_vitavi->MultiplyInternalTransposed(new_vitavi,partial_v[2],avi) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,364,"There was a problem multiplying transpose and avi");
			return NULL;
		}
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			if (norm_new_vitavi->MultiplyInternalTransposed(norm_new_vitavi,norm_partial_v[2],norm_avi) != ConstRC::Ok)
			{
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,364,"There was a problem multiplying transpose and avi");
				return NULL;
			}
			norm_new_vitavi->Modulo(2);
			if (norm_new_vitavi->Compare(new_vitavi) == NOT_EQUAL)
			{
				norm_new_vitavi->PrintToScreen();
				new_vitavi->PrintToScreen();
				wait();
			} else
			{
				printf("\nNew vitavi computed ConstRC::Ok.");				
			}	
		}
#endif
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - sub_start;
                if (diff > 0){
                        total_vitavi += diff;
                }
#endif
                ret = temp1->Copy(new_vitavi);
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			norm_temp1->Copy(norm_new_vitavi);
		}
#endif
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - phase_start;
                if (diff > 0){
                        total_init += diff;
                }
                phase_start = clock();
#endif
                /*rank = partial_v[2]->CalculateRank();
                krylov_subspace += rank;
                statistics[rank] += 1;
                printf("  rank %d\n",rank);*/
                boolean = !(temp1->IsZero());
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - phase_start;
                if (diff > 0){
                        total_rank += diff;
                }
                phase_start = clock();
#endif
		//printf("Matrix temp_1 (V_i^T AV_i) vysla jako:\n");
		//temp1->PrintToScreen();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,365,"There was a problem copying new_vitavi to temp1");
			return NULL;
		}
		j = temp1->GetMaximalRowIndex();
		// Numbering of columns c_0, ... c_{N-1} according to Montgomery.
		k=0;
		for (i=0; i< word; i++)
		{
			if (alternate_s_i_minus_1_representation & transpose->K_SHIFTS[i]){
				permutation[j]  = i;
				permutation3[k] = i;
				j--;
				k++;
			}
		}
		for (k=j+1; k<=temp1->GetMaximalRowIndex(); k++)
		{
			permutation[k]=permutation3[k-j-1];
		}
		for (i=0; i<=temp1->GetMaximalRowIndex(); i++)
		{
			permutation2[i]=1;
		}
		for (i=j+1; i<=temp1->GetMaximalRowIndex(); i++)
		{
			permutation2[permutation[i]]=0;
		}
		k=0;
		for (i=0; i<=j; i++)
		{
			while(!permutation2[k])
			{
				k++;
			}
			permutation[i]=k;
			k++;
		}
		// Now our permutation is ready.
#ifdef PRINT_MATRICES				
		printf("Permutation: \n");
		for (i=0; i<word; i++){
			printf("[%2d: %2d]",i,permutation[i]);
		}
		printf("\n");
#endif		
		ret = temp2->Zeroize();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,366,"There was a problem zeroizing temp2");
			return NULL;
		}
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			norm_temp2->Zeroize();	
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,366,"There was a problem zeroizing temp2");
				return NULL;
			}
		}
#endif
		for (i=0; i<= temp2->GetMaximalRowIndex(); i++){
			temp2->PutOne(i,i);
#ifdef NORMAL_MATRIX_ASSERTION
			if (this->iAssertionMode)
			{
				norm_temp2->PutNumber(1,i,i);
			}
#endif
		}

		for (j=0;j<=temp1->GetMaximalRowIndex();j++)
		{
#ifdef PRINT_MATRICES		
			printf("Matice temp1 je pri %d-te iteraci selekce S_i rovna: ",j);
			temp1->PrintToScreen();
			printf("Matice temp2 je pri %d-te iteraci selekce S_i rovna: ",j);
			temp2->PrintToScreen();
#endif
			// Using column renumbering due to Montgomery //
			for (k=j; k<=temp1->GetMaximalRowIndex(); k++)
			{
				if (! temp1->IsOne(permutation[j],permutation[j])){
					if (temp1->IsOne(permutation[k],permutation[j])){
						temp1->SwapRows(permutation[j],permutation[k]);
						temp2->SwapRows(permutation[j],permutation[k]);
					}
				}
			}
			if (temp1->IsOne(permutation[j],permutation[j])){
				for (k=0; k<=temp1->GetMaximalRowIndex(); k++){
					if (k!=j && temp1->IsOne(permutation[k],permutation[j])){
						temp1->AddRows(permutation[k],permutation[j]);
						temp2->AddRows(permutation[k],permutation[j]);
					}
				}
			}
			else // column c_j is a linear combination of the others and we will omit it //
			{
				// looking for a pivot in the right part of matrix M
				for (k=j; k<=(*temp2).GetMaximalRowIndex(); k++)
				{
					if (temp2->IsZero(permutation[j],permutation[j]) && temp2->IsOne(permutation[k],permutation[j])){
						temp1->SwapRows(permutation[j],permutation[k]);
						temp2->SwapRows(permutation[j],permutation[k]);
					}
				}
				// If there is none, the algorithm ends. We must randomize Y again
				if (!temp2->IsOne(permutation[j],permutation[j])){
					if (this->iInfoMode){
						printf("Singular case - restarted with a new random Y.");
						goto select;
					}
				}
				for (k=0; k<=temp2->GetMaximalRowIndex(); k++)
				{
					// zeroize the rest of c_j+N column
					if ((k!=j) && temp2->IsOne(permutation[k],permutation[j]))
					{
						temp1->AddRows(permutation[k],permutation[j]);
						temp2->AddRows(permutation[k],permutation[j]);
					}
				}
				// zeroize the whole column c_j and rows c_j in both temp1 and temp2
				temp1->ZeroizeColumn(permutation[j]);
				temp1->ZeroizeRow(permutation[j]);
				temp2->ZeroizeRow(permutation[j]);
			}
		} // end of S_i selection. Now, S_iSi^T should be in temp1 and  W_inv_i  in temp2
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			ret = norm_temp1->Copy(temp1);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,368,"There was a problem copying temp1 to norm_temp1");
				return NULL;
			}
			ret = norm_temp2->Copy(temp2);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,368,"There was a problem copying temp2 to norm_temp2");
				return NULL;
			}
		}
#endif
		// We'll express S_i alternatively.
		alternate_s_i_representation = 0;
		rank = 0;
		for (j=0;j<=temp1->GetMaximalRowIndex();j++) {
			if (temp1->IsOne(j,j)) {
				alternate_s_i_representation |= BitMatrix::K_SHIFTS[j];
				rank++;
			}
		}
		krylov_subspace += rank;
		statistics[rank] += 1;
		//printf("Alternate s_i expression: 0x%x. Rank is %d\n",alternate_s_i_representation,rank);
		//temp1->PrintToScreen();
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - phase_start;
                if (diff > 0){
                        total_s += diff;
                }
                //printf("  rank %d",rank);
                phase_start = clock();
#endif
		// copy temp1 to s_i and temp2 to w_inv[2]
#ifdef PRINT_MATRICES		
		printf("Matice temp1 pri selekci S_i vysla jako: ");
		temp1->PrintToScreen();
		printf("Matice temp2 pri selekci S_i vysla jako: ");
		temp2->PrintToScreen();
#endif		
/*
		ret = s_i->Copy(temp1);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,367,"There was a problem copying temp1 to s_i");
			return NULL;
		}
*/
		/*rank = 0;
		for (i=0; i<= s_i->GetMaximalRowIndex(); i++){
			if (s_i->IsOne(i,i))	rank++;
		}*/
		ret = inv_partial_w[2]->Copy(temp2);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,368,"There was a problem copying temp2 to inv_partial_w[2]");
			return NULL;
		}
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			ret = norm_inv_partial_w[2]->Copy(norm_temp2);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",0,368,"There was a problem copying temp2 to inv_partial_w[2]");
				return NULL;
			}
		}
#endif
		// if the iteration has not finished yet, we add other elements to X
		if (boolean){
			/*ret = big_temp_1->MultiplyInternal(big_temp_1,partial_v[2],inv_partial_w[2]);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,369,"There was a problem multiplying V_i x W_inv_i");
				return NULL;
			}*/
			/*ret = partial_v[2]->Transpose(transpose);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,370,"There was a problem transposing V_i");
				return NULL;
			}*/
			// Calculation of the "inner product". There are two methods, according to the phase. One of them
			// must be used for cycle <=2 and means full inner product computation. The other one exploits 
			// property observed by Henk Boender (see original Montgomery's article, section 9).
			if (cycle <= 2){
				ret = temp2->MultiplyInternalTransposed(temp2,partial_v[2],v_zero);
				if (ret != ConstRC::Ok){
					CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,371,"There was a problem multiplying V_i^T x AV_0");
					return NULL;
				}
#ifdef NORMAL_MATRIX_ASSERTION
				if (this->iAssertionMode)
				{
					ret = norm_temp2->MultiplyInternalTransposed(norm_temp2,norm_partial_v[2],norm_v_zero);
					if (ret != ConstRC::Ok){
						CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,371,"There was a problem multiplying V_i^T x AV_0");
						return NULL;
					}
					if (norm_temp2->Compare(temp2) == NOT_EQUAL)
					{
						printf("\nSmall cycle multiply transposed failed.");
					} else 
					{
						printf("\nSmall cycle multiply transposed ConstRC::Ok.");
					}
				}
#endif
				ret = inner_products[3]->Copy(temp2);
				if (ret != ConstRC::Ok){
					CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,3171,"There was a problem in copy");
					return NULL;
				}
#ifdef NORMAL_MATRIX_ASSERTION
				if (this->iAssertionMode)
				{
					ret = norm_inner_products[3]->Copy(norm_temp2);
				}
#endif
			} 			
			if (cycle >= 3) {
			        ret = temp1->MultiplyInternalTransposed(temp1,matrix_d,inner_products[2]);
				if (ret != ConstRC::Ok){
					CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,3172,"There was a problem in D_{i+1}^T V_i");
					return NULL;
				}
				// temp1 now includes D_{i+1}^T V_i
				ret = temp2->MultiplyInternalTransposed(temp2,matrix_e,inner_products[1]);
				if (ret != ConstRC::Ok){
					CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,3173,"There was a problem in E_{i+1}^T V_{i-1}");
					return NULL;
				}
				// temp2 now includes E_{i+1}^T V_{i-1}
				ret = temp1->Add(temp3,temp2);
				// temp3 now includes D_{i+1}^T V_i + E_{i+1}^T V_{i-1}
				if (ret != ConstRC::Ok){
					CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,3174,"There was a problem in D_{i+1}^T V_i + E_{i+1}^T V_{i-1}");
					return NULL;
				}
				ret = temp1->MultiplyInternalTransposed(temp1,matrix_f,inner_products[0]);
				// temp1 now includes F_{i+1}^T V_{i-2}
				if (ret != ConstRC::Ok){
					CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,3175,"There was a problem in F_{i+1}^T V_{i-2}");
					return NULL;
				}
				ret = temp1->Add(temp2,temp3);
				// temp2 now includes F_{i+1}^T V_{i-2} + D_{i+1}^T V_i + E_{i+1}^T V_{i-1}
				if (ret != ConstRC::Ok){
					CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,3176,"There was a problem in D_{i+1}^T V_i + E_{i+1}^T V_{i-1} and F_{i+1}^T V_{i-2}");
					return NULL;
				}				
				ret = inner_products[3]->Copy(temp2);
				/*if (!temp2->Equals(inner_products[3])){
					inner_products[3]->PrintToScreen();
					temp2->PrintToScreen();
				}*/
			}
			ret = temp1->MultiplyInternal(temp1,inv_partial_w[2],inner_products[3]);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,369,"There was a problem multiplying W_inv_i x V_{i+1}^T V_0");
				return NULL;
			}	
#ifdef NORMAL_MATRIX_ASSERTION
			if (this->iAssertionMode)
			{
				ret = norm_temp1->MultiplyInternal(norm_temp1,norm_inv_partial_w[2],norm_inner_products[3]);
				if (ret != ConstRC::Ok){
					CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,369,"There was a problem multiplying W_inv_i x V_{i+1}^T V_0");
					return NULL;
				}	
				if (temp1->Compare(norm_temp1) == NOT_EQUAL)
				{
					printf("\nTemp 1 failed.");
				} else 
				{
					printf("\nTemp 1 ConstRC::Ok.");
				}
			}
#endif
			ret = big_temp_2->MultiplyInternal(big_temp_2,partial_v[2],temp1);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,372,"There was a problem multiplying V_i x W_{inv}_iV_i^TAV_0");
				return NULL;
			}
#ifdef NORMAL_MATRIX_ASSERTION
			if (this->iAssertionMode)
			{
				ret = norm_big_temp_2->MultiplyInternal(norm_big_temp_2,norm_partial_v[2],norm_temp1);
				if (ret != ConstRC::Ok){
					CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,372,"There was a problem multiplying V_i x W_{inv}_iV_i^TAV_0");
					return NULL;
				}
				if (big_temp_2->Compare(norm_big_temp_2) == NOT_EQUAL)
				{
					printf("\nBig temp 2 failed.");
				} else
				{
					printf("\nBig temp 2 ConstRC::Ok.");
				}
			}
#endif
			ret = x_minus_y->Add(big_temp_3, big_temp_2);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,373,"There was a problem adding (X-Y) + V_iW_{inv}_i x V_i^TAV_0");
				return NULL;
			}
			ret = x_minus_y->Copy(big_temp_3);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,374,"There was a problem copying big_temp_3 to x_minus_y");
				return NULL;
			}
		} // end of x_minus_y recalculation
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - phase_start;
                if (diff > 0){
                        total_xy += diff;
                }
                // Start of calculations. The following matrices are stored in variables (see Montgomery)  //
                // partial_v[0] ... V_{i-2}             partial_v[1] ... V_{i-1}                partial_v[2] ... V_i //
                // partial_v[3] ... AV_i
                                 //
                // s_i_minus_1  ...     S_{i-1}S_{i-1}^T        s_i ... S_{i}S_{i}^T
 //
                // inv_partial_w[0] ... W_{i-2}^{inv},  inv_partial_w[1] ... W_{i-1}^{inv}                       //
                // inv_partial_w[2] ... W_{i}^{inv}
// Our aim is to calculate V_{i+1}//

                phase_start = clock();
#endif
		// Calculation of F_{i+1} //
	if (last_rank == word) {
		ret = big_temp_3->Zeroize();
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,3482,"error zeroizing V_i x F_i");
			return NULL;
		}
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			norm_big_temp_3->Zeroize();
		}
#endif
		ret = temp4->Zeroize();
		if (ret != ConstRC::Ok){
                        CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,3483,"error zeroizing F_i");
                        return NULL;
                }
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			norm_temp4->Zeroize();
		}
#endif
                matrix_f->Copy(temp4);	
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			norm_matrix_f->Copy(norm_temp4);
		}
#endif

	} else {
		ret = temp1->PerformColumnMask(temp1, old_vitaquadvi, alternate_s_i_minus_1_representation);
		//ret = temp1->MultiplyInternal(temp1,old_vitaquadvi,s_i_minus_1);
		if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle-1,375,"There was a problem multiplying V_i^TAAV_i x S_iS_i^T");
				return NULL;
		}
		ret = temp1->Add(temp2,old_vitavi);
		if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle-1,376,"There was a problem adding V_i^TAAV_iS_iS_i^T + V_i^T x AV_i");
				return NULL;
		}
		ret = temp1->Zeroize();
		if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle-1,377,"There was a problem zeroizing temp1");
				return NULL;
		}
		for (i=0; i<= temp1->GetMaximalRowIndex(); i++){
			temp1->PutOne(i,i);
		}
		ret = temp4->MultiplyInternal(temp4,old_vitavi,inv_partial_w[1]);
		if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle-1,378,"There was a problem multiplying V_i^TAV_i x W_{inv}_i");
				return NULL;
		}
		ret = temp1->Add(temp3,temp4);
		if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle-1,379,"There was a problem adding I_N+V_i^TAV_iW_{inv}_i");
				return NULL;
		}
		ret = temp4->MultiplyInternal(temp4,temp3,temp2);
		if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle-1,380,"There was a problem multiplying (I_N+V_i^TAV_iW_{inv}_i)(V_i^TAAV_iS_iS_i^T+V_i^TAV_i)");
				return NULL;
		}
		ret = temp1->PerformColumnMask(temp3, temp4, alternate_s_i_representation);
		//ret = temp3->MultiplyInternal(temp3,temp4,s_i);
		if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle-1,380,"There was a problem multiplying (I_N+V_i^TAV_iW_{inv}_i)(V_i^TAAV_iS_iS_i^T+V_i^TAV_i)S_i");
				return NULL;
		}
		ret = temp4->MultiplyInternal(temp4,inv_partial_w[0],temp3);
		if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle-1,381,"There was a problem multiplying W_{inv}_i(I_N+V_i^TAV_iW_{inv}_i)(V_i^TAAV_iS_iS_i^T+V_i^TAV_i)S_i");
				return NULL;
		}
		// end of F_{i+1} calculation. It is contained in temp4
		matrix_f->Copy(temp4);
		// if S_i_minus_1 was identity matrix, F_{i+1} should be zero - an invariant
		if (this->iAssertionMode){
			if (last_rank == word){
				for (i=0; i<word; i++){
					if (alternate_s_i_minus_1_representation & transpose->K_SHIFTS[i]){
						CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,cycle,"assertion failed: last rank indicates that s_i_minus_1 was identity, but it is not");
						return NULL;
					}
				}
				if (!temp4->IsZero()){
						CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",i,cycle,"assertion failed: last rank indicates that F_{i+1} should be zero, but it is not");
						return NULL;
				}
			}
		} // end of assertion
#ifdef PRINT_MATRICES		
		printf("Matice F vysla jako: \n");
		temp4->PrintToScreen();
#endif
#ifdef LANCZOS_TIME_MESSAGE
                sub_start = clock();
#endif
                ret = big_temp_3->MultiplyInternal(big_temp_3,partial_v[0],temp4);
                if (ret != ConstRC::Ok){
                        CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,382,"error multiplying matrices V_i x F_i");
                        return NULL;
                }
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - sub_start;
                if (diff > 0){
                        total_f_end += diff;
                }
#endif
	} // end of else (case of F neq zero matrix)
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - phase_start;
                if (diff > 0){
                        total_f += diff;
                }
                phase_start = clock();
#endif
		// Calculation of E_{i+1}
		ret = big_temp_4->PerformColumnMask(big_temp_4,avi, alternate_s_i_representation);
		//ret = big_temp_4->MultiplyInternal(big_temp_4,avi,s_i);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",big_temp_4->GetSpecies(),383,"error multiplying matrices A V_i x S_iS_i^T");
			return NULL;
		}
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			ret  = norm_big_temp_4->PerformColumnMask(norm_big_temp_4,norm_avi, alternate_s_i_representation);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",ret,383,"error multiplying matrices A V_i x S_iS_i^T");
				return NULL;
			}
			if (big_temp_4->Compare(norm_big_temp_4) == NOT_EQUAL)
			{
				printf("\nColumn mask Failed");
			} else 
			{
				printf("\nColumn mask ConstRC::Ok");
			}
		}
#endif
		// Now, V_{i}^T A V_{i} is stored in temp3
		// and  AV_iS_iS_i^T in big_temp4
		ret = temp4->MultiplyInternal(temp4,inv_partial_w[1],new_vitavi);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,384,"error multiplying matrices W_inv_i x V_i^T A V_i");
			return NULL;
		}
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			ret = norm_temp4->MultiplyInternal(norm_temp4,norm_inv_partial_w[1],norm_new_vitavi);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,384,"error multiplying matrices W_inv_i x V_i^T A V_i");
				return NULL;
			}
			if (norm_temp4->Compare(temp4) == NOT_EQUAL)
			{
				printf("\nMultiply internal 4 Failed");
			} else 
			{
				printf("\nMultiply internal 4 ConstRC::Ok");
			}			
		}
#endif
		ret = temp1->PerformColumnMask(temp1,temp4,alternate_s_i_representation);
		//ret = temp1->MultiplyInternal(temp1,temp4,s_i);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,384,"error multiplying matrices W_inv_iV_i^T A V_i and S_i");
			return NULL;
		}
		// now, temp1 contains E_{i+1}
		matrix_e->Copy(temp1);
#ifdef NORMAL_MATRIX_ASSERTION
		if (this->iAssertionMode)
		{
			ret = norm_temp1->PerformColumnMask(norm_temp1,norm_temp4,alternate_s_i_representation);
			//ret = temp1->MultiplyInternal(temp1,temp4,s_i);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,384,"error multiplying matrices W_inv_iV_i^T A V_i and S_i");
				return NULL;
			}
			if (temp1->Compare(norm_temp1) == NOT_EQUAL)
			{
				printf("\nColumn mask on 4 to 1 failed.");
			} else 
			{
				printf("\nColumn mask on 4 to 1 ConstRC::Ok.");
			}
			// now, temp1 contains E_{i+1}
			norm_matrix_e->Copy(norm_temp1);
		}
#endif

#ifdef PRINT_MATRICES		
		printf("Matice E vysla jako: \n");
		temp1->PrintToScreen();
#endif
#ifdef LANCZOS_TIME_MESSAGE
                sub_start = clock();
#endif
                ret = big_temp_2->MultiplyInternal(big_temp_2,partial_v[1],temp1);
                if (ret != ConstRC::Ok){
                        CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,385,"error multiplying matrices W_iE_i");
                        return NULL;
                }
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - sub_start;
                if (diff > 0) {
                        total_e_end += diff;
                }
                /*ret = avi->Transpose(transpose2);
                if (ret != ConstRC::Ok){
                        CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,386,"error transposing matrix avi");
                        return NULL;
                }*/
                sub_start = clock();
#endif
                ret = new_vitaquadvi->MultiplyInternalTransposed(new_vitaquadvi,avi,avi);
                if (ret != ConstRC::Ok){
                        CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,387,"error multiplying matrix Vi^TA x AV_i");
                        return NULL;
                }
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - sub_start;
                if (diff > 0) {
                        total_vitaquadvi += diff;
                }
#endif
#ifdef PRINT_MATRICES		
		printf("Matice new_vitaquadvi vysla jako: \n");
		new_vitaquadvi->PrintToScreen();
#endif
		ret = big_temp_2->Add(big_temp_1,big_temp_3);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,388,"error multiplying matrix V_iE_i+V_iF_i");
			return NULL;
		}
#ifdef PRINT_MATRICES		
		printf("Matice big_temp_1 vysla jako: \n");
		big_temp_1->PrintToScreen();
#endif
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - phase_start;
                if (diff > 0){
                        total_e += diff;
                }
                // now, V_{i-1}E_{i+1} + V_{i-2}F_{i+1} is stored in big_temp_1
                phase_start = clock();
                sub_start = clock();
#endif
                ret = temp1->PerformColumnMask(temp1,new_vitaquadvi, alternate_s_i_representation);
                //ret = temp1->MultiplyInternal(temp1,new_vitaquadvi,s_i);
                if (ret != ConstRC::Ok){
                        CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,389,"error multiplying matrix V_i^TAAV_i x S_iS_i^T");
                        return NULL;
                }
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - sub_start;
                if (diff > 0){
                        sub_1 += diff;
                }
                sub_start = clock();
#endif
#ifdef PRINT_MATRICES		
		printf("Matice temp1 vysla jako: \n");
		temp1->PrintToScreen();
#endif
		ret = temp1->Add(temp2,new_vitavi);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,390,"error adding matrices V_i^TAAV_i + V_i^T A V_i");
			return NULL;
		}
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - sub_start;
                if (diff > 0){
                        sub_2 += diff;
                }
#endif
#ifdef PRINT_MATRICES		
		printf("Matice temp2 vysla jako: \n");
		temp2->PrintToScreen();
#endif		
#ifdef LANCZOS_TIME_MESSAGE
		sub_start = clock();
#endif
		ret = temp3->MultiplyInternal(temp3,inv_partial_w[2],temp2);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,391,"error multiplying matrices W_i x (V_i^TAAV_i+V_i^T A V_i)");
			return NULL;
		}
#ifdef PRINT_MATRICES		
		printf("Matice temp3 vysla jako: \n");
		temp3->PrintToScreen();
#endif
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - sub_start;
                if (diff > 0){
                        sub_3 += diff;
                }
                sub_start = clock();
#endif
		ret = temp1->Zeroize();		
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,392,"error zeroizing matrix temp1");
			return NULL;
		}
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - sub_start;
                if (diff > 0){
                        sub_5 += diff;
                }
                sub_start = clock();
#endif
                for (i=0; i<=temp1->GetMaximalRowIndex(); i++){
                        temp1->PutOne(i,i);
                }
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - sub_start;
                if (diff > 0){
                        sub_4 += diff;
                }
                sub_start = clock();
#endif
		ret = temp1->Add(temp2,temp3);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,393,"error adding matrices I_N + W_{inv}_i(V_i^TAAV_iS_i+V_i^T A V_i)");
			return NULL;
		}
#ifdef PRINT_MATRICES		
		printf("Matice temp2 vysla jako: \n");
		temp2->PrintToScreen();
#endif
		// now, D_{i+1} is contained in temp2
		matrix_d->Copy(temp2);
#ifdef PRINT_MATRICES		
		printf("Matice D vysla jako: \n");
		matrix_d->PrintToScreen();
#endif
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - sub_start;
                if (diff > 0){
                        sub_6 += diff;
                }
                sub_start = clock();
#endif
		ret = big_temp_2->MultiplyInternal(big_temp_2,partial_v[2],temp2);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,394,"error adding matrices V_i x D_i");
			return NULL;
		}
#ifdef PRINT_MATRICES		
		printf("Soucin V_iD vysel jako: \n");
		big_temp_2->PrintToScreen();
#endif
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - phase_start;
                if (diff > 0){
                        total_d_end += diff;
                }
#endif
		ret = big_temp_2->Add(big_temp_3,big_temp_1);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,395,"error adding matrices V_iD_i + (V_iE_i+V_iF_i)");
			return NULL;
		}
		ret = big_temp_3->Add(partial_v[3],big_temp_4);
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,396,"error adding matrices AV_%dS_%d+(V_%dD_%d + V_%dE_%d+V_%dF_%d");
			return NULL;
		}
#ifdef PRINT_MATRICES		
		printf("Soucet V_1 vysel jako: \n");
		partial_v[3]->PrintToScreen();
#endif
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - phase_start;
                if (diff > 0){
                        total_d += diff;
                }
                phase_start = clock();
#endif
		// now, V_{i+1} is stored in partial_v[3]
		// we test invariants: W_j^T A V_{i+1} should be zero for j<=i. We'll test only
		// W_{i-1}AV_{i+1} a W_iAV_{i+1}.
		if (this->iAssertionMode){
			/*ret = partial_v[2]->Transpose(transpose);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,397,"assertion; error transposing matrix partial_v[2]");
				return NULL;
			}*/
                        // Use of late binding - virtual method
                        ret = aMatrixB->MultiplyInternalTransposed(repository,aMatrixB,partial_v[3]);
                        if (ret != ConstRC::Ok){
                                CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,398,"assertion; error multiplying matrices atransposed and partial_v[3]");
                                return NULL;
                        }
                        // Use of late binding - virtual method
                        ret = aMatrixB->MultiplyInternal(big_temp_1,aMatrixB,repository);
                        if (ret != ConstRC::Ok){
                                CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,399,"assertion; error multiplying matrices aMatrixB and repository");
                                return NULL;
                        }
#ifdef PRINT_MATRICES			
			printf("Asercni mod: big-temp-1:\n");
			big_temp_1->PrintToScreen();
#endif			
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,399,"assertion; error multiplying matrices aMatrixB and repository");
				return NULL;
			}
			ret = temp4->MultiplyInternalTransposed(temp4,partial_v[2],big_temp_1);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,400,"assertion; error multiplying matrices transpose and big_temp_1");
				return NULL;
			}
#ifdef PRINT_MATRICES			
			printf("Asercni mod: temp-4:\n");
			temp4->PrintToScreen();
#endif			
			//ret = temp3->MultiplyInternal(temp3,s_i,temp4);
			ret = temp3->PerformRowMask(temp3,temp4,alternate_s_i_representation);
			//ret = temp3->PerformColumnMask(temp3,temp4,alternate_s_i_representation);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,401,"assertion; error multiplying matrices s_i and temp4");
				return NULL;
			}
			if (!temp3->IsZero()){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,402,"assertion failed; temp3 is not zero, and it should be");
				printf("S_{i} = 0x%x.\n",alternate_s_i_representation);
				temp3->PrintToScreen();
				//return NULL;
				//wait();
			}
			/*ret = partial_v[1]->Transpose(transpose);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,403,"assertion; error transposing matrix partial_v[1]");
				return NULL;
			}*/
			ret = temp4->MultiplyInternalTransposed(temp4,partial_v[1],big_temp_1);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,404,"assertion; error multiplying matrices transpose and big_temp_1");
				return NULL;
			}
//			ret = temp3->MultiplyInternal(temp3,s_i_minus_1,temp4);
			ret = temp3->PerformRowMask(temp3,temp4,alternate_s_i_minus_1_representation);
			//ret = temp3->PerformColumnMask(temp3,temp4,alternate_s_i_minus_1_representation);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,405,"assertion; error multiplying matrices s_i_minus_1 and temp4");
				return NULL;
			}
			if (!temp3->IsZero()){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,406,"assertion failed; temp3 is not zero, and it should be.");
				printf("S_{i-1} = 0x%x.\n",alternate_s_i_minus_1_representation);
				temp3->PrintToScreen();
				//return NULL;
				//wait();
			}
		} // end of assertion
		
		// cyclic shift: V_{i+1}-> V_i, V_{i}-> V_{i-1}, V_{i-1}->V_{i+1}
		aux=partial_v[0];
		partial_v[0]=partial_v[1];
		partial_v[1]=partial_v[2];
		partial_v[2]=partial_v[3];
		partial_v[3]=aux; //  in  V_{i+1} an old value is stored. It will be overwritten.
		// cyclic shift: W_{i+1}-> W_i, W_{i}-> W_{i-1}, W_{i-1}->V_{i+1}
		aux=inv_partial_w[0];
		inv_partial_w[0]=inv_partial_w[1];
		inv_partial_w[1]=inv_partial_w[2];
		inv_partial_w[2]=aux;
		// cyclic shift: inner products V_iV_0
		aux=inner_products[0];
		inner_products[0]=inner_products[1];
		inner_products[1]=inner_products[2];
		inner_products[2]=inner_products[3];
		inner_products[3]=aux; //  in  V_{i+1} an old value is stored. It will be overwritten.
//		aux=s_i_minus_1;
//		s_i_minus_1=s_i;
		alternate_s_i_minus_1_representation = alternate_s_i_representation;
		last_rank = rank; // rank of s_i_minus_1
//		s_i=aux;
		cycle++;
		old_vitavi->Copy(new_vitavi);
		old_vitaquadvi->Copy(new_vitaquadvi);
#ifdef LANCZOS_TIME_MESSAGE
                diff = clock() - phase_start;
                if (diff > 0){
                        total_cyclic += diff;
                }
                diff = clock() - ps_start;
                if (diff > 0){
                        total_perform += diff;
                }
#elif defined MATRIX_OPERATIONS_TIME_MESSAGE
                diff = clock() - ps_start;
                if (diff > 0){
                        total_perform += diff; // the total perform part must be collected even if LANCZOS_TIME_MESSAGE is undefined
                }
#endif
	} while (boolean); // end of the iteration loop
  	if (this->iInfoMode){
		printf(" End of iteration process.      \n");
		printf(" Lanczos matrix statistics:     \n");
		for (i=0; i<=word; i++){
			if (statistics[i]) printf("   Rank %3ld: altogether %4ld matrices.\n",i,statistics[i]);
		}
		printf(" Total rank was %d, number of columns was %ld",krylov_subspace,aMatrixB->GetMaximalColumnIndex()+1);
		fflush(stdout);
   	}
        if (this->iRankCalculationMode || this->iDensityMode){
		int density;
		if (this->iMatrixType == SSparseMatrix){
			BitMatrix* symmetric = new 	BitMatrix(SMatrixB->GetMaximalRowIndex()+1,SMatrixB->GetMaximalRowIndex()+1);
			printf("\nMatrix parameter calculations: constructing symmetric matrix A ...");
			fflush(stdout);
			ret = SMatrixB->MultiplyInternalBigWithTransposition(symmetric, SMatrixB, SMatrixB,TRUE);
			if (ret != ConstRC::Ok){
				CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos 	method",cycle,498,"error constructing symmetric matrix");
				return NULL;
			}
			printf(" done.\n");
			fflush(stdout);
			// density must be calculated before rank!
			if (this->iDensityMode){
				printf(" Calculating density of A ... ");
				fflush(stdout);
				density = symmetric->NonzeroElements();
				printf(" done, %d (%6.2f per cent in %ldx%ld matrix).\n",density,(100*density)/((float) (symmetric->GetMaximalRowIndex()+1)*(symmetric->GetMaximalColumnIndex()+1)),symmetric->GetMaximalRowIndex()+1,symmetric->GetMaximalColumnIndex()+1);
				fflush(stdout);
			}
			if (this->iRankCalculationMode){
				printf(" Calculating rank of A ... ");
				fflush(stdout);
				rank = symmetric->CalculateRank(true);
				printf(" done, %d (in %ldx%ld 	matrix).\n",rank,symmetric->GetMaximalRowIndex()+1,symmetric->GetMaximalColumnIndex()+1);
				fflush(stdout);
			}			
			delete symmetric;
			symmetric = NULL;
			printf(" Converting B to bit matrix ... ");
			fflush(stdout);
			symmetric = SMatrixB->ToBitMatrix();
			printf(" done.\n");
			fflush(stdout);
			if (this->iDensityMode){
				printf(" Calculating density of B ... ");
				fflush(stdout);
				density = symmetric->NonzeroElements();
				printf(" done, %d (%6.2f per cent in %ldx%ld matrix).\n",density,(100*density)/((float) (symmetric->GetMaximalRowIndex()+1)*(symmetric->GetMaximalColumnIndex()+1)),symmetric->GetMaximalRowIndex()+1,symmetric->GetMaximalColumnIndex()+1);
				fflush(stdout);
			}
			if (this->iRankCalculationMode){
				printf(" Calculating rank of B ... ");
				fflush(stdout);
				rank = symmetric->CalculateRank(true);
				printf(" done, %d (in %ldx%ld 	matrix).\n",rank,symmetric->GetMaximalRowIndex()+1,symmetric->GetMaximalColumnIndex()+1);
			}
			
			delete symmetric;
			symmetric = NULL;
		}
	} // end of rank calculation mode
	if (this->iCheckFinalResult) {//(this->iAssertionMode){
		switch (this->iMatrixType){
			case SSparseMatrix: ret = SMatrixB->MultiplyInternalTransposed(repository,SMatrixB,x_minus_y);
						break;
			case SBitMatrix:       ret = BMatrixB->MultiplyInternalTransposed(repository,BMatrixB,x_minus_y);
						break;
			default:
						CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,1499,"Erroneous sieving matrix type");
						break;
		}
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,498,"assertion; error multiplying matrices atransposed and x_minus_y");
			return NULL;
		}
		switch (this->iMatrixType){
			case SSparseMatrix:  ret = SMatrixB->MultiplyInternal(big_temp_1,SMatrixB,repository);
						break;
			case SBitMatrix:        ret = BMatrixB->MultiplyInternal(big_temp_1,BMatrixB,repository);
						break;
			default:
						CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,1699,"Erroneous sieving matrix type");
						break;
		}
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,499,"assertion; error multiplying matrices aMatrixB and repository");
			return NULL;
		}
		/*rank = big_temp_1->CalculateRank();
		printf("\nThe rank of A(X-Y) was calculated as %d.\n",rank);*/
		switch (this->iMatrixType){
			//case EBCMatrix:
			case SSparseMatrix: ret = SMatrixB->MultiplyInternalTransposed(repository,SMatrixB,partial_v[1]);
						break;
			case SBitMatrix:       ret = BMatrixB->MultiplyInternalTransposed(repository,BMatrixB,partial_v[1]);
						break;
			default:
						CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,1598,"Erroneous sieving matrix type");
						break;
		}
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,598,"assertion; error multiplying matrices atransposed and x_minus_y");
			return NULL;
		}
		switch (this->iMatrixType){
			//case EBCMatrix:
			case SSparseMatrix:  ret = SMatrixB->MultiplyInternal(big_temp_1,SMatrixB,repository);
						break;
			case SBitMatrix:        ret = BMatrixB->MultiplyInternal(big_temp_1,BMatrixB,repository);
						break;
			default:
						CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,1599,"Erroneous sieving matrix type");
						break;
		}
		if (ret != ConstRC::Ok){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,599,"assertion; error multiplying matrices aMatrixB and repository");
			return NULL;
		}
		/*rank = partial_v[1]->CalculateRank();
		printf("\nThe rank of AV_m was calculated as %d.\n",rank);*/
		if (!big_temp_1->IsZero()){
			CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,406,"assertion failed; big_temp_1 is not zero, and it should be");
			if (this->iAssertionMode)
			{
				big_temp_1->PrintToScreen();
			}
			//return NULL;
		} else 
		{
			printf("\nLanczos: check final result: all required matrix products are zero.\n");
		}
	} // end of assertion

	// Some deletions to ease memory needed?
	BitMatrix *az, *bz, *z;
	az = new BitMatrix(2*word,aMatrixB->GetMaximalRowIndex()+1);
	bz = new BitMatrix(2*word,aMatrixB->GetMaximalColumnIndex()+1);
	z  = new BitMatrix(2*word,aMatrixB->GetMaximalRowIndex()+1);
	ret = bz->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,407,"allocation of bz failed");
		return NULL;
	}
	ret = z->Allocate();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,408,"allocation of z failed");
		return NULL;
	}
	// Filling in matrix Z
/*
	ret = x_minus_y->Transpose(transpose);
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,409,"transposition of x_minus_y failed");
		return NULL;
	}
*/
	for (i=0;i<=x_minus_y->GetMaximalColumnIndex();i++)
	{
		for (j=0;j<=x_minus_y->GetMaximalRowIndex();j++)
		{
			if(x_minus_y->IsOne(j,i)) { z->PutOne(i,j); }
		}
	}
/*
	ret = partial_v[1]->Transpose(transpose);
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,410,"transposition of partial_v[1] failed");
		return NULL;
	}
*/
	for (i=0;i<=partial_v[1]->GetMaximalColumnIndex();i++)
	{
		for (j=0;j<=partial_v[1]->GetMaximalRowIndex();j++)
		{
			if(partial_v[1]->IsOne(j,i)) { z->PutOne(i+word,j); }
		}
	}
	//BitMatrix* intermediate;
	//intermediate = aMatrixB->MultiplyInternal(aMatrixB,z);
	//ret = bz->Transpose(intermediate)
	BitMatrix* z_transposed = new BitMatrix(z->GetMaximalColumnIndex()+1,z->GetMaximalRowIndex()+1);
	z->Transpose(z_transposed);
	BitMatrix* bz_transposed = NULL;
	BitMatrix* az_transposed = NULL;
        bz_transposed = new BitMatrix(aMatrixB->GetMaximalColumnIndex()+1,z_transposed->GetMaximalColumnIndex()+1);
        az_transposed = new BitMatrix(aMatrixB->GetMaximalRowIndex()+1,z_transposed->GetMaximalColumnIndex()+1);
        // Use of late binding (virtual method)
        aMatrixB->MultiplyInternalTransposed(bz_transposed,aMatrixB,z_transposed);
        aMatrixB->MultiplyInternal(az_transposed,aMatrixB,bz_transposed);
        if (!bz_transposed) {
                CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,311,"multiplication of Z^T and B^T failed");
                return NULL;
        }
	ret = bz_transposed->Transpose(bz);
	ret = az_transposed->Transpose(az);
	long rank_az, rank_bz;
	rank_az = az->CalculateRank();
	rank_bz = bz->CalculateRank();
	printf("\n Rank AZ = %ld, rank BZ = %ld, maximum possible = %ld.\n",rank_az, rank_bz, 2*word);
	delete bz_transposed;
	delete az_transposed;
	delete z_transposed;
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,411,"transposition of Z^TB");
		return NULL;
	}
	ret = bz->ToSemidiagonalShapeForLanczos(z);
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",ret,412,"semidiagonal shape for Lanczos failed");
		return NULL;
	}

	for (i=0; i<=bz->GetMaximalRowIndex(); i++){
		for (j=0; j<=bz->GetMaximalColumnIndex(); j++){
			if (bz->IsOne(i,j)){
				z->ZeroizeRow(i);
			}
		}
	}
	ret = z->ToSemidiagonalShape();
	if (ret != ConstRC::Ok){
		CLog::LogError(WHEREARG,"BitMatrix class","Block Lanczos method",cycle,413,"semidiagonal shape for Z failed");
		return NULL;
	}
	delete bz;
	bz = NULL;
	rank=0;
	for (i=0;i<=z->GetMaximalRowIndex();i++)
	{
		for (j=0; j<=z->GetMaximalColumnIndex();j++)
		{
			if (z->IsOne(i,j))
			{
				rank++;
				break;
			}

		}
	}
	BitMatrix* result = NULL;
	if (rank){
		 result= new BitMatrix(rank,z->GetMaximalColumnIndex()+1);
		 rank = 0;
 	  	 result->Allocate();
		 for (i=0; i<= z->GetMaximalRowIndex(); i++){
		 	for (j=0; j<=z->GetMaximalColumnIndex(); j++){
				if (z->IsOne(i,j)){
					for (k=0; k<=z->GetMaximalColumnIndex(); k++)
					{
						if (z->IsOne(i,k)) { result->PutOne(rank,k); }
					}
					rank++;
					break;
				}
			}
		 }
	}
	if (iTimeMessageMode) {
#ifdef MATRIX_OPERATIONS_TIME_MESSAGE
	printf("      Overall time spent in  MultiplyInternalBig  is %12.4f sec (%6.4f %%).\n",AbstractMatrix::total_big_multiply/(double) CLOCKS_PER_SEC, 100*(AbstractMatrix::total_big_multiply/((double) total_perform)));
	printf("      Overall time spent in  MultiplyInternalWord is %12.4f sec (%6.4f %%).\n",AbstractMatrix::total_word_multiply/(double) CLOCKS_PER_SEC, 100*(AbstractMatrix::total_word_multiply/((double) total_perform)));
	printf("      Overall time spent in  MultiplyInternalTran is %12.4f sec (%6.4f %%).\n",AbstractMatrix::total_word_transpose_multiply/(double) CLOCKS_PER_SEC, 100*(AbstractMatrix::total_word_transpose_multiply/((double) total_perform)));
	printf("      Overall time spent in  Transpose            is %12.4f sec (%6.4f %%).\n",AbstractMatrix::total_transpose/(double) CLOCKS_PER_SEC, 100*(AbstractMatrix::total_transpose/((double) total_perform)));
	printf("      Overall time spent in  Add                  is %12.4f sec (%6.4f %%).\n",AbstractMatrix::total_add/(double) CLOCKS_PER_SEC, 100*(AbstractMatrix::total_add/((double) total_perform)));
	printf("      Overall time spent in  Copy                 is %12.4f sec (%6.4f %%).\n",AbstractMatrix::total_copy/(double) CLOCKS_PER_SEC, 100*(AbstractMatrix::total_copy/((double) total_perform)));
	printf("      Overall time spent in  Zeroize              is %12.4f sec (%6.4f %%).\n",AbstractMatrix::total_zeroize/(double) CLOCKS_PER_SEC, 100*(AbstractMatrix::total_zeroize/((double) total_perform)));
	printf("      Overall time spent in  IsZero               is %12.4f sec (%6.4f %%).\n",AbstractMatrix::total_iszero/(double) CLOCKS_PER_SEC, 100*(AbstractMatrix::total_iszero/((double) total_perform)));
#endif
#ifdef LANCZOS_TIME_MESSAGE
	printf("      ----------------------------------------------\n");
	printf("      Overall time spent in phase  Big            is %12.4f sec (%6.4f %%).\n",total_big/(double) CLOCKS_PER_SEC, 100*(total_big/((double) total_perform)));
	printf("      Overall time spent in phase  Init           is %12.4f sec (%6.4f %%).\n",total_init/(double) CLOCKS_PER_SEC,       100*(total_init/((double) total_perform)));
        printf("                    From that in   vitavi            %12.4f sec (%6.4f %%).\n",total_vitavi/(double) CLOCKS_PER_SEC,     100*(total_vitavi/((double) total_perform)));
        printf("      Overall time spent in phase  Rank           is %12.4f sec (%6.4f %%).\n",total_rank/(double) CLOCKS_PER_SEC, 100*(total_rank/((double) total_perform)));
        printf("      Overall time spent in phase  S              is %12.4f sec (%6.4f %%).\n",total_s/(double) CLOCKS_PER_SEC, 100*(total_s/((double) total_perform)));
        printf("      Overall time spent in phase  X-Y            is %12.4f sec (%6.4f %%).\n",total_xy/(double) CLOCKS_PER_SEC, 100*(total_xy/((double) total_perform)));
        printf("      Overall time spent in phase  F              is %12.4f sec (%6.4f %%).\n",total_f/(double) CLOCKS_PER_SEC, 100*(total_f/((double) total_perform)));
        printf("                    From that in   end               %12.4f sec (%6.4f %%).\n",total_f_end/(double) CLOCKS_PER_SEC,
        100*(total_f_end/((double) total_perform)));
        printf("      Overall time spent in phase  E              is %12.4f sec (%6.4f %%).\n",total_e/(double) CLOCKS_PER_SEC, 100*(total_e/((double) total_perform)));
        printf("      Overall time spent in phase  E              is %12.4f sec (%6.4f %%).\n",total_e/(double) CLOCKS_PER_SEC, 100*(total_e/((double) total_perform)));
        printf("                    From that in   end               %12.4f sec (%6.4f %%).\n",total_e_end/(double) CLOCKS_PER_SEC,      100*(total_e_end/((double) total_perform)));
        printf("                    From that in   vitaqvi           %12.4f sec (%6.4f %%).\n",total_vitaquadvi/(double)
CLOCKS_PER_SEC,
        100*(total_vitaquadvi/((double) total_perform)));
        printf("      Overall time spent in phase  D              is %12.4f sec (%6.4f %%).\n",total_d/(double) CLOCKS_PER_SEC, 100*(total_d/((double) total_perform)));
        printf("                    From that in   1                 %12.4f sec (%6.4f %%).\n",sub_1/(double) CLOCKS_PER_SEC,100*(sub_1/((double) total_perform)));
        printf("                    From that in   2                 %12.4f sec (%6.4f %%).\n",sub_2/(double) CLOCKS_PER_SEC,
        100*(sub_2/((double) total_perform)));
        printf("                    From that in   3                 %12.4f sec (%6.4f %%).\n",sub_3/(double) CLOCKS_PER_SEC,
        100*(sub_3/((double) total_perform)));
        printf("                    From that in   4                 %12.4f sec (%6.4f %%).\n",sub_4/(double) CLOCKS_PER_SEC,
                100*(sub_4/((double) total_perform)));
        printf("                    From that in   5                 %12.4f sec (%6.4f %%).\n",sub_5/(double) CLOCKS_PER_SEC,
        100*(sub_5/((double) total_perform)));
        printf("                    From that in   6                 %12.4f sec (%6.4f %%).\n",sub_6/(double) CLOCKS_PER_SEC,
        100*(sub_6/((double) total_perform)));
        printf("                    From that in   end               %12.4f sec (%6.4f %%).\n",total_d_end/(double) CLOCKS_PER_SEC,
        100*(total_d_end/((double) total_perform)));

        printf("      Overall time spent in phase  Cyclic         is %12.4f sec (%6.4f %%).\n",total_cyclic/(double) CLOCKS_PER_SEC, 100*(total_cyclic/((double) total_perform)));
#endif
	}
	delete az;
	delete z;
	// Deletions
	delete repository;
	delete avi;
	delete new_vitaquadvi;
	delete new_vitavi;
	delete old_vitaquadvi;
	delete old_vitavi;
	delete temp4;
	delete temp3;
	delete temp2;
	delete temp1;
//	delete s_i_minus_1;
//	delete s_i;
	delete big_temp_4;
	delete big_temp_3;
	delete big_temp_2;
	delete big_temp_1;
	delete y;
	delete transpose;
	delete v_zero;
	delete x_minus_y;
	delete matrix_d;
	delete matrix_e;
	delete matrix_f;
	for (i=0; i<=3; i++){
		delete inv_partial_w[i];
		delete partial_v[i];
		delete inner_products[i];
	}
#ifdef NORMAL_MATRIX_ASSERTION
	if (this->iAssertionMode)
	{
		delete norm_repository;
		delete norm_avi;
		delete norm_new_vitaquadvi;
		delete norm_new_vitavi;
		delete norm_old_vitaquadvi;
		delete norm_old_vitavi;
		delete norm_temp4;
		delete norm_temp3;
		delete norm_temp2;
		delete norm_temp1;
		delete norm_x_minus_y;
//	delete s_i_minus_1;
//	delete s_i;
		delete norm_big_temp_4;
		delete norm_big_temp_3;
		delete norm_big_temp_2;
		delete norm_big_temp_1;
		delete norm_y;
		delete norm_transpose;
 		delete norm_v_zero;
		delete norm_matrix_d;
		delete norm_matrix_e;
		delete norm_matrix_f;
		for (i=0; i<=3; i++){
			delete norm_inv_partial_w[i];
			delete norm_partial_v[i];
			delete norm_inner_products[i];
		}
	}

#endif
	free((void*) statistics);
	free((void*) permutation);
	free((void*) permutation2);
	free((void*) permutation3);
	return result; // NULL if rank was 0
}

