#include "log.h"

#include <iostream>
#include "mserializable.h"
#include "const_filenames.h"

using namespace std;

FILE *	CLog::iOutputFile = NULL;
bool	CLog::iWriteCode = false;
bool	CLog::iWriteMessage = true;
int		CLog::iPriority = 0;

const char * CLog::CODE_DELIMITER = ": ";

void CLog::UseFile(const string & aIdentifier, const string & aSerializationDirectory)
{
	if (CLog::iOutputFile != NULL)
	{
		fclose(CLog::iOutputFile);
		CLog::iOutputFile = NULL;
	}

	string filename = MSerializable::CreateFileNamePlain(aSerializationDirectory, ConstFN::LOG_FILE, aIdentifier, TXT_FILE_EXTENSION);
	CLog::iOutputFile = fopen(filename.c_str(), FILE_MODE_APPEND);

	// if result is NULL we just inform but we will use console
	if (CLog::iOutputFile == NULL)
	{
		CLog::LogError(WHEREARG,"CLog","UseFile",0,0,"Cannot open file.");
	}
}

void CLog::LogError(int aLineNumber, const string & aSourceFile, const string & aClass, const string & aMethod, int aErrorCode, int aLabel, const string & aExplanation)
{
	printf("------ !!! ERROR !!! ------\n");
	printf("Class / Process:\t\t%s\n", aClass.c_str());
	printf("Method / Problem:\t\t%s\n", aMethod.c_str());
	printf("Source file:\t%s\n", aSourceFile.c_str());
	printf("Line number:\t%d\n", aLineNumber);
	printf("Explanation:\t%s\n", aExplanation.c_str());
	printf("Error code:\t%d\n", aErrorCode);
	printf("Label:\t\t%d\n", aLabel);


	if (CLog::iOutputFile != NULL)
	{
		fprintf(CLog::iOutputFile, "------ !!! ERROR !!! ------\n");
		fprintf(CLog::iOutputFile, "Class / Process:\t\t%s\n", aClass.c_str());
		fprintf(CLog::iOutputFile, "Method / Problem:\t\t%s\n", aMethod.c_str());
		fprintf(CLog::iOutputFile, "Source file:\t%s\n", aSourceFile.c_str());
		fprintf(CLog::iOutputFile, "Line number:\t%d\n", aLineNumber);
		fprintf(CLog::iOutputFile, "Explanantion:\t%s\n", aExplanation.c_str());
		fprintf(CLog::iOutputFile, "Error code:\t%d\n", aErrorCode);
		fprintf(CLog::iOutputFile, "Label:\t\t%d\n", aLabel);
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str());
			}
			else
			{
				printf("\n");
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str());
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str());
			}
			else
			{
				fprintf(CLog::iOutputFile, "\n");
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, int aV1)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str(), aV1);
			}
			else
			{
				printf("%d\n", aV1);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str(), aV1);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str(), aV1);
			}
			else
			{
				fprintf(CLog::iOutputFile, "%d\n", aV1);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, int aV1, int aV2)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str(), aV1, aV2);
			}
			else
			{
				printf("%d %d\n", aV1, aV2);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str(), aV1, aV2);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2);
			}
			else
			{
				fprintf(CLog::iOutputFile, "%d %d\n", aV1, aV2);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, int aV1, int aV2, int aV3)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str(), aV1, aV2, aV3);
			}
			else
			{
				printf("%d %d %d\n", aV1, aV2, aV3);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str(), aV1, aV2, aV3);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2, aV3);
			}
			else
			{
				fprintf(CLog::iOutputFile, "%d %d %d\n", aV1, aV2, aV3);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, int aV1, int aV2, int aV3, int aV4)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str(), aV1, aV2, aV3, aV4);
			}
			else
			{
				printf("%d %d %d %d\n", aV1, aV2, aV3, aV4);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str(), aV1, aV2, aV3, aV4);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2, aV3, aV4);
			}
			else
			{
				fprintf(CLog::iOutputFile, "%d %d %d %d\n", aV1, aV2, aV3, aV4);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, int aV1, int aV2, int aV3, int aV4, int aV5, void * aV6, int aV7)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str(), aV1, aV2, aV3, aV4, aV5, aV6, aV7);
			}
			else
			{
				printf("%d %d %d %d %d %p %d\n", aV1, aV2, aV3, aV4, aV5, aV6, aV7);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str(), aV1, aV2, aV3, aV4, aV5, aV6, aV7);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2, aV3, aV4, aV5, aV6, aV7);
			}
			else
			{
				fprintf(CLog::iOutputFile, "%d %d %d %d %d %p %d\n", aV1, aV2, aV3, aV4, aV5, aV6, aV7);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, int aV1, int aV2, int aV3, int aV4, int aV5, int aV6, int aV7, double aV8)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str(), aV1, aV2, aV3, aV4, aV5, aV6, aV7, aV8);
			}
			else
			{
				printf("%d %d %d %d %d %d %d %f\n", aV1, aV2, aV3, aV4, aV5, aV6, aV7, aV8);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str(), aV1, aV2, aV3, aV4, aV5, aV6, aV7, aV8);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2, aV3, aV4, aV5, aV6, aV7, aV8);
			}
			else
			{
				fprintf(CLog::iOutputFile, "%d %d %d %d %d %d %d %f\n", aV1, aV2, aV3, aV4, aV5, aV6, aV7, aV8);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, long aV1)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str(), aV1);
			}
			else
			{
				printf("%ld\n", aV1);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str(), aV1);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str(), aV1);
			}
			else
			{
				fprintf(CLog::iOutputFile, "%ld\n", aV1);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, float aV1)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str(), aV1);
			}
			else
			{
				cout << "#" << aV1 << endl;
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str(), aV1);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str(), aV1);
			}
			else
			{
				fprintf(CLog::iOutputFile, "#%f\n", aV1);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, double aV1)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str(), aV1);
			}
			else
			{
				printf("%f\n", aV1);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str(), aV1);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str(), aV1);
			}
			else
			{
				fprintf(CLog::iOutputFile, "%f\n", aV1);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, double aV1, double aV2)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str(), aV1, aV2);
			}
			else
			{
				printf("%f %f\n", aV1, aV2);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str(), aV1, aV2);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2);
			}
			else
			{
				fprintf(CLog::iOutputFile, "%f %f\n", aV1, aV2);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, mpz_t & aV1)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_printf(aMessage.c_str(), aV1);
			}
			else
			{
				gmp_printf("%Zd\n", aV1);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				gmp_printf(aMessage.c_str(), aV1);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_fprintf(CLog::iOutputFile, aMessage.c_str(), aV1);
			}
			else
			{
				gmp_fprintf(CLog::iOutputFile, "%Zd\n", aV1);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, long aV1, mpz_t & aV2)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_printf(aMessage.c_str(), aV1, aV2);
			}
			else
			{
				gmp_printf("%Zd %d\n", aV1, aV2);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				gmp_printf(aMessage.c_str(), aV1, aV2);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2);
			}
			else
			{
				gmp_fprintf(CLog::iOutputFile, "%Zd %d\n", aV1, aV2);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, mpz_t & aV1, int aV2)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_printf(aMessage.c_str(), aV1, aV2);
			}
			else
			{
				gmp_printf("%Zd %d\n", aV1, aV2);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				gmp_printf(aMessage.c_str(), aV1, aV2);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2);
			}
			else
			{
				gmp_fprintf(CLog::iOutputFile, "%Zd %d\n", aV1, aV2);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, mpz_t & aV1, int aV2, int aV3)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_printf(aMessage.c_str(), aV1, aV2, aV3);
			}
			else
			{
				gmp_printf("%Zd %d %d\n", aV1, aV2, aV3);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				gmp_printf(aMessage.c_str(), aV1, aV2, aV3);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2, aV3);
			}
			else
			{
				gmp_fprintf(CLog::iOutputFile, "%Zd %d %d\n", aV1, aV2, aV3);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, mpz_t & aV1, mpz_t & aV2, mpz_t & aV3)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_printf(aMessage.c_str(), aV1, aV2, aV3);
			}
			else
			{
				gmp_printf("%Zd %d %d\n", aV1, aV2, aV3);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				gmp_printf(aMessage.c_str(), aV1, aV2, aV3);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2, aV3);
			}
			else
			{
				gmp_fprintf(CLog::iOutputFile, "%Zd %d %d\n", aV1, aV2, aV3);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, mpz_t & aV1, int aV2, int aV3, int aV4)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_printf(aMessage.c_str(), aV1, aV2, aV3, aV4);
			}
			else
			{
				gmp_printf("%Zd %d %d %d\n", aV1, aV2, aV3, aV4);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				gmp_printf(aMessage.c_str(), aV1, aV2, aV3, aV4);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2, aV3, aV4);
			}
			else
			{
				gmp_fprintf(CLog::iOutputFile, "%Zd %d %d %d\n", aV1, aV2, aV3, aV4);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, mpz_t & aV1, int aV2, int aV3, int aV4, int aV5)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_printf(aMessage.c_str(), aV1, aV2, aV3, aV4, aV5);
			}
			else
			{
				gmp_printf("%Zd %d %d %d %d\n", aV1, aV2, aV3, aV4, aV5);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				gmp_printf(aMessage.c_str(), aV1, aV2, aV3, aV4, aV5);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2, aV3, aV4, aV5);
			}
			else
			{
				gmp_fprintf(CLog::iOutputFile, "%Zd %d %d %d %d\n", aV1, aV2, aV3, aV4, aV5);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, mpz_t & aV1, int aV2, int aV3, int aV4, int aV5, int aV6)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_printf(aMessage.c_str(), aV1, aV2, aV3, aV4, aV5, aV6);
			}
			else
			{
				gmp_printf("%Zd %d %d %d %d %d\n", aV1, aV2, aV3, aV4, aV5, aV6);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				gmp_printf(aMessage.c_str(), aV1, aV2, aV3, aV4, aV5, aV6);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2, aV3, aV4, aV5, aV6);
			}
			else
			{
				gmp_fprintf(CLog::iOutputFile, "%Zd %d %d %d %d %d\n", aV1, aV2, aV3, aV4, aV5, aV6);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, mpz_t & aV1, int aV2, int aV3, int aV4, int aV5, int aV6, int aV7, int aV8)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_printf(aMessage.c_str(), aV1, aV2, aV3, aV4, aV5, aV6, aV7, aV8);
			}
			else
			{
				gmp_printf("%Zd %d %d %d %d %d %d %d\n", aV1, aV2, aV3, aV4, aV5, aV6, aV7, aV8);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				gmp_printf(aMessage.c_str(), aV1, aV2, aV3, aV4, aV5, aV6, aV7, aV8);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2, aV3, aV4, aV5, aV6, aV7, aV8);
			}
			else
			{
				gmp_fprintf(CLog::iOutputFile, "%Zd %d %d %d %d %d %d %d\n", aV1, aV2, aV3, aV4, aV5, aV6, aV7, aV8);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, mpf_t & aV1)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_printf(aMessage.c_str(), aV1);
			}
			else
			{
				gmp_printf("%Ff\n", aV1);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				gmp_printf(aMessage.c_str(), aV1);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_fprintf(CLog::iOutputFile, aMessage.c_str(), aV1);
			}
			else
			{
				gmp_fprintf(CLog::iOutputFile, "%Ff\n", aV1);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, mpf_t & aV1, mpf_t & aV2)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_printf(aMessage.c_str(), aV1, aV2);
			}
			else
			{
				gmp_printf("%Ff %Ff\n", aV1, aV2);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				gmp_printf(aMessage.c_str(), aV1, aV2);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				gmp_fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2);
			}
			else
			{
				gmp_fprintf(CLog::iOutputFile, "%Ff %Ff\n", aV1, aV2);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, int aV1, int aV2, void * aV3, void * aV4)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str(), aV1, aV2, aV3, aV4);
			}
			else
			{
				printf("%d %d %p %p\n", aV1, aV2, aV3, aV4);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str(), aV1, aV2, aV3, aV4);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2, aV3, aV4);
			}
			else
			{
				fprintf(CLog::iOutputFile, "%d %d %p %p\n", aV1, aV2, aV3, aV4);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
void CLog::Log(int aPriority, const string & aCode, const string & aMessage, int aV1, double aV2)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str(), aV1, aV2);
			}
			else
			{
				printf("%d %f\n", aV1, aV2);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str(), aV1, aV2);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2);
			}
			else
			{
				fprintf(CLog::iOutputFile, "%d %f\n", aV1, aV2);
			}
			fflush(CLog::iOutputFile);
		}
	}
}

void CLog::Log(int aPriority, const string & aCode, const string & aMessage, int aV1, double aV2, double aV3)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str(), aV1, aV2, aV3);
			}
			else
			{
				printf("%d %f %f\n", aV1, aV2, aV3);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str(), aV1, aV2, aV3);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2, aV3);
			}
			else
			{
				fprintf(CLog::iOutputFile, "%d %f %f\n", aV1, aV2, aV3);
			}
			fflush(CLog::iOutputFile);
		}
	}
}

void CLog::Log(int aPriority, const string & aCode, const string & aMessage, int aV1, long aV2)
{
	if (aPriority >= CLog::iPriority)
	{
		if (CLog::iOutputFile == NULL)
		{
			if (CLog::iWriteCode && aCode.length() > 0)
			{
				printf(aCode.c_str());
				printf(CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				printf(aMessage.c_str(), aV1, aV2);
			}
			else
			{
				printf("%d %ld\n", aV1, aV2);
			}
		}
		else
		{
			if (aPriority >= CLog::FOURTH_PRIORITY_BOUND)
			{
				// very important message
				printf(aMessage.c_str(), aV1, aV2);
			}

			if (CLog::iWriteCode && aCode.length() > 0)
			{
				fprintf(CLog::iOutputFile, aCode.c_str());
				fprintf(CLog::iOutputFile, CLog::CODE_DELIMITER);
			}
			if (CLog::iWriteMessage)
			{
				fprintf(CLog::iOutputFile, aMessage.c_str(), aV1, aV2);
			}
			else
			{
				fprintf(CLog::iOutputFile, "%d %ld\n", aV1, aV2);
			}
			fflush(CLog::iOutputFile);
		}
	}
}
