#ifndef __LOG_H__
#define __LOG_H__

#include "gmp.h"
#include <string>

/// !!!important!!! only log messages with priority equal or bigger are compiled 
#define LOG_MINIMAL_MESSAGE_PRIORITY 5000

/*!
 * \brief Priority logging
 */
class CLog
{
public:
	/// default message priority
	static const int DEFAULT_MESSAGE_PRIORITY	= 100;


private:
	/// File which is used for output if defined
	static FILE * iOutputFile;
	/// If we should write message code 
	static bool iWriteCode;
	/// If we should write whole message or just values
	static bool iWriteMessage;
	/// Minimum priority of messages which should be written
	static int iPriority;

	/// delimiter between code and message
	static const char * CODE_DELIMITER; 
	/// priority bound for less important messages
	static const int FIRST_PRIORITY_BOUND	= 100;
	/// priority bound for methods messages
	static const int SECOND_PRIORITY_BOUND	= 1000;
	/// priority bound for statistics messages
	static const int THIRD_PRIORITY_BOUND	= 10000;
	/// priority bound for more important messages, upper this bound all messages are always written to the console 
	static const int FOURTH_PRIORITY_BOUND	= 100000;

public:

	static void CloseLog() { if (CLog::iOutputFile != NULL) { fclose(CLog::iOutputFile); CLog::iOutputFile = NULL; } }
	static void UseFile(const std::string & aIdentifier, const std::string & aSerializationDirectory);
	static void UseConsole() { if (CLog::iOutputFile != NULL) { fclose(CLog::iOutputFile); CLog::iOutputFile = NULL; } }

	static void Set_Priority(int aPriority) { CLog::iPriority = aPriority; }
	static void Set_TechnicalMode(bool aValue) { CLog::iWriteCode = aValue; CLog::iWriteMessage = !aValue; }

	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, int aV1);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, int aV1, int aV2);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, int aV1, int aV2, int aV3);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, int aV1, int aV2, int aV3, int aV4);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, int aV1, int aV2, int aV3, int aV4, int aV5, void * aV6, int aV7);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, int aV1, int aV2, int aV3, int aV4, int aV5, int aV6, int aV7, double aV8);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, long aV1);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, float aV1);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, double aV1);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, double aV1, double aV2);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, mpz_t & aV1);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, long aV1, mpz_t & aV2);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, mpz_t & aV1, int aV2);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, mpz_t & aV1, int aV2, int aV3);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, mpz_t & aV1, mpz_t & aV2, mpz_t & aV3);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, mpz_t & aV1, int aV2, int aV3, int aV4);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, mpz_t & aV1, int aV2, int aV3, int aV4, int aV5);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, mpz_t & aV1, int aV2, int aV3, int aV4, int aV5, int aV6);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, mpz_t & aV1, int aV2, int aV3, int aV4, int aV5, int aV6, int aV7, int aV8);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, mpf_t & aV1);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, mpf_t & aV1, mpf_t & aV2);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, int aV1, int aV2, void * aV3, void *aV4);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, int aV1, double aV2);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, int aV1, double aV2, double aV3);
	static void Log(int aPriority, const std::string & aCode, const std::string & aMessage, int aV1, long aV2);

	static void LogError(int aLineNumber, const std::string & aSourceFile, const std::string & aClass, const std::string & aMethod, int aErrorCode, int aLabel, const std::string & aExplanation);


};

#endif // __LOG_H__
