/*
 * mpz_matrix.hpp
 *      Author: jero
 */

#ifndef MPZ_MATRIX_HPP_
#define MPZ_MATRIX_HPP_

#include "gmp.h"
#include "abstract_mpz_matrix.h"
#include "integer_sparse_matrix.h"

/*!
 * \brief Classical dense mpz matrix
 **/
class MpzMatrix : public AbstractMpzMatrix {
public:
	MpzMatrix();
	~MpzMatrix();

	virtual int  Allocate(); ///< This method will be used for allocation of dynamic internal structures of a subclass.

	MpzMatrix * clone();

	int ApplyModulo();
	bool Equals(MpzMatrix * aMatrix);

	virtual int  Zeroize(); ///< This method will be used for initialization of the matrix by 0s.
	//virtual bool Equals(AbstractMatrix* aMatrix) = 0;
	virtual int  PutOne(long aRow, long aColumn); ///< This method will be used to put number 1 to the entry indexed by aRow and aColumn.
	virtual int  PutZero(long aRow, long aColumn); ///< This method will be used to put number 0 to the entry indexed by aRow and aColumn.
	virtual int  IsOne(long aRow, long aColumn); ///< This method will respond whether there is a 1 at the entry.
	virtual int  IsZero(long aRow, long aColumn); ///< This method will respond whether there is a 0 at the entry.
	// Swaps etc.
	virtual int  ZeroizeRow(long aRow);; ///< This method will be used to put 0s into a row given by index aRow.

	virtual int  SwapRows(long aRow1, long aRow2); ///< 			This method will be used to swap rows with indices given by aRow1 and aRow2 parameters.

	virtual int  AddRows(long aTarget, long aSource); ///< 			This method will be used to add row with index aSource to row with index aTarget.

	virtual int  ZeroizeColumn(long aColumn); ///< 			This method will be used to put 0s into a column given by index aColumn.
	virtual int  ZeroizeColumn(long* aColumnList, long aListMaxIndex); ///<  This method will be used to put 0s into columns, whose indices are given by array aColumnList.			The aListMaxIndex parameter gives the final index in aColumnList array, in order to prevent			buffer overflow error.

	virtual int  SwapColumns(long aColumn1, long aColumn2);; ///< This method will be used to swap columns with indices given by aColumn1 and aColumn2 parameters.
	virtual int  AddColumns(long aTarget, long aSource); ///< This method will be used to add column with index aSource to column with index aTarget.

	virtual int  AddToRow(long aRow,AbstractMatrix* aOperand, long aRow2); ///< This method will be used to add rows from two different matrices of the same type.
	virtual int  AddToRow(long aRow,AbstractIntegerMatrix* aOperand, long aRow2); ///< This method will be used to (partially) add rows from two different matrices of the same type.
	virtual int  AddToRow(long aRow,MpzMatrix* aOperand, long aRow2); ///< This method will be used to add rows from two different matrices of the same type.

	int Add(MpzMatrix & aOperand1, MpzMatrix & aOperand2);

	int Multiply(MpzMatrix & aOperand1, MpzMatrix & aOperand2); ///< Compute this=aOperand1*aOperand2
	int Multiply( MpzMatrix & aOperand1, AbstractIntegerMatrix & aOperand2);///< Compute this=aOperand1*aOperand
	int Multiply( AbstractIntegerMatrix & aOperand1, MpzMatrix & aOperand2);///< Compute this=aOperand1*aOperand2
	int Multiply(IntegerSparseMatrix & aOperand1, MpzMatrix & aOperand2);///< Compute this=aOperand1*aOperand2
	int MultiplyTransposed( MpzMatrix & aOperand1, AbstractIntegerMatrix & aOperand2);///< Compute this=aOperand1*aOperand^T
	int MultiplyTransposed( MpzMatrix & aOperand1, IntegerSparseMatrix & aOperand2);///< Compute this=aOperand1*aOperand^T

	virtual int  Multiply(mpz_t aValue);; ///< This method will multiply whole matrix by scalar
	virtual int  Multiply(long aValue);; ///< This method will multiply whole matrix by scalar

	virtual int ColumnCRT(long aColumnIndex, long aRows, MpzMatrix *aValues, long aValColIndex, mpz_t aResult, mpz_t *aMultiplier, bool allow_signed);

	virtual int  Save(char *aName);
	virtual int  Load(char *aName);

	virtual int WriteData(xmlTextWriterPtr aWriter) const;
	virtual int ReadData(xmlTextReaderPtr aReader);

	virtual int PutValueMpz(long aRow, long aColumn, mpz_t aValue);
	virtual int GetValueMpz(long aRow, long aColumn, mpz_t aValue);
	virtual mpz_t * GetValueDirect(long aRow, long aColumn);

	virtual int PutValue(long aRow, long aColumn, integer_matrix_type aValue);
	virtual int GetValue(long aRow, long aColumn, integer_matrix_type & aValue);

private:
	long max_allocated_row_index;
	long max_allocated_column_index;
	mpz_t ** matrix;
};


#endif /* MPZ_MATRIX_HPP_ */
