/*
 * MpzSparseMatrix.cpp
 *
 *      Author: jero
 */

#include <iostream>
#include "mpz_sparse_matrix.h"
#include  "const_return_codes.h"
#include "log.h"

SparseRow::SparseRow()  {
	mpz_zero = 0;
};

int SparseRow::PutValue(long aColumn, mpz_t aValue) {
	if (mpz_cmp_ui(aValue,0)== 0)
		items.erase(aColumn);
	else
		mpz_set(items[aColumn].get_mpz_t(),aValue);
		// items[aColumn] = mpz_class(aValue);
}

int SparseRow::PutValue(long aColumn, long aValue) {
	if (aValue == 0)
		items.erase(aColumn);
	else
		mpz_set_ui(items[aColumn].get_mpz_t(),aValue);
}

mpz_class * SparseRow::GetValue(long aColumn) {
	std::map<long,mpz_class>::iterator iter;
	if ((iter = items.find(aColumn)) != items.end())
		return & iter->second;
	else
		return & mpz_zero;
}

void SparseRow::Print() {
	std::cout <<" (" <<items.size() << " items) : ";
	for (std::map<long, mpz_class>::iterator iter = items.begin(); iter != items.end(); iter++) {
		cout <<"["<<iter->first << "→" << iter->second << "]";
	}
}

long SparseRow::getItemCount() const {
	return items.size();
}


SparseRow::~SparseRow() {
}

int SparseRow::Multiply(mpz_t aValue) {
	for (std::map<long, mpz_class>::iterator iter = items.begin(); iter != items.end(); iter++) {
		mpz_mul(iter->second.get_mpz_t(),iter->second.get_mpz_t(),aValue);
	};
	return ConstRC::Ok;
}

void SparseRow::Test() {
	Print();
	mpz_t testvalue;
	mpz_init(testvalue);
	for (int i = 5; i<11; i++) {
		mpz_set_ui(testvalue,i+1000);
		PutValue(i,testvalue);
	}
	for (int i = 20; i>15; i--) {
		mpz_set_ui(testvalue,i+1000);
		PutValue(i,testvalue);
	}
	Print();
	mpz_clear(testvalue);
}

MpzClassSparseMatrix::MpzClassSparseMatrix() {
	mpz_zero = 0;
	mpz_init(mpz_t_interface);
}

MpzClassSparseMatrix::~MpzClassSparseMatrix() {
	mpz_clear(mpz_t_interface);
}

int MpzClassSparseMatrix::Allocate() {
}

int MpzClassSparseMatrix::Zeroize() {
	rows.clear();
}

void MpzClassSparseMatrix::PrintToScreen() {
	gmp_printf("\n(%ld x %ld)",GetMaximalRowIndex()+1, GetMaximalColumnIndex()+1);
	mpz_class * Value;
	for (long row = 0;row <=maximal_row_index; row++) {
		cout << endl;
		for (long column = 0; column <= maximal_column_index; column++){
			GetValueMpzClass(row,column,Value);
			cout << "|" << *Value << "\t";
		}
	}
	for (std::map<long,SparseRow>::iterator rowIterator = rows.begin(); rowIterator != rows.end(); rowIterator++) {
	    cout << endl << rowIterator->first << " ";
	    rowIterator->second.Print();
	}
}

int MpzClassSparseMatrix::PutOne(long aRow, long aColumn) {
	PutValue(aRow,aColumn,1);
}

int MpzClassSparseMatrix::PutZero(long aRow, long aColumn) {
	PutValue(aRow,aColumn,0);
}

int MpzClassSparseMatrix::IsOne(long aRow, long aColumn) {
	return (IsZero(aRow,aColumn)==0);
}

int MpzClassSparseMatrix::IsZero(long aRow, long aColumn) {
	std::map<long,SparseRow>::iterator row = rows.find(aRow);
	if (row == rows.end()) return 1;
	std::map<long,mpz_class>::iterator column = row->second.items.find(aColumn);
	if (column == row->second.items.end()) return 1;
	if (column->second == 0)
		return 1;
	else
		return 0;
}

int MpzClassSparseMatrix::ZeroizeRow(long aRow) {
	rows.erase(aRow);
	return ConstRC::Ok;
}

int MpzClassSparseMatrix::SwapRows(long aRow1, long aRow2) {
	std::map<long,SparseRow>::iterator row1=rows.find(aRow1);
	std::map<long,SparseRow>::iterator row2=rows.find(aRow2);
	swap(row1,row2);
}

int MpzClassSparseMatrix::AddRows(long aTarget, long aSource) {
}

int MpzClassSparseMatrix::ZeroizeColumn(long aColumn) {
}

int MpzClassSparseMatrix::ZeroizeColumn(long * aColumnList, long aListMaxIndex) {
}

int MpzClassSparseMatrix::SwapColumns(long aColumn1, long aColumn2) {
}

int MpzClassSparseMatrix::AddColumns(long aTarget, long aSource) {
}

int MpzClassSparseMatrix::AddToRow(long aRow, AbstractMatrix* aOperand, long aRow2) {
}

int MpzClassSparseMatrix::AddToRow(long aRow, MpzMatrix* aOperand, long aRow2) {
}

int MpzClassSparseMatrix::Save(char* aName) {
}

int MpzClassSparseMatrix::Load(char* aName) {
}

int MpzClassSparseMatrix::WriteData(xmlTextWriterPtr aWriter) const {
}

int MpzClassSparseMatrix::ReadData(xmlTextReaderPtr aReader) {
}

int MpzClassSparseMatrix::PutValueMpz(long aRow, long aColumn, mpz_t aValue) {
	gmp_printf("(%d,%d)← *%Zd |M|",aRow,aColumn,aValue);
	rows[aRow].PutValue(aColumn,aValue);
	if(mpz_cmp_ui(aValue,0)==0) {
		if (rows[aRow].getItemCount() == 0)
			rows.erase(aRow);
	}
}

int MpzClassSparseMatrix::GetValueMpz(long aRow, long aColumn, mpz_t aValue) {
    std::map<long,SparseRow>::iterator row = rows.find(aRow);
	if (row == rows.end())
		{
		aValue = mpz_zero.get_mpz_t();
		return ConstRC::Ok;
		}
	cout << "GetValueMpz:" << row->second.GetValue(aColumn);
	aValue = row->second.GetValue(aColumn)->get_mpz_t();
	return ConstRC::Ok;
}

int MpzClassSparseMatrix::PutValue(long aRow, long aColumn, integer_matrix_type aValue) {
	cout << "(" << aRow << "," << aColumn << ")←" << aValue << "|";
	rows[aRow].PutValue(aColumn,aValue);
	if(aValue==0) {
		if (rows[aRow].getItemCount() == 0)
			rows.erase(aRow);
	}
}

int MpzClassSparseMatrix::GetValue(long aRow, long aColumn, integer_matrix_type & aValue) {
	std::map<long,SparseRow>::iterator row = rows.find(aRow);
	if (row == rows.end())
		{
		aValue = 0;
		return ConstRC::Ok;
		}
	std::map<long,mpz_class>::iterator column = row->second.items.find(aColumn);
	if (column == row->second.items.end())
		aValue = 0;
	else
		aValue = mpz_get_si(column->second.get_mpz_t());
	return ConstRC::Ok;
}

mpz_t* MpzClassSparseMatrix::GetValueDirect(long aRow, long aColumn) {
	CLog::LogError(WHEREARG,"MpzClassSparseMatrix","GetValueDirect",aRow,aColumn,"GetValueDirect not supported.");
	return NULL;
}

MpzClassSparseMatrix * MpzClassSparseMatrix::clone() {
	return new MpzClassSparseMatrix();
}

int MpzClassSparseMatrix::ApplyModulo() {
	if (!modular) return ConstRC::Ok;
	for(std::map<long,SparseRow>::iterator row= rows.begin();row != rows.end();){
		for(std::map<long,mpz_class>::iterator column = row->second.items.begin(); column != row->second.items.end(); ) {
			mpz_mod(column->second.get_mpz_t(), column->second.get_mpz_t(),modulo);
			// Maybe the result is zero?
			if (column->second == 0)
				row->second.items.erase(column++);
			else
				++column;
		}
		// Maybe the row is empty now?
		if (row->second.items.size() == 0)
			rows.erase(row++);
		else
			row++;
	}
}

int MpzClassSparseMatrix::GetValueMpzClass(long aRow, long aColumn,
	mpz_class * & aValue) {
	std::map<long,SparseRow>::iterator row = rows.find(aRow);
	if (row == rows.end())
		{
		aValue = & mpz_zero;
		return ConstRC::Ok;
		}
	aValue =  row->second.GetValue(aColumn);
	return ConstRC::Ok;
}

int MpzClassSparseMatrix::MultiplyRow(long aRow, mpz_t aValue) {
	std::map<long,SparseRow>::iterator row = rows.find(aRow);
	//if there is no row, there is just nothing to multiply
	if (row == rows.end()) return ConstRC::Ok;
	return row->second.Multiply(aValue);
}

int MpzClassSparseMatrix::Test() {
	PrintToScreen();
	mpz_t testvalue;
	mpz_init(testvalue);
	for (int i = 5; i<11; i++) {
		mpz_set_ui(testvalue,i+1000);
		PutValueMpz(i,i,testvalue);
	}
	SetModulo(2);
	ApplyModulo();
	for (int i = 20; i>10; i--) {
		mpz_set_ui(testvalue,i+1000);
		PutValueMpz(i/3+10,i,testvalue);
	}
	PrintToScreen();
	for (int i = 5; i<11; i++) {
		mpz_set_ui(testvalue,0);
		PutValueMpz(i,i,testvalue);
	}
	PrintToScreen();
	mpz_clear(testvalue);
	Zeroize();
}


