/*
 *
 *      Author: jero
 */

#ifndef MPZSPARSEMATRIX_H_
#define MPZSPARSEMATRIX_H_

#include "abstract_mpz_matrix.h"
#include "abstract_matrix.h"
#include "mpz_matrix.hpp"
#include "gmp.h"
#include "gmpxx.h"

/*!
 * \brief Row of sparse mpz matrix
 */
class SparseRow {
	public:
		SparseRow();
		~SparseRow();

		long getItemCount() const;

		int PutValue(long aColumn, mpz_t aValue);
		int PutValue(long aColumn, long aValue);
		mpz_class * GetValue(long aColumn);

		int Multiply(mpz_t aValue);

		void Print();
		void Test();

		std::map<long,mpz_class> items;
		mpz_class mpz_zero;
};

/*!
 * \brief Sparse matrix with mpz elements
 */
class MpzClassSparseMatrix : public AbstractMpzMatrix {
public:
	MpzClassSparseMatrix();
	virtual ~MpzClassSparseMatrix();
	MpzClassSparseMatrix * clone();

	virtual int  Allocate(); ///< This method will be used for allocation of dynamic internal structures of a subclass.
	virtual int  Zeroize(); ///< This method will be used for initialization of the matrix by 0s.
	//virtual bool Equals(AbstractMatrix* aMatrix) = 0;
	virtual void PrintToScreen(); ///< This method will be used for display of the matrix.
	virtual int  PutOne(long aRow, long aColumn); ///< This method will be used to put number 1 to the entry indexed by aRow and aColumn.
	virtual int  PutZero(long aRow, long aColumn); ///< This method will be used to put number 0 to the entry indexed by aRow and aColumn.
	virtual int  IsOne(long aRow, long aColumn); ///< This method will respond whether there is a 1 at the entry.
	virtual int  IsZero(long aRow, long aColumn); ///< This method will respond whether there is a 0 at the entry.
	// Swaps etc.
	virtual int  ZeroizeRow(long aRow);; ///< This method will be used to put 0s into a row given by index aRow.

	virtual int MultiplyRow(long aRow, mpz_t aValue);

	virtual int  SwapRows(long aRow1, long aRow2); ///< 			This method will be used to swap rows with indices given by aRow1 and aRow2 parameters.

	virtual int  AddRows(long aTarget, long aSource); ///< 			This method will be used to add row with index aSource to row with index aTarget.

	virtual int  ZeroizeColumn(long aColumn); ///< 			This method will be used to put 0s into a column given by index aColumn.
	virtual int  ZeroizeColumn(long* aColumnList, long aListMaxIndex); ///<  This method will be used to put 0s into columns, whose indices are given by array aColumnList.			The aListMaxIndex parameter gives the final index in aColumnList array, in order to prevent			buffer overflow error.

	virtual int  SwapColumns(long aColumn1, long aColumn2);; ///< This method will be used to swap columns with indices given by aColumn1 and aColumn2 parameters.
	virtual int  AddColumns(long aTarget, long aSource); ///< This method will be used to add column with index aSource to column with index aTarget.

	virtual int  AddToRow(long aRow,AbstractMatrix* aOperand, long aRow2); ///< This method will be used to add rows from two different matrices of the same type.
	virtual int  AddToRow(long aRow,MpzMatrix* aOperand, long aRow2); ///< This method will be used to add rows from two different matrices of the same type.

	virtual int  Save(char *aName);
	virtual int  Load(char *aName);

	virtual int ApplyModulo();

	virtual int WriteData(xmlTextWriterPtr aWriter) const;
	virtual int ReadData(xmlTextReaderPtr aReader);

	int PutValueMpz(long aRow, long aColumn, mpz_t aValue);
	int GetValueMpz(long aRow, long aColumn, mpz_t aValue);
	int GetValueMpzClass(long aRow, long aColumn, mpz_class * & aValue);

	int PutValue(long aRow, long aColumn, integer_matrix_type aValue);
	int GetValue(long aRow, long aColumn, integer_matrix_type & aValue);
	virtual mpz_t * GetValueDirect(long aRow, long aColumn);

	int Test();

protected:
	std::map<long,SparseRow> rows;
	mpz_class mpz_zero;
	mpz_t mpz_t_interface;
};

#endif /* MPZSPARSEMATRIX_H_ */
