#ifndef __MSERIALIZABLE_H__
#define __MSERIALIZABLE_H__

#include "common_xml_helper.h"
#include "write_read_mutex.h"
#include <string>
#include "serialization_definitions.h"

// Forward declaration
class CBaseParameters;

///Interface for serializable classes
class MSerializable
{
private:
	/*!
		Default filename for serialization if no is given in serialization parameters.
		It must be protected by write-read mutex "iDefaultFileNameMutex"
	*/
	std::string iDefaultFileName;
	///Mutex protecting "iDefaultFileName"
	mutable CWriteReadMutex iDefaultFileNameMutex;

public:

	MSerializable() : iDefaultFileName(FILE_NAME_MSERIALIZABLE) {}
	MSerializable(std::string aDefaultFileName) : iDefaultFileName(aDefaultFileName) {}
	MSerializable(const MSerializable& aOperand) : iDefaultFileName(aOperand.Get_DefaultFileName()) {}

	virtual int Serialize(const CBaseParameters& aParam) const;
	virtual int Serialize(const CBaseParameters& aParam, xmlTextWriterPtr& aWriter) const = 0;
	virtual int Deserialize(const CBaseParameters& aParam);
	virtual int Deserialize(const CBaseParameters& aParam, xmlTextReaderPtr& aReader) = 0;

	// GETTERS/SETTERS

	void Set_DefaultFileName(std::string aDefaultFileName);
	std::string Get_DefaultFileName() const;

	// Static methods

	static int PrepareXMLReader(const CBaseParameters& aParam, const std::string& aElementName, xmlTextReaderPtr& aReader);
	static std::string CreateFileNameAuto(const CBaseParameters& aParam, const std::string& aFileNameDefault);
	static std::string CreateFileNameAuto(const CBaseParameters& aParam, const char* aFileNameDefault);
	static std::string CreateNewIdentifier();
	static void AppendPathSeparator(std::string& aPath);
	static void AdaptPathSeparator(std::string& aPath);
	static std::string CreateFileNamePlain(const char* aDirectoryName, const char* aFileName, const char* aIdentifier, const char* aExtension);
	static std::string CreateFileNamePlain(const std::string& aDirectoryName, const std::string& aFileName, const std::string& aIdentifier, const std::string& aExtension);
	static std::string CreateFileNameCompressed(const char* aDirectoryName, const char* aFileName, const char* aIdentifier, const char* aExtension);
	static std::string CreateFileNameCompressed(const std::string& aDirectoryName, const std::string& aFileName, const std::string& aIdentifier, const std::string& aExtension);
	static void RemoveFile(const char* aDirectoryName, const char* aFileName, const char* aIdentifier, const char* aExtension);
	static void RemoveFile(const std::string& aDirectoryName, const std::string& aFileName, const std::string& aIdentifier, const std::string& aExtension);
	static FILE* OpenFile(const char* aMode, const char* aFilePath);
	static int MCopyFile(const std::string& aInFile, const std::string& aOutFile);
	static bool ExistFile(const std::string& aFullFileName);
	static int RenameFile(const std::string& aOldFullFileName, const std::string& aNewFullFileName, bool aRemoveIfExists = true);

};

#endif // __MSERIALIZABLE_H__
