#ifndef __NODE_H__
#define __NODE_H__

#include "types_common.h"
#include <vector>	// c++ stdlib
#include <string>	// c++ stdlib
#include <pthread.h>	// POSIX thread
#include "receiver.h"
#include "abstract_message.h"

#define DEFAULT_CENTER_LIST_NAME "centers.xml"

using namespace std;

class AbstractConnector; // forward declaration

class CenterInfo;

class ConnectionInfo;

/*!
 * \brief Node part of distribution
 */
class Node : public Receiver {

public:
	Node(AbstractSerializator& aSerializator);
	virtual ~Node();

	void	RegisterCenter(CenterInfo* aInfo);

	int	ParseCenters(const char *str, const char *separ);

	int	Run();
	
	void	Break();

	void	LoadCenterList(const char* aDirectoryName, const char* aFileName = DEFAULT_CENTER_LIST_NAME);	

	virtual const char*  GetThisClassName()	const;

	void	RunDistNode();

protected:
	
	void	Lock();
	void    Unlock();

	void    SendReadyMessageToCenter(CenterInfo* aInfo, unsigned int aDelay = NO_DELAY_SEND);

	void	SendAckMessageToCenter(CenterInfo* aInfo, unsigned int aInReplyTo);
	void	SendLockedMessageToCenter(CenterInfo* aInfo, unsigned int aInReplyTo);
	void	SendNodeBusyMessageToCenter(CenterInfo* aInfo, unsigned int aInReplyTo);
	void	SendAliveMessageToCenter(CenterInfo* aInfo);
	void 	SendFreshDataMessageToCenter(CenterInfo* aInfo);
	
	// The execution method for SendAck, SendLocked, SendNodeBusy, SendAlive
	void	SendSimpleMessageToCenter(CenterInfo* aInfo, unsigned int aInReplyTo, message_type aType, const char* aSubject);

	void	ResendPendingMessages();

	int	CheckInitConsistency();
	void	PrintCenters();

	void	CreateNodeThread();	
	
	
	virtual void	ProcessReceivedMessage(AbstractMessage* aMessage);

	virtual bool	ProcessMessageByType(AbstractMessage* aMessage);

	virtual bool	ProcessNoJobMessage(AbstractMessage* aMessage);

	virtual bool	ProcessJobParametersMessage(AbstractMessage* aMessage);

	virtual bool	ProcessQuitMessage(AbstractMessage* aMessage);

	virtual bool	CheckMessageSequenceConsistency(AbstractMessage* aMessage);

	virtual void	RemoveFromPending(const char* aTargetName, unsigned int aCounter);

	virtual CenterInfo*	FindCenterByName(const char* aCenterName) const;
	

protected: // Member variables	

	bool		   iBreakNow;	///< If this is set to TRUE, the instance will be immediately terminated.
	
	vector<CenterInfo*> 	   iCenters;

	CenterInfo*	iCurrentlyLockedCenter;

	// Centers with non-Ready connectors
	vector<CenterInfo*>	   iFailedCenters;

	vector<CenterInfo*>	   iDupliciteCenters;

	bool		iLocked;
	
	unsigned int    iCounter; // Message counter

	unsigned int    iLastArchived; // Last 

	pthread_t 	threads[3];

	static const unsigned int KDataHarvestDefaultIntervalSecs;

	unsigned int	iDataHarvestIntervalSecs;

	static const unsigned int KNodeInstanceCheckIntervalSecs;

	string		iJobId;
};
#endif
