#include "node_info.h"
#include "abstract_message.h"
#include <time.h>
#include <string.h>

NodeInfo::NodeInfo(AbstractMessage* aMessage) : ConnectorInfo("",EUnknownConnector), iDataReceivedCount(0)
{	
	iLocked = FALSE;
	iNodeId.assign("(unknown)");	
	if (aMessage)
	{
		ConnectorInfo::Load(aMessage);
		if (aMessage->Get_Type() == EReady)
		{
			const char* senderId = aMessage->Get_SenderId();
			iNodeId.assign(senderId);
			//printf("NodeInfo::NodeInfo. Name read as %s\n",iNodeId.c_str());		
		}		
	}
}

NodeInfo::~NodeInfo()
{
}

const char* NodeInfo::GetId() const
{
	return iNodeId.c_str();
}

bool  NodeInfo::Identical(NodeInfo* aOtherNode)
{
	if (!aOtherNode) return FALSE;
	if (!aOtherNode->GetId()) return FALSE;
	if (!GetId()) return FALSE;
	return !(strcmp(aOtherNode->GetId(),GetId()));
}

void	NodeInfo::Set_Locked()
{
	 // Will set current time as iLockTime.
	iLocked = TRUE;
	iLockTime = time(NULL);
}
void	NodeInfo::Update_LastReceived()
{
 	// Will set current time as iLastMessageReceivedTime
	iLastMessageReceivedTime = time(NULL);
}

void	NodeInfo::PrintInfo() const
{
 	// Prints out a short info.
	const char* lockedStatus = ( iLocked ? "LOCKED SINCE " : "NOT LOCKED ");
	printf("Node\t\"%10s\"\t: %10s",iNodeId.c_str(), lockedStatus);
	if (iLocked)
	{
		PrintTime(iLockTime);
	}	
	printf("; Last message arrived at ");
	PrintTime(iLastMessageReceivedTime);
	printf(". Data received: %8d %s so far.",iDataReceivedCount,iDataUnit.c_str());
	printf("\n");
}

	// Prints time "intelligently"
void 	NodeInfo::PrintTime(const time_t& aTime)
{	
	// Seconds since Jan 1, 1970
	// If the time is in the past, but less than 24 hrs:
	struct tm * timeinfo;		  
  	timeinfo = localtime ( &aTime );

	if (IsToday(aTime))
	{
		char buf[25];
		memset(buf, 0, 25); 		
		strftime ( buf, 25, "%H:%M:%S", timeinfo );
		printf("%s",buf);
	}
	else
	{
		char buf[60];
		memset(buf, 0, 60);
		strftime ( buf, 25, "%a %b %d %Y, %H:%M:%S", timeinfo );
		printf("%s",buf);
	}
}

bool 	NodeInfo::IsToday(const time_t& aTime)
{
	time_t now = time(NULL);
	int day1 = now / 86400;
	int day2 = aTime / 86400;
	return (day1 == day2);
}

void	NodeInfo::IncrementDataReceived(unsigned int aHowMuch)
{
	iDataReceivedCount += aHowMuch;
}

void	NodeInfo::SetDataUnit(const string& aDataUnit)
{
	iDataUnit.assign(aDataUnit);
}



