#ifndef URL_H
#define URL_H

#include <string>

using namespace std;

const int KNoPortDetermined = 0;

typedef enum { EOk, EMalformedScheme, EGeneralError, ECannotProcessQueryAndFragment } TParsingResult;

/*!
 * \brief URL processing class
 */
class URL 
{
public:
	URL(const string& aUrl);
	virtual ~URL();

	string GetScheme()   const;
	bool   HasScheme()   const;

	string GetUsername() const;
	bool   HasUsername() const;

	string GetPassword() const;
	bool   HasPassword() const;

	string GetDomain()   const;
	bool   HasDomain()   const;

	string GetFullPath() const;
	bool   HasFullPath() const;

	string GetPath()     const;
	bool   HasPath()     const;

	string GetFilename() const;
	bool   HasFilename() const;

	string GetExtension() const;
	bool   HasExtension() const;

	string GetParamString() const;
	bool   HasParamString() const;
	
	string GetQuery() const;
	bool   HasQuery() const;

	string GetFragment() const;
	bool   HasFragment() const;

	int    GetPort() const;
	bool   HasPort() const;

	string GetCanonicalForm() const;

	TParsingResult GetParsingResult() const;
	bool   IsOk() const;

	void   PrintExtended() const;

	static void SelfTest();

protected:
	void   Parse(const string& aUrl);
	void   Reset(); // Clears all.

private:
	void   FindPort(const string& aDomain);

protected:
	string iCanonicalForm;
	string iScheme;
	string iUsername;
	string iPassword;
	string iDomain;
	int    iPort;
	string iPath;
	string iFullPath;
	string iFilename;
	string iExtension;
	string iParamString;
	string iQuery;
	string iFragment;

	TParsingResult iResult;	
};

#endif
