/* polynom_Z_n *
BM_matrix (unsigned int *sequence, unsigned long int length)
{
	unsigned int lambda, t, d, i, i2, move;
	signed long long m;

	if ((P = (polynom_GF2 *) malloc (sizeof (polynom_GF2))) == NULL)
	{
		fprintf (stderr,
			 "Not enough memory. (BM_bit_matrix/wiedemann.c)");
		return NULL;
	};
	P->m = allocate_matrix(0, length);
	zeroize (P->m);
	put_one (P->m, 0, 0);
	P->length = 1;
	P2.m = allocate_matrix(0, length);
	zeroize (P2.m);
	put_one (P2.m, 0, 0);
	P2.length = 1;
	T.m = allocate_bit_matrix (0, length);

	lambda = 0;
	m = -1;

	for (t = 0; t < length; t++)
	{
		// spocitani "discrepancy"
		d = sequence[t];
		if (lambda > 0)
		{
			for (i = 1; i <= lambda; i++)
			{
				d += bit_element (sequence,
						  t -
						  i) & bit_element (P->m->
								    matrix[0],
								    i);
			};
			d = d % Modulus;
		};
		if (d != 0)
		{
			T.length = P->length;
			copy_bit_matrix (T.m, P->m);
			// Aktualizace P(X)
			if (P->length < P2.length + t - m)
				P->length = P2.length + t - m;
			i = t - m;
			move = i % (sizeof (unsigned int) * 8);
			i = i / (sizeof (unsigned int) * 8);
			i2 = 0;
			while (i2 <= P2.length / (sizeof (unsigned int) * 8))
			{
				P->m->matrix[0][i++] ^=
					P2.m->matrix[0][i2] << move;
				P->m->matrix[0][i] ^=
					P2.m->
					matrix[0][i2++] >>
					(sizeof (unsigned int) * 8 - move);
			};
			// Konec aktualizace P(X)
			if (2 * lambda <= t)
			{
				lambda = t + 1 - lambda;
				m = t;
				P2.length = T.length;
				copy_bit_matrix (P2.m, T.m);
			};
		};
	};
	delete_bit_matrix (&P2.m);
	delete_bit_matrix (&T.m);
	return P;
};
*/

#include "wiedemann_z_p.h"
#include "const_return_codes.h"
#include "abstract_mpz_matrix.h"
#include "abstract_integer_matrix.h"
#include "abstract_matrix.h"
#include <iostream>

WiedemannZP::WiedemannZP() {
	mpz_init(modulo);
	mpz_init(tempmpz);
	RightMatrix = NULL;
	LeftMatrix = NULL;
	polynomial = NULL;
}

WiedemannZP::~WiedemannZP() {
	if (RightMatrix != NULL) delete RightMatrix;
	mpz_clear(modulo);
	mpz_clear(tempmpz);
}

int WiedemannZP::SetMainMatrix(AbstractIntegerMatrix* aMatrix) {
	if (aMatrix->GetMaximalColumnIndex() != aMatrix->GetMaximalRowIndex()) return ConstRC::GeneralError;
	matrix = aMatrix;
	return ConstRC::Ok;
}


int WiedemannZP::SetModulo(mpz_t* aModulo) {
	mpz_set(modulo, *aModulo);
}


int WiedemannZP::SetModulo(int aValue) {
	mpz_set_ui(modulo,aValue);
	return ConstRC::Ok;
}

/*
IntegerSparseMatrix matrix nxn
RS MpzMatrix 1xn

leftmatrixMpzMatrix nxk
multresult 1xk
sequence Lxk
*/


int WiedemannZP::GenerateSequence() {
	MpzMatrix *leftmatrix = new MpzMatrix();
	MpzMatrix *rightmatrix = new MpzMatrix();
	MpzMatrix *rightmatrix2 = new MpzMatrix();
	MpzMatrix *swappointer;
	if (RightMatrix == NULL)
		RightMatrix = new MpzMatrix();
	MpzMatrix *multresult = new MpzMatrix();


	leftmatrix->SetMaximalColumnIndex(matrix->GetMaximalRowIndex());
	leftmatrix->SetMaximalRowIndex(0);
	RightMatrix->SetMaximalColumnIndex(0);
	RightMatrix->SetMaximalRowIndex(matrix->GetMaximalRowIndex());
	rightmatrix->SetMaximalColumnIndex(0);
	rightmatrix->SetMaximalRowIndex(matrix->GetMaximalRowIndex());
	rightmatrix2->SetMaximalColumnIndex(0);
	rightmatrix2->SetMaximalRowIndex(matrix->GetMaximalRowIndex());

	leftmatrix->Allocate();
	leftmatrix->Copy(LeftMatrix);
    leftmatrix->SetModulo(modulo);
    leftmatrix->ApplyModulo();

	RightMatrix->Allocate();
	RightMatrix->SetModulo(modulo);
	RightMatrix->Randomize();
	rightmatrix->Allocate();
	rightmatrix->Copy(RightMatrix);
	rightmatrix2->Allocate();
	rightmatrix2->SetModulo(modulo);


	multresult->SetMaximalRowIndex(leftmatrix->GetMaximalRowIndex());
    multresult->SetMaximalColumnIndex(rightmatrix->GetMaximalColumnIndex());
    multresult->Allocate();
    multresult->SetModulo(modulo);

    // cout << "\nLeft:";
    // leftmatrix->PrintToScreen();
    // cout << "\nRight";
    // rightmatrix->PrintToScreen();

    sequence_length = 2*(matrix->GetMaximalRowIndex()+1);

    sequence = new MpzMatrix();
    sequence->SetMaximalRowIndex(sequence_length-1);
    sequence->SetMaximalColumnIndex(multresult->GetMaximalColumnIndex());
    sequence->Allocate();

    int rc;
    //cout << endl;
    for (long index = 0; index < sequence_length; index ++) {
		// cout << "\r Generating sequence for Wiedemann: " << index << "/" << sequence_length - 1<<"     ";
		// right->PrintToScreen();
		rc = multresult->Multiply(*leftmatrix, *rightmatrix);
		if (ConstRC::Ok != rc) abort();

		sequence->AddToRow(index, multresult,0);

		rc = rightmatrix2->Multiply(*matrix, *rightmatrix);
		if (ConstRC::Ok != rc) abort();

		swappointer = rightmatrix2;
		rightmatrix2 = rightmatrix;
		rightmatrix = swappointer;
	};
    //cout << endl;
	// sequence->PrintToScreen();
	delete leftmatrix;
	delete rightmatrix;
	delete rightmatrix2;
	delete multresult;
};

int WiedemannZP::ComputePolynomial() {
	Polynomial *P = new Polynomial();
	Polynomial *P2 = new Polynomial();
	Polynomial *T = new Polynomial();
	Polynomial *T2 = new Polynomial();
	Polynomial *swappointer;

	T->Zeroize();

	mpz_t d;
	mpz_t d2;
	mpz_t multemp;
	mpz_t multemp2;
	mpz_init(d);
	mpz_init(d2);
	mpz_init(multemp);
	mpz_init(multemp2);

	long lambda = 0;
	long m = -1;
	P->Zeroize();
	P->Set(0,1);
	P->Modulo(modulo);
	P2->Zeroize();
	P2->Set(0,1);
	P2->Modulo(modulo);

	for (long t = 0; t < sequence_length; t++)
	{
		// spocitani "discrepancy"
		sequence->GetValueMpz(t,0,d);
		if (lambda > 0)
		{
			for (long i = 1; i <= lambda; i++)
			{
				sequence->GetValueMpz(t-i, 0,multemp);
				mpz_mul(multemp,multemp,*P->Get2(i));
				mpz_add(d,d,multemp);
			};
			mpz_mod(d,d,modulo);
		};
		if (mpz_cmp_ui(d,0) !=0)
		{
			// T(x) := P(x)-d*d2^-1*x^(t-m)*P2(x)
			P2->Copy(T2);
			T2->MultiplyByPowerOfX(t-m);
			mpz_invert(multemp,d2,modulo);
			mpz_mul(multemp,multemp,d);
			T->Multiply(T,T2,multemp);
			T2->Substract(T2,P,T);
			if (2*lambda > t) {
				swappointer = P;
				P = T2;
				T2 = swappointer;
			}else {
				swappointer = P2;
				P2 = P;
				P = T2;
				T2 = swappointer;
				m = t;
				mpz_set(d2,d);
				lambda = t + 1 - lambda;
			}


		}
		/*
		// debug after every iteration
		cout << "Lambda:" << lambda << "    ";
		P->Modulo(modulo);
		P->PrintToScreen();
		for (long debug_t = lambda; debug_t <= t; debug_t++) {
			mpz_set_ui(d,0);
			for (long i = 0; i <= lambda; i++)
			{
				sequence->GetValueMpz(debug_t-i, 0,multemp);
				mpz_mul(multemp,multemp,*P->Get2(i));
				mpz_add(d,d,multemp);
			};
			mpz_mod(d,d,modulo);
			if (mpz_cmp_ui(d,0) != 0) cout  << "!"<<debug_t;
				else cout << ".";
		};
		*/
	};
	if (NULL != polynomial) delete polynomial;
	polynomial = P;
	polynomial->Modulo(modulo);
	polynomial_length = lambda;

	mpz_invert(d,*polynomial->Leading2(),modulo);
	// polynomial->MultiplyMod(d,modulo);
	// polynomial->PrintToScreen();
	// cout <<"\n Length:" <<lambda;
	// CheckPolynomial(sequence_length);
	delete P2;
	delete T;
	delete T2;
	mpz_clear(d);
	mpz_clear(d2);
	mpz_clear(multemp);
	mpz_clear(multemp2);
}

int WiedemannZP::SetRightMatrix(AbstractIntegerMatrix* aMatrix, long aRow) {
	return ConstRC::NotSupported;
}

int WiedemannZP::SetLeftMatrix(AbstractIntegerMatrix* aMatrix, long aRow) {
	if (NULL != LeftMatrix) delete LeftMatrix;
	LeftMatrix = new MpzMatrix();
	LeftMatrix->SetMaximalRowIndex(0);
	LeftMatrix->SetMaximalColumnIndex(aMatrix->GetMaximalColumnIndex());
	LeftMatrix->Allocate();

	// TODO: CopyToRow
	LeftMatrix->Zeroize();
	LeftMatrix->AddToRow(0,aMatrix,aRow);
}

/*! If degree of minimal polynomial if equal to the order of the matrix,
 *  the minimal polynomial is equal to the characteristic polynomial
 *  and its constant coefficient is equal to the determinant of the matrix
 *  multiplied by (-1)^order
 *  \param aDeterminant the probable value of determinant is returned in this variable
 *  \return
 *  - ConstRC::Ok if minimal polynomial is equal to the characteristic polynomial a we are sure we have determinant
 *  - ConstRC::GeneralError if minimal polynomial is not equal to the characteristic polynomial
 */

int WiedemannZP::GetDeterminant(mpz_t aDeterminant) {
	mpz_set(aDeterminant,* polynomial->Leading2());
	mpz_invert(aDeterminant,aDeterminant,modulo);
	mpz_mul(aDeterminant, aDeterminant, * (polynomial->Get2(0)));
	if ((matrix->GetMaximalColumnIndex()+1) % 2 == 1) mpz_mul_si(aDeterminant,aDeterminant,-1);
	if (polynomial->GetDegree() == matrix->GetMaximalColumnIndex()+1)
		return ConstRC::Ok;
	else
		return ConstRC::GeneralError;
}

int WiedemannZP::Compute(AbstractIntegerMatrix *aResultMatrix, long aRowIndex) {
	MpzMatrix *leftmatrix = new MpzMatrix();
	MpzMatrix *leftmatrix2 = new MpzMatrix();
	MpzMatrix *swappointer;

	AbstractIntegerMatrix * transposedmatrix = matrix->clone();
	transposedmatrix->SetMaximalRowIndex(matrix->GetMaximalColumnIndex());
	transposedmatrix->SetMaximalColumnIndex(matrix->GetMaximalRowIndex());
	transposedmatrix->Allocate();
	transposedmatrix->Transpose(matrix);

	leftmatrix->SetMaximalColumnIndex(matrix->GetMaximalRowIndex());
	leftmatrix->SetMaximalRowIndex(0);
	leftmatrix2->SetMaximalColumnIndex(matrix->GetMaximalRowIndex());
	leftmatrix2->SetMaximalRowIndex(0);

	leftmatrix->SetModulo(modulo);
	leftmatrix2->SetModulo(modulo);

	leftmatrix->Allocate();
	leftmatrix2->Allocate();
	leftmatrix->Zeroize();

	polynomial->PrintToScreen();
	for (long index = 0;index < polynomial_length; index ++) {
		leftmatrix2->MultiplyTransposed(*leftmatrix, *transposedmatrix);
		leftmatrix->Copy(LeftMatrix);
		leftmatrix->Multiply(* polynomial->Get2(index));
		// cout <<"Add:";
		// leftmatrix2->PrintToScreen();
		leftmatrix->AddToRow(0,leftmatrix2,0);
		// cout <<"\nIter:" << index;
		// leftmatrix->PrintToScreen();
	};

	mpz_invert(tempmpz,*polynomial->Get2(polynomial->GetDegree()

	), modulo);
	mpz_sub(tempmpz,modulo, tempmpz);
	leftmatrix->Multiply(tempmpz);
	if (aResultMatrix)
	{
		// TODO: CopyToRow
		aResultMatrix->ZeroizeRow(aRowIndex);
		aResultMatrix->AddToRow(aRowIndex, leftmatrix,0);
	}
	// cout << "\n Result:";
	// leftmatrix->PrintToScreen();
	leftmatrix2->MultiplyTransposed(*leftmatrix,*transposedmatrix);
	  // cout << "\n Multiplied:";
      // leftmatrix2->PrintToScreen();
	 leftmatrix->Copy(LeftMatrix);
	 leftmatrix->SetModulo(modulo);
	 leftmatrix->ApplyModulo();
	  // cout << "\n vs Given:";
	  // leftmatrix->PrintToScreen();
	bool correct = leftmatrix->Equals(leftmatrix2);


	delete leftmatrix;
	delete leftmatrix2;
	delete transposedmatrix;
	if (correct)
		return ConstRC::Ok;
	else
		return ConstRC::GeneralError;

}

int
WiedemannZP::CheckPolynomial(int length) {
	long iterations = length - polynomial_length;
	mpz_t tempadd;
	mpz_t tempmul;
	mpz_init(tempadd);
	mpz_init(tempmul);
	for (long index = 0; index<iterations; index++){
		mpz_set_ui(tempadd,0);
		for(long poly=0;poly<=polynomial_length;poly++) {
			mpz_mul(tempmul,*polynomial->Get2(polynomial_length - poly), *sequence->GetValueDirect(index+poly,0));
			// gmp_printf("[%Zd*%Zd=%Zd]",*polynomial->Get2(polynomial_length - poly), *sequence->GetValueDirect(index+poly,0), tempmul);
			mpz_add(tempadd,tempadd,tempmul);
		}
		mpz_mod(tempadd,tempadd, modulo);
		// gmp_printf(" ->%Zd<-",tempadd);
	}
	mpz_clear(tempadd);
	mpz_clear(tempmul);
}

int WiedemannZP::Check() {
	int rc;
	MpzMatrix * testmatrix = new MpzMatrix();
	// cout << "\n CHECK!";
	testmatrix->SetMaximalColumnIndex(matrix->GetMaximalColumnIndex());
	testmatrix->SetMaximalRowIndex(matrix->GetMaximalRowIndex());
	testmatrix->Allocate();
	testmatrix->Copy(matrix);
	testmatrix->SetModulo(modulo);
	testmatrix->ApplyModulo();
	// testmatrix->PrintToScreen();
	MpzMatrix * addmatrix = new MpzMatrix();
	addmatrix->SetModulo(modulo);
	addmatrix->SetMaximalColumnIndex(testmatrix->GetMaximalColumnIndex());
	addmatrix->SetMaximalRowIndex(0);
	addmatrix->Allocate();
	addmatrix->Zeroize();
	MpzMatrix * mulmatrix = new MpzMatrix();
	mulmatrix->SetModulo(modulo);
	mulmatrix->SetMaximalColumnIndex(testmatrix->GetMaximalColumnIndex());
	mulmatrix->SetMaximalRowIndex(0);
	mulmatrix->Allocate();
	mulmatrix->Zeroize();
	MpzMatrix * swappointer;
    for (long degree = 0; degree <= polynomial_length; degree++) {
    	mulmatrix->Multiply(* addmatrix,*matrix);
    	// cout <<"Iter:" << degree << endl;
     	// mulmatrix->PrintToScreen();
    	addmatrix->Copy(LeftMatrix);
    	addmatrix->Multiply(* polynomial->Get2(degree));
    	// addmatrix->PrintToScreen();
    	addmatrix->AddToRow(0,mulmatrix,0);
    }
    // cout <<"\nCheck";
	// addmatrix->PrintToScreen();
	MpzMatrix * onetimesone = new MpzMatrix();
	onetimesone->SetMaximalColumnIndex(0);
	onetimesone->SetMaximalRowIndex(0);
	onetimesone->SetModulo(modulo);
	onetimesone->Allocate();
	onetimesone->Multiply(*addmatrix, * RightMatrix);
	// onetimesone->PrintToScreen();
	bool checkvector = (TRUE == addmatrix->AbstractIntegerMatrix::IsZero());
	bool checkscalar = (TRUE == onetimesone->AbstractIntegerMatrix::IsZero());
	delete mulmatrix;
	delete addmatrix;
	delete testmatrix;
	delete onetimesone;
	if (true == checkvector)
		return ConstRC::Ok;
	else
		return ConstRC::GeneralError;
}
