/*
 * wiedeman_z_p.h
 *
 *      Author: jero
 */

#ifndef WIEDEMAN_Z_P_H_
#define WIEDEMAN_Z_P_H_

#include "integer_sparse_matrix.h"
#include "mpz_matrix.hpp"

/*!
 * \brief Wiedemann solver for matrices over Z_p
 */
class WiedemannZP {
public:
	WiedemannZP ();
	~WiedemannZP ();
	int SetMainMatrix(AbstractIntegerMatrix *aMatrix);
	int SetRightMatrix(AbstractIntegerMatrix *aMatrix, long aRow);
	int SetLeftMatrix(AbstractIntegerMatrix *aMatrix, long 	aRow);
	int SetModulo(mpz_t *aModulo);
	int SetModulo(int aValue);

	int GenerateSequence();
	int ComputePolynomial();
	int CheckPolynomial(int length);
	int GetDeterminant(mpz_t aDeterminant);
	int Compute(AbstractIntegerMatrix *ResultMatrix, long aRowIndex);

	int Check();

private:
	AbstractIntegerMatrix *matrix;
	MpzMatrix *rightmatrix;
	MpzMatrix *RightMatrix;

	MpzMatrix *LeftMatrix;
	MpzMatrix *sequence;

	long sequence_length;

	Polynomial *polynomial;
	long polynomial_length;

	mpz_t modulo;
	mpz_t tempmpz;
};


#endif /* WIEDEMAN_Z_P_H_ */
