#include "write_read_mutex.h"
#include "common_definitions.h"
#include "common_definitions.h"

CWriteReadMutex::CWriteReadMutex() : iMaxReaders(DEFAULT_WRMUTEX_MAX_READERS)
{
	this->iReadersCount = 0;
	/* Initialize mutex and condition variable objects */
	pthread_mutex_init(&this->iReadersCountMutex, NULL);
	pthread_mutex_init(&this->iWriteMutex, NULL);
	pthread_cond_init(&this->iReadersCountCond, NULL);
}

CWriteReadMutex::CWriteReadMutex(const int& aMaxReaders) : iMaxReaders(aMaxReaders)
{
	this->iReadersCount = 0;
	/* Initialize mutex and condition variable objects */
	pthread_mutex_init(&this->iReadersCountMutex, NULL);
	pthread_mutex_init(&this->iWriteMutex, NULL);
	pthread_cond_init(&this->iReadersCountCond, NULL);
}

CWriteReadMutex::CWriteReadMutex(const CWriteReadMutex &aOperand) : iMaxReaders(aOperand.iMaxReaders)
{
	this->iReadersCount = 0;
	/* Initialize mutex and condition variable objects */
	pthread_mutex_init(&this->iReadersCountMutex, NULL);
	pthread_mutex_init(&this->iWriteMutex, NULL);
	pthread_cond_init(&this->iReadersCountCond, NULL);
}

CWriteReadMutex::~CWriteReadMutex()
{
	/* Clean up */
	pthread_mutex_destroy(&this->iReadersCountMutex);
	pthread_mutex_destroy(&this->iWriteMutex);
	pthread_cond_destroy(&this->iReadersCountCond);
}

void CWriteReadMutex::LockRead()
{
	pthread_mutex_lock(&this->iReadersCountMutex);
	while (this->iReadersCount == this->iMaxReaders)
	{
		pthread_cond_wait(&this->iReadersCountCond, &this->iReadersCountMutex);
	}

	this->iReadersCount++;

	pthread_mutex_unlock(&this->iReadersCountMutex);
}

void CWriteReadMutex::UnlockRead()
{
	pthread_mutex_lock(&this->iReadersCountMutex);

	this->iReadersCount--;
	if (this->iReadersCount == this->iMaxReaders - 1)
	{
		pthread_cond_broadcast(&this->iReadersCountCond);
	}

	pthread_mutex_unlock(&this->iReadersCountMutex);
}

void CWriteReadMutex::LockWrite()
{
	pthread_mutex_lock(&this->iWriteMutex);
	pthread_mutex_lock(&this->iReadersCountMutex);

	for (int i = 0; i < this->iMaxReaders; i++)
	{
		while (this->iReadersCount == this->iMaxReaders)
		{
			pthread_cond_wait(&this->iReadersCountCond, &this->iReadersCountMutex);
		}

		this->iReadersCount++;
	}

	pthread_mutex_unlock(&this->iReadersCountMutex);
	pthread_mutex_unlock(&this->iWriteMutex);
}

void CWriteReadMutex::UnlockWrite()
{
	pthread_mutex_lock(&this->iReadersCountMutex);

	this->iReadersCount = 0;
	
	pthread_cond_broadcast(&this->iReadersCountCond);
	
	pthread_mutex_unlock(&this->iReadersCountMutex);
}
