#include "abstract_factor_alg_parameters.h"
#include "const_return_codes.h"
#include "log.h"


using namespace std;

AFactorAlgParameters::AFactorAlgParameters()
{
	this->iPhasesCount = 0;
	this->iPhasesParameters = NULL;
	this->iPhasesNames = NULL;
}
AFactorAlgParameters::AFactorAlgParameters(const AFactorAlgParameters& aOperand) : MSerializable(aOperand)
{
	this->iPhasesCount = aOperand.iPhasesCount;
	this->iPhasesNames = new string[this->iPhasesCount];
	this->iPhasesParameters = new CBaseParameters[this->iPhasesCount];

	for (int i = 0; i < this->iPhasesCount; i++)
	{
		this->iPhasesNames[i] = aOperand.iPhasesNames[i];
		aOperand.iPhasesParameters[i].Copy2(this->iPhasesParameters[i]);
	}
}
AFactorAlgParameters::~AFactorAlgParameters()
{}
int AFactorAlgParameters::AddPhaseNonSerializableParam(const string& aName, int aPhase)
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","AddPhaseNonSerializableParam",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","AddPhaseNonSerializableParam",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].AddNonSerializableParam(aName);
}
int AFactorAlgParameters::AddPhaseParameter(const string &aName, const string &aValue, int aPhase)
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","AddPhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","AddPhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].AddParameter(aName, aValue);
}
int AFactorAlgParameters::AddParameterForOthers(const string &aName, const string &aValue, int aFromPhase)
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","AddParameterForOthers",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aFromPhase < 0 || aFromPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","AddParameterForOthers",0,0,"aFromPhase is out og range.");
		return ConstRC::BadArgument;
	}

	int rc = ConstRC::Ok;

	for (int i = aFromPhase + 1; i < this->iPhasesCount; i++)
	{
		rc = min(rc, this->iPhasesParameters[i].AddParameter(aName, aValue));
	}

	return rc;
}
int AFactorAlgParameters::SetPhaseParameter(const string &aName, const string &aValue, int aPhase)
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","SetPhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","SetPhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].SetParameter(aName, aValue);
}
int AFactorAlgParameters::SetParameterForOthers(const string &aName, const string &aValue, int aFromPhase)
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","SetParameterForOthers",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aFromPhase < 0 || aFromPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","SetParameterForOthers",0,0,"aFromPhase is out og range.");
		return ConstRC::BadArgument;
	}

	int rc = ConstRC::Ok;

	for (int i = aFromPhase + 1; i < this->iPhasesCount; i++)
	{
		rc = min(rc, this->iPhasesParameters[i].SetParameter(aName, aValue));
	}

	return rc;
}
bool AFactorAlgParameters::ContainsPhaseParameter(const string &aName, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","ContainsPhaseParameter",0,0,"Phase parameters are NULL.");
		return false;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","ContainsPhaseParameter",0,0,"aPhase is out og range.");
		return false;
	}

	return this->iPhasesParameters[aPhase].ContainsParameter(aName);
}
int AFactorAlgParameters::GetStringPhaseParameter(const string &aName, string &aResult, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetStringPhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetStringPhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].GetStringParameter(aName, aResult);
}
int AFactorAlgParameters::GetStringPhaseParameter(const string &aName, string aDefaultValue, string &aResult, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetStringPhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetStringPhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].GetStringParameter(aName, aDefaultValue, aResult);
}
int AFactorAlgParameters::GetIntegerPhaseParameter(const string &aName, int &aResult, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetIntegerPhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetIntegerPhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].GetIntegerParameter(aName, aResult);
}
int AFactorAlgParameters::GetIntegerPhaseParameter(const string &aName, int aDefaultValue, int &aResult, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetIntegerPhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetIntegerPhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].GetIntegerParameter(aName, aDefaultValue, aResult);
}
int AFactorAlgParameters::GetLongPhaseParameter(const string &aName, long &aResult, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetLongPhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetLongPhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].GetLongParameter(aName, aResult);
}
int AFactorAlgParameters::GetLongPhaseParameter(const string &aName, long aDefaultValue, long &aResult, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetLongPhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetLongPhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].GetLongParameter(aName, aDefaultValue, aResult);
}
int AFactorAlgParameters::GetBoolPhaseParameter(const string &aName, bool &aResult, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetBoolPhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetBoolPhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].GetBoolParameter(aName, aResult);
}
int AFactorAlgParameters::GetBoolPhaseParameter(const string &aName, bool aDefaultValue, bool &aResult, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetBoolPhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetBoolPhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].GetBoolParameter(aName, aDefaultValue, aResult);
}
int AFactorAlgParameters::GetDoublePhaseParameter(const string &aName, double &aResult, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetDoublePhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetDoublePhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].GetDoubleParameter(aName, aResult);
}
int AFactorAlgParameters::GetDoublePhaseParameter(const string &aName, double aDefaultValue, double &aResult, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetDoublePhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetDoublePhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].GetDoubleParameter(aName, aDefaultValue, aResult);
}
int AFactorAlgParameters::GetMpzPhaseParameter(const string &aName, mpz_t aResult, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetMpzPhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetMpzPhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].GetMpzParameter(aName, aResult);
}
int AFactorAlgParameters::GetMpzPhaseParameter(const string &aName, mpz_t aDefaultValue, mpz_t aResult, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetMpzPhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetMpzPhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].GetMpzParameter(aName, aDefaultValue, aResult);
}
int AFactorAlgParameters::GetPolynomialPhaseParameter(const string &aName, Polynomial *& aResult, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetPolynomialPhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetPolynomialPhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].GetPolynomialParameter(aName, aResult);
}
int AFactorAlgParameters::GetPolynomialPhaseParameter(const string &aName, const Polynomial * aDefaultValue, Polynomial *& aResult, int aPhase) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetPolynomialPhaseParameter",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","GetPolynomialPhaseParameter",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}

	return this->iPhasesParameters[aPhase].GetPolynomialParameter(aName, aDefaultValue, aResult);
}
int AFactorAlgParameters::CreateFullFileName(string& aResult, const string& aParamFileName, const string& aParamDir, const string& aParamCompLevel, const string& aDir, const string& aFileName, const string& aIdentifier, int aPhase, const string& aExtension) const
{
	if (this->iPhasesParameters == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","CreateFullFileName",0,0,"Phase parameters are NULL.");
		return ConstRC::GeneralError;
	}

	if (aPhase < 0 || aPhase >= this->iPhasesCount)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","CreateFullFileName",0,0,"aPhase is out og range.");
		return ConstRC::BadArgument;
	}
	
	string dir;
	string file;
	int comp;

	this->iPhasesParameters[aPhase].GetStringParameter(aParamFileName, aFileName, file);
	this->iPhasesParameters[aPhase].GetStringParameter(aParamDir, aDir, dir);
	this->iPhasesParameters[aPhase].GetIntegerParameter(aParamCompLevel, DEFAULT_COMPRESSION_LEVEL, comp); 

	// control path separator
	MSerializable::AdaptPathSeparator(dir);

	// get file name with path
	if (comp == 0)
	{
		aResult = MSerializable::CreateFileNamePlain(dir, file, aIdentifier, aExtension);
	}
	else
	{
		aResult = MSerializable::CreateFileNameCompressed(dir, file, aIdentifier, aExtension);
	}

	return ConstRC::Ok;
}
int AFactorAlgParameters::Serialize(const CBaseParameters &aParam) const
{
	return MSerializable::Serialize(aParam);
}
int AFactorAlgParameters::Serialize(const CBaseParameters& aParam, xmlTextWriterPtr &aWriter) const
{
	string mainTagName;
	aParam.GetStringParameter(PARAM_SERIALIZATION_ELEMENT_NAME, XML_SERIAL_FACTOR_ALG_PARAM_NAME, mainTagName);

	int rc = xmlTextWriterStartElement(aWriter, BAD_CAST mainTagName.c_str());
    if (rc < 0) 
	{
        CLog::LogError(WHEREARG,"AFactorAlgParameters","Serialize Parameters",rc,0,"Could not write the parameter tag.");
    	return ConstRC::GeneralError;
    }
	rc = CommonXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	// we can serialize phase's parameters
	CBaseParameters parameters;
	for (int i = 0; i < this->iPhasesCount; i++)
	{
		parameters.SetParameter(PARAM_SERIALIZATION_ELEMENT_NAME, this->iPhasesNames[i]);
		rc = this->iPhasesParameters[i].Serialize(parameters, aWriter);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	rc = xmlTextWriterEndElement(aWriter); // mainTagName
    if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","Serialize Parameters",rc,0,"Could not end the parameter tag.");
        return ConstRC::GeneralError;
    }
	rc = CommonXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	return ConstRC::Ok;    
}
int AFactorAlgParameters::Deserialize(const CBaseParameters &aParam)
{
	return MSerializable::Deserialize(aParam);
}
int AFactorAlgParameters::Deserialize(const CBaseParameters& aParam, xmlTextReaderPtr &aReader)
{
	string mainTagName;
	aParam.GetStringParameter(PARAM_SERIALIZATION_ELEMENT_NAME, XML_SERIAL_FACTOR_ALG_PARAM_NAME, mainTagName);

	int rc;
	xmlChar* name = xmlTextReaderName(aReader);

	if (name == NULL)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","Deserialize Parameters",0,0,"Deserialization main element name not found.");
		return ConstRC::GeneralError;  
	}

	if (xmlStrEqual(name,BAD_CAST mainTagName.c_str()) == 0)
	{
		CLog::LogError(WHEREARG,"AFactorAlgParameters","Deserialize Parameters",0,0,"Main element name is wrong.");
		xmlFree(name);
		return ConstRC::GeneralError; 
	}

	int depth = xmlTextReaderDepth(aReader);
	CBaseParameters parameters;

	while ((rc = CXMLHelper::MoveToNextNode(aReader, depth)) == ConstRC::Ok)
	{
		if (NULL != name) xmlFree(name);
		name = xmlTextReaderName(aReader);

		for (int i = 0; i < this->iPhasesCount; i++)
		{
			if (xmlStrEqual(name, BAD_CAST this->iPhasesNames[i].c_str()))
			{
				parameters.SetParameter(PARAM_SERIALIZATION_ELEMENT_NAME, this->iPhasesNames[i]);
				rc = this->iPhasesParameters[i].Deserialize(parameters, aReader);
				break;
			}
		}

		if (rc != ConstRC::Ok)
		{
			break;
		}
	}
	if (NULL != name) xmlFree(name);
	if (rc == ConstRC::GeneralError || rc == ConstRC::NotEnoughMemory)
        {
                return rc;
        }

	return ConstRC::Ok;
}

// print parameters
void AFactorAlgParameters::Print() const {
        int i;

        fprintf(stdout, "\n");
	fprintf(stdout, "%s:\n\n", XML_SERIAL_NFS_PARAM_ELEMENT_NAME);
        for (i = 0; i < iPhasesCount; i++) {
                fprintf(stdout, "\t%s:\n\n", (iPhasesNames+i)->c_str());
                (iPhasesParameters+i)->Print(stdout, "\t\t");
                fprintf(stdout, "\n");
        }

        return;
}

