#ifndef __ABSTRACT_FACTOR_ALG_PARAMETERS_H__
#define __ABSTRACT_FACTOR_ALG_PARAMETERS_H__

#include "base_parameters.h"
#include "common_utils.h"
#include "enums_common.h"
#include "enums.h"
#include "xml_helper.h"

///Abstract class for parameters used in factorization algorithm
class AFactorAlgParameters : public virtual MSerializable
{
protected:
	///Number of phases of the algorithm including one for 'global' parameters
	int iPhasesCount;
	///Field of phase's parameter classes including one for 'global' parameters 
	CBaseParameters * iPhasesParameters;
	///Field of names of phases
	std::string * iPhasesNames;

public:

	AFactorAlgParameters();
	AFactorAlgParameters(const AFactorAlgParameters& aOperand);
	virtual ~AFactorAlgParameters();	

	virtual int ParseInputParameters(int argc, char *argv[]) = 0;

	virtual int AddPhaseNonSerializableParam(const std::string& aName, int aPhase = 0);

	// Phase Parameters
	virtual int AddPhaseParameter(const std::string& aName, const std::string& aValue, int aPhase = 0);
	virtual int AddParameterForOthers(const std::string& aName, const std::string& aValue, int aFromPhase = 0);
	virtual int SetPhaseParameter(const std::string& aName, const std::string& aValue, int aPhase = 0);
	virtual int SetParameterForOthers(const std::string& aName, const std::string& aValue, int aFromPhase = 0);
	
	virtual bool ContainsPhaseParameter(const std::string& aName, int aPhase = 0) const;

	virtual int GetStringPhaseParameter(const std::string& aName, std::string& aResult, int aPhase) const;
	virtual int GetStringPhaseParameter(const std::string& aName, std::string aDefaultValue, std::string& aResult, int aPhase) const;

	virtual int GetIntegerPhaseParameter(const std::string& aName, int& aResult, int aPhase) const;
	virtual int GetIntegerPhaseParameter(const std::string& aName, int aDefaultValue, int& aResult, int aPhase) const;

	virtual int GetLongPhaseParameter(const std::string& aName, long& aResult, int aPhase) const;
	virtual int GetLongPhaseParameter(const std::string& aName, long aDefaultValue, long& aResult, int aPhase) const;

	virtual int GetDoublePhaseParameter(const std::string& aName, double& aResult, int aPhase) const;
	virtual int GetDoublePhaseParameter(const std::string& aName, double aDefaultValue, double& aResult, int aPhase) const;

	virtual int GetBoolPhaseParameter(const std::string& aName, bool& aResult, int aPhase) const;
	virtual int GetBoolPhaseParameter(const std::string& aName, bool aDefaultValue, bool& aResult, int aPhase) const;

	virtual int GetPolynomialPhaseParameter(const std::string& aName, Polynomial *& aResult, int aPhase) const;
	virtual int GetPolynomialPhaseParameter(const std::string& aName, const Polynomial * aDefaultValue, Polynomial *& aResult, int aPhase) const;

	virtual int GetMpzPhaseParameter(const std::string& aName, mpz_t aResult, int aPhase) const;
	virtual int GetMpzPhaseParameter(const std::string& aName, mpz_t aDefaultValue, mpz_t aResult, int aPhase) const;

	virtual int CreateFullFileName(std::string& aResult, const std::string& aParamFileName, 
		const std::string& aParamDir, const std::string& aParamCompLevel, 
		const std::string& aDir, const std::string& aFileName, const std::string& aIdentifier, 
		int aPhase = 0, const std::string& aExtension = XML_FILE_EXTENSION) const;

	// De/Serialization methods
	int Serialize(const CBaseParameters& aParam) const;
	int Serialize(const CBaseParameters& aParam, xmlTextWriterPtr& aWriter) const;

	int Deserialize(const CBaseParameters& aParam);
	int Deserialize(const CBaseParameters& aParam, xmlTextReaderPtr& aReader);

	// print parameters
	void Print() const;

};


#endif // __ABSTRACT_FACTOR_ALG_PARAMETERS_H__
