#include "abstract_linear_phase.h"
#include "const_linear_phase.h"
#include "const_parameter_names.h"
#include "lanczos_class.h"
#include "log.h"

using namespace std;

ALinearPhase::ALinearPhase()
{
	this->iMatrixResult = NULL;

	int rc;
	if ((rc = pthread_mutex_init(&(this->iRankCalculationModeMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"ALinearPhase","ALinearPhase",rc,0,"Cannot initialize mutex.");
	}
	if ((rc = pthread_mutex_init(&(this->iDensityModeMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"ALinearPhase","ALinearPhase",rc,0,"Cannot initialize mutex.");
	}
	if ((rc = pthread_mutex_init(&(this->iTimeMessageModeMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"ALinearPhase","ALinearPhase",rc,0,"Cannot initialize mutex.");
	}
	if ((rc = pthread_mutex_init(&(this->iCheckFinalResultMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"ALinearPhase","ALinearPhase",rc,0,"Cannot initialize mutex.");
	}
	if ((rc = pthread_mutex_init(&(this->iMatrixResultFullFileNameMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"ALinearPhase","ALinearPhase",rc,0,"Cannot initialize mutex.");
	}
	if ((rc = pthread_mutex_init(&(this->iMatrixResultMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"ALinearPhase","ALinearPhase",rc,0,"Cannot initialize mutex.");
	}
}
int ALinearPhase::DisposeMutexes()
{
	int rc;

	if ((rc = pthread_mutex_destroy(&(this->iRankCalculationModeMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"ALinearPhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}
	if ((rc = pthread_mutex_destroy(&(this->iDensityModeMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"ALinearPhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}
	if ((rc = pthread_mutex_destroy(&(this->iTimeMessageModeMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"ALinearPhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}
	if ((rc = pthread_mutex_destroy(&(this->iCheckFinalResultMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"ALinearPhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}
	if ((rc = pthread_mutex_destroy(&(this->iMatrixResultFullFileNameMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"ALinearPhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}
	if ((rc = pthread_mutex_destroy(&(this->iMatrixResultMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"ALinearPhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}

	return APhase::DisposeMutexes();
}
int ALinearPhase::InitParameters()
{
	int rc = APhase::InitParameters();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	AFactorAlgParameters * param = Get_Parameters();
	if (param == NULL)
	{
		return ConstRC::GeneralError;
	}

	bool auxBool;
	// ALinearPhase members
	param->GetBoolPhaseParameter(ConstPN::RANK_CALC_MODE, ConstLPhase::RANK_CALC_MODE, 
		auxBool, Get_PhaseNbr());
	Set_RankCalculationMode(auxBool);
	param->GetBoolPhaseParameter(ConstPN::CHECK_RESULT, ConstLPhase::CHECK_FINAL_RESULT, 
		auxBool, Get_PhaseNbr());
	Set_CheckFinalResult(auxBool);
	param->GetBoolPhaseParameter(ConstPN::DENSITY_MODE, ConstLPhase::DENSITY_MODE, 
		auxBool, Get_PhaseNbr());
	Set_DensityMode(auxBool);
	param->GetBoolPhaseParameter(ConstPN::TIME_MESSAGE_MODE, ConstLPhase::TIME_MESSAGE_MODE, 
		auxBool, Get_PhaseNbr());
	Set_TimeMessageMode(auxBool);

	// Matrix result full filename
	string auxStr;
	if (param->ContainsPhaseParameter(ConstPN::MATRIX_RESULT_FULL_FILENAME, Get_PhaseNbr()))
	{
		param->GetStringPhaseParameter(ConstPN::MATRIX_RESULT_FULL_FILENAME, auxStr, Get_PhaseNbr());
		MSerializable::AdaptPathSeparator(auxStr);
	}
	else
	{
		param->CreateFullFileName(auxStr, ConstPN::MATRIX_RESULT_FILENAME, 
			ConstPN::MATRIX_RESULT_DIRECTORY, ConstPN::MATRIX_RESULT_COMPRESSION, Get_SerialMainDirectory(),
			ConstLPhase::MATRIX_RESULT_FILENAME, Get_SerialIdentifier(), Get_PhaseNbr());
	}
	Set_MatrixResultFullFileName(auxStr);
	// Relation matrix full filename
	if (param->ContainsPhaseParameter(ConstPN::RELATION_MATRIX_FULL_FILENAME, Get_PhaseNbr()))
	{
		param->GetStringPhaseParameter(ConstPN::RELATION_MATRIX_FULL_FILENAME, this->iRelationMatrixFullFileName, Get_PhaseNbr());
		MSerializable::AdaptPathSeparator(auxStr);
	}
	else
	{
		param->CreateFullFileName(this->iRelationMatrixFullFileName, ConstPN::RELATION_MATRIX_FILENAME, 
			ConstPN::RELATION_MATRIX_DIRECTORY, ConstPN::RELATION_MATRIX_COMPRESSION, Get_SerialMainDirectory(),
			ConstLPhase::RELATION_MATRIX_FILENAME, Get_SerialIdentifier(), Get_PhaseNbr());
	}
	
	int matrixType;
	param->GetIntegerPhaseParameter(ConstPN::RELATION_MATRIX_TYPE, 
		ConstLPhase::RELATION_MATRIX_TYPE, matrixType, Get_PhaseNbr());
	this->iMatrixType = (species_of_matrices)matrixType;

	// setup matrix solver
	int matrixSolverType;
	param->GetIntegerPhaseParameter(ConstPN::MATRIX_SOLVER_TYPE, ConstLPhase::MATRIX_SOLVER_TYPE, 
		matrixSolverType, Get_PhaseNbr());
	this->iMatrixSolverType = (matrix_solver_type)matrixSolverType;

	switch (this->iMatrixSolverType)
	{
		case ELanczos:
		default:
			this->iMatrixSolver = new Lanczos();
			int matrixType;
			param->GetIntegerPhaseParameter(ConstPN::LANCZOS_AUX_MATRIX_TYPE, 
				ConstLPhase::LANCZOS_AUX_MATRIX_TYPE, matrixType, Get_PhaseNbr());
			((Lanczos*)this->iMatrixSolver)->SetAuxiliaryMatricesType((species_of_matrices)matrixType);
			break;
	}

	this->iMatrixSolver->Set_InfoMode(Get_InfoMode());
	this->iMatrixSolver->Set_AssertionMode(Get_AssertionMode());
	this->iMatrixSolver->Set_DensityMode(Get_DensityMode());
	this->iMatrixSolver->Set_TimeMessageMode(Get_TimeMessageMode());
	this->iMatrixSolver->Set_RankCalculationMode(Get_RankCalculationMode());
	this->iMatrixSolver->Set_CheckFinalResult(Get_CheckFinalResult());	
	this->iMatrixSolver->Set_MatrixType(this->iMatrixType);

	return ConstRC::Ok;
}

int ALinearPhase::InnerStateFinished() const
{
	return (int)ELPhaseFinished;
}
int ALinearPhase::CleanUp(AFactorAlgParameters *aParameters, int aPhaseNBR, cleaning_types aType, bool aError)
{
	if (aParameters == NULL)
	{
		CLog::LogError(WHEREARG,"ALinearPhase","CleanUp",0,0,"Parameters are NULL.");
		return ConstRC::BadArgument;
	}

	string identifier;
	string directory = "";
	string file;

	if (aParameters->GetStringPhaseParameter(ConstPN::SERIALIZATION_IDENTIFIER, identifier, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"ALinearPhase","CleanUp",0,0,"Didn't find serial identifier.");
		return ConstRC::GeneralError;
	}
	aParameters->GetStringPhaseParameter(ConstPN::SERIALIZATION_DIRECTORY, directory, (int)EMain);

	switch (aType)
	{
		case ECleanAllIfNotError:
			if (aError)
			{
				break;
			}
		case ECleanAll:
		case ECleanAllExcCF:
		case ECleanAllExcCFRF:
			// Matrix result full filename
			if (aParameters->ContainsPhaseParameter(ConstPN::MATRIX_RESULT_FULL_FILENAME, aPhaseNBR))
			{
				aParameters->GetStringPhaseParameter(ConstPN::MATRIX_RESULT_FULL_FILENAME, file, aPhaseNBR);
				MSerializable::AdaptPathSeparator(file);
			}
			else
			{
				aParameters->CreateFullFileName(file, ConstPN::MATRIX_RESULT_FILENAME, 
					ConstPN::MATRIX_RESULT_DIRECTORY, ConstPN::MATRIX_RESULT_COMPRESSION, directory,
					ConstLPhase::MATRIX_RESULT_FILENAME, identifier, aPhaseNBR);
			}
			remove(file.c_str());
			break;
		default:
			break;
	}

	return APhase::CleanUp(aParameters, aPhaseNBR, aType, aError);
}
// Getters and Setters
bool ALinearPhase::Get_RankCalculationMode() const
{
	bool ret = false;

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iRankCalculationModeMutex));
	ret = this->iRankCalculationMode;
	pthread_mutex_unlock(&(this->iRankCalculationModeMutex));

	return ret;
}
int  ALinearPhase::Set_RankCalculationMode(bool aRankCalculationMode)
{
	if (Running())
	{
		pthread_mutex_lock(&(this->iRankCalculationModeMutex));
		this->iRankCalculationMode = aRankCalculationMode;
		pthread_mutex_unlock(&(this->iRankCalculationModeMutex));

		return ConstRC::Ok;
	}

	return ConstRC::CannotBeDone;
}
bool ALinearPhase::Get_DensityMode() const
{
	bool ret = false;

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iDensityModeMutex));
	ret = this->iDensityMode;
	pthread_mutex_unlock(&(this->iDensityModeMutex));

	return ret;
}
int  ALinearPhase::Set_DensityMode(bool aDensityMode)
{
	if (Running())
	{
		pthread_mutex_lock(&(this->iDensityModeMutex));
		this->iDensityMode = aDensityMode;
		pthread_mutex_unlock(&(this->iDensityModeMutex));

		return ConstRC::Ok;
	}

	return ConstRC::CannotBeDone;
}
bool ALinearPhase::Get_TimeMessageMode() const
{
	bool ret = false;

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iTimeMessageModeMutex));
	ret = this->iTimeMessageMode;
	pthread_mutex_unlock(&(this->iTimeMessageModeMutex));

	return ret;
}
int  ALinearPhase::Set_TimeMessageMode(bool aTimeMessageMode)
{
	if (Running())
	{
		pthread_mutex_lock(&(this->iTimeMessageModeMutex));
		this->iTimeMessageMode = aTimeMessageMode;
		pthread_mutex_unlock(&(this->iTimeMessageModeMutex));

		return ConstRC::Ok;
	}

	return ConstRC::CannotBeDone;
}
bool ALinearPhase::Get_CheckFinalResult() const
{
	bool ret = false;

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iCheckFinalResultMutex));
	ret = this->iCheckFinalResult;
	pthread_mutex_unlock(&(this->iCheckFinalResultMutex));

	return ret;
}
int  ALinearPhase::Set_CheckFinalResult(bool aCheckFinalResult)
{
	if (Running())
	{
		pthread_mutex_lock(&(this->iCheckFinalResultMutex));
		this->iCheckFinalResult = aCheckFinalResult;
		pthread_mutex_unlock(&(this->iCheckFinalResultMutex));

		return ConstRC::Ok;
	}

	return ConstRC::CannotBeDone;
}

std::string ALinearPhase::Get_MatrixResultFullFileName() const
{
	string ret = "";

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iMatrixResultFullFileNameMutex));
	ret = this->iMatrixResultFullFileName;
	pthread_mutex_unlock(&(this->iMatrixResultFullFileNameMutex));

	return ret;
}
int			ALinearPhase::Set_MatrixResultFullFileName(std::string aMatrixResultFullFileName)
{
	if (Running())
	{
		pthread_mutex_lock(&(this->iMatrixResultFullFileNameMutex));
		this->iMatrixResultFullFileName = aMatrixResultFullFileName;
		pthread_mutex_unlock(&(this->iMatrixResultFullFileNameMutex));

		return ConstRC::Ok;
	}

	return ConstRC::CannotBeDone;
}
AbstractMatrix* ALinearPhase::Get_MatrixResult() const
{
	AbstractMatrix* ret = NULL;

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iMatrixResultMutex));
	ret = this->iMatrixResult;
	pthread_mutex_unlock(&(this->iMatrixResultMutex));

	return ret;
}
int             ALinearPhase::Set_MatrixResult(AbstractMatrix* aMatrixResult)
{
	if (Running())
	{
		pthread_mutex_lock(&(this->iMatrixResultMutex));
		if (this->iMatrixResult != NULL)
		{
			// TODO
			// this free sometimes causes double-free error (usually when program closes)
//			delete this->iMatrixResult;
			this->iMatrixResult = NULL;
		}
		this->iMatrixResult = aMatrixResult;
		pthread_mutex_unlock(&(this->iMatrixResultMutex));

		return ConstRC::Ok;
	}

	return ConstRC::CannotBeDone;
}

