#ifndef __ABSTRACT_LINEAR_PHASE_H__
#define __ABSTRACT_LINEAR_PHASE_H__

#include "abstract_phase.h"
#include "abstract_matrix_solver.h"


///Abstract class for linear phases
class ALinearPhase : public APhase
{
private:
	///Will ranks of A and B be calculated and displayed?
	bool iRankCalculationMode;
	mutable pthread_mutex_t iRankCalculationModeMutex;
	///Will density of A and B be calculated and displayed?
	bool iDensityMode;
	mutable pthread_mutex_t iDensityModeMutex;
	///Will the list of phases and time spent in them be displayed?
	bool iTimeMessageMode;
	mutable pthread_mutex_t iTimeMessageModeMutex;
	///Will finally B and result be checked to give zero matrix?
	bool iCheckFinalResult;
	mutable pthread_mutex_t iCheckFinalResultMutex;

	///Full filename with results of matrix in matrix form
	std::string iMatrixResultFullFileName;
	mutable pthread_mutex_t iMatrixResultFullFileNameMutex;
	///Results in form of matrix
	AbstractMatrix* iMatrixResult;
	mutable pthread_mutex_t iMatrixResultMutex;

protected:
	///Full filename with relation matrix
	std::string iRelationMatrixFullFileName;
	///Type of matrix to solve
	species_of_matrices iMatrixType;
	///Type of algorithm used for solving matrix
	matrix_solver_type iMatrixSolverType;
	///Algorithm used for solving matrix
	AMatrixSolver* iMatrixSolver;

protected:
	virtual int SolveSystem() = 0; 

	int InitParameters();
	int DisposeMutexes();

	///Return number of finished state
	int InnerStateFinished() const;

public:
	///Constructor  - initialization of mutexes
	ALinearPhase();

	///Delete all used files according to enum cleaning_types
	int CleanUp(AFactorAlgParameters* aParameters, int aPhaseNBR, cleaning_types aType, bool aError);


	// Getters and Setters
public:
	bool Get_RankCalculationMode() const;
	int  Set_RankCalculationMode(bool aRankCalculationMode);
	bool Get_DensityMode() const;
	int  Set_DensityMode(bool aDensityMode);
	bool Get_TimeMessageMode() const;
	int  Set_TimeMessageMode(bool aTimeMessageMode);
	bool Get_CheckFinalResult() const;
	int  Set_CheckFinalResult(bool aCheckFinalResult);

	std::string Get_MatrixResultFullFileName() const;
	int			Set_MatrixResultFullFileName(std::string aMatrixResultFullFileName);

protected:
	AbstractMatrix* Get_MatrixResult() const;
	int             Set_MatrixResult(AbstractMatrix* aMatrixResult);

};

#endif // __ABSTRACT_LINEAR_PHASE_H__