#include "abstract_phase.h"
#include "const_parameter_names.h"
#include "const_filenames.h"
#include "log.h"

using namespace std;

APhase::APhase()
{
	this->iTimes = NULL;
	this->iStats = NULL;
	this->iParameters = NULL;
	this->iPhaseNumber = -1;
	this->iFunctorsInit = false;

	this->iRSerialParameters = NULL;
	this->iRSerialInit = false;
	this->iRSerialRunning = false;
	this->iRSerialRun = false;

	// initialize accepted return codes
	iAcceptedReturnCodes.push_back(ConstRC::Ok);

	int rc;
	if ((rc = pthread_mutex_init(&(this->iAssertionModeMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","APhase",rc,0,"Cannot initialize mutex.");
	}
	if ((rc = pthread_mutex_init(&(this->iAutoSerializableMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","APhase",rc,0,"Cannot initialize mutex.");
	}
	if ((rc = pthread_mutex_init(&(this->iInfoModeMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","APhase",rc,0,"Cannot initialize mutex.");
	}
	if ((rc = pthread_mutex_init(&(this->iInnerStateMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","APhase",rc,0,"Cannot initialize mutex.");
	}
	if ((rc = pthread_mutex_init(&(this->iPhaseNumberMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","APhase",rc,0,"Cannot initialize mutex.");
	}
	if ((rc = pthread_mutex_init(&(this->iSerialIdentifierMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","APhase",rc,0,"Cannot initialize mutex.");
	}
	if ((rc = pthread_mutex_init(&(this->iSerialMainDirectoryMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","APhase",rc,0,"Cannot initialize mutex.");
	}
	if ((rc = pthread_mutex_init(&(this->iFunctorsInitMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","APhase",rc,0,"Cannot initialize mutex.");
	}
	if ((rc = pthread_mutex_init(&(this->iParametersFullFileNameMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","APhase",rc,0,"Cannot initialize mutex.");
	}
	if ((rc = pthread_mutex_init(&(this->iSerialPhaseFullFileNameMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","APhase",rc,0,"Cannot initialize mutex.");
	}
}
int APhase::DisposeFunctorField()
{
	if (Get_FunctorsInit())
	{
		int max = (int)this->iPhaseFunctors.size();
		for (int i = 0; i < max; i++)
		{
			if (this->iPhaseFunctors[i] != NULL)
			{
				delete this->iPhaseFunctors[i];
				this->iPhaseFunctors[i] = NULL;
			}
		}
		Set_FunctorsInit(false);
	}

	return ConstRC::Ok;
}
int APhase::DisposeMutexes()
{
	int rc;
	if ((rc = pthread_mutex_destroy(&(this->iAssertionModeMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}
	if ((rc = pthread_mutex_destroy(&(this->iAutoSerializableMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}
	if ((rc = pthread_mutex_destroy(&(this->iInfoModeMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}
	if ((rc = pthread_mutex_destroy(&(this->iInnerStateMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}
	if ((rc = pthread_mutex_destroy(&(this->iPhaseNumberMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}
	if ((rc = pthread_mutex_destroy(&(this->iSerialIdentifierMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}
	if ((rc = pthread_mutex_destroy(&(this->iSerialMainDirectoryMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}
	if ((rc = pthread_mutex_destroy(&(this->iFunctorsInitMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}
	if ((rc = pthread_mutex_destroy(&(this->iParametersFullFileNameMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}
	if ((rc = pthread_mutex_destroy(&(this->iSerialPhaseFullFileNameMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","DisposeMutexes",rc,0,"Cannot destroy mutex.");
	}

	return ConstRC::Ok;
}

void APhase::DisposeAcceptedReturnCodes() {
	iAcceptedReturnCodes.clear();

	return;
}

int APhase::Reset()
{
	int rc = ConstRC::Ok;

	if (this->iRSerialInit)
	{
		rc = EndRunningSerialization();
	}

	return rc;
}
int APhase::SetStarted()
{
	pthread_mutex_lock(&(this->iInnerStateMutex));

	int rc = ConstRC::AlreadyRunning;
	if (this->iInnerState == InnerStateNotStarted() || 
		this->iInnerState == InnerStateFinished())
	{
		this->iInnerState = InnerStateNotStarted();
		this->iInnerState++;
		rc = ConstRC::Ok;
	}

	pthread_mutex_unlock(&(this->iInnerStateMutex));
	return rc;
}
int APhase::InnerStateNotStarted() const
{
	return 0;
}
bool APhase::Running() const
{
	return Get_InnerState() != InnerStateNotStarted();
}
int APhase::CleanUp(AFactorAlgParameters *aParameters, int aPhaseNBR, cleaning_types aType, bool aError)
{
	if (aParameters == NULL)
	{
		CLog::LogError(WHEREARG,"APhase","CleanUp",0,0,"Parameters are NULL.");
		return ConstRC::BadArgument;
	}

	string identifier;
	string directory = "";
	string file;

	if (aParameters->GetStringPhaseParameter(ConstPN::SERIALIZATION_IDENTIFIER, identifier, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","CleanUp",0,0,"Didn't find serial identifier.");
		return ConstRC::GeneralError;
	}
	aParameters->GetStringPhaseParameter(ConstPN::SERIALIZATION_DIRECTORY, directory, (int)EMain);

	switch (aType)
	{
		case ECleanAllIfNotError:
			if (aError)
			{
				break;
			}
		case ECleanAll:
		case ECleanAllExcCF:
		case ECleanAllExcCFRF:
			// Serial Phase full filename
			if (aParameters->ContainsPhaseParameter(ConstPN::PHASE_FULL_FILENAME, aPhaseNBR))
			{
				aParameters->GetStringPhaseParameter(ConstPN::PHASE_FULL_FILENAME, file, aPhaseNBR);
				MSerializable::AdaptPathSeparator(file);
			}
			else
			{
				aParameters->CreateFullFileName(file, ConstPN::PHASE_FILENAME, 
					ConstPN::PHASE_DIRECTORY, ConstPN::PHASE_COMPRESSION, directory,
					MSerializable::Get_DefaultFileName(), identifier, aPhaseNBR);
			}
			remove(file.c_str());
			break;
		default:
			break;
	}

	return ConstRC::Ok;
}
/*!
	We setup APhase's members.
	We suppose that we already knew iPhaseNumber.
*/
int APhase::InitParameters()
{
	if (this->iParameters == NULL)
	{
		CLog::LogError(WHEREARG,"APhase","InitParameters",0,0,"Parameters aren't set.");
		return ConstRC::GeneralError;
	}

	string auxStr;

	// we set current running phase
	auxStr = Utils::ConvertToString(Get_PhaseNbr());
	this->iParameters->SetPhaseParameter(ConstPN::CURRENT_PHASE, auxStr, (int)EMain);

	// we need identifier for serialization
	if (this->iParameters->GetStringPhaseParameter(ConstPN::SERIALIZATION_IDENTIFIER, auxStr, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","InitParameters",0,0,"Didn't find serial identifier.");
		return ConstRC::GeneralError;
	}
	Set_SerialIdentifier(auxStr);

	// we may be want to serialize to another directory
	auxStr = "";
	this->iParameters->GetStringPhaseParameter(ConstPN::SERIALIZATION_DIRECTORY, auxStr, (int)EMain);
	Set_SerialMainDirectory(auxStr);

	// APhase members - we prefer phase settings against global settings
	bool auxBool;
	this->iParameters->GetBoolPhaseParameter(ConstPN::INFO_MODE, false, auxBool, (int)EMain);
	this->iParameters->GetBoolPhaseParameter(ConstPN::INFO_MODE, auxBool, auxBool, Get_PhaseNbr());
	Set_InfoMode(auxBool);

	// log setting
	this->iParameters->GetBoolPhaseParameter(ConstPN::INFO_MODE_TECHNICAL, false, auxBool, (int)EMain);
	this->iParameters->GetBoolPhaseParameter(ConstPN::INFO_MODE_TECHNICAL, auxBool, auxBool, Get_PhaseNbr());
	CLog::Set_TechnicalMode(auxBool);

	// log setting
	this->iParameters->GetBoolPhaseParameter(ConstPN::INFO_MODE_USE_FILE, false, auxBool, (int)EMain);
	this->iParameters->GetBoolPhaseParameter(ConstPN::INFO_MODE_USE_FILE, auxBool, auxBool, Get_PhaseNbr());
	if (auxBool)
	{
		CLog::UseFile(Get_SerialIdentifier(), Get_SerialMainDirectory());
	}
	else
	{
		CLog::UseConsole();
	}

	// log setting
	int auxInt;
	this->iParameters->GetIntegerPhaseParameter(ConstPN::INFO_MODE_PRIORITY, CLog::DEFAULT_MESSAGE_PRIORITY, auxInt, (int)EMain);
	this->iParameters->GetIntegerPhaseParameter(ConstPN::INFO_MODE_PRIORITY, auxInt, auxInt, Get_PhaseNbr());
	CLog::Set_Priority(auxInt);

	// assertion mode
	this->iParameters->GetBoolPhaseParameter(ConstPN::ASSERTION_MODE, false, auxBool, (int)EMain);
	this->iParameters->GetBoolPhaseParameter(ConstPN::ASSERTION_MODE, auxBool, auxBool, Get_PhaseNbr());
	Set_AssertionMode(auxBool);

	// Parameters full filename
	if (this->iParameters->ContainsPhaseParameter(ConstPN::CONTROL_FILE_FULL_FILENAME, (int)EMain))
	{
		this->iParameters->GetStringPhaseParameter(ConstPN::CONTROL_FILE_FULL_FILENAME, auxStr, (int)EMain);
		MSerializable::AdaptPathSeparator(auxStr);
	}
	else
	{
		this->iParameters->CreateFullFileName(auxStr, ConstPN::CONTROL_FILE_FILENAME, 
			ConstPN::CONTROL_FILE_DIRECTORY, ConstPN::CONTROL_FILE_COMPRESSION, Get_SerialMainDirectory(),
			ConstFN::CONTROL_FILE, Get_SerialIdentifier(), (int)EMain);
	}
	Set_ParametersFullFileName(auxStr);

	// Serial Phase full filename
	if (this->iParameters->ContainsPhaseParameter(ConstPN::PHASE_FULL_FILENAME, Get_PhaseNbr()))
	{
		this->iParameters->GetStringPhaseParameter(ConstPN::PHASE_FULL_FILENAME, auxStr, Get_PhaseNbr());
		MSerializable::AdaptPathSeparator(auxStr);
	}
	else
	{
		this->iParameters->CreateFullFileName(auxStr, ConstPN::PHASE_FILENAME, 
			ConstPN::PHASE_DIRECTORY, ConstPN::PHASE_COMPRESSION, Get_SerialMainDirectory(),
			MSerializable::Get_DefaultFileName(), Get_SerialIdentifier(), Get_PhaseNbr());
	}
	Set_SerialPhaseFullFileName(auxStr);

	// prepare iPhaseFunctors 
	FillFunctorField();

	return ConstRC::Ok;
}

int APhase::RunPhase(AFactorAlgParameters *aParameters)
{
	if (aParameters == NULL)
	{
		CLog::LogError(WHEREARG,"APhase","RunPhase",0,0,"Parameters are NULL.");
		return ConstRC::BadArgument;
	}

	// one instance - one run
	if (SetStarted() != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APhase","RunPhase",0,0,"Phase is already running.");
		return ConstRC::GeneralError;
	}

	// prepare for new run
	Reset();
	int rc = Set_Parameters(aParameters);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	// + deserialization
	rc = InitParameters();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	// now we go thought all functions which correspond to the inner states of the phase
	// starting from current state
	int max = (int)this->iPhaseFunctors.size();
	for (int i = Get_InnerState(); i < max; i++)
	{
		// some of them could be NULL - may be some auxiliary state
		if (this->iPhaseFunctors[i] != NULL)
		{
			// make function call
			rc = this->iPhaseFunctors[i]->Call();
			if (!CheckReturnCode(rc))
			{
				// error
				Reset();
				return rc;
			}
		}
	}

	return ConstRC::Ok;
}

void * APhase::RunningSerialization(void * aParam)
{
	if (aParam == NULL)
	{
		CLog::LogError(WHEREARG,"APhase","RunningSerialization",0,0,"Phase is NULL.");
		pthread_exit(NULL);
	}

	APhase * phase = (APhase*)aParam;
	if (phase->iRSerialParameters == NULL)
	{
		CLog::LogError(WHEREARG,"APhase","RunningSerialization",0,0,"Serial Parameters are NULL.");
		pthread_exit(NULL);
	}

	int rc;

	phase->Set_RSerialRunning(true);

	pthread_mutex_lock(&(phase->iRSerialRunMutex));

	while (true)
	{
		pthread_cond_wait(&(phase->iRSerialRunCond), &(phase->iRSerialRunMutex));
		rc = phase->Serialize(*phase->iRSerialParameters);
		if (rc != ConstRC::Ok)
		{
			break;
		}
		if (phase->iRSerialRun == false)
		{
			break;
		}
	}

	pthread_mutex_unlock(&(phase->iRSerialRunMutex));
	phase->Set_RSerialRunning(false);
	pthread_exit(NULL);
	return NULL;
}
int APhase::StartRunningSerialization(const CBaseParameters &aParameters)
{
	int rc;
	if (this->iRSerialInit == false)
	{
		if ((rc = pthread_mutex_init(&(this->iRSerialRunningMutex), NULL)) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"APhase","StartRunningSerialization",rc,0,"Cannot initialize mutex.");
			return ConstRC::GeneralError;
		}
		if ((rc = pthread_mutex_init(&(this->iRSerialRunMutex), NULL)) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"APhase","StartRunningSerialization",rc,0,"Cannot initialize mutex.");
			return ConstRC::GeneralError;
		}
		if ((rc = pthread_cond_init(&(this->iRSerialRunCond), NULL)) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"APhase","StartRunningSerialization",rc,0,"Cannot initialize condition.");
			return ConstRC::GeneralError;
		}
		if ((rc = pthread_attr_init(&(this->iRSerialThreadAttr))) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"APhase","StartRunningSerialization",rc,0,"Cannot initialize attribut.");
			return ConstRC::GeneralError;
		}
		pthread_attr_setdetachstate(&(this->iRSerialThreadAttr), PTHREAD_CREATE_JOINABLE);

		this->iRSerialInit = true;
	}

	if (Get_RSerialRunning())
	{
		CLog::LogError(WHEREARG,"APhase","StartRunningSerialization",0,0,"RSerial is already running.");
		return ConstRC::GeneralError;
	}

	if (this->iRSerialParameters != NULL)
	{
		delete this->iRSerialParameters;
		this->iRSerialParameters = NULL;
	}
	this->iRSerialParameters = new CBaseParameters(aParameters);
	pthread_create(&(this->iRSerialThread), &(this->iRSerialThreadAttr), APhase::RunningSerialization, (void *)this);

	return ConstRC::Ok;
}
int APhase::RunRunningSerialization()
{
	if (this->iRSerialInit && Get_RSerialRunning())
	{
		pthread_mutex_lock(&(this->iRSerialRunMutex));
		// serialize
		this->iRSerialRun = true;
		int rc = PrepareRunningSerialization();
		if (rc != ConstRC::Ok)
		{
			// some error
			pthread_mutex_unlock(&(this->iRSerialRunMutex));
			return rc;
		}
		// signal to the serialization thread to wake up
		pthread_cond_signal(&(this->iRSerialRunCond));
		pthread_mutex_unlock(&(this->iRSerialRunMutex));

		return ConstRC::Ok;
	}
	else
	{
		return ConstRC::Impossible;
	}
}
int APhase::PrepareRunningSerialization()
{
	// nothing to do here
	return ConstRC::Ok;
}
int APhase::EndRunningSerialization()
{
	if (this->iRSerialInit && Get_RSerialRunning())
	{
		pthread_mutex_lock(&(this->iRSerialRunMutex));
		// no other serialization
		this->iRSerialRun = false;
		PrepareRunningSerialization();
		// signal to the serialization thread to wake up
		pthread_cond_signal(&(this->iRSerialRunCond));
		pthread_mutex_unlock(&(this->iRSerialRunMutex));

		// join the serialization thread - wait for the end
		pthread_join(this->iRSerialThread, NULL);

		if (this->iRSerialParameters != NULL)
		{
			delete this->iRSerialParameters;
			this->iRSerialParameters = NULL;
		}
	}

	if (this->iRSerialInit)
	{
		// destroy all used members
		int rc;
		if ((rc = pthread_mutex_destroy(&(this->iRSerialRunningMutex))) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"APhase","EndRunningSerialization",rc,0,"Cannot destroy mutex.");
		}
		if ((rc = pthread_mutex_destroy(&(this->iRSerialRunMutex))) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"APhase","EndRunningSerialization",rc,0,"Cannot destroy mutex.");
		}
		if ((rc = pthread_cond_destroy(&(this->iRSerialRunCond))) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"APhase","EndRunningSerialization",rc,0,"Cannot destroy condition.");
		}
		if ((rc = pthread_attr_destroy(&(this->iRSerialThreadAttr))) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"APhase","EndRunningSerialization",rc,0,"Cannot destroy attribut.");
		}

		if (this->iRSerialParameters != NULL)
		{
			delete this->iRSerialParameters;
			this->iRSerialParameters = NULL;
		}

		this->iRSerialInit = false;
	}

	return ConstRC::Ok;
}

void APhase::AcceptReturnCode(const int rc) {
	iAcceptedReturnCodes.push_back(rc);

	return;
}

// returns true if the code is accepted
bool APhase::CheckReturnCode(const int rc) {
	for (size_t i = 0; i < iAcceptedReturnCodes.size(); i++) {
		if (iAcceptedReturnCodes.at(i) == rc) return true;
	}

	return false;
}

// Getters and Setters
bool APhase::Get_InfoMode() const
{
	bool ret = false;

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iInfoModeMutex));
	ret = this->iInfoMode;
	pthread_mutex_unlock(&(this->iInfoModeMutex));

	return ret;
}
int  APhase::Set_InfoMode(bool aInfoMode)
{
	if (Running())
	{
		pthread_mutex_lock(&(this->iInfoModeMutex));
		this->iInfoMode = aInfoMode;
		pthread_mutex_unlock(&(this->iInfoModeMutex));

		return ConstRC::Ok;
	}

	return ConstRC::CannotBeDone;
}
bool APhase::Get_AssertionMode() const
{
	bool ret = false;

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iAssertionModeMutex));
	ret = this->iAssertionMode;
	pthread_mutex_unlock(&(this->iAssertionModeMutex));

	return ret;
}
int  APhase::Set_AssertionMode(bool aAssertionMode)
{
	if (Running())
	{
		pthread_mutex_lock(&(this->iAssertionModeMutex));
		this->iAssertionMode = aAssertionMode;
		pthread_mutex_unlock(&(this->iAssertionModeMutex));

		return ConstRC::Ok;
	}

	return ConstRC::CannotBeDone;
}
bool APhase::Get_AutoSerializable() const
{
	bool ret = false;

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iAutoSerializableMutex));
	ret = this->iAutoSerializable;
	pthread_mutex_unlock(&(this->iAutoSerializableMutex));

	return ret;
}
int  APhase::Set_AutoSerializable(bool aAutoSerializable)
{
	pthread_mutex_lock(&(this->iAutoSerializableMutex));
	this->iAutoSerializable = aAutoSerializable;
	pthread_mutex_unlock(&(this->iAutoSerializableMutex));

	return ConstRC::Ok;
}
string APhase::Get_SerialIdentifier() const
{
	string ret = "";

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iSerialIdentifierMutex));
	ret = this->iSerialIdentifier;
	pthread_mutex_unlock(&(this->iSerialIdentifierMutex));

	return ret;
}
int    APhase::Set_SerialIdentifier(string aSerialIdentifier)
{
	if (Running())
	{
		pthread_mutex_lock(&(this->iSerialIdentifierMutex));
		this->iSerialIdentifier = aSerialIdentifier;
		pthread_mutex_unlock(&(this->iSerialIdentifierMutex));

		return ConstRC::Ok;
	}

	return ConstRC::CannotBeDone;
}
string APhase::Get_SerialMainDirectory() const
{
	string ret = "";

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iSerialMainDirectoryMutex));
	ret = this->iSerialMainDirectory;
	pthread_mutex_unlock(&(this->iSerialMainDirectoryMutex));

	return ret;
}
int    APhase::Set_SerialMainDirectory(string aSerialMainDirectory)
{
	if (Running())
	{
		pthread_mutex_lock(&(this->iSerialMainDirectoryMutex));
		this->iSerialMainDirectory = aSerialMainDirectory;
		pthread_mutex_unlock(&(this->iSerialMainDirectoryMutex));

		return ConstRC::Ok;
	}

	return ConstRC::CannotBeDone;
}
string APhase::Get_SerialPhaseFullFileName() const
{
	string ret = "";

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iSerialPhaseFullFileNameMutex));
	ret = this->iSerialPhaseFullFileName;
	pthread_mutex_unlock(&(this->iSerialPhaseFullFileNameMutex));

	return ret;
}
int    APhase::Set_SerialPhaseFullFileName(string aSerialPhaseFullFileName)
{
	if (Running())
	{
		pthread_mutex_lock(&(this->iSerialPhaseFullFileNameMutex));
		this->iSerialPhaseFullFileName = aSerialPhaseFullFileName;
		pthread_mutex_unlock(&(this->iSerialPhaseFullFileNameMutex));

		return ConstRC::Ok;
	}

	return ConstRC::CannotBeDone;
}
string APhase::Get_ParametersFullFileName() const
{
	string ret = "";

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iParametersFullFileNameMutex));
	ret = this->iParametersFullFileName;
	pthread_mutex_unlock(&(this->iParametersFullFileNameMutex));

	return ret;
}
int    APhase::Set_ParametersFullFileName(string aParametersFullFileName)
{
	if (Running())
	{
		pthread_mutex_lock(&(this->iParametersFullFileNameMutex));
		this->iParametersFullFileName = aParametersFullFileName;
		pthread_mutex_unlock(&(this->iParametersFullFileNameMutex));

		return ConstRC::Ok;
	}

	return ConstRC::CannotBeDone;
}
int APhase::Get_PhaseNbr() const
{
	int ret = 0;

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iPhaseNumberMutex));
	ret = this->iPhaseNumber;
	pthread_mutex_unlock(&(this->iPhaseNumberMutex));

	return ret;
}
int APhase::Set_PhaseNbr(int aPhaseNumber)
{
	if (Running())
	{
		pthread_mutex_lock(&(this->iPhaseNumberMutex));
		this->iPhaseNumber = aPhaseNumber;
		pthread_mutex_unlock(&(this->iPhaseNumberMutex));

		return ConstRC::Ok;
	}

	return ConstRC::CannotBeDone;
}


int APhase::Get_InnerState() const
{
	int ret = 0;

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iInnerStateMutex));
	ret = this->iInnerState;
	pthread_mutex_unlock(&(this->iInnerStateMutex));

	return ret;
}
int APhase::Set_InnerState(int aInnerState)
{
	pthread_mutex_lock(&(this->iInnerStateMutex));
	this->iInnerState = aInnerState;
	pthread_mutex_unlock(&(this->iInnerStateMutex));

	return ConstRC::Ok;
}
bool APhase::Get_FunctorsInit() const
{
	bool ret = false;

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iFunctorsInitMutex));
	ret = this->iFunctorsInit;
	pthread_mutex_unlock(&(this->iFunctorsInitMutex));

	return ret;
}
int  APhase::Set_FunctorsInit(bool aFunctorsInit)
{
	pthread_mutex_lock(&(this->iFunctorsInitMutex));
	this->iFunctorsInit = aFunctorsInit;
	pthread_mutex_unlock(&(this->iFunctorsInitMutex));

	return ConstRC::Ok;
}


bool APhase::Get_RSerialRunning() const
{
	bool ret = false;

	// we ignore if the phase is running
	pthread_mutex_lock(&(this->iRSerialRunningMutex));
	ret = this->iRSerialRunning;
	pthread_mutex_unlock(&(this->iRSerialRunningMutex));

	return ret;
}
int  APhase::Set_RSerialRunning(bool aRunning)
{
	pthread_mutex_lock(&(this->iRSerialRunningMutex));
	this->iRSerialRunning = aRunning;
	pthread_mutex_unlock(&(this->iRSerialRunningMutex));

	return ConstRC::Ok;
}


bool APhase::Finished() const
{
	return Get_InnerState() == InnerStateFinished();
}
