#include "abstract_poly_selection_phase.h"
#include "integer_class.h"
#include <math.h>
#include "const_poly_selection_phase.h"
#include "const_parameter_names.h"
#include "log.h"


//void APolyPhase::Get_N(mpz_t aResult) const
//{
//	mpz_set(aResult, this->iN);
//}
//void APolyPhase::Get_Root(mpz_t aResult) const
//{
//	mpz_set(aResult, this->iRoot);
//}


APolyPhase::APolyPhase()
{
	this->iRhoTaylor = NULL;
	this->iSin = NULL;
	this->iCos = NULL;

	this->iSubintervalsCount = 0;
	this->iSelfInitializingDelay = 0;

	mpf_init(this->iRhoMax);
	mpf_init(this->iSubintervalLength);

	mpf_init(this->iAuxMpfCo);
	mpf_init(this->iAuxMpfPow);
	mpf_init(this->iAuxMpfC);
	mpf_init(this->iAuxMpfS);

	mpz_init_set_si(this->iRoot, 0);
	mpz_init_set_si(this->iN, 0);

	mpf_init_set_str(this->iPiHalf, "1.5707963267948966192313216916397514420985846996876", 10);
	mpf_init_set_str(this->iLogTwo, "0.69314718055994530941723212145817656807550013436026", 10);
}
int APolyPhase::Reset()
{
	mpz_set_si(this->iRoot, 0);
	mpz_set_si(this->iN, 0);
	this->iCandidatesCount = 0;

	return APhase::Reset();
}
int APolyPhase::DisposeGMP()
{
	long i, j;
	if (this->iRhoTaylor != NULL) 
	{
		for (i = 0; i < ConstPSPhase::MAXIMAL_RHO_VALUE - 1; i++)
		{
			for (j = 0; j < ConstPSPhase::RHO_EXPANSION_LENGTH; j++)
			{
				mpf_clear(this->iRhoTaylor[i][j]);
			}
			free(this->iRhoTaylor[i]);
		}
		free(this->iRhoTaylor);
		this->iRhoTaylor = NULL;
	}

	if (this->iSin != NULL) 
	{
		for (i = 0; i < this->iSubintervalsCount; i++)
		{
			mpf_clear(this->iSin[i]);
			mpf_clear(this->iCos[i]);
		}
		free(this->iSin);
		free(this->iCos);
		this->iSin = NULL;
		this->iCos = NULL;
	}
	mpf_clear(this->iSubintervalLength);
	mpf_clear(this->iPiHalf);
	mpf_clear(this->iLogTwo);
	mpf_clear(this->iRhoMax); 

	mpf_clear(this->iAuxMpfCo);
	mpf_clear(this->iAuxMpfPow);
	mpf_clear(this->iAuxMpfC);
	mpf_clear(this->iAuxMpfS);

	mpz_clear(this->iN);
	mpz_clear(this->iRoot);

	return ConstRC::Ok;
}
int APolyPhase::DisposeMutexes()
{
	return APhase::DisposeMutexes();
}
int APolyPhase::InnerStateFinished() const
{
	return (int)EPSPhaseFinished;
}
int APolyPhase::CleanUp(AFactorAlgParameters *aParameters, int aPhaseNBR, cleaning_types aType, bool aError)
{
	if (aParameters == NULL)
	{
		CLog::LogError(WHEREARG,"APolyPhase","CleanUp",0,0,"Parameters are NULL.");
		return ConstRC::BadArgument;
	}

	return APhase::CleanUp(aParameters, aPhaseNBR, aType, aError);
}
int APolyPhase::InitParameters()
{
	int rc = APhase::InitParameters();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	AFactorAlgParameters * param = Get_Parameters();

	int phaseNBR = Get_PhaseNbr();

	long auxLong;
	param->GetLongPhaseParameter(ConstPN::MPF_T_PRECISION, ConstPSPhase::MPF_T_PRECISION, auxLong, phaseNBR);
	Set_Precision(auxLong);
	param->GetLongPhaseParameter(ConstPN::HALF_CIRCLE_SUBINTERVALS_COUNT, 
		ConstPSPhase::HALF_CIRCLE_SUBINTERVALS_COUNT, auxLong, phaseNBR);
	Set_SubintervalsCount(auxLong);
	param->GetLongPhaseParameter(ConstPN::SELF_INITIALIZING_DELAY, 
		ConstPSPhase::SELF_INITIALIZING_DELAY, this->iSelfInitializingDelay, phaseNBR);

	param->GetLongPhaseParameter(ConstPN::CANDIDATE_POLY_COUNT, ConstPSPhase::CANDIDATES_COUNT, this->iCandidatesCount, phaseNBR);
	param->GetIntegerPhaseParameter(ConstPN::CONTENT_TRIES, ConstPSPhase::CONTENT_TRIES, this->iContentTries, phaseNBR);
	param->GetLongPhaseParameter(ConstPN::CONTENT_BOUND, ConstPSPhase::CONTENT_BOUND, this->iContentBound, phaseNBR);
	param->GetIntegerPhaseParameter(ConstPN::ALPHA_PRIME_BOUND, ConstPSPhase::ALPHA_PRIME_BOUND, this->iAlphaPrimeBound, phaseNBR);
	param->GetIntegerPhaseParameter(ConstPN::ALPHA_SMALLER_PRIME_BOUND, ConstPSPhase::ALPHA_SMALLER_PRIME_BOUND, this->iAlphaSmallerPrimeBound, phaseNBR);
	param->GetDoublePhaseParameter(ConstPN::ALPHA_ABORT, ConstPSPhase::ALPHA_ABORT, this->iAlphaAbort, phaseNBR);
	param->GetDoublePhaseParameter(ConstPN::ALPHA_BOUND, ConstPSPhase::ALPHA_BOUND, this->iAlphaBound, phaseNBR);

	rc = param->GetMpzPhaseParameter(ConstPN::NUMBER_N, this->iN, (int)EMain);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"APolyPhase","InitParameters",0,rc,"Didn't find N.");
		return rc;
	}

	return ConstRC::Ok;
}

///Computes content p of the homogenized polynomial in area iContentBound x iContentBound
int APolyPhase::ContPHomPoly(mpf_t aResult, Polynomial *aPoly, unsigned int aPrime) const
{
	if (aPoly == NULL)
	{
		CLog::LogError(WHEREARG,"APolyPhase","ContPHomPoly",0,0,"Polynomial is NULL.");
		return ConstRC::GeneralError;
	}

	int d = aPoly->GetDegree();
	int j;
	long a,b;
	long c,e;
	mpz_t r,s,x;
	mpz_init(x);
	mpz_init(r);
	mpz_init(s);
	mpf_set_ui(aResult,0);
	for (int i = 0; i < this->iContentTries; i++)
	{
		// find coprime a and b
		do 
		{
			a = CIntegerClass::RandomNumber(this->iContentBound) + 1;
			b = CIntegerClass::RandomNumber(this->iContentBound) + 1;
		} 
		while (CIntegerClass::ExtendedEuclid(&c, &e, a, b) != 1);
		// x = a^d
		mpz_ui_pow_ui(x, a, d);
		mpz_set_ui(r, 0);
		for(j = d; j >= 0; j--)
		{
			mpz_mul(s, x, *aPoly->Get2(j));
			mpz_add(r, r, s);
			if (j)
			{
				mpz_divexact_ui(x, x, a);
				mpz_mul_ui(x, x, b);
			}
		}

		while (mpz_divisible_ui_p(r, aPrime))
		{
			mpf_add_ui(aResult, aResult, 1);
			mpz_fdiv_q_ui(r, r, aPrime);
		}
	}
	mpz_clear(x);
	mpz_clear(s);
	mpz_clear(r);
	mpf_div_ui(aResult, aResult, (unsigned long)this->iContentTries);

	return ConstRC::Ok;
}

///Computes content p of the polynomial in range 0 ... iContentBound
int APolyPhase::ContPPoly(mpf_t aResult, Polynomial *aPoly, unsigned int aPrime) const
{
	if (aPoly == NULL)
	{
		CLog::LogError(WHEREARG,"APolyPhase","ContPPoly",0,0,"Polynomial is NULL.");
		return ConstRC::GeneralError;
	}

	int d = aPoly->GetDegree();
	int j;
	long a;
	mpz_t r;
	mpz_init(r);
	mpf_set_ui(aResult,0);
	for (int i = 0; i < this->iContentTries; i++)
	{
		a = CIntegerClass::RandomNumber(this->iContentBound) + 1;
		
		mpz_set_ui(r,0);
		for(j = d; j > 0; j--)
		{
			mpz_add(r,r,*aPoly->Get2(j));
			mpz_mul_si(r,r,a);
		}
		mpz_add(r,r,*aPoly->Get2(j));

		while (mpz_divisible_ui_p(r,aPrime))
		{
			mpf_add_ui(aResult,aResult,1);
			mpz_fdiv_q_ui(r,r,aPrime);
		}
	}
	mpz_clear(r);
	mpf_div_ui(aResult, aResult, (unsigned long)this->iContentTries);

	return ConstRC::Ok;
}

///Computes the alpha of the homogenized polynomial aPoly
int APolyPhase::AlphaHomPoly(mpf_t aResult, long * aRoots, Polynomial *aPoly) const
{
	if (aPoly == NULL || aRoots == NULL)
	{
		CLog::LogError(WHEREARG,"APolyPhase","AlphaHomPoly",0,0,"Polynomial or Roots is NULL.");
		return ConstRC::GeneralError;
	}

	mpz_t p;
	mpz_init(p);
	int d = aPoly->GetDegree();
	long r, pp;

	mpf_set_ui(aResult, 0);
	// for (p = 2; p <= iAlphaPrimeBound; p = nextprime(p))
	for (mpz_set_ui(p, 2); mpz_cmp_si(p, this->iAlphaPrimeBound) <= 0; mpz_nextprime(p, p))
	{
		pp = mpz_get_ui(p);
		mpf_set_ui(this->iAuxMpfCo, pp - 1);
		mpf_ui_div(this->iAuxMpfS, 1, this->iAuxMpfCo);
		ContPHomPoly(this->iAuxMpfCo, aPoly, pp);
		mpf_sub(this->iAuxMpfS, this->iAuxMpfS, this->iAuxMpfCo);
		mpf_set_d(this->iAuxMpfCo, log((double)pp));
		mpf_mul(this->iAuxMpfS, this->iAuxMpfS, this->iAuxMpfCo);
		mpf_add(aResult, aResult, this->iAuxMpfS);
	}

	//TODO dodelat podminku jen pro nespecialni prvocisla
	for(; mpz_cmp_si(p, this->iAlphaSmallerPrimeBound) <= 0; mpz_nextprime(p, p))
	{
		pp = mpz_get_ui(p);
		r = aPoly->RootsModPrimeGeneral(aRoots, pp);
		if (r == -1)
		{
			CLog::LogError(WHEREARG,"APolyPhase","AlphaHomPoly",ConstRC::NotEnoughMemory,0,"Not enough memory to calculate number of roots mod p of aPoly.");
			return ConstRC::NotEnoughMemory;
		}
		if (mpz_divisible_p(*aPoly->Get2(d), p))	// add projective roots to r
		{
			r++;
		}
		r--;
		r *= -pp;
		r++;	// r = p + 1 - q_p*p where q_p is number of all roots of aPoly(x,y) mod p
		mpf_set_d(this->iAuxMpfS, r * log((double)pp));
		pp = pp * pp - 1;
		mpf_div_ui(this->iAuxMpfS, this->iAuxMpfS, pp);
		mpf_add(aResult, aResult, this->iAuxMpfS);
	}

	mpz_clear(p);

	return ConstRC::Ok;
}

int APolyPhase::AlphaPoly(mpf_t aResult, long * aRoots, Polynomial *aPoly) const
{
	if (aPoly == NULL || aRoots == NULL)
	{
		CLog::LogError(WHEREARG,"APolyPhase","AlphaPoly",0,0,"Polynomial or Roots is NULL.");
		return ConstRC::GeneralError;
	}

	mpz_t p;
	mpz_init(p);
	long r, pp;
	mpf_t sum, cont;

	mpf_init(sum);
	mpf_init(cont);
	mpf_set_ui(aResult, 0);
	for (mpz_set_ui(p, 2); mpz_cmp_si(p, this->iAlphaPrimeBound) <= 0; mpz_nextprime(p, p))
	{
		pp = mpz_get_ui(p);
		mpf_set_ui(cont, pp - 1);
		mpf_ui_div(sum, 1, cont);
		ContPPoly(cont, aPoly, pp);
		mpf_sub(sum, sum, cont);
		mpf_set_d(cont, log((double)pp));
		mpf_mul(sum, sum, cont);

#ifdef DEBUG_INFO_APolyPhase
		gmp_printf("%Ff\n",sum);
#endif

		mpf_add(aResult, aResult, sum);
	}
	for(; mpz_cmp_si(p, this->iAlphaSmallerPrimeBound) <= 0; mpz_nextprime(p, p))
	{
		// alpha(f) = sum_{p < B} (1 - r)log(p) / (p - 1) 
		pp = mpz_get_ui(p);
		r = aPoly->RootsModPrimeGeneral(aRoots, pp);

#ifdef DEBUG_INFO_APolyPhase
		printf("%ld korenu, ",r);
#endif

		r = 1 - r;
		mpf_set_d(sum, r * log((double)pp));
		mpf_div_ui(sum, sum, pp - 1);
		mpf_add(aResult, aResult, sum);

#ifdef DEBUG_INFO_APolyPhase
		gmp_printf("%Ff\n", sum);
#endif

	}

	mpz_clear(p);
	mpf_clear(sum);
	mpf_clear(cont);

	return ConstRC::Ok;
}

int APolyPhase::AlphaEstimateHomPoly(mpf_t aResult, Polynomial *aPoly) const
{
	if (aPoly == NULL)
	{
		CLog::LogError(WHEREARG,"APolyPhase","AlphaEstimateHomPoly",0,0,"Polynomial is NULL.");
		return ConstRC::GeneralError;
	}

	mpz_t p;
	mpz_init(p);
	int d = aPoly->GetDegree();
	long *Roots = NULL, r, pp;
	r = 1;
	mpf_t sum;
	mpf_init(sum);
	mpf_set_ui(aResult, 0);
	Roots = (long*)Utils::AllocationMachine((void **)&Roots, d * sizeof(long));
	if (Roots == NULL)
	{
		CLog::LogError(WHEREARG,"APolyPhase","AlphaEstimateHomPoly",0,0,"Could not allocate memory for Roots.");
		return ConstRC::NotEnoughMemory;
	}

	for (mpz_set_ui(p, 2); 
		 (mpz_cmp_si(p, this->iAlphaPrimeBound) <= 0) && (mpf_cmp_d(aResult, this->iAlphaAbort) <= 0); 
		 mpz_nextprime(p, p))
	{
		pp = mpz_get_ui(p);
		r = aPoly->RootsModPrimeGeneral(Roots, pp);
		if (r < 0)
		{
			return ConstRC::GeneralError;
		}
		if (mpz_divisible_p(*aPoly->Get2(d), p))
		{
			r++;
		}

#ifdef DEBUG_INFO_APolyPhase
		printf("%ld korenu, ", r);
#endif

		r--;
		r *= -pp;
		r++;
//      e = mpz_sizeinbase(p, 2);
//      r *= (--e);
//      mpf_set_si(sum, r);
		mpf_set_d(sum, r * log((double) pp));
		pp = pp * pp - 1;
		mpf_div_ui(sum, sum, pp);
		mpf_add(aResult, aResult, sum);

#ifdef DEBUG_INFO_APolyPhase
		gmp_printf("%Ff\n", sum);
#endif

	}

	mpz_clear(p);
	mpf_clear(sum);
	free(Roots);

	return ConstRC::Ok;
}
int APolyPhase::AlphaEstimateHomPoly(mpf_t aResult, long * aRoots, Polynomial *aPoly) const
{
	if (aPoly == NULL || aRoots == NULL)
	{
		CLog::LogError(WHEREARG,"APolyPhase","AlphaEstimateHomPoly",0,0,"Polynomial or Roots is NULL.");
		return ConstRC::GeneralError;
	}

	mpz_t p;
	mpz_init(p);
	int d = aPoly->GetDegree();
	long r, pp;
	mpf_t sum;
	mpf_init(sum);
	mpf_set_ui(aResult, 0);

	for (mpz_set_ui(p, 2); 
		 (mpz_cmp_si(p, this->iAlphaPrimeBound) <= 0) && (mpf_cmp_d(aResult, this->iAlphaAbort) <= 0); 
		 mpz_nextprime(p, p))
	{
		pp = mpz_get_ui(p);
		r = aPoly->RootsModPrimeGeneral(aRoots, pp);
		if (r < 0)
		{
			return ConstRC::GeneralError;
		}
		if (mpz_divisible_p(*aPoly->Get2(d), p))
		{
			r++;
		}

#ifdef DEBUG_INFO_APolyPhase
		printf("%ld korenu, ", r);
#endif

		r--;
		r *= -pp;
		r++;
		//e = mpz_sizeinbase(p, 2);
		//r *= (--e);
		//mpf_set_si(sum, r);
		mpf_set_d(sum, r * log((double) pp));
		pp = pp * pp - 1;
		mpf_div_ui(sum, sum, pp);
		mpf_add(aResult, aResult, sum);

#ifdef DEBUG_INFO_APolyPhase
		gmp_printf("%Ff\n", sum);
#endif

	}

	mpz_clear(p);
	mpf_clear(sum);

	return ConstRC::Ok;
}
int APolyPhase::AlphaEstimatePoly(mpf_t aResult, long * aRoots, Polynomial *aPoly) const
{
	if (aPoly == NULL || aRoots == NULL)
	{
		CLog::LogError(WHEREARG,"APolyPhase","AlphaEstimatePoly",0,0,"Polynomial or Roots is NULL.");
		return ConstRC::GeneralError;
	}

	mpz_t p;
	mpz_init(p);
	long r, pp;
	mpf_t sum;
	mpf_init(sum);
	mpf_set_ui(aResult, 0);

	for (mpz_set_ui(p, 2); 
		 (mpz_cmp_si(p, this->iAlphaPrimeBound) <= 0) && (mpf_cmp_d(aResult, this->iAlphaAbort) <= 0); 
		 mpz_nextprime(p, p))
	{
		// alpha(f) = sum_{p < B} (1 - r)log(p) / (p - 1) 
		pp = mpz_get_ui(p);
		r = aPoly->RootsModPrimeGeneral(aRoots, pp);
		if (r < 0)
		{
			return ConstRC::GeneralError;
		}

#ifdef DEBUG_INFO_APolyPhase
		printf("%ld korenu, ", r);
#endif

		r = 1 - r;
		mpf_set_d(sum, r * log((double) pp));
		mpf_div_ui(sum, sum, pp - 1);
		mpf_add(aResult, aResult, sum);

#ifdef DEBUG_INFO_APolyPhase
		gmp_printf("%Ff\n", sum);
#endif

	}

	mpz_clear(p);
	mpf_clear(sum);

	return ConstRC::Ok;
}
/*!
	Sets the mpf_t precision to aPrecision bits.
*/

void APolyPhase::Set_Precision(unsigned long aPrecision)
{
	mpf_set_default_prec(aPrecision);
	mpz_t fact;
	mpz_init_set_ui(fact, 1);
	long i;
	for (i = 2; mpz_sizeinbase(fact, 2) < aPrecision; i++)
	{
		mpz_mul_si(fact, fact, i);
	}
	this->iFactorialToPrecision = i - 1;

	mpz_clear(fact);
	
	if (this->iSubintervalsCount > 0) 
	{
		Set_SubintervalsCount(this->iSubintervalsCount);
	}
}

/*!
	Sets the subintervals of the half circle count to aCount.
*/

void APolyPhase::Set_SubintervalsCount(long aCount)
{
	mpf_set_str(this->iSubintervalLength, "3.1415926535897932384626433832795028841971693993751", 10);
	mpf_div_ui(this->iSubintervalLength, this->iSubintervalLength, aCount);
	if (this->iSin != NULL) 
	{
		for (long i = 0; i < this->iSubintervalsCount; i++)
		{
			mpf_clear(this->iSin[i]);
			mpf_clear(this->iCos[i]);
		}
		free(this->iSin);
		free(this->iCos);
		this->iSin = NULL;
		this->iCos = NULL;
	}
	this->iSubintervalsCount = aCount;
}

/*!
	Returns the sin and the cos of the given angle/
*/
int APolyPhase::Goniometric(mpf_t aSin, mpf_t aCos, mpf_t aAngle)
{
	mpf_t x,aux,pow,fact;

	mpf_init(x);
	mpf_init(pow);
	mpf_init(fact);
	mpf_init_set(aux, this->iPiHalf);

	mpf_mul_2exp(aux, aux, 2);
	mpf_div(x, aAngle, aux);
	mpf_floor(pow, x);
	mpf_sub(x, x, pow);
	if (mpf_cmp_d(x, 0.5) > 0)
	{
		mpf_add_ui(pow, pow, 1);
	}

	mpf_mul(pow, pow, aux);
	mpf_sub(x, aAngle, pow);
	int Quartal = mpf_sgn(x);
	mpf_abs(x, x);

	if(mpf_cmp(this->iPiHalf, x) < 0)
	{
		Quartal = 2 * Quartal;
		mpf_sub(x, x, this->iPiHalf);
		mpf_sub(x, this->iPiHalf, x);
	}

	bool rot = false;
	mpf_mul_2exp(aux, x, 1);
	if (mpf_cmp(aux,this->iPiHalf)>0)
	{
		rot = true;
		mpf_sub(x, this->iPiHalf, x);
	}

	mpf_set_ui(aSin, 0);
	mpf_set_ui(aCos, 1);
	mpf_set_ui(pow, 1);
	mpf_set_ui(fact, 1);

	for (long i = 1; i <= this->iFactorialToPrecision; i++)
	{
		mpf_mul(pow, pow, x);
		mpf_mul_ui(fact, fact, i);
		mpf_div(aux, pow, fact);
		switch (i & 3)
		{
			case 0: 
				mpf_add(aCos, aCos, aux);
				break;
			case 1: 
				mpf_add(aSin, aSin, aux);
				break;
			case 2: 
				mpf_sub(aCos, aCos, aux);
				break;
			case 3:
				mpf_sub(aSin, aSin, aux);
				break;
		}
	}
	if (rot) 
	{
		mpf_swap(aCos, aSin);
	}
	if (Quartal < 0) 
	{
		mpf_neg(aSin, aSin);
	}
	if (Quartal * Quartal == 4) 
	{
		mpf_neg(aCos, aCos);
	}

	mpf_clear(x);
	mpf_clear(fact);
	mpf_clear(aux);
	mpf_clear(pow);

	return ConstRC::Ok;
}

/*!
	Evaluates the polynomial aPolynomial in the pair (1024*cos aAngle,1024*sin aAngle).
*/
int APolyPhase::EvaluateAngle(mpf_t aResult, Polynomial *aPolynomial, mpf_t aAngle)
{
	mpf_t co,sn,cs,pow;

	mpf_init(co);
	mpf_init(sn);
	mpf_init(cs);
	mpf_init(pow);

	mpf_set_ui(aResult, 0);

	Goniometric(sn, cs, aAngle);

	mpf_mul_2exp(sn, sn, 10);
	mpf_mul_2exp(cs, cs, 10);
	mpf_pow_ui(pow, cs, aPolynomial->GetDegree());

	for (int i = aPolynomial->GetDegree(); i >= 0; i--)
	{
		mpf_set_z(co, *aPolynomial->Get2(i));
		mpf_mul(co, co, pow);
		if (i)
		{
			mpf_div(pow, pow, cs);
			mpf_mul(pow, pow, sn);
		}
		mpf_add(aResult, aResult, co);
	}

	mpf_clear(co);
	mpf_clear(sn);
	mpf_clear(cs);
	mpf_clear(pow);

	return ConstRC::Ok;
}
int APolyPhase::GenerateSinCos()
{
	if (this->iSin == NULL)
	{
		mpf_t aux;
		mpf_init(aux);

		this->iSin = (mpf_t*) Utils::AllocationMachine((void**)&this->iSin, this->iSubintervalsCount * sizeof (mpf_t));
		if (!this->iSin)
		{
			CLog::LogError(WHEREARG, "APolyPhase","GenerateSinCos",ConstRC::NotEnoughMemory,0,"There was not enough memory to allocate array for iSin.");
			return ConstRC::NotEnoughMemory;
		}
		this->iCos = (mpf_t*) Utils::AllocationMachine((void**)&this->iCos, this->iSubintervalsCount * sizeof (mpf_t));
		if (!this->iCos)
		{
			CLog::LogError(WHEREARG, "APolyPhase","GenerateSinCos",ConstRC::NotEnoughMemory,0,"There was not enough memory to allocate array for iCos.");
			return ConstRC::NotEnoughMemory;
		}
		mpf_div_2exp(aux, this->iSubintervalLength, 1);	// aux = iSubintervalLength/2
		for (long i = 0; i < this->iSubintervalsCount; i++)
		{
			mpf_init(this->iSin[i]);
			mpf_init(this->iCos[i]);
			Goniometric(this->iSin[i], this->iCos[i], aux);
			mpf_add(aux, aux, this->iSubintervalLength);
			//TODO zjistit, jaky ma zakomentovani vliv na rating v poly_sel_phase.cpp
			//mpf_mul_2exp(this->iSin[i], this->iSin[i], 10);
			//mpf_mul_2exp(this->iCos[i], this->iCos[i], 10);
		}

		mpf_clear(aux);
	}

	return ConstRC::Ok;
}
/*!
	Evaluates the polynomial aPolynomial in the aInterval-th point of the half circle.
*/
int APolyPhase::EvaluateAngle(mpf_t aResult, Polynomial *aPolynomial, long aInterval)
{
	mpf_t co, pow;

	mpf_init(co);
	mpf_init(pow);
	mpf_set_ui(aResult,0);
	
	if (this->iSin == NULL)
	{
		GenerateSinCos();
	}
	mpf_pow_ui(pow, this->iCos[aInterval], aPolynomial->GetDegree());
	for(int i = aPolynomial->GetDegree(); i >=0; i--)
	{
		mpf_set_z(co,*aPolynomial->Get2(i));
		mpf_mul(co,co,pow);
		if (i)
		{
			mpf_div(pow,pow,this->iCos[aInterval]);
			mpf_mul(pow,pow,this->iSin[aInterval]);
		}
		mpf_add(aResult,aResult,co);
	}

	mpf_clear(co);
	mpf_clear(pow);

	return ConstRC::Ok;
}

/*!
	Evaluates the polynomial aPolynomial in the aInterval-th point of the ellipse with axes aAxis1 and aAxis2.
*/

int APolyPhase::EvaluateEllipseAngle(mpf_t aResult, Polynomial *aPolynomial, long aInterval, mpf_t aAxis1, mpf_t aAxis2)
{
	int rc;
	mpf_set_ui(aResult, 0);

	if (this->iSin == NULL)
	{
		rc = GenerateSinCos();
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG, "APolyPhase","EvaluateEllipseAngle",ConstRC::NotEnoughMemory,0,"Some unexpected error occurs during subroutine.");
			return ConstRC::NotEnoughMemory;
		}
	}

	mpf_mul(this->iAuxMpfC, this->iCos[aInterval], aAxis1);
	mpf_mul(this->iAuxMpfS, this->iSin[aInterval], aAxis2);
	mpf_pow_ui(this->iAuxMpfPow, this->iAuxMpfC, aPolynomial->GetDegree());

	for (int i = aPolynomial->GetDegree(); i >= 0; i--)
	{
		mpf_set_z(this->iAuxMpfCo, *aPolynomial->Get2(i));
		mpf_mul(this->iAuxMpfCo, this->iAuxMpfCo, this->iAuxMpfPow);
		if (i)
		{
			mpf_div(this->iAuxMpfPow, this->iAuxMpfPow, this->iAuxMpfC);
			mpf_mul(this->iAuxMpfPow, this->iAuxMpfPow, this->iAuxMpfS);
		}
		mpf_add(aResult, aResult, this->iAuxMpfCo);
	}

	return ConstRC::Ok;
}

/*!
	Generate the rho function Taylor coefficients.
*/

int APolyPhase::GenerateRho()
{
	if (this->iRhoTaylor != NULL)
	{
		return ConstRC::Ok;
	}
	this->iRhoTaylor = (mpf_t**)Utils::AllocationMachine((void**)&this->iRhoTaylor, (ConstPSPhase::MAXIMAL_RHO_VALUE - 1) * sizeof(mpf_t*));
	if (!this->iRhoTaylor) 
	{
		return ConstRC::NotEnoughMemory;
	}
	this->iRhoTaylor[0] = NULL;
	this->iRhoTaylor[0] = (mpf_t*)Utils::AllocationMachine((void**)&this->iRhoTaylor[0], ConstPSPhase::RHO_EXPANSION_LENGTH * sizeof(mpf_t));
	if (!this->iRhoTaylor[0])
	{
		return ConstRC::NotEnoughMemory;
	}
	int i,j,l;
	// c_i^(2) = 1/(i*2^i) i > 0
	for (i = 1; i < ConstPSPhase::RHO_EXPANSION_LENGTH; i++)
	{
		mpf_init_set_ui(this->iRhoTaylor[0][i], 1);
		mpf_div_ui(this->iRhoTaylor[0][i], this->iRhoTaylor[0][i], i);
		mpf_div_2exp(this->iRhoTaylor[0][i], this->iRhoTaylor[0][i], i);
	}
	// c_0^(2) = 1 - log(2)
	mpf_init(this->iRhoTaylor[0][0]);
	mpf_ui_sub(this->iRhoTaylor[0][0], 1, this->iLogTwo);

	for (l = 1; l <= ConstPSPhase::MAXIMAL_RHO_VALUE - 2; l++)	// l iterates correctly through iRhoTaylor, but its value is l-2 compared to theory => (*)
	{
		this->iRhoTaylor[l] = NULL;
		this->iRhoTaylor[l] = (mpf_t*)Utils::AllocationMachine((void**)&this->iRhoTaylor[l], ConstPSPhase::RHO_EXPANSION_LENGTH * sizeof(mpf_t));
		if (!this->iRhoTaylor[l]) 
		{
			return ConstRC::NotEnoughMemory;
		}
		// c_i^(l) = sum_{j=0}^{i-1} \frac{c_j^(l-1)}{i*l^{i-j}}
		for (i = 1; i < ConstPSPhase::RHO_EXPANSION_LENGTH; i++)
		{
			mpf_init(this->iRhoTaylor[l][i]);
			mpf_set_ui(this->iAuxMpfS, l + 2);
			for (j = i - 1; j >= 0; j--)
			{
				mpf_div_ui(this->iAuxMpfC, this->iRhoTaylor[l - 1][j], i);
				mpf_div(this->iAuxMpfC, this->iAuxMpfC, this->iAuxMpfS);
				mpf_mul_ui(this->iAuxMpfS, this->iAuxMpfS, l + 2);	// (*)
				mpf_add(this->iRhoTaylor[l][i], this->iRhoTaylor[l][i], this->iAuxMpfC);
			}
		}
		// c_0^(l) = 1/(l-1)*sum_{j=1}^inf \frac{c_j^(l)}{j+1}
		mpf_init(this->iRhoTaylor[l][0]);
		for (j = 1; j < ConstPSPhase::RHO_EXPANSION_LENGTH; j++)
		{
			mpf_div_ui(this->iAuxMpfC, this->iRhoTaylor[l][j], j + 1);
			mpf_add(this->iRhoTaylor[l][0], this->iRhoTaylor[l][0], this->iAuxMpfC);
		}
		mpf_div_ui(this->iRhoTaylor[l][0], this->iRhoTaylor[l][0], l + 1); // (*)
	}

	return ConstRC::Ok;
}

/*!
	Computes the Dickman's rho function at aValue.
*/

int APolyPhase::Rho(mpf_t aResult, mpf_t aValue)
{
	int rc;
	if (mpf_cmp_ui(aValue,1) <= 0)
	{
		mpf_set_ui(aResult,1);
		return ConstRC::Ok;
	}
	if (mpf_cmp_ui(aValue, ConstPSPhase::MAXIMAL_RHO_VALUE) > 0)
	{
		mpf_set_ui(aResult,0);
		return ConstRC::Ok;
	}
	if (this->iRhoTaylor == NULL) 
	{
		rc = GenerateRho();
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG, "APolyPhase","Rho",rc,0,"Some unexpected error occurs during GenerateRho.");
			return rc;
		}

	}
	unsigned int cela = mpf_get_ui(aValue);
	if (mpf_cmp_ui(aValue, cela))
	{
		cela++;
	}
	mpf_t pow,sum,x;
	mpf_init_set_ui(pow, 1);
	mpf_init(sum);
	mpf_init(x);
	mpf_ui_sub(x,cela, aValue);
	mpf_set_ui(aResult, 0);
	cela -= 2;
	for (int i = 0; i < ConstPSPhase::RHO_EXPANSION_LENGTH - 5; i++)
	{
		mpf_mul(sum, this->iRhoTaylor[cela][i], pow);
		mpf_add(aResult, aResult, sum);
		mpf_mul(pow, pow, x);
	}
	mpf_clear(pow);
	mpf_clear(sum);
	mpf_clear(x);
	/// TODO hodnota iRhoMax se dale nijak nevyuziva
	if (mpf_cmp(aValue, this->iRhoMax) > 0)
	{
		mpf_set(this->iRhoMax,aValue);
	}
	return ConstRC::Ok;
}

