#include "adv_factor_base_helper.h"

#include <cstring>
#include <math.h>

#include "const_return_codes.h"
#include "const_element_flags.h"
#include "const_sieving_phase.h"
#include "integer_class.h"
#include "const_debug.h"
#include "log.h"

using namespace std;

bool CAdvFBHelper::AssertionMode = false;
bool CAdvFBHelper::InfoMode = false;

int CAdvFBHelper::InitAndPrepareLatticeFB(CAdvLFBInfo &aAdvFB, CFBInfo & aFB)
{
	if (aAdvFB.FactorBase != NULL)
	{
		CLog::LogError(WHEREARG,"CAdvFBHelper","InitAndPrepareLatticeFB",0,0,"Advanced FB already inited.");
		return ConstRC::InconsistentState;
	}
	
	aAdvFB.FactorBase = (lattice_type2*) Utils::AllocationMachine((void**) &aAdvFB.FactorBase, (aFB.FBMaxAllocatedIndex + 1) * sizeof(lattice_type2));
	if (aAdvFB.FactorBase == NULL)
	{
		CLog::LogError(WHEREARG,"CAdvFBHelper","InitAndPrepareLatticeFB",0,(aFB.FBMaxAllocatedIndex + 1)*sizeof(lattice_type2),"Could not allocate advanced integral FB.");
		return ConstRC::NotEnoughMemory;
	}
	memset(aAdvFB.FactorBase, 0, (aFB.FBMaxAllocatedIndex + 1) * sizeof(lattice_type2));
	
	aAdvFB.Count = aFB.FBMaxIndex + 1;

	lattice_type2* curr = aAdvFB.FactorBase;
	fb_element * fb = aFB.FactorBase;
	long max = aFB.FBMaxIndex;
	for (int i = 0; i <= max; i++, fb++, curr++)
	{
		curr->p		= fb->prime;
		curr->r		= fb->root;
		curr->logp	= fb->log;
		if (fb->flags & ConstEF::SPECIAL_PRIME)
		{
			curr->flags |= ConstEF::SPECIAL_PRIME_MINI_FLAG;
		}
		if (fb->flags & ConstEF::DIVIDES_ROOT)
		{
			curr->flags |= ConstEF::DIVIDES_ROOT_MINI_FLAG;
		}
		if (fb->flags & ConstEF::DIVIDES_L)
		{
			curr->flags |= ConstEF::DIVIDES_L_MINI_FLAG;
		}
	}

	return ConstRC::Ok;
}
int CAdvFBHelper::FillLatticeFB(CAdvLFBInfo & aAdvFB, main_lattice_info & aInfo)
{
	lattice_type2* curr = aAdvFB.FactorBase;
	lattice_type2* end = aAdvFB.FactorBase + aAdvFB.Count;

	// we know that I is power of 2
	long posI = aInfo.I;
	long negI = - aInfo.I;
	long halfI = aInfo.I >> 1;
	main_sieving_type newR;

	int k;
	int a;
	main_sieving_type p;
	main_sieving_type temp;
	main_sieving_type ik;
	main_sieving_type ik_1;
	main_sieving_type jk;
	main_sieving_type jk_1;

	INT64 aux1, aux2;

#ifdef DEBUG_SIEVING_PHASE_ADV_FB_CONTROL
	int bad = 0;
#endif

	aAdvFB.BadPrimes.clear();

	if (curr->p == -1)
	{
		// on the 0th index can be -1. No sieving occurs for -1!
		curr++;
	}

	// recompute cr for small primes
	while (curr != aAdvFB.MiddleFB)
	{
		if (curr->flags & ConstEF::LATTICE_NOT_FOR_SIEVING_MINI_FLAG)
		{
			curr++;
			continue;
		}

		// (a1 - b1*r)/(r*b0 - a0) mod p

		p = curr->p;

		aux1 = (INT64)aInfo.b0 * curr->r;
		aux1 -= aInfo.a0;
		temp = aux1 % p;
		
		if (temp == 0)
		{
			// wrong prime - cannot be used for sieving
			curr->cr = -1; // flag for special factorization - not pernament flag
			aAdvFB.BadPrimes.push_back(*curr);
#ifdef DEBUG_SIEVING_PHASE_ADV_FB_CONTROL
			bad++;
#endif
			curr++;
			continue;
		}

		aux1 = (INT64)aInfo.b1 * curr->r;
		aux1 = aInfo.a1 - aux1;
		aux1 %= p;

		if (aux1 != 0)
		{
			aux2 = CIntegerClass::InverseModN(temp, p);
			aux1 *= aux2;
			aux1 %= p;
		}

		newR = (main_sieving_type)aux1;

		if (newR < 0)
		{
			newR += p;
		}

		curr->cr = newR;

//		if (newR == 0 && p >= posI)
//		{
//			// wrong prime - cannot be used for sieving ???
//#ifdef DEBUG_SIEVING_PHASE_ADV_FB_CONTROL
//			bad++;
//#endif
//			continue;
//		}

		curr++;
	}

	lattice_update_t* update = aAdvFB.MiddlePrimes;
	while (curr <= aAdvFB.LastMiddleFB)
	{
		if (curr->flags & ConstEF::LATTICE_NOT_FOR_SIEVING_MINI_FLAG)
		{
			update->h = -1; // flag for not use for sieving
			update->start = -1;
			update->flags = curr->flags;
			update->logp = curr->logp;
			update->element = curr;
			update++;
			curr++;
			continue;
		}

		// (a1 - b1*r)/(r*b0 - a0) mod p

		p = curr->p;

		aux1 = (INT64)aInfo.b0 * curr->r;
		aux1 -= aInfo.a0;
		temp = aux1 % p;
		
		if (temp == 0)
		{
			// wrong prime - cannot be used for sieving
			update->h = -1; // flag for not use for sieving - not pernament flag
			update->start = -1;
			curr->cr = -1; // flag for special factorization - not pernament flag
			aAdvFB.BadPrimes.push_back(*curr);
#ifdef DEBUG_SIEVING_PHASE_ADV_FB_CONTROL
			bad++;
#endif
			update++;
			curr++;
			continue;
		}

		aux1 = (INT64)aInfo.b1 * curr->r;
		aux1 = aInfo.a1 - aux1;
		aux1 %= p;

		if (aux1 != 0)
		{
			aux2 = CIntegerClass::InverseModN(temp, p);
			aux1 *= aux2;
			aux1 %= p;
		}

		newR = (main_sieving_type)aux1;

		if (newR < 0)
		{
			newR += p;
		}

		curr->cr = newR;

//		if (newR == 0 && p >= posI)
//		{
//			// wrong prime - cannot be used for sieving ???
//			update->h = -1; // flag for not use for sieving
//			update->start = -1;
//#ifdef DEBUG_SIEVING_PHASE_ADV_FB_CONTROL
//			bad++;
//#endif
//			update++;
//			curr++;
//			continue;
//		}

		// i = x % I
		// S(x) += log(p)
		// if i >= b0 then x += a
		// if i < b1 then x += c

		if (p < posI)
		{
			// small primes - we don't need to use lattice for them

			// start value - we use interval 0 - I <--> - I/2 - I/2-1
			update->h = (halfI + newR) % p;
			update->start = -1;
			update->flags = curr->flags;
			update->logp = curr->logp;

			// shift to next row
			update->a = (posI + newR) % p;

			// x = x + p
			update->c = p;

			update->b0 = posI - curr->p;
			update->b1 = posI - update->a - ConstSPhase::LATTICE_FB_B1_CHANGE;
			update->element = curr;
		}
		else if (newR == 0)
		{
			// p is too big - only one value in each row
			update->h = halfI;
			update->start = -1;
			update->flags = curr->flags;
			update->logp = curr->logp;
			update->b0 = 0;
			update->b1 = 0; // b1 = '0', b0 = 0 -- b1 as small as possible - b1 cannot be set to zero
			update->a = posI;
			update->c = 0;
			update->element = curr;
		}
		else
		{
			a = 0;
			k = 1;
			ik_1 = - p;
			jk_1 = 0;
			ik = newR;
			jk = 1;

			//while (negI < ik && ik < posI && (posI <= ik_1 || ik_1 <= negI))
			while (negI >= ik || ik >= posI || (posI > ik_1 && ik_1 > negI))
			{
				a = abs(ik_1) / abs(ik);
				temp = ik_1 + a * ik;
				ik_1 = ik;
				ik = temp;
				temp = jk_1 + a * jk;
				jk_1 = jk;
				jk = temp;
				
				k++;
			}

			a = ceil(((float)abs(ik_1) - posI)/(float)abs(ik)); 

			// b0 = I - gamma
			// b1 = - alpha
			// a = I * beta + alpha
			// c = I * delta + gamma

			if (k % 2 == 1)
			{
				// (alpha, beta)  = (ik_1, jk_1) + a * (ik, jk)
				// (gamma, delta) = (ik, jk)
				aux1 = - ik_1 - a * ik;
				update->flags = curr->flags;
				update->logp = curr->logp;
				update->b0 = posI - ik;
				update->b1 = - ik_1 - a * ik - ConstSPhase::LATTICE_FB_B1_CHANGE;
				temp = jk_1 + a * jk;
				if (temp > aInfo.J)
				{
					update->a = posI * aInfo.J;
				}
				else
				{
					update->a = posI * temp - aux1;
				}
				update->c = posI * jk + ik;
			}
			else
			{
				// (alpha, beta)  = (ik, jk)
				// (gamma, delta) = (ik_1, jk_1) + a * (ik, jk)
				temp = ik_1 + a * ik; // gamma
				update->flags = curr->flags;
				update->logp = curr->logp;
				update->b0 = posI - temp;
				update->b1 = - ik - ConstSPhase::LATTICE_FB_B1_CHANGE;
				update->a = posI * jk + ik;
				temp = jk_1 + a * jk; // delta
				if (temp > aInfo.J)
				{
					update->c = posI * aInfo.J;
				}
				else
				{
					update->c = posI * temp - update->b0 + posI;
				}
			}

			// start - index of (0,0) is -halfI => it is halfI modulo I
			update->h = - halfI;
			if (halfI >= update->b0)
			{
				update->h += update->a;
			}
			if (halfI < (update->b1 + ConstSPhase::LATTICE_FB_B1_CHANGE))
			{
				update->h += update->c;
			}
			update->start = -1;
			update->element = curr;
		}

		update++;
		curr++;
	}

	update = aAdvFB.LargishPrimes;
	if (update != NULL)
	{
		while (curr != end)
		{
			if (curr->flags & ConstEF::LATTICE_NOT_FOR_SIEVING_MINI_FLAG)
			{
				update->h = -1; // flag for not use for sieving
				update->start = -1;
				update->flags = curr->flags;
				update->logp = curr->logp;
				update->element = curr;
				update++;
				curr++;
				continue;
			}

			// (a1 - b1*r)/(r*b0 - a0) mod p

			p = curr->p;

			aux1 = (INT64)aInfo.b0 * curr->r;
			aux1 -= aInfo.a0;
			temp = aux1 % p;
			
			if (temp == 0)
			{
				// wrong prime - cannot be used for sieving
				update->h = -1; // flag for not use for sieving
				update->start = -1;
				if (p >= posI)
				{
					aAdvFB.BadPrimes.push_back(*curr);
				}
#ifdef DEBUG_SIEVING_PHASE_ADV_FB_CONTROL
				bad++;
#endif
				update++;
				curr++;
				continue;
			}

			aux1 = (INT64)aInfo.b1 * curr->r;
			aux1 = aInfo.a1 - aux1;
			aux1 %= p;

			if (aux1 != 0)
			{
				aux2 = CIntegerClass::InverseModN(temp, p);
				aux1 *= aux2;
				aux1 %= p;
			}

			newR = (main_sieving_type)aux1;

			if (newR < 0)
			{
				newR += p;
			}

			curr->cr = newR;

//			if (newR == 0 && p >= posI)
//			{
//				// wrong prime - cannot be used for sieving ???
//				update->h = -1; // flag for not use for sieving
//				update->start = -1;
//#ifdef DEBUG_SIEVING_PHASE_ADV_FB_CONTROL
//				bad++;
//#endif
//				update++;
//				curr++;
//				continue;
//			}

			// i = x % I
			// S(x) += log(p)
			// if i >= b0 then x += a
			// if i < b1 then x += c

			if (p < posI)
			{
				// small primes - we don't need to use lattice for them

				// start value - we use interval 0 - I <--> - I/2 - I/2-1
				update->h = (halfI + newR) % p;
				update->start = -1;
				update->flags = curr->flags;
				update->logp = curr->logp;

				// shift to next row
				update->a = (posI + newR) % p;

				// x = x + p
				update->c = p;

				update->b0 = posI - curr->p;
				update->b1 = posI - update->a - ConstSPhase::LATTICE_FB_B1_CHANGE;
				update->element = curr;
			}
			else if (newR == 0)
			{
				// p is too big - only one value in each row
				update->h = halfI;
				update->start = -1;
				update->flags = curr->flags;
				update->logp = curr->logp;
				update->b0 = 0;
				update->b1 = 0; // b1 = '0', b0 = 0 -- b1 as small as possible - b1 cannot be set to zero
				update->a = posI;
				update->c = 0;
				update->element = curr;
			}
			else
			{
				a = 0;
				k = 1;
				ik_1 = - p;
				jk_1 = 0;
				ik = newR;
				jk = 1;

				//while (negI < ik && ik < posI && (posI <= ik_1 || ik_1 <= negI))
				while (negI >= ik || ik >= posI || (posI > ik_1 && ik_1 > negI))
				{
					a = abs(ik_1) / abs(ik);
					temp = ik_1 + a * ik;
					ik_1 = ik;
					ik = temp;
					temp = jk_1 + a * jk;
					jk_1 = jk;
					jk = temp;
					
					k++;
				}

				a = ceil(((float)abs(ik_1) - posI)/(float)abs(ik)); 

				// b0 = I - gamma
				// b1 = - alpha
				// a = I * beta + alpha
				// c = I * delta + gamma

				if (k % 2 == 1)
				{
					// (alpha, beta)  = (ik_1, jk_1) + a * (ik, jk)
					// (gamma, delta) = (ik, jk)
					aux1 = - ik_1 - a * ik;
					update->flags = curr->flags;
					update->logp = curr->logp;
					update->b0 = posI - ik;
					update->b1 = - ik_1 - a * ik - ConstSPhase::LATTICE_FB_B1_CHANGE;
					temp = jk_1 + a * jk;
					if (temp > aInfo.J)
					{
						update->a = posI * aInfo.J;
					}
					else
					{
						update->a = posI * temp - aux1;
					}
					update->c = posI * jk + ik;
				}
				else
				{
					// (alpha, beta)  = (ik, jk)
					// (gamma, delta) = (ik_1, jk_1) + a * (ik, jk)
					temp = ik_1 + a * ik; // gamma
					update->flags = curr->flags;
					update->logp = curr->logp;
					update->b0 = posI - temp;
					update->b1 = - ik - ConstSPhase::LATTICE_FB_B1_CHANGE;
					update->a = posI * jk + ik;
					temp = jk_1 + a * jk; // delta
					if (temp > aInfo.J)
					{
						update->c = posI * aInfo.J;
					}
					else
					{
						update->c = posI * temp - update->b0 + posI;
					}
				}

				// start - index of (0,0) is -halfI => it is halfI modulo I
				update->h = - halfI;
				if (halfI >= update->b0)
				{
					update->h += update->a;
				}
				if (halfI < (update->b1 + ConstSPhase::LATTICE_FB_B1_CHANGE))
				{
					update->h += update->c;
				}
				update->start = -1;
				update->element = curr;
			}

			curr++;
			update++;
		}
	}

#ifdef DEBUG_SIEVING_PHASE_ADV_FB_CONTROL
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 50
	CLog::Log(50, "0640000", "FillLatticeFB: bad elements = %d\n", bad);
#endif
#endif

	return ConstRC::Ok;
}
int CAdvFBHelper::InitAndFillAdvFB(nfs_fb_type *&aAdvFB, CFBInfo & aFB, nfs_sieving_direction aDirection, main_sieving_type aA, main_sieving_type aB, mpz_t aLC, mpz_t aM)
{
	if (aAdvFB != NULL)
	{
		CLog::LogError(WHEREARG,"CAdvFBHelper","InitAndFillAdvFB",0,0,"Advanced FB already inited.");
		return ConstRC::InconsistentState;
	}
	
	aAdvFB = (nfs_fb_type*) Utils::AllocationMachine((void**) &aAdvFB, (aFB.FBMaxAllocatedIndex + 1) * sizeof(nfs_fb_type));
	if (aAdvFB == NULL)
	{
		CLog::LogError(WHEREARG,"CAdvFBHelper","InitAndFillAdvFB",0,(aFB.FBMaxAllocatedIndex + 1) * sizeof(nfs_fb_type),"Could not allocate advanced FB.");
		return ConstRC::NotEnoughMemory;
	}
	memset(aAdvFB, 0, (aFB.FBMaxAllocatedIndex + 1) * sizeof(nfs_fb_type));
	
	nfs_fb_type* curr = aAdvFB;
	fb_element * fb = aFB.FactorBase;

	log_type exp = 0;
	INT64 int1;
	mpz_t aux_1, aux_p, auxL;
	mpz_init(aux_1);
	mpz_init(aux_p);
	mpz_init_set(auxL, aLC);

	// special case - integral part has number -1
	int start = 0;
	if (fb->prime == -1)
	{
		start = 1;
		curr->p = fb->prime;		
		curr++;
		fb++;
	}

	for (long i = start; i <= aFB.FBMaxIndex; i++, curr++, fb++)
	{
		curr->p      = fb->prime;		
		curr->r		 = fb->root;
		curr->log_p  = fb->log;
		curr->flags  = 0;

		if (fb->flags & ConstEF::SPECIAL_PRIME)
		{
			// special prime
			curr->root_1 = -1;
			curr->flags = ConstEF::SPECIAL_PRIME_MINI_FLAG;
			// we need to know if the leading coeff is divisible by p - for factorization of sieving values
			if (exp > 0 || mpz_divisible_ui_p(auxL, curr->p))
			{
				curr->flags |= ConstEF::DIVIDES_L_MINI_FLAG;
				while (!mpz_fdiv_qr_ui(aux_1, aux_p, auxL, curr->p))
				{
					exp++;
					mpz_swap(auxL, aux_1);
				}
				if (exp > ConstEF::MAX_L_VALUATION)
				{
					mpz_clear(aux_1);
					mpz_clear(aux_p);
					mpz_clear(auxL);
					CLog::LogError(WHEREARG,"CAdvFBHelper","InitAndFillAdvFB",0,curr->p,"Prime valuation of leading coeff is too high!!!");
					return ConstRC::InconsistentState;
				}
				curr->flags |= exp;

				if (i + 1 > aFB.FBMaxIndex || (fb + 1)->prime != curr->p)
				{
					// next prime is different
					exp = 0;
				}
			}
		} 
		else if (fb->flags & ConstEF::DIVIDES_L)
		{
			// for two ideals:
			// - special kind of ideal: (p,p)
			// - and prime ideal: (p, \bar{\theta} - 0)
			curr->root_1 = -1;
			curr->flags = ConstEF::DIVIDES_L_MINI_FLAG;

			if (exp > 0 || mpz_divisible_ui_p(auxL, curr->p))
			{
				while (!mpz_fdiv_qr_ui(aux_1, aux_p, auxL, curr->p))
				{
					exp++;
					mpz_swap(auxL, aux_1);
				}
				if (exp > ConstEF::MAX_L_VALUATION)
				{
					mpz_clear(aux_1);
					mpz_clear(aux_p);
					mpz_clear(auxL);
					CLog::LogError(WHEREARG,"CAdvFBHelper","InitAndFillAdvFB",0,curr->p,"Prime valuation of leading coeff is too high!!!");
					return ConstRC::InconsistentState;
				}
				curr->flags |= exp;

				if (i + 1 > aFB.FBMaxIndex || (fb + 1)->prime != curr->p)
				{
					// next prime is different
					exp = 0;
				}
			}
		}
		else 
		{
			if (aDirection == EIncrementB || aDirection == EDecrementB)
			{
				// -root*b - 1 mod p
				curr->root_1 = CIntegerClass::Mulm(fb->root, aB, fb->prime) - 1;
				if (curr->root_1 < 0)
				{
					curr->root_1 += fb->prime;
				}
				
				// Check-out part
				if (CAdvFBHelper::AssertionMode)
				{
					mpz_mul_si(aux_1, aM, aB);		
					mpz_set_si(aux_p, curr->root_1 + 1);
					mpz_add(aux_1, aux_1, aux_p);
					int1 = mpz_mod_ui(aux_1, aux_1, fb->prime);
					if (int1 != 0) 
					{
						CLog::Log(MAXIMAL_INT_VALUE, "0640001", "Error for p = %d: result is %ld.\n", fb->prime, (long) int1);
					}
				}
			}
			else
			{
				mpz_set_si(aux_1, aA);
				mpz_set_si(aux_p, fb->inversion_of_root);
				mpz_mul(aux_1, aux_1, aux_p);
				curr->root_1 = fb->prime - mpz_mod_ui(aux_1, aux_1, fb->prime) - 1;

				// Check-out part
				if (CAdvFBHelper::AssertionMode)
				{
					mpz_mul_si(aux_1, aM, curr->root_1 + 1);		
					mpz_set_si(aux_p, aA);
					mpz_add(aux_1, aux_1, aux_p);
					int1 = mpz_mod_ui(aux_1, aux_1, fb->prime);
					if (int1 != 0) 
					{
						CLog::Log(MAXIMAL_INT_VALUE, "0640002", "Error for p = %d: result is %ld.\n", fb->prime, (long) int1);
					}
				}
			}

			// Non-special prime ideal.
			// shift to next row or column
			switch (aDirection)
			{
				case EIncrementB:
					curr->auxVal = fb->root;
					break;
				case EDecrementB:
					curr->auxVal = - fb->root;
					break;
				case EIncrementA:
					curr->auxVal = fb->inversion_of_root;
					// We will not sieve with this integral prime
					if (fb->flags & ConstEF::DIVIDES_ROOT)
					{
						curr->root_1 = -1;
						curr->flags |= ConstEF::DIVIDES_ROOT_MINI_FLAG;
					}
					break;
				case EDecrementA:
					curr->auxVal = - fb->inversion_of_root;
					// We will not sieve with this integral prime
					if (fb->flags & ConstEF::DIVIDES_ROOT)
					{
						curr->root_1 = -1;
						curr->flags |= ConstEF::DIVIDES_ROOT_MINI_FLAG;
					}
					break;
			}
		}
	}

	mpz_clear(aux_1);
	mpz_clear(aux_p);
	mpz_clear(auxL);

	return ConstRC::Ok;
}
