#ifndef __ALGEBRAIC_NUMBER_CLASS__
#define __ALGEBRAIC_NUMBER_CLASS__

#include "hermite_matrix_class.h"
#include "polynom_class.h"
#include <gmp.h>

/*!
 * \brief Reprezentation of algebraic numbers
 */
class AlgebraicNumber 
{
public:
	AlgebraicNumber(unsigned int aRank, HermiteMatrix **aThetaPowers, mpz_t *aDenominator);
	~AlgebraicNumber();
//getters and setters
	int Set(unsigned int aIndex,long aValue);
	int Set(unsigned int aIndex,mpz_t aValue);
	long Get(unsigned int aIndex);
	mpz_t *Get2(unsigned int aIndex);
	int Inc(unsigned int aIndex, mpz_t aValue);
	int Inc(unsigned int aIndex, long aValue);
	int Dec(unsigned int aIndex, mpz_t aValue);
	int Dec(unsigned int aIndex, long aValue);
	int ComputeMultiplicationMatrix();
	int PrintToScreen();
	int Zeroize();
//arithmetical operations
	int Multiply(AlgebraicNumber *aTarget, AlgebraicNumber* aOperand,bool aComputeMatrix);
	int Add(AlgebraicNumber *aTarget, AlgebraicNumber* aOperand,bool aComputeMatrix);
	AlgebraicNumber *Multiply(AlgebraicNumber *aOperand,bool aComputeMatrix);
	AlgebraicNumber *Add(AlgebraicNumber *aOperand,bool aComputeMatrix);
	int Multiply(AlgebraicNumber *aTarget, long aScalar,bool aComputeMatrix);
	int Divide(AlgebraicNumber *aTarget, long aScalar,bool aComputeMatrix);
	int Multiply(AlgebraicNumber *aTarget, mpz_t aScalar,bool aComputeMatrix);
	int Divide(AlgebraicNumber *aTarget, mpz_t aScalar,bool aComputeMatrix);
	int Power(AlgebraicNumber *aTarget, unsigned int aPower);
//field operations
	int Norm(mpz_t aNorm, Polynomial *aThetaPolynomial);
	int MinimalPolynomial(Polynomial *aPolynomial);


public:	
	HermiteMatrix *multiplication;
	unsigned int Rank;

	
protected:
	mpz_t *vector;
	short int mult_allocated;
	// just pointers
	HermiteMatrix **ThetaPowers;
	mpz_t *denominator;
};

#endif
