#ifndef __CANDIDATE_PAIRS_H__
#define __CANDIDATE_PAIRS_H__

#include "polynom_class.h"
#include "xml_helper.h"

/*!
\brief  An enveloping class for the candidate polynomials pairs
*/
class CCandidatePairs : public virtual MSerializable
{
protected:
	long allocated;		///<The number of allocated entries
	long count;			///<The number of used entries

public:
	Polynomial **FirstPoly;		///<The candidate polynomials - first in the pair
	Polynomial **SecondPoly;	///<The candidate polynomials - second in the pair
	mpz_t *Root;				///<The roots of the candidate polynomials pair
	mpf_t *Alpha;				///<The alphas of the candidate polynomials pair
	mpf_t *Rating;				///<The ratings of the candidate polynomials pair
	mpf_t *Skewness;			///<The skewness of the candidate polynomials pair

protected:
	int Allocate();
	int Deallocate();

public:
	CCandidatePairs(long aAllocation);
	virtual ~CCandidatePairs();

	///This method add new candidate pair to the field, only if it is better than existing pairs
	int AddCandidatePair(Polynomial * aFirstPoly, Polynomial * aSecondPoly, mpz_t aRoot, mpf_t aAlpha, mpf_t aRating, mpf_t aSkewness);
	///Copy all allocated polynomial pairs and values to given candidate pairs object
	int Copy2(CCandidatePairs * aTarget);

	// De/Serialization methods
	int Serialize(const CBaseParameters& aParam) const;
	int Serialize(const CBaseParameters& aParam, xmlTextWriterPtr& aWriter) const;

	int Deserialize(const CBaseParameters& aParam);
	int Deserialize(const CBaseParameters& aParam, xmlTextReaderPtr& aReader);


	// Getters and Setters
public:
	int Get_Count() const { return this->count; }

};

#endif // __CANDIDATE_PAIRS_H__
