#ifndef __CLIQUE_COMPONENT_ARRAY_H__
#define __CLIQUE_COMPONENT_ARRAY_H__

#include "structures.h"

/*!
 * \brief Components of prime ideals - relations graph
 */
class CCliqueComponentArray
{
public:

	int	Count;
	int	AllocatedSize;
	clique_component_t * Components;

public:

	CCliqueComponentArray();
	~CCliqueComponentArray();

	///Deletes array of components 
	void Reset();
	///Allocates array of components
	int Setup(int aAllocatedSize);
	///Resets counter
	void Clear()
	{
		this->Count = 0;
	}

	/*
		Creates new component and adds all prime ideals to this component.
		We just encrease count of components and set prime ideals index to this old count.
	*/
	int AddNewComponent(int aRelationWeight, fhashtable_entry ** aPrimeIdeals, int aPrimeIdealsCount);

	/*
		Returns index of the main component as aMainComponentIndex.
		Components can be connected, this method find the main component index.
	*/
	int GetMainComponent(relation_index_type aComponentIndex, relation_index_type& aMainComponentIndex) const;

	/*
		Connects all components which contains given prime ideals to the main component.
		The main component must be really main (component ID = component index).
		This means to set ID of given components to the main component and recalculates weight of main component.
		If some prime ideals are not in any component they will be added right to the main component.
	*/
	int ConnectComponents(relation_index_type aMainComponentIndex, int aRelationWeight, fhashtable_entry ** aPrimeIdeals, int aPrimeIdealsCount);
	
	/*
		Finds the largest component according to weight.
		We look only for components with limited number of relations.
	*/
	int FindLargestComponent(relation_index_type aMaxCountOfRelation, relation_index_type & aComponentSize, relation_index_type & aComponentIndex) const;

	/*
		Set weight to 0 in given index.
	*/
	int ResetWeight(relation_index_type aComponentIndex);


private:

	CCliqueComponentArray(const CCliqueComponentArray& aOperand) {}

};

#endif // __CLIQUE_COMPONENT_ARRAY_H__
