#ifndef __CONST_ELEMENT_FLAGS_H_
#define __CONST_ELEMENT_FLAGS_H_

#include "types.h"

/*!
 * \brief Class of element flags
*/
class ConstEF
{
public:

	// last 4 bits of B1 byte in nfs_sieving_element.flags
	static const unsigned int SPECIAL_PRIME = 0x80000000;
	static const unsigned int DIVIDES_ROOT	= 0x40000000;
	static const unsigned int DIVIDES_L		= 0x20000000;

	// mask for different countings
	static const unsigned int F_VALUATION_MASK		= 0xfffff00;
	static const unsigned int L_VALUATION_MASK		= 0xf0;
	static const unsigned int INERTIAL_DEGREE_MASK	= 0xf;

	///bit shift of p-valuation of the F(a,b) in nfs_sieving_element.flags - for special prime or ideal (p,p)
	static const int F_VALUATION_SHIFT				= 8;
	///bit shift of p-valuation of the leading coeff in nfs_sieving_element.flags
	static const int L_VALUATION_SHIFT				= 4;
	///bit shift of flags in Lattice FB element flagslogp
	static const int LATTICE_FB_FLAGS_SHIFT			= 8;

	// mini flags for factor base types	
	static const log_type SPECIAL_PRIME_MINI_FLAG	= 0x80;
	static const log_type DIVIDES_ROOT_MINI_FLAG	= 0x40;
	static const log_type DIVIDES_L_MINI_FLAG		= 0x20;
	// just for lattice sieving - lattice FB has special element flagslogp so
	static const log_type ONE_SIEVING_MINI_FLAG		= 0x10;
	static const log_type NEG_ONE_SIEVING_MINI_FLAG = 0xef;
	static const log_type LATTICE_SPECIAL_PRIME_MINI_FLAG	= 0x80;
	static const log_type LATTICE_DIVIDES_L_MINI_FLAG		= 0x20;

	///Elements with these flag are not suitable for sieving 
	static const log_type NOT_FOR_SIEVING_MINI_FLAG = SPECIAL_PRIME_MINI_FLAG | DIVIDES_L_MINI_FLAG;
	static const log_type LATTICE_NOT_FOR_SIEVING_MINI_FLAG = LATTICE_SPECIAL_PRIME_MINI_FLAG | LATTICE_DIVIDES_L_MINI_FLAG;

	// mini mask for factor base types	
	static const log_type L_VALUATION_MINI_MASK		= 0xf;
	
	///Maximum for p-valuation of leading coefficient - if p-valuation is bigger then this implementation cannot work properly
	static const int MAX_L_VALUATION				= 15;

};

#endif // __CONST_ELEMENT_FLAGS_H_
