#ifndef __CONST_POLY_SELECTION_PHASE_H_
#define __CONST_POLY_SELECTION_PHASE_H_

#include "enums.h"
#include "const_filenames.h"

/*!
 * \brief Class of constants for poly selection phases
*/
class ConstPSPhase
{
public:
	static const bool	AUTO_SERIALIZABLE;
	static const int	INNER_STATE_COUNT;
	static const int	INFO_STAT_COUNT;
	static const int	INFO_TIME_COUNT;
	static const int	HALF_CIRCLE_SUBINTERVALS_COUNT;
	static const int	MPF_T_PRECISION;
	static const int	MPF_T_HIGH_PRECISION;
	static const long	CANDIDATES_COUNT;
	static const int	ALPHA_PRIME_BOUND;
	static const int	ALPHA_SMALLER_PRIME_BOUND;
	static const int	CONTENT_TRIES;
	static const long	CONTENT_BOUND;
	static const int	CHI_INVERSE;
	static const int	CHI_MIN_LOG;
	static const int	CHI_MAX_LOG;
	static const long	MAX_CANDIDATES_PER_LC;
	static const int	MIN_AD_PER_C;
	static const int	C_BASE; // 8 * 9 * 5 * 7
	static const int	RHO_EXPANSION_LENGTH;
	static const int	MAXIMAL_RHO_VALUE;
	static const int	PRIME_NUMBERS_DIVIDING_LEADING;
	static const int	STEEPEST_DESCENT_LEVEL;
	static const int	SKEWNESS;
	static const int	J0									= 1000;
	static const int	J1;
	static const int	MAX_J_PRIME							= 130;
	static const int	SMALL_PRIME_COUNT					= 96;
	static const int	C_PRIME_COUNT						= 56;
	static const int	MIN_POLY_DEGREE;
	static const int	POLY_DEGREE_STEP; // in binary system
	static const int	SELF_INITIALIZING_DELAY;
	static const int	MONIC_SEARCH_2_MULTIPLY_COUNT;
	static const int	C_CHANGE_COUNT;
	static const int	M_CHANGE_COUNT;

	static const int	QUADRATIC_POLY_DEGREE;
	static const int	PRIME_CHANGE_COUNT;
	static const int	MONTGOMERY_CHI_INVERSE;

	static const int	GENERATED_POLYS_COUNT = 2;

	static const poly_gen	POLY_GEN_TYPE;

	static const double INITIAL_STEEPEST_DESCENT_STEP;
	static const double SKEWED_LEADING_COEFFICIENT_LOW_BOUND;
	static const double SKEWED_LEADING_COEFFICIENT_HIGH_BOUND;
	static const double THIRD_COEFFICIENT_RELATIVE_LENGTH;
	static const double IFS_BOUND;
	static const double IFS_IMPROVEMENT;
	static const double ALPHA_BOUND;
	static const double ALPHA_ABORT;
	static const double ALPHA_STEP;
	static const double ALPHA_STEP_BACK;

	static const double MONTGOMERY_ALPHA_BOUND;
	static const double MONTGOMERY_ALPHA_ABORT;

	static const int	KLEINJUNG_MIN_POLY_DEGREE			= 4;
	static const int	KLEINJUNG_LEADING_COEFF_LOWER_BOUND	= 1;
	static const int	KLEINJUNG_LEADING_COEFF_BASE		= 60; ///< 2 * 2 * 3 * 5
	static const int	KLEINJUNG_DEFAULT_L					= 7; ///<Number of primes in factorization of p, denoted as l, default value is 7
	static const int	KLEINJUNG_POSSIBLE_SUBSETS_PP		= 500; ///<Number of possible subsets of Q_{a_{d}} of size l
	static const int	KLEINJUNG_DEFAULT_PRIME_BOUND		= 1000; ///<Bound for primes in factorization of p, denoted as p_{b}, default value is 1000
	static const int	KLEINJUNG_CHI_INVERSE				= 1;
	static const int	AD_CHANGE_COUNT						= 100; ///<How often will be run serialization - after 100 a_{d} changes

};

#endif // __CONST_POLY_SELECTION_PHASE_H_
