#ifndef __CONST_SIEVING_PHASE_H_
#define __CONST_SIEVING_PHASE_H_

#include "const_poly_selection_phase.h"
#include "types.h"
#include "enums_common.h"
#include "abstract_sieve.h"

/*!
 * \brief Class of constants for sieving phases
*/
class ConstSPhase
{
public:
	static const bool AUTO_SERIALIZABLE;
	static const bool CHECK_ALL_MODE;
	static const int INNER_STATE_COUNT;
	static const int INFO_TIME_COUNT;
	static const int INFO_STAT_COUNT;
	static const int INTEGRAL_ERROR_FACTOR;
	static const int ALGEBRAIC_ERROR_FACTOR;
	static const int EXCEPTED_ALGEBRAIC_LOG_STEP;
	static const int QUAD_CHARS_COUNT;
	static const int RELATION_RESERVE;
	static const int UNLIMITED_ITERATIONS;
	static const int INTERMEDIATE_ROOTS_COUNT;
	static const int MAX_INDEPENDENT_CYCLES_COUNT;
	static const int MIN_PRIME_TO_SIEVE_WITH;
	static const int PARTIAL_FACTOR_MULTIPLIER;
	static const int RELATION_BUFFER_RESERVE;
	static const int MAX_LINE_SIEVE_HALFLINE;
	static const int MAX_FACTOR_BASE_UPPER_BOUND;
	static const int TIME_MESSAGE_INTERVAL;
	static const int MAX_ITERATIONS;
	static const int MAX_RELATION_BUFFER_SIZE;
	static const int THRESHOLD_OPTIMIZER_POLICY;
	static const int THRESHOLD_OPTIMIZER_INT;
	static const int LOGARITHM_BASE; // base for log in sieving int
	static const int NOT_SET_INIT_BOUND_VALUE_CHANGE = 0x80000000;
	static const int HASHTABLE_SIZE = 67108864; // 64MB
	/// if we should use free relations when generating relaions
	static const bool USE_FREE_RELATIONS = true;

	static const double	POLY_SKEWNESS;
	///If we count average sum of primes over a interval, then we want to use only a portion
	static const double PROPAB_PRIMES_SUM_RATIO;

	///Position of bit where mask is saved
	static const unsigned int ELEMENT_MASK_BIT = 0x80000000;
	static const int ELEMENT_MASK_COUNT = 2;
	/*!
		For construction graph of the large primes.
		We need distinguish parts of the relation.
		First = zero bit in the first position = 0x00000000
		Second = one bit in the first position = 0x80000000

		- we support only relations with 2 parts!!!
	*/
	static const unsigned int ELEMENT_MASK[ConstSPhase::ELEMENT_MASK_COUNT];


	// for variations - value of enum for variations types has info about number of large primes 
	static const int VAR_FIRST_MASK		= 0xf;
	static const int VAR_SECOND_MASK	= 0xf0;
	static const int VAR_FIRST_SHIFT	= 0;
	static const int VAR_SECOND_SHIFT	= 4;
	static const int VAR_NEED_FACTOR_MASK	= 0xee; // we will need to use factorization algorithm to factorize remainder
	static const int VAR_FIRST_FACTOR_MASK	= 0xe;  // same as above but only for first part
	static const int VAR_SECOND_FACTOR_MASK	= 0xe0; // same as above but only for second part

	// for lattice FB
	/*!
		b1 value is not exactly save in b1b0 
		- b1 can be I, but never 0
		-- so we can get 2^16 (max) but we have only 16 bits 
		- b0 can be 0, but never I
		--- this was not proved... TODO
	*/
	static const int LATTICE_FB_B1_CHANGE	= 1;
	static const int LATTICE_FB_B1_SHIFT	= 16;
	static const unsigned int LATTICE_FB_B1_MASK	= 0xffff0000;
	static const unsigned int LATTICE_FB_B0_MASK	= 0x0000ffff;
	static const unsigned int LATTICE_FB_LOG_MASK	= 0x000000ff;
	///Max length of sieving interval <= 2^{16} - because of b0 and b1 bounds - so half is 2^{15}
	static const int LATTICE_MAX_HALF_INTERVAL_LENGTH	= 0x0008000;


	static const supporting_factor_alg_types SUPPORTING_FACTOR_ALG;
	static const variations_types VARIATIONS_TYPE;
	static const line_sieve_size LINE_SIEVE_SIZE;
	static const nfs_sieving_direction SIEVING_DIRECTION;

	static const int CANDIDATE_RATE = 20000;
	static const int MIN_CANDIDATE_COUNT = 5;
	static const int CANDIDATE_RATE_DOUBLE_LARGE = 2000;
	static const int FACTOR_AVERAGE_COUNT = 12;
	static const int CANDIDATE_RESIZE_COUNT = 3;
	static const int FACTOR_RESIZE_COUNT = 3;

	static const int OPTIMIZER_MIN_CHANGE = -10;
	static const int OPTIMIZER_MAX_CHANGE = 10;

	static const log_type FACTOR_BLOCK_COLLISION = 0xff;

};

#endif // __CONST_SIEVING_PHASE_H_
