#include "couveignes_root_finder.h"

#include "common_utils.h"
#include <typeinfo>
#include "log.h"

//============================================================

CCouveignesRootFinder::CCouveignesRootFinder() 
{
	mpz_init(iN);
	mpz_init(iThetaImage);
	mpz_init(iLowestPrime);
	mpz_set_ui(iLowestPrime,10000);
	iModPower=15;
}

CCouveignesRootFinder::~CCouveignesRootFinder() 
{
	mpz_clear(iN);
	mpz_clear(iLowestPrime);
	mpz_clear(iThetaImage);
}



//============================================================




int CCouveignesRootFinder::RootImageFind(mpz_t Result){
	mpz_t temp1,temp2,temp3;
	Polynomial *tempP,*tempP2,*tempP3;
	unsigned long i,j;
	int ret=0;

	if(0==(iNumFieldPolynomial->GetDegree()%2))
	{
		CLog::LogError(WHEREARG,(char *)typeid(this).name(),
				"Couveignes method can be used only for number fields of odd degree.",-1,-1,
				"");
		return -1;
	};	

	mpz_init(temp1);
	// 1a) Compute Bound
	ComputeBoundLog(temp1,iLowestPrime);
	i=((unsigned long int) 1)<<(iModPower-1);
	mpz_fdiv_q_ui(temp1,temp1,i);
	if(0==mpz_get_ui(temp1)) mpz_set_ui(temp1,1);
	if (mpz_fits_ulong_p(temp1)==0) {
		CLog::LogError(WHEREARG,(char *)typeid(this).name(),
				"iLowestPrime chosen too low",-1,-1,
				"iModCount does not fit in unsigned long int! (in CouveignesRootFinder)");
		ret = -7;
		mpz_clear(temp1);
		return ret;
	};


	mpz_init(temp2);
	mpz_init(temp3);
	tempP= new Polynomial(1);
	tempP2= new Polynomial(1);
	tempP3= new Polynomial(1);

	iModCount = mpz_get_ui(temp1);
	printf("\n #moduli:%lu;",iModCount);
	fflush(stdout);
	iMods=new mpz_t[iModCount];
	iModsPowered=new mpz_t[iModCount];
	iSqRootsMods=new Polynomial[iModCount];
	iSqRootsModsIterated=new Polynomial[iModCount];
	iGammaMod=new Polynomial[iModCount];
	iEvalMod=new mpz_t[iModCount];

	rem_m_i = new mpz_t[iModCount];
	a_i = new mpz_t[iModCount];
	rem_n = new mpz_t[iModCount];
	mpz_init(iMModN);
	for(i=0;i<iModCount;i++){
	  mpz_init(rem_m_i[i]);
	  mpz_init(rem_n[i]);
	  mpz_init(a_i[i]);
	  mpz_init(iEvalMod[i]);
	};
	
	do{
	// 1b) find suitable primes
	mpz_nextprime(temp1,iLowestPrime);
	i=0;
	while(i<iModCount){
		if (!iNumFieldPolynomial->IsIrreducibleModPrime(mpz_get_ui(temp1)))
		{
			mpz_nextprime(temp1,temp1);
		}
		else {
			mpz_init(iMods[i]);
			mpz_init(iModsPowered[i]);
			mpz_set(iMods[i],temp1);
			mpz_pow_ui(iModsPowered[i],iMods[i],1<<(iModPower-1));
			i++;
			mpz_nextprime(temp1,temp1);
		};
	};
	gmp_printf("\n Modules found:from %Zu to %Zu",iMods[0],iMods[iModCount-1]);
	printf("\nActMod\t\tMul\t\tSh-To\t\tNewIter");
	fflush(stdout);
 	// 3a) compute \gamma_i
	for(i=0;i<iModCount;i++){
		printf("\n%ld\t\t",i);
		iGammaMod[i].Zeroize();
		iGammaMod[i].Set(0,1);
		for(j=0;j<iNumFactors;j++){
		// Squaring mod prime power
			if (0 != tempP->Multiply(tempP,&iGammaMod[i],iFactors[j])){
				CLog::LogError(WHEREARG,(char *)typeid(this).name(),
				"Error in Multiply!",-1,-1,
				"(in CouveignesRootFinder)");
						};
			tempP->ReduceInGF(iNumFieldPolynomial,iModsPowered[i]);
			tempP->Copy(&iGammaMod[i]);
			};
		printf("%ld\t\t",i);
	// 3b) compute square root in finite field and lift using Newton iteration
		iSqRootsModsIterated[i].Zeroize();
		if(0!=(ret = Shanks_Tonelli_GF(&iSqRootsMods[i],&iGammaMod[i], iMods[i]))){
			break;
		};
		printf("%ld\t\t",i);
		if (iModPower!=1)
			SquareRootNewtonIter(&iSqRootsModsIterated[i],&iSqRootsMods[i],&iGammaMod[i],iMods[i],iModPower,false,false,false);	
		else
			iSqRootsMods[i].Copy(&iSqRootsModsIterated[i]);
		printf("%ld\t\t",i);
	};
	if(0!=ret) break;	
	// 2) Compute rem(M_i,m_i), a_i, rem(M_i,n)
	mpz_set_ui(iMModN,1);
	for(j=0;j<iModCount;j++) {
		  mpz_mul(iMModN,iMModN,iModsPowered[j]);
		  mpz_mod(iMModN,iMModN,iN);
	};
	printf(" [2]");
	fflush(stdout);
	for(i=0;i<iModCount;i++){
		printf(" %ld ",i);
		fflush(stdout);
	  mpz_set_ui(rem_m_i[i],1);
	  for(j=0;j<iModCount;j++)
	  if (j!=i)
	  {
		  mpz_mul(rem_m_i[i],rem_m_i[i],iModsPowered[j]);
		  mpz_mod(rem_m_i[i],rem_m_i[i],iModsPowered[i]);
	  }
	  mpz_invert(a_i[i],rem_m_i[i],iModsPowered[i]);
	  mpz_invert(temp1,iModsPowered[i],iN);
	  mpz_set_ui(rem_n[i],1);
	  for(j=0;j<iModCount;j++)
	  if (j!=i)
	  {
		  mpz_mul(rem_n[i],rem_n[i],iModsPowered[j]);
		  mpz_mod(rem_n[i],rem_n[i],iN);
	  }
	};
	// 4) choose square roots using norms
	if (NULL == this->iNormBases){
		CLog::LogError(WHEREARG,(char *)typeid(this).name(),
				"Pointer 'iNorms' not initialized! ",-1,-1,
				"iNorms==NULL");
		ret= -1;
		break;
	}
	for (i = 0; i < iModCount; i++)
	{
		mpz_set_ui(temp1, 1);
		for (unsigned int factor = 0; factor < iNumNormFactors; factor++)
		{
			mpz_powm_ui(temp2, iNormBases[factor], iNormExponents[factor], iMods[i]);
			mpz_mul(temp1,temp1,temp2);
			mpz_mod(temp1,temp1,iMods[i]);
		}
		mpz_pow_ui(temp2,iMods[i],iNumFieldPolynomial->GetDegree());
		mpz_sub_ui(temp2,temp2,1);
		mpz_sub_ui(temp3,iMods[i],1);
		mpz_fdiv_q(temp3,temp2,temp3);
		iSqRootsModsIterated[i].PowerInGF(tempP,temp3,iNumFieldPolynomial,iMods[i]);
		mpz_mod(temp3,(*tempP->Get2(0)),iMods[i]);
		if(0!=mpz_cmp(temp3,temp1))
		{
			tempP->Multiply(tempP,&iSqRootsModsIterated[i],-1);
			tempP->Copy(&iSqRootsModsIterated[i]);
		};
		iSqRootsModsIterated[i].Evaluate(iEvalMod[i],iThetaImage);
		mpz_mod(iEvalMod[i],iEvalMod[i],iModsPowered[i]);
	}
	// 5) compute B(m)
	mpz_set(temp1,iModsPowered[iModCount-1]);
	i=0;
	while(0!=mpz_cmp_ui(temp1,0)){
		mpz_fdiv_q_ui(temp1,temp1,2);
		i++;
	};
//	mpf_set_default_prec(2*i);	
	mpf_t tef,tef2,tef3;
	mpf_init(tef);
	mpf_init(tef2);
	mpf_init(tef3);
	mpf_set_ui(tef,0);
	mpz_set_ui(temp3,0);
	for(i=0;i<iModCount;i++){
		mpz_mul(temp1,iEvalMod[i],a_i[i]);
		mpz_fdiv_qr(temp2,temp1,temp1,iModsPowered[i]);
		mpz_add(temp3,temp2,temp3);
		mpf_set_z(tef2,temp1);
		mpf_set_z(tef3,iModsPowered[i]);
		mpf_div(tef2,tef2,tef3);
		mpf_add(tef,tef,tef2);
	};
	mpf_set_ui(tef3,1);
	mpf_set_ui(tef2,2);
	mpf_div(tef2,tef3,tef2);
	mpf_add(tef,tef,tef2);
	mpz_set_f(temp1,tef);
	mpz_add(temp1,temp1,temp3);
	mpf_clear(tef);
	mpf_clear(tef2);
	mpf_clear(tef3);

	mpz_set_ui(temp2,0);
	for(i=0;i<iModCount;i++){
		mpz_mul(temp3,a_i[i],rem_n[i]);
		mpz_mul(temp3,temp3,iEvalMod[i]);
		mpz_add(temp2,temp3,temp2);
		mpz_mod(temp2,temp2,iN);
	};
	mpz_mul(temp3,temp1,iMModN);
	mpz_sub(temp3,temp2,temp3);
	mpz_mod(Result,temp3,iN);



	} while (false);

	for(i=0;i<iModCount;i++){
		mpz_clear(iModsPowered[i]);
	};
	for(i=0;i<iModCount;i++){
		mpz_clear(rem_m_i[i]);
	};
	for(i=0;i<iModCount;i++){
		mpz_clear(rem_n[i]);
	};
	for(i=0;i<iModCount;i++){
		mpz_clear(iMods[i]);
	};
	for(i=0;i<iModCount;i++){
		mpz_clear(a_i[i]);
	};
		for(i=0;i<iModCount;i++){
		mpz_clear(iEvalMod[i]);
	};
	delete [] iGammaMod;
	delete [] iSqRootsMods;
	delete [] iSqRootsModsIterated;
	delete [] iMods;
	delete [] iModsPowered;
	delete [] iEvalMod;
	delete [] rem_n;
	delete [] a_i;
	delete [] rem_m_i;
	mpz_clear(temp1);
	mpz_clear(temp2);
	mpz_clear(temp3);
	mpz_clear(iMModN);
	delete tempP;
	delete tempP2;
	delete tempP3;

	return ret;
};


int CCouveignesRootFinder::SetNorms(mpz_t *aBases, int * aExponents, long aNumNormFactors)
{
	if (aBases != NULL && aExponents != NULL)
	{
		this->iNormBases = aBases;
		this->iNormExponents = aExponents;
		this->iNumNormFactors = aNumNormFactors;
		return 0;
	}
	else
	{
		CLog::LogError(WHEREARG,(char *)typeid(this).name(),
				"NULL pointer given.",-1,-1,
				"NULL pointer given to SetNorms().");
		return -1;
	}
}


int CCouveignesRootFinder::SetHomomorphism(mpz_t aThetaImage, mpz_t aN){
	mpz_set(iThetaImage,aThetaImage);
	mpz_set(iN,aN);
	return(0);
};



int CCouveignesRootFinder::SetModPower(unsigned long int aModPower){
	iModPower=aModPower;
	return(0);
};



int CCouveignesRootFinder::CRT(mpz_t Result,unsigned int aNum_moduli,mpz_t *aValues,mpz_t *aModuli){
	unsigned int i;
	mpz_t Modul;
	mpz_init(Modul);
	mpz_t temp1,temp2,temp3;
	mpz_init(temp1);
	mpz_init(temp2);
	mpz_init(temp3);

	mpz_set_ui(Modul,1);
	for(i=0;i<aNum_moduli;i++)
		mpz_mul(Modul,Modul,aModuli[i]);

	mpz_set_ui(Result,0);
	for(i=0;i<aNum_moduli;i++)
	{
		mpz_fdiv_q(temp1,Modul,aModuli[i]);
		mpz_invert(temp2,temp1,aModuli[i]);
		mpz_mul(temp3,temp2,aValues[i]);
		mpz_mul(temp3,temp3,temp1);
		mpz_add(Result,Result,temp3);
		mpz_mod(Result,Result,Modul);
	};

	for(i=0;i<aNum_moduli;i++)
	{
		mpz_mod(temp1,Result,aModuli[i]);
		if(mpz_cmp(temp1,aValues[i])!=0)
			printf("CRT:error");
	};
	mpz_clear(temp1);
	mpz_clear(temp2);
	mpz_clear(temp3);
	mpz_clear(Modul);

	// Added by M.Kechlibar, Nov 6, 2007. Feel free to correct
	return ConstRC::Ok;
};



















