#ifndef __COUVEIGNES_ROOT_FINDER_H__
#define __COUVEIGNES_ROOT_FINDER_H__

#include "newton_root_finder.h"

/*!
 * \brief Compute square root using ideas developed by Jean-Marc Couveignes

 * see Couveignes, J.-M.:
 * Computing A Square Root For The Number Field Sieve (1993)
 *
 */
class CCouveignesRootFinder: public CNewtonRootFinder 
{
public:
	CCouveignesRootFinder();
	~CCouveignesRootFinder();
	
	/// Main method, actual computation
	int RootImageFind(mpz_t aResult);
	/// Norm of square root should be given as product...
	int SetNorms(mpz_t *aBases, int * aExponents, long aNumNormFactors);
	/// Sets homomorphism given image of theta and target ring
	int SetHomomorphism(mpz_t aThetaImage, mpz_t aN);
	/// Set number desired of Newton iterations 
	int SetModPower(unsigned long int aModPower);

private:
	/// Chinese Remainder Theorem (only for testing purposes)
	int CRT(mpz_t Result,unsigned int aNum_moduli,mpz_t *aValues,mpz_t *aModuli);

	mpz_t iLowestPrime;
	/// Number of needed moduli
	unsigned long int iModCount; 
	/// Number of iterations
	unsigned long int iModPower;
	/// prime bases of moduli
	mpz_t *iMods;
	/// moduli -- powered primes
	mpz_t *iModsPowered;
	/// bases of norm of square root
	mpz_t *iNormBases;
	/// exponents of base - norm = base^{exponent}
	int  *iNormExponents;
	/// number of norm factors
	long iNumNormFactors;
	mpz_t iMModN;
	mpz_t *rem_m_i, *rem_n, *a_i;
	Polynomial * iSqRootsMods;
	Polynomial * iSqRootsModsIterated;
	Polynomial * iGammaMod;
	mpz_t * iEvalMod;
	/// Number to be factorized
	mpz_t iN; 
	/// image of theta
	mpz_t iThetaImage;
};

#endif // __COUVEIGNES_ROOT_FINDER_H__