#ifndef __ENUMS_H_
#define __ENUMS_H_

enum cleaning_types { ECleanNothing = 0, ECleanAll, ECleanAllIfNotError, ECleanAllExcCF, ECleanAllExcCFRF }; 

/*!
 Phases of NFS
*/
enum nfs_phase { EMain = 0, EPolySelection, ESieving, ERelProcessing, ELinear, ESquareRoot };

/*!
 Internal states of polynomial selection phase
*/
enum poly_sel_phase_states { EPSPhaseNotStarted = 0, EPSPhaseInitialization, EPSPhaseSearchingForCandidates, EPSPhaseChoosingFromCandidates, EPSPhaseSavingResult, EPSPhaseFinished };
/*!
 Type of polynomial generation mathods for NFS
*/
enum poly_gen { EGenerateMonicPolynomial, EGenerateNonmonicPolynomial, EGenerateSkewedPolynomial };
/*!
 Statistic Info used in polynomial selection phase
*/
enum poly_sel_stats { EPSSTested = 0, EPSSRejectedCoeff, EPSSRejectedAlpha};
/*!
 Info Times used in polynomial selection phase
*/
enum poly_sel_times { EPSTGeneratePoly = 0, EPSTCompC, EPSTCompAlpha, EPSTChooseBest };


/*!
 Internal states of sieving phase
*/
enum sieving_phase_states { ESPhaseNotStarted = 0, ESPhaseInitialization, ESPhaseInitFB, ESPhaseInitAdvFB, ESPhaseSieving, ESPhaseSavingResult, ESPhaseFinished };
/*!
 Info Times used in sieving phase
*/
enum sieving_times { ESTRunSieve = 0, ESTInitFB, ESTLineSieving, ESTFirstFillHT, ESTSecondFillHT, ESTRootUpdate, ESTBlockSieving, ESTFirstSieving, ESTSecondSieving, ESTFirstFactorization, ESTSecondFactorization, ESTRSerial, ESTPartialFactorization };
/*!
 Statistic Info used in sieving phase
*/
enum sieving_stats { ESSTested = 0, ESSNotCoprime, ESSRejected, ESSRemainderTooLarge, ESSPrimeTooLarge, ESSFactorTooLarge, ESSAccepted, ESSFF, ESSPartialR, ESSFP, ESSPF, ESSPP, ESSEdges, ESSComponents, ESSVertices, ESSAverageSizeIP_log2, ESSAverageSizeAP_log2, ESSAverageSizeIP_log10, ESSAverageSizeAP_log10, ESSDivrem };
/*!
 Statistic Info used in sieving phase
*/
enum sieving_stats_adv { ESSSmooth = ESSFF, ESS1Partial = ESSFP, ESS2Partial = ESSPF, ESS3Partial = ESSPP, ESS4Partial = ESSEdges, ESSXPartial = ESSComponents};
/*!
 Sieving directions using in NFS
*/
enum nfs_sieving_direction { EIncrementB = 0, EDecrementB, EIncrementA, EDecrementA };

/*!
 Internal states of relation processing phase 
*/
enum rel_pro_phase_states { ERPPhaseNotStarted = 0, ERPPhaseInitialization, ERPPhaseRemovingLargeSingletons, ERPPhaseConstructingCycles, ERPPhaseRemovingSmallSingletons, ERPPhaseCreatingRelMatrix, ERPPhaseSavingResult, ERPPhaseFinished };

/*!
 Internal states of filtering phase 
*/
enum filtering_phase_states { EFPhaseNotStarted = 0, EFPhaseInitialization, EFPhaseJoiningFiles, EFPhaseRemovingDuplicates, EFPhaseRemovingSingletons, EFPhaseRemovingRedundancy, EFPhaseMergingRelations, EFPhaseCreatingRelMatrix, EFPhaseSavingResult, EFPhaseFinished };


/*!
 Internal states of linear phase 
*/
enum linear_phase_states { ELPhaseNotStarted = 0, ELPhaseInitialization, ELPhaseSolvingSystem, ELPhaseSavingResult, ELPhaseFinished };

/*!
 Internal states of square root phase 
*/
enum sq_root_phase_states { ESRPhaseNotStarted = 0, ESRPhaseInitialization, ESRPhaseTryingDependencies, ESRPhaseSavingResult, ESRPhaseFinished };
/*!
 Statistic Info used in square root phase
*/
enum sq_root_stats { ESRSSmooth = 0, ESRSPartial, ESRSFree, ESRSNormalPrime, ESRSSpecialPrime};
/*!
 Info Times used in square root phase
*/
enum sq_root_times { ESRTFirstRoot = 0, ESRTSecondRoot, ESRTThirdRoot, ESRTFourthRoot, ESRTLoad };

/*!
 Types of inner states of interruptible objects
*/
enum interruptible_states { ENotRunning = 0, EShortRun, EMediumRun, ELongRun };

/*!
 Type of phases in factorization algorithm
*/
enum phase_types { ENFSClassicPolySel = 0, ENFSMontgomeryPolySel = 1, ENFSKleinjungPolySel = 2, ENFSKleinjung2PolySel = 3, ENFSMurphyPolySel = 4, ENFSClassicalSieving = 10, ENFSLatticeSieving = 15, ENFSRelProcessing = 20, ENFSFiltering = 25, ENFSGeneralLinear = 30, ENFSSqrRoot = 40 };

/*!
 Types of square root finders
*/
enum root_finder_types { ENewton = 0, ECouveignes };

enum relation_types { ERelAbstract = 0, ERelNfs, ERelMpqs, ERelTwoAlg};


#endif // __ENUMS_H_

