#ifndef __FACTOR_BASE_INFO_H__
#define __FACTOR_BASE_INFO_H__

#include "complex_structures.h"
#include "xml_helper.h"

/*!
 * \brief	This is general factor base reprezentation.
*/
class CFBInfo : public virtual MSerializable
{
private:
	mpz_t iAuxMpz1;
	mpz_t iAuxMpz2;
	mpz_t iAuxMpz3;

	// CONSTANTS
	///Expected number of special ideals - reserve for vector
	static const int DEFAULT_SPECIAL_IDEALS_COUNT	= 20;
	///For algebraic FB reallocation - we don't know how big it should be
	static const int DEFAULT_PRIME_IDEALS_REALLOC	= 5000;
	///Expected number of primes with zero root - reserve for vector
	static const int DEFAULT_ROOT_ZERO_PRIMES_COUNT	= 30;

public:
	///Max index in Integral FB
	long FBMaxIndex;
	///Max allocated index in Integral FB
	long FBMaxAllocatedIndex;
	///Factor base
	fb_element * FactorBase;
	///Full filename with FB
	std::string FBFullFileName;
	///Upper bound - all primes in FB have to be smaller
	long UpperFBBound;

	bool AssertionMode;
	bool InfoMode;

public:
	CFBInfo();
	~CFBInfo();

	int Dispose();
	int Reset();
	int InitFB();
	int FillFB(CNumberFieldInfo & aNFInfo);
	int FillAuxFields(CNumberFieldInfo & aNFInfo);
	int RefillFB(const long &aNewBound, CNumberFieldInfo & aNFInfo);
	int CleanFB(const long * aDetectionField);

	long GetMaxPrime() const;
	long FindIndexInFB(nfs_sieving_element * aElement) const;
	int PrintToScreen() const;
	int PrintLCPrimes() const;

	// De/Serialization methods
	int Serialize(const CBaseParameters& aParam) const;
	int Serialize(const CBaseParameters& aParam, xmlTextWriterPtr& aWriter) const;

	int Deserialize(const CBaseParameters& aParam);
	int Deserialize(const CBaseParameters& aParam, xmlTextReaderPtr& aReader);

	int DeserializeFBInfo(const CBaseParameters & aParam);

private:
	CFBInfo(const CFBInfo& aOperand) {}

	int AddPrimeIdealToFB(main_sieving_type aPrime, log_type aLog, main_sieving_type aRoot, unsigned int aNext, bool aSpecial);
	int FillIntegralFB(CNumberFieldInfo & aNFInfo);
	int FillAlgebraicFB(CNumberFieldInfo & aNFInfo);
	int FillRootZeroPrimes(std::vector<long> & aRZeros_Indices) const;
	int FillSpecialIdeals(IntegralDomain * aDomain, std::vector<PrimeIdeal*> & aSpecialIdeals) const;
};


#endif // __FACTOR_BASE_INFO_H__
