#ifndef __FACTORIZATION_INFO_H__
#define __FACTORIZATION_INFO_H__

#include "types.h"
#include "nfs_relation.h"


/*!
 * \brief Helper class for factorization. 
*/
class CFactorInfo
{
private:
	friend class CFactorizationInfo;
	/*
		First number is current max index.
		Then it is factorization of the candidate.
	*/
	main_sieving_type*	iPrimes;
	/*
		First number is offset of the candidate.
		Then it is corresponding root for the prime in the same index in field Primes
	*/
	main_sieving_type*	iRoots;

public:
	
	/*
		Max number of primes in the factorization.
	*/
	int					Length;

private:
	CFactorInfo();
	~CFactorInfo();

	void Reset();
	void Clear(int aCount);
	int Init(int aCount, int aLength);

	/*
		Resize number of candidates - copy offset from iRoots
	*/
	int ResizeCandidates(int aCountOld, int aCountNew);

public:
	///WARNING without control
	int * Get_Primes(int aInnerIndex) {	return this->iPrimes + aInnerIndex * this->Length; }
	///WARNING without control
	int * Get_Roots(int aInnerIndex)  { return this->iRoots + aInnerIndex * this->Length;  }
	///NOTE: Very slow if you use this method
	int SaveFactor(int aCount, int aIndex, int aPrime, int aRoot);
	/*
		Resize number of possible factor in the candidate factorization - copy all fields
	*/
	int ResizeFactorization(int aCount);

private:
	CFactorInfo(const CFactorInfo & aOperand) {}
};

/*!
 * \brief Helper class for factorization. 
*/
class CFactorizationInfo
{
private:

	///Number of currently used elements
	int					iCurrentCount;
	int					iRelationMode;

	CFactorInfo			iFactors[CNFSRelation::DEFAULT_NFS_PARTS_COUNT];

public:
	
	main_sieving_type*	Rows;
	main_sieving_type*	Cols;
	/*
		Count of factorizations of different candidates.
	*/
	int					Count;
	CNFSRelation**		Relations;

public:
	CFactorizationInfo();
	~CFactorizationInfo();

	int * Get_Primes(int aIndex, int aInnerIndex);
	int * Get_Roots(int aIndex, int aInnerIndex);

	int SaveOffset(int aOffset);
	int FindOffset(int aOffset);

	void Reset();
	void Clear();
	int Init(int aBlockSize, int aRelationMode);

	/*
		Resize number of candidates - copy offset from iRoots
	*/
	int ResizeCandidates();
	/*
		Resize number of possible factor in the candidate factorization - copy all fields
	*/
	int ResizeFactorization(int aIndex);

	void ClearRelations();

// Getters And Setters 
	int Get_CurrentCount() const { return this->iCurrentCount; }
	CFactorInfo * Get_FactorInfo(int aIndex) { return this->iFactors + aIndex; }

private:
	CFactorizationInfo(const CFactorizationInfo & aOperand) {}
};

#endif // __FACTORIZATION_INFO_H__
