#ifndef __FILTERING_NFS_RELATION_H__
#define __FILTERING_NFS_RELATION_H__

#include "buffered_file_writer_item.h"
#include "nfs_relation.h"
#include "polynom_class.h"
#include "filtering_relation_part.h"

/*!
 * \brief Relation for filtering phase
 */
class CFNFSRelation : public virtual MSerializable, public virtual CBufferedFileWriterItem
{
private:
// CONSTANTS
	static const int DIVISOR_READING_BUFFER_SIZE = 100000;
	static const int DEFAULT_DIVISORS_ALLOCATE = 32;

	///Relation parts (integral part, algebraic part...) - field of CFRelationPart
	CFRelationPart * iParts;
	///Length of field iParts
	int iPartsCount;
	///Number of relations used for building this relation
	int iNumberOfRelations;

public:
	main_sieving_type	A;
	main_sieving_type	B;
	relation_index_type Index;

public:

	CFNFSRelation(main_sieving_type * aLargePrimeBounds, int aPartCount = CNFSRelation::DEFAULT_NFS_PARTS_COUNT);
	CFNFSRelation(const CFNFSRelation& aOperand);
	virtual ~CFNFSRelation();

	int Reset();
	int Copy2(CFNFSRelation &aRelation);
	int Copy2(CNFSRelation &aRelation);
	int CopyFrom(CNFSRelation &aRelation);

	bool Equals(const CFNFSRelation &aOperand) const;

	int ReadFromFile(FILE* aFr);
	// implements pure virtual method of CBufferedFileWriterItem class
	int PrintToFile(FILE* aFw);
	void PrintToScreen();

	// implements pure virtual method of CBufferedFileWriterItem class
	CFNFSRelation *CopyItem();
	// implements pure virtual method of CBufferedFileWriterItem class
	void FreeItem();

	int	Get_NumberOfRelations() const { return this->iNumberOfRelations; }
	CFRelationPart*	Get_Parts(int aIndex) 
	{
#ifdef DEBUG_RELATION_CONTROL_ACCESS
		if (this->iParts == NULL)
		{
			Utils::ErrorMessage(__LINE__,"CFNFSRelation","Get_Parts",0,0,"filtering_nfs_relation.h","Field of Parts is NULL");		
			return NULL;
		}

		if (aIndex < 0 || this->iPartsCount <= aIndex)
		{
			Utils::ErrorMessage(__LINE__,"CFNFSRelation","Get_Parts",0,0,"filtering_nfs_relation.h","Index out of bounds.");				
			return NULL;
		}
#endif
		return this->iParts + aIndex;
	}

	int Get_HammingWeight();

	// De/Serialization methods
	int Serialize(const CBaseParameters& aParam) const;
	int Serialize(const CBaseParameters& aParam, xmlTextWriterPtr& aWriter) const;

	int Deserialize(const CBaseParameters& aParam);
	int Deserialize(const CBaseParameters& aParam, xmlTextReaderPtr& aReader);

private:

	// De/Serialization methods
	int SerializeParts(xmlTextWriterPtr& aWriter) const;
	int DeserializeParts(xmlTextReaderPtr& aReader);

public:

	/// Combine two relation to the new one (new relation is created)
	static int CombineRelations(CFNFSRelation* aOperand1, CFNFSRelation* aOperand2, int aThetaPolyCount, Polynomial ** aThetaPolys, CFNFSRelation *& aResult);

};

#endif // __FILTERING_NFS_RELATION_H__
