#ifndef __FREQUENCY_HASHTABLE_H__
#define __FREQUENCY_HASHTABLE_H__

#include "structures.h"
#include "filtering_relation_part.h"
#include "clique_component_array.h"
#include <set>

/*!
 * \brief Frequency hashtable for filtering phase
 */
class CFrequencyHashtable
{
public:
	///Internal hashtable for largish prime ideals.
	fhashtable_entry *	Hashtable;
	///Number of the allocated hashtable entries
	relation_index_type	AllocatedSize;
	///Hash mask - hash is large prime AND mask 
	relation_index_type	Mask;
	///Max number of prime ideals in relation with was added to this hashtable
	int					MaxPrimeIdeals;
	///Marked if some prime ideals was added to hashtable
	bool Filled;

public:

	CFrequencyHashtable();
	~CFrequencyHashtable();

	void DeleteHashtable();
	int ClearHashtable();
	int SetupHashtableProb(relation_index_type aAmount);
	int SetupHashtableEx(relation_index_type aAmount);

	int FindSingletonRelations(detection_type * aDetectionField, long& aNewSingletonCount);
	int FindSingletonRelations(detection_type * aDetectionField, relation_index_type& aNewSingletonCount, relation_index_type& aHFPrimeIdeals);
	int AddToTable(CFRelationPart * aPart, unsigned long aRootMask, relation_index_type aIndex);
	int AddToTable(CRelationPart * aPart, unsigned long aRootMask, relation_index_type aIndex);
	int AddToTableCM(CFRelationPart * aPart, unsigned long aRootMask, relation_index_type& aCurrentCount);
	int FindPrimeInHashtable(main_sieving_type aPrime, main_sieving_type aRootFingerprint, bool & aPresent, relation_index_type & aIndex) const;
	int FindPrimeInHashtable(main_sieving_type aPrime, bool & aPresent, relation_index_type & aIndex) const;
	int GetPrimeFromHashtable(nfs_sieving_element * aElement, unsigned int aRootMask, fhashtable_entry *& aEntry) const;

	int CountPrimeIdealsWithFrequency(unsigned int aFrequency, relation_index_type & aCountWithFreq, relation_index_type & aCountAll);
	int CountPrimeIdeals(relation_index_type * aLargePrimeIdealsCount, const unsigned int * aMasks, main_sieving_type * aLargePrimeBounds, int aRelationPartCount, relation_index_type & aPrimeIdealsCount);
	int GetAllPrimeIdealsInComponent(int aComponentID, CCliqueComponentArray * aComponents, std::set<prime_ideal_t, prime_ideal_comp> * aPrimeIdeals);

	static relation_index_type GetSuitableHashtableSizeProb(relation_index_type aAmount);

	bool Get_IsSetup() const { return this->Hashtable != NULL; }

private:

	CFrequencyHashtable(const CFrequencyHashtable& aOperand) {}

	int CollisionResolution(relation_index_type aHash, main_sieving_type aPrime, main_sieving_type aRootFingerprint, relation_index_type & aIndex) const;
	int CollisionResolution(relation_index_type aHash, main_sieving_type aPrime, main_sieving_type aRootFingerprint, fhashtable_entry *& aEntry) const;
	int CollisionResolution(relation_index_type aHash, main_sieving_type aPrime, relation_index_type & aIndex) const;

	int EnlargeHashtable();

};

#endif // __FREQUENCY_HASHTABLE_H__
