#include "generationC.h"

#include <iostream>
#include "const_return_codes.h"
#include "log.h"
#include "const_parameter_names.h"
#include "const_filenames.h"


using namespace std;

const int CGenerateC::iPrimesForC[CGenerateC::PRIME_COUNT_FOR_C] = 
{ 2, 2, 2, 3, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 
59, 61, 67, 71, 73, 79, 83, 89, 97, 101, 103, 107, 109, 113, 127, 131,
137,139,149,151,157,163,167,173,179,181,191,193,197,199,
211};
/*	
	223,227,229,233,239,241,251,257,263,269,271,277,281,283,
293,307,311,313,317,331,337,347,349,353,359,367,373,379,383,389,
397,401,409,419,421,431,433,439,443,449,457,461,463,467,479,487,491,499};
*/

CGenerateC::CGenerateC(): MSerializable(ConstFN::POLY_PHASE)
{
	iBase = SEQUENCE_BASE; 
	iFirstBigger = FIRST_BIGGER; 
	iCountBigger = COUNT_BIGGER; 
	iMaxPrime = MAX_PRIME; 
	mpz_init(iUpperBound);
	mpz_init(iLowerBound);
	mpz_set_si(iUpperBound, UPPER_BOUND);
	mpz_set_si(iLowerBound, LOWER_BOUND); 

	iBegin = true;
	mpz_init_set_si(iResultProduct,iBase);
	iNextPrime = 0;
	iIndex = 0;
	iIndexBigger = 0;
}


CGenerateC::CGenerateC(int aFirstBigger, int aMaxPrime, char* aLowerBound, char* aUpperBound, int aBase, int aCountBigger): MSerializable(ConstFN::POLY_PHASE)
{
	iBase = aBase; 
	iFirstBigger = aFirstBigger; 
	iCountBigger = aCountBigger; 
	iMaxPrime = aMaxPrime; 
	mpz_init(iUpperBound);
	mpz_init(iLowerBound);
	mpz_set_str(iUpperBound, aUpperBound, 10);
	mpz_set_str(iLowerBound, aLowerBound, 10); 

	iBegin = true;
	mpz_init_set_si(iResultProduct,iBase);
	iNextPrime = 0;
	iIndex = 0;
	iIndexBigger = 0;
}


CGenerateC::CGenerateC(int aFirstBigger, int aMaxPrime, mpz_t aLowerBound, mpz_t aUpperBound, int aBase, int aCountBigger): MSerializable(ConstFN::POLY_PHASE)
{
	iBase = aBase; 
	iFirstBigger = aFirstBigger; 
	iCountBigger = aCountBigger; 
	iMaxPrime = aMaxPrime; 
	mpz_init(iUpperBound);
	mpz_init(iLowerBound);
	mpz_set(iUpperBound, aUpperBound);
	mpz_set(iLowerBound, aLowerBound); 

	iBegin = true;
	mpz_init_set_si(iResultProduct,iBase);
	iNextPrime = 0;
	iIndex = 0;
	iIndexBigger = 0;
}


CGenerateC::~CGenerateC()
{
	mpz_clear(iResultProduct);
	mpz_clear(iUpperBound);
	mpz_clear(iLowerBound);
}

/*!
	According to factorization algorithm parameters setups inner parameters. 
	Original values are used if not found in these parameters.
*/
int CGenerateC::Setup(const AFactorAlgParameters * aParameters, int aPhaseNumber)
{
	if (aParameters->ContainsPhaseParameter(ConstPN::CGENERATOR_FIRST_BIGGER, aPhaseNumber))
	{
		aParameters->GetIntegerPhaseParameter(ConstPN::CGENERATOR_FIRST_BIGGER, this->iFirstBigger, aPhaseNumber);
	}

	if (aParameters->ContainsPhaseParameter(ConstPN::CGENERATOR_COUNT_BIGGER, aPhaseNumber))
	{
		aParameters->GetIntegerPhaseParameter(ConstPN::CGENERATOR_COUNT_BIGGER, this->iCountBigger, aPhaseNumber);
	}

	if (aParameters->ContainsPhaseParameter(ConstPN::CGENERATOR_MAX_PRIME, aPhaseNumber))
	{
		aParameters->GetIntegerPhaseParameter(ConstPN::CGENERATOR_MAX_PRIME, this->iMaxPrime, aPhaseNumber);
	}

	if (aParameters->ContainsPhaseParameter(ConstPN::CGENERATOR_BASE, aPhaseNumber))
	{
		aParameters->GetIntegerPhaseParameter(ConstPN::CGENERATOR_BASE, this->iBase, aPhaseNumber);
		mpz_set_si(this->iResultProduct, this->iBase);
	}

	if (aParameters->ContainsPhaseParameter(ConstPN::CGENERATOR_LOWER_BOUND, aPhaseNumber))
	{
		aParameters->GetMpzPhaseParameter(ConstPN::CGENERATOR_LOWER_BOUND, this->iLowerBound, aPhaseNumber);
	}

	if (aParameters->ContainsPhaseParameter(ConstPN::CGENERATOR_UPPER_BOUND, aPhaseNumber))
	{
		aParameters->GetMpzPhaseParameter(ConstPN::CGENERATOR_UPPER_BOUND, this->iUpperBound, aPhaseNumber);
	}

	return ConstRC::Ok;
}

/*!
	the inicialisation of auxilliary variables, the control of the parameters
*/
int CGenerateC::Initialisation()

{
	//the control of the parameters
	if (iBase <= 0)
	{
		CLog::LogError(WHEREARG,"CGenerateC","Inicialisation",0,0,"The parameter for the base of the sequence must be bigger than zero.");
		return ConstRC::BadArgument;
	}
	if (mpz_cmp(iUpperBound,iLowerBound) <= 0)
	{
		CLog::LogError(WHEREARG,"CGenerateC","Inicialisation",0,0,"The parametr for the upper bound must be bigger than the parameter for the lower bound.");
		return ConstRC::BadArgument;
	}
	if (iCountBigger > MAX_COUNT_BIGGER)
	{
		CLog::LogError(WHEREARG,"CGenerateC","Inicialisation",0,0,"The parametr for the count of bigger primes is too big.");
		return ConstRC::BadArgument;
	}
	if (iFirstBigger > PRIME_COUNT_FOR_C)
	{
		CLog::LogError(WHEREARG,"CGenerateC","Inicialisation",0,0,"The parametr for the first bigger prime is too big.");
		return ConstRC::BadArgument;
	}
	if (iFirstBigger > iMaxPrime)
	{
		CLog::LogError(WHEREARG,"CGenerateC","Inicialisation",0,0,"The parametr for the first bigger prime must be smaller than the parametr for the maximal prime.");
		return ConstRC::BadArgument;
	}
	if (iMaxPrime > PRIME_COUNT_FOR_C)
	{
		CLog::LogError(WHEREARG,"CGenerateC","Inicialisation",0,0,"The parametr for the maximal prime is too big.");
		return ConstRC::BadArgument;
	}
	
	iNextPrime = 0;
	iIndex = 0;
		
	iIndexBigger = 0;
	iNextBiggerPrime = iFirstBigger;
	iSmall = false;
	
	//cleaning arrays not necessary
	int i;
	for (i = 0; i < PRIME_COUNT_FOR_C; i++)
	{
		this-> iWorkingSequence[i] = 0;
	}
	for (i = 0; i < MAX_COUNT_BIGGER; i++)
	{
		this-> iWorkingSequenceBigger[i] = 0;
	}
	
	return ConstRC::Ok;

}



/*!
	this method can be more complicated, now there is the comparison of the product with the upper bound only
	aProduct - a parameter saving the multiplication
*/ 
bool CGenerateC::ProductInBounds(mpz_t& aProduct, int aValue)
{
	mpz_t Product;
	mpz_init(Product);
	mpz_mul_si(Product, aProduct, aValue); 
	
	if (mpz_cmp(Product, iUpperBound) < 0) //(Product < iUpperBound) 
	{
		mpz_set(aProduct, Product);
		mpz_clear(Product);
		return true;
	}
	else
	{
		mpz_clear(Product);
		return false;
	} 
	
}

/*!
	this method adds small primes into the sequence; at the end, the resulting product is lower than the upper bound but we cannot add any more small primes
	usage: when the last small prime is increased or at the begining of the generation
*/ 
int CGenerateC::CompleteToMax()
{
	
	while (iNextPrime < iFirstBigger && ProductInBounds(iResultProduct,iPrimesForC[iNextPrime]))
	{
		iWorkingSequence[iIndex] = iNextPrime;
		iNextPrime++;
		iIndex++;
		if (iNextPrime == iFirstBigger) //the end of lower primes
		{
			break;
		}

	} //while 
	
	return ConstRC::Ok;
}

/*!
	search for the next prime with a different value in iPrimesForC - the problem of repeated primes in the array
*/
int CGenerateC::NextDifferentPrime(int aIndex)
{
	int value = iPrimesForC[aIndex];
	int index = aIndex + 1;
	
	while (value == iPrimesForC[index])
	{
		index++;
	}
	return index;
}

/*!
	main method of the class, get the next product (next sequence)
*/
int CGenerateC::NextSequence(mpz_t& aResult) 
{
	bool found = false;
	int rc;
	
	if (iBegin) //beginning of the generating sequences
	{
		rc = Initialisation();

		if (rc != ConstRC::Ok)
		{
			return rc;
		}
		
		mpz_mul_si(iResultProduct, iResultProduct, iPrimesForC[iFirstBigger]);
		iWorkingSequenceBigger[0] = iFirstBigger;
		iNextBiggerPrime++;
		iIndexBigger++;
		CompleteToMax();
		iBegin = false;
		mpz_set(aResult, iResultProduct);
		return ConstRC::Ok;
	}
	
	if (iIndex == 0) // end of the generation, beginning of the array of small primes
	{
		
		return ConstRC::NotFound;
	}	
	
	do //now we try to transform the sequence with one of the following possibilities
	{
		if (iIndexBigger < iCountBigger && iNextBiggerPrime <= iMaxPrime && ProductInBounds(iResultProduct,iPrimesForC[iNextBiggerPrime]))
		{
			//1) adding one bigger prime at the end
				
		   iWorkingSequenceBigger[iIndexBigger] = iNextBiggerPrime;
		   iIndexBigger++;
		   iNextBiggerPrime++;
		}
		else
		{
			mpz_divexact_ui(iResultProduct, iResultProduct, iPrimesForC[iWorkingSequenceBigger[iIndexBigger-1]]);
			//iResultProduct = iResultProduct / iPrimesForC[iWorkingSequenceBigger[iIndexBigger-1]];	
			if (iNextBiggerPrime <= iMaxPrime && ProductInBounds(iResultProduct,iPrimesForC[iNextBiggerPrime]))
			{
				//2) increase the last bigger one
				iWorkingSequenceBigger[iIndexBigger-1] = iNextBiggerPrime;
				iNextBiggerPrime++;
			}
			else
			{	
				found = false;
				do  //when the number of bigger primes is more then 2
				{	
					iIndexBigger--;
					if (iIndexBigger > 0)
					{	
						mpz_divexact_ui(iResultProduct, iResultProduct, iPrimesForC[iWorkingSequenceBigger[iIndexBigger-1]]);
						//iResultProduct = iResultProduct / iPrimesForC[iWorkingSequenceBigger[iIndexBigger - 1]];
						iNextBiggerPrime = iWorkingSequenceBigger[iIndexBigger - 1] + 1;					
					}	
				
				
					if (iIndexBigger > 0 && iNextBiggerPrime <= iMaxPrime && ProductInBounds(iResultProduct,iPrimesForC[iNextBiggerPrime]))
					{
					  //3) delete the last bigger prime and increase the preceding bigger one
				
						iWorkingSequenceBigger[iIndexBigger-1] = iNextBiggerPrime;
						iNextBiggerPrime++;
						found = true; //the product can be small hence we need continue in the main while cyklus
					}
				} while (iIndexBigger > 0 && !found);	
				
				if (!found) //the last while-cyklus has found nothing = there aren´t possibilities to transform the sequence of bigger primes, we try to transform the small primes
				{	
					
					iWorkingSequenceBigger[0] = iFirstBigger;
					iNextBiggerPrime = iFirstBigger + 1;
					iIndexBigger = 1;
					mpz_divexact_ui(iResultProduct, iResultProduct, iPrimesForC[iWorkingSequence[iIndex-1]]);
					//iResultProduct = iResultProduct / iPrimesForC[iWorkingSequence[iIndex-1]];
					if (iNextPrime < iFirstBigger && ProductInBounds(iResultProduct,iPrimesForC[iNextPrime]*iPrimesForC[iFirstBigger]))
					{
						//4) increase the last small prime, add only the first bigger prime, then add small primes up to the upper bound 
						iWorkingSequence[iIndex-1] = iNextPrime;
						iNextPrime++;
						CompleteToMax();
					}	
					else
					{
						//5) delete the last small one and add only first bigger prime
						iIndex--;
						if (iIndex == 0) //the end of the generation
						{
							return ConstRC::NotFound;
						}
					
						//iResultProduct = iResultProduct * iPrimesForC[iFirstBigger];
						mpz_mul_si(iResultProduct, iResultProduct, iPrimesForC[iFirstBigger]);
						if (iIndex > 0)
						{
							iNextPrime = NextDifferentPrime(iWorkingSequence[iIndex-1]); //skipping the equal primes
						}	
						if (mpz_cmp(iResultProduct, iLowerBound) < 0) //(iResultProduct < iLowerBound)
						{
							if (iSmall) //since the last redution there wasn´t result > lower bound - end of the generation
							{
								return ConstRC::NotFound;
							}
							iSmall = true;
						}		
					}
				}	
			}	
		} // if
	
	} while (mpz_cmp(iResultProduct, iLowerBound) < 0); //(iResultProduct < iLowerBound); //if resulting product is small, we try the next sequences
	iSmall = false;
	mpz_set(aResult, iResultProduct);
	return ConstRC::Ok;
}	

/*!
	for testing only - the method prints all the sequences with the products
*/
void CGenerateC::PrintAllResults()
{
	//#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	//print parameters
	CLog::Log(GENER_C_LOG_PRIORITY, "1081000", string("The upper bound = ")+GetUpperBound()+string("\n"));
	CLog::Log(GENER_C_LOG_PRIORITY, "1081001", string("The lower bound = ")+GetLowerBound()+string("\n"));
	CLog::Log(GENER_C_LOG_PRIORITY, "1081002", "The maximal prime = %d\n",GetMaxPrime());
	CLog::Log(GENER_C_LOG_PRIORITY, "1081003", "The first bigger prime = %d\n",GetFirstBiggerPrime());
	CLog::Log(GENER_C_LOG_PRIORITY, "1081004", "The number of bigger primes = %d\n",GetCountBiggerPrimes());
	CLog::Log(GENER_C_LOG_PRIORITY, "1081005", "The base = %d\n\n",GetBase());

	int Value;
	mpz_t Result;
	mpz_init(Result);
	
	int index = 1;

	do
	{	
		Value = NextSequence(Result);

		if (Value == ConstRC::Ok)
		{		
			CLog::Log(GENER_C_LOG_PRIORITY, "1081006","%d)  ",index);
			PrintSequence();
			index++;
		}	
	}	
	while (Value == ConstRC::Ok); 	
	mpz_clear(Result);
//#endif
}

/*!
	for testing only - prints the numbers in the actual sequence with the product of this numbers
*/
void CGenerateC::PrintSequence()
{
	
	int i;

	CLog::Log(GENER_C_LOG_PRIORITY, "1081006","%d  ",iBase);
			  
	for (i = 0; i < iIndex; i++)
	{
		CLog::Log(GENER_C_LOG_PRIORITY, "1081006","%d  ",iPrimesForC[iWorkingSequence[i]]);
	} 
	
	for (i = 0; i < iIndexBigger; i++)
	{
		CLog::Log(GENER_C_LOG_PRIORITY, "1081006","%d  ",iPrimesForC[iWorkingSequenceBigger[i]]);
	} 	

	CLog::Log(GENER_C_LOG_PRIORITY, "1081006",string(" = ")+mpz_get_str(NULL,10,iResultProduct)+string("\n"));

}

//---------------getters-----------------------

int CGenerateC::GetFirstBiggerPrime()
{
	return iPrimesForC[iFirstBigger];
}

int CGenerateC::GetMaxPrime()
{
	return iPrimesForC[iMaxPrime];
}

int CGenerateC::GetCountBiggerPrimes()
{
	return iCountBigger;
}

int CGenerateC::GetBase()
{
	return iBase;
}

char* CGenerateC::GetLowerBound()
{
	return mpz_get_str(NULL,10,iLowerBound);
}

char* CGenerateC::GetUpperBound()
{
	return mpz_get_str(NULL,10,iUpperBound);
}

void CGenerateC::GetActualSequence(int aSequence[],int& aSize)
{
	int i;
	aSequence[0] = iBase;
	
	for (i = 0; i < iIndex; i++)
	{
		aSequence[i+1] = iPrimesForC[iWorkingSequence[i]];
	}
	
	for (i = 0; i < iIndexBigger; i++)
	{
		aSequence[i+iIndex+1] = iPrimesForC[iWorkingSequenceBigger[i]];
	}
	
	aSize = iIndex + iIndexBigger + 1;
}

//------------serialization-----------------------

int CGenerateC::Serialize(const CBaseParameters &aParam) const
{
	int rc = MSerializable::Serialize(aParam);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	return ConstRC::Ok; 
}
int CGenerateC::Serialize(const CBaseParameters& aParam, xmlTextWriterPtr& aWriter) const
{
	string mainTagName;	
	aParam.GetStringParameter(ConstPN::SERIALIZATION_ELEMENT_NAME, XML_SERIAL_GENERATION_C_NAME, mainTagName);

	int rc = xmlTextWriterStartElement(aWriter, BAD_CAST mainTagName.c_str());
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"CGenerateC","Serialize",rc,0,"Could not write the parameter tag.");
		return ConstRC::GeneralError;
	}
	
	rc = CXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}
//-------------parameters

	rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_FIRST_BIGGER_NAME, this->iFirstBigger);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_COUNT_BIGGER_NAME, this->iCountBigger);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_MAX_PRIME_NAME, this->iMaxPrime);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_BASE_NAME, this->iBase);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeMpz(aWriter, XML_SERIAL_UPPER_BOUND_NAME, this->iUpperBound);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeMpz(aWriter, XML_SERIAL_LOWER_BOUND_NAME, this->iLowerBound);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

//----auxilliary variables
	rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_INDEX_NAME, this->iIndex);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_NEXT_PRIME_NAME, this->iNextPrime);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_INDEX_B_NAME, this->iIndexBigger);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_NEXT_PRIME_B_NAME, this->iNextBiggerPrime);
	if (rc != ConstRC::Ok)
	{ 
		return rc;
	}

	rc = CXMLHelper::SerializeBoolean(aWriter, XML_SERIAL_BEGIN_NAME, this->iBegin); 
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeBoolean(aWriter, XML_SERIAL_SMALL_NAME, this->iSmall);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeMpz(aWriter, XML_SERIAL_RESULT_PRODUCT_NAME, this->iResultProduct);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

//---- array iWorkingSequence
	rc = xmlTextWriterStartElement(aWriter, BAD_CAST XML_SERIAL_SEQUENCE_NAME);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"CGenerateC","Serialize",rc,0,"Could not write the tag.");
		return ConstRC::GeneralError;
	}
	
	rc = CXMLHelper::NewLine(aWriter, FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	int value;
	for (int i = 0; i < iIndex; i++)
	{
		value = iWorkingSequence[i];
		rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_ELEMENT_NAME, value);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	rc = xmlTextWriterEndElement(aWriter); // XML_SERIAL_SEQUENCE_NAME - iWorkingSequence
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"CGenerateC","Serialize",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}

	rc = CXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}
//---- array iWorkingSequenceBigger
	rc = xmlTextWriterStartElement(aWriter, BAD_CAST XML_SERIAL_SEQUENCE_B_NAME);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"CGenerateC","Serialize",rc,0,"Could not write the tag.");
		return ConstRC::GeneralError;
	}
	
	rc = CXMLHelper::NewLine(aWriter, FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	for (int i = 0; i < iIndexBigger; i++)
	{
		value = iWorkingSequenceBigger[i];
		rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_ELEMENT_NAME, value);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	rc = xmlTextWriterEndElement(aWriter); // XML_SERIAL_SEQUENCE_B_NAME - iWorkingSequenceBigger
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"CGenerateC","Serialize",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}

//-------END
	rc = CXMLHelper::NewLine(aWriter, FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = xmlTextWriterEndElement(aWriter); // mainTagName
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"CGenerateC","Serialize",rc,0,"Could not end the parameter tag.");
		return ConstRC::GeneralError;
	}
	rc = CXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}
	
	return ConstRC::Ok; 

}

int CGenerateC::Deserialize(const CBaseParameters& aParam, xmlTextReaderPtr& aReader)
{
	string mainTagName;	
	aParam.GetStringParameter(ConstPN::SERIALIZATION_ELEMENT_NAME, XML_SERIAL_GENERATION_C_NAME, mainTagName);

	int rc;
	xmlChar* name = xmlTextReaderName(aReader);

	if (name == NULL)
	{
		CLog::LogError(WHEREARG,"CGenerateC","Deserialize",0,0,"Deserialization main element name not found.");
		return ConstRC::GeneralError;  
	}

	if (xmlStrEqual(name,BAD_CAST mainTagName.c_str()) == 0)
	{
		CLog::LogError(WHEREARG,"CGenerateC","Deserialize",0,0,"Main element name is wrong.");
		return ConstRC::GeneralError; 
	}

	
	ResetAll(); // delete all data - not neccessary
	
	int depth = xmlTextReaderDepth(aReader);

	while ((rc = CXMLHelper::MoveToNextNode(aReader, depth)) == ConstRC::Ok)
	{
		name = xmlTextReaderName(aReader);
	
		//-------------parameters
		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_FIRST_BIGGER_NAME))
		{
			
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize iFirstBigger...", this->iFirstBigger, false);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}

			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_COUNT_BIGGER_NAME))
		{
			
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize iCountBigger ...", this->iCountBigger, false);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}

			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_MAX_PRIME_NAME))
		{
			
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize iMaxPrime...", this->iMaxPrime, false);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}

			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_BASE_NAME))
		{
			
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize iBase...", this->iBase, false);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}

			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_UPPER_BOUND_NAME))
		{
			
			rc = CXMLHelper::DeserializeMpz(aReader, "Deserialize iUpperBound...", this->iUpperBound, false);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}

			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_LOWER_BOUND_NAME))
		{
			
			rc = CXMLHelper::DeserializeMpz(aReader, "Deserialize iLowerBound...", this->iLowerBound, false);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}

			continue;
		}

		//----auxilliary variables
		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_INDEX_NAME))
		{
			
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize iIndex...", this->iIndex, false);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}

			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_NEXT_PRIME_NAME))
		{
			
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize iNextPrime...", this->iNextPrime, false);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}

			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_INDEX_B_NAME))
		{
			
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize iIndexBigger...", this->iIndexBigger, false);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}

			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_NEXT_PRIME_B_NAME))
		{
			
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize iNextBiggerPrime...", this->iNextBiggerPrime, false);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}

			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_BEGIN_NAME))
		{
			int bool1;
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize iBegin  ...", bool1, false);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			this->iBegin = (bool1==1) ? true : false; 
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_SMALL_NAME))
		{
			int bool2;
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize iSmall...", bool2, false);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			this->iSmall = (bool2==1) ? true : false; 
			continue;
		}
		
		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_RESULT_PRODUCT_NAME))
		{
			
			rc = CXMLHelper::DeserializeMpz(aReader, "Deserialize iResultProduct...", this->iResultProduct, false);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}

			continue;
		}

		//---- array iWorkingSequence
		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_SEQUENCE_B_NAME))
		{
			int elementsDepth = xmlTextReaderDepth(aReader);
			
			for (int i = 0; i < this->iIndexBigger; i++)
			{
				if (CXMLHelper::MoveToNextNode(aReader, elementsDepth) != ConstRC::Ok)
				{
					rc = ConstRC::GeneralError;
					break;
				}
				rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize iWorkingSequenceBigger...", iWorkingSequenceBigger[i], false);
				if (rc != ConstRC::Ok)
				{
					rc = ConstRC::GeneralError;
					break;
				}
				
			}
			continue;
		}

		//---- array iWorkingSequenceBigger
		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_SEQUENCE_NAME))
		{
			int elementsDepth = xmlTextReaderDepth(aReader);
			
			for (int i = 0; i < this->iIndex; i++)
			{
				if (CXMLHelper::MoveToNextNode(aReader, elementsDepth) != ConstRC::Ok)
				{
					rc = ConstRC::GeneralError;
					break;
				}
				rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize iWorkingSequence...", iWorkingSequence[i], false);
				if (rc != ConstRC::Ok)
				{
					rc = ConstRC::GeneralError;
					break;
				}
				
			}
			continue;
		}

	} //while

	if (rc == ConstRC::GeneralError || rc == ConstRC::NotEnoughMemory)
	{
		return rc;
	}
	
	return ConstRC::Ok;

}

int CGenerateC::ResetAll()
{
	iFirstBigger = 0; 
	iMaxPrime = 0;
	iCountBigger = 0;
	mpz_set_si(iUpperBound, 0);
	mpz_set_si(iLowerBound, 0); 
	iIndex = 0;
	iNextPrime = 0;
	iIndexBigger = 0;
	iNextBiggerPrime = 0;
	iBegin = false;
	iSmall = false;
	mpz_set_si(iResultProduct, 0);

	//cleaning arrays 
	int i;
	for (i = 0; i < PRIME_COUNT_FOR_C; i++)
	{
		this-> iWorkingSequence[i] = 0;
	}
	for (i = 0; i < MAX_COUNT_BIGGER; i++)
	{
		this-> iWorkingSequenceBigger[i] = 0;
	}
	return ConstRC::Ok; 
}
