#ifndef __HASHTABLE_H__
#define __HASHTABLE_H__

#include "structures.h"

/*!
 * \brief General hashtable class
 */
class CHashtables
{
private:
	///Security margin for the field of hashtables
	static const int SECURITY_MARGIN = 1;

public:
	///Internal hashtable for sieving with largish prime ideals.
	hashtable_t *	Hashtables;
	///Caches for writing into the internal hashtable.
	cache_t *		Caches;
	///Number of the allocated hashtables
	int				AllocatedSize;
	///Max index of elements
	int				MaxIndex;

public:

	CHashtables();
	~CHashtables();

	int  AddToTable(main_sieving_type aPrime, main_sieving_type aRoot, log_type aLog, main_sieving_type aStart, int aHashtableIndex);
	void DeleteHashtables();
	int  SetupHashtables(int aMaxIndex);
	void ZeroizeLastHashtable();
	int  SwapHashtable(int aIndex);
	int  ClearHashtables();
	int  AddScript(script_t* aLast);
	int  FlushUpdateCache(hashtable_t* aHashtable, cache_t* aCache);
	int  FlushAllUpdateCaches();

	// Print methods
	void PrintHashtables();
	void PrintUpdateInfo(update_t * aInfo);

	// Test methods
	void SearchForPrimeInHashtable(main_sieving_type aPrime, main_sieving_type aRoot);


private:
	int  AllocateHashtables();
	int  AllocateCaches();

	CHashtables(const CHashtables& aOperand) {}

};


#endif // __HASHTABLE_H__
