#include <abstract_factor_alg_parameters.h>
#include <base_parameters.h>
#include <common_definitions.h>
#include <const_filenames.h>
#include <const_parameter_names.h>
#include <const_return_codes.h>
#include <constants.h>
#include <enums.h>
#include <enums_common.h>
#include <gmp.h>
#include <integer_class.h>
#include <kleinjung2_poly_sel_phase.h>
#include <log.h>
#include <map> 
//#include <math.h>
#include <nfs_utils.h>
#include <phase_functor.h>
#include <polynom_class.h>
#include <statistic_container.h>
#include <time_container.h>
#include <xml_helper.h>
//#include "test_suite.h"
#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <string>
#include <vector>

using namespace std;

/*
Initializing all values
*/
CKleinjung2PolySelPhase::CKleinjung2PolySelPhase() : MSerializable(ConstFN::POLY_PHASE)
{
	this->iFoundCandidatePolyCount = 0;
	this->iRSerialFoundCandPolyCount = 0;
	this->iRSerialRunAdChangeCount = 0;
	this->iRSerialRunPolyCount = 0;
	this->iRound = 0;
	this->iRSerialRound = 0;

	for (int i = 0; i < ConstPSPhase::GENERATED_POLYS_COUNT; i++)
	{
		mpf_init_set_d(this->iFBBoundLog[i], 0);
	}

	this->iCandidatePolyPairs = NULL;
	this->iRSerialCandidatePolyPairs = NULL;
	this->iResultFirstPoly = NULL;
	this->iResultSecondPoly = NULL;

	mpz_init(this->iSievingIntervalLength);

	mpz_init(this->iNtilde);
	mpz_init(this->iMRoot0);
	mpz_init(this->iMRoot1);
	mpz_init(this->iMRoot2);
	mpz_init(this->iAuxAdm1);
	mpz_init(this->iAuxBound);
	mpz_init(this->iAuxBound2);
	mpz_init(this->iAuxBoundD);
	mpz_init(this->iAdMax);
	mpz_init(this->iAd);
	mpz_init(this->idAd);
	mpz_init(this->iAdMin);
	mpz_init(this->iRSerialAd);

	mpz_init(this->iAuxMpz1);
	mpz_init(this->iAuxMpz2);
	mpz_init(this->iAuxMpz3);
	mpz_init(this->iAuxMpz4);
	mpz_init(this->iAuxMpz5);

	mpf_init2(this->iSkewness, ConstPSPhase::MPF_T_HIGH_PRECISION);
	mpf_init2(this->iRating, ConstPSPhase::MPF_T_HIGH_PRECISION);
	mpf_init2(this->iAlpha, ConstPSPhase::MPF_T_HIGH_PRECISION);

	mpf_init(this->iAuxMpfSum);
	mpf_init(this->iAuxMpf);
	mpf_init(this->iAUxMpfS1);
	mpf_init(this->iAUxMpfS2);
	mpf_init(this->iAUxMpfA1);
	mpf_init(this->iAUxMpfA2);
	
	this->iPolyDegree = 0;
	this->iRegSize = 0;
	this->iSetPSize = 0;
	this->iSetQSize = 0;
	this->iNModSetPQSize = 0;
	this->iNumberOfPairsPr = 0;
	this->iNumberOfPairsQr = 0;
	this->iNumberOfPairsPi = 0;
	this->iPrimeBound = 0;
	this->iAdBase = 0;
	this->iPreM2 = 0;
	this->iCollision = 0;

	this->iSetP = NULL;
	this->iSetQ = NULL;
	this->iPrimitiveElementsP = NULL;
	this->iPrimitiveElementsQ = NULL;
	this->iNModSetPQ = NULL;
	this->iMatrixPowersP = NULL;
	this->iMatrixPowersQ = NULL;
	this->iRoots = NULL;

	this->iPairSolutionPr.clear();
	this->iPairSolutionQr.clear();
	this->iPairSolutionPi.clear();

	this->iPolyImprovement = NULL;

	Set_AutoSerializable(ConstPSPhase::AUTO_SERIALIZABLE);
	Set_InnerState((int)EPSPhaseNotStarted);

	this->iStats = new CStatContainer(ConstPSPhase::INFO_STAT_COUNT);
	this->iTimes = new CTimeContainer(ConstPSPhase::INFO_TIME_COUNT);

}

/*
Destructor
*/
CKleinjung2PolySelPhase::~CKleinjung2PolySelPhase()
{
	Reset();

	if (this->iStats != NULL)
	{
		delete this->iStats;
		this->iStats = NULL;
	}
	if (this->iTimes != NULL)
	{
		delete this->iTimes;
		this->iTimes = NULL;
	}

	DisposeFunctorField();
	DisposeGMP();
	DisposeMutexes();
	DisposeAcceptedReturnCodes();
}

/*
Reset all values
*/
int CKleinjung2PolySelPhase::Reset()
{
	int i;

	this->iFoundCandidatePolyCount = 0;
	this->iRSerialFoundCandPolyCount = 0;
	this->iRound = 0;
	this->iRSerialRound = 0;

	mpz_set_ui(this->iAuxBound, 0);
	mpz_set_ui(this->iAuxBound2, 0);
	mpz_set_ui(this->iAuxBoundD, 0);
	mpz_set_ui(this->iAdMax, 0);
	mpz_set_ui(this->iAdMin, 0);
	mpz_set_ui(this->iAd, 0);
	mpz_set_ui(this->idAd, 0);
	mpz_set_ui(this->iNtilde, 0);
	mpz_set_ui(this->iMRoot0, 0);
	mpz_set_ui(this->iAuxAdm1, 0);
	mpz_set_ui(this->iMRoot1, 0);
	mpz_set_ui(this->iMRoot2, 0);

	mpz_set_ui(this->iRSerialAd, 0);

	if (this->iSetP != NULL)
	{
		free(this->iSetP);
		this->iSetP = NULL;
	}
	if (this->iSetQ != NULL)
	{
		free(this->iSetQ);
		this->iSetQ = NULL;
	}
	if (this->iPrimitiveElementsP != NULL)
	{
		free(this->iPrimitiveElementsP);
		this->iPrimitiveElementsP = NULL;
	}
	if (this->iPrimitiveElementsQ != NULL)
	{
		free(this->iPrimitiveElementsQ);
		this->iPrimitiveElementsQ = NULL;
	}
	if (this->iNModSetPQ != NULL)
	{
		free(this->iNModSetPQ);
		this->iNModSetPQ = NULL;
	}
	if (this->iRoots != NULL)
	{
		free(this->iRoots);
		this->iRoots = NULL;
	}

	if (this->iMatrixPowersP != NULL)
	{
		for (i = 0; i < this->iSetPSize; i++)
		{
			if (this->iMatrixPowersP[i] != NULL)
			{
				free(this->iMatrixPowersP[i]);
				this->iMatrixPowersP[i] = NULL;
			}
		}
		free(this->iMatrixPowersP);
		this->iMatrixPowersP = NULL;
	}
	if (this->iMatrixPowersQ != NULL)
	{
		for (i = 0; i < this->iSetQSize; i++)
		{
			if (this->iMatrixPowersQ[i] != NULL)
			{
				free(this->iMatrixPowersQ[i]);
				this->iMatrixPowersQ[i] = NULL;
			}
		}
		free(this->iMatrixPowersQ);
		this->iMatrixPowersQ = NULL;
	}

	this->iPolyDegree = 0;
	this->iRegSize = 0;
	this->iSetPSize = 0;
	this->iSetQSize = 0;
	this->iNModSetPQSize = 0;
	this->iNumberOfPairsPr = 0;
	this->iNumberOfPairsQr = 0;
	this->iNumberOfPairsPi = 0;
	this->iPrimeBound = 0;
	this->iAdBase = 0;
	this->iPreM2 = 0;
	this->iCollision = 0;

	this->iPairSolutionPr.clear();
	this->iPairSolutionQr.clear();
	this->iPairSolutionPi.clear();

	for (i = 0; i < ConstPSPhase::GENERATED_POLYS_COUNT; i++)
	{
		mpf_set_d(this->iFBBoundLog[i], 0);
	}

	if (this->iPolyImprovement != NULL)
	{
		delete this->iPolyImprovement;
		this->iPolyImprovement = NULL;
	}

	if (this->iCandidatePolyPairs != NULL)
	{
		delete this->iCandidatePolyPairs;
		this->iCandidatePolyPairs = NULL;
	}

	if (this->iRSerialCandidatePolyPairs != NULL)
	{
		delete this->iRSerialCandidatePolyPairs;
		this->iRSerialCandidatePolyPairs = NULL;
	}

	if (this->iResultFirstPoly != NULL)
	{
		delete this->iResultFirstPoly;
		this->iResultFirstPoly = NULL;
	}

	if (this->iResultSecondPoly != NULL)
	{
		delete this->iResultSecondPoly;
		this->iResultSecondPoly = NULL;
	}

	if (this->iStats != NULL)
	{
		this->iStats->ResetAll();
	}
	if (this->iTimes != NULL)
	{
		this->iTimes->ResetAll();
	}

	APolyPhase::Reset();

	Set_InnerState((int)EPSPhaseNotStarted);
	return ConstRC::Ok;
}

/*
Reset values conencted to selected a_d
*/
int CKleinjung2PolySelPhase::PartialResetAd()
{
	mpz_set_ui(this->iNtilde, 0);
	mpz_set_ui(this->iMRoot0, 0);
	mpz_set_ui(this->iAuxAdm1, 0);

	if (this->iNModSetPQ != NULL)
	{
		free(this->iNModSetPQ);
		this->iNModSetPQ = NULL;
	}

	this->iPairSolutionPr.clear();
	this->iPairSolutionQr.clear();
	this->iPairSolutionPi.clear();
	this->iNumberOfPairsPr = 0;
	this->iNumberOfPairsQr = 0;
	this->iNumberOfPairsPi = 0;

	this->iNModSetPQSize = 0;
	this->iPreM2 = 0;
	this->iCollision = 0;

	return ConstRC::Ok;
}

/*
Called by Destructor: Clear all values
*/
int CKleinjung2PolySelPhase::DisposeGMP()
{
	mpz_clear(this->iAuxBound);
	mpz_clear(this->iAuxBound2);
	mpz_clear(this->iAuxBoundD);
	mpz_clear(this->iAdMax);
	mpz_clear(this->iAdMin);
	mpz_clear(this->iAd);

	mpz_clear(this->iSievingIntervalLength);

	for (int i = 0; i < ConstPSPhase::GENERATED_POLYS_COUNT; i++)
	{
		mpf_clear(this->iFBBoundLog[i]);
	}

	mpz_clear(this->iRSerialAd);

	mpz_clear(this->iAuxMpz1);
	mpz_clear(this->iAuxMpz2);
	mpz_clear(this->iAuxMpz3);
	mpz_clear(this->iAuxMpz4);
	mpz_clear(this->iAuxMpz5);

	mpf_clear(this->iSkewness);
	mpf_clear(this->iRating);
	mpf_clear(this->iAlpha);

	mpf_clear(this->iAuxMpfSum);
	mpf_clear(this->iAuxMpf);
	mpf_clear(this->iAUxMpfS1);
	mpf_clear(this->iAUxMpfS2);
	mpf_clear(this->iAUxMpfA1);
	mpf_clear(this->iAUxMpfA2);

	return APolyPhase::DisposeGMP();
}

/*
Called by Destructor: Dispose mutex function
*/
int CKleinjung2PolySelPhase::DisposeMutexes()
{
	return APolyPhase::DisposeMutexes();
}

/*
Cleaning up files
*/
int CKleinjung2PolySelPhase::CleanUp(AFactorAlgParameters* aParameters, int aPhaseNBR, cleaning_types aType, bool aError)
{
	if (aParameters == NULL)
	{
		CLog::LogError(WHEREARG,"CKleinjung2PolySelPhase","CleanUp",0,0,"Parameters are NULL.");
		return ConstRC::BadArgument;
	}

	string identifier;
	string directory = "";
	string file;

	int phaseNBR;
	if (aParameters->GetIntegerPhaseParameter(ConstPN::POLY_PHASE_NUMBER, phaseNBR, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CKleinjung2PolySelPhase","CleanUp",0,0,"Didn't find phase number.");
		return ConstRC::GeneralError;
	}

	if (aParameters->GetStringPhaseParameter(ConstPN::SERIALIZATION_IDENTIFIER, identifier, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CKleinjung2PolySelPhase","CleanUp",0,0,"Didn't find serial identifier.");
		return ConstRC::GeneralError;
	}
	aParameters->GetStringPhaseParameter(ConstPN::SERIALIZATION_DIRECTORY, directory, (int)EMain);

	switch (aType)
	{
		case ECleanAllIfNotError:
			if (aError)
			{
				break;
			}
		case ECleanAll:
		case ECleanAllExcCF:
		case ECleanAllExcCFRF:
			// Running Serialization full filename - default is same as iSerialPhaseFullFileName
			if (aParameters->ContainsPhaseParameter(ConstPN::RSERIAL_POLY_SEL_FULL_FILENAME, phaseNBR))
			{
				aParameters->GetStringPhaseParameter(ConstPN::RSERIAL_POLY_SEL_FULL_FILENAME, file, phaseNBR);
				MSerializable::AdaptPathSeparator(file);
			}
			else
			{
				aParameters->CreateFullFileName(file, ConstPN::RSERIAL_POLY_SEL_FILENAME, 
					ConstPN::RSERIAL_POLY_SEL_DIRECTORY, ConstPN::RSERIAL_POLY_SEL_COMPRESSION, directory,
					ConstFN::POLY_PHASE, identifier, phaseNBR);
			}
			remove(file.c_str());

			// Candidate polynomials full filename
			if (aParameters->ContainsPhaseParameter(ConstPN::CAND_POLY_PAIR_FULL_FILENAME, phaseNBR))
			{
				aParameters->GetStringPhaseParameter(ConstPN::CAND_POLY_PAIR_FULL_FILENAME, file, phaseNBR);
				MSerializable::AdaptPathSeparator(file);
			}
			else
			{
				aParameters->CreateFullFileName(file, ConstPN::CAND_POLY_PAIR_FILENAME, 
					ConstPN::CAND_POLY_PAIR_DIRECTORY, ConstPN::CAND_POLY_PAIR_COMPRESSION, directory,
					ConstFN::CANDIDATE_POLY_PAIRS, identifier, phaseNBR);
			}
			remove(file.c_str());
			break;
		default:
			break;
	}

	return APolyPhase::CleanUp(aParameters, phaseNBR, aType, aError);
}	

/*
Called by InitParameters in APhase
*/
int CKleinjung2PolySelPhase::FillFunctorField()
{
	if (Get_FunctorsInit() == false)
	{
		Set_FunctorsInit(true);
		this->iPhaseFunctors.reserve(ConstPSPhase::INNER_STATE_COUNT);
		this->iPhaseFunctors.clear();
		this->iPhaseFunctors.push_back(NULL); // EPSPhaseNotStarted
		this->iPhaseFunctors.push_back(NULL); // EPSPhaseInitialization
		this->iPhaseFunctors.push_back(new CPhaseFunctor<CKleinjung2PolySelPhase>(this, &CKleinjung2PolySelPhase::GenerateCandidates)); // EPSPhaseSearchingForCandidates
		this->iPhaseFunctors.push_back(new CPhaseFunctor<CKleinjung2PolySelPhase>(this, &CKleinjung2PolySelPhase::ChooseBest)); // EPSPhaseChoosingFromCandidates
		this->iPhaseFunctors.push_back(new CPhaseFunctor<CKleinjung2PolySelPhase>(this, &CKleinjung2PolySelPhase::SaveResult)); // EPSPhaseSavingResult
		this->iPhaseFunctors.push_back(NULL); // EPSPhaseFinished
	}

	return ConstRC::Ok;
}

/*
Setting and initializing parameters, polynomials, ect + Serialization
*/
int CKleinjung2PolySelPhase::InitParameters()
{
	Set_InnerState((int)ESPhaseInitialization);

	AFactorAlgParameters * param = Get_Parameters();

	if (param == NULL)
	{
		CLog::LogError(WHEREARG,"CKleinjung2PolySelPhase","InitParameters",0,0,"Parameters aren't set.");
		return ConstRC::GeneralError;
	}

	// generally we don't know order of the sieving phase
	int phaseNBR;
	if (param->GetIntegerPhaseParameter(ConstPN::POLY_PHASE_NUMBER, phaseNBR, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CKleinjung2PolySelPhase","InitParameters",0,0,"Didn't find phase number.");
		return ConstRC::GeneralError;
	}
	Set_PhaseNbr(phaseNBR);

	// now we initialize members from parents's classes
	int rc = APolyPhase::InitParameters();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	// get sieving interval length
	if (param->GetMpzPhaseParameter(ConstPN::SIEVING_REGION_WIDTH, this->iSievingIntervalLength, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CKleinjung2PolySelPhase","InitParameters",0,0,"Didn't find sieving interval length.");
		return ConstRC::GeneralError;
	}

	//get sieving region LINE_SIEVE_SIZE
	if (param->GetIntegerPhaseParameter(ConstPN::LINE_SIEVE_SIZE, this->iRegSize, phaseNBR+1) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CKleinjung2PolySelPhase","InitParameters",0,0,"Didn't find line-sieve-size.");
		return ConstRC::GeneralError;
	}

	//get poly degree step
	int polyDegreeStep = 0;
	param->GetIntegerPhaseParameter(ConstPN::KLEINJUNG_POLY_DEGREE_STEP, ConstPSPhase::POLY_DEGREE_STEP, polyDegreeStep, phaseNBR);
	// set degree of the first polynomial
	this->iPolyDegree = mpz_sizeinbase(this->iN, 2) / polyDegreeStep;	// should for N < 105 dec => d = 4, N > 105 dec => d = 5
	if (this->iPolyDegree < ConstPSPhase::KLEINJUNG_MIN_POLY_DEGREE)
	{
		this->iPolyDegree = ConstPSPhase::KLEINJUNG_MIN_POLY_DEGREE;
	}

	// get a_{d} base
	int auxInt = 0;
	param->GetIntegerPhaseParameter(ConstPN::LEADING_COEFF_BASE, ConstPSPhase::KLEINJUNG_LEADING_COEFF_BASE, auxInt, phaseNBR);
	this->iAdBase = (unsigned long)auxInt;

	// get a_{d} lower bound
	if (param->ContainsPhaseParameter(ConstPN::LEADING_COEFF_LOWER_BOUND, phaseNBR))
	{
		param->GetMpzPhaseParameter(ConstPN::LEADING_COEFF_LOWER_BOUND, this->iAdMin, phaseNBR);
	}
	else
	{
		// set default value
		mpz_set_si(this->iAdMin, ConstPSPhase::KLEINJUNG_LEADING_COEFF_LOWER_BOUND);
	}

	// set a_{d} - initialization setup - deserialization can change
	if (mpz_cmp_ui(this->iAdMin, this->iAdBase) > 0)
	{
		//set a_{d} as multiple of base bigger than a_{d} lower bound
		mpz_cdiv_q_ui(this->iAd, this->iAdMin, this->iAdBase);
		mpz_mul_ui(this->iAd, this->iAd, this->iAdBase);
	}
	else
	{
		//set a_{d} as base
		mpz_set_ui(this->iAd, this->iAdBase);
	}

	string auxStr;
	long bound = 0;
	// we try to find value for all 
	param->GetLongPhaseParameter(ConstPN::FB_UPPER_BOUND, bound, bound, (int)EMain);
	for (int i = 0; i < ConstPSPhase::GENERATED_POLYS_COUNT; i++)
	{
		auxStr = NFSUtils::ConvertToString(i + 1);
		// we try to find individual value
		param->GetLongPhaseParameter(ConstPN::FB_UPPER_BOUND + auxStr, bound, bound, (int)EMain);
		mpf_init_set_d(this->iFBBoundLog[i], log((double)bound));
	}

	// get p_{b} - bound for primes in factorization of p
	param->GetIntegerPhaseParameter(ConstPN::KLEINJUNG_PRIME_BOUND, ConstPSPhase::KLEINJUNG_DEFAULT_PRIME_BOUND, this->iPrimeBound, phaseNBR);

	// get chi inverse - ratio size of poly coefficients
	param->GetIntegerPhaseParameter(ConstPN::CHI_INVERSE, ConstPSPhase::KLEINJUNG_CHI_INVERSE, this->iChiInverse, phaseNBR);

	param->GetLongPhaseParameter(ConstPN::RSERIALIZATION_AD_CHANGE_COUNT, ConstPSPhase::AD_CHANGE_COUNT, this->iRSerialRunAdChangeCount, phaseNBR);
	param->GetLongPhaseParameter(ConstPN::RSERIALIZATION_POLY_COUNT, this->iCandidatesCount, this->iRSerialRunPolyCount, phaseNBR);

	// Running Serialization full filename - default is same as iSerialPhaseFullFileName
	if (param->ContainsPhaseParameter(ConstPN::RSERIAL_POLY_SEL_FULL_FILENAME, phaseNBR))
	{
		param->GetStringPhaseParameter(ConstPN::RSERIAL_POLY_SEL_FULL_FILENAME, this->iRunningSerialFullFileName, phaseNBR);
		MSerializable::AdaptPathSeparator(this->iRunningSerialFullFileName);
	}
	else
	{
		param->CreateFullFileName(this->iRunningSerialFullFileName, ConstPN::RSERIAL_POLY_SEL_FILENAME, 
			ConstPN::RSERIAL_POLY_SEL_DIRECTORY, ConstPN::RSERIAL_POLY_SEL_COMPRESSION, Get_SerialMainDirectory(),
			ConstFN::POLY_PHASE, Get_SerialIdentifier(), phaseNBR);
	}

	// Candidate polynomials full filename
	if (param->ContainsPhaseParameter(ConstPN::CAND_POLY_PAIR_FULL_FILENAME, phaseNBR))
	{
		param->GetStringPhaseParameter(ConstPN::CAND_POLY_PAIR_FULL_FILENAME, this->iCandPolyPairsFullFileName, phaseNBR);
		MSerializable::AdaptPathSeparator(this->iCandPolyPairsFullFileName);
	}
	else
	{
		param->CreateFullFileName(this->iCandPolyPairsFullFileName, ConstPN::CAND_POLY_PAIR_FILENAME, 
			ConstPN::CAND_POLY_PAIR_DIRECTORY, ConstPN::CAND_POLY_PAIR_COMPRESSION, Get_SerialMainDirectory(),
			ConstFN::CANDIDATE_POLY_PAIRS, Get_SerialIdentifier(), phaseNBR);
	}

	// should we deserialize or serialize?
	bool deserialize = false;
	param->GetBoolPhaseParameter(ConstPN::DESERIALIZE, false, deserialize, Get_PhaseNbr());
	CBaseParameters bparam;
	bparam.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, Get_SerialPhaseFullFileName());

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	if (Get_InfoMode())
	{
		CLog::Log(10000, "1101000", "Kleinjung 2 Polynomial Selection Phase is running ...\n\n");
	}
#endif

	if (deserialize)
	{
		rc = Deserialize(bparam);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}
	else
	{
		rc = Serialize(bparam);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	return ConstRC::Ok;
}

/*
Seting sets of primes, values, bounds, empty polynomial, ect.
*/
int CKleinjung2PolySelPhase::PrepareForGenerating()
{

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	if (Get_InfoMode())
	{
		CLog::Log(5000, "1101020", "  Prepare for generating.\n");
	}
#endif

	int rc;

	//{K2}bounds for leading coeficient - copy from Klein BEGIN

	// set bound M - auxiliary bound used for calculation
	// max M is the (degree + 1)-th root of N, times chi
	mpz_root(this->iAuxMpz1, this->iN, (unsigned long)(this->iPolyDegree + 1));
	mpz_fdiv_q_ui(this->iAuxBound, this->iAuxMpz1, (unsigned long)this->iChiInverse);
	// set bound M^{2} - auxiliary bound used for calculation
	mpz_pow_ui(this->iAuxBound2, this->iAuxBound, 2);

	// set bound M^{(2d - 6)/(d - 2)} - auxiliary bound used for calculation
	// aux1 = (M^{2})^{d - 3}
	mpz_pow_ui(this->iAuxMpz1, this->iAuxBound2, (unsigned long)(this->iPolyDegree - 3));
	// MD = sqrt(d - 2, aux1)
	mpz_root(this->iAuxBoundD, this->iAuxMpz1, (unsigned long)(this->iPolyDegree - 2));

	// set upper bound for a_{d}
	// a_{d, max} = (M^{2d - 2} / N)^{1 / (d - 3)}
	// aux1 = M^{2d - 2}
	mpz_pow_ui(this->iAuxMpz1, this->iAuxBound, (unsigned long)(2 * this->iPolyDegree - 2));
	// aux2 = aux1 / N
	mpz_fdiv_q(this->iAuxMpz2, this->iAuxMpz1, this->iN);
	// a_{d, max} = sqrt(aux2, d - 3)
	if (this->iPolyDegree > 4)
	{
		mpz_root(this->iAdMax, this->iAuxMpz2, (unsigned long)(this->iPolyDegree - 3));
	}
	else
	{
		mpz_set(this->iAdMax, this->iAuxMpz2); 
	}

	// check a_{d, min} and a_{d, max}
	if (mpz_cmp(this->iAdMin, this->iAdMax) >= 0)
	{
		// reset a_{d, min}
		mpz_set_si(this->iAdMin, ConstPSPhase::KLEINJUNG_LEADING_COEFF_LOWER_BOUND);
		// write warning
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101004", "  Lower bound for leading coefficient was reset to default value.\n");
		}
#endif
	}

	//{K2}bounds for leading coeficient - copy from Klein END

	// write info
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	if (Get_InfoMode())
	{
		CLog::Log(5000, "", "\n");
		CLog::Log(5000, "1101005", "  The first poly degree is %d \n", this->iPolyDegree);
		CLog::Log(5000, "1101006", "  Auxiliary coeff bound is %Zd \n", this->iAuxBound);
		CLog::Log(5000, "1101007", "  Upper bound for leading coeff is %Zd \n", this->iAdMax);
		CLog::Log(5000, "1101008", "  Lower bound for leading coeff is %Zd \n", this->iAdMin);
	}
#endif

	// alocate polynomial improvement class
	if (this->iPolyImprovement == NULL)
	{
		this->iPolyImprovement = new CPolynomialImprovement(this->iCandidatesCount);
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101036", "  WARNING - polynomial improvement class already allocated!\n");
		}	
	}
#endif

	// allocate candidate pairs containers
	if (this->iCandidatePolyPairs == NULL)
	{
		this->iCandidatePolyPairs = new CCandidatePairs(this->iCandidatesCount);
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101025", "  WARNING - candidate poly pairs already allocated!\n");
		}	
	}
#endif

	// allocate iRoots - roots modulo prime of selected polynomails - for rating calculation
	if (this->iRoots == NULL)
	{
		this->iRoots = (long*) NFSUtils::AllocationMachine((void **) &(this->iRoots), this->iPolyDegree * sizeof(long));
		if (!this->iRoots)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iPolyDegree,0,"There was not enough memory to allocate array.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iRoots, 0, this->iPolyDegree * sizeof(long));
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101035", "  WARNING - roots already allocated!\n");
		}
	}
#endif

	if (this->iRSerialCandidatePolyPairs == NULL)
	{
		this->iRSerialCandidatePolyPairs = new CCandidatePairs(this->iCandidatesCount);
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101026", "  WARNING - rserial candidate poly pairs allocated!\n");
		}	
	}
#endif

	// generate sets P of prime r fulfilling r == 1 mod 4 and Q of prime r fulfilling r != 1 mod 4
	// this method also set values iSetPSize and iSetQSize
	rc = GenerateSetP();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	// allocate and fill matrix Powers, allocate array of primitive elements, modulo set P
	rc = GeneratePowers();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}
	
	this->iResultFirstPoly = new Polynomial(this->iPolyDegree);
	this->iResultSecondPoly = new Polynomial(1);

	return ConstRC::Ok;
}

/* 
Main function supposed to be called from outside
Serilization and calling Generate function 
*/
int CKleinjung2PolySelPhase::GenerateCandidates()
{
	//Aphase - setting internal values of phases of the program
	Set_InnerState(EPSPhaseSearchingForCandidates);

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	if (Get_InfoMode())
	{
		CLog::Log(10000, "1101019", "  Generating polynomial ...\n\n");
	}
#endif

	//Counting time
	this->iTimes->Start(EPSTGeneratePoly);

	//Serilization
	CBaseParameters param;
	param.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iRunningSerialFullFileName);
	int rc = StartRunningSerialization(param);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	//Kleinjung 2 algorithm - generate candidates
	rc = Generate();
	if (rc != ConstRC::Ok)
	{
		EndRunningSerialization();
		return rc;
	}

	rc = EndRunningSerialization();	

	this->iTimes->End(EPSTGeneratePoly);

	return rc;
}

/*
Determining the best polynomial pair
*/
int CKleinjung2PolySelPhase::ChooseBest()
{
	Set_InnerState(EPSPhaseChoosingFromCandidates);

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	if (Get_InfoMode())
	{
		CLog::Log(10000, "1101042", "  Total number of found polynomial: %d\n", this->iFoundCandidatePolyCount);
		CLog::Log(10000, "1101001", "  Choosing the best polynomial ...\n\n");
	}
#endif

	if (this->iFoundCandidatePolyCount <= 0 || this->iCandidatePolyPairs->Get_Count() <= 0)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","ChooseBest",ConstRC::NoCandidates,0,"No polynomial candidates found.");
		return ConstRC::NoCandidates;
	}

	if (this->iResultFirstPoly != NULL)
	{
		delete this->iResultFirstPoly;
	}
	if (this->iResultSecondPoly != NULL)
	{
		delete this->iResultSecondPoly;
	}

	// best polynomial pair is on the first position
	// TODO - feel free to use more complicated way of determining the best poly pair - testing sieving ?? or Murphy's E??
	this->iResultFirstPoly = new Polynomial();
	this->iCandidatePolyPairs->FirstPoly[0]->Copy(this->iResultFirstPoly);
	this->iResultSecondPoly = new Polynomial();
	this->iCandidatePolyPairs->SecondPoly[0]->Copy(this->iResultSecondPoly);
	mpz_set(this->iRoot, this->iCandidatePolyPairs->Root[0]);

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	if (Get_InfoMode())
	{
		CLog::Log(5000, "1101002", "  The best polynomial pair found is\n");
		this->iResultFirstPoly->PrintToScreen();
		this->iResultSecondPoly->PrintToScreen();
		CLog::Log(5000, "1101003", "    with alpha = %Ff and rating = %Ff\n\n", this->iCandidatePolyPairs->Alpha[0], this->iCandidatePolyPairs->Rating[0]);
	}
#endif

	return ConstRC::Ok;
}

/*
Saving results from generated polynomial
*/
int CKleinjung2PolySelPhase::SaveResult()
{
	if (this->iResultFirstPoly == NULL || this->iResultSecondPoly == NULL)
	{
		CLog::LogError(WHEREARG,"CKleinjung2PolySelPhase","SaveResult",ConstRC::GeneralError,0,"Result polynomial is not determined.");
		return ConstRC::GeneralError;
	}

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	if (Get_InfoMode())
	{
		CLog::Log(10000, "1101041", "\n  Saving the chosen polynomials.\n");
	}
#endif
	
	Set_InnerState(EPSPhaseSavingResult);

	AFactorAlgParameters * param = Get_Parameters();
	string auxStr = NFSUtils::ConvertToString(this->iRoot);
	param->SetPhaseParameter(ConstPN::ROOT_M, auxStr, (int)EMain);
	param->SetPhaseParameter(ConstPN::SIEVING_POLY1, this->iResultFirstPoly->ToString(ELongFormat), (int)EMain);
	param->SetPhaseParameter(ConstPN::SIEVING_POLY2, this->iResultSecondPoly->ToString(ELongFormat), (int)EMain);

	Set_InnerState(EPSPhaseFinished);

	return ConstRC::Ok;
}

/*

double CKleinjung2PolySelPhase::GetSpecialValuation (Polynomial * f, unsigned long p, mpz_t disc)
{
	double v;
	int p_divides_lc;
	int pvaluation_disc = 0;
	double pd = (double) p;
	int d = f->GetDegree();

	if (mpz_divisible_ui_p(disc, p)) 
	{
		mpz_t t;
		pvaluation_disc++;
		mpz_init(t);
		mpz_divexact_ui(t, disc, p);
		if (mpz_divisible_ui_p(t, p))
		{
			pvaluation_disc++;
		}
		mpz_clear(t);
	}

	p_divides_lc = false;
	if (f->Leading() % p == 0)
	{
		p_divides_lc = true;
	}

	int e;
	e = f->EvaluateModPrime(0, p);
	if (pvaluation_disc == 0) 
	{
		if (p_divides_lc) 
		{
		  if(f->Get(d - 1) % p == 0)
		  {
			e++;
		  }
		}
		return (pd * e) / (pd * pd - 1);
	} 
	else if (pvaluation_disc == 1) 
		{
			if (p_divides_lc)
			{
				e ++;
			}
			// something special here.
			return (pd * e - 1) / (pd * pd - 1);
		} 
		else 
		{
			v = special_val0(f, p) * pd;
			if (p_divides_lc) 
			{
				// compute g(x) = f(1/(px))*(px)^d, i.e., g[i] = f[d-i]*p^i
				Polynomial * G = new Polynomial(d);
				mpz_t *g;
				mpz_t t;
				int i;

				g = G->coeff;
				mpz_init_set_ui(t, 1);// will contains p^i
				for (i = 0; i <= d; i++) 
				{
					mpz_mul(g[i], f->Get(d - i), t);
					mpz_mul_ui(t, t, p);
				}
				v += special_val0(G, p);
				delete G;
				mpz_clear(t);
			}
		v /= pd + 1.0;
		return v;
		}
}
*/

/*
Counts Alpha for selected polynomial
*/
/*
int CKleinjung2PolySelPhase::GetAlpha(Polynomial * thePolynomial, unsigned long B)
{
  double alpha, e;
  unsigned long p;
  unsigned int d = thePolynomial->GetDegree();

  //Discriminant
  mpz_t disc;
  mpz_init (disc);
  thePolynomial->Discriminant(disc);

  e = GetSpecialValuation(thePolynomial, 2, disc);
  alpha = (1.0 - e) * log (2.0);

  for (p = 3; p <= B; p += 2)
    if (ulong_isprime (p))
      {
        e = special_valuation (f, p, disc);
        alpha += (1.0 / (double) (p - 1) - e) * log ((double) p);
      }
	mpz_clear (disc);
	mpf_set_d (this->iAlpha, alpha);
	return ConstRC::Ok;
}
*/

/*
Calculate Dickman Rho function. The method was copied from CADO NFS
*/
double CKleinjung2PolySelPhase::DickmanRho(double x)
{
	if (x < 0)
	{
		return 0.0;
	}

	if (x <= 1.0)
	{
		return 1.0;
	}

	if (x <= 2.0) /* 1 < x <= 2 */
	{
		x = 2 * x - 3.0; 
		// the approximation - Sage in CADO
		return .59453489206592253066+(-.33333334045356381396+(.55555549124525324224e-1+(-.12345536626584334599e-1+(.30864445307049269724e-2+(-.82383544119364408582e-3+(.22867526556051420719e-3+(-.63554707267886054080e-4+(.18727717457043009514e-4+(-.73223168705152723341e-5+.21206262864513086463e-5*x)*x)*x)*x)*x)*x)*x)*x)*x)*x;
	}

	if (x <= 3.0) /* 2 < x <= 3 */
	{
		x = 2 * x - 5.0; 
		// the approximation - Sage in CADO
		return .13031956190159661490+(-.11890697945147816983+(.45224027972316855319e-1+(-.97335515428611864000e-2+(.20773419306178919493e-2+(-.45596184871832967815e-3+(.10336375145598596368e-3+(-.23948090479838959902e-4+(.58842414590359757174e-5+(-.17744830412711835918e-5+.42395344408760226490e-6*x)*x)*x)*x)*x)*x)*x)*x)*x)*x;
	}

	if (x <= 4.0) /* 3 < x <= 4 */
	{
		x = 2 * x - 7.0; 
		// the approximation - Sage in CADO
		return .16229593252987041396e-1+(-.18617080355889960242e-1+(.98231465619823138710e-2+(-.30890609038683816355e-2+(.67860233545741575724e-3+(-.13691972815251836202e-3+(.27142792736320271161e-4+(-.54020882619058856352e-5+(.11184852221470961276e-5+(-.26822513121459348050e-6+.53524419961799178404e-7*x)*x)*x)*x)*x)*x)*x)*x)*x)*x;
	}

	if (x <= 5.0) /* 4 < x <= 5 */
	{
		x = 2 * x - 9.0; 
		// the approximation - Sage in CADO
		return .13701177421087314589e-2+(-.18032881447340571269e-2+(.11344648599460666353e-2+(-.44785452702335769551e-3+(.12312893763747451118e-3+(-.26025855424244979420e-4+(.49439561514862370128e-5+(-.89908455922138763242e-6+(.16408096470458054605e-6+(-.32859809253342863007e-7+.55954809519625267389e-8*x)*x)*x)*x)*x)*x)*x)*x)*x)*x;
	}

	if (x <= 6.0) /* 5 < x <= 6 */
	{
		x = 2 * x - 11.0; 
		// the approximation - Sage in CADO
		return .86018611204769414547e-4+(-.12455615869107014099e-3+(.87629281627689306328e-4+(-.39688578340310188782e-4+(.12884593599298603240e-4+(-.31758435723373651519e-5+(.63480088103905159169e-6+(-.11347189099214240765e-6+(.19390866486609035589e-7+(-.34493644515794744033e-8+.52005397653635760845e-9*x)*x)*x)*x)*x)*x)*x)*x)*x)*x;
	}

	if (x <= 7.0) /* 6 < x <= 7 */
	{
		x = 2 * x - 13.0; 
		// the approximation - Sage in CADO
		return .42503555236283394611e-5+(-.66168162622648216578e-5+(.50451140426428793943e-5+(-.25056278110193975867e-5+(.90780066639285274491e-6+(-.25409423435688445924e-6+(.56994106166489666850e-7+(-.10719440462879689904e-7+(.18244728209885020524e-8+(-.30691539036795813350e-9+.42848520313019466802e-10*x)*x)*x)*x)*x)*x)*x)*x)*x)*x;
	}

	if (x <= 8.0) /* 7 < x <= 8 */
	{
		x = 2 * x - 15.0; 
		// the approximation - Sage in CADO
		return .17178674964103956208e-6+(-.28335703551339176870e-6+(.23000575057461623263e-6+(-.12233608702949464058e-6+(.47877500071532198696e-7+(-.14657787264925438894e-7+(.36368896326425931590e-8+(-.74970784803203153691e-9+(.13390834297490308260e-9+(-.22532330111335516039e-10+.30448478436493554124e-11*x)*x)*x)*x)*x)*x)*x)*x)*x)*x;
	}

	if (x <= 9.0) /* 8 < x <= 9 */
	{
		x = 2 * x - 17.0; 
		/// the approximation - Sage in CADO
		return .58405695885954012372e-8+(-.10105102932563601939e-7+(.86312378192120507636e-8+(-.48483948355429166070e-8+(.20129738917787451004e-8+(-.65800969013673567377e-9+(.17591641970556292848e-9+(-.39380340623747158286e-10+(.75914903051312238224e-11+(-.13345077002021028171e-11+.18138420114766605833e-12*x)*x)*x)*x)*x)*x)*x)*x)*x)*x;
	}

	if (x <= 10.0) /* 9 < x <= 10 */
	{
		x = 2 * x - 19.0; 
		/// the approximation - Sage in CADO
		return .17063527516986966493e-9+(-.30739839907061929947e-9+(.27401311519313991860e-9+(-.16103964872970941164e-9+(.70152211692303181302e-10+(-.24143747383189013980e-10+(.68287507297199575436e-11+(-.16282751120203734207e-11+(.33676191519721199042e-12+(-.63207992345353337190e-13+.88821884863349801119e-14*x)*x)*x)*x)*x)*x)*x)*x)*x)*x;
	}

	if (x <= 11.0) /* 10 < x <= 11 */
	{
		x = 2 * x - 21.0; 
		/// the approximation - Sage in CADO
		return .43559526824795217852e-11+(-.81254892374415436871e-11+(.75124702775657186758e-11+(-.45879032842537328635e-11+(.20810226978994617073e-11+(-.74742298793568571039e-12+(.22118242218483402875e-12+(-.55382747996458896000e-13+(.12115887170289387926e-13+(-.24203055184086403014e-14+.35552802442946562758e-15*x)*x)*x)*x)*x)*x)*x)*x)*x)*x;
	}

	if (x <= 12.0) /* 11 < x <= 12 */
	{
		x = 2 * x - 23.0; 
		/// the approximation - Sage in CADO
		return .19934633853036819003e-14+(-.39390567960226078379e-14+(.38665626810909948382e-14+(-.25132152019284365490e-14+(.12165850034414917031e-14+(-.46768237317409899068e-15+(.14856141016839901209e-15+(-.40061895880500065473e-16+(.95227478488277343487e-17+(-.20899104913735568600e-17+.32985921210508602052e-18*x)*x)*x)*x)*x)*x)*x)*x)*x)*x;
	}

	if (x <= 14.0) /* 13 < x <= 14 */
	{
		x = 2 * x - 27.0; 
		// the approximation - Sage in CADO
		return .36468388101796492224e-16+(-.73831973563371575758e-16+(.74312671625965740134e-16+(-.49570201041373844974e-16+(.24648267921024808625e-16+(-.97426560588907822003e-17+(.31850597161723004412e-17+(-.88485037859050841047e-18+(.21736499712587315716e-18+(-.49450337815428832609e-19+.80095079043879918725e-20*x)*x)*x)*x)*x)*x)*x)*x)*x)*x;
	}

	if (x <= 15.0) /* 14 < x <= 15 */
	{
		x = 2 * x - 29.0; 
		// the approximation - Sage in CADO
		return .60765098932992206115e-18+(-.12575305192897762077e-17+(.12946447870858888856e-17+(-.88393035793877424780e-18+(.45020754487871940709e-18+(-.18241969586978266280e-18+(.61176643040672211241e-19+(-.17448851858831783538e-19+(.44129951768989290506e-20+(-.10359223715080577635e-20+.17147161493537144558e-21*x)*x)*x)*x)*x)*x)*x)*x)*x)*x;
	}

	// for x > 15, Dickman's function differs from the approximation below by at most 3.5e-21
	{
		static int count = 0;
		if (count ++ == 0)
		fprintf (stderr, "Dickman rho is imprecise for x > 15\n");
	}

	return 0.0332357434363490 * pow (x, -x);
}

/* Murphy's E-value is defined on pages 86 and 87 of Murphy's thesis:

   E(f,g) = sum(rho(u_f(theta_i))*rho(u_g(theta_i)), i=1..K)
   where theta_i = Pi/K*(i-1/2)
   and u_f(theta_i) = (log(|F(cos(theta_i)*s^(1/2),sin(theta_i)/s^(1/2))|) + alpha_f)/log(B_f)
*/
double CKleinjung2PolySelPhase::MurphyE(double Bf, double Bg, double area, int K) //cado_poly
{
	//!!in progress not working because of retyping!!
	const double pi = 3.14159265358979324;
	// const int taylor = 20;
	// const int round = 10000;

	double E = 0, x, y, ti;
	double xi, yi, skewness;
	double one_over_logBf, one_over_logBg, vf, vg;

	mpf_t xiyi, tmp;	
	//mpf_t vf, vg;
	mpf_init (xiyi);
	mpf_init (tmp);
	//mpf_init (vf); mpf_init (vg); 

	mpf_set_ui(xiyi, 0);

	skewness = mpf_get_d(this->iSkewness);
	//cout << "Skewness is "<< skewness << endl;
	//gmp_printf ("MPF skewness is %.*FiSkewness with %d digits\n", 15, this->iSkewness, 15);
	x = sqrt (area * skewness);
	y = sqrt (area / skewness);
	//cout << "Alpha is "<< alpha_f << endl;
	//gmp_printf ("MPF Alpha is %.*FiAlpha with %d digits\n", 15, iAlpha, 15);
	one_over_logBf = 1.0 / log(Bf);
	one_over_logBg = 1.0 / log(Bg);
	
	int i;
	for (i = 0; i < K; i++) //count all K parts
	{
		ti = pi / (double) K * ((double) i + 0.5);
		xi = x * cos (ti);
		yi = y * sin (ti);
		//cout << "xi is "<< xi << "yi is "<< yi << "ti is "<< ti << endl;

		if (yi != 0) // F(xi, yi)
		{
			mpf_set_d(xiyi, xi / yi);
			iResultFirstPoly->EvaluateDecimal(tmp, xiyi); //& to have the result here?? TODO
			vf = mpf_get_d(tmp);

			iResultSecondPoly->EvaluateDecimal(tmp, xiyi);
			vg = mpf_get_d(tmp);

			vf *= pow (yi, this->iPolyDegree);
			vg *= pow (yi, 2);
		}
		else		// F(xi, 0) = a_d * xi^d
		{
			vf = iResultFirstPoly->Leading() * pow(xi, this->iPolyDegree);
			vg = iResultSecondPoly->Leading()* xi * xi;
		}

		vf = log( abs(vf) );
		vg = log( abs(vg) );

		vf += mpf_get_d(this->iAlpha);
		vg += 0.569959993064325; // R. Barbulescu, A. Lachand 2014, Approx: Alpha(g) = 0.56 Prop 3.2

		vf *= one_over_logBf;
		vg *= one_over_logBg;


		E += DickmanRho(vf) * DickmanRho(vg);
		
		/*
		//Taylor serie for two logarithms
		mpf_sub_ui(vf, vf, 1);
		mpf_sub_ui(vg, vg, 1);
		for (i = 2; i < taylor; i++)
		{
			mpf_pow_ui(tmp, vf, i);
			mpf_div_ui(tmp, tmp, i);
			if (i%2 == 0)
			{
				mpf_sub(vf, vf, tmp);
			}
			else
			{
				mpf_add(vf, vf, tmp);
			}

			mpf_pow_ui(tmp, vg, i);
			mpf_div_ui(tmp, tmp, i);
			if (i%2 == 0)
			{
				mpf_sub(vg, vg, tmp);
			}
			else
			{
				mpf_add(vg, vg, tmp);
			}
		}
		//cout << "After Taylor:"<< endl;
		//gmp_printf ("MPF Result is %.*Ff with %d digits\n", 15, vf, 15);
		//gmp_printf ("MPF Result is %.*Ff with %d digits\n", 15, vg, 15);
		

		mpf_add(vf, vf, this->iAlpha);
		mpf_add(vg, vg, this->iAlpha);

		mpf_mul_ui (vf, vf, long (round*one_over_logBf));
		mpf_div_ui (vf, vf, round);
		mpf_mul_ui (vg, vg, long (round*one_over_logBg));
		mpf_div_ui (vg, vg, round);
		//gmp_printf("\vf = %Ff, vg = %Ff\n",vf,vg);

		E += DickmanRho(mpf_get_d(vf)) * DickmanRho(mpf_get_d(vg));
		cout << " E is "<< E << endl;
		*/
	}

	return (E / (double) K);
}

/*
{K2}Prepares a set P and Q of primes lower than some bound for finding m_2, and a set P2 of sqares of P.
*/
int CKleinjung2PolySelPhase::GenerateSetP()
{
	if (this->iSetP != NULL)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GenerateSetP",0,0,"Array p already allocated.");
		return ConstRC::BadArgument;
	}

	if (this->iSetQ != NULL)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GenerateSetQ",0,0,"Array q already allocated.");
		return ConstRC::BadArgument;
	}

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	if (Get_InfoMode())
	{
		CLog::Log(5000, "1101021", "    Generate sets P and Q.\n");
	}
#endif

	unsigned long count = 0;
	long * primes = NULL;
	long bound = this->iPrimeBound;
	int rc = CIntegerClass::GetPrimes(10, bound, count, primes); // d*a_d should not be divisible by this primes
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GenerateSetP",rc,0,"Some error during generating primes.");
		return rc;
	}
	
	int * auxArrayP = NULL;
	auxArrayP = (int*) NFSUtils::AllocationMachine((void **) &(auxArrayP), count * sizeof(int));
	if (!auxArrayP)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GenerateSetP",count,0,"There was not enough memory to allocate array for set P.");
		return ConstRC::NotEnoughMemory;
	}
		
	int * auxArrayQ = NULL;
	auxArrayQ = (int*) NFSUtils::AllocationMachine((void **) &(auxArrayQ), count * sizeof(int));
	if (!auxArrayQ)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GenerateSetP",count,0,"There was not enough memory to allocate array for set Q.");
		return ConstRC::NotEnoughMemory;
	}


	int * currP = auxArrayP;
	int * currQ = auxArrayQ;
	this->iSetPSize = 0;
	this->iSetQSize = 0;
	for (unsigned int i = 0; i < count; i++, primes++)
	{
		if (*primes % 4 == 1) //primes == 1 mod 4 goes to P set, others go to Q set
		{
			this->iSetPSize++;
			*currP = *primes;
			currP++;
		}
		else
		{
			this->iSetQSize++;
			*currQ = *primes;
			currQ++;
			
		}
	}

	this->iSetP = (int*) NFSUtils::AllocationMachine((void **) &(this->iSetP), this->iSetPSize * sizeof(int));
	if (!this->iSetP)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GenerateSetP",this->iSetPSize,0,"There was not enough memory to allocate array P.");
		return ConstRC::NotEnoughMemory;
	}

	this->iSetQ = (int*) NFSUtils::AllocationMachine((void **) &(this->iSetQ), this->iSetQSize * sizeof(int));
	if (!this->iSetQ)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GenerateSetP",this->iSetPSize,0,"There was not enough memory to allocate array Q.");
		return ConstRC::NotEnoughMemory;
	}

	// copy values from aux array to set P
	memcpy(this->iSetP, auxArrayP, this->iSetPSize * sizeof(int));

	// copy values from aux array to set Q
	memcpy(this->iSetQ, auxArrayQ, this->iSetQSize * sizeof(int));

	free(auxArrayP);
	auxArrayP = NULL;

	free(auxArrayQ);
	auxArrayQ = NULL;

	return ConstRC::Ok;
}


/*
{K2}Counting solution for congruences based on set P
*/
int CKleinjung2PolySelPhase::GeneratePowers()
{
	//a check if we have a set P
	if (this->iSetP == NULL || this->iSetPSize == 0)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GeneratePowers",0,0,"Set P is not prepared.");
		return ConstRC::BadArgument;
	}

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	if (Get_InfoMode())
	{
		CLog::Log(5000, "1101022", "    Generate powers.\n");
	}
#endif

	// allocate iPrimitiveElements - array of primitive elements modulo primes p from P, size of the array is same as P = iSetPSize
	if (this->iPrimitiveElementsP == NULL)
	{
		this->iPrimitiveElementsP = (int*) NFSUtils::AllocationMachine((void **) &(this->iPrimitiveElementsP), this->iSetPSize * sizeof(int));
		if (!this->iPrimitiveElementsP)
		{
			CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GeneratePowers",this->iSetPSize,0,"There was not enough memory to allocate array.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iPrimitiveElementsP, 0, this->iSetPSize * sizeof(int));
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101016", "  WARNING - array of primitive elements P already allocated!\n");
		}
	}
#endif

	if (this->iPrimitiveElementsQ == NULL)
	{
		this->iPrimitiveElementsQ = (int*) NFSUtils::AllocationMachine((void **) &(this->iPrimitiveElementsQ), this->iSetQSize * sizeof(int));
		if (!this->iPrimitiveElementsQ)
		{
			CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GeneratePowers",this->iSetQSize,0,"There was not enough memory to allocate array.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iPrimitiveElementsQ, 0, this->iSetQSize * sizeof(int));
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101016", "  WARNING - array of primitive elements Q already allocated!\n");
		}
	}
#endif	

	// allocate iMatrixPowersP - iSetPSize x iPrimeBound matrix, for element a_{i,j} holds e_{i}^a_{i,j} = j (mod p_{i}), where p_{i} from P and e_{i} is primitive element modulo p_{i} 
	if (this->iMatrixPowersP == NULL)
	{
		this->iMatrixPowersP = (int **) NFSUtils::AllocationMachine((void **) &this->iMatrixPowersP, this->iSetPSize * sizeof(int*));
		if (!this->iMatrixPowersP)
		{
			CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GeneratePowers",this->iSetPSize,0,"There was not enough memory to allocate matrix rows.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iMatrixPowersP, 0, this->iSetPSize * sizeof(int*));
		for (int i = 0; i < this->iSetPSize; i++)
		{
			this->iMatrixPowersP[i] = (int*) NFSUtils::AllocationMachine((void **) &(this->iMatrixPowersP[i]), this->iPrimeBound * sizeof(int));
			if (!this->iMatrixPowersP[i])
			{
				CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GeneratePowers",this->iPrimeBound,0,"There was not enough memory to allocate matrix columns.");
				return ConstRC::NotEnoughMemory;
			}
			memset(this->iMatrixPowersP[i], 0, this->iPrimeBound * sizeof(int));
		}
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101018", "  WARNING - matrix Powers P already allocated!\n");
		}	
	}
#endif	
	// allocate iMatrixPowersQ - iSetQSize x iPrimeBound matrix, for element a_{i,j} holds e_{i}^a_{i,j} = j (mod q_{i}), where q_{i} from Q and e_{i} is primitive element modulo p_{i} 
	if (this->iMatrixPowersQ == NULL)
	{
		this->iMatrixPowersQ = (int **) NFSUtils::AllocationMachine((void **) &this->iMatrixPowersQ, this->iSetQSize * sizeof(int*));
		if (!this->iMatrixPowersQ)
		{
			CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GeneratePowers",this->iSetQSize,0,"There was not enough memory to allocate matrix rows.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iMatrixPowersQ, 0, this->iSetQSize * sizeof(int*));
		for (int i = 0; i < this->iSetQSize; i++)
		{
			this->iMatrixPowersQ[i] = (int*) NFSUtils::AllocationMachine((void **) &(this->iMatrixPowersQ[i]), this->iPrimeBound * sizeof(int));
			if (!this->iMatrixPowersQ[i])
			{
				CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GeneratePowers",this->iPrimeBound,0,"There was not enough memory to allocate matrix columns.");
				return ConstRC::NotEnoughMemory;
			}
			memset(this->iMatrixPowersQ[i], 0, this->iPrimeBound * sizeof(int));
		}
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101018", "  WARNING - matrix Powers Q already allocated!\n");
		}	
	}
#endif	


	//Fill Primitive elements and Matrix of powers for modular d-th root aritmetics
	int rc = ConstRC::Ok;
	rc = CIntegerClass::GetPrimitiveRootsAndPowers(this->iSetP, this->iSetPSize, this->iPrimeBound, this->iPrimitiveElementsP, this->iMatrixPowersP);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GeneratePowers",rc,0,"Some error during generating primitive roots and powers.");
		return rc;
	}
	rc = CIntegerClass::GetPrimitiveRootsAndPowers(this->iSetQ, this->iSetQSize, this->iPrimeBound, this->iPrimitiveElementsQ, this->iMatrixPowersQ);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GeneratePowers",rc,0,"Some error during generating primitive roots and powers.");
		return rc;
	}

	return rc;
}


/*
{K2}Prepare N tilde modulo set P, elements sqared
*/
int CKleinjung2PolySelPhase::GenerateNModSetPQ(int * SetPQ, int SetSize)
{
	//checks if N tilde != 0 and the Set P is not empty
	if (this->iNtilde == 0)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GenerateNModSetP",0,0,"N tilde is not prepared.");
		return ConstRC::BadArgument;
	}
	if (SetPQ == NULL || SetSize == 0)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GenerateNModSetP",0,0,"Set P or Q is not prepared.");
		return ConstRC::BadArgument;
	}
/*
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 50000
	if (Get_InfoMode())
	{
		CLog::Log(5000, "1101023", "    Generate N tilde mod set P.\n");
	}
#endif
*/
	//if the array is already created from previous cycles, clear it
	if (this->iNModSetPQ != NULL)
	{
		free(this->iNModSetPQ);
		this->iNModSetPQ = NULL;
		this->iNModSetPQSize = 0;
	}

	// allocate iNModSetPQ - array of N tilde modulo sqares of primes p from P, size of the array is same as P = iSetPSize
	this->iNModSetPQ = (int*) NFSUtils::AllocationMachine((void **) &(this->iNModSetPQ), SetSize * sizeof(int));
	if (!this->iNModSetPQ)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","GenerateNModSetP",SetSize,0,"There was not enough memory to allocate array.");
		return ConstRC::NotEnoughMemory;
	}
	memset(this->iNModSetPQ, 0, SetSize * sizeof(int));
	

	// calculate N tilde mod p^2
	int * currR = SetPQ;
	int * currN = this->iNModSetPQ;
	for (int i = 0; i < SetSize; i++, currR++, currN++)
	{
		mpz_fdiv_r_ui(this->iAuxMpz1, this->iNtilde, (unsigned long) (*currR)*(*currR));
		*currN = mpz_get_si(this->iAuxMpz1);
		this->iNModSetPQSize++;
	}

	//the result is in this->iNModSetPQ, his size is this->iNModSetPQSize
	return ConstRC::Ok;
}


/*
{K2}solves a congruence and prepares a list (multimap) of pairs (p, rp)
*/
int CKleinjung2PolySelPhase::SolveCongrModSqare( multimap<int,int>& PairSolution, int& NumberOfPairs, int * SetPQ, int SetSize, int * iPrimitiveElements, int ** iMatrixPowers)
{
	if (SetSize != this->iNModSetPQSize)
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","SolveCongrModSqare",SetSize,0,"The set N tilde mod P or Q is of a different size than set P or Q.");
		return ConstRC::CannotBeDone;
	}
	

	int elementP, elementP2, NtildeModP2; // element from P, sqare and N tilde mod p sqared
	int l = 0; // l = number of solutions
	mpz_t rp; //the solution r of the congruence, forms a pair (p, r)
	mpz_init (rp);
	int k, f, x, c0, c1, c, m0, tmpBase, tmpResult;

	int i, j, jExp;
	
	for (i = 0; i < SetSize; i++)	
	{
		elementP = SetPQ[i];
		elementP2 = elementP*elementP;
		NtildeModP2 = this->iNModSetPQ[i];

		//m0 mod elementP2
		mpz_cdiv_r_ui (rp,iMRoot0,(unsigned long)elementP2);
		m0 = - mpz_get_si(rp);
		if (m0 < 0)
		{
			m0 +=elementP2;
		}
		//cout << " m0 is "<< m0 << endl;
		
		if(NtildeModP2 != 0)//test, if N tilde is not divisible by elementP, the kongruence is equal to 0
		{
			if (NtildeModP2 % elementP != 0) // if NtildeModP2 % elementP == 0 - the solution does not exist
			{
				// NtildeModP2 = e^{f} (mod elemetP2), e is primitive root modulo elementP
				// we will solve the congruence (mod elementP), and the solution will be lifted: r + k*elementP (mod elementP2)
				NtildeModP2 = NtildeModP2 % elementP; // N tilde (mod p)
				f = iMatrixPowers[i][NtildeModP2];	//a power where (primitive element)^f = N tilde (mod p_i)
										
				if (f % this->iPolyDegree == 0)				// if root of elementP exists we need the exponent to be 
				{
					k = (elementP - 1) / this->iPolyDegree; //elementP is prime, order is (elementP - 1) and we are looking for d-th root
					x = (f / this->iPolyDegree) % k;		// e^xd = e^f mod elementP  -> x = f/d mod ((elementP-1)/d) as d|f and d|elementP-1		
					c1 = CIntegerClass::PowerModN(iPrimitiveElements[i], x, elementP);	//{K2}d-th root calculate first d-th root c_1 = e^x mod p
					c0 = CIntegerClass::PowerModN(iPrimitiveElements[i], k, elementP);
					
					//here should be Hensel lifting of solution mod elementP to solution modulo elementP2
					do
					{
						for(j = 0; j < elementP; j++)
						{
							c = (c1 + j*elementP) % elementP2;

							//c^j (mod p^2)
							tmpBase = c;
							tmpResult = 1;
							jExp = this->iPolyDegree;
							while (jExp > 0) 
							{
								if (jExp & 1) 
								{
									tmpResult = (tmpResult * tmpBase) % elementP2;
								}
								tmpBase = (tmpBase * tmpBase) % elementP2;
								jExp >>= 1;
							}

							if ( tmpResult == iNModSetPQ[i] && mpz_get_ui(this->iAd) % elementP != 0 )
							{
								PairSolution.insert( pair<int,int>(elementP, ((c - m0 + elementP2))) ); //a solution pair (elemetP, rp)
							}
						}
						l++;		//increase the counter of pairs (elemetP, rp)
						c1 = (c1 * c0) % elementP;			// calculate other possible roots - c_i+1 = c_i * e^((r-1)/d) = c_i * c0 
						x += k;
					}
					while (x < (elementP - 1));
				}
			}
		}
		else
		{
			if (mpz_get_ui(this->iAd) % elementP != 0)
			{
				PairSolution.insert( pair<int,int>(elementP, m0)); //a solution pair (elemetP, rp)
				l++; //increase the number of pairs (elemetP, rp)
				if (this->iPolyDegree % 2 == 0)
				{
					//N tilde (mod p) = 0, the congruence is also: {m_0 + r_p == p (mod p^2)}
					PairSolution.insert( pair<int,int>(elementP, (elementP - m0)) );
					l++; //increase the number of pairs (elemetP, rp)
				}
			}
		}
	}

	NumberOfPairs = l;	

	return ConstRC::Ok;
}

/*
{K2}Returns n^-1 (mod p), no checks if exists returns only 0 if not found
*/
int CKleinjung2PolySelPhase::modInverse(int n, int p) 
{
    n %= p;
	if (n % p != 0)
	{
		for(int x = 1; x < p; x++) 
		{
			if((n*x) % p == 1) 
			{
				return x;
			}
		}
	}
	
	return 0; //no solution found
}


/*
{K2} Calculate i for: rq + i*q^2 == rp (mod p^2) for each p from iSetP
and calls find collisions
*/
int CKleinjung2PolySelPhase::CalculateRpRqI()
{
	multimap<int, int> :: iterator itr1, itr2;
	int tmp1, tmp2, p, pp, q, qq;
	int rc = ConstRC::Ok;

	//for each q
	for (itr1 = iPairSolutionQr.begin(); itr1!=iPairSolutionQr.end(); ++itr1)
	{
		p = 0; //aux value - if no other p used
		q = (*itr1).first;
		qq = q*q;

		//for each p calculate i from rq + i*q^2 == rp (mod p^2)
		for (itr2 = iPairSolutionPr.begin(); itr2!=iPairSolutionPr.end(); ++itr2)
		{
			p = (*itr2).first;
			pp = p*p;
	
			if(q % p != 0) // invers for q^2 (mod p) exists
			{
				tmp1 = (*itr2).second - (*itr1).second % pp; //rp - rq (mod p^2)

				tmp2 = modInverse(qq, pp);// calculate invers for q^2 (mod p)
				tmp1 = (tmp1*tmp2) % pp; // i = (rp - rq)*(q^-2) (mod p^2)
				if (mpz_get_ui(this->iAd) % p != 0)
				{
					iPairSolutionPi.insert( pair<int,int>(p, tmp1) );
				}
			}
		}

		if (p > 0) // there was at least one pair (p, r)
		{	
			//{K2}finds collision (p1, rp) and (p2, rp)
			rc = FindCollision(iPairSolutionPi, q);
			if (rc != ConstRC::Ok)
			{
				return rc;
			}
		}
	}

	return rc;
}

/*
{K2}Function solving congruency, INPUT: a list of pairs, looking for a collision: rp2 (mod p1) == rp1 (mod p2)
*/
int CKleinjung2PolySelPhase::FindCollision( multimap<int,int> PairSolution, int q )
{
	multimap<int, int> :: iterator itr1, itr2;
	int tmp1, tmp2;
	int possibleM2 = 0, tmpM2 = 0; //future root iRoot2
	int tmpSolution = 0; //auxilarity value, the solution r

	mpz_t tmpGCD;
	mpz_init (tmpGCD);

	for (itr1 = PairSolution.begin(); itr1!=PairSolution.end(); ++itr1)
	{
		itr2 = itr1;
		itr2++;
		for (; itr2!=PairSolution.end(); ++itr2)
		{
			tmp1 = (*itr2).second % (*itr1).first; //rp2 (mod p1)
			tmp2 = (*itr1).second % (*itr2).first; //rp1 (mod p2)
			if (tmp1 == tmp2) //collision detected rp2 (mod p1) == rp1 (mod p2)
			{
				possibleM2 = (*itr1).first * (*itr2).first * q;
				
				//select the biggest collision
				if (tmpM2 < possibleM2 )
				{
					mpz_set_ui(tmpGCD, possibleM2);
					mpz_gcd (tmpGCD, idAd, tmpGCD);

					if (mpz_cmp_ui (tmpGCD, 1) == 0)
					{
						// p1 * p2 * q
						tmpM2 = possibleM2;
						// r
						tmpSolution= (*itr1).second;
					}
				}
			}
			if (this->iPreM2 < tmpM2)
			{
				this->iPreM2 = tmpM2;
				this->iCollision = tmpSolution;
			}
		}
	}


	return ConstRC::Ok;
}

/*
{K2}Generating poly-candidates from variables, 
TODO should be base-(m,p) method, where m and p are input
parametrs removed: int aSubsetPPIndex, double aEpsilon
*/
int CKleinjung2PolySelPhase::FindPolynomials()
{
	//TODO paramets - will be called from K2 (not needed) or from K1 (needed)
	int rc = CKleinjung2PolySelPhase::BaseMPExpansion(this->iMRoot1, this->iMRoot2); 
	if (rc != ConstRC::Ok)
	{
		return rc;
	}
	
	//Calls Improve, evaluates improved polynomials, save only the good pairs
	rc = SaveCandidate();		///TODO zkontrolovat
	if (rc != ConstRC::Ok)
	{
		return rc;
	}


	return ConstRC::Ok;
}

/*
{K2}Prepares roots m1 and m2 and calls FindPolynomials()
*/
int CKleinjung2PolySelPhase::PrepareM1M2(int p1p2q1, int ri)
{
	int rc = ConstRC::Ok;
	mpz_t aAux;
	mpz_init(aAux);
	
	//m_2 from the input
	mpz_set_ui(this->iMRoot2, p1p2q1);
	
	///m tilde = m_0 + rp, rp is the solution (mod p^2)
	mpz_add_ui (this->iMRoot0, this->iMRoot0, ri);

	
	///a_{d-1} == m tilde / m_2 (mod d*a_{d})
	mpz_invert (this->iAuxAdm1, iMRoot2, this->idAd);
	mpz_mul (this->iAuxAdm1, this->iAuxAdm1, this->iMRoot0); 
	mpz_mod (this->iAuxAdm1, this->iAuxAdm1, this->idAd); 

	///m_1 = ( (m tilde - a_{d-1}*m_2) / (d*a_{d}))
	mpz_mul (this->iAuxAdm1, this->iAuxAdm1, iMRoot2);
	mpz_sub (this->iAuxAdm1, this->iMRoot0, this->iAuxAdm1);
	mpz_divexact(this->iMRoot1, this->iAuxAdm1, this->idAd); //because (m tilde == a_{d-1}*m_2) (mod d*a_{d})


	//TODO optimize - could be replaced as enforced above
	//ensure N == a_d m^d (mod p)
	mpz_pow_ui(aAux, this->iMRoot1, this->iPolyDegree);
	mpz_mul(aAux, aAux, this->iAd);
	mpz_sub(aAux, this->iN, aAux);
	mpz_mod(aAux, aAux, this->iMRoot2);

	//if not modify m_1 - in progress not working, check in Bai how to count m1 and m2
	mpz_sub_ui(this->iMRoot1, this->iMRoot1, 1);
	do{
		mpz_add_ui(this->iMRoot1, this->iMRoot1, 1);
		mpz_pow_ui(aAux, this->iMRoot1, this->iPolyDegree);
		mpz_mul(aAux, aAux, this->iAd);
		if (mpz_cmp(aAux, this->iN) > 0)
		{
			mpz_sub(aAux, aAux, this->iN);
		}
		else
		{
			mpz_sub(aAux, this->iN, aAux);
		}
		mpz_mod(aAux, aAux, this->iMRoot2);
	}
	while(mpz_cmp_ui(aAux, 0) != 0);
	//TODO optimize END


	///{K2} base-(m_1, m_2) method, there must be gcd(m_1, m_2) = 1, otherwise no polynomial will be generated
	mpz_gcd(this->iAuxMpz5, iMRoot1, iMRoot2);
	if (mpz_cmp_ui(this->iAuxMpz5,1) == 0)
	{
		rc = FindPolynomials();			//TODO paramets - if called from K2 (not needed) or from K1 (needed)
	}
	
	return rc; 
}

int CKleinjung2PolySelPhase::BaseMPExpansion(mpz_t aM1, mpz_t aM2)
{
	// must be GCD(m1,m2) = 1
	mpz_gcd(this->iAuxMpz5, aM1, aM2);
	if (mpz_cmp_ui(this->iAuxMpz5,1) > 0) {
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","BaseMPExpansion",ConstRC::BadArgument,0,"Base-(m,p) expansion cannot be calculated due to values m and p are not coprime.");
		return ConstRC::BadArgument;
	}

	int i;
	mpz_t aAux;
	mpz_init(aAux);

	//N == a_d m^d (mod p)
	mpz_pow_ui(aAux, aM1, this->iPolyDegree);
	mpz_mul(aAux, aAux, this->iAd);
	mpz_sub(aAux, this->iN, aAux);
	mpz_mod(aAux, aAux, aM2);

	/*
	if(mpz_cmp_ui(aAux, 0) == 0)
	{
		printf(" N == a_d m^d (mod p) ");
	}
	else
	{
		printf(" NEPLATI KONGRUENCE N != a_d m^d (mod p) ");
	}
	*/

	// set leading (d-th) coeficient ad
	this->iResultFirstPoly->Set(this->iPolyDegree, this->iAd);
	// aux4 is dedicated for mi
	mpz_pow_ui(this->iAuxMpz4, aM1, (unsigned long)this->iPolyDegree);
	// aux2 is dedicated for ri
	mpz_set(this->iAuxMpz2, this->iN);
	// aux3 is dedicated for ai
	mpz_set(this->iAuxMpz3, this->iAd);
	for (i = this->iPolyDegree - 1; i > 0; i--)
	{
		// aux2 = (ri - ai*mi)/p
		mpz_mul(this->iAuxMpz5, this->iAuxMpz3, this->iAuxMpz4);
		mpz_sub(this->iAuxMpz2, this->iAuxMpz2, this->iAuxMpz5);
		mpz_divexact(this->iAuxMpz2, this->iAuxMpz2, aM2); // zde pp dlitelnost p
		// aux4 = mi/m
		mpz_divexact(this->iAuxMpz4, this->iAuxMpz4, aM1);
		// aux3 = |ai| = ri/mi
		mpz_cdiv_q(this->iAuxMpz3, this->iAuxMpz2, this->iAuxMpz4);
		// find delta_i
		// aux5 = ri - |ai|*mi
		mpz_mul(this->iAuxMpz5, this->iAuxMpz3, this->iAuxMpz4);
		mpz_sub(this->iAuxMpz5, this->iAuxMpz2, this->iAuxMpz5);
		// aux = x (biggest s.t. <= aux5 and divisible by p)
		mpz_fdiv_q(aAux, this->iAuxMpz5, aM2);
		mpz_mul(aAux, aAux, aM2);
		// aux = gamma_i (difference from zero of aux5 mod p)
		mpz_sub(aAux, this->iAuxMpz5, aAux);
		// aux5 = inverse of mi mod p
		mpz_invert(this->iAuxMpz5, this->iAuxMpz4, aM2);
		// aux5 = delta_i
		mpz_mul(this->iAuxMpz5, aAux, this->iAuxMpz5);
		mpz_mod(this->iAuxMpz5,this->iAuxMpz5,aM2);
		// aux3 = ai = |ai| + delta_i
		mpz_add(this->iAuxMpz3, this->iAuxMpz3, this->iAuxMpz5);
		this->iResultFirstPoly->Set(i, this->iAuxMpz3);
	}
	// aux3 = a0 = (r1 - a1)/p
	mpz_sub(this->iAuxMpz5, this->iAuxMpz2, this->iAuxMpz3);
	mpz_fdiv_q(this->iAuxMpz3,this->iAuxMpz5,aM2);
	this->iResultFirstPoly->Set(0, this->iAuxMpz3);

	mpz_clear(aAux);

	return ConstRC::Ok;
}

/*
Improve and Save generated polynomial
*/
int CKleinjung2PolySelPhase::SaveCandidate() //int aIndex1, int aIndex2, int aDPower1, int aDPower2, int aL, int aLHalf
{
	int rc;

	//Anezky vypis
	printf("\n----BEFORE improve---------\n");
	/*
	this->iResultFirstPoly->PrintToScreen();
	printf(", ");
	gmp_printf(", m1 = %Zd",this->iMRoot1);
	printf(", ");
	gmp_printf(", m2 = %Zd",this->iMRoot2);
	printf("\n-------------\n");
	*/

	// improve
	// iPolyImprovement should be inicialized before calling Improve and reset after calling Improve because of allocation of used variables
	this->iPolyImprovement->Init(this->iPolyDegree);
	// Improve generated polynomial, input is m, empty skewness, again m, root2 m_2
	rc = this->iPolyImprovement->Improve(this->iResultFirstPoly, this->iAuxMpz1, this->iSkewness, this->iAuxMpz1, this->iMRoot2);
	
	this->iPolyImprovement->Reset();
	if (rc == ConstRC::GoodCandidate)	///TODO zkontrolovat improvement
	{
		// f_{2} = m_2 x - m_1
		this->iResultSecondPoly->Set(1, this->iMRoot2);
		this->iResultSecondPoly->Set(0, this->iMRoot1);

		
		// get rating
		//rc = ConstRC::Ok;
		rc = Rating(this->iRating, this->iAlpha, this->iSkewness);	///TODO zkontrolovat rating
		if (rc != ConstRC::Ok)
		{
			return rc;
		}

		/*
		//Murphy E rating
		double murphyEfce = MurphyE(5000000, 4000000, this->iRegSize, 5); //TODO Debug
		cout << "Murphy's E is " << murphyEfce << endl;
		*/
		//mpf_set_d(this->iRating, (double) MurphyE(5000000, 4000000, this->iRegSize, 15));

		//Anezky vypis
		printf("\n-----AFTER improve--------\n");
		this->iResultFirstPoly->PrintToScreen();
		printf(", ");
		this->iResultSecondPoly->PrintToScreen();
		//gmp_printf(", m1 = %Zd",this->iAuxMpz1);
		printf("\n-------------\n");

		//double murphyEfce = MurphyE(5000000, 4000000, this->iRegSize, 15); //TODO Debug
		//cout << "Murphy's E is " << murphyEfce << endl; //Adela vypis a p�idat i do save as jako dal�� polo�ku(?)

		// save as candidate
		rc = this->iCandidatePolyPairs->AddCandidatePair(this->iResultFirstPoly, this->iResultSecondPoly, this->iAuxMpz1, this->iAlpha, this->iRating, this->iSkewness);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
		this->iFoundCandidatePolyCount++;
	}
	else if (rc != ConstRC::Ok)
	{
		return rc;
	}

	return ConstRC::Ok;
}

/*
TODO Rating - Vytahnout ven? Bude pro oba Klein1 i Klein2 stejn�
*/
int CKleinjung2PolySelPhase::Rating(mpf_t aResult, mpf_t aAlpha, mpf_t aSkewness)
{
	if (mpf_cmp_ui(aSkewness,0) <= 0)	//impossible to sqrt negative value or divide by sqrt of zero
	{
		CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","Rating",ConstRC::BadArgument,0,"Bad value of argument aSkewness.");
		return ConstRC::BadArgument;
	}
	mpf_sqrt(this->iAUxMpfS1, aSkewness);
	mpf_ui_div(this->iAUxMpfS2, 1, this->iAUxMpfS1);
	AlphaHomPoly(this->iAUxMpfA1, this->iRoots, this->iResultFirstPoly);
	AlphaHomPoly(this->iAUxMpfA2, this->iRoots, this->iResultSecondPoly);

	mpf_set_ui(aResult ,0);
	for (long i = 0; i < this->iSubintervalsCount; i++)
	{
		EvaluateEllipseAngle(this->iAuxMpfSum, this->iResultFirstPoly, i, this->iAUxMpfS1, this->iAUxMpfS2);
		mpf_set_d(this->iAuxMpf, NFSUtils::Ln(this->iAuxMpfSum));
		mpf_add(this->iAuxMpfSum, this->iAUxMpfA1, this->iAuxMpf);
		mpf_div(this->iAuxMpf, this->iAuxMpfSum, this->iFBBoundLog[0]);
		Rho(aAlpha, this->iAuxMpf); // we use aAlpha as aux variable

		EvaluateEllipseAngle(this->iAuxMpfSum, this->iResultSecondPoly, i, this->iAUxMpfS1, this->iAUxMpfS2);
		mpf_set_d(this->iAuxMpf, NFSUtils::Ln(this->iAuxMpfSum));
		mpf_add(this->iAuxMpfSum, this->iAUxMpfA2, this->iAuxMpf);
		mpf_div(this->iAuxMpf, this->iAuxMpfSum, this->iFBBoundLog[1]);
		Rho(this->iAuxMpfSum, this->iAuxMpf);

		mpf_mul(this->iAuxMpfSum, this->iAuxMpfSum, aAlpha);
		mpf_add(aResult, aResult, this->iAuxMpfSum);
	}

	mpf_add(this->iAuxMpfSum, this->iAUxMpfA1, this->iAUxMpfA2);
	mpf_div_ui(aAlpha, this->iAuxMpfSum, 2);

	return ConstRC::Ok;
}

/*
Kleinjung 2 algorithm
Generating candidates - set a_{d}, generate m_1 and m_2, call FindPolynomials which is base-(m-1, m_2)
*/
int CKleinjung2PolySelPhase::Generate()
{
	//initialiaztion of all constants, empty values and polynomilas
	int rc = PrepareForGenerating();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	if (Get_InfoMode())
	{
		CLog::Log(10000, "1101039", "\n  Generating candidates ...\n");
	}
#endif

	//TODO only for debug
	mpz_set_ui(this->iAd, 100020);
	//mpz_set_ui(this->iAdMax, 932400);
	//mpz_set_ui(this->iAdMax, 907000);

	//gmp_printf("iAdMax = %Zd \n", iAdMax);

	// set and increase the leading coefficient a_{d}
	for (; mpz_cmp(this->iAdMax, this->iAd) > 0; this->iRound++, mpz_add_ui(this->iAd, this->iAd, this->iAdBase))
	{
		int rc;
		//vypis
 		printf("\r");
 		printf("iRound = %ld", iRound);
 		gmp_printf("\t iAd = %Zd", iAd);
 		fflush(stdout);
		
		if (this->iRound % this->iRSerialRunAdChangeCount == 0)
		{
			rc = RunRunningSerialization();	///TODO Anezka? nic neloguje => dopsat sem logovani?
			if (rc != ConstRC::Ok)
			{
				CLog::LogError(WHEREARG, "CKleinjung2PolySelPhase","Generate",rc,0,"An error occurs during calling RunRunningSerialization.");
				return rc;
			}
		}


		//{K2}set N tilde = d^d * a_d^(d-1) * N 
		mpz_mul_ui (this->idAd, this->iAd, this->iPolyDegree);
		mpz_pow_ui (this->iNtilde, this->idAd, this->iPolyDegree - 1);
		mpz_mul_ui (this->iNtilde, this->iNtilde, this->iPolyDegree);
		mpz_mul (this->iNtilde, this->iNtilde, this->iN); 

		//{K2}set auxilarity value m_0
		mpz_root (this->iMRoot0, this->iNtilde, this->iPolyDegree);


		//{K2}generates an array of N tilde (mod p^2) elements of the set P
		rc = GenerateNModSetPQ(this->iSetP, this->iSetPSize);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}

		//{K2}solving congruences mod p^2, result is a list of pairs (p, rp)
		rc = SolveCongrModSqare(iPairSolutionPr, this->iNumberOfPairsPr, this->iSetP, this->iSetPSize, this->iPrimitiveElementsP, this->iMatrixPowersP);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
		
		//{K2}set N tilde and generates an array of N tilde (mod q^2) elements of the set Q
		rc = GenerateNModSetPQ(this->iSetQ, this->iSetQSize);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}

		//{K2}solving congruences mod q^2, result is a list of pairs (q, rq)
		rc = SolveCongrModSqare( iPairSolutionQr, this->iNumberOfPairsQr, this->iSetQ, this->iSetQSize, this->iPrimitiveElementsQ, this->iMatrixPowersQ);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
		

		//{K2}for each q finds pairs (p, i) and collisins are identified. Prepares values iPreM2 and iCollision.
		rc = CalculateRpRqI();
		if (rc != ConstRC::Ok)
		{
			return rc;
		}

		//if iPreM2 = p_1*p_2*q was found continue to find m_1 and m_2 a generate polynomials, otherwise there is no solution for this iAd
		if (this->iPreM2 != 0)
		{
			PrepareM1M2(this->iPreM2, this->iCollision);
		}
		else
		{
			cout << endl << " No solution found, no polynomial for this leading coefficient a_d. "<<endl;
		}

	//TODO - check: clear fields with additive values.
	PartialResetAd();
	}

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	if (Get_InfoMode())
	{
		CLog::Log(10000, "1101040", "\n\n  Generating candidates ... Done.\n\n");
	}
#endif
	return ConstRC::Ok;
}

int CKleinjung2PolySelPhase::PrepareRunningSerialization()
{
	this->iStats->ConfirmAll();
	this->iTimes->ConfirmAll();

	this->iRSerialFoundCandPolyCount = this->iFoundCandidatePolyCount;
	this->iRSerialRound = this->iRound;

	mpz_set(this->iRSerialAd, this->iAd);

	if (this->iCandidatePolyPairs != NULL && this->iRSerialCandidatePolyPairs != NULL)
	{
		int rc = this->iCandidatePolyPairs->Copy2(this->iRSerialCandidatePolyPairs);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101024", "  WARNING - Candidate poly pairs object not allocated - running serialization!\n");
		}
	}
#endif

	return ConstRC::Ok;
}
/*!
	Shouldn't be call from outside!!!
*/
int CKleinjung2PolySelPhase::Serialize(const CBaseParameters &aParam) const
{
	int rc = MSerializable::Serialize(aParam);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	CBaseParameters paramb;
	if (Running())
	{
		AFactorAlgParameters * param = Get_Parameters();

		// for deserialization at new start
		param->SetPhaseParameter(ConstPN::DESERIALIZE, NFSUtils::ConvertToString(true), Get_PhaseNbr());
		param->SetPhaseParameter(ConstPN::PHASE_FULL_FILENAME, 
			MSerializable::CreateFileNameAuto(aParam, ConstFN::POLY_PHASE), Get_PhaseNbr());

		paramb.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, Get_ParametersFullFileName()); 
		rc = param->Serialize(paramb);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	if (this->iRSerialCandidatePolyPairs != NULL)
	{
		// we want serialize only assigned polynomials
		string auxStr = NFSUtils::ConvertToString(this->iRSerialCandidatePolyPairs->Get_Count());
		paramb.SetParameter(ConstPN::MAX_ELEMENT_COUNT, auxStr);
		// we want serialize rating only for rated polynomials - all polynomials are rated
		paramb.SetParameter(ConstPN::MAX_RATED_ELEMENT_COUNT, auxStr);
		paramb.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iCandPolyPairsFullFileName);
		rc = this->iRSerialCandidatePolyPairs->Serialize(paramb);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	return ConstRC::Ok; 
}
/*!
	Shouldn't be call from outside!!!
*/
int CKleinjung2PolySelPhase::Serialize(const CBaseParameters &aParam, xmlTextWriterPtr &aWriter) const
{
	string mainTagName;	
	aParam.GetStringParameter(ConstPN::SERIALIZATION_ELEMENT_NAME, XML_SERIAL_KLEINJUNG_POLY_SEL_PHASE_NAME, mainTagName);

	int rc = xmlTextWriterStartElement(aWriter, BAD_CAST mainTagName.c_str());
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"CKleinjung2PolySelPhase","Serialize",rc,0,"Could not write the parameter tag.");
		return ConstRC::GeneralError;
	}
	rc = CXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_INNER_STATE_NAME, Get_InnerState());
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedLong(aWriter, XML_SERIAL_FOUND_CAND_POLY_NAME, this->iRSerialFoundCandPolyCount);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedLong(aWriter, XML_SERIAL_ROUND_NAME, this->iRSerialRound);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeMpz(aWriter, XML_SERIAL_RUNNING_AD, this->iRSerialAd);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	// TODO - add serialization of iPolyImprovement - save its inner state

	CBaseParameters param;
	rc = this->iStats->Serialize(param, aWriter);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = this->iTimes->Serialize(param, aWriter);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = xmlTextWriterEndElement(aWriter); // mainTagName
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"CKleinjung2PolySelPhase","Serialize",rc,0,"Could not end the parameter tag.");
		return ConstRC::GeneralError;
	}
	rc = CXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	return ConstRC::Ok; 
}
/*!
	Shouldn't be call from outside!!!
*/
int CKleinjung2PolySelPhase::Deserialize(const CBaseParameters &aParam)
{
	// we can deserialize only during initialization
	if (Get_InnerState() != EPSPhaseInitialization)
	{
		return ConstRC::CannotBeDone;
	}

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	if (Get_InfoMode())
	{
		CLog::Log(10000, "1101037", " Deserializing Kleinjung Polynomial Selection Phase...\n\n");
	}
#endif

	int rc = MSerializable::Deserialize(aParam);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	if (MSerializable::ExistFile(this->iCandPolyPairsFullFileName))
	{
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
		if (Get_InfoMode())
		{
			CLog::Log(10000, "1101038", " Deserializing found polynomial pairs...\n\n");
		}
#endif

		CBaseParameters param;
		param.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iCandPolyPairsFullFileName);
		this->iCandidatePolyPairs = new CCandidatePairs(this->iCandidatesCount);
		rc = this->iCandidatePolyPairs->Deserialize(param);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	return ConstRC::Ok;
}
/*!
	Shouldn't be call from outside!!!
*/
int CKleinjung2PolySelPhase::Deserialize(const CBaseParameters &aParam, xmlTextReaderPtr &aReader)
{
	// we can deserialize only during initialization
	if (Get_InnerState() != EPSPhaseInitialization)
	{
		return ConstRC::CannotBeDone;
	}

	string mainTagName;	
	aParam.GetStringParameter(ConstPN::SERIALIZATION_ELEMENT_NAME, XML_SERIAL_KLEINJUNG_POLY_SEL_PHASE_NAME, mainTagName);

	int rc;
	xmlChar* name = xmlTextReaderName(aReader);

	if (name == NULL)
	{
		CLog::LogError(WHEREARG,"CKleinjung2PolySelPhase","Deserialize",0,0,"Deserialization main element name not found.");
		return ConstRC::GeneralError;  
	}

	if (xmlStrEqual(name,BAD_CAST mainTagName.c_str()) == 0)
	{
		CLog::LogError(WHEREARG,"CKleinjung2PolySelPhase","Deserialize",0,0,"Main element name is wrong.");
		return ConstRC::GeneralError; 
	}

	int auxInt = 0;
	int depth = xmlTextReaderDepth(aReader);
	CBaseParameters param;

	while ((rc = CXMLHelper::MoveToNextNode(aReader, depth)) == ConstRC::Ok)
	{
		name = xmlTextReaderName(aReader);
	
		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_INNER_STATE_NAME))
		{
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize inner state...", auxInt, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			Set_InnerState(auxInt);
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_RUNNING_AD))
		{
			rc = CXMLHelper::DeserializeMpz(aReader, "Deserialize running a_d...", this->iAd, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			mpz_set(this->iRSerialAd, this->iAd);
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_ROUND_NAME))
		{
			rc = CXMLHelper::DeserializeSignedLong(aReader, "Deserialize round...", this->iRound, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			this->iRSerialRound = this->iRound;
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_FOUND_CAND_POLY_NAME))
		{
			rc = CXMLHelper::DeserializeSignedLong(aReader, "Deserialize found candidate poly pairs count...", this->iFoundCandidatePolyCount, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			this->iRSerialFoundCandPolyCount = this->iFoundCandidatePolyCount;
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_STATISTIC_CONTAINER_NAME))
		{
			rc = this->iStats->Deserialize(param, aReader);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_TIME_CONTAINER_NAME))
		{
			rc = this->iTimes->Deserialize(param, aReader);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			continue;
		}
	}

	if (rc == ConstRC::GeneralError || rc == ConstRC::NotEnoughMemory)
	{
		return rc;
	}

	return ConstRC::Ok;
}	
