#include <abstract_factor_alg_parameters.h>
#include <base_parameters.h>
#include <common_definitions.h>
#include <const_filenames.h>
#include <const_parameter_names.h>
#include <const_return_codes.h>
#include <constants.h>
#include <enums.h>
#include <enums_common.h>
#include <gmp.h>
#include <integer_class.h>
#include <kleinjung_poly_sel_phase.h>
#include <log.h>
#include <kleinjung2_poly_sel_phase.h>
//#include <math.h>
#include <nfs_utils.h>
#include <phase_functor.h>
#include <polynom_class.h>
#include <statistic_container.h>
#include <time_container.h>
#include <xml_helper.h>
//#include "test_suite.h"
#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <string>
#include <vector>

using namespace std;


CKleinjungPolySelPhase::CKleinjungPolySelPhase() : MSerializable(ConstFN::POLY_PHASE)
{
	this->iFoundCandidatePolyCount = 0;
	this->iRSerialFoundCandPolyCount = 0;
	this->iRSerialRunAdChangeCount = 0;
	this->iRSerialRunPolyCount = 0;
	this->iRound = 0;
	this->iRSerialRound = 0;

	for (int i = 0; i < ConstPSPhase::GENERATED_POLYS_COUNT; i++)
	{
		mpf_init_set_d(this->iFBBoundLog[i], 0);
	}

	this->iCandidatePolyPairs = NULL;
	this->iRSerialCandidatePolyPairs = NULL;
	this->iResultFirstPoly = NULL;
	this->iResultSecondPoly = NULL;

	mpz_init(this->iSievingIntervalLength);

	mpz_init(this->iAuxBound);
	mpz_init(this->iAuxBound2);
	mpz_init(this->iAuxBoundD);
	mpz_init(this->iAdMax);
	mpz_init(this->iAd1Max);
	mpz_init(this->iAd2Max);
	mpz_init(this->iAuxMRoot); 
	mpz_init(this->iMRoot0); 
	mpz_init(this->iP);
	mpz_init(this->iAd);
	mpz_init(this->iAdMin);
	mpz_init(this->iRSerialAd);

	mpz_init(this->iAuxMpz1);
	mpz_init(this->iAuxMpz2);
	mpz_init(this->iAuxMpz3);
	mpz_init(this->iAuxMpz4);
	mpz_init(this->iAuxMpz5);

	mpf_init2(this->iSkewness, ConstPSPhase::MPF_T_HIGH_PRECISION);
	mpf_init2(this->iRating, ConstPSPhase::MPF_T_HIGH_PRECISION);
	mpf_init2(this->iAlpha, ConstPSPhase::MPF_T_HIGH_PRECISION);

	mpf_init(this->iAuxMpfSum);
	mpf_init(this->iAuxMpf);
	mpf_init(this->iAUxMpfS1);
	mpf_init(this->iAUxMpfS2);
	mpf_init(this->iAUxMpfA1);
	mpf_init(this->iAUxMpfA2);
	
	this->iPolyDegree = 0;
	this->iSetPSize = 0;
	this->iSetQadSize = 0;
	this->iL = 0;
	this->iLMax = 0;
	this->iPrimeBound = 0;
	this->iF0 = 0.0;
	this->iAdBase = 0;
	this->iMatrixPPRowsAllocated = 0;

	this->iSetQad = NULL;
	this->iPP = NULL;
	this->iLSizes = NULL;
	this->iSetP = NULL;
	this->iPrimitiveElements = NULL;
	this->iNModSetP = NULL;
	this->iMatrixPP = NULL;
	this->iMatrixX = NULL;
	this->iMatrixXMod = NULL;
	this->iMatrixM = NULL;
	this->iMatrixE = NULL;
	this->iMatrixF = NULL;
	this->iMatrixPowers = NULL;
	this->iESearch1 = NULL;
	this->iESearch2 = NULL;
	this->iRoots = NULL;

	this->iPolyImprovement = NULL;

	Set_AutoSerializable(ConstPSPhase::AUTO_SERIALIZABLE);
	Set_InnerState((int)EPSPhaseNotStarted);

	this->iStats = new CStatContainer(ConstPSPhase::INFO_STAT_COUNT);
	this->iTimes = new CTimeContainer(ConstPSPhase::INFO_TIME_COUNT);

}

CKleinjungPolySelPhase::~CKleinjungPolySelPhase()
{
	Reset();

	if (this->iStats != NULL)
	{
		delete this->iStats;
		this->iStats = NULL;
	}
	if (this->iTimes != NULL)
	{
		delete this->iTimes;
		this->iTimes = NULL;
	}

	DisposeFunctorField();
	DisposeGMP();
	DisposeMutexes();
	DisposeAcceptedReturnCodes();
}

int CKleinjungPolySelPhase::Reset()
{
	int i, j;

	this->iFoundCandidatePolyCount = 0;
	this->iRSerialFoundCandPolyCount = 0;
	this->iRound = 0;
	this->iRSerialRound = 0;

	mpz_set_ui(this->iAuxBound, 0);
	mpz_set_ui(this->iAuxBound2, 0);
	mpz_set_ui(this->iAuxBoundD, 0);
	mpz_set_ui(this->iAdMax, 0);
	mpz_set_ui(this->iAdMin, 0);
	mpz_set_ui(this->iAd, 0);
	mpz_set_ui(this->iAd1Max, 0);
	mpz_set_ui(this->iAd2Max, 0);
	mpz_set_ui(this->iAuxMRoot, 0); 
	mpz_set_ui(this->iP, 0);

	mpz_set_ui(this->iRSerialAd, 0);

	if (this->iSetQad != NULL)
	{
		free(this->iSetQad);
		this->iSetQad = NULL;
	}
	if (this->iPP != NULL)
	{
		free(this->iPP);
		this->iPP = NULL;
	}
	if (this->iLSizes != NULL)
	{
		free(this->iLSizes);
		this->iLSizes = NULL;
	}
	if (this->iSetP != NULL)
	{
		free(this->iSetP);
		this->iSetP = NULL;
	}
	if (this->iPrimitiveElements != NULL)
	{
		free(this->iPrimitiveElements);
		this->iPrimitiveElements = NULL;
	}
	if (this->iNModSetP != NULL)
	{
		free(this->iNModSetP);
		this->iNModSetP = NULL;
	}
	if (this->iESearch1 != NULL)
	{
		free(this->iESearch1);
		this->iESearch1 = NULL;
	}
	if (this->iESearch2 != NULL)
	{
		free(this->iESearch2);
		this->iESearch2 = NULL;
	}
	if (this->iRoots != NULL)
	{
		free(this->iRoots);
		this->iRoots = NULL;
	}
	
	if (this->iMatrixPP != NULL)
	{
		for (i = 0; i < this->iMatrixPPRowsAllocated; i++)
		{
			if (this->iMatrixPP[i] != NULL)
			{
				free(this->iMatrixPP[i]);
				this->iMatrixPP[i] = NULL;
			}
		}
		free(this->iMatrixPP);
		this->iMatrixPP = NULL;
	}
	if (this->iMatrixX != NULL)
	{
		for (i = 0; i < this->iLMax; i++)
		{
			if (this->iMatrixX[i] != NULL)
			{
				for (j = 0; j < this->iPolyDegree; j++)
				{
					mpz_clear(this->iMatrixX[i][j]);
				}
				free(this->iMatrixX[i]);
				this->iMatrixX[i] = NULL;
			}
		}
		free(this->iMatrixX);
		this->iMatrixX = NULL;
	}
	if (this->iMatrixM != NULL)
	{
		for (i = 0; i < this->iLMax; i++)
		{
			if (this->iMatrixM[i] != NULL)
			{
				for (j = 0; j < this->iPolyDegree; j++)
				{
					mpz_clear(this->iMatrixM[i][j]);
				}
				free(this->iMatrixM[i]);
				this->iMatrixM[i] = NULL;
			}
		}
		free(this->iMatrixM);
		this->iMatrixM = NULL;
	}
	if (this->iMatrixE != NULL)
	{
		for (i = 0; i < this->iLMax; i++)
		{
			if (this->iMatrixE[i] != NULL)
			{
				for (j = 0; j < this->iPolyDegree; j++)
				{
					mpz_clear(this->iMatrixE[i][j]);
				}
				free(this->iMatrixE[i]);
				this->iMatrixE[i] = NULL;
			}
		}
		free(this->iMatrixE);
		this->iMatrixE = NULL;
	}
	if (this->iMatrixF != NULL)
	{
		for (i = 0; i < this->iLMax; i++)
		{
			if (this->iMatrixF[i] != NULL)
			{
				free(this->iMatrixF[i]);
				this->iMatrixF[i] = NULL;
			}
		}
		free(this->iMatrixF);
		this->iMatrixF = NULL;
	}
	if (this->iMatrixXMod != NULL)
	{
		for (i = 0; i < this->iSetPSize; i++)
		{
			if (this->iMatrixXMod[i] != NULL)
			{
				free(this->iMatrixXMod[i]);
				this->iMatrixXMod[i] = NULL;
			}
		}
		free(this->iMatrixXMod);
		this->iMatrixXMod = NULL;
	}
	if (this->iMatrixPowers != NULL)
	{
		for (i = 0; i < this->iSetPSize; i++)
		{
			if (this->iMatrixPowers[i] != NULL)
			{
				free(this->iMatrixPowers[i]);
				this->iMatrixPowers[i] = NULL;
			}
		}
		free(this->iMatrixPowers);
		this->iMatrixPowers = NULL;
	}

	this->iPolyDegree = 0;
	this->iSetPSize = 0;
	this->iSetQadSize = 0;
	this->iL = 0;
	this->iLMax = 0;
	this->iPrimeBound = 0;
	this->iAdBase = 0;
	this->iMatrixPPRowsAllocated = 0;
	this->iF0 = 0.0;

	for (i = 0; i < ConstPSPhase::GENERATED_POLYS_COUNT; i++)
	{
		mpf_set_d(this->iFBBoundLog[i], 0);
	}

	if (this->iPolyImprovement != NULL)
	{
		delete this->iPolyImprovement;
		this->iPolyImprovement = NULL;
	}

	if (this->iCandidatePolyPairs != NULL)
	{
		delete this->iCandidatePolyPairs;
		this->iCandidatePolyPairs = NULL;
	}

	if (this->iRSerialCandidatePolyPairs != NULL)
	{
		delete this->iRSerialCandidatePolyPairs;
		this->iRSerialCandidatePolyPairs = NULL;
	}

	if (this->iResultFirstPoly != NULL)
	{
		delete this->iResultFirstPoly;
		this->iResultFirstPoly = NULL;
	}

	if (this->iResultSecondPoly != NULL)
	{
		delete this->iResultSecondPoly;
		this->iResultSecondPoly = NULL;
	}

	if (this->iStats != NULL)
	{
		this->iStats->ResetAll();
	}
	if (this->iTimes != NULL)
	{
		this->iTimes->ResetAll();
	}

	APolyPhase::Reset();

	Set_InnerState((int)EPSPhaseNotStarted);
	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::DisposeGMP()
{
	mpz_clear(this->iAuxBound);
	mpz_clear(this->iAuxBound2);
	mpz_clear(this->iAuxBoundD);
	mpz_clear(this->iAdMax);
	mpz_clear(this->iAd1Max);
	mpz_clear(this->iAd2Max);
	mpz_clear(this->iAuxMRoot); 
	mpz_clear(this->iMRoot0); 
	mpz_clear(this->iP);
	mpz_clear(this->iAdMin);
	mpz_clear(this->iAd);

	mpz_clear(this->iSievingIntervalLength);

	for (int i = 0; i < ConstPSPhase::GENERATED_POLYS_COUNT; i++)
	{
		mpf_clear(this->iFBBoundLog[i]);
	}

	mpz_clear(this->iRSerialAd);

	mpz_clear(this->iAuxMpz1);
	mpz_clear(this->iAuxMpz2);
	mpz_clear(this->iAuxMpz3);
	mpz_clear(this->iAuxMpz4);
	mpz_clear(this->iAuxMpz5);

	mpf_clear(this->iSkewness);
	mpf_clear(this->iRating);
	mpf_clear(this->iAlpha);

	mpf_clear(this->iAuxMpfSum);
	mpf_clear(this->iAuxMpf);
	mpf_clear(this->iAUxMpfS1);
	mpf_clear(this->iAUxMpfS2);
	mpf_clear(this->iAUxMpfA1);
	mpf_clear(this->iAUxMpfA2);

	return APolyPhase::DisposeGMP();
}

int CKleinjungPolySelPhase::DisposeMutexes()
{
	return APolyPhase::DisposeMutexes();
}

int CKleinjungPolySelPhase::CleanUp(AFactorAlgParameters* aParameters, int aPhaseNBR, cleaning_types aType, bool aError)
{
	if (aParameters == NULL)
	{
		CLog::LogError(WHEREARG,"CKleinjungPolySelPhase","CleanUp",0,0,"Parameters are NULL.");
		return ConstRC::BadArgument;
	}

	string identifier;
	string directory = "";
	string file;

	int phaseNBR;
	if (aParameters->GetIntegerPhaseParameter(ConstPN::POLY_PHASE_NUMBER, phaseNBR, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CKleinjungPolySelPhase","CleanUp",0,0,"Didn't find phase number.");
		return ConstRC::GeneralError;
	}

	if (aParameters->GetStringPhaseParameter(ConstPN::SERIALIZATION_IDENTIFIER, identifier, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CKleinjungPolySelPhase","CleanUp",0,0,"Didn't find serial identifier.");
		return ConstRC::GeneralError;
	}
	aParameters->GetStringPhaseParameter(ConstPN::SERIALIZATION_DIRECTORY, directory, (int)EMain);

	switch (aType)
	{
		case ECleanAllIfNotError:
			if (aError)
			{
				break;
			}
		case ECleanAll:
		case ECleanAllExcCF:
		case ECleanAllExcCFRF:
			// Running Serialization full filename - default is same as iSerialPhaseFullFileName
			if (aParameters->ContainsPhaseParameter(ConstPN::RSERIAL_POLY_SEL_FULL_FILENAME, phaseNBR))
			{
				aParameters->GetStringPhaseParameter(ConstPN::RSERIAL_POLY_SEL_FULL_FILENAME, file, phaseNBR);
				MSerializable::AdaptPathSeparator(file);
			}
			else
			{
				aParameters->CreateFullFileName(file, ConstPN::RSERIAL_POLY_SEL_FILENAME, 
					ConstPN::RSERIAL_POLY_SEL_DIRECTORY, ConstPN::RSERIAL_POLY_SEL_COMPRESSION, directory,
					ConstFN::POLY_PHASE, identifier, phaseNBR);
			}
			remove(file.c_str());

			// Candidate polynomials full filename
			if (aParameters->ContainsPhaseParameter(ConstPN::CAND_POLY_PAIR_FULL_FILENAME, phaseNBR))
			{
				aParameters->GetStringPhaseParameter(ConstPN::CAND_POLY_PAIR_FULL_FILENAME, file, phaseNBR);
				MSerializable::AdaptPathSeparator(file);
			}
			else
			{
				aParameters->CreateFullFileName(file, ConstPN::CAND_POLY_PAIR_FILENAME, 
					ConstPN::CAND_POLY_PAIR_DIRECTORY, ConstPN::CAND_POLY_PAIR_COMPRESSION, directory,
					ConstFN::CANDIDATE_POLY_PAIRS, identifier, phaseNBR);
			}
			remove(file.c_str());
			break;
		default:
			break;
	}

	return APolyPhase::CleanUp(aParameters, phaseNBR, aType, aError);
}	

int CKleinjungPolySelPhase::FillFunctorField()
{
	if (Get_FunctorsInit() == false)
	{
		Set_FunctorsInit(true);
		this->iPhaseFunctors.reserve(ConstPSPhase::INNER_STATE_COUNT);
		this->iPhaseFunctors.clear();
		this->iPhaseFunctors.push_back(NULL); // EPSPhaseNotStarted
		this->iPhaseFunctors.push_back(NULL); // EPSPhaseInitialization
		this->iPhaseFunctors.push_back(new CPhaseFunctor<CKleinjungPolySelPhase>(this, &CKleinjungPolySelPhase::GenerateCandidates)); // EPSPhaseSearchingForCandidates
		this->iPhaseFunctors.push_back(new CPhaseFunctor<CKleinjungPolySelPhase>(this, &CKleinjungPolySelPhase::ChooseBest)); // EPSPhaseChoosingFromCandidates
		this->iPhaseFunctors.push_back(new CPhaseFunctor<CKleinjungPolySelPhase>(this, &CKleinjungPolySelPhase::SaveResult)); // EPSPhaseSavingResult
		this->iPhaseFunctors.push_back(NULL); // EPSPhaseFinished
	}

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::InitParameters()
{
	Set_InnerState((int)ESPhaseInitialization);

	AFactorAlgParameters * param = Get_Parameters();

	if (param == NULL)
	{
		CLog::LogError(WHEREARG,"CKleinjungPolySelPhase","InitParameters",0,0,"Parameters aren't set.");
		return ConstRC::GeneralError;
	}

	// generally we don't know order of the sieving phase
	int phaseNBR;
	if (param->GetIntegerPhaseParameter(ConstPN::POLY_PHASE_NUMBER, phaseNBR, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CKleinjungPolySelPhase","InitParameters",0,0,"Didn't find phase number.");
		return ConstRC::GeneralError;
	}
	Set_PhaseNbr(phaseNBR);

	// now we initialize members from parents's classes
	int rc = APolyPhase::InitParameters();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	// get sieving interval length
	if (param->GetMpzPhaseParameter(ConstPN::SIEVING_REGION_WIDTH, this->iSievingIntervalLength, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CKleinjungPolySelPhase","InitParameters",0,0,"Didn't find sieving interval length.");
		return ConstRC::GeneralError;
	}
	//get poly degree step
	int polyDegreeStep = 0;
	param->GetIntegerPhaseParameter(ConstPN::KLEINJUNG_POLY_DEGREE_STEP, ConstPSPhase::POLY_DEGREE_STEP, polyDegreeStep, phaseNBR);
	// set degree of the first polynomial
	this->iPolyDegree = mpz_sizeinbase(this->iN, 2) / polyDegreeStep;	// should for N < 105 dec => d = 4, N > 105 dec => d = 5
	if (this->iPolyDegree < ConstPSPhase::KLEINJUNG_MIN_POLY_DEGREE)
	{
		this->iPolyDegree = ConstPSPhase::KLEINJUNG_MIN_POLY_DEGREE;
	}

	// get a_{d} base
	int auxInt = 0;
	param->GetIntegerPhaseParameter(ConstPN::LEADING_COEFF_BASE, ConstPSPhase::KLEINJUNG_LEADING_COEFF_BASE, auxInt, phaseNBR);
	this->iAdBase = (unsigned long)auxInt;

	// get a_{d} lower bound
	if (param->ContainsPhaseParameter(ConstPN::LEADING_COEFF_LOWER_BOUND, phaseNBR))
	{
		param->GetMpzPhaseParameter(ConstPN::LEADING_COEFF_LOWER_BOUND, this->iAdMin, phaseNBR);
	}
	else
	{
		// set default value
		mpz_set_si(this->iAdMin, ConstPSPhase::KLEINJUNG_LEADING_COEFF_LOWER_BOUND);
	}

	// set a_{d} - initialization setup - deserialization can change
	if (mpz_cmp_ui(this->iAdMin, this->iAdBase) > 0)
	{
		//set a_{d} as multiple of base bigger than a_{d} lower bound
		mpz_cdiv_q_ui(this->iAd, this->iAdMin, this->iAdBase);
		mpz_mul_ui(this->iAd, this->iAd, this->iAdBase);
	}
	else
	{
		//set a_{d} as base
		mpz_set_ui(this->iAd, this->iAdBase);
	}

	// get l - minimal number of primes in factorization of p
	param->GetIntegerPhaseParameter(ConstPN::KLEINJUNG_L, ConstPSPhase::KLEINJUNG_DEFAULT_L, this->iL, phaseNBR);

	// get l_{max} - maximal number of primes in factorization of p - default value is 2 * l
	param->GetIntegerPhaseParameter(ConstPN::KLEINJUNG_L_MAX, this->iL * 2, this->iLMax, phaseNBR);

	// consistency check of l
	if (this->iL > this->iLMax)
	{
		// set max as l
		this->iLMax = this->iL;
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101027", "  WARNING - l_{max} is less than l - reset to l\n");
		}
#endif		
	}

	string auxStr;
	long bound = 0;
	// we try to find value for all 
	param->GetLongPhaseParameter(ConstPN::FB_UPPER_BOUND, bound, bound, (int)EMain);
	for (int i = 0; i < ConstPSPhase::GENERATED_POLYS_COUNT; i++)
	{
		auxStr = NFSUtils::ConvertToString(i + 1);
		// we try to find individual value
		param->GetLongPhaseParameter(ConstPN::FB_UPPER_BOUND + auxStr, bound, bound, (int)EMain);
		mpf_init_set_d(this->iFBBoundLog[i], log((double)bound));
	}

	// get iMatrixPPRowsAllocated - number of possible subsets of Q_{a_{d}} of size l
	param->GetIntegerPhaseParameter(ConstPN::KLEINJUNG_POSSIBLE_SUBSETS_PP, ConstPSPhase::KLEINJUNG_POSSIBLE_SUBSETS_PP, this->iMatrixPPRowsAllocated, phaseNBR);

	// get p_{b} - bound for primes in factorization of p
	param->GetIntegerPhaseParameter(ConstPN::KLEINJUNG_PRIME_BOUND, ConstPSPhase::KLEINJUNG_DEFAULT_PRIME_BOUND, this->iPrimeBound, phaseNBR);

	// get chi inverse - ratio size of poly coefficients
	param->GetIntegerPhaseParameter(ConstPN::CHI_INVERSE, ConstPSPhase::KLEINJUNG_CHI_INVERSE, this->iChiInverse, phaseNBR);

	param->GetLongPhaseParameter(ConstPN::RSERIALIZATION_AD_CHANGE_COUNT, ConstPSPhase::AD_CHANGE_COUNT, this->iRSerialRunAdChangeCount, phaseNBR);
	param->GetLongPhaseParameter(ConstPN::RSERIALIZATION_POLY_COUNT, this->iCandidatesCount, this->iRSerialRunPolyCount, phaseNBR);

	// Running Serialization full filename - default is same as iSerialPhaseFullFileName
	if (param->ContainsPhaseParameter(ConstPN::RSERIAL_POLY_SEL_FULL_FILENAME, phaseNBR))
	{
		param->GetStringPhaseParameter(ConstPN::RSERIAL_POLY_SEL_FULL_FILENAME, this->iRunningSerialFullFileName, phaseNBR);
		MSerializable::AdaptPathSeparator(this->iRunningSerialFullFileName);
	}
	else
	{
		param->CreateFullFileName(this->iRunningSerialFullFileName, ConstPN::RSERIAL_POLY_SEL_FILENAME, 
			ConstPN::RSERIAL_POLY_SEL_DIRECTORY, ConstPN::RSERIAL_POLY_SEL_COMPRESSION, Get_SerialMainDirectory(),
			ConstFN::POLY_PHASE, Get_SerialIdentifier(), phaseNBR);
	}

	// Candidate polynomials full filename
	if (param->ContainsPhaseParameter(ConstPN::CAND_POLY_PAIR_FULL_FILENAME, phaseNBR))
	{
		param->GetStringPhaseParameter(ConstPN::CAND_POLY_PAIR_FULL_FILENAME, this->iCandPolyPairsFullFileName, phaseNBR);
		MSerializable::AdaptPathSeparator(this->iCandPolyPairsFullFileName);
	}
	else
	{
		param->CreateFullFileName(this->iCandPolyPairsFullFileName, ConstPN::CAND_POLY_PAIR_FILENAME, 
			ConstPN::CAND_POLY_PAIR_DIRECTORY, ConstPN::CAND_POLY_PAIR_COMPRESSION, Get_SerialMainDirectory(),
			ConstFN::CANDIDATE_POLY_PAIRS, Get_SerialIdentifier(), phaseNBR);
	}

	// should we deserialize or serialize?
	bool deserialize = false;
	param->GetBoolPhaseParameter(ConstPN::DESERIALIZE, false, deserialize, Get_PhaseNbr());
	CBaseParameters bparam;
	bparam.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, Get_SerialPhaseFullFileName());

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	if (Get_InfoMode())
	{
		CLog::Log(10000, "1101000", "Kleinjung Polynomial Selection Phase is running ...\n\n");
	}
#endif

	if (deserialize)
	{
		rc = Deserialize(bparam);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}
	else
	{
		rc = Serialize(bparam);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::PrepareForGenerating()
{

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	if (Get_InfoMode())
	{
		CLog::Log(5000, "1101020", "  Prepare for generating.\n");
	}
#endif

	// set bound M - auxiliary bound used for calculation
	// M < sqrt(N, d + 1) - set according to chi = max(|a_{i}|/m) for i = 0,..,d
	// => M = sqrt(N, d + 1) * chi
	mpz_root(this->iAuxMpz1, this->iN, (unsigned long)(this->iPolyDegree + 1));  // max M is the (degree + 1)-th root of N
	mpz_fdiv_q_ui(this->iAuxBound, this->iAuxMpz1, (unsigned long)this->iChiInverse);
	// set bound M^{2} - auxiliary bound used for calculation
	mpz_pow_ui(this->iAuxBound2, this->iAuxBound, 2);

	// set bound M^{(2d - 6)/(d - 2)} - auxiliary bound used for calculation
	// aux1 = (M^{2})^{d - 3}
	mpz_pow_ui(this->iAuxMpz1, this->iAuxBound2, (unsigned long)(this->iPolyDegree - 3));
	// MD = sqrt(d - 2, aux1)
	mpz_root(this->iAuxBoundD, this->iAuxMpz1, (unsigned long)(this->iPolyDegree - 2));

	// set upper bound for a_{d}
	// a_{d, max} = (M^{2d - 2} / N)^{1 / (d - 3)}
	// aux1 = M^{2d - 2}
	mpz_pow_ui(this->iAuxMpz1, this->iAuxBound, (unsigned long)(2 * this->iPolyDegree - 2));
	// aux2 = aux1 / N
	mpz_fdiv_q(this->iAuxMpz2, this->iAuxMpz1, this->iN);
	// a_{d, max} = sqrt(aux2, d - 3)
	if (this->iPolyDegree > 4)
	{
		mpz_root(this->iAdMax, this->iAuxMpz2, (unsigned long)(this->iPolyDegree - 3));
	}
	else
	{
		mpz_set(this->iAdMax, this->iAuxMpz2); 
	}

	// check a_{d, min} and a_{d, max}
	if (mpz_cmp(this->iAdMin, this->iAdMax) >= 0)
	{
		// reset a_{d, min}
		mpz_set_si(this->iAdMin, ConstPSPhase::KLEINJUNG_LEADING_COEFF_LOWER_BOUND);
		// write warning
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101004", "  Lower bound for leading coefficient was reset to default value.\n");
		}
#endif
	}

	// write info
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	if (Get_InfoMode())
	{
		CLog::Log(5000, "", "\n");
		CLog::Log(5000, "1101005", "  The first poly degree is %d \n", this->iPolyDegree);
		CLog::Log(5000, "1101006", "  Auxiliary coeff bound is %Zd \n", this->iAuxBound);
		CLog::Log(5000, "1101007", "  Upper bound for leading coeff is %Zd \n", this->iAdMax);
		CLog::Log(5000, "1101008", "  Lower bound for leading coeff is %Zd \n", this->iAdMin);
	}
#endif

	int i, j;
	int rc;

	// generate set P of prime r fulfilling r = 1 mod d, r -| N, r < p_{b}
	// this method also set value iSetPSize
	rc = GenerateSetP();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}


	// allocate iSetQad - set Q_{a_{d}} of primes r from P fulfilling a_{d}/N != 0 mod r and it is d-th power modulo r, allocated size of the array is same as P = iSetPSize
	if (this->iSetQad == NULL)
	{
		this->iSetQad = (int*) NFSUtils::AllocationMachine((void **) &(this->iSetQad), this->iSetPSize * sizeof(int));
		if (!this->iSetQad)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iSetPSize,0,"There was not enough memory to allocate array.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iSetQad, 0, this->iSetPSize * sizeof(int));
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101015", "  WARNING - array Qad already allocated!\n");
		}
	}
#endif	

	// allocate iLSizes - Array of actual sizes l of subsets of Q_{a_{d}} (allocated as iMatrixPPRowsAllocated)
	if (this->iLSizes == NULL)
	{
		this->iLSizes = (int*) NFSUtils::AllocationMachine((void **) &(this->iLSizes), this->iMatrixPPRowsAllocated * sizeof(int));
		if (!this->iLSizes)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iMatrixPPRowsAllocated,0,"There was not enough memory to allocate array.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iLSizes, 0, this->iMatrixPPRowsAllocated * sizeof(int));
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101028", "  WARNING - array L sizes already allocated!\n");
		}
	}
#endif	

	// allocate iPP - subset of Q_{a_{d}} of minimal size l (allocated as l_{max})
	if (this->iPP == NULL)
	{
		this->iPP = (int*) NFSUtils::AllocationMachine((void **) &(this->iPP), this->iLMax * sizeof(int));
		if (!this->iPP)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iLMax,0,"There was not enough memory to allocate array.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iPP, 0, this->iLMax * sizeof(int));
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101029", "  WARNING - set PP already allocated!\n");
		}
	}
#endif	

	// get d^{[l_{max}]/2}
	i = (this->iLMax / 2) + 1;
	j = CIntegerClass::IPow(this->iPolyDegree, i);
	// allocate iESearch1 - aux array for finding values mod Z in [-e, e]
	if (this->iESearch1 == NULL)
	{
		this->iESearch1 = (e_search_element*) NFSUtils::AllocationMachine((void **) &(this->iESearch1), j * sizeof(e_search_element));
		if (!this->iESearch1)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",j,0,"There was not enough memory to allocate array.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iESearch1, 0, j * sizeof(e_search_element));
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101033", "  WARNING - set E search 1 already allocated!\n");
		}
	}
#endif	

	// allocate iESearch2 - aux array for finding values mod Z in [-e, e]
	if (this->iESearch2 == NULL)
	{
		this->iESearch2 = (e_search_element*) NFSUtils::AllocationMachine((void **) &(this->iESearch2), j * sizeof(e_search_element));
		if (!this->iESearch2)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",j,0,"There was not enough memory to allocate array.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iESearch2, 0, j * sizeof(e_search_element));
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101034", "  WARNING - set E search 2 already allocated!\n");
		}
	}
#endif	

	// allocate iRoots - roots modulo prime of selected polynomials - for rating calculation
	if (this->iRoots == NULL)
	{
		this->iRoots = (long*) NFSUtils::AllocationMachine((void **) &(this->iRoots), this->iPolyDegree * sizeof(long));
		if (!this->iRoots)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iPolyDegree,0,"There was not enough memory to allocate array.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iRoots, 0, this->iPolyDegree * sizeof(long));
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101035", "  WARNING - roots already allocated!\n");
		}
	}
#endif	

	// allocate iMatrixPP - matrix of subsets of Q_{a_{d}} of size l_{max}, number of rows is given in variable iMatrixPPRowsAllocated - parameter with default value
	if (this->iMatrixPP == NULL)
	{
		this->iMatrixPP = (int **) NFSUtils::AllocationMachine((void **) &this->iMatrixPP, this->iMatrixPPRowsAllocated * sizeof(int*));
		if (!this->iMatrixPP)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iMatrixPPRowsAllocated,0,"There was not enough memory to allocate matrix rows.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iMatrixPP, 0, this->iMatrixPPRowsAllocated * sizeof(int*));
		for (i = 0; i < this->iMatrixPPRowsAllocated; i++)
		{
			this->iMatrixPP[i] = (int*) NFSUtils::AllocationMachine((void **) &(this->iMatrixPP[i]), this->iLMax * sizeof(int));
			if (!this->iMatrixPP[i])
			{
				CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iLMax,0,"There was not enough memory to allocate matrix columns.");
				return ConstRC::NotEnoughMemory;
			}
			memset(this->iMatrixPP[i], 0, this->iLMax * sizeof(int));
		}
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101014", "  WARNING - matrix PP already allocated!\n");
		}
	}
#endif

	// allocate iMatrixX - l_{max} x d matrix of x_{i,j} - d^{l} solutions of N = a_{d} * x^{d} mod p
	if (this->iMatrixX == NULL)
	{
		this->iMatrixX = (mpz_t **) NFSUtils::AllocationMachine((void **) &this->iMatrixX, this->iLMax * sizeof(mpz_t*));
		if (!this->iMatrixX)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iLMax,0,"There was not enough memory to allocate matrix rows.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iMatrixX, 0, this->iLMax * sizeof(mpz_t*));
		for (i = 0; i < this->iLMax; i++)
		{
			this->iMatrixX[i] = (mpz_t*) NFSUtils::AllocationMachine((void **) &(this->iMatrixX[i]), this->iPolyDegree * sizeof(mpz_t));
			if (!this->iMatrixX[i])
			{
				CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iPolyDegree,0,"There was not enough memory to allocate matrix columns.");
				return ConstRC::NotEnoughMemory;
			}
			for (j = 0; j < this->iPolyDegree; j++)
			{
				mpz_init_set_si(this->iMatrixX[i][j], 0);
			}
		}
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101009", "  WARNING - matrix X already allocated!\n");
		}
	}
#endif	

	// allocate iMatrixXMod - iSetPSize x d matrix of solutions of N = a_{d} * x^{d} mod r from Q_{a_{d}}, number of used rows is actual size of Q_{a_{d}}
	if (this->iMatrixXMod == NULL)
	{
		this->iMatrixXMod = (int **) NFSUtils::AllocationMachine((void **) &this->iMatrixXMod, this->iSetPSize * sizeof(int*));
		if (!this->iMatrixXMod)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iSetPSize,0,"There was not enough memory to allocate matrix rows.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iMatrixXMod, 0, this->iSetPSize * sizeof(int*));
		for (i = 0; i < this->iSetPSize; i++)
		{
			this->iMatrixXMod[i] = (int*) NFSUtils::AllocationMachine((void **) &(this->iMatrixXMod[i]), this->iPolyDegree * sizeof(int));
			if (!this->iMatrixXMod[i])
			{
				CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iPolyDegree,0,"There was not enough memory to allocate matrix columns.");
				return ConstRC::NotEnoughMemory;
			}
			memset(this->iMatrixXMod[i], 0, this->iPolyDegree * sizeof(int));
		}
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101010", "  WARNING - matrix X mod already allocated!\n");
		}	
	}
#endif	

	// allocate iMatrixM - l_{max} x d matrix of m_{i,j} 
	if (this->iMatrixM == NULL)
	{
		this->iMatrixM = (mpz_t **) NFSUtils::AllocationMachine((void **) &this->iMatrixM, this->iLMax * sizeof(mpz_t*));
		if (!this->iMatrixM)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iLMax,0,"There was not enough memory to allocate matrix rows.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iMatrixM, 0, this->iLMax * sizeof(mpz_t*));
		for (i = 0; i < this->iLMax; i++)
		{
			this->iMatrixM[i] = (mpz_t*) NFSUtils::AllocationMachine((void **) &(this->iMatrixM[i]), this->iPolyDegree * sizeof(mpz_t));
			if (!this->iMatrixM[i])
			{
				CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iPolyDegree,0,"There was not enough memory to allocate matrix columns.");
				return ConstRC::NotEnoughMemory;
			}
			for (j = 0; j < this->iPolyDegree; j++)
			{
				mpz_init_set_si(this->iMatrixM[i][j], 0);
			}
		}
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101011", "  WARNING - matrix M already allocated!\n");
		}	
	}
#endif	

	// allocate iMatrixE - l_{max} x d matrix of e_{i,j}
	if (this->iMatrixE == NULL)
	{
		this->iMatrixE = (mpz_t **) NFSUtils::AllocationMachine((void **) &this->iMatrixE, this->iLMax * sizeof(mpz_t*));
		if (!this->iMatrixE)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iLMax,0,"There was not enough memory to allocate matrix rows.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iMatrixE, 0, this->iLMax * sizeof(mpz_t*));
		for (i = 0; i < this->iLMax; i++)
		{
			this->iMatrixE[i] = (mpz_t*) NFSUtils::AllocationMachine((void **) &(this->iMatrixE[i]), this->iPolyDegree * sizeof(mpz_t));
			if (!this->iMatrixE[i])
			{
				CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iPolyDegree,0,"There was not enough memory to allocate matrix columns.");
				return ConstRC::NotEnoughMemory;
			}
			for (j = 0; j < this->iPolyDegree; j++)
			{
				mpz_init_set_si(this->iMatrixE[i][j], 0);
			}
		}
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101012", "  WARNING - matrix E already allocated!\n");
		}	
	}
#endif	

	// allocate double **	iMatrixF - l_{max} x d matrix of f_{i,j}
	if (this->iMatrixF == NULL)
	{
		this->iMatrixF = (double **) NFSUtils::AllocationMachine((void **) &this->iMatrixF, this->iLMax * sizeof(double*));
		if (!this->iMatrixF)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iLMax,0,"There was not enough memory to allocate matrix rows.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iMatrixF, 0, this->iLMax * sizeof(double*));
		for (i = 0; i < this->iLMax; i++)
		{
			this->iMatrixF[i] = (double*) NFSUtils::AllocationMachine((void **) &(this->iMatrixF[i]), this->iPolyDegree * sizeof(double));
			if (!this->iMatrixF[i])
			{
				CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","PrepareForGenerating",this->iPolyDegree,0,"There was not enough memory to allocate matrix columns.");
				return ConstRC::NotEnoughMemory;
			}
			memset(this->iMatrixF[i], 0, this->iPolyDegree * sizeof(double));
		}
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101013", "  WARNING - matrix F already allocated!\n");
		}	
	}
#endif	

	// allocate polynomial improvement class
	if (this->iPolyImprovement == NULL)
	{
		this->iPolyImprovement = new CPolynomialImprovement(this->iCandidatesCount);
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101036", "  WARNING - polynomial improvement class already allocated!\n");
		}	
	}
#endif

	// allocate candidate pairs containers
	if (this->iCandidatePolyPairs == NULL)
	{
		this->iCandidatePolyPairs = new CCandidatePairs(this->iCandidatesCount);
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101025", "  WARNING - candidate poly pairs already allocated!\n");
		}	
	}
#endif

	if (this->iRSerialCandidatePolyPairs == NULL)
	{
		this->iRSerialCandidatePolyPairs = new CCandidatePairs(this->iCandidatesCount);
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101026", "  WARNING - rserial candidate poly pairs allocated!\n");
		}	
	}
#endif
	
	// allocate and fill matrix Powers, allocate array of primitive elements, useful by generating of iSetQad
	// we need set P and size of P
	rc = GeneratePowers();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	// allocate and fill array of N mod r from set P, useful by generating of iSetQad
	rc = GenerateNModSetP();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	this->iResultFirstPoly = new Polynomial(this->iPolyDegree);
	this->iResultSecondPoly = new Polynomial(1);

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::GenerateCandidates()
{
	Set_InnerState(EPSPhaseSearchingForCandidates);

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	if (Get_InfoMode())
	{
		CLog::Log(10000, "1101019", "  Generating polynomial ...\n\n");
	}
#endif

	this->iTimes->Start(EPSTGeneratePoly);

	CBaseParameters param;
	param.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iRunningSerialFullFileName);
	int rc = StartRunningSerialization(param);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = Generate();
	if (rc != ConstRC::Ok)
	{
		EndRunningSerialization();
		return rc;
	}

	rc = EndRunningSerialization();	

	this->iTimes->End(EPSTGeneratePoly);

	return rc;
}

int CKleinjungPolySelPhase::ChooseBest()
{
	Set_InnerState(EPSPhaseChoosingFromCandidates);

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	if (Get_InfoMode())
	{
		CLog::Log(10000, "1101042", "  Total number of found polynomial: %d\n", this->iFoundCandidatePolyCount);
		CLog::Log(10000, "1101001", "  Choosing the best polynomial ...\n\n");
	}
#endif

	if (this->iFoundCandidatePolyCount <= 0 || this->iCandidatePolyPairs->Get_Count() <= 0)
	{
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","ChooseBest",ConstRC::NoCandidates,0,"No polynomial candidates found.");
		return ConstRC::NoCandidates;
	}

	if (this->iResultFirstPoly != NULL)
	{
		delete this->iResultFirstPoly;
	}
	if (this->iResultSecondPoly != NULL)
	{
		delete this->iResultSecondPoly;
	}

	// best polynomial pair is on the first position
	// TODO - feel free to use more complicated way of determining the best poly pair - testing sieving ??
	this->iResultFirstPoly = new Polynomial();
	this->iCandidatePolyPairs->FirstPoly[0]->Copy(this->iResultFirstPoly);
	this->iResultSecondPoly = new Polynomial();
	this->iCandidatePolyPairs->SecondPoly[0]->Copy(this->iResultSecondPoly);
	mpz_set(this->iRoot, this->iCandidatePolyPairs->Root[0]);

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	if (Get_InfoMode())
	{
		CLog::Log(5000, "1101002", "  The best polynomial pair found is\n");
		this->iResultFirstPoly->PrintToScreen();
		this->iResultSecondPoly->PrintToScreen();
		CLog::Log(5000, "1101003", "\twith alpha = %Ff and rating = %Ff\n\n", this->iCandidatePolyPairs->Alpha[0], this->iCandidatePolyPairs->Rating[0]);
	}
#endif

#ifdef DEBUG_POLY_SEL_PHASE_KLEINJUNG_DEBUG_ON
// show all generated polynomials
	for (int i = 0; i < this->iFoundCandidatePolyCount; i++)
	{
		this->iCandidatePolyPairs->FirstPoly[i]->PrintToScreen();
		gmp_printf("alpha = %Ff\trating = %Ff\n",this->iCandidatePolyPairs->Alpha[i], this->iCandidatePolyPairs->Rating[i]);
	}
#endif


	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::SaveResult()
{
	if (this->iResultFirstPoly == NULL || this->iResultSecondPoly == NULL)
	{
		CLog::LogError(WHEREARG,"CKleinjungPolySelPhase","SaveResult",ConstRC::GeneralError,0,"Result polynomial is not determined.");
		return ConstRC::GeneralError;
	}

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	if (Get_InfoMode())
	{
		CLog::Log(10000, "1101041", "\n  Saving the chosen polynomials.\n");
	}
#endif
	
	Set_InnerState(EPSPhaseSavingResult);

	AFactorAlgParameters * param = Get_Parameters();
	string auxStr = NFSUtils::ConvertToString(this->iRoot);
	param->SetPhaseParameter(ConstPN::ROOT_M, auxStr, (int)EMain);
	param->SetPhaseParameter(ConstPN::SIEVING_POLY1, this->iResultFirstPoly->ToString(ELongFormat), (int)EMain);
	param->SetPhaseParameter(ConstPN::SIEVING_POLY2, this->iResultSecondPoly->ToString(ELongFormat), (int)EMain);

	Set_InnerState(EPSPhaseFinished);

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::GenerateSetP()
{
	if (this->iSetP != NULL)
	{
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","GenerateSetP",0,0,"Array p already allocated.");
		return ConstRC::BadArgument;
	}

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	if (Get_InfoMode())
	{
		CLog::Log(5000, "1101021", "    Generate set P.\n");
	}
#endif

	unsigned long count = 0;
	long * primes = NULL;
	long bound = this->iPrimeBound;
	int rc = CIntegerClass::GetPrimes(0, bound, count, primes);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","GenerateSetP",rc,0,"Some error during generating primes.");
		return rc;
	}
	
	int * auxArray = NULL;
	auxArray = (int*) NFSUtils::AllocationMachine((void **) &(auxArray), count * sizeof(int));
	if (!auxArray)
	{
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","GenerateSetP",count,0,"There was not enough memory to allocate array.");
		return ConstRC::NotEnoughMemory;
	}

	int * curr = auxArray;
	this->iSetPSize = 0;
	for (unsigned int i = 0; i < count; i++, primes++)
	{
		if (*primes % this->iPolyDegree == 1)
		{
			this->iSetPSize++;
			*curr = *primes;
			curr++;
		}
	}

	this->iSetP = (int*) NFSUtils::AllocationMachine((void **) &(this->iSetP), this->iSetPSize * sizeof(int));
	if (!this->iSetP)
	{
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","GenerateSetP",this->iSetPSize,0,"There was not enough memory to allocate array.");
		return ConstRC::NotEnoughMemory;
	}

	// copy values from aux array to set P
	memcpy(this->iSetP, auxArray, this->iSetPSize * sizeof(int));

	free(auxArray);
	auxArray = NULL;

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::GeneratePowers()
{
	if (this->iSetP == NULL || this->iSetPSize == 0)
	{
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","GeneratePowers",0,0,"Set P is not prepared.");
		return ConstRC::BadArgument;
	}

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	if (Get_InfoMode())
	{
		CLog::Log(5000, "1101022", "    Generate powers.\n");
	}
#endif

	// allocate iPrimitiveElements - array of primitive elements modulo primes r from P, size of the array is same as P = iSetPSize
	if (this->iPrimitiveElements == NULL)
	{
		this->iPrimitiveElements = (int*) NFSUtils::AllocationMachine((void **) &(this->iPrimitiveElements), this->iSetPSize * sizeof(int));
		if (!this->iPrimitiveElements)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","GeneratePowers",this->iSetPSize,0,"There was not enough memory to allocate array.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iPrimitiveElements, 0, this->iSetPSize * sizeof(int));
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101016", "  WARNING - array of primitive elements already allocated!\n");
		}
	}
#endif	

	// allocate iMatrixPowers - iSetPSize x iPrimeBound matrix, for element a_{i,j} holds e_{i}^a_{i,j} = j mod r_{i} where r_{i} from P and e_{i} is primitive element modulo r_{i} 
	if (this->iMatrixPowers == NULL)
	{
		this->iMatrixPowers = (int **) NFSUtils::AllocationMachine((void **) &this->iMatrixPowers, this->iSetPSize * sizeof(int*));
		if (!this->iMatrixPowers)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","GeneratePowers",this->iSetPSize,0,"There was not enough memory to allocate matrix rows.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iMatrixPowers, 0, this->iSetPSize * sizeof(int*));
		for (int i = 0; i < this->iSetPSize; i++)
		{
			this->iMatrixPowers[i] = (int*) NFSUtils::AllocationMachine((void **) &(this->iMatrixPowers[i]), this->iPrimeBound * sizeof(int));
			if (!this->iMatrixPowers[i])
			{
				CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","GeneratePowers",this->iPrimeBound,0,"There was not enough memory to allocate matrix columns.");
				return ConstRC::NotEnoughMemory;
			}
			memset(this->iMatrixPowers[i], 0, this->iPrimeBound * sizeof(int));
		}
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101018", "  WARNING - matrix Powers already allocated!\n");
		}	
	}
#endif	

	//start filling
	int rc = CIntegerClass::GetPrimitiveRootsAndPowers(this->iSetP, this->iSetPSize, this->iPrimeBound, this->iPrimitiveElements, this->iMatrixPowers);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","GeneratePowers",rc,0,"Some error during generating primitive roots and powers.");
		return rc;
	}

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::GenerateNModSetP()
{
	if (this->iSetP == NULL || this->iSetPSize == 0)
	{
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","GenerateNModSetP",0,0,"Set P is not prepared.");
		return ConstRC::BadArgument;
	}

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	if (Get_InfoMode())
	{
		CLog::Log(5000, "1101023", "    Generate N mod set P.\n");
	}
#endif

	// allocate iNModSetP - array of N modulo prime r from P, size of the array is same as P = iSetPSize
	if (this->iNModSetP == NULL)
	{
		this->iNModSetP = (int*) NFSUtils::AllocationMachine((void **) &(this->iNModSetP), this->iSetPSize * sizeof(int));
		if (!this->iNModSetP)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","GenerateNModSetP",this->iSetPSize,0,"There was not enough memory to allocate array.");
			return ConstRC::NotEnoughMemory;
		}
		memset(this->iNModSetP, 0, this->iSetPSize * sizeof(int));
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101017", "  WARNING - array N mod set P already allocated!\n");
		}
	}
#endif

	// calculate N mod r
	int * currR = this->iSetP;
	int * currN = this->iNModSetP;
	for (int i = 0; i < this->iSetPSize; i++, currR++, currN++)
	{
		mpz_mod_ui(this->iAuxMpz1, this->iN, (unsigned long)(*currR));
		*currN = mpz_get_si(this->iAuxMpz1);
	}

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::FillSetQAd()
{
	int i, j, k, l, a, f, x, c, c0, r, ad;		
	// l = size of Qad
	l = 0;	
	for (i = 0; i < this->iSetPSize; i++)	
	{
		r = this->iSetP[i];
		// check if a_{d} != 0 mod r
		ad = mpz_mod_ui(this->iAuxMpz1, this->iAd, (unsigned long)r);
		if (ad != 0)
		{
			a = CIntegerClass::InverseModNPositive(ad, r);
			a = (a * this->iNModSetP[i]) % r;
			// a = e^{f} mod r, e is primitive root modulo r
			f = this->iMatrixPowers[i][a];

			if (f % this->iPolyDegree == 0)					// if root of r exists
			{
				k = (r - 1) / this->iPolyDegree;
				x = (f / this->iPolyDegree) % k;  // e^xd = e^f mod r  -> x = f/d mod ((r-1)/d) as d|f and d|r-1
				this->iSetQad[l] = r;					
				j = 0;
				c = CIntegerClass::PowerModN(this->iPrimitiveElements[i], x, r);	// calculate first d-th root c_1 = e^x mod r
				c0 = CIntegerClass::PowerModN(this->iPrimitiveElements[i], k, r);
				do
				{
					this->iMatrixXMod[l][j] = c;
					c = (c * c0) % r;			// calculate other roots - c_i+1 = c_i * e^((r-1)/d) = c_i * c0 
					x += k;
					j++;
				}
				while (x < (r - 1));
				l++; // increase index and size of Qad
			} 
		}
	}

	this->iSetQadSize = l;	

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::AddSubsetMatrixPP(int aCount, int & aRowsCount)
{
	if (aRowsCount < this->iMatrixPPRowsAllocated)
	{
		int * setPP = this->iMatrixPP[aRowsCount];
		for (int i = 0; i < this->iLMax; i++) {
			if (i < aCount) {
				setPP[i] = this->iPP[i];
			} else {
				setPP[i] = 0;
			}
		}

		this->iLSizes[aRowsCount] = aCount;

		aRowsCount++;

		return ConstRC::Ok;
	}
	else
	{
#ifdef DEBUG_POLY_SEL_PHASE_KLEINJUNG_DEBUG_ON
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 100000
		if (Get_InfoMode())
		{
			CLog::Log(100000, "1101030", "  WARNING - AddSubsetMatrixPP - too many rows - allocate more space!\n");
		}

#endif	
#endif

		return ConstRC::All;
	}
}

int CKleinjungPolySelPhase::GenerateSubsetPP(int aPosition, int aCount, mpz_t aProd, int & aRowsCount)
{
	int rc;
	if (aCount >= this->iL && aCount < this->iLMax)
	{
		rc = AddSubsetMatrixPP(aCount, aRowsCount);
		if (rc == ConstRC::All)
		{
			// too many rows - stop
			return ConstRC::All;
		}
	}
	
	if (aCount >= this->iLMax - 1)
	{
		// too many elements in set
#ifdef DEBUG_POLY_SEL_PHASE_KLEINJUNG_DEBUG_ON
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 100000
		if (Get_InfoMode())
		{
			CLog::Log(100000, "1101031", "  WARNING - GenerateSubsetPP - too elements in PP - allocate more space!\n");
		}
#endif	
#endif
		return ConstRC::Ok;
	}

	mpz_t nextProd;
	mpz_init(nextProd);
	while (++aPosition < this->iSetQadSize)
	{
		mpz_mul_si(nextProd, aProd, this->iSetQad[aPosition]);
		if (mpz_cmp(nextProd, this->iAd1Max) <= 0)
		{
			this->iPP[aCount] = this->iSetQad[aPosition];
			rc = GenerateSubsetPP(aPosition, aCount + 1, nextProd, aRowsCount);
			if (rc == ConstRC::All)
			{
				// too many rows - stop
				return ConstRC::All;
			}
			this->iPP[aCount] = 0;
		}
		else 
		{
			break;
		}
	}
	mpz_clear(nextProd);

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::FillMatrixPP(int & aRowsCount)
{
	mpz_t prod;
	mpz_init_set_ui(prod, 1);
	aRowsCount = 0;
	// call recursive method with start values
	int rc = GenerateSubsetPP(-1, 0, prod, aRowsCount);

	mpz_clear(prod);

	if (rc != ConstRC::All && rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","FillMatrixPP",rc,0,"Some unexpected error occurs during generating subsets PP.");
		return rc;
	}

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::GenerateMatrixXandP(int aSubsetPPIndex)
{
	// get row from matrix PP with subset PP
	int * subsetPP = this->iMatrixPP[aSubsetPPIndex];
	// get size of subset PP
	int sizePP = this->iLSizes[aSubsetPPIndex];
	int i = 0;
	int j = 0;
	int k = 0;
	int * xMods = NULL;
	mpz_t * xs = NULL;
	
	// at first calculate p
	mpz_set_si(this->iP, 1);
	for (i = 0; i < sizePP; i++)
	{
		mpz_mul_si(this->iP, this->iP, subsetPP[i]);
	}

	for (i = 0; i < sizePP; i++)
	{
		// get p/r_{i}
		mpz_fdiv_q_ui(this->iAuxMpz1, this->iP, (unsigned long)subsetPP[i]);

		// get xmod_{i,j} - find current index for prime r in set Qad
		for (; j < this->iSetQadSize; j++)
		{
			if (subsetPP[i] == this->iSetQad[j])
			{
				// index found
				break;
			}
		}

		mpz_set_si(this->iAuxMpz2, subsetPP[i]);
		// g = gcd(a,b);  g = a * s + b * t, where a = p/r_i and b = r_i    
		// mpz_gcdext(mpz_t g, mpz_t s, mpz_t t, mpz_t a, mpz_t b)
		mpz_gcdext(this->iAuxMpz5, this->iAuxMpz3, this->iAuxMpz4, this->iAuxMpz1, this->iAuxMpz2); 

#ifdef DEBUG_POLY_SEL_PHASE_KLEINJUNG_DEBUG_ON
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 100000
		if (mpz_cmp_si(this->iAuxMpz5, 1) != 0)
		{
			if (Get_InfoMode())
			{
				CLog::Log(100000, "1101032", "  WARNING - GenerateMatrixXandP - gcd is %Zd!\n", this->iAuxMpz5);
			}
		}
#endif	
#endif

		// get row with solutions for r_{i}
		xMods = this->iMatrixXMod[j];
		xs = this->iMatrixX[i];

		for (k = 0; k < this->iPolyDegree; k++)
		{
			// auxMpz5 = p/r_{i} * xmod_{i,j}
			mpz_mul_si(this->iAuxMpz5, this->iAuxMpz1, xMods[k]);
			// x_{i,j} = auxMpz5 * s
			mpz_mul(xs[k], this->iAuxMpz5, this->iAuxMpz3);
			mpz_mod(xs[k], xs[k], this->iP);
		}
	}

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::GenerateMatrixM(int aSubsetPPIndex)
{
	int sizePP = this->iLSizes[aSubsetPPIndex];
	if (sizePP > 0)
	{
		int i,j;
		for (j = 0; j < this->iPolyDegree; j++)
		{
			mpz_add(this->iMatrixM[0][j], this->iMatrixX[0][j], this->iMRoot0);
		}

		for (i = 1; i < sizePP; i++)
		{
			for (j = 0; j < this->iPolyDegree; j++)
			{
				mpz_set(this->iMatrixM[i][j], this->iMatrixX[i][j]);
			}
		}
	}
	else {
		return ConstRC::BadArgument;
	}

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::GenerateMatrixE(int aSubsetPPIndex)
{
	int sizePP = this->iLSizes[aSubsetPPIndex];
	if (sizePP > 0)
	{
		// set e_{1, j} = a_{d - 1, (j,1,..,1)} mod p
		int i, j, k;

		for (j = 0; j < this->iPolyDegree; j++)
		{
			// m_{u} = /sum_{i = 1}^{l} m_{i, u_{i}}, where u = (u_{1},...,u_{l})
			// get m_{(j,1,..,1)}
			mpz_set(this->iAuxMpz1, this->iMatrixM[0][j]);
			for (k = 1; k < sizePP; k++)
			{
				mpz_add(this->iAuxMpz1, this->iAuxMpz1, this->iMatrixM[k][0]);
			}
			
			CalculateAd1ModP(this->iAuxMpz1, this->iMatrixE[0][j]);
		}

		// set e_{i, 1} = 0, for i > 1
		for (i = 1; i < sizePP; i++)
		{
			mpz_set_ui(this->iMatrixE[i][0], 0);
		}

		// set e_{i, j} = a_{d - 1, (1,..1,j,1,..,1)} - a_{d - 1, (1,1,..,1)} mod p, where j is at the i-th place, for i > 1 and j > 1
		for (i = 1; i < sizePP; i++)
		{
			for (j = 1; j < this->iPolyDegree; j++)
			{
				// get m_{(1,..,1,j,1,..,1)}
				mpz_set_ui(this->iAuxMpz1, 0);
				for (k = 0; k < sizePP; k++)
				{
					if (k != i)
					{
						mpz_add(this->iAuxMpz1, this->iAuxMpz1, this->iMatrixM[k][0]);
					}
					else
					{
						mpz_add(this->iAuxMpz1, this->iAuxMpz1, this->iMatrixM[k][j]);
					}
				}

				CalculateAd1ModP(this->iAuxMpz1, this->iMatrixE[i][j]);
				mpz_sub(this->iMatrixE[i][j], this->iMatrixE[i][j], this->iMatrixE[0][0]);
				if (mpz_cmp_ui(this->iMatrixE[i][j], 0) < 0)
				{
					mpz_add(this->iMatrixE[i][j], this->iMatrixE[i][j], this->iP);
				}
			}
		}
	} else {
		return ConstRC::BadArgument;
	}

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::CalculateAd1ModP(mpz_t aM, mpz_t & Ad1)
{	// a_{d - 1} = ((N - a_{d} * m^d) / p) * (1/ m^{d-1}) mod p ... N - a_d * m^d must be divisible by p because of the definition of base-(m,p) decomposition

	//aux2 = ad*m
	mpz_mul(this->iAuxMpz2, this->iAd, aM);
	//aux1 = m^(d-1)
	mpz_pow_ui(this->iAuxMpz1, aM, this->iPolyDegree-1);
	//aux3 = 1/m^(d-1) mod p
	mpz_invert(this->iAuxMpz3, this->iAuxMpz1, this->iP);
	//aux2 = ad * m^d
	mpz_mul(this->iAuxMpz2, this->iAuxMpz2, this->iAuxMpz1);
	//aux4 = N - ad*m^d
	mpz_sub(this->iAuxMpz4, this->iN, this->iAuxMpz2);
	//aux2 = (N - ad*m^d)/p
	mpz_divexact(this->iAuxMpz2, this->iAuxMpz4, this->iP);
	//aux1 = aux2 * aux3
	mpz_mul(this->iAuxMpz1, this->iAuxMpz2, this->iAuxMpz3);
	//ad1 = aux1 mod p
	mpz_mod(Ad1, this->iAuxMpz1, this->iP);

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::GenerateMatrixF(int aSubsetPPIndex, double & aEpsilon)
{
	// m_{0}^{d - 1}
	mpz_pow_ui(this->iAuxMpz1, this->iMRoot0, (unsigned long)(this->iPolyDegree - 1));
	// p^{2}
	mpz_pow_ui(this->iAuxMpz5, this->iP, 2);
	// p^{2} * m_{0}^{d - 1}
	mpz_mul(this->iAuxMpz2, this->iAuxMpz1, this->iAuxMpz5);
	
	// m_{0}^{d}
	mpz_mul(this->iAuxMpz1, this->iAuxMpz1, this->iMRoot0);
	// a_{d} * m_{0}^{d}
	mpz_mul(this->iAuxMpz1, this->iAuxMpz1, this->iAd);
	// N - a_{d} * m_{0}^{d}
	mpz_sub(this->iAuxMpz3, this->iN, this->iAuxMpz1);


	mpf_t result;
	mpf_t op1, op2;

	mpf_init(result);
	mpf_init(op1);
	mpf_init(op2);

	// get f_{0}
	mpf_set_z(op1, this->iAuxMpz3);
	mpf_set_z(op2, this->iAuxMpz2);
	mpf_div(result, op1, op2);
	this->iF0 = mpf_get_d(result); // mod Z
	this->iF0 = this->iF0 - CIntegerClass::Trunc(this->iF0);

	// p^{2}
	mpf_set_z(op2, this->iAuxMpz5);
	// a_{d} * d
	mpz_mul_si(this->iAuxMpz1, this->iAd, this->iPolyDegree);

	double temp;
	int i, j;
	int sizePP = this->iLSizes[aSubsetPPIndex];

	for (i = 0; i < sizePP; i++)
	{
		for (j = 0; j < this->iPolyDegree; j++)
		{
			// e_{i,j} * p
			mpz_mul(this->iAuxMpz2, this->iMatrixE[i][j], this->iP);
			// a_{d} * d * x_{i,j}
			mpz_mul(this->iAuxMpz3, this->iMatrixX[i][j], this->iAuxMpz1);
			// a_{d} * d * x_{i,j} + e_{i,j} * p
			mpz_add(this->iAuxMpz4, this->iAuxMpz2, this->iAuxMpz3);
			// - a_{d} * d * x_{i,j} - e_{i,j} * p
			mpz_neg(this->iAuxMpz4, this->iAuxMpz4);
			mpf_set_z(op1, this->iAuxMpz4);
			// (- a_{d} * d * x_{i,j} - e_{i,j} * p) / p^{2}
			mpf_div(result, op1, op2);
			// get f_{i,j}
			temp = mpf_get_d(result); // mod Z
			this->iMatrixF[i][j] = temp - CIntegerClass::Trunc(temp);
		}
	}

	//epsilon = a_{d-2, max} / m_{0}
	double auxDouble;
	mpf_set_z(op1, this->iAd2Max);
	mpf_set_z(op2, this->iMRoot0);
	mpf_div(result, op1, op2);
	auxDouble = mpf_get_d(result);
	//upper bound for epsilon
	if (auxDouble > 0.5) {
		aEpsilon = 0.5;
	} else if (auxDouble < 0.0001) {
		aEpsilon = 0.0001;
	}
	else {
		aEpsilon = auxDouble;
	}

	mpf_clear(result);
	mpf_clear(op1);
	mpf_clear(op2);
	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::FindPolynomials(int aSubsetPPIndex, double aEpsilon)
{
	int i, j;
	int lsize = this->iLSizes[aSubsetPPIndex];
	int rc;

	// get d^{[l_{max}]/2}
	int lhalf = (lsize / 2);
	if (lsize % 2 == 1)
	{
		lhalf++;
	}
	int dPower1 = CIntegerClass::IPow(this->iPolyDegree, lhalf);
	int dPower2 = dPower1;

	// generate esearch array for first half
	GenerateESearchArray(this->iESearch1, 0, 0, lhalf, 0, dPower1);
	if (lsize % 2 == 1)
	{
		dPower2 = dPower1 / this->iPolyDegree;
	}

	// generate esearch array for second half
	GenerateESearchArray(this->iESearch2, 0, lhalf, lsize, 0, dPower2);
	for (j = 0; j < dPower2; j++)
	{
		this->iESearch2[j].value = - this->iESearch2[j].value;
	}

	// qsort: sort dPowerX elements of array iESearchX (every element has sizeof(e_search_element) bytes), uses function CompareESearchElements for compare and sort
	qsort(this->iESearch1, dPower1, sizeof(e_search_element), &NFSUtils::CompareESearchElements);
	qsort(this->iESearch2, dPower2, sizeof(e_search_element), &NFSUtils::CompareESearchElements);

	// find suitable pair
	double first;
	double second;
	double lowerBound;
	double upperBound = -2.0;	// first iteration of outer loop should pass whole
	for (i = 0; i < dPower1; i++)
	{
		first = this->iESearch1[i].value + this->iF0;
		first = first - CIntegerClass::Trunc(first);
		if (first < (upperBound + aEpsilon)){
			continue;
		}
		lowerBound = first - aEpsilon;
		upperBound = first + aEpsilon;
		for (j = 0; j < dPower2; j++)
		{
			second = this->iESearch2[j].value;
			if (second < upperBound)
			{
				if (second > lowerBound)
				{
					rc = SaveCandidate(this->iESearch1[i].index, this->iESearch2[j].index, dPower1, dPower2, lsize, lhalf);

					if (rc != ConstRC::Ok)
					{
						return rc;
					}
					break;
				}
				else
				{
					// too small - stop searching
					break;
				}
			}
			//else too big - continue
		}
	}

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::GenerateESearchArray(e_search_element * aResults, double aSum, int aI, int aMaxI, int aIndex, int aDPower)
{
	int nextI = aI + 1;
	if (nextI < aMaxI)
	{
		int dPowerNext = aDPower / this->iPolyDegree;
		for (int j = 0; j < this->iPolyDegree; j++)
		{
			GenerateESearchArray(aResults, aSum + this->iMatrixF[aI][j], nextI, aMaxI, aIndex + aDPower * j, dPowerNext);
		}
	}
	else
	{
		// aDPower should be 1
		aResults = aResults + aIndex / this->iPolyDegree;
		for (int j = 0; j < this->iPolyDegree; j++, aResults++)
		{
			aResults->value = aSum + this->iMatrixF[aI][j];
			aResults->index = aIndex + j;
		}
	}
	return ConstRC::Ok;
}


int CKleinjungPolySelPhase::BaseMPExpansion(mpz_t aM, mpz_t aP)
{
	// must be GCD(m,p) = 1
	mpz_gcd(this->iAuxMpz5, aP, aM);
	if (mpz_cmp_ui(this->iAuxMpz5,1) > 0) {
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","BaseMPExpansion",ConstRC::BadArgument,0,"Base-(m,p) expansion cannot be calculated due to values m and p are not coprime.");
		return ConstRC::BadArgument;
	}
	int i;
	// aux = temporary variable
	mpz_t aAux;
	mpz_init(aAux);

	this->iResultFirstPoly->Set(this->iPolyDegree, this->iAd);
	// aux4 is dedicated for mi
	mpz_pow_ui(this->iAuxMpz4, aM, (unsigned long)this->iPolyDegree);
	// aux2 is dedicated for ri
	mpz_set(this->iAuxMpz2, this->iN);
	// aux3 is dedicated for ai
	mpz_set(this->iAuxMpz3, this->iAd);
	for (i = this->iPolyDegree - 1; i > 0; i--)
	{
		// aux2 = (ri - ai*mi)/p
		mpz_mul(this->iAuxMpz5, this->iAuxMpz3, this->iAuxMpz4);
		mpz_sub(this->iAuxMpz2, this->iAuxMpz2, this->iAuxMpz5);
		mpz_divexact(this->iAuxMpz2, this->iAuxMpz2, aP);
		// aux4 = mi/m
		mpz_divexact(this->iAuxMpz4, this->iAuxMpz4, aM);
		// aux3 = |ai| = ri/mi
		mpz_cdiv_q(this->iAuxMpz3, this->iAuxMpz2, this->iAuxMpz4);
		// find delta_i
		// aux5 = ri - |ai|*mi
		mpz_mul(this->iAuxMpz5, this->iAuxMpz3, this->iAuxMpz4);
		mpz_sub(this->iAuxMpz5, this->iAuxMpz2, this->iAuxMpz5);
		// aux = x (biggest s.t. <= aux5 and divisible by p)
		mpz_fdiv_q(aAux, this->iAuxMpz5, aP);
		mpz_mul(aAux, aAux, aP);
		// aux = gamma_i (difference from zero of aux5 mod p)
		mpz_sub(aAux, this->iAuxMpz5, aAux);
		// aux5 = inverse of mi mod p
		mpz_invert(this->iAuxMpz5, this->iAuxMpz4, aP);
		// aux5 = delta_i
		mpz_mul(this->iAuxMpz5, aAux, this->iAuxMpz5);
		mpz_mod(this->iAuxMpz5,this->iAuxMpz5, aP);
		// aux3 = ai = |ai| + delta_i
		mpz_add(this->iAuxMpz3, this->iAuxMpz3, this->iAuxMpz5);
		this->iResultFirstPoly->Set(i, this->iAuxMpz3);
	}
	// aux3 = a0 = (r1 - a1)/p
	mpz_sub(this->iAuxMpz5, this->iAuxMpz2, this->iAuxMpz3);
	mpz_fdiv_q(this->iAuxMpz3,this->iAuxMpz5, aP);
	this->iResultFirstPoly->Set(0, this->iAuxMpz3);

	mpz_clear(aAux);

	return ConstRC::Ok;
}


int CKleinjungPolySelPhase::SaveCandidate(int aIndex1, int aIndex2, int aDPower1, int aDPower2, int aL, int aLHalf)
{
	int rc;
	// get root m from indexes
	mpz_set_ui(this->iAuxMpz1, 0);
	int i, j;
	for (i = 0; i < aLHalf; i++)
	{
		if (aIndex1 >= aDPower1)
		{
			j = aIndex1 / aDPower1;
			aIndex1 = aIndex1 - j * aDPower1;
		}
		else
		{
			// too small - j = 0
			j = 0;
		}
		mpz_add(this->iAuxMpz1, this->iAuxMpz1, this->iMatrixM[i][j]);
		aDPower1 = aDPower1 / this->iPolyDegree;
	}

	for (i = aLHalf; i < aL; i++)
	{
		if (aIndex2 >= aDPower2)
		{
			j = aIndex2 / aDPower2;
			aIndex2 = aIndex2 - j * aDPower2;
		}
		else
		{
			// too small - j = 0
			j = 0;
		}
		mpz_add(this->iAuxMpz1, this->iAuxMpz1, this->iMatrixM[i][j]);
		aDPower2 = aDPower2 / this->iPolyDegree;
	}

	rc = BaseMPExpansion(this->iAuxMpz1, this->iP);
	if (rc != ConstRC::Ok) {
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","SaveCandidate",rc,0,"Some unexpected error occurs while computing base-(m,p) expansion.");
		return rc;
	}

	// improve
	// iPolyImprovement should be initialized before calling Improve and reset after calling Improve because of allocation of used variables
	this->iPolyImprovement->Init(this->iPolyDegree);
	rc = this->iPolyImprovement->Improve(this->iResultFirstPoly, this->iAuxMpz1, this->iSkewness, this->iAuxMpz1, this->iP);
	this->iPolyImprovement->Reset();
	if (rc == ConstRC::GoodCandidate)
	{
		// f_{2} = px - m
		mpz_neg(this->iAuxMpz2, this->iAuxMpz1);
		this->iResultSecondPoly->Set(1, this->iP);
		this->iResultSecondPoly->Set(0, this->iAuxMpz2);

		// get rating
		rc = Rating(this->iRating, this->iAlpha, this->iSkewness);

		if (rc != ConstRC::Ok)
		{
			return rc;
		}


		// save as candidate
		rc = this->iCandidatePolyPairs->AddCandidatePair(this->iResultFirstPoly, this->iResultSecondPoly, this->iAuxMpz1, this->iAlpha, this->iRating, this->iSkewness);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
		this->iFoundCandidatePolyCount++;

	}
	else if (rc != ConstRC::Ok)
	{
		return rc;
	}

	return ConstRC::Ok;
}

int CKleinjungPolySelPhase::Rating(mpf_t aResult, mpf_t aAlpha, mpf_t aSkewness)
{
	int rc;
	if (mpf_cmp_ui(aSkewness,0) <= 0)	//impossible to sqrt negative value or divide by sqrt of zero
	{
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","Rating",ConstRC::BadArgument,0,"Bad value of argument aSkewness.");
		return ConstRC::BadArgument;
	}
	mpf_sqrt(this->iAUxMpfS1, aSkewness);
	mpf_ui_div(this->iAUxMpfS2, 1, this->iAUxMpfS1);

	// calculate alpha(F)
	rc = AlphaHomPoly(this->iAUxMpfA1, this->iRoots, this->iResultFirstPoly);

	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","Rating",rc,0,"Some unexpected error occurs while computing AlphaHomPoly.");
		return rc;
	}

	// calculate alpha(G)
	rc = AlphaHomPoly(this->iAUxMpfA2, this->iRoots, this->iResultSecondPoly);

	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","Rating",rc,0,"Some unexpected error occurs while computing AlphaHomPoly.");
		return rc;
	}

	mpf_set_ui(aResult ,0);
	for (long i = 0; i < this->iSubintervalsCount; i++)
	{
		rc = EvaluateEllipseAngle(this->iAuxMpfSum, this->iResultFirstPoly, i, this->iAUxMpfS1, this->iAUxMpfS2);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","Rating",rc,0,"Some unexpected error occurs while computing EvaluateEllipseAngle.");
			return rc;
		}
		mpf_set_d(this->iAuxMpf, NFSUtils::Ln(this->iAuxMpfSum));
		mpf_add(this->iAuxMpfSum, this->iAUxMpfA1, this->iAuxMpf);
		mpf_div(this->iAuxMpf, this->iAuxMpfSum, this->iFBBoundLog[0]);	// iFBBoundLog[0] = "fb-upper-bound1" from input parameters
		Rho(aAlpha, this->iAuxMpf); // we use aAlpha as aux variable

		rc = EvaluateEllipseAngle(this->iAuxMpfSum, this->iResultSecondPoly, i, this->iAUxMpfS1, this->iAUxMpfS2);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","Rating",rc,0,"Some unexpected error occurs while computing EvaluateEllipseAngle.");
			return rc;
		}
		mpf_set_d(this->iAuxMpf, NFSUtils::Ln(this->iAuxMpfSum));
		mpf_add(this->iAuxMpfSum, this->iAUxMpfA2, this->iAuxMpf);
		mpf_div(this->iAuxMpf, this->iAuxMpfSum, this->iFBBoundLog[1]); // iFBBoundLog[1] = "fb-upper-bound2" from input parameters
		Rho(this->iAuxMpfSum, this->iAuxMpf);

		mpf_mul(this->iAuxMpfSum, this->iAuxMpfSum, aAlpha);
		mpf_add(aResult, aResult, this->iAuxMpfSum);
	}

	mpf_add(this->iAuxMpfSum, this->iAUxMpfA1, this->iAUxMpfA2);
	mpf_div_ui(aAlpha, this->iAuxMpfSum, 2);

	return ConstRC::Ok;
}


int CKleinjungPolySelPhase::Generate()
{
	int rc = PrepareForGenerating();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}
#ifdef DEBUG_POLY_SEL_PHASE_KLEINJUNG_DEBUG_ON
	printf("DEBUG mode ON");
#endif


///-------------------
	int i;
	int rowsCount = 0;
	double epsilon = 0;

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	if (Get_InfoMode())
	{
		CLog::Log(10000, "1101039", "\n  Generating candidates ...\n");
	}
#endif
	// generating candidates

#ifdef DEBUG_POLY_SEL_PHASE_KLEINJUNG_DEBUG_ON
	gmp_printf("iAdMax = %Zd \n", iAdMax);
#endif
	if (Get_InfoMode())
	{
		PrintHeader();
	}

	for (; mpz_cmp(this->iAdMax, this->iAd) > 0; this->iRound++, mpz_add_ui(this->iAd, this->iAd, this->iAdBase))
	{
#ifdef DEBUG_POLY_SEL_PHASE_KLEINJUNG_DEBUG_ON
 		printf("\r");
 		printf("iRound = %ld", iRound);
 		gmp_printf("\t iAd = %Zd", iAd);
 		fflush(stdout);
#endif
		
		if (this->iRound % this->iRSerialRunAdChangeCount == 0)
		{
			rc = RunRunningSerialization();
			if (rc != ConstRC::Ok)
			{
				CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","Generate",rc,0,"An error occurs during calling RunRunningSerialization.");
				return rc;
			}
		}

		// calculate iAuxMRoot - auxiliary root m of f_{i} mod N, am = sqrt(d, N / a_{d})
		mpz_fdiv_q(this->iAuxMpz1, this->iN, this->iAd);
		mpz_root(this->iAuxMRoot, this->iAuxMpz1, (unsigned long)this->iPolyDegree);

		// calculate iAd1Max - a_{d - 1, max} = M^{2} / am
		mpz_fdiv_q(this->iAd1Max, this->iAuxBound2, this->iAuxMRoot);

		// calculate iAd2Max - a_{d - 2, max} = (M^{2d - 6} / am^{d - 4})^{1 / (d - 2)}
		if (this->iPolyDegree > 4)
		{
			// aux1 = am^{d - 4}
			mpz_pow_ui(this->iAuxMpz1, this->iAuxMRoot, (unsigned long)(this->iPolyDegree - 4));
			// aux2 = M^{2d-6}
			mpz_pow_ui(this->iAuxMpz2, this->iAuxBound2, (unsigned long)(this->iPolyDegree - 3));
			// aux3 = aux2/aux1
			mpz_fdiv_q(this->iAuxMpz3, this->iAuxMpz2, this->iAuxMpz1);
			// iAd2Max = sqrt(d - 2, aux3)
			mpz_root(this->iAd2Max, this->iAuxMpz3, (unsigned long)(this->iPolyDegree - 2));
		}
		else
		{
			// am^{0} = 1
			mpz_set(this->iAd2Max, this->iAuxBoundD);
		}


		// fill set Q_{a_{d}}
		rc = FillSetQAd();
		if (rc != ConstRC::Ok)
		{
			return rc;
		}

		// generate subsets of Q_{a_{d}}
		rc = FillMatrixPP(rowsCount);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}

		// for each subset PP of Q_{a_{d}}
		for (i = 0; i < rowsCount; i++)
		{
			// calculate x_{i,j}
			rc = ConstRC::GeneralError;
			rc = GenerateMatrixXandP(i);
			if (rc != ConstRC::Ok)
			{
				CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","Generate",rc,0,"Some unexpected error occurs during generating matrices X and P.");
				return rc;
			}

			// calculate m_{0} - the smallest integer bigger then auxiliary root m and divisible by p
			mpz_fdiv_q(this->iAuxMpz1, this->iAuxMRoot, this->iP);
			mpz_add_ui(this->iAuxMpz1, this->iAuxMpz1, 1);
			mpz_mul(this->iMRoot0, this->iAuxMpz1, this->iP);

			// calculate m_{i,j}
			rc = GenerateMatrixM(i);
			if (rc != ConstRC::Ok)
			{
				CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","Generate",rc,0,"Some unexpected error occurs during generating matrix M.");
				return rc;
			}

			// calculate e_{i,j}
			rc = GenerateMatrixE(i);
			if (rc != ConstRC::Ok)
			{
				CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","Generate",rc,0,"Some unexpected error occurs during generating matrix E.");
				return rc;
			}

			// calculate f_{0} and f_{i,j}
			rc = GenerateMatrixF(i, epsilon);
			if (rc != ConstRC::Ok)
			{
				CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","Generate",rc,0,"Some unexpected error occurs during generating matrix F.");
				return rc;
			}

			rc = FindPolynomials(i, epsilon);
			if (rc != ConstRC::Ok)
			{
				CLog::LogError(WHEREARG, "CKleinjungPolySelPhase","Generate",rc,0,"Some unexpected error occurs during finding polynomials.");
				return rc;
			}
		}
		if (Get_InfoMode())
		{
			PrintValues();
		}
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	if (Get_InfoMode())
	{
		CLog::Log(10000, "1101040", "\n\n  Generating candidates ... Done.\n\n");
	}
#endif
	return ConstRC::Ok;
}

void CKleinjungPolySelPhase::PrintHeader()
{
	printf("\n\n");
	printf(" __________________________________________________");
	printf("\n");
	printf("|      a_{d}      |   alpha   |  rating  |  found  |");
	printf("\n");
}

void CKleinjungPolySelPhase::PrintValues()
{
	printf("\r|");
	gmp_printf(" %15Zd |", this->iAd);
	gmp_printf("  %3.4Ff  |", this->iAlpha);
	gmp_printf("  %3.4Ff  |", this->iRating);
	printf(" %7ld |", this->iFoundCandidatePolyCount);
	fflush(stdout);
}

int CKleinjungPolySelPhase::PrepareRunningSerialization()
{
	this->iStats->ConfirmAll();
	this->iTimes->ConfirmAll();

	this->iRSerialFoundCandPolyCount = this->iFoundCandidatePolyCount;
	this->iRSerialRound = this->iRound;

	mpz_set(this->iRSerialAd, this->iAd);

	if (this->iCandidatePolyPairs != NULL && this->iRSerialCandidatePolyPairs != NULL)
	{
		int rc = this->iCandidatePolyPairs->Copy2(this->iRSerialCandidatePolyPairs);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 5000
	else
	{
		if (Get_InfoMode())
		{
			CLog::Log(5000, "", "\n");
			CLog::Log(5000, "1101024", "  WARNING - Candidate poly pairs object not allocated - running serialization!\n");
		}
	}
#endif

	return ConstRC::Ok;
}
/*!
	Shouldn't be call from outside!!!
*/
int CKleinjungPolySelPhase::Serialize(const CBaseParameters &aParam) const
{
	int rc = MSerializable::Serialize(aParam);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	CBaseParameters paramb;
	if (Running())
	{
		AFactorAlgParameters * param = Get_Parameters();

		// for deserialization at new start
		param->SetPhaseParameter(ConstPN::DESERIALIZE, NFSUtils::ConvertToString(true), Get_PhaseNbr());
		param->SetPhaseParameter(ConstPN::PHASE_FULL_FILENAME, 
			MSerializable::CreateFileNameAuto(aParam, ConstFN::POLY_PHASE), Get_PhaseNbr());

		paramb.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, Get_ParametersFullFileName()); 
		rc = param->Serialize(paramb);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	if (this->iRSerialCandidatePolyPairs != NULL)
	{
		// we want serialize only assigned polynomials
		string auxStr = NFSUtils::ConvertToString(this->iRSerialCandidatePolyPairs->Get_Count());
		paramb.SetParameter(ConstPN::MAX_ELEMENT_COUNT, auxStr);
		// we want serialize rating only for rated polynomials - all polynomials are rated
		paramb.SetParameter(ConstPN::MAX_RATED_ELEMENT_COUNT, auxStr);
		paramb.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iCandPolyPairsFullFileName);
		rc = this->iRSerialCandidatePolyPairs->Serialize(paramb);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	return ConstRC::Ok; 
}
/*!
	Shouldn't be call from outside!!!
*/
int CKleinjungPolySelPhase::Serialize(const CBaseParameters &aParam, xmlTextWriterPtr &aWriter) const
{
	string mainTagName;	
	aParam.GetStringParameter(ConstPN::SERIALIZATION_ELEMENT_NAME, XML_SERIAL_KLEINJUNG_POLY_SEL_PHASE_NAME, mainTagName);

	int rc = xmlTextWriterStartElement(aWriter, BAD_CAST mainTagName.c_str());
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"CKleinjungPolySelPhase","Serialize",rc,0,"Could not write the parameter tag.");
		return ConstRC::GeneralError;
	}
	rc = CXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_INNER_STATE_NAME, Get_InnerState());
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedLong(aWriter, XML_SERIAL_FOUND_CAND_POLY_NAME, this->iRSerialFoundCandPolyCount);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedLong(aWriter, XML_SERIAL_ROUND_NAME, this->iRSerialRound);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeMpz(aWriter, XML_SERIAL_RUNNING_AD, this->iRSerialAd);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	// TODO - add serialization of iPolyImprovement - save its inner state

	CBaseParameters param;
	rc = this->iStats->Serialize(param, aWriter);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = this->iTimes->Serialize(param, aWriter);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = xmlTextWriterEndElement(aWriter); // mainTagName
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"CKleinjungPolySelPhase","Serialize",rc,0,"Could not end the parameter tag.");
		return ConstRC::GeneralError;
	}
	rc = CXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	return ConstRC::Ok; 
}
/*!
	Shouldn't be call from outside!!!
*/
int CKleinjungPolySelPhase::Deserialize(const CBaseParameters &aParam)
{
	// we can deserialize only during initialization
	if (Get_InnerState() != EPSPhaseInitialization)
	{
		return ConstRC::CannotBeDone;
	}

#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
	if (Get_InfoMode())
	{
		CLog::Log(10000, "1101037", " Deserializing Kleinjung Polynomial Selection Phase...\n\n");
	}
#endif

	int rc = MSerializable::Deserialize(aParam);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	if (MSerializable::ExistFile(this->iCandPolyPairsFullFileName))
	{
#if LOG_MINIMAL_MESSAGE_PRIORITY <= 10000
		if (Get_InfoMode())
		{
			CLog::Log(10000, "1101038", " Deserializing found polynomial pairs...\n\n");
		}
#endif

		CBaseParameters param;
		param.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iCandPolyPairsFullFileName);
		this->iCandidatePolyPairs = new CCandidatePairs(this->iCandidatesCount);
		rc = this->iCandidatePolyPairs->Deserialize(param);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	return ConstRC::Ok;
}
/*!
	Shouldn't be call from outside!!!
*/
int CKleinjungPolySelPhase::Deserialize(const CBaseParameters &aParam, xmlTextReaderPtr &aReader)
{
	// we can deserialize only during initialization
	if (Get_InnerState() != EPSPhaseInitialization)
	{
		return ConstRC::CannotBeDone;
	}

	string mainTagName;	
	aParam.GetStringParameter(ConstPN::SERIALIZATION_ELEMENT_NAME, XML_SERIAL_KLEINJUNG_POLY_SEL_PHASE_NAME, mainTagName);

	int rc;
	xmlChar* name = xmlTextReaderName(aReader);

	if (name == NULL)
	{
		CLog::LogError(WHEREARG,"CKleinjungPolySelPhase","Deserialize",0,0,"Deserialization main element name not found.");
		return ConstRC::GeneralError;  
	}

	if (xmlStrEqual(name,BAD_CAST mainTagName.c_str()) == 0)
	{
		CLog::LogError(WHEREARG,"CKleinjungPolySelPhase","Deserialize",0,0,"Main element name is wrong.");
		return ConstRC::GeneralError; 
	}

	int auxInt = 0;
	int depth = xmlTextReaderDepth(aReader);
	CBaseParameters param;

	while ((rc = CXMLHelper::MoveToNextNode(aReader, depth)) == ConstRC::Ok)
	{
		name = xmlTextReaderName(aReader);
	
		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_INNER_STATE_NAME))
		{
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize inner state...", auxInt, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			Set_InnerState(auxInt);
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_RUNNING_AD))
		{
			rc = CXMLHelper::DeserializeMpz(aReader, "Deserialize running a_d...", this->iAd, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			mpz_set(this->iRSerialAd, this->iAd);
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_ROUND_NAME))
		{
			rc = CXMLHelper::DeserializeSignedLong(aReader, "Deserialize round...", this->iRound, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			this->iRSerialRound = this->iRound;
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_FOUND_CAND_POLY_NAME))
		{
			rc = CXMLHelper::DeserializeSignedLong(aReader, "Deserialize found candidate poly pairs count...", this->iFoundCandidatePolyCount, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			this->iRSerialFoundCandPolyCount = this->iFoundCandidatePolyCount;
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_STATISTIC_CONTAINER_NAME))
		{
			rc = this->iStats->Deserialize(param, aReader);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_TIME_CONTAINER_NAME))
		{
			rc = this->iTimes->Deserialize(param, aReader);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			continue;
		}
	}

	if (rc == ConstRC::GeneralError || rc == ConstRC::NotEnoughMemory)
	{
		return rc;
	}

	return ConstRC::Ok;
}	
