#include "matrix_helper.h"

#include "hermite_matrix_class.h"
#include "sparse_matrix_class.h"
#include "const_return_codes.h"
#include "log.h"


AbstractMatrix * CMatrixHelper::CreateMatrix(species_of_matrices aType)
{
	switch (aType)
	{
		case SSparseMatrix:
			return new SparseMatrix();
		case SBCMatrix:
			return new BCMatrix();
		case SNormalMatrix:
			return new NormalMatrix();
		case SBitMatrix:
		default:
			return new BitMatrix();
	}
}
AbstractMatrix * CMatrixHelper::DeserializeMatrix(const CBaseParameters &aParam)
{
	species_of_matrices type = CMatrixHelper::GetMatrixSpecies(aParam);

	if (type == SAbstractMatrix)
	{
		// sign of some error
		return NULL;
	}

	AbstractMatrix * matrix = CMatrixHelper::CreateMatrix(type);

	matrix->Deserialize(aParam);

	return matrix;
}
species_of_matrices CMatrixHelper::GetMatrixSpecies(const CBaseParameters &aParam)
{
	// get file name with path
	std::string fileNameFull = MSerializable::CreateFileNameAuto(aParam, FILE_NAME_MATRIX);

	xmlTextReaderPtr reader = xmlNewTextReaderFilename(fileNameFull.c_str());
	if (reader == NULL)
	{
		CLog::LogError(WHEREARG,"CMatrixHelper","GetMatrixSpecies",0,0,"Could not create xml reader.");
		// sign of error
		return SAbstractMatrix;
	}
	int rc = CXMLHelper::MoveToNextNode(reader);
	if (rc != ConstRC::Ok)
	{
		xmlFreeTextReader(reader);
		// sign of error
		return SAbstractMatrix;
	}	

	xmlChar* name = xmlTextReaderName(reader);

	if (name == NULL)
	{
		xmlFreeTextReader(reader);
		CLog::LogError(WHEREARG,"CMatrixHelper","GetMatrixSpecies",0,0,"Deserialization main element name not found.");
		// sign of error
		return SAbstractMatrix; 
	}

	int depth = xmlTextReaderDepth(reader);
	int type = (int)SAbstractMatrix;

	while ((rc = CXMLHelper::MoveToNextNode(reader, depth)) == ConstRC::Ok)
	{
		name = xmlTextReaderName(reader);
	
		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_MATRIX_TYPE_NAME))
		{
			rc = CXMLHelper::DeserializeSignedInt(reader, "Deserialize matrix type...", type, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			break;
		}
	}
	
	xmlFreeTextReader(reader);

	if (rc != ConstRC::Ok)
	{
		// sign of error
		return SAbstractMatrix;
	}

	return (species_of_matrices)type;
}

