#ifndef __MERGE_FREQUENCY_HASHTABLE_H__
#define __MERGE_FREQUENCY_HASHTABLE_H__

#include "structures.h"

/*!
 * \brief Frequency hashtable for merging
 */
class CMergeFrequencyHashtable
{
public:
	///Internal hashtable for largish prime ideals.
	merge_fhashtable_entry *	Hashtable;
	///Indexes of relation which contain fixed largish prime ideal. Size is AllocatedSize * iMaxFrequency.
	relation_index_type * RelationIndexes;
	///Number of the allocated hashtable entries
	relation_index_type	AllocatedSize;
	///Hash mask - hash is large prime AND mask 
	relation_index_type	Mask;

private:
	///Max ideal frequency for which we remember all relations which contain given largish prime ideal.
	int iMaxFrequency;

public:

	CMergeFrequencyHashtable();
	~CMergeFrequencyHashtable();

	void DeleteHashtable();
	int ClearHashtable();
	int SetupHashtable(relation_index_type aRelationCount, int aMaxFrequency);

	int AddToTable(main_sieving_type aPrime, main_sieving_type aRootFingerprint, relation_index_type aRelationIndex, int aWeight);
	int RemoveFromTable(main_sieving_type aPrime, main_sieving_type aRootFingerprint, relation_index_type aRelationIndex, int aWeight);
	int FindPrimeInHashtable(main_sieving_type aPrime, main_sieving_type aRootFingerprint, bool & aPresent, relation_index_type & aIndex) const;

	int CountPrimeIdealsWithFrequency(unsigned int aFrequency, relation_index_type & aCountWithFreq, relation_index_type & aCountAll);

	int GetRelationIndexes(relation_index_type aIndex, relation_index_type *& aRelationIndexes);  

public:
	int Get_MaxFrequency() const { return this->iMaxFrequency; }

private:

	CMergeFrequencyHashtable(const CMergeFrequencyHashtable& aOperand) {}

	int CollisionResolution(relation_index_type aHash, main_sieving_type aPrime, main_sieving_type aRootFingerprint, relation_index_type & aIndex) const;

};

#endif // __MERGE_FREQUENCY_HASHTABLE_H__
