#include "minterruptible.h"

#include "nfs_utils.h"
#include "constants.h"
#include "const_return_codes.h"
#include "log.h"

MInterruptible::MInterruptible()
{
	this->iInnerState = ENotRunning;

	int rtn;
	if ((rtn = pthread_mutex_init(&(this->iInnerStateMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","MInterruptible",rtn,0,"Cannot initialize mutex.");
	}
	if ((rtn = pthread_mutex_init(&(this->iShouldBePausedMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","MInterruptible",rtn,0,"Cannot initialize mutex.");
	}
	if ((rtn = pthread_mutex_init(&(this->iStopMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","MInterruptible",rtn,0,"Cannot initialize mutex.");
	}
	if ((rtn = pthread_mutex_init(&(this->iPausedMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","MInterruptible",rtn,0,"Cannot initialize mutex.");
	}
	if ((rtn = pthread_cond_init(&(this->iPausedCond), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","MInterruptible",rtn,0,"Cannot initialize condition.");
	}	
}

MInterruptible::MInterruptible(const MInterruptible &aOperand) : iInnerState(aOperand.iInnerState)
{
	int rtn;
	if ((rtn = pthread_mutex_init(&(this->iInnerStateMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","MInterruptible",rtn,0,"Cannot initialize mutex.");
	}
	if ((rtn = pthread_mutex_init(&(this->iShouldBePausedMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","MInterruptible",rtn,0,"Cannot initialize mutex.");
	}
	if ((rtn = pthread_mutex_init(&(this->iStopMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","MInterruptible",rtn,0,"Cannot initialize mutex.");
	}
	if ((rtn = pthread_mutex_init(&(this->iPausedMutex), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","MInterruptible",rtn,0,"Cannot initialize mutex.");
	}
	if ((rtn = pthread_cond_init(&(this->iPausedCond), NULL)) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","MInterruptible",rtn,0,"Cannot initialize condition.");
	}
}

MInterruptible::~MInterruptible()
{
	int rtn;
	if ((rtn = pthread_mutex_destroy(&(this->iInnerStateMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","~MInterruptible",rtn,0,"Cannot destroy mutex.");
	}
	if ((rtn = pthread_mutex_destroy(&(this->iShouldBePausedMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","~MInterruptible",rtn,0,"Cannot destroy mutex.");
	}
	if ((rtn = pthread_mutex_destroy(&(this->iStopMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","~MInterruptible",rtn,0,"Cannot destroy mutex.");
	}
	if ((rtn = pthread_mutex_destroy(&(this->iPausedMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","~MInterruptible",rtn,0,"Cannot destroy mutex.");
	}
	if ((rtn = pthread_cond_destroy(&(this->iPausedCond))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","~MInterruptible",rtn,0,"Cannot destroy condition.");
	}	
}

int MInterruptible::Continue()
{
	int rtn;
	if ((rtn = pthread_mutex_lock(&(this->iPausedMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Continue",rtn,0,"Cannot lock mutex.");	
		return ConstRC::GeneralError;
	}

	bool paused = this->iPaused;

	int ret = ConstRC::Ok;

	if (paused)
	{
		// object is paused so we wake it
		if ((rtn = pthread_cond_broadcast(&(this->iPausedCond))) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"MInterruptible","Continue",rtn,0,"Cannot broadcast condition.");
			if ((rtn = pthread_mutex_unlock(&(this->iPausedMutex))) != ConstRC::Ok)
			{
				CLog::LogError(WHEREARG,"MInterruptible","Continue",rtn,0,"Cannot unlock mutex.");
				return ConstRC::GeneralError;
			}
			
			return ConstRC::GeneralError;
		}
	}
	else
	{
		ret = ConstRC::Impossible;
	}

	if ((rtn = pthread_mutex_unlock(&(this->iPausedMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Continue",rtn,0,"Cannot unlock mutex.");
		return ConstRC::GeneralError;
	}

	return ret;
}

int MInterruptible::Pause()
{
	int rtn;
	if ((rtn = pthread_mutex_lock(&(this->iPausedMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Pause",rtn,0,"Cannot lock mutex.");	
		return ConstRC::GeneralError;
	}

	bool paused = this->iPaused;

	if ((rtn = pthread_mutex_unlock(&(this->iPausedMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Pause",rtn,0,"Cannot unlock mutex.");
		return ConstRC::GeneralError;
	}

	if (paused)
	{
		// object is already paused...
		return ConstRC::Ok;
	}
	
	if ((rtn = pthread_mutex_lock(&(this->iShouldBePausedMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Pause",rtn,0,"Cannot lock mutex.");	
		return ConstRC::GeneralError;
	}

	this->iShouldBePaused = true;

	if ((rtn = pthread_mutex_unlock(&(this->iShouldBePausedMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Pause",rtn,0,"Cannot unlock mutex.");
		return ConstRC::GeneralError;
	}

	return ConstRC::Ok;
}

int MInterruptible::Stop()
{
	int rtn;
	if ((rtn = pthread_mutex_lock(&(this->iPausedMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Stop",rtn,0,"Cannot lock mutex.");	
		return ConstRC::GeneralError;
	}

	bool paused = this->iPaused;

	if ((rtn = pthread_mutex_unlock(&(this->iPausedMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Stop",rtn,0,"Cannot unlock mutex.");
		return ConstRC::GeneralError;
	}

	if ((rtn = pthread_mutex_lock(&(this->iStopMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Stop",rtn,0,"Cannot lock mutex.");	
		return ConstRC::GeneralError;
	}

	this->iStop = true;

	if ((rtn = pthread_mutex_unlock(&(this->iStopMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Stop",rtn,0,"Cannot unlock mutex.");
		return ConstRC::GeneralError;
	}

	if (paused)
	{
		// object is paused so we can stop it by destroying the thread
		return ConstRC::CannotBeDone;
	}

	if ((rtn = pthread_mutex_lock(&(this->iInnerStateMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Stop",rtn,0,"Cannot lock mutex.");	
		return ConstRC::GeneralError;
	}

	interruptible_states state = this->iInnerState;

	if ((rtn = pthread_mutex_unlock(&(this->iInnerStateMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Stop",rtn,0,"Cannot unlock mutex.");
		return ConstRC::GeneralError;
	}

	switch (state)
	{
		case EShortRun:
		case EMediumRun:
			return ConstRC::Ok;
		case ELongRun:
		case ENotRunning:
		default:
			return ConstRC::CannotBeDone;
	}
}

int MInterruptible::Control(interruptible_states aNextState)
{
	int rtn;
	if ((rtn = pthread_mutex_lock(&(this->iShouldBePausedMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Control",rtn,0,"Cannot lock mutex.");	
		return ConstRC::GeneralError;
	}

	// control pause
	if (this->iShouldBePaused)
	{
		if ((rtn = pthread_mutex_lock(&(this->iPausedMutex))) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"MInterruptible","Control",rtn,0,"Cannot lock mutex.");	
			if ((rtn = pthread_mutex_unlock(&(this->iShouldBePausedMutex))) != ConstRC::Ok)
			{
				CLog::LogError(WHEREARG,"MInterruptible","Control",rtn,0,"Cannot unlock mutex.");	
				return ConstRC::GeneralError;
			}
			return ConstRC::GeneralError;
		}
 
		// set paused
		this->iPaused = true;
		this->iShouldBePaused = false;

		if ((rtn = pthread_mutex_unlock(&(this->iShouldBePausedMutex))) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"MInterruptible","Control",rtn,0,"Cannot unlock mutex.");	
			if ((rtn = pthread_mutex_unlock(&(this->iPausedMutex))) != ConstRC::Ok)
			{
				CLog::LogError(WHEREARG,"MInterruptible","Control",rtn,0,"Cannot unlock mutex.");	
				return ConstRC::GeneralError;
			}
			return ConstRC::GeneralError;
		}

		// wait for continuation
		pthread_cond_wait(&(this->iPausedCond), &(this->iPausedMutex));

		if ((rtn = pthread_mutex_unlock(&(this->iPausedMutex))) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"MInterruptible","Control",rtn,0,"Cannot unlock mutex.");	
			return ConstRC::GeneralError;
		}
	}
	else
	{
		if ((rtn = pthread_mutex_unlock(&(this->iShouldBePausedMutex))) != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"MInterruptible","Control",rtn,0,"Cannot unlock mutex.");	
			return ConstRC::GeneralError;
		}
	}

	if ((rtn = pthread_mutex_lock(&(this->iStopMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Control",rtn,0,"Cannot lock mutex.");	
		return ConstRC::GeneralError;
	}

	int ret = ConstRC::GeneralError;

	if (this->iStop)
	{
		ret = ConstRC::StopNow;
	}

	if ((rtn = pthread_mutex_unlock(&(this->iStopMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Control",rtn,0,"Cannot unlock mutex.");	
		return ret;
	}

	if ((rtn = pthread_mutex_lock(&(this->iInnerStateMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Control",rtn,0,"Cannot lock mutex.");	
		return ret;
	}

	this->iInnerState = aNextState;

	if ((rtn = pthread_mutex_unlock(&(this->iInnerStateMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Control",rtn,0,"Cannot unlock mutex.");
		return ret;
	}

	return ret == ConstRC::StopNow ? ConstRC::StopNow : ConstRC::Ok;
}

bool MInterruptible::Get_Stop() const
{
	int rtn;
	if ((rtn = pthread_mutex_lock(&(this->iStopMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Get_Stop",rtn,0,"Cannot lock mutex.");	
		return false;
	}

	bool ret = this->iStop;

	if ((rtn = pthread_mutex_unlock(&(this->iStopMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Get_Stop",rtn,0,"Cannot unlock mutex.");
		return false;
	}

	return ret;
}

bool MInterruptible::Get_Paused() const
{
	int rtn;
	if ((rtn = pthread_mutex_lock(&(this->iPausedMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Get_Paused",rtn,0,"Cannot lock mutex.");	
		return false;
	}

	bool ret = this->iPaused;

	if ((rtn = pthread_mutex_unlock(&(this->iPausedMutex))) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"MInterruptible","Get_Paused",rtn,0,"Cannot unlock mutex.");
		return false;
	}

	return ret;
}
