#ifndef __MINTERRUPTIBLE_H__
#define __MINTERRUPTIBLE_H__

#include <pthread.h>
#include "enums.h"

///Interface for interruptible classes
class MInterruptible
{

protected:

	///state of object for determination how long will take to stop/pause
	interruptible_states iInnerState;

	///if object is paused, so it is waiting for condition for continuation
	bool iPaused;
	///object should be paused as soon as possible
	bool iShouldBePaused;
	///object should stop as soon as possible
	bool iStop;

	// mutexes
	mutable pthread_mutex_t iInnerStateMutex;
	mutable pthread_mutex_t iStopMutex;
	mutable pthread_mutex_t iPausedMutex;
	mutable pthread_mutex_t iShouldBePausedMutex;

	// condition
	mutable pthread_cond_t  iPausedCond;

protected:

	virtual int Control(interruptible_states aNextState = ELongRun);

public:

	MInterruptible();
	MInterruptible(const MInterruptible& aOperand);
	~MInterruptible();

	virtual int Stop();
	virtual int Pause();
	virtual int Continue();

	bool Get_Paused() const;
	bool Get_Stop() const;

};


#endif // __MINTERRUPTIBLE_H__