#ifndef __MONTGOMERY_POLY_SELECTION_PHASE_H__
#define __MONTGOMERY_POLY_SELECTION_PHASE_H__

#include "abstract_poly_selection_phase.h"
#include "candidate_pairs.h"
#include "const_poly_selection_phase.h"
#include "xml_helper.h"


///Montgomery's polynomial selection phase for NFS. Type (2,2)
class CMontgomeryPolySelPhase : public APolyPhase
{	
protected:

	///We have two polynomials so we need two factor bases bounds
	long		iFBBound[ConstPSPhase::GENERATED_POLYS_COUNT];
	///The inverse value of chi
	int			iChiInverse;

	mpz_t iSievingIntervalLength;
	CCandidatePairs * iCandidatePolyPairs;

	mpz_t		iRunningPrime;
	long		iRound;
	long		iFoundCandidatePolyCount;
	long		iRatedCandidatePolyCount;
	Polynomial *	iResultFirstPoly;
	Polynomial *	iResultSecondPoly;

	long		iRSerialFoundCandPolyCount;
	long		iRSerialRatedCandPolyCount;
	long		iRSerialRound;
	mpz_t		iRSerialRunningPrime;

	long		iRSerialRunPrimeChangeCount;
	long		iRSerialRunPolyCount;

	std::string iCandPolyPairsFullFileName;
	std::string iRunningSerialFullFileName;

protected:
	/*!
		Gauss base reduction (like LLL for 2-dim lattice) 
		more Cohen - A Course in Computational Algebraic Number Theory - algorithm 1.3.14
	*/
	int GaussReduction(Polynomial *& a, Polynomial *& b);

	int Generate(bool aAdvanced, bool aEnforceRealRoots);
	int Rating(mpf_t aRating, mpf_t aAlpha, long * aRoots, Polynomial *aPoly1, Polynomial *aPoly2, mpf_t aLogBase1, mpf_t aLogBase2);

	// Phase methods

	/*!
		Fill the iPhaseFunctors with correct function pointers.
		This method is called from InitParameters in APhase.
	*/
	int FillFunctorField();

	///Dispose all resources which was used and prepare for new start. Also set inner state.
	int Reset();
	int DisposeGMP();
	int DisposeMutexes();

	int InitParameters();
	int GenerateCandidates();
	int ChooseBest();
	int SaveResult();

	// Printing methods
	void PrintHeader();
	void PrintValues();
	void PrintEffectiveness();

	// For running serialization
	int PrepareRunningSerialization();

public:

	CMontgomeryPolySelPhase();
	~CMontgomeryPolySelPhase();

	///Delete all used files according to enum cleaning_types
	int CleanUp(AFactorAlgParameters* aParameters, int aPhaseNBR, cleaning_types aType, bool aError);

	int Serialize(const CBaseParameters& aParam, xmlTextWriterPtr& aWriter) const;
	int Serialize(const CBaseParameters& aParam) const;

	int Deserialize(const CBaseParameters& aParam, xmlTextReaderPtr& aReader);
	int Deserialize(const CBaseParameters& aParam);


};

#endif // __MONTGOMERY_POLY_SELECTION_PHASE_H__
