#ifndef __MULTIVARIABLE_POLYNOMIAL_H__
#define __MULTIVARIABLE_POLYNOMIAL_H__

#include "polynom_class.h"
#include "const_poly_selection_phase.h"


// don't modify the following ones unless you REALLY know what you're doing!

#define MULT_VAR_VARIABLES 5
#define C0_LOC 0
#define C1_LOC 1
#define C2_LOC 2
#define T_LOC 3
#define S_LOC 4


/*!
\brief	An auxilliary class of a multivariable term
*/
struct MultVarTerm
{
	signed int Variables[MULT_VAR_VARIABLES];	///<The exponents of variables
	mpf_t Coeff;		///<The real number coefficient
	Polynomial *XPolynomial;	///<The integer polynomial
};

/*!
\brief	An auxilliary class of multivariate polynomials

	This class serves for computations with polynomials in variables c<sub>0</sub>, c<sub>1</sub>,
	c<sub>2</sub>, t, s and x. Actually, there is always either s or x, never both. If there
	are x variables then the coefficients are integer polynomials in x, otherwise the coefficients
	are real numbers.
*/
class MultVarPolynomial
{
public:
	MultVarPolynomial(unsigned aAllocation, unsigned aPolynomialDegree);
	~MultVarPolynomial();
	int PrintToScreen();
	bool EqualPowers(signed int *aVariables1,signed int *aVariables2);
	int Copy(MultVarPolynomial *aResult);
	int Add(MultVarPolynomial *aResult, MultVarPolynomial *aOperand);
	int Multiply(MultVarPolynomial *aResult, MultVarPolynomial *aOperand);
	int Evaluate(mpf_t aResult, mpf_t **PowerTable, mpf_t *SPowerTable, unsigned aDegree);
	int PartialDerivative(MultVarPolynomial *aResult, unsigned aVariable);
	int ThreeVariableFunction(mpf_t aResult[T_LOC][T_LOC + 1], mpf_t **PowerTable, mpf_t *SPowerTable, unsigned aDegree);
	int Reset();
	
	unsigned length;	///<The number of terms
	MultVarTerm *Term;	///<The terms of the polynomial

protected:
	unsigned degree;	///<The maximal degree of x-polynomials in each term
	unsigned allocated;	///<The number of allocated terms

	mpf_t iAuxMPF;
};

#endif // __MULTIVARIABLE_POLYNOMIAL_H__