#include "newton_root_finder.h"

#include "nfs_utils.h"
#include <typeinfo>
#include "log.h"

int CNewtonRootFinder::FindRoot(Polynomial *aResult)
{
	if (NULL==aResult)
	{
		CLog::LogError(WHEREARG,(char *)typeid(this).name(),
				"NULL pointer given.",-1,-1,
				"NULL pointer given to FindRoot.");
		return -1;
	}
	int ret=0;

	printf("\n\n --> Newton Algebraic Square Root Finder <--");
	// initialize temporary variables
	Polynomial *Multiple=new Polynomial();
	Polynomial *Root=new Polynomial();
	Polynomial *TempSqRoot=new Polynomial();
	Polynomial *TempSqRoot2=new Polynomial();
	mpz_t prime,temp;
	mpz_init(prime);
	mpz_init(temp);
	unsigned long int i;
	
	do {
		printf("\n Multiplying factors...");
		fflush(stdout);
		ret = MultipleFactors(Multiple);
		if (ret!=0)
		{	CLog::LogError(WHEREARG,(char *)typeid(this).name(),
				"Attempt to multiple given factors was unsuccessful",
				-1,ret,"in method FindRoot");
			break;
		}

		// Find suitable prime (characteristic of Galois field in which we are going to use Shanks-Tonelli algorithm)
		//mpz_set_ui(prime,8000);
		mpz_set_ui(prime,30);
		mpz_nextprime(prime,prime);

		gmp_printf("\nSearching suitable prime starting from: %Zd  \n",prime);
		fflush(stdout);
		while(!iNumFieldPolynomial->IsIrreducibleModPrime(mpz_get_ui(prime)))
		{
			mpz_nextprime(prime,prime);
		};
		gmp_printf("\tsuitable prime is: %Zd  \n", prime);
		
		// Call Shanks-Tonelli in order to find square in GF(prime^d)
		ret = Shanks_Tonelli_GF(TempSqRoot,Multiple,prime);
		if (ret!=0)
		{	CLog::LogError(WHEREARG,(char *)typeid(this).name(),
				"Call to Shanks-Tonelli was unsuccessful",
				-1,ret,"in method FindRoot");
			break;
		}
		
		ComputeBoundLog(temp,prime);
		i=0;
		while(mpz_cmp_ui(temp,0)!=0){
			i++;
			mpz_tdiv_q_ui(temp,temp,2);
		};
		printf("\nIterations:%lu:",i);
		SquareRootNewtonIter(aResult,TempSqRoot,Multiple,prime,i,true,true,true);
		if (ret!=0)
		{	CLog::LogError(WHEREARG,(char *)typeid(this).name(),
				"Call to SquareRootNewtonIter was unsuccessful",
				-1,ret,"in method FindRoot");
			break;
		}
	}
	while (false);
	
	delete Multiple;
	delete Root;
	delete TempSqRoot;
	delete TempSqRoot2;
	mpz_clear(prime);
	mpz_clear(temp);
	return ret;
};

int CNewtonRootFinder::MultipleFactors(Polynomial *aResult){
	if(NULL==aResult)
	{
		CLog::LogError(WHEREARG,(char *)typeid(this).name(),"NULL pointer given.",-1,-1,
					"NULL pointer given to MultipleFactors().");
		return -1;
	};

	int ret=0;

	// Temporary polynomials
	Polynomial *Temp,*Temp2;
	Temp=new Polynomial();
	Temp2=new Polynomial();
	unsigned long int i;
	
	// Square = 1
	aResult->Zeroize();
	aResult->Set(0,1);
	// Multiply Factors ....
	for(i=0;i<iNumFactors;i++){
		Temp->Multiply(Temp,aResult,iFactors[i]);
		aResult->PowerMod(aResult,Temp,1,iNumFieldPolynomial);
			
	};

	delete Temp;
	delete Temp2;
	return ret;
};


int CNewtonRootFinder::Shanks_Tonelli_GF(Polynomial *Result,Polynomial *aSquare,mpz_t aPrime)
{
	int return_value;
	if(NULL==aSquare)
	{
		CLog::LogError(WHEREARG,(char *)typeid(this).name(),"NULL pointer given.",-1,-1,
				"NULL pointer given to Shanks_Tonelli_GF");
		return -1;
	};

	mpz_t order,temp,s,test;
	long prime = mpz_get_si(aPrime);
	unsigned long int max_power_of_two=0;
	unsigned long int r,m,i;
	Polynomial *z=new Polynomial();
	Polynomial *x=new Polynomial();
	Polynomial *b=new Polynomial();
	Polynomial *c=new Polynomial();
	Polynomial *e=new Polynomial();
	Polynomial *y=new Polynomial();
	Polynomial *t=new Polynomial();
	Polynomial *d=new Polynomial();
	aSquare->PowerInGF(e,1,iNumFieldPolynomial,aPrime);

	mpz_init(order);
	mpz_init(s);
	mpz_init(temp);
	mpz_init(test);
	do {

		mpz_set(temp,aPrime);
		mpz_pow_ui(order,temp,iNumFieldPolynomial->GetDegree());

		mpz_sub_ui(s,order,1);
		mpz_fdiv_q_ui(test,s,2);

		i= mpz_fdiv_qr_ui(s,temp,s, 2);
		while (0==i){
			i=mpz_fdiv_qr_ui(s,temp,s, 2);
			max_power_of_two++;
		}
		mpz_mul_ui(s,s,2);
		mpz_add_ui(s,s,1);


		aSquare->PowerInGF(c,test,iNumFieldPolynomial, aPrime);
		if (!c->IsOne()){
			c->PrintToScreen();
			return_value=-2;
			break;
		}

		// As in Cohen...
		// 1.step
		do {
			x->Set(rand()%iNumFieldPolynomial->GetDegree(), rand() % (prime-1) + 1);
			x->PowerInGF(c,test,iNumFieldPolynomial, aPrime);
		} while (c->IsOne());
		x->PowerInGF(y,s,iNumFieldPolynomial,aPrime);
		// 2.step
		r=max_power_of_two;
		mpz_sub_ui(s,s,1);
		mpz_fdiv_q_ui(s,s,2);
		aSquare->PowerInGF(x,s,iNumFieldPolynomial,aPrime);
		mpz_mul_ui(s,s,2);
		mpz_add_ui(s,s,1);
		x->PowerInGF(c,2,iNumFieldPolynomial,aPrime);
		aSquare->Multiply(z,aSquare,c);
		z->PowerInGF(b,1,iNumFieldPolynomial,aPrime);
		aSquare->Multiply(z,aSquare,x);
		z->PowerInGF(x,1,iNumFieldPolynomial,aPrime);

		//	b->PrintToScreen();
		// 3.step+4.step
		while  (!(b->IsOne())){
			b->Multiply(c,b,aSquare);
			c->PowerInGF(z,1,iNumFieldPolynomial,aPrime);
			x->PowerInGF(c,2,iNumFieldPolynomial,aPrime);
			m=1;
			b->PowerInGF(c,2,iNumFieldPolynomial,aPrime);
			while (!(c->IsOne())){
				c->PowerInGF(e,2,iNumFieldPolynomial,aPrime);
				e->Copy(c);
				m++;

			}
			if (m==r) {
				x->Zeroize();
				break;
			};
			// t=y^(2^(r-m-1))
			y->Copy(t);
			for(i=1;i<=r-m-1;i++)
			{
				t->PowerInGF(c,2,iNumFieldPolynomial,aPrime);
				c->Copy(t);
			}
			t->PowerInGF(y,2,iNumFieldPolynomial,aPrime);
			r=m;
			x->Multiply(c,x,t);
			c->PowerInGF(x,1,iNumFieldPolynomial,aPrime);
			b->Multiply(c,b,y);
			c->PowerInGF(b,1,iNumFieldPolynomial,aPrime);
		}
		x->Copy(Result);
		return_value=ConstRC::Ok;

	} while (false);
	delete x;
	delete b;
	delete c;
	delete d;
	delete e;
	delete y;
	delete z; delete t;
	mpz_clear(s);
	mpz_clear(temp);
	mpz_clear(order);
	mpz_clear(test);
	return return_value;
}

int CNewtonRootFinder::SquareRootNewtonIter(Polynomial *aResult, Polynomial
		*aSquareRoot, Polynomial * aSquare,mpz_t  aPrime,unsigned int
		aModulPower,bool aSwapAllowed,bool aTestSquare, bool aTestSquareMod){

	mpz_t mptemp,modul;
	mpz_init(mptemp);
	mpz_init(modul);
	mpz_set(modul,aPrime);

	//compute inverse of square root
	Polynomial *delta=new Polynomial();
	Polynomial *temp=new Polynomial();
	Polynomial *temp2=new Polynomial();
	unsigned int i;
	if(0!=aSquareRoot->InverseInGF(delta,iNumFieldPolynomial,modul))
	{
		CLog::LogError(WHEREARG,(char *)typeid(this).name(),
				"Error computing inverse",-1,-1,
				"(in SquareRootNewtonIter())");
	};
	printf("\n[");
	for(i=1;i<aModulPower;i++)
	{	
		printf(" %i ",i);
		fflush(stdout);
		mpz_mul(modul,modul,modul);
		delta->PowerInGF(temp,2,iNumFieldPolynomial,modul);
		aSquare->MultiplyInGF(temp2,temp,aSquare,iNumFieldPolynomial,modul);
		temp2->Dec(0,3);
		temp->MultiplyInGF(temp,temp2,delta,iNumFieldPolynomial,modul);
		mpz_add_ui(mptemp,modul,1);
		mpz_fdiv_q_ui(mptemp,mptemp,2);
		mpz_mul_si(mptemp,mptemp,-1);
		delta->Zeroize();
		delta->Set(0,mptemp);
		delta->Multiply(temp2,temp,delta);
		temp2->Divide(temp,delta,temp2,iNumFieldPolynomial);
		delta->Modulo(modul);
		
	};
	printf("]");
	fflush(stdout);
	if(0!=delta->MultiplyInGF(aResult,delta,aSquare,iNumFieldPolynomial,modul))
	{
		CLog::LogError(WHEREARG,(char *)typeid(this).name(),
				"Error computing inverse",-1,-1,
				"(in SquareRootNewtonIter())");
	};
	mpz_tdiv_q_ui(mptemp,modul,2);
	// in case "the other" square root was found 
	if ((aSwapAllowed)
		&&(mpz_cmpabs(*aResult->Get2(aResult->GetDegree()),mptemp)==1)){
// 		printf("Swap!!!!");
		temp->Zeroize();
		temp->Set(0,-1);
		temp->MultiplyInGF(temp2,aResult,temp,iNumFieldPolynomial,modul);
		temp2->Copy(aResult);
	};
	for(i=0;i<=(unsigned long int)aResult->GetDegree();i++) {
		mpz_fdiv_r(*aResult->Get2(i),*aResult->Get2(i),modul);
		if ((aSwapAllowed)
			&&(mpz_cmpabs(*aResult->Get2(i),mptemp)==1)){
			mpz_cdiv_r(*aResult->Get2(i),*aResult->Get2(i),modul);
		};
	};
	if(aTestSquareMod){
		delta->MultiplyInGF(temp,delta,delta,iNumFieldPolynomial,modul);
		delta->MultiplyInGF(temp2,temp,aSquare,iNumFieldPolynomial,modul);
		if(!temp2->IsOne()) printf("\n TestSquareMod: Error in Newton Iteration.");
			else printf("\n TestSquareMod: Square Root found succesfully.");
	};
	if(aTestSquare){
		temp->Multiply(temp,aResult,aResult);
		temp->PowerMod(delta,temp,1,iNumFieldPolynomial);
		temp2->Substract(temp,delta,aSquare);
		temp->Set(0,temp->Get(0)+1);
		if(!temp->IsOne()) {
			printf("\n TestSquare: Error - Not equal!");
			printf("\n Result of Newton Iteration:\n");
// 			aResult->PrintToScreen();
			printf("\n Square:\n");
// 			aSquare->PrintToScreen();
		}
		else
			printf("\n TestSquare: Square Root found succesfully");
	};
	delete delta;
	delete temp;
	delete temp2;
	mpz_clear(mptemp);
	mpz_clear(modul);
	return ConstRC::Ok;
}


int CNewtonRootFinder::ComputeBoundLog(mpz_t aResult, mpz_t aBase)
{
	mpz_t max,norm_pol,temp1,temp2;
	mpz_init(max);
	mpz_init(norm_pol);
	mpz_init(temp1);
	mpz_init(temp2);
	unsigned long int i;
	
	mpz_set_ui(max,0);
	for (i = 0; i < iNumFactors; i++)
	{
		iFactors[i]->GetMaxCoeffSize(temp1);
		if (mpz_cmp(max, temp1) < 0)
		{
			mpz_set(max, temp1);
		}
	}

	i=1;
	while(mpz_cmp_ui(max,0)!=0){
		i++;
		mpz_tdiv_q(max,max,aBase);
	};
	mpz_set_ui(max,i);
		
	
	mpz_set_ui(norm_pol,0);
	for(i=0;i<(unsigned long int)iNumFieldPolynomial->GetDegree();i++){
		mpz_mul(temp1,*(iNumFieldPolynomial->Get2(i)),*(iNumFieldPolynomial->Get2(i)));
		mpz_add(norm_pol,temp1,norm_pol);
	};
	while(mpz_cmp_ui(norm_pol,0)!=0){
		i++;
		mpz_tdiv_q(norm_pol,norm_pol,aBase);
	};
	mpz_set_ui(norm_pol,i);

	mpz_set_ui(temp1,iNumFieldPolynomial->GetDegree());
	i=1;
	while(mpz_cmp_ui(temp1,0)!=0){
		i++;
		mpz_tdiv_q(temp1,temp1,aBase);
	};
	i=i*3;
	i=i/2;
	
	mpz_set_ui(aResult,i);
	mpz_mul_ui(temp1,norm_pol,iNumFieldPolynomial->GetDegree()-2);
	mpz_add(aResult,temp1,aResult);
	mpz_add_ui(temp1,max,1);
	mpz_add(temp1,temp1,norm_pol);
	mpz_mul_ui(temp1,temp1,iNumFactors);
	mpz_tdiv_q_ui(temp1,temp1,2);
	mpz_add(aResult,aResult,temp1);

	
	mpz_clear(max);
	mpz_clear(norm_pol);
	mpz_clear(temp1);
	mpz_clear(temp2);
	return 0;
};
