#ifndef __NFS_DEFINITIONS_H__
#define __NFS_DEFINITIONS_H__

/*!
  \file This file contains definitions of NFS default constants.
*/

#include "constants.h"
#include "gmp.h"
#include <time.h>

#define NFS_VERSION     0.1

// Undef this to disable XML serialization and deserialization
// By undefing this constant, you will get a lightweight, no-XML version of NFS
// No loading/saving of states really possible.
#define WITH_XML_SERIALIZATOR

// This must be defined in order to have time messages in QS (after each X polynomials a message concerning time expensiveness of tasks).
#define TIME_MESSAGE 

// This must be defined in order to have time messages after block Lanczos run, concerning phases of Lanczos computation
#define LANCZOS_TIME_MESSAGE 
// This must be defined in order to collect time expensiveness of matrix (BitMatrix and SparseMatrix) operations.
#define MATRIX_OPERATIONS_TIME_MESSAGE

#define DEFAULT_INTERMEDIATE_ROOTS  200

// delimiters for MPQS/SIQS command line options
#define MAX_EXTENT        100001
#define MAX_POLY 	  1000000000
#define MAX_ITER 	  10000
#define MAX_OMIT 	  1000
#define IMPLICIT_INT_FACTORBASE 50000
#define IMPLICIT_ALG_FACTORBASE 50000
#define IMPLICIT_SIEVE_INT  10000
#define IMPLICIT_TMESS 	    100
#define IMPLICIT_ITER	    10
#define MAXIMAL_LINE_SIEVE_HALFLINE 100000000
#define MAXIMAL_FACTOR_BASE_UPPER_BOUND 100000000

#define INCONSISTENT_STATE -10

#define NOT_ENOUGH_INTEGRAL_PRIMES -10
#define IO_ERROR           -20
#define LEGENDRE_SYMBOL_ASSERTION_FAILED -30

//#define CHECK_SPARSE_MATRICES
#define RANK_COMPUTATIONS

#define _LOG_FACTOR     	0x1
#define DEFAULT_LOG_BASE        4
#define SIEVE_ROUNDOFF_ERROR 	0x8
#define TEST_VALUE  		0x80
#define CHECK_LEVEL_STEP	0x04

// For Carrier-Wagstaff method of 'a' generation.
#define MACHINE_BOUND    	25

//#define UNSUITABLE_FOR_SIQS     1
//#define CW_EVEN_INDEX_START	26
//#define CW_EVEN_INDEX_SIZE	100
//#define CW_EVEN_INDEX_AVERAGE   2400
//#define CW_ODD_INDEX_START      27
//#define CW_ODD_INDEX_SIZE       60
//#define TRIPLE_SIZE      	4
//#define TRIPLES_LOG_FACTOR 	131072

// For sieving with prime powers (since 1.15)
#define DEFAULT_PRIME_POWERS_ALLOCATED 1000
#define DEFAULT_LIMIT_MULTIPLIER 10

// For Pollard Rho factoring method
#define DEFAULT_POLLARD_BLOCK_SIZE 50

// For Double Large Prime Variation
// 2^23 entries
//#define DEFAULT_CYCLE_TABLE_SIZE 8388608
//#define DEFAULT_CYCLE_TABLE_SIZE 131072
//#define DEFAULT_CYCLE_TABLE_SIZE 32768
				  

// Masks for row_element.inited
#define ARG_INITED 	0x1
#define FAC_INITED 	0x2
#define REM_INITED 	0x4
#define LSV_INITED 	0x8
#define RSV_INITED 	0x10

// Other constants for row_element
#define NO_POLY_USED    -1

// Masks for abstractsieve and quadsieve.inited
#define N_INITED  	0x1000
#define kN_INITED 	0x2000
#define M_INITED  	0x4000
#define MAX_INT_PF_INITED 0x8000
#define MAX_ALG_PF_INITED 0x10000
#define F1_INITED       0x80000
#define F2_INITED       0x100000
#define STIME_MAIN_INITED 0x400000
#define STIME_L1_INITED   0x800000
#define STIME_L2_INITED   0x1000000
#define IDEAL_A_VALUE_INITED 0x2000000
#define MIN_C_VALUE_INITED   0x4000000
#define MAX_C_VALUE_INITED   0x8000000

// Masks for quadsieve.open
#define PARTIALS_OPEN   	0x1
#define SMOOTHS_OPEN    	0x2
#define ALL_PARTIALS_OPEN 	0x4
#define USABLE_PARTIALS_OPEN    0x8
#define TEMPORARY_OPEN 		0x10
#define JOB_FILE_OPEN		0x20
#define LAST_STATE_FILE_OPEN    0x40
#define CURRENT_STATE_FILE_OPEN 0x80
#define INTERESTING_VALUES_OPEN 0x100
#define POLYNOMIAL_FILE_OPEN    0x200
#define DEPENDENCIES_FILE_OPEN  0x400
#define LEGENDRE_SMOOTHS_FILE_OPEN 0x800

// Masks for quad_polynomial.inited
#define A_INITED        0x1
#define B_INITED        0x2
#define C_INITED        0x4
#define D_INITED        0x8
#define D2_INITED       0x10

// Masks for variation.inited
#define VAR_INITED      0x1

// Masks for auxiliary inited
#define AUX_1_INITED    0x1
#define AUX_2_INITED    0x2
#define AUX_3_INITED    0x4
#define AUX_4_INITED    0x8
#define AUX_5_INITED    0x10
#define AUX_A_INITED    0x20
#define AUX_B_INITED    0x40
#define AUX_C_INITED    0x80
#define AUX_D_INITED    0x100
#define	AUX_P_INITED    0x200
#define AUX_P_EXP_INITED 0x400
#define AUX_A_INV_INITED 0x800
#define AUX_K_INITED     0x1000
#define AUX_L_INITED     0x2000

// Masks for auxiliary - Shanks-Tonelli inited
#define ST_AUX_A_INITED           0x1
#define ST_AUX_A_INVERSE_INITED   0x2
#define ST_AUX_B_INITED           0x4
#define ST_AUX_R_INITED           0x8
#define ST_AUX_ROOT_INITED        0x10
#define ST_AUX_N_INITED           0x20
#define ST_AUX_P_MINUS_1_INITED   0x40
#define ST_AUX_P_INITED           0x80
#define ST_AUX_1_INITED           0x100
#define ST_AUX_B_EXP_2I_INITED    0x200
#define ST_AUX_NON_INITED         0x400

// Masks for WIB auxiliary variables
#define WIB_DIFF_INITED       0x1
#define WIB_RATIO_INITED      0x2
#define WIB_MUL_P_INITED      0x4
#define WIB_ARG_INITED        0x8
#define WIB_ARG_2_INITED      0x10
#define WIB_ARG_PLUS_INITED   0x20

// Primes smaller than this constant will be generated if primefile not found. NormalMessage("   Reading primes from file ...",NO_NUMBER);
#define DEFAULT_PRIMEFILE_BOUND 10000000

// Error return codes - init environment etc.
#define CALLED_TOO_EARLY     -101
#define NOT_SUPPORTED        -102

// Quadratic sieve defaults
// přesunuto do libs/definitions.h
//#define DEFAULT_EXTENT          1
//#define DEFAULT_MARGIN          40
//#define DEFAULT_VALUE_M         10000
//#define DEFAULT_NR_POLYNOMIALS  1
//#define DEFAULT_DIV_ALLOCATE    16
//#define INIT_VALUE_FACTOR       0
//#define INIT_VALUE_N		-1
//#define INIT_VALUE_k		0
//
#define DEFAULT_SPEC_PRIMES_REALLOC 20

//// Return values of RunMPQS()
// přesunuto do libs/definitions.h
//#define FACTORIZED              1
//#define PRIME_NUMBER            2
//#define DIVISIBLE_BY_SMALL_PRIME 3
//#define NOT_FACTORIZED          -20000
//#define NOT_ENOUGH_SMOOTHS      -3
//#define ERROR_INIT_ENVIRONMENT  -4
//#define ERROR_SIEVING           -5
//#define ERROR_LINALG            -6
//#define TOO_SMALL_FACTORBASE    -7
//#define NO_DEPENDENCIES         -8
//#define ERROR_PROCESSING_RELATIONS -9
//#define EXCESSIVE_FACTOR_BASE   -10
//
// Return values of RunTTR()
#define TTR_OK                  0
#define TTR_NOT_OK              -1

// Checks for primality - number of rounds in Miller-Rabin test
#define MILLER_RABIN_ROUNDS_NORMAL 	15
#define MILLER_RABIN_ROUNDS_STRICT 	40

// Default file names
#define DEFAULT_PARTIAL_FILE_NAME "partials"
#define DEFAULT_SMOOTH_FILE_NAME  "smooths"
#define DEFAULT_PRIMEFILE_NAME    "primefil"

// Since 2.00
#define DEFAULT_TEMPORARY_FILE_NAME "temp"
#define DEFAULT_ALL_PARTIALS_FILE_NAME "all_partials"
#define DEFAULT_USABLE_PARTIALS_FILE_NAME "usable_partials"
// Since 2.02
#define DEFAULT_JOB_FILE_NAME "job"
#define DEFAULT_LAST_STATE_FILE_NAME "last_state"
#define DEFAULT_CURRENT_STATE_FILE_NAME "current_state"
#define DEFAULT_INTERESTING_VALUES_FILE_NAME "interesting"
#define DEFAULT_DEPENDENCIES_FILE_NAME "dependencies"
#define DEFAULT_LEGENDRE_FILE_NAME "legendre_smooths"

//#define DEFAULT_CYCLE_ARRAY_ENTRIES 15
#define DEFAULT_ROOT_LIST           20

// Default reallocation steps
#define SMOOTH_REALLOCATION_STEP  	100
#define USELESS_PRIME_REALLOCATION_STEP 200
#define USELESS_ROW_REALLOCATION_STEP   200

// SIQS defaults (added in 1.09)
#define DEFAULT_S		8
#define DEFAULT_Q_SHIFT         2
#define DEFAULT_LEAST_SIQS_DIVISOR	1024

// Variation-related constants
#define VAR_INITED                0x1
#define NO_INDEX                  -1
#define VARIATION_FACTOR          128

// For allocating polynomials
#define DEFAULT_POLYNOMIALS_ALLOCATED   128
#define DEFAULT_TIME_MESSAGE_INTERVAL   100

// Rough mode settings - from this prime rough mode runs
#define ROUGH_MODE_LOWER_BOUND    100
#define DEFAULT_ROUGH_INDEX	  15

// Sieving matrix - how much divisors are allocated at the beginning
#define DIV_ALLOCATE    	  8


// Quadratic equation pathologies
#define TRIVIAL_EQUATION          1
#define NOT_SUITABLE_MODULUS      -1
#define INSOLUBLE_EQUATION        -2

// Solutions backup
#define THERE                      1
#define BACK                       2

// Levels of MPQS execution
#define MAIN_LEVEL                 0
#define LEVEL_1                    1
#define LEVEL_2                    2

// Constants connected with variations
#define NONE                       0
#define LEFT                       1
#define RIGHT                      2
#define VAR_INITED                 0x1

// Constant connected with hashes
#define HASH_INITED                0x1000

// Masks for integer solutions, added in 1.04
#define ALLOCATED_1_SOLS           0x1
#define ALLOCATED_2_SOLS           0x2
#define ALLOCATED_USABLE_SOLS      0x4

#define VALID_1                    0x1
#define VALID_2                    0x2

// Bit Matrix - some return values
#define EXCESSIVE_ROW_INDEX        -10
#define EXCESSIVE_COLUMN_INDEX     -20
#define NEGATIVE_INDEX             -25
#define NULL_POINTER_SUPPLIED      -30
#define SIZE_MISMATCH              -40

// Sparse Matrix - some macros
#define SPARSE_REALLOCATION_STEP   10
#define ALLOCATED(x) SPARSE_REALLOCATION_STEP*((((x)+2)/SPARSE_REALLOCATION_STEP)+1)

// Large Prime Variation - reallocation in array of line numbers
#define LPV_LINE_REALLOCATION_STEP  4
#define LPV_ALLOCATED(x) LPV_LINE_REALLOCATION_STEP*((((x)+2)/LPV_LINE_REALLOCATION_STEP)+1)

// Large Prime Variation - reallocation in arrays
#define DEFAULT_INTERMEDIATE_ARRAY_ALLOCATE 100
#define DEFAULT_LPS_ARRAY_ALLOCATE_SINGLE   2
#define DEFAULT_LPS_ARRAY_ALLOCATE_DOUBLE   40

//#define FULL_MASK 		0xffffffff
//#define HASH_MASK                  0x7ffff
//#define NONSINGLETON_MASK       0x80000000
//
//// For Large Singleton Removal
//#define SINGLETON               0
//#define NONSINGLETON            1
//#define FRESH_SINGLETON         2
//
//// For finding cycles
//#define ROOT_FLAG               0x7fffffff
//// For combination of relation
//#define FINISHED                0xffffffff
//#define PARITY_MASK		0x80000000

// For factorization of 2-partials
#define FACTOR_TOO_LARGE        -4

// Checkauto - constants
#define CHECKAUTO_MAXINDEX	3
#define PARTIAL_1_WEIGHT	5
#define PARTIAL_2_WEIGHT	1
#define RUNNING                 2
#define DEFAULT_INTERVAL_STEP   3500

// Value distribution
#define DISTRIBUTION_STEP       1000
#define MAX_DISTANCE            100
#define SMOOTH_VALUE		1
#define PARTIAL_1_VALUE		2
#define PARTIAL_2_VALUE		4

// For factorization algorithms generally
#define OPTIMIZATION_LEVEL_0     0
#define OPTIMIZATION_LEVEL_1     1
#define OPTIMIZATION_LEVEL_2     2

#define INTEGRAL_PART_INITED     0x1
#define INTEGRAL_REMAINING_INITED 0x2
#define ALGEBRAIC_NORM_INITED     0x4
#define ALGEBRAIC_NORM_REMAINING_INITED 0x8

//#define SPECIAL_FLAG			0x80000000
//#define DIVIDES_L				0x10000000
//#define DIVIDES_ROOT				0x40000000
//#define INERTIAL_DEGREE_MASK	0xff
//#define NUMBER_OF_COMBINATION_MASK 0xff00
//#define ONE_COMBINATION			0x0100
//
//#define SPECIAL_PRIME_MINI_FLAG		0x8
//#define DIVIDES_ROOT_MINI_FLAG			0x4
//#define DIVIDES_L_MINI_FLAG			0x10
//#define DIVIDES_ROOT_MINI_FLAG		0x2

#define DEFAULT_M_DIVISORS 30
#define DEFAULT_C_P_DIVISORS 30

#define RESULT_OVERFLOW 	0xf0000000
#define DEFAULT_ADD_INTEGERS    20
#define DEFAULT_ADD_PRIME_IDEALS    20

#define FOR_SINGLE_PP		0x1
#define FOR_DOUBLE_PP		0x2
#define FOR_CYCLE_CONSTRUCTION  0x100

// Number of generalized Legendre symbols
#define LEGENDRE_SYMBOLS	64

//// Small singleton detection
//#define NOT_YET			-10
//#define TWICE			0xffffffff

//// Some constants regarding list for deletion
//#define IMPLICIT_LIST_FOR_DELETION_SIZE 1000
//#define IMPLICIT_LD_RESIZE 500


// squareroot algorithm 
#define SQUARE_ROOT_NEWTON	1
#define SQUARE_ROOT_COUVEIGNES	2
#define SQUARE_ROOT_MONTGOMERY  3

#define SIZE_OF_PRIME_IN_PRIMEFIL 4

#define SIZE_OF_AUX_CHAR 300000

#define DEFAULT_MAX_ITERATIONS 1000
#define MAX_INDEPENDENT_CYCLES 1000000000

// sieving region constants
# define SIEVING_REGION_NO_HEIGHT_BOUND 0
# define SIEVING_REGION_DEFAULT_START 1

# define NUMBER_OF_SMOOTHS_NOT_GIVEN 0

//  sieving types. Must have 4 chars
#define LINE_SIEVING 		"Line"
#define LATTICE_SIEVING 	"Latt"
#define OPTIMIZED_LINE_SIEVING  "OpLi"

// precisely one of these modes should be uncommented at a given time.
//#define QS_LINE_SIEVING
#define GNFS_LINE_SIEVING

#define MAX_UPDATES 500
#define MINIMAL_PRIME_TO_SIEVE_WITH 128

#define POSITIVE  1
#define NEGATIVE -1
#define CACHE_UPDATES 64
#define CHECK_MASK 0x80808080
#define UNLIMITED_SIEVING 0

#define NO_UPDATE_INDEX -1
#define DEFAULT_QUADRATIC_CHARACTERS 32

// Text messages and their indices
#define GEN_NEW_POLY   10
#define GEN_NEW_POLY_TEXT "      Overall time spent in  GenerateNewPoly  is"

#define QUAD_MACHINE   20
#define QUAD_MACHINE_TEXT "      Overall time spent in  QuadraticMachine is"

#define PERF_SIEVING   30
#define PERF_SIEVING_TEXT "      Overall time spent in  PerformSievingPo is"

#define SIEVING_LOOP   40
#define SIEVING_LOOP_TEXT "                         Where  SievingLoopLogar"

#define REFILL_MAINM   50
#define REFILL_MAINM_TEXT "                         Where  RefillMainMatrix"

#define SORT_SMOOTHS   60
#define SORT_SMOOTHS_TEXT "                         Where  SortSmoothValues"

#define DIVISORS_OFA   70
#define DIVISORS_OFA_TEXT "                              Where Divisor Of A"

#define SMALL_FACTOR   80
#define SMALL_FACTOR_TEXT "                              Where Small Factor"

#define COUNT_CYCLES   90
#define COUNT_CYCLES_TEXT "                              Where Count Cycles"

#define BLOCK_SIEVES  100
#define BLOCK_SIEVES_TEXT "      Overall time spent in   Block Set Sieve is"

#define FILL_HSTABLE  110
#define FILL_HSTABLE_TEXT "      Overall time spent in   Fill Hashtables is"

#define FILL_HST_NEG  120
#define FILL_HST_NEG_TEXT "                          Where FillHasht-NEG is"

#define ROOTS_UPDATE  130
#define ROOTS_UPDATE_TEXT "      Overall time spent in   Root Update/New is"

#define FIND_FACTORZ  140
#define FIND_FACTORZ_TEXT "                          Where Find&FactorCa is"

#define UNKNOWN_PART_TEXT "      ! Unknown part ! "

#define TERMINATED 100

// Relation processing flags
#define RELATIONS_READY_FLAG    		  0x1
#define LARGE_SINGLETONS_REMOVED_FLAG  		  0x2
#define CYCLES_CONSTRUCTED_FLAG	  		  0x4
#define RELATION_COLLECTION_FINISHED_FLAG 	  0x8
#define SIEVING_RUNNING_FLAG			  0x10


// Distribution flags
#define NO_CENTERS_DEFINED	-500
#define NODE_ALREADY_REGISTERED -501
#define NO_SIEVE_INSTANCE_REGISTERED -502
#define NO_CONNECTORS_REGISTERED -503



// Input parameters parsing
#define INPUT_PARAM_SEPARATOR ":"
#define INPUT_PARAM_SEPARATOR_COUNT 2
#define INPUT_PARAM_SEPARATOR_COUNT_DEFAULT 1

// Sieving Phase Const

#define SIEVING_PHASE_CHECK_MASK 0x80808080
#define SIEVING_PHASE_MASK 0x80
#define SIEVING_PHASE_64BITS_SIZE 8
#define SIEVING_PHASE_SIZES_COUNT 10

// Rel Processing Phase Const

#define REL_PRO_PHASE_AUTO_SERIALIZABLE false
#define REL_PRO_PHASE_RELATION_MATRIX_FILENAME "relation_matrix"
#define REL_PRO_PHASE_QUAD_CHARS_FILENAME FILE_NAME_QUAD_CHARS
#define REL_PRO_PHASE_ROOTS_LIST 20
#define REL_PRO_PHASE_QUAD_CHARS_COUNT 64
#define REL_PRO_PHASE_RELATION_MATRIX_TYPE ESparseBitMatrix

// Linear Phase Const

#define LINEAR_PHASE_AUTO_SERIALIZABLE false
#define LINEAR_PHASE_RANK_CALC_MODE false
#define LINEAR_PHASE_DENSITY_MODE false
#define LINEAR_PHASE_TIME_MESSAGE_MODE false
#define LINEAR_PHASE_CHECK_FINAL_RESULT true
#define LINEAR_PHASE_MATRIX_RESULT_FILENAME "matrix_result"
#define LINEAR_PHASE_RELATION_MATRIX_FILENAME REL_PRO_PHASE_RELATION_MATRIX_FILENAME
#define LINEAR_PHASE_MATRIX_SOLVER_TYPE ELanczos
#define LINEAR_PHASE_RELATION_MATRIX_TYPE REL_PRO_PHASE_RELATION_MATRIX_TYPE
#define LINEAR_PHASE_LANCZOS_AUX_MATRIX_TYPE EBCMatrix

// Square Root Phase Const

#define SQUARE_ROOT_PHASE_AUTO_SERIALIZABLE false
#define SQUARE_ROOT_PHASE_MATRIX_RESULT_FILENAME LINEAR_PHASE_MATRIX_RESULT_FILENAME
#define SQUARE_ROOT_PHASE_ROOT_FINDER ECouveignes
#define SQUARE_ROOT_PHASE_RELATIONS_FILENAME FILE_NAME_SMOOTH_RELATIONS
#define SQUARE_ROOT_PHASE_QUAD_CHARS_FILENAME REL_PRO_PHASE_QUAD_CHARS_FILENAME
#define SQUARE_ROOT_PHASE_MATRIX_RESULT_TYPE EBitMatrix


// default values

#define DEFAULT_ROOTS_COUNT 8
#define DEFAULT_START_PHASE "1"  // EPolySelection ToString
#define DEFAULT_NFS_WORKING_THREADS_COUNTS 1
#define DEFAULT_NFS_MAX_POOL_QUEUE_SIZE 10

#define DEFAULT_PERIOD_SERIAL_COMPRESSION_LEVEL "1"
#define DEFAULT_PERIOD_SERIAL_TEMP_FILE "period_serialization_temp"
#define DEFAULT_PERIOD_SERIAL_FILE "period_serialization"
#define DEFAULT_PERIOD_SERIAL_PERIOD 5

// CNFSRelation

#define NFS_RELATION_PARTS_COUNT 4
#define NFS_RELATION_PRIMES_PART 0
#define NFS_RELATION_LARGE_PRIMES_PART 1
#define NFS_RELATION_PRIMEIDEALS_PART 2
#define NFS_RELATION_LARGE_PRIMEIDEALS_PART 3
#define NFS_RELATION_PRIMES_DISC "Integer part"
#define NFS_RELATION_LARGE_PRIMES_DISC "Large integer part"
#define NFS_RELATION_PRIMEIDEALS_DISC "Algebraic part"
#define NFS_RELATION_LARGE_PRIMEIDEALS_DISC "Large algebraic part"



// FBHelper - constants

#define FB_HELPER_ROOTS_COUNT DEFAULT_ROOTS_COUNT
#define FB_HELPER_DEFAULT_SPECIAL_IDEALS 40
#define FB_HELPER_DEFAULT_CP_ZERO_PRIMES 30

// NFSParameters

#define NFS_PARAMETERS_PHASES_COUNT 6 // include one for 'global' parameters

// Relations

#define RELATION_UNDEFINED_COUNT -1
#define RELATION_DEFAULT_POLY_DEGREE 3
#define RELATION_DEFAULT_MODE FOR_DOUBLE_PP

#define RELATION_PART_DEFAULT_ADD 20
#define RELATION_PART_MAX_EXP 15
#define RELATION_PART_2_MAX_EXP 25

// Relation types
#define RELATION_FULL_INTEGRAL		0x1
#define	RELATION_PARTIAL_INTEGRAL	0x2	
#define RELATION_TOO_LARGE_INTEGRAL	0x80
#define RELATION_FULL_ALGEBRAIC		0x100
#define RELATION_PARTIAL_ALGEBRAIC	0x200
#define RELATION_TOO_LARGE_ALGEBRAIC	0x8000
#define RELATION_FINISHING_CYCLE		0x800000
#define RELATION_FREE					0x8000000

#define RELATION_FULL_ALGEBRAIC1		0x1
#define	RELATION_PARTIAL_ALGEBRAIC1		0x2	
#define RELATION_TOO_LARGE_ALGEBRAIC1	0x80
#define RELATION_FULL_ALGEBRAIC2		0x100
#define RELATION_PARTIAL_ALGEBRAIC2		0x200
#define RELATION_TOO_LARGE_ALGEBRAIC2	0x8000

// Structures

#define STRUCTURE_CACHE_UPDATES 64
#define STRUCTURE_MAX_UPDATES 500

// General definitions

#define MAXIMAL_LONG_VALUE 0x7fffffff // should be better (32 or 64bit) TODO
#define MAXIMAL_INT_VALUE 0x7fffffff // should be better (32 or 64bit) TODO

#endif
