#ifndef __NFS_RELATION_H__
#define __NFS_RELATION_H__

#include "abstract_relation.h"
#include "buffered_file_writer_item.h"
#include "polynom_class.h"
#include "complex_structures.h"

/*!
 * \brief Relation for NFS
 */
class CNFSRelation : public ARelation, public virtual CBufferedFileWriterItem
{
private:
	mpz_t	iFactor1;
	mpz_t	iFactor2;

// CONSTANTS
	static const int DIVISOR_READING_BUFFER_SIZE = 100000;
	///Max number of main divisors for square root mode
	static const int DEFAULT_SR_DIVISORS_ALLOCATE	= 32768;

public:
	algebraic_coeff_type A;
	algebraic_coeff_type B;

// CONSTANTS
	/*!
		Mode of parts initializations - how the number of elements should be determine
		This relation will be used for cycle construction - relation processing phase

		Other way is used only number in interval [0, 3] - it means no large prime, 1 large prime ...
		Or aggregate relation for square root phase.
	*/
	static const unsigned int MODE_CYCLE_CONSTRUCTION	= 0x100;
	///Mode of parts initializations - for square root phase - aggregate relation
	static const unsigned int MODE_SQUARE_ROOT			= 0x200;
	///We have usually only two parts - integral and algebraic - or two algebraic
	static const int DEFAULT_NFS_PARTS_COUNT = 2;

protected:
	///Dispose used mpz members
	void DisposeMpz();

public:

	CNFSRelation(int aMode, int aPartCount = DEFAULT_NFS_PARTS_COUNT);
	CNFSRelation(const CNFSRelation& aOperand);
	~CNFSRelation();

	int Setup(algebraic_coeff_type aA, algebraic_coeff_type aB, int aNFInfoCount, CNumberFieldInfo * aNFInfos);
	int DetermineType(int aInfoCount, CVariationsInfo * aInfos);
	int Reset();
	int Copy2(CNFSRelation &aRelation);
	bool TestCoprimality();

	bool Equals(const CNFSRelation &aOperand) const;

	int ReadFromFile(FILE* aFr);
	// implements pure virtual method of CBufferedFileWriterItem class
	int PrintToFile(FILE* aFw);
	void PrintToScreen();

	// implements pure virtual method of CBufferedFileWriterItem class
	CNFSRelation *CopyItem();
	// implements pure virtual method of CBufferedFileWriterItem class
	void FreeItem();
	
	// De/Serialization methods
	int Serialize(const CBaseParameters& aParam) const;
	int Serialize(const CBaseParameters& aParam, xmlTextWriterPtr& aWriter) const;

	int Deserialize(const CBaseParameters& aParam);
	int Deserialize(const CBaseParameters& aParam, xmlTextReaderPtr& aReader);

};

#endif // __NFS_RELATION_H__
