#include "phase_creator.h"

#include "poly_selection_phase.h"
#include "montgomery_poly_sel_phase.h"
#include "nfs_classical_sieving_phase.h"
#include "lattice_sieving_phase.h"
#include "rel_processing_phase.h"
#include "filtering_phase.h"
#include "linear_phase.h"
#include "square_root_phase.h"
#include "log.h"
#include "kleinjung_poly_sel_phase.h"
#include "kleinjung2_poly_sel_phase.h"
#include "murphy_poly_sel_phase.h"

int CPhaseCreator::CreatePhase(phase_types aPhaseType, APhase*& aPhase)
{
	if (aPhase != NULL)
	{
		CLog::LogError(WHEREARG,"CPhaseCreator","CreatePhase",0,0,"Phase is not NULL.");
		return ConstRC::GeneralError;
	}

	switch (aPhaseType)
	{
		case ENFSClassicPolySel:
			aPhase = new CPolySelectionPhase();
			return ConstRC::Ok;
		case ENFSMontgomeryPolySel:
			aPhase = new CMontgomeryPolySelPhase();
			return ConstRC::Ok;
		case ENFSClassicalSieving:
			aPhase = new CNFSClassicalSievingPhase(); 
			return ConstRC::Ok;
		case ENFSLatticeSieving:
			aPhase = new CLatticeSievingPhase(); 
			return ConstRC::Ok;
		case ENFSRelProcessing:
			aPhase = new CRelProcessingPhase();
			return ConstRC::Ok;
		case ENFSFiltering:
			aPhase = new CFilteringPhase();
			return ConstRC::Ok;
		case ENFSGeneralLinear:
			aPhase = new CLinearPhase();
			return ConstRC::Ok;
		case ENFSSqrRoot:
			aPhase = new CSquareRootPhase();
			return ConstRC::Ok;
		case ENFSKleinjungPolySel:					//Kleinjung's polynomial selection phase 
			aPhase = new CKleinjungPolySelPhase();
			return ConstRC::Ok;
		case ENFSKleinjung2PolySel:					//Kleinjung's Second polynomial selection phase
			aPhase = new CKleinjung2PolySelPhase();
			return ConstRC::Ok;
		case ENFSMurphyPolySel:						//Murphy's polynomial selection phase
			aPhase = new CMurphyPolySelPhase();
			return ConstRC::Ok;	
		default:
			return ConstRC::NotFound;
	}
}
