#include <cstring>
#include "poly_selection_phase.h"
#include "integer_class.h"
#include <math.h>
#include "const_parameter_names.h"
#include "const_filenames.h"
#include "test_suite.h"
#include "log.h"


const int CPolySelectionPhase::iCoefficientPrimes[ConstPSPhase::SMALL_PRIME_COUNT] = 
{ 2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 
59, 61, 67, 71, 73, 79, 83, 89, 97, 101, 103, 107, 109, 113, 127, 131,
137,139,149,151,157,163,167,173,179,181,191,193,197,199,
211,223,227,229,233,239,241,251,257,263,269,271,277,281,283,
293,307,311,313,317,331,337,347,349,353,359,367,373,379,383,389,
397,401,409,419,421,431,433,439,443,449,457,461,463,467,479,487,491,499};

//int CPolySelectionPhase::iCPrimes[ConstPSPhase::C_PRIME_COUNT] = 
//{5, 7, 11, 13, 4, 9, 17, 19, 23, 29, 2, 3};

int CPolySelectionPhase::iCPrimes[ConstPSPhase::C_PRIME_COUNT] = 
{17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71,
 73, 79, 83, 89, 97,101,103,107,109,113,127,131,137,139,143 /* 11 * 13 */,
149,151,157,163,167,173,179,181,191,193,197,199,211,
223,227,229,233,239,241,251,257,263,269,271,277,281,283};

using namespace std;

CPolySelectionPhase::CPolySelectionPhase() : MSerializable(ConstFN::POLY_PHASE)
{
	this->iFoundCandidatePolyCount = 0;
	this->iRatedCandidatePolyCount = 0;
	this->iRSerialFoundCandPolyCount = 0;
	this->iRSerialRatedCandPolyCount = 0;
	this->iRSerialRunCChangeCount = 0;
	this->iRSerialRunMChangeCount = 0;
	this->iRSerialRunPolyCount = 0;

	this->iFBBound = 0;

	this->iResultSkewness = 1.0;

	this->iCandidatePolynomials = NULL;
	this->iResultPoly = NULL;
	this->iRSerialCPrimes = NULL;
	this->iWorkingCPrimes = NULL;

	mpz_init(this->iWorkingMonicM);
	mpz_init(this->iRSerialMonicM);
	mpz_init(this->iC);
	mpz_init(this->iMainAd);

	Set_AutoSerializable(ConstPSPhase::AUTO_SERIALIZABLE);
	Set_InnerState((int)EPSPhaseNotStarted);

	this->iStats = new CStatContainer(ConstPSPhase::INFO_STAT_COUNT);
	this->iTimes = new CTimeContainer(ConstPSPhase::INFO_TIME_COUNT);
}

CPolySelectionPhase::~CPolySelectionPhase()
{
	Reset();

	if (this->iStats != NULL)
	{
		delete this->iStats;
		this->iStats = NULL;
	}
	if (this->iTimes != NULL)
	{
		delete this->iTimes;
		this->iTimes = NULL;
	}

	DisposeFunctorField();
	DisposeGMP();
	DisposeMutexes();
	DisposeAcceptedReturnCodes();
}
int CPolySelectionPhase::Reset()
{
	this->iFoundCandidatePolyCount = 0;
	this->iRatedCandidatePolyCount = 0;
	this->iRSerialFoundCandPolyCount = 0;
	this->iRSerialRatedCandPolyCount = 0;

	this->iFBBound = 0;

	if (this->iCandidatePolynomials != NULL)
	{
		delete this->iCandidatePolynomials;
		this->iCandidatePolynomials = NULL;
	}
	if (this->iResultPoly != NULL)
	{
		delete this->iResultPoly;
		this->iResultPoly = NULL;
		this->iResultSkewness = 1.0;
	}

	this->iRSerialCPrimesCount = 0;
	if (this->iRSerialCPrimes != NULL)
	{
		free(this->iRSerialCPrimes);
		this->iRSerialCPrimes = NULL;
	}
	if (this->iWorkingCPrimes != NULL)
	{
		free(this->iWorkingCPrimes);
		this->iWorkingCPrimes = NULL;
	}

	if (this->iStats != NULL)
	{
		this->iStats->ResetAll();
	}
	if (this->iTimes != NULL)
	{
		this->iTimes->ResetAll();
	}

	APolyPhase::Reset();

	Set_InnerState((int)EPSPhaseNotStarted);
	return ConstRC::Ok;
}
int CPolySelectionPhase::DisposeGMP()
{
	mpz_clear(this->iWorkingMonicM);
	mpz_clear(this->iRSerialMonicM);
	mpz_clear(this->iC);
	mpz_clear(this->iMainAd);

	return APolyPhase::DisposeGMP();
}
int CPolySelectionPhase::DisposeMutexes()
{
	return APolyPhase::DisposeMutexes();
}
int CPolySelectionPhase::FillFunctorField()
{
	if (Get_FunctorsInit() == false)
	{
		Set_FunctorsInit(true);
		this->iPhaseFunctors.reserve(ConstPSPhase::INNER_STATE_COUNT);
		this->iPhaseFunctors.clear();
		this->iPhaseFunctors.push_back(NULL); // EPSPhaseNotStarted
		this->iPhaseFunctors.push_back(NULL); // EPSPhaseInitialization
		this->iPhaseFunctors.push_back(new CPhaseFunctor<CPolySelectionPhase>(this, &CPolySelectionPhase::GenerateCandidates)); // EPSPhaseSearchingForCandidates
		this->iPhaseFunctors.push_back(new CPhaseFunctor<CPolySelectionPhase>(this, &CPolySelectionPhase::ChooseBest)); // EPSPhaseChoosingFromCandidates
		this->iPhaseFunctors.push_back(new CPhaseFunctor<CPolySelectionPhase>(this, &CPolySelectionPhase::SaveResult)); // EPSPhaseSavingResult
		this->iPhaseFunctors.push_back(NULL); // EPSPhaseFinished
	}

	return ConstRC::Ok;
}
int CPolySelectionPhase::CleanUp(AFactorAlgParameters* aParameters, int aPhaseNBR, cleaning_types aType, bool aError)
{
	if (aParameters == NULL)
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","CleanUp",0,0,"Parameters are NULL.");
		return ConstRC::BadArgument;
	}

	string identifier;
	string directory = "";
	string file;

	int phaseNBR;
	if (aParameters->GetIntegerPhaseParameter(ConstPN::POLY_PHASE_NUMBER, phaseNBR, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","CleanUp",0,0,"Didn't find phase number.");
		return ConstRC::GeneralError;
	}

	if (aParameters->GetStringPhaseParameter(ConstPN::SERIALIZATION_IDENTIFIER, identifier, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","CleanUp",0,0,"Didn't find serial identifier.");
		return ConstRC::GeneralError;
	}
	aParameters->GetStringPhaseParameter(ConstPN::SERIALIZATION_DIRECTORY, directory, (int)EMain);

	switch (aType)
	{
		case ECleanAllIfNotError:
			if (aError)
			{
				break;
			}
		case ECleanAll:
		case ECleanAllExcCF:
		case ECleanAllExcCFRF:
			// Running Serialization full filename - default is same as iSerialPhaseFullFileName
			if (aParameters->ContainsPhaseParameter(ConstPN::RSERIAL_POLY_SEL_FULL_FILENAME, phaseNBR))
			{
				aParameters->GetStringPhaseParameter(ConstPN::RSERIAL_POLY_SEL_FULL_FILENAME, file, phaseNBR);
				MSerializable::AdaptPathSeparator(file);
			}
			else
			{
				aParameters->CreateFullFileName(file, ConstPN::RSERIAL_POLY_SEL_FILENAME, 
					ConstPN::RSERIAL_POLY_SEL_DIRECTORY, ConstPN::RSERIAL_POLY_SEL_COMPRESSION, directory,
					ConstFN::POLY_PHASE, identifier, phaseNBR);
			}
			remove(file.c_str());

			// Candidate polynomials full filename
			if (aParameters->ContainsPhaseParameter(ConstPN::CAND_POLY_FULL_FILENAME, phaseNBR))
			{
				aParameters->GetStringPhaseParameter(ConstPN::CAND_POLY_FULL_FILENAME, file, phaseNBR);
				MSerializable::AdaptPathSeparator(file);
			}
			else
			{
				aParameters->CreateFullFileName(file, ConstPN::CAND_POLY_FILENAME, 
					ConstPN::CAND_POLY_DIRECTORY, ConstPN::CAND_POLY_COMPRESSION, directory,
					ConstFN::CANDIDATE_POLYS, identifier, phaseNBR);
			}
			remove(file.c_str());
			break;
		default:
			break;
	}

	return APolyPhase::CleanUp(aParameters, phaseNBR, aType, aError);
}
int CPolySelectionPhase::InitParameters()
{
	Set_InnerState((int)ESPhaseInitialization);

	AFactorAlgParameters * param = Get_Parameters();

	if (param == NULL)
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","InitParameters",0,0,"Parameters aren't set.");
		return ConstRC::GeneralError;
	}

	// generally we don't know order of the poly selection phase ;)
	int phaseNBR;
	if (param->GetIntegerPhaseParameter(ConstPN::POLY_PHASE_NUMBER, phaseNBR, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","InitParameters",0,0,"Didn't find phase number.");
		return ConstRC::GeneralError;
	}
	Set_PhaseNbr(phaseNBR);

	// now we initialize members from parents's classes
	int rc = APolyPhase::InitParameters();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	param->GetIntegerPhaseParameter(ConstPN::MIN_AD_PER_C, ConstPSPhase::MIN_AD_PER_C, this->iMinAdPerC, phaseNBR);
	param->GetIntegerPhaseParameter(ConstPN::CHI_MIN_LOG, ConstPSPhase::CHI_MIN_LOG, this->iChiMinLog, phaseNBR);
	param->GetIntegerPhaseParameter(ConstPN::CHI_MAX_LOG, ConstPSPhase::CHI_MAX_LOG, this->iChiMaxLog, phaseNBR);
	param->GetIntegerPhaseParameter(ConstPN::CHI_INVERSE, ConstPSPhase::CHI_INVERSE, this->iChiInverse, phaseNBR);
	param->GetLongPhaseParameter(ConstPN::RSERIALIZATION_C_CHANGE_COUNT, ConstPSPhase::C_CHANGE_COUNT, this->iRSerialRunCChangeCount, phaseNBR);
	param->GetLongPhaseParameter(ConstPN::RSERIALIZATION_M_CHANGE_COUNT, ConstPSPhase::M_CHANGE_COUNT, this->iRSerialRunMChangeCount, phaseNBR);
	param->GetLongPhaseParameter(ConstPN::RSERIALIZATION_POLY_COUNT, this->iCandidatesCount, this->iRSerialRunPolyCount, phaseNBR);
	param->GetLongPhaseParameter(ConstPN::MAX_CANDIDATES_PER_LC, 
		ConstPSPhase::MAX_CANDIDATES_PER_LC, this->iMaxPerCoefficient, phaseNBR);

	this->iSteepestDescentLevel = ConstPSPhase::STEEPEST_DESCENT_LEVEL;
	this->iThirdCoefficientBound = ConstPSPhase::THIRD_COEFFICIENT_RELATIVE_LENGTH;
	this->iIFSBound = ConstPSPhase::IFS_BOUND;
	this->iIFSImprovement = ConstPSPhase::IFS_IMPROVEMENT;
	this->iPrimesDividingLeading = ConstPSPhase::PRIME_NUMBERS_DIVIDING_LEADING;

	int auxInt;
	param->GetIntegerPhaseParameter(ConstPN::POLY_GEN_TYPE, ConstPSPhase::POLY_GEN_TYPE, auxInt, phaseNBR);
	this->iPolyGenType = (poly_gen)auxInt;

	// we try to find value for all 
	param->GetLongPhaseParameter(ConstPN::FB_UPPER_BOUND, this->iFBBound, (int)EMain);
	// We know that second factor base is algebraic
	rc = param->GetLongPhaseParameter(ConstPN::FB_UPPER_BOUND2, this->iFBBound, this->iFBBound, (int)EMain);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","InitParameters",0,0,"Didn't find the algebraic base upper bound.");
		return rc;
	}

	// Running Serialization full filename - default is same as iSerialPhaseFullFileName
	if (param->ContainsPhaseParameter(ConstPN::RSERIAL_POLY_SEL_FULL_FILENAME, phaseNBR))
	{
		param->GetStringPhaseParameter(ConstPN::RSERIAL_POLY_SEL_FULL_FILENAME, this->iRunningSerialFullFileName, phaseNBR);
		MSerializable::AdaptPathSeparator(this->iRunningSerialFullFileName);
	}
	else
	{
		param->CreateFullFileName(this->iRunningSerialFullFileName, ConstPN::RSERIAL_POLY_SEL_FILENAME, 
			ConstPN::RSERIAL_POLY_SEL_DIRECTORY, ConstPN::RSERIAL_POLY_SEL_COMPRESSION, Get_SerialMainDirectory(),
			ConstFN::POLY_PHASE, Get_SerialIdentifier(), phaseNBR);
	}

	// Candidate polynomials full filename
	if (param->ContainsPhaseParameter(ConstPN::CAND_POLY_FULL_FILENAME, phaseNBR))
	{
		param->GetStringPhaseParameter(ConstPN::CAND_POLY_FULL_FILENAME, this->iCandidatePolyFullFileName, phaseNBR);
		MSerializable::AdaptPathSeparator(this->iCandidatePolyFullFileName);
	}
	else
	{
		param->CreateFullFileName(this->iCandidatePolyFullFileName, ConstPN::CAND_POLY_FILENAME, 
			ConstPN::CAND_POLY_DIRECTORY, ConstPN::CAND_POLY_COMPRESSION, Get_SerialMainDirectory(),
			ConstFN::CANDIDATE_POLYS, Get_SerialIdentifier(), phaseNBR);
	}

	// should we deserialize or serialize?
	bool deserialize = false;
	param->GetBoolPhaseParameter(ConstPN::DESERIALIZE, false, deserialize, Get_PhaseNbr());
	CBaseParameters bparam;
	bparam.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, Get_SerialPhaseFullFileName());

	if (Get_InfoMode())
	{
		Utils::NormalMessage("Classic Polynomial Selection Phase is running ...");
	}

	if (deserialize)
	{
		rc = Deserialize(bparam);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}
	else
	{
		rc = Serialize(bparam);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	return ConstRC::Ok;
}
int CPolySelectionPhase::GenerateCandidates()
{
	Set_InnerState(EPSPhaseSearchingForCandidates);

	if (Get_InfoMode())
	{
		Utils::NormalMessage("\tGenerating polynomial ...");
	}

	this->iTimes->Start(EPSTGeneratePoly);

	CBaseParameters param;
	param.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iRunningSerialFullFileName);
	int rc = StartRunningSerialization(param);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	switch (this->iPolyGenType)
	{
		case EGenerateMonicPolynomial: 
			rc = GenerateMonic();
			break;
		case EGenerateNonmonicPolynomial: 
			rc = GenerateNonmonic();
			break;
		case EGenerateSkewedPolynomial: 
		default:
			rc = GenerateSkewed();
			break;
	}

	if (rc != ConstRC::Ok)
	{
		EndRunningSerialization();
		return rc;
	}

	rc = EndRunningSerialization();	

	this->iTimes->End(EPSTGeneratePoly);

	return rc;
}
int CPolySelectionPhase::ChooseBest()
{
	Set_InnerState(EPSPhaseChoosingFromCandidates);

	if (Get_InfoMode())
	{
		Utils::NormalMessage("\tChoosing the best polynomial ...\n");
	}

	if (this->iFoundCandidatePolyCount <= 0)
	{
		return ConstRC::NoCandidates;
	}

	this->iTimes->Start(EPSTChooseBest);

	CBaseParameters param;
	param.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iRunningSerialFullFileName);
	int rc = StartRunningSerialization(param);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	mpf_t aux;

	mpf_init_set_si(aux, this->iFBBound);
	mpf_set_d(aux, Utils::Ln(aux));

	long index;
	rc = ChoosePolynomial(index, aux);
	if (rc != ConstRC::Ok)
	{
		EndRunningSerialization();
		return rc;
	}

	this->iResultPoly = new Polynomial();
	this->iCandidatePolynomials->Poly[index]->Copy(this->iResultPoly);
	mpz_set(this->iRoot, this->iCandidatePolynomials->Root[index]);
	this->iResultSkewness = mpf_get_d(this->iCandidatePolynomials->Skewness[index]);

	mpf_clear(aux);

	rc = EndRunningSerialization();

	this->iTimes->End(EPSTChooseBest);

	return rc;
}
int CPolySelectionPhase::SaveResult()
{
	if (this->iResultPoly == NULL)
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","SaveResult",0,0,"Result polynomial is not determined.");
		return ConstRC::GeneralError;
	}

	Set_InnerState(EPSPhaseSavingResult);

	AFactorAlgParameters * param = Get_Parameters();
	string auxStr = Utils::ConvertToString(this->iRoot);
	param->SetPhaseParameter(ConstPN::ROOT_M, auxStr, (int)EMain);

	// now we create linear polynomial : f(x) = x - m
	Polynomial * poly = new Polynomial();
	poly->Set(1, 1); // monic linear poly
	mpz_t r;
	mpz_init(r);
	mpz_neg(r, this->iRoot); // -m
	poly->Set(0, r);
	mpz_clear(r);

	param->SetPhaseParameter(ConstPN::SIEVING_POLY1, poly->ToString(ELongFormat), (int)EMain);
	param->SetPhaseParameter(ConstPN::SIEVING_POLY2, this->iResultPoly->ToString(ELongFormat), (int)EMain);
	auxStr = Utils::ConvertToString(this->iResultSkewness);
	param->SetPhaseParameter(ConstPN::POLY_SKEWNESS, auxStr, (int)EMain);

	delete poly;
	poly = NULL;

	Set_InnerState(EPSPhaseFinished);

	CBaseParameters bparam;
	bparam.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, Get_SerialPhaseFullFileName());

	int rc = Serialize(bparam);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	return ConstRC::Ok;
}
/*!
	Returns the skewness of the polynomial aPolynomial. The iterating process starts at aStartingSkewness.
*/
int CPolySelectionPhase::Skewness(mpf_t aResult, Polynomial *aPolynomial, mpf_t aStartingSkewness)
{
	int d = aPolynomial->GetDegree();
	if (d < 0)
	{
		return ConstRC::GeneralError;
	}

	mpf_t* integral = NULL;
	integral = (mpf_t*) Utils::AllocationMachine((void**)&integral, (d + 1) * sizeof(mpf_t));
	mpf_t deriv, aux, val, pow, sq;
	int i;
	bool found = false, jumped = false;
	for (i = 0; i <= d; i++)
	{
		mpf_init(integral[i]);
	}

	DoubleIntegralAcrossSkewedRegion(integral, aPolynomial);

	for (i = 0; 2 * i <= d; i++)
	{
		mpf_mul_ui(integral[i], integral[i], d - 2 * i);
		mpf_neg(integral[i], integral[i]);
	}
	for (; i <= d; i++)
	{
		mpf_mul_ui(integral[i], integral[i], 2 * i - d);
	}

	mpf_init(deriv);
	mpf_init(aux);
	mpf_init(val);
	mpf_init(pow);
	mpf_init(sq);
	mpf_set(aResult, aStartingSkewness);

	while (!found) 
	{
		mpf_pow_ui(sq, aResult, 2);
		mpf_pow_ui(pow, aResult, d - 1);
		mpf_set_ui(val, 0);
		for (i = d; i >= 0; i--)
		{
			mpf_mul(aux, pow, integral[i]);
			mpf_div(pow, pow, sq);
			mpf_add(val, val, aux);
		}

		mpf_abs(aux, val);
		if (mpf_cmp_d(aux, 0.00001) < 0)
		{
			found = true;
		}
		else 
		{
			if (d > 2)
			{
				mpf_pow_ui(pow, aResult, d - 2);
			}
			else
			{
				mpf_ui_div(pow, 1, aResult);
			}
			mpf_set_ui(deriv, 0);
			for (i = d; i >= 0; i--)
			{
				mpf_mul(aux, pow, integral[i]);
				if (2 * i > d)
				{
					mpf_mul_ui(aux, aux, 2 * i - d - 1);
				}
				else 
				{
					mpf_mul_ui(aux, aux, d - 2 * i + 1);
					mpf_neg(aux, aux);
				}
				mpf_div(pow, pow, sq);
				mpf_add(deriv, deriv, aux);
			}

			mpf_neg(aux, val);
			mpf_div(sq, aux, deriv);
			mpf_add(aux, sq, aResult);
			mpf_sub(sq, aux, aResult);
			mpf_abs(pow, sq);

			if (mpf_cmp_d(pow, 0.00001) < 0)
			{
				found = true;
			}

			if (mpf_cmp_ui(aux, 0) <= 0) 
			{
				if (!jumped)
				{ 
					jumped = true;
					mpf_set_d(aResult, 0.1);
				}
				else
				{
					found = true;
				}
			} 
			else 
			{
				mpf_set(aResult, aux);
			}
		}
	}

	mpf_clear(deriv);
	mpf_clear(aux);
	mpf_clear(pow);
	mpf_clear(val);
	mpf_clear(sq);

	for (i = 0; i <= d; i++)
	{
		mpf_clear(integral[i]);
	}
	free(integral);
	integral = NULL;
	
	return ConstRC::Ok;
}

/*!
	Skews the polynomial using the steepest descent method. Input is a polynomial aPolynomial with a root aOldRoot
	and output is a polynomial aResult with a root aNewRoot, skewness aSkewness and its I(F,S) aIntegral.
*/
int CPolySelectionPhase::SkewPolynomial(Polynomial *aResult, mpf_t aSkewness, mpz_t aNewRoot, double *aIntegral, Polynomial *aPolynomial, mpz_t aOldRoot)
{
	unsigned i,k,d;
	int j;
	d = aPolynomial->GetDegree();
	MultVarPolynomial *Tr,*Rot,*f,*auxf;
	Tr = new MultVarPolynomial(2, 1);
	Rot = new MultVarPolynomial(4, 2);
	f = new MultVarPolynomial(d + 7, d);
	auxf = new MultVarPolynomial(5 * d + 25, 2 * d);
	
	for (i = 0; i < MULT_VAR_VARIABLES; i++)
	{
		Tr->Term[0].Variables[i]=0;
		Tr->Term[1].Variables[i]=0;
		f->Term[0].Variables[i]=0;
		Rot->Term[0].Variables[i]=0;
		Rot->Term[1].Variables[i]=0;
		Rot->Term[2].Variables[i]=0;
	}
	Tr->Term[0].XPolynomial->Set(1, 1);
	Tr->Term[1].XPolynomial->Set(0, -1);
	Tr->Term[1].Variables[T_LOC] = 1;
	Tr->length = 2;
	f->length = 1;
	f->Term[0].XPolynomial->Set(0, *aPolynomial->Get2(d));
	for (i = 0; i < d; i++)
	{
		f->Multiply(auxf, Tr);
		auxf->Copy(f);
		f->Term[0].XPolynomial->Inc(0, *aPolynomial->Get2(d - i - 1));
	}

	Rot->Term[0].XPolynomial->Set(0, 1);
	Rot->Term[0].Variables[C0_LOC] = 1;
	if (d > 4) 
	{
		Rot->Term[1].XPolynomial->Set(1, 1);
		Rot->Term[1].Variables[C1_LOC] = 1;
		if (d > 5) 
		{
			Rot->Term[2].XPolynomial->Set(2, 1);
			Rot->Term[2].Variables[C2_LOC] = 1;
			Rot->length = 3;
		} 
		else
		{
			Rot->length=2;
		}
	} 
	else
	{
		Rot->length=1;
	}

	Tr->Term[0].XPolynomial->Dec(0, aOldRoot);
	Tr->Multiply(auxf, Rot);
	f->Add(f, auxf);

	
	//f->Term[0].Variables[0]=1;
	//f->Term[0].Variables[1]=1;
	//f->Term[1].Variables[2]=1;
	//f->Term[2].Variables[3]=1;
	//f->Term[2].Variables[4]=-1;
	//f->Term[0].XPolynomial->Set(0,1);
	//f->Term[1].XPolynomial->Set(0,1);
	//f->Term[2].XPolynomial->Set(0,1);
	//f->Term[3].XPolynomial->Set(0,-6);
	//f->length=4;
	
	f->Multiply(auxf, f);

	MultVarPolynomial *Integ;
	Integ = new MultVarPolynomial(3 * d * (d + 1)/2 + (d + 1) * 12, 0);
	DoubleIntegralAcrossSkewedRegion(Integ, auxf, d);
	/*
	for(i=0;i<auxf->length;i++){
	mpf_set_ui(Integ->Term[i].Coeff,auxf->Term[i].XPolynomial->Get(0));
	for(k=0;k<MULT_VAR_VARIABLES;k++)
		Integ->Term[i].Variables[k]=auxf->Term[i].Variables[k];
	}
	Integ->length=auxf->length;
	*/
	

	// Steepest descent minimum algorithm
	
	MultVarPolynomial *deriv[MULT_VAR_VARIABLES];
	for (i = 0; i < MULT_VAR_VARIABLES; i++)
	{
		deriv[i] = new MultVarPolynomial(3 * d * (d + 1)/2 + (d + 1)*12, 0);
		Integ->PartialDerivative(deriv[i],i);
	}

	mpf_t grad[MULT_VAR_VARIABLES], P[MULT_VAR_VARIABLES], P1[MULT_VAR_VARIABLES], valP, valP1;
	mpf_t **EvaluatedPowers = NULL, *SPowers = NULL, aux1, aux2;
	double step = ConstPSPhase::INITIAL_STEEPEST_DESCENT_STEP, r, ir;
	unsigned cont = 0, good = 0, smallImprove = 0;
	EvaluatedPowers = (mpf_t**) Utils::AllocationMachine((void**)&EvaluatedPowers, (MULT_VAR_VARIABLES - 1) * sizeof(mpf_t*));
	SPowers = (mpf_t*) Utils::AllocationMachine((void**)&SPowers, (2 * d + 2) * sizeof(mpf_t));
	for (k = 0; k < MULT_VAR_VARIABLES - 1; k++)
	{
		EvaluatedPowers[k] = NULL;
		EvaluatedPowers[k] = (mpf_t*) Utils::AllocationMachine((void**)&EvaluatedPowers[k], (2 * d + 1) * sizeof(mpf_t));
		for (i = 0; i <= 2 * d; i++)
		{
			mpf_init(EvaluatedPowers[k][i]);
		}
		mpf_set_ui(EvaluatedPowers[k][0], 1);
	}
	for (i = 0; i < MULT_VAR_VARIABLES; i++)
	{
		mpf_init2(grad[i], ConstPSPhase::MPF_T_HIGH_PRECISION);
		mpf_init(P[i]);
		mpf_init(P1[i]);
	}    
	for (i = 0; i < 2 * d + 2; i++)
	{
		mpf_init(SPowers[i]);
	}

	mpf_init2(valP, ConstPSPhase::MPF_T_HIGH_PRECISION);
	mpf_init2(valP1, ConstPSPhase::MPF_T_HIGH_PRECISION);

	for (i = 0; i < MULT_VAR_VARIABLES - 1; i++)
	{
		mpf_set_ui(P[i], 0);
	}

	mpf_set_ui(P[S_LOC], ConstPSPhase::SKEWNESS);
	mpf_set_ui(SPowers[d + 1], 1);
	mpf_init2(aux1, ConstPSPhase::MPF_T_HIGH_PRECISION);
	mpf_init2(aux2, ConstPSPhase::MPF_T_HIGH_PRECISION);

	for (i = 1; i <= 2 * d; i++)
	{
		mpf_mul(EvaluatedPowers[T_LOC][i],EvaluatedPowers[T_LOC][i-1],P[T_LOC]);
	}
	for (i = 1; i <= d; i++)
	{   
		mpf_mul(SPowers[d + i + 1], SPowers[d + i], P[S_LOC]);
		mpf_ui_div(SPowers[d - i + 1], 1, SPowers[d + i +1 ]);
	}
	mpf_div(SPowers[0], SPowers[1], P[S_LOC]);
	CMinimum(P, Integ, EvaluatedPowers, SPowers, d);

	do 
	{
		if (!cont)
		{
			for (k = 0; k <= C2_LOC; k++)
			{
				for (i = 1; i <= 2; i++)
				{
					mpf_mul(EvaluatedPowers[k][i], EvaluatedPowers[k][i - 1], P[k]);
				}
			}

			for (i = 1; i <= 2 * d; i++)
			{
				mpf_mul(EvaluatedPowers[T_LOC][i], EvaluatedPowers[T_LOC][i - 1], P[T_LOC]);
			}
			for (i = 1; i <= d; i++)
			{   
				mpf_mul(SPowers[d + i + 1], SPowers[d + i], P[S_LOC]);
				mpf_ui_div(SPowers[d - i + 1], 1, SPowers[d + i + 1]);
			}

			mpf_div(SPowers[0],SPowers[1],P[S_LOC]);
			for (k = T_LOC; k < MULT_VAR_VARIABLES; k++)
			{
				deriv[k]->Evaluate(grad[k], EvaluatedPowers, SPowers, d);
			}
		}
		mpf_set_ui(aux2, 0);
		for (k = T_LOC; k < MULT_VAR_VARIABLES; k++)
		{
			mpf_pow_ui(aux1, grad[k], 2);
			mpf_add(aux2, aux2, aux1);
		}
		mpf_sqrt(aux1, aux2);

		for (k = T_LOC; k < MULT_VAR_VARIABLES; k++)
		{
			mpf_div(grad[k], grad[k], aux1);
		}

		Integ->Evaluate(valP, EvaluatedPowers, SPowers, d);

		do 
		{
			cont = 0;
			for (k = T_LOC; k < MULT_VAR_VARIABLES; k++)
			{
				mpf_set_d(P1[k], -step);
				mpf_mul(P1[k], grad[k], P1[k]);
				mpf_add(P1[k], P[k], P1[k]);
			}
			
			mpf_abs(P1[S_LOC], P1[S_LOC]);
			/*
			for(i=1;i<=2;i++)
			{
				for(k=0;k<=C2_LOC;k++)
				{
					mpf_mul(EvaluatedPowers[k][i],EvaluatedPowers[k][i-1],P1[k]);
				}
			}
			*/
			for (i = 1; i <= 2 * d; i++)
			{
				mpf_mul(EvaluatedPowers[T_LOC][i], EvaluatedPowers[T_LOC][i - 1], P1[T_LOC]);
			}
			for (i = 1; i <= d; i++)
			{   
				mpf_mul(SPowers[d + i + 1], SPowers[d + i], P1[S_LOC]);
				mpf_ui_div(SPowers[d - i + 1], 1, SPowers[d + i + 1]);
			}
			mpf_div(SPowers[0], SPowers[1], P1[S_LOC]);
			Integ->Evaluate(valP1, EvaluatedPowers, SPowers, d);
			if (mpf_cmp(valP, valP1) < 0)
			{
				step = step / 2;
				cont = 1;
				good = 0;
			} 
			else 
			{
				good++;
				r = log(sqrt(mpf_get_d(valP))) - log(sqrt(mpf_get_d(valP1)));
				ir = fabs(r / (0.5 * log(mpf_get_d(valP))));
				if (ir < 0.0002) 
				{
					smallImprove++;
					if (ir < 0.000001)
					smallImprove += 10;
				} 
				else 
				{
					smallImprove = 0;
				}
			}
			
			if (good >= 4) 
			{
				step *= 2;
				good = 0;
			}
		} 
		while (cont);

		for (i = 1; i <= 2 * d; i++)
		{
			mpf_mul(EvaluatedPowers[T_LOC][i], EvaluatedPowers[T_LOC][i - 1], P1[T_LOC]);
		}
		for (i = 1; i <= d; i++)
		{   
			mpf_mul(SPowers[d + i + 1], SPowers[d + i], P1[S_LOC]);
			mpf_ui_div(SPowers[d - i + 1], 1, SPowers[d + i + 1]);
		}
		mpf_div(SPowers[0], SPowers[1], P1[S_LOC]);
		CMinimum(P1, Integ, EvaluatedPowers, SPowers, d);
		
		for (k = 0; k < MULT_VAR_VARIABLES; k++)
		{
			mpf_set(P[k], P1[k]);
		}
	} 
	while ((step > (0.1 / this->iSteepestDescentLevel) && (smallImprove < 100 * this->iSteepestDescentLevel)));

	mpf_set_d(aux1, 0.5);
	char * s = NULL;
	s = (char*) Utils::AllocationMachine((void**)&s, 258 * sizeof(char));
	mp_exp_t t;
	mpz_t FC[MULT_VAR_VARIABLES - 1];
	for (k = 0; k < MULT_VAR_VARIABLES - 1; k++)
	{
		mpf_add(P[k], P[k], aux1);
		mpf_floor(P[k], P[k]);
		mpf_get_str(s, &t, 36, 256, P[k]);
		mpz_init(FC[k]);
		mpz_set_str(FC[k], s, 36);
	}
	mpz_t aux, sum;
	mpz_init(aux);
	mpz_init(sum);
	aResult->SetDegree(-1);

	for (i = 0; i < f->length; i++)
	{
		mpz_set_ui(sum, 1);
		for (k = 0; k < MULT_VAR_VARIABLES - 1; k++)
		{
			mpz_pow_ui(aux, FC[k], f->Term[i].Variables[k]);
			mpz_mul(sum, sum, aux);
		}
		for (j = f->Term[i].XPolynomial->GetDegree(); j >= 0; j--)
		{
			mpz_mul(aux, sum, *f->Term[i].XPolynomial->Get2(j));
			aResult->Inc(j, aux);
		}
	}

	Skewness(aSkewness, aResult, P[S_LOC]);

	mpz_add(aNewRoot,aOldRoot,FC[T_LOC]);
	for (k = 0; k <= C2_LOC; k++)
	{
		for (i = 1; i <= 2; i++)
		{
			mpf_mul(EvaluatedPowers[k][i], EvaluatedPowers[k][i - 1], P[k]);
		}
	}
	for (i = 1; i <= 2 * d; i++)
	{
		mpf_mul(EvaluatedPowers[T_LOC][i], EvaluatedPowers[T_LOC][i - 1], P[k]);
	}
	for (i = 1; i <= d; i++)
	{   
		mpf_mul(SPowers[d + i + 1], SPowers[d + i], aSkewness);
		mpf_ui_div(SPowers[d - i + 1], 1, SPowers[d + i + 1]);
	}
	Integ->Evaluate(valP, EvaluatedPowers, SPowers, d);
	(*aIntegral) = Utils::Ln(valP);
	(*aIntegral) /= 2;

	delete Tr;
	delete Rot;
	delete f;
	delete auxf;
	delete Integ;

	for (i = 0; i < MULT_VAR_VARIABLES; i++)
	{
		delete deriv[i];
		mpf_clear(grad[i]);
		mpf_clear(P[i]);
		mpf_clear(P1[i]);
	}
	for (i = 0; i < MULT_VAR_VARIABLES - 1; i++)
	{
		mpz_clear(FC[i]);
		for (j = 0; (unsigned)j <= 2 * d; j++)
		{
			mpf_clear(EvaluatedPowers[i][j]);
		}
		free(EvaluatedPowers[i]);
	}
	free(EvaluatedPowers);

	for (i = 0; i < 2 * d + 2; i++)
	{
		mpf_clear(SPowers[i]);
	}
	free(SPowers);
	
	mpz_clear(aux);
	mpz_clear(sum);
	mpf_clear(aux1);
	mpf_clear(aux2);
	mpf_clear(valP);
	mpf_clear(valP1);
	free(s);
	
	return ConstRC::Ok;
}

/*!
	Performs a sieve over different values of j<sub>0</sub> and j<sub>1</sub> to find
	such values that the polynomial aPolynomial with a root aM has the best alpha estimate aAlpha.
*/
int CPolySelectionPhase::SieveForBestAlpha(int *aJ0, int *aJ1, double *aAlpha, Polynomial *aPolynomial, mpz_t aM)
{
	int p,k,l,j1,j0,pi,mpi,pk,m,lm,lmp,flp,pkl,j0b;
	long fl;
	double cont, Malfa = 10.0;
	for (mpi = 0; this->iCoefficientPrimes[mpi] < ConstPSPhase::MAX_J_PRIME; mpi++);

	for (j1 = - ConstPSPhase::J1; j1 <= ConstPSPhase::J1; j1++)
	{
		for (j0 = - ConstPSPhase::J0; j0 <= ConstPSPhase::J0; j0++)
		{
			jAlpha[j0 + ConstPSPhase::J0] = 0.0;
		}

		for (pi = 0; pi < mpi; pi++)
		{
			p = this->iCoefficientPrimes[pi];
			pk = p;
			for (k = 1; p * pk < ConstPSPhase::MAX_J_PRIME; k++)
			{
				pk *= p;
			}

			m = mpz_fdiv_ui(aM, pk);
			for (l = 0; l < pk; l++)
			{
				for (j0 = 0; j0 < pk; j0++)
				{
					jValuations[j0][l] = 0;
				}

				fl = aPolynomial->EvaluateModPrime(l, pk);
				fl += j1 * l * l;
				fl -= j1 * l * m;
				fl %= pk;
				lm = l - m;
				lmp = 0;
				pkl = pk;

				while ((lmp < k) && (!(lm % p))) 
				{
					lm /= p;
					pkl /= p;
					lmp++;
				}

				flp = 0;
				while ((flp < lmp) && (!(fl % p))) 
				{
					fl /= p;
					flp++;
				}

				if ((flp < lmp)||(lmp == k)) 
				{
					if (flp) 
					{
						for (j0 = 0; j0 < pk; j0++)
						{
							jValuations[j0][l] = flp;
						}
					}
				} 
				else 
				{
					j0b = (CIntegerClass::InverseModN(lm, pkl) * fl) % pkl;
					if (j0b < 0)
					{
						j0b += pkl;
					}

					while (pkl > 1) 
					{
						for (j0 = j0b; j0 < pk; j0 += pkl)
						{
							jValuations[j0][l]++;
						}

						pkl /= p;
						j0b %= pkl;
					}
					if (lmp) 
					{
						for (j0 = 0; j0 < pk; j0++)
						{
							jValuations[j0][l] += lmp;
						}
					}
				}
			}

			for (j0 = 0; j0 < pk; j0++)
			{
				j0b = 0;
				for (l = 0; l < pk; l++)
				{
					j0b += jValuations[j0][l];
				}

				cont = (double)j0b / (double)pk;

				if (mpz_divisible_ui_p(*aPolynomial->Leading2(), p))
				{
					cont += (double)(p) / (double)(2 * p * p - 1);
				}

				jAlphaP[j0] = (1.0/(double)(p - 1) - cont) * log((double)p);
			}

			for (j0 = - ConstPSPhase::J0; j0 <= ConstPSPhase::J0; j0++)
			{
				k = j0 % pk;
				if (k < 0) 
				{
					k += pk;
				}
				jAlpha[j0 + ConstPSPhase::J0] += jAlphaP[k];
			}
		}

		for (j0 = - ConstPSPhase::J0; j0 <= ConstPSPhase::J0; j0++)
		{
			if (jAlpha[j0 + ConstPSPhase::J0] < Malfa) 
			{
				(*aJ0) = j0;
				(*aJ1) = j1;
				Malfa = jAlpha[j0 + ConstPSPhase::J0];
			}
		}
	}

	(*aAlpha) = Malfa;

	return ConstRC::Ok;
}

/*!
	Returns the minimum of the function in variables c<sub>0</sub>, c<sub>1</sub> and c<sub>2</sub>.
*/
int CPolySelectionPhase::CMinimum(mpf_t *aResult, MultVarPolynomial *aOperand, mpf_t **aPowerTable, mpf_t *aSPowerTable, unsigned aDegree)
{
	mpf_t M[T_LOC][T_LOC + 1], aux;
	int i, j, k;
	for (i = 0; i < T_LOC; i++)
	{
		for (j = 0; j <= T_LOC; j++)
		{
			mpf_init(M[i][j]);
		}
	}
	mpf_init(aux);

	aOperand->ThreeVariableFunction(M, aPowerTable, aSPowerTable, aDegree);
	for (i = 0; i < T_LOC; i++) 
	{
		if (mpf_cmp_ui(M[i][i],0))
		{
			for (j = T_LOC; j >= i; j--)
			{
				mpf_div(M[i][j], M[i][j], M[i][i]);
			}
			for (k = 0; k < T_LOC; k++) 
			{
				if (k != i)
				{
					for (j = T_LOC; j > i; j--)
					{
						mpf_mul(aux, M[i][j], M[k][i]);
						mpf_sub(M[k][j], M[k][j], aux);
					}
					mpf_set_ui(M[k][i], 0);
				}
			}
		}
	}

	for (i = 0; i < T_LOC; i++)
	{
		mpf_set(aResult[i], M[i][T_LOC]);
		for (j = 0; j <= T_LOC; j++)
		{
			mpf_clear(M[i][j]);
		}
	}
	mpf_clear(aux);
	
	return ConstRC::Ok;
}

/*!
	Returns the s-coefficients of the definite integral of (aOperand)<sup>2</sup>
	across the sieving region.
*/
int CPolySelectionPhase::DoubleIntegralAcrossSkewedRegion(mpf_t *aResult, Polynomial *aOperand)
{
	unsigned i, j, d = aOperand->GetDegree();
	mpz_t auxZ;
	mpf_t auxF;
	mpz_init(auxZ);
	mpf_init(auxF);

	for (i = 0; i <= d; i++)
	{
		mpf_set_ui(aResult[i], 0);
	}

	for (i = 0; i <= d; i++)
	{
		for (j = 0; j <= i; j++)
		{
			if (!((i + j) & 1))
			{
				mpz_mul(auxZ, *aOperand->Get2(i), *aOperand->Get2(j));
				mpf_set_z(auxF, auxZ);
				if (i != j)
				{
					mpf_mul_2exp(auxF, auxF, 1);
				}
				mpf_mul_2exp(auxF, auxF, 2);
				mpf_div_ui(auxF, auxF, i + j + 1);
				mpf_div_ui(auxF, auxF, 2 * d - j - i + 1);
				mpf_add(aResult[(i + j)/2], aResult[(i + j)/2], auxF);
			}
		}
	}

	mpz_clear(auxZ);
	mpf_clear(auxF);

	return ConstRC::Ok;
}

/*!
	Returns the multivariate polynomial being the definite integral of aOperand across the sieving region.
*/
int CPolySelectionPhase::DoubleIntegralAcrossSkewedRegion(MultVarPolynomial *aResult, MultVarPolynomial *aOperand, unsigned aDegree)
{
	unsigned i,j,k,l,d = aDegree << 1;
	mpf_t auxF;
	mpf_init2(auxF, ConstPSPhase::MPF_T_HIGH_PRECISION);

	l = 0;
	for (i = 0; i < aOperand->length; i++)
	{
		for (j = 0; j <= d; j++) 
		{
			if ((!(j & 1)) && (mpz_cmp_ui(*aOperand->Term[i].XPolynomial->Get2(j), 0)))
			{
				mpf_set_z(auxF, *aOperand->Term[i].XPolynomial->Get2(j));
				mpf_mul_2exp(auxF, auxF, 2);
				mpf_div_ui(auxF, auxF, j + 1);
				mpf_div_ui(auxF, auxF, d - j + 1);
				for (k = 0; k < MULT_VAR_VARIABLES - 1; k++)
				{
					aResult->Term[l].Variables[k] = aOperand->Term[i].Variables[k];
				}
				aResult->Term[l].Variables[k] = (signed)j-(signed)aDegree;
				mpf_set(aResult->Term[l].Coeff, auxF);
				l++;
			}
		}
	}

	aResult->length = l;

	mpf_clear(auxF);

	return ConstRC::Ok;
}

/*!
	Computes the rating of the non-skewed polynomial aPolynomial with the factor base length logarithm
	aLogBase. Returns also alpha of the polynomial.
*/
int CPolySelectionPhase::Rating(mpf_t aResult, mpf_t aAlpha, long * aRoots, Polynomial *aPolynomial, mpf_t aLogBase)
{
	mpf_t aux,sum;
	mpf_init(aux);
	mpf_init(sum);
	AlphaHomPoly(aAlpha, aRoots, aPolynomial);
	mpf_set_ui(aResult ,0);
	for (long i = 0; i < this->iSubintervalsCount; i++)
	{
		EvaluateAngle(sum, aPolynomial, i);
		mpf_set_d(aux, Utils::Ln(sum));
		mpf_add(sum, aAlpha, aux);
		mpf_div(aux, sum, aLogBase);
		Rho(sum, aux);
		mpf_add(aResult, aResult, sum);
	}
	mpf_clear(aux);
	mpf_clear(sum);
	return ConstRC::Ok;
}

/*!
	Computes the rating of the skewed polynomial aPolynomial 
	with skewness aSkewness and with the factor base length logarithm
	aLogBase. Returns also alpha of the polynomial.
*/
int CPolySelectionPhase::Rating(mpf_t aResult, mpf_t aAlpha, long * aRoots, Polynomial *aPolynomial, mpf_t aSkewness, mpf_t aLogBase)
{
	mpf_t aux, sum, s1, s2;
	mpf_init(aux);
	mpf_init(sum);
	mpf_init(s1);
	mpf_init(s2);
	mpf_sqrt(s1, aSkewness);
	mpf_ui_div(s2, 1, s1);
	AlphaHomPoly(aAlpha, aRoots, aPolynomial);
	mpf_set_ui(aResult, 0);

	for (long i = 0; i < this->iSubintervalsCount; i++)
	{
		EvaluateEllipseAngle(sum, aPolynomial, i, s1, s2);
		mpf_set_d(aux, Utils::Ln(sum));
		mpf_add(sum, aAlpha, aux);
		mpf_div(aux, sum, aLogBase);
		Rho(sum, aux);
		mpf_add(aResult, aResult, sum);
	}
	mpf_clear(aux);
	mpf_clear(sum);
	mpf_clear(s1);
	mpf_clear(s2);
	return ConstRC::Ok;
}

/*!
	Chooses the best polynomial among all candidate polynomials.
*/
int CPolySelectionPhase::ChoosePolynomial(long &aResult, mpf_t aLogBase)
{
	if (this->iFoundCandidatePolyCount <= 0)
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","ChoosePolynomial",0,0,"Not enough iFoundCandidatePolyCount. Count is zero!!!.");
		return ConstRC::NotFound;
	}

	int d = this->iCandidatePolynomials->Poly[0]->GetDegree();
	long * roots = NULL;
	roots = (long *)Utils::AllocationMachine((void **)&roots, d * sizeof(long));
	if (roots == NULL)
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","ChoosePolynomial",0,0,"Could not allocate memory for roots.");
		return ConstRC::NotEnoughMemory;
	}

	int rc;
	mpf_t mx;
	aResult = 0;
	mpf_init_set_ui(mx,0);

	for (long i = 0; i < this->iRatedCandidatePolyCount; i++)
	{
		if (mpf_cmp(mx, this->iCandidatePolynomials->Rating[i]) < 0) 
		{
			aResult = i;
			mpf_set(mx, this->iCandidatePolynomials->Rating[i]);
		}
	}
	
	for (; this->iRatedCandidatePolyCount < this->iFoundCandidatePolyCount; this->iRatedCandidatePolyCount++)
	{
		if (Get_InfoMode())
		{
			printf("\r\t\tCandidate polynomials rated: %ld", this->iRatedCandidatePolyCount);
			fflush(stdout);
		}

		if (mpf_cmp_si(this->iCandidatePolynomials->Skewness[this->iRatedCandidatePolyCount], 1) == 0)
		{
			Rating(this->iCandidatePolynomials->Rating[this->iRatedCandidatePolyCount], 
				this->iCandidatePolynomials->Alpha[this->iRatedCandidatePolyCount], 
				roots, this->iCandidatePolynomials->Poly[this->iRatedCandidatePolyCount], aLogBase);
		}
		else
		{
			Rating(this->iCandidatePolynomials->Rating[this->iRatedCandidatePolyCount], 
				this->iCandidatePolynomials->Alpha[this->iRatedCandidatePolyCount], 
				roots, this->iCandidatePolynomials->Poly[this->iRatedCandidatePolyCount],
				this->iCandidatePolynomials->Skewness[this->iRatedCandidatePolyCount], aLogBase);
		}

		if (mpf_cmp(mx, this->iCandidatePolynomials->Rating[this->iRatedCandidatePolyCount]) < 0) 
		{
			aResult = this->iRatedCandidatePolyCount;
			mpf_set(mx, this->iCandidatePolynomials->Rating[this->iRatedCandidatePolyCount]);
		}
		if ((this->iRatedCandidatePolyCount + 1) % this->iRSerialRunPolyCount == 0)
		{
			rc = RunRunningSerialization();
			if (rc != ConstRC::Ok)
			{
				mpf_clear(mx);
				free(roots);
				roots = NULL;
				return rc;
			}
		}
	}

	if (Get_InfoMode())
	{
		printf("\r\t\tCandidate polynomials rated: %ld", this->iRatedCandidatePolyCount);
		printf("\n\tThe best polynomial found is\n");
		this->iCandidatePolynomials->Poly[aResult]->PrintToScreen();
		gmp_printf("\n\t\twith alpha = %.*Ff and rating = %.*Ff\n\n", 10, this->iCandidatePolynomials->Alpha[aResult], 10, mx);
	}

	mpf_clear(mx);
	free(roots);
	roots = NULL;

	return ConstRC::Ok;
}
int CPolySelectionPhase::FindOptimalBounds(unsigned int aDegree, mpz_t aValue, mpz_t aLowAd, mpz_t aHighAd)
{
	int logN = mpz_sizeinbase(aValue, 2);
	double base = ((double)logN/(double)(aDegree) + (double)logN/(double)(aDegree + 1)) / 2.0;
	
	mpz_setbit(aLowAd, (unsigned long)((logN - aDegree * base) * ConstPSPhase::SKEWED_LEADING_COEFFICIENT_LOW_BOUND));
	mpz_setbit(aHighAd, (unsigned long)((logN - aDegree * base) * ConstPSPhase::SKEWED_LEADING_COEFFICIENT_HIGH_BOUND));
	
	return ConstRC::Ok;
}
int CPolySelectionPhase::FindOptimalChiValue(unsigned int aDegree, mpz_t aValue, mpz_t aLowLeading, mpz_t aHighLeading, int aStrictness)
{
	if (aStrictness < 1) 
	{
		return ConstRC::BadArgument;
	}
	mpz_t pow,aux,val,R,mm,dif,ad,m;
	int i,j,can;
	long * chi = NULL, cchi;
	gmp_randstate_t State;
	gmp_randinit_default(State);

	mpz_init(pow);
	mpz_init(val);
	mpz_init(aux);
	mpz_init(R);
	mpz_init(mm);
	mpz_init(ad);
	mpz_init(dif);
	mpz_init(m);

	chi = (long*) Utils::AllocationMachine((void **)&chi, aStrictness * sizeof(long));
	if (chi == NULL)
	{
		return ConstRC::GeneralError;
	}

	mpz_sub(dif, aHighLeading, aLowLeading);

	for(i = 0; i < aStrictness; i++)
	{
		chi[i] = 20;
	}

	Polynomial * A = new Polynomial(aDegree);
	for (can = 0; can < 10000; can++) 
	{
		mpz_urandomm(aux, State, dif);
		mpz_add(ad, aLowLeading, aux);
		A->Set(aDegree, ad);
		mpz_fdiv_q(aux,aValue,ad);
		mpz_root(mm,aux,aDegree);
		mpz_add_ui(mm,mm,(long)(20000.0*rand()/(RAND_MAX+1.0)));
		mpz_sub_ui(m,mm,10000);
		mpz_pow_ui(pow,m,aDegree);
		mpz_mul(aux,pow,ad);
		mpz_sub(val,aValue,aux);
		mpz_mul_ui(mm,m,10);
		mpz_fdiv_q_2exp(R,m,1);
		cchi = 65535;
		for (i = aDegree - 1; i >= 0; i--)
		{
			if (i)
			{
				mpz_divexact(pow,pow,m);
				mpz_fdiv_qr(aux,val,val,pow);
				A->Set(i,aux);
				if ((i<(signed)aDegree-1)&&(mpz_cmp(aux,R)>0)) 
				{
					A->Dec(i,m);
					A->Inc(i+1,1);
				}
			}
			else 
			{
				A->Set(0,val);
				if (mpz_cmp(val,R)>0) 
				{
					A->Dec(0,m);
					A->Inc(1,1);
				}
			}
			mpz_abs(aux, *A->Get2(i));
			if (mpz_cmp_ui(aux, 0) != 0) 
			{
				mpz_fdiv_q(aux, mm, aux);
			}
			if (mpz_cmp_ui(aux, cchi) < 0) 
			{
				cchi = mpz_get_ui(aux);
			}
		}
		for (i = 0; (i < aStrictness) && (chi[i] > cchi); i++);
		
		for (j = aStrictness - 1; j > i; j--)
		{
			chi[j] = chi[j - 1];
		}
		if (i < aStrictness) 
		{
			chi[i] = cchi;
		}
	}

	delete A;
	A = NULL;
	mpz_clear(pow);
	mpz_clear(val);
	mpz_clear(R);
	mpz_clear(mm);
	mpz_clear(dif);
	mpz_clear(ad);
	mpz_clear(aux);
	mpz_clear(m);

	// save result
	this->iChiInverse = chi[aStrictness - 1];

	if (Get_InfoMode()) 
	{
		printf("\n\t\tOptimal chi value is set to 10/%d.", this->iChiInverse);
		fflush(stdout);
	}
	free(chi);
	chi = NULL;

	return ConstRC::Ok;
}
/*!
	Chooses a random leading coefficient, not too smaller than aHighCoeff.
*/
int CPolySelectionPhase::ChooseLeadingCoefficient(mpz_t aResult, mpz_t aHighCoeff)
{
	mpz_t c, c_limit, aux;
	mpz_init_set_ui(c, 1058400);	    // 32*27*25*49
	mpz_init(c_limit);
	mpz_init(aux);
	mpz_fdiv_q_ui(c_limit, aHighCoeff, 30000);
	
	if (mpz_cmp(c, c_limit) > 0)
	{
		mpz_set_ui(c, 420);
		if (mpz_cmp(c, c_limit) > 0) 
		{
			mpz_set_ui(c, 12);
			if (mpz_cmp(c, c_limit) > 0) 
			{
				mpz_set_ui(c, 2);
			}
		}
	}

	int primes, p;
	if (mpz_cmp_ui(aHighCoeff, 1000) < 0)
	{
		primes = this->iPrimesDividingLeading >> 1;
	}
	else 
	{
		primes = this->iPrimesDividingLeading;
	}
	mpz_fdiv_q_ui(c_limit, aHighCoeff, 2);
	//mpz_div_2exp(c_limit, aHighCoeff, 1);

	while (mpz_cmp(c, c_limit) < 0) 
	{
		p = this->iCoefficientPrimes[CIntegerClass::RandomNumber(primes)];
		mpz_mul_ui(aux, c, p);
		if (mpz_cmp(aux, aHighCoeff) < 0)
		{
			mpz_set(c, aux);
		}
	}
	mpz_set(aResult, c);

	mpz_clear(c);
	mpz_clear(aux);
	mpz_clear(c_limit);
	
	return ConstRC::Ok;
}
int CPolySelectionPhase::GenerateMonic()
{
	int degree = mpz_sizeinbase(this->iN, 2) / ConstPSPhase::POLY_DEGREE_STEP;
	if (degree < ConstPSPhase::MIN_POLY_DEGREE)
	{
		degree = ConstPSPhase::MIN_POLY_DEGREE;
	}

	mpz_t high_m,low_m,high_ad,low_ad;
	mpz_t best_m,coeff_bound,k,ad1;
	mpf_t alfa;
	Polynomial *A;
	mpz_t aux,pow,val,R;
	int dir, good, delay = 0;
	long i;

	mpz_init(aux);
	mpz_init(best_m);
	mpz_init(coeff_bound);
	mpz_init(k);
	mpz_init(ad1);
	mpz_init(R);
	mpz_init(val);
	mpz_init(pow);
	mpz_init(high_m);
	mpz_init(low_m);
	mpz_init(high_ad);
	mpz_init(low_ad);
	mpf_init(alfa);
	A = new Polynomial(degree);

	mpz_set_ui(high_ad, 2);
	mpz_set_ui(low_ad, 1); // leading coeff from 1 to 1
	mpz_mul_ui(aux, this->iN, 2); // 2N
	mpz_root(high_m, aux, degree); // max root_M is the degree-th root of 2N
	mpz_cdiv_q_ui(aux, aux, 3);  // 2N/3
	mpz_root(low_m, aux, degree); // min root_M is the degree-th root of 2N/3
	this->iMaxPerCoefficient = this->iCandidatesCount;

	if (Get_InfoMode())
	{
		gmp_printf("\t\tMonic polynomial search started.\n\t\tRoots are taken from %Zd to %Zd.\n", low_m, high_m);
	}
	if (this->iChiInverse <= 0) 
	{
		if (!this->iChiInverse) // Chi == 0
		{
			this->iChiInverse = ConstPSPhase::CHI_INVERSE;
		}
		FindOptimalChiValue(degree, this->iN, low_ad, high_ad, -this->iChiInverse);
	}

	mpz_root(best_m, this->iN, degree);  // root_M is the degree-th root of aux

	// coeff_bound according to Chi
	mpz_fdiv_q_ui(coeff_bound, best_m, this->iChiInverse);
	mpz_sub(k, high_m, low_m);  // interval for root_M

	//if (Get_InfoMode())
	//{
	//	gmp_printf("The range is %Zd\n",k);
	//}

	A->Set(degree, 1); // monic polynomial
	mpz_add(ad1, best_m, k);    // root_M + k
	
	if (mpz_cmp(ad1, high_m) > 0)  // if too big
	{
		mpz_set(ad1, high_m);
	}
	good = 0;

	if (this->iCandidatePolynomials == NULL) // it could be deserialize
	{
		this->iCandidatePolynomials = new CandidateClass(this->iCandidatesCount);

		mpz_set(this->iWorkingMonicM, best_m);  // as iWorkingMonicM choose root_M
		dir = 1;
		this->iFoundCandidatePolyCount = 0;
	} 
	else 
	{
		// we deserialized this phase
		if (mpz_cmp(this->iWorkingMonicM, best_m) > 0)  // iWorkingMonicM > root_M
		{
			dir = 1;
		}
		else   // trying_root_M <= root_M
		{
			dir = -1;
			mpz_sub(ad1, best_m, k);     // root_M - k
			if (mpz_cmp(ad1, low_m) < 0) // if too small
			{
				mpz_set(ad1, low_m);
			}
		}
	}

	int rc = ConstRC::Ok;
	long round = 0;

	for (; this->iFoundCandidatePolyCount < this->iCandidatesCount;
		 (dir == 1 ? mpz_add_ui(this->iWorkingMonicM, this->iWorkingMonicM, 1) : mpz_sub_ui(this->iWorkingMonicM, this->iWorkingMonicM, 1)))
	{
		if (!good)  // good == 0
		{
			this->iStats->Increase(EPSSTested);

			round++;
			if (round % this->iRSerialRunMChangeCount == 0)
			{
				rc = RunRunningSerialization();
				if (rc != ConstRC::Ok)
				{
					break;
				}
			}
			
			//gmp_printf("\r%Zd: ", this->iWorkingMonicM);
			mpz_pow_ui(pow, this->iWorkingMonicM, degree); // m^{degree}
			mpz_sub(val, this->iN, pow);   // val = N - m^{degree}
			mpz_fdiv_q_2exp(R, this->iWorkingMonicM, 1);    // R = m/2 -- it is ">> 1" for gmp
			for (i = degree - 1; i >= 0; i--) // create poly
			{
				//gmp_printf(" %Zd ",val);
				if (i)
				{
					mpz_divexact(pow, pow, this->iWorkingMonicM); // lower m power
					mpz_fdiv_qr(aux, val, val, pow); // aux is quotient, val is remainder
					A->Set(i, aux);                  // aux is new coef
					if ((i < (signed)degree - 1) && (mpz_cmp(aux, R) > 0)) // if is bigger than R = m/2
					{
						// shift coeff
						A->Dec(i, this->iWorkingMonicM);
						A->Inc(i + 1, 1);
					}
				}
				else 
				{
					A->Set(0, val);
					if (mpz_cmp(val, R) > 0) // if is bigger than R = m/2
					{
						// shift coeff
						A->Dec(0, this->iWorkingMonicM);
						A->Inc(1, 1);
					}
				}

				mpz_abs(aux, *A->Get2(i)); // get abs value
				if (mpz_cmp(aux, coeff_bound) >= 0)  // if is too big for coeff
				{
					good = ((i + degree) >> 2) + 1;
					this->iStats->Increase(EPSSRejectedCoeff);
					break;
				}
			}

			if (!good) // good == 0
			{
				AlphaEstimateHomPoly(alfa, A);
				if (mpf_cmp_d(alfa, this->iAlphaBound) <= 0)
				{
					A->Copy(this->iCandidatePolynomials->Poly[this->iFoundCandidatePolyCount]);
					mpz_set(this->iCandidatePolynomials->Root[this->iFoundCandidatePolyCount], this->iWorkingMonicM);
//						A->PrintToScreen();
//						gmp_printf(" je cislo %d %Ff\n",iFoundCandidatePolyCount,alfa);
					this->iFoundCandidatePolyCount++;

					if (this->iSelfInitializingDelay)
					{
						this->iAlphaBound -= ConstPSPhase::ALPHA_STEP_BACK;
					}
					delay = 0;

					if (Get_InfoMode())
					{
						printf("\r\t\tCandidate polynomials found: %ld", this->iFoundCandidatePolyCount);
						fflush(stdout);
					}

					if ((dir == 1) && ((this->iFoundCandidatePolyCount << 1) >= this->iMaxPerCoefficient))
					{
						dir = -1;
						mpz_set(this->iWorkingMonicM, best_m);
						mpz_sub(ad1, this->iWorkingMonicM, k);
						if(mpz_cmp(ad1, low_m) < 0)
						{
							mpz_set(ad1, low_m);
						}
					}
				}
				else
				{
					delay++; // change alpha bound for finding something?
					if ((this->iSelfInitializingDelay) && (delay > this->iSelfInitializingDelay))
					{
						delay = 0;
						this->iAlphaBound += ConstPSPhase::ALPHA_STEP;
						//printf("Change of alpha to %f\n", this->iAlphaBound);
					}
					this->iStats->Increase(EPSSRejectedAlpha);
				}
			} 
			else 
			{
				good--;
			}

			if (dir == 1)
			{
				if (mpz_cmp(this->iWorkingMonicM, ad1) >= 0)
				{
					// we change direction
					dir = -1;
					mpz_set(this->iWorkingMonicM, best_m);
					mpz_sub(ad1, this->iWorkingMonicM, k);
					if (mpz_cmp(ad1, low_m) < 0) 
					{
						mpz_set(ad1, low_m);
					}
				}
			}
			else if (mpz_cmp(this->iWorkingMonicM, ad1) <= 0)
			{
				// we try all posibilities
				break;	
			}
		}
		else 
		{
			good--;
		}
	}

	if (Get_InfoMode())
	{
		printf("\n");
	}

	mpz_clear(aux);
	mpz_clear(high_m);
	mpz_clear(low_m);
	mpz_clear(high_ad);
	mpz_clear(low_ad);
	mpz_clear(best_m);
	mpz_clear(coeff_bound);
	mpz_clear(k);
	mpz_clear(ad1);
	mpz_clear(R);
	mpz_clear(val);
	mpz_clear(pow);
	mpf_clear(alfa);
	delete A;
	A = NULL;

	return rc;
}


int CPolySelectionPhase::GenerateNonmonic()
{
	int rc = ConstRC::Ok;
	if (this->iCandidatePolynomials == NULL) // it could be deserialize
	{
		this->iCandidatePolynomials = new CandidateClass(this->iCandidatesCount);
	}

	int degree = mpz_sizeinbase(this->iN, 2) / ConstPSPhase::POLY_DEGREE_STEP;
	if (degree < ConstPSPhase::MIN_POLY_DEGREE)
	{
		degree = ConstPSPhase::MIN_POLY_DEGREE;
	}

	Polynomial * poly = new Polynomial(degree);
	long * roots = NULL;
	roots = (long *)Utils::AllocationMachine((void **)&roots, degree * sizeof(long));
	if (roots == NULL)
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","GenerateNonmonic",0,0,"Could not allocate memory for roots.");
		delete poly;
		poly = NULL;
		return ConstRC::NotEnoughMemory;
	}

	mpz_t dc;
	mpz_t dMainAd;
	mpz_t maxAd;
	mpz_t minAd;
	mpz_t mainAd_1;
	mpz_t runningAd_1;
	mpz_t runningM;
	mpz_t mainM;
	mpz_t powerM;
	mpz_t aux1;
	mpz_t coeffBound;
	mpz_t remainder;
	mpz_t halfM;
	mpf_t alpha;

#ifdef DEBUG_POLY_SEL_PHASE_ASSERTION_MODE_ON
	mpz_t control;
	mpz_t coeffBoundNeg;
	mpz_init(control);
	mpz_init(coeffBoundNeg);
#endif

	mpz_init(dc);
	mpz_init(dMainAd);
	mpz_init(maxAd);
	mpz_init(minAd);
	mpz_init(mainAd_1);
	mpz_init(runningAd_1);
	mpz_init(runningM);
	mpz_init(mainM);
	mpz_init(powerM);
	mpz_init(aux1);
	mpz_init(coeffBound);
	mpz_init(remainder);
	mpz_init(halfM);
	mpf_init(alpha);

	int delay = 0;

	int logN = mpz_sizeinbase(this->iN, 2);

	// (d * log(chi) + log N) / d + 1 = bound
	mpz_setbit(minAd, (degree * this->iChiMinLog + logN) / (degree + 1));
	mpz_setbit(maxAd, (degree * this->iChiMaxLog + logN) / (degree + 1));

	if (Get_InfoMode())
	{
		gmp_printf("\n\t\tMin a_{d} bound is %Zd.", minAd);
		gmp_printf("\n\t\tMax a_{d} bound is %Zd.", maxAd);
		printf("\n");
	}

	if (this->iChiInverse <= 0) 
	{
		if (!this->iChiInverse) // Chi == 0
		{
			this->iChiInverse = ConstPSPhase::CHI_INVERSE;
		}
		FindOptimalChiValue(degree, this->iN, minAd, maxAd, -this->iChiInverse);
	}

	long minK;
	long maxK;
	long k;

	int * cprimesEND;
	int * allCPrimesEND;
	long round;

#ifdef DEBUG_POLY_SEL_PHASE_TIME_MESSAGE_ON
	this->iTimes->Start(EPSTCompC);
#endif

	// computing the number of primes in c

	if (this->iRSerialCPrimes == NULL) // it could be deserialize
	{
		mpz_sub(aux1, maxAd, minAd);   // length of a_{d} interval
		mpz_cdiv_q_ui(aux1, aux1, (unsigned long)this->iMinAdPerC);     // max this->iC value
		if (mpz_cmp(minAd, aux1) < 0)  // min from a_{d}^{(min)} and length of a_{d} interval
		{
			mpz_set(aux1, minAd);
		}
		mpz_cdiv_q_ui(aux1, aux1, (unsigned long)ConstPSPhase::C_BASE); // max for this->iC value
		this->iRSerialCPrimesCount = mpz_sizeinbase(aux1, 10) / 2;      // how much primes can we use?
		this->iRSerialCPrimes = (int *)Utils::AllocationMachine((void**)&this->iRSerialCPrimes, this->iRSerialCPrimesCount * sizeof(int));
		if (this->iRSerialCPrimes == NULL)
		{
			CLog::LogError(WHEREARG,"CPolySelectionPhase","GenerateNonmonic",0,0,"Could not allocate memory.");
			rc = ConstRC::NotEnoughMemory;
			goto generate_clear_gmp;
		}

		// fill first combination
		for (k = 0; k < this->iRSerialCPrimesCount; k++)
		{
			this->iRSerialCPrimes[k] = this->iCPrimes[k];
		}
	}

	this->iWorkingCPrimes = (int *)Utils::AllocationMachine((void**)&this->iWorkingCPrimes, this->iRSerialCPrimesCount * sizeof(int));
	if (this->iWorkingCPrimes == NULL)
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","GenerateNonmonic",0,0,"Could not allocate memory.");
		rc = ConstRC::NotEnoughMemory;
		goto generate_clear_gmp;
	}

	// fill first combination started according to RSerialCPrimes
	for (k = 0; k < this->iRSerialCPrimesCount; k++)
	{
		this->iWorkingCPrimes[k] = this->iRSerialCPrimes[k];
	}

	// aux pointers
	cprimesEND = this->iWorkingCPrimes + this->iRSerialCPrimesCount;
	allCPrimesEND = this->iCPrimes + ConstPSPhase::C_PRIME_COUNT;

	if (Get_InfoMode())
	{
		PrintHeader();
	}

	round = 0;

	do
	{
		round++;
		if (round % this->iRSerialRunCChangeCount == 0)
		{
			rc = RunRunningSerialization();
			if (rc != ConstRC::Ok)
			{
				goto generate_clear_gmp;
			}
		}

		mpz_set_si(this->iC, ConstPSPhase::C_BASE); // base is 2 * 2 * 3
		for (k = 0; k < this->iRSerialCPrimesCount; k++)
		{
			mpz_mul_ui(this->iC, this->iC, this->iWorkingCPrimes[k]);
		}

#ifdef DEBUG_POLY_SEL_PHASE_TIME_MESSAGE_ON
		this->iTimes->End(EPSTCompC);
#endif

		mpz_mul_si(dc, this->iC, degree);

		// trying leading coeff ...
		// we start with first c-multiplier bigger than minAd and go to maxAd
		// ceil(minAd / c) * c => c*first > minAd // for next computation we need degree * this->iMainAd
		for (mpz_cdiv_q(this->iMainAd, minAd, this->iC), mpz_mul(this->iMainAd, this->iMainAd, this->iC), mpz_mul_si(dMainAd, this->iMainAd, degree);
			 mpz_cmp(this->iMainAd, maxAd) < 0;  // this->iC*first + this->iC*i < maxAd
			 // c*first + c*i -> c*first + c*(i+1) = c*first + c*i + c
			 // d * (c*first + c*i) -> d * (c*first + c*(i+1)) = d * (c*first + c*i) + d * c
			 mpz_add(this->iMainAd, this->iMainAd, this->iC), mpz_add(dMainAd, dMainAd, dc))
		{
			poly->Set(degree, this->iMainAd);

			if (Get_InfoMode())
			{
				PrintValues();
			}

			// now we find best m
			mpz_fdiv_q(mainM, this->iN, this->iMainAd);
			mpz_root(mainM, mainM, degree);

			// we have coeff a_{d} and we want corresponding coeff a_{d - 1}
			mpz_pow_ui(powerM, mainM, degree); // m^{degree}
			mpz_mul(aux1, this->iMainAd, powerM);     // a_{d} * m^{degree}
			mpz_sub(aux1, this->iN, aux1);        // N -  a_{d} * m^{degree}
			mpz_divexact(powerM, powerM, mainM);  // m^{degree - 1}
			mpz_fdiv_q(mainAd_1, aux1, powerM);   // (N -  a_{d} * m^{degree}) / m^{degree - 1}

			// trying corresponding m ...
			// preparation - firstly minK and maxK
			// coeff bound - it is not exact number
			mpz_fdiv_q_ui(coeffBound, mainM, (unsigned long)this->iChiInverse);
#ifdef DEBUG_POLY_SEL_PHASE_ASSERTION_MODE_ON
			mpz_neg(coeffBoundNeg, coeffBound);
#endif
			// minK = (a_{d - 1} - coeffBound) / (d * a_{d})
			mpz_sub(aux1, mainAd_1, coeffBound);
			mpz_cdiv_q(aux1, aux1, dMainAd);
			if (mpz_fits_slong_p(aux1))
			{
				minK = mpz_get_si(aux1);
			}
			else
			{
				minK = - (this->iMaxPerCoefficient >> 1);
			}
			// maxK = (a_{d - 1} + coeffBound) / (d * a_d{d})
			mpz_add(aux1, mainAd_1, coeffBound);
			mpz_cdiv_q(aux1, aux1, dMainAd);
			if (mpz_fits_slong_p(aux1))
			{
				maxK = mpz_get_si(aux1);
			}
			else
			{
				maxK = this->iMaxPerCoefficient >> 1;
			}		

			// now other things...
			mpz_mul_si(aux1, dMainAd, minK);                   // starting k * d * a_{d}
			mpz_sub(runningAd_1, mainAd_1, aux1);       // starting 'a_{d - 1}' = a_{d - 1} - k * d * a_{d}
			mpz_sub_ui(runningM, mainM, (unsigned long)-minK); // starting 'm'
			mpz_fdiv_q_2exp(halfM, runningM, 1);               // starting 'm / 2'
			for (k = minK; k < maxK; k++)
			{
#ifdef DEBUG_POLY_SEL_PHASE_ASSERTION_MODE_ON
				if (Get_AssertionMode())
				{
					if (k <= 0)
					{
						if (mpz_cmp(runningAd_1, coeffBound) >= 0)
						{
							gmp_printf("STRANGE too big a_{d - 1} = %Zd, bound = %Zd", runningAd_1, coeffBound);
						}
					}
					else
					{
						if (mpz_cmp(runningAd_1, coeffBoundNeg) <= 0)
						{
							gmp_printf("STRANGE too small a_{d - 1} = %Zd, bound = %Zd", runningAd_1, coeffBoundNeg);
						}
					}
				}
#endif
#ifdef DEBUG_POLY_SEL_PHASE_INFO_STATS_ON
				this->iStats->Increase(EPSSTested);
#endif
				poly->Set(degree - 1, runningAd_1);

				// compute all poly coeff
				mpz_pow_ui(powerM, runningM, degree);      // (m + k)^{degree}
				mpz_mul(aux1, this->iMainAd, powerM);      // a_{d} * (m + k)^{degree}
				mpz_sub(remainder, this->iN, aux1);        // N -  a_{d} * (m + k)^{degree}
				mpz_divexact(powerM, powerM, runningM);    // (m + k)^{degree - 1}
				mpz_mul(aux1, runningAd_1, powerM);        // a_{d - 1} * (m + k)^{degree - 1}
				mpz_sub(remainder, remainder, aux1);       // N -  a_{d} * (m + k)^{degree} - a_{d - 1) * (m + k)^{degree - 1}
			
				if (ComputeRemainingCoeff(poly, degree, runningM, powerM, remainder, halfM, coeffBound, aux1))
				{
#ifdef DEBUG_POLY_SEL_PHASE_ASSERTION_MODE_ON
					if (Get_AssertionMode())
					{
						poly->Evaluate(control, runningM);
						mpz_mod(control, control, this->iN);
						if (mpz_cmp_ui(control, 0))
						{
							gmp_printf("Error - %Zd\n", control);
							poly->PrintToScreen();
							printf("\n");
						}
					}
#endif
#ifdef DEBUG_POLY_SEL_PHASE_TIME_MESSAGE_ON
					this->iTimes->Start(EPSTCompAlpha);
#endif
					AlphaEstimateHomPoly(alpha, roots, poly);
#ifdef DEBUG_POLY_SEL_PHASE_TIME_MESSAGE_ON
					this->iTimes->End(EPSTCompAlpha);
#endif
					if (mpf_cmp_d(alpha, this->iAlphaBound) <= 0)
					{
						poly->Copy(this->iCandidatePolynomials->Poly[this->iFoundCandidatePolyCount]);
						mpz_set(this->iCandidatePolynomials->Root[this->iFoundCandidatePolyCount], runningM);
						this->iFoundCandidatePolyCount++;

						if (this->iFoundCandidatePolyCount >= this->iCandidatesCount)
						{
							rc = ConstRC::Ok;
							goto generate_clear_gmp;
						}

						if (this->iSelfInitializingDelay)
						{
							this->iAlphaBound -= ConstPSPhase::ALPHA_STEP_BACK;
						}
						delay = 0;
					}
					else
					{
#ifdef DEBUG_POLY_SEL_PHASE_INFO_STATS_ON
						this->iStats->Increase(EPSSRejectedAlpha);
#endif
						delay++; // change alpha bound for finding something?
						if ((this->iSelfInitializingDelay) && (delay > this->iSelfInitializingDelay))
						{
							delay = 0;
							this->iAlphaBound += ConstPSPhase::ALPHA_STEP;
							//printf("Change of alpha to %f\n", this->iAlphaBound);
						}
					}

				} // end if - good poly	
#ifdef DEBUG_POLY_SEL_PHASE_INFO_STATS_ON
				else 
				{
					this->iStats->Increase(EPSSRejectedCoeff);
				}
#endif

				mpz_add_ui(runningM, runningM, 1);          // next running m
				mpz_sub(runningAd_1, runningAd_1, dMainAd); // next running a_{d - 1}
				// we suppose that we started with even k
				if (k % 2 == 1)
				{
					mpz_add_ui(halfM, halfM, 1);
				}

			} // end for cycle - trying corresponding m

		} // end for cycle - trying leading coeff

#ifdef DEBUG_POLY_SEL_PHASE_TIME_MESSAGE_ON
		this->iTimes->Start(EPSTCompC);
#endif

	} 
	while (Utils::NextCombination(this->iCPrimes, allCPrimesEND, this->iWorkingCPrimes, cprimesEND)); 

	rc = ConstRC::Ok;

generate_clear_gmp:

	if (Get_InfoMode())
	{
		PrintValues();
		printf("\n");
	}

	mpz_clear(dc);
	mpz_clear(dMainAd);
	mpz_clear(maxAd);
	mpz_clear(minAd);
	mpz_clear(mainAd_1);
	mpz_clear(runningAd_1);
	mpz_clear(runningM);
	mpz_clear(mainM);
	mpz_clear(powerM);
	mpz_clear(aux1);
	mpz_clear(coeffBound);
	mpz_clear(remainder);
	mpz_clear(halfM);
	mpf_clear(alpha);

#ifdef DEBUG_POLY_SEL_PHASE_ASSERTION_MODE_ON
	mpz_clear(control);
	mpz_clear(coeffBoundNeg);
#endif

	if (poly != NULL)
	{
		delete poly;
		poly = NULL;
	}
	if (roots != NULL)
	{
		free(roots);
		roots = NULL;
	}

	return rc;
}
int CPolySelectionPhase::GenerateSkewed()
{
	int rc = ConstRC::Ok;
	if (this->iCandidatePolynomials == NULL) // it could be deserialize
	{
		this->iCandidatePolynomials = new CandidateClass(this->iCandidatesCount);
	}

	int degree = mpz_sizeinbase(this->iN, 2) / ConstPSPhase::POLY_DEGREE_STEP;
	if (degree < ConstPSPhase::MIN_POLY_DEGREE)
	{
		degree = ConstPSPhase::MIN_POLY_DEGREE;
	}

	Polynomial * poly = new Polynomial(degree);

	mpz_t maxAd;
	mpz_t minAd;
	mpz_t mainAd_1;
	mpz_t runningM;
	mpz_t mainM;
	mpz_t powerM;
	mpz_t aux1;
	mpz_t remainder;
	mpz_t halfM;

	mpz_init(maxAd);
	mpz_init(minAd);
	mpz_init(mainAd_1);
	mpz_init(runningM);
	mpz_init(mainM);
	mpz_init(powerM);
	mpz_init(aux1);
	mpz_init(remainder);
	mpz_init(halfM);

#ifdef DEBUG_POLY_SEL_PHASE_ASSERTION_MODE_ON
	mpz_t control;
	mpz_init(control);
#endif

	FindOptimalBounds(degree, this->iN, minAd, maxAd);

	if (Get_InfoMode())
	{
		gmp_printf("\n\t\tMin a_{d} bound is %Zd.", minAd);
		gmp_printf("\n\t\tMax a_{d} bound is %Zd.", maxAd);
		printf("\n");
	}

	long k;
	int * cprimesEND;
	int * allCPrimesEND;

#ifdef DEBUG_POLY_SEL_PHASE_TIME_MESSAGE_ON
	this->iTimes->Start(EPSTCompC);
#endif

	long round = 0;
	int logM;
	int logAd;
	mpf_t skew;
	mpf_init(skew);
	double integral, gIFS, alfa;
	int j0, j1;

	// computing the number of primes in c

	if (this->iRSerialCPrimes == NULL) // it could be deserialize
	{
		mpz_sub(aux1, maxAd, minAd);   // length of a_{d} interval
		mpz_cdiv_q_ui(aux1, aux1, (unsigned long)this->iMinAdPerC);     // max this->iC value
		if (mpz_cmp(minAd, aux1) < 0)  // min from a_{d}^{(min)} and length of a_{d} interval
		{
			mpz_set(aux1, minAd);
		}
		mpz_cdiv_q_ui(aux1, aux1, (unsigned long)ConstPSPhase::C_BASE); // max for this->iC value
		this->iRSerialCPrimesCount = mpz_sizeinbase(aux1, 10) / 2 + 1;  // how much primes can we use?
		this->iRSerialCPrimes = (int *)Utils::AllocationMachine((void**)&this->iRSerialCPrimes, this->iRSerialCPrimesCount * sizeof(int));
		if (this->iRSerialCPrimes == NULL)
		{
			CLog::LogError(WHEREARG,"CPolySelectionPhase","GenerateNonmonic",0,0,"Could not allocate memory.");
			rc = ConstRC::NotEnoughMemory;
			goto generate_skewed_clear_gmp;
		}

		// fill first combination
		for (k = 0; k < this->iRSerialCPrimesCount; k++)
		{
			this->iRSerialCPrimes[k] = this->iCPrimes[k];
		}
	}

	this->iWorkingCPrimes = (int *)Utils::AllocationMachine((void**)&this->iWorkingCPrimes, this->iRSerialCPrimesCount * sizeof(int));
	if (this->iWorkingCPrimes == NULL)
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","GenerateNonmonic",0,0,"Could not allocate memory.");
		rc = ConstRC::NotEnoughMemory;
		goto generate_skewed_clear_gmp;
	}

	// fill first combination started according to RSerialCPrimes
	for (k = 0; k < this->iRSerialCPrimesCount; k++)
	{
		this->iWorkingCPrimes[k] = this->iRSerialCPrimes[k];
	}

	// aux pointers
	cprimesEND = this->iWorkingCPrimes + this->iRSerialCPrimesCount;
	allCPrimesEND = this->iCPrimes + ConstPSPhase::C_PRIME_COUNT;

	if (Get_InfoMode())
	{
		PrintHeader();
	}
	
	do
	{
		round++;
		if (round % this->iRSerialRunCChangeCount == 0)
		{
			rc = RunRunningSerialization();
			if (rc != ConstRC::Ok)
			{
				goto generate_skewed_clear_gmp;
			}
		}

		mpz_set_si(this->iC, ConstPSPhase::C_BASE); // base is 2 * 2 * 3
		for (k = 0; k < this->iRSerialCPrimesCount; k++)
		{
			mpz_mul_ui(this->iC, this->iC, this->iWorkingCPrimes[k]);
		}

#ifdef DEBUG_POLY_SEL_PHASE_TIME_MESSAGE_ON
		this->iTimes->End(EPSTCompC);
#endif


		// trying leading coeff ...
		// we start with first c-multiplier bigger than minAd and go to maxAd
		// ceil(minAd / c) * c => c*first > minAd 
		for (mpz_cdiv_q(this->iMainAd, minAd, this->iC), mpz_mul(this->iMainAd, this->iMainAd, this->iC);
			 mpz_cmp(this->iMainAd, maxAd) < 0;  // this->iC*first + this->iC*i < maxAd
			 // c*first + c*i -> c*first + c*(i+1) = c*first + c*i + c
			 mpz_add(this->iMainAd, this->iMainAd, this->iC))
		{

#ifdef DEBUG_POLY_SEL_PHASE_INFO_STATS_ON
			this->iStats->Increase(EPSSTested);
#endif
			// now we find best m
			mpz_fdiv_q(mainM, this->iN, this->iMainAd);
			mpz_root(mainM, mainM, degree);

			// we have coeff a_{d} and we want corresponding coeff a_{d - 1}
			mpz_pow_ui(powerM, mainM, degree);    // m^{degree}
			mpz_mul(aux1, this->iMainAd, powerM); // a_{d} * m^{degree}
			mpz_sub(remainder, this->iN, aux1);   // N -  a_{d} * m^{degree}
			mpz_divexact(powerM, powerM, mainM);  // m^{degree - 1}
			mpz_fdiv_qr(mainAd_1, remainder, remainder, powerM);   // (N -  a_{d} * m^{degree}) / m^{degree - 1}

			logM = mpz_sizeinbase(mainM, 2);
			logAd = mpz_sizeinbase(this->iMainAd, 2);

			if ((mpz_sizeinbase(remainder,2) - (degree - 2)* logM) <= 
				(logM - logAd) * this->iThirdCoefficientBound + logAd)
			{
				if (Get_InfoMode())
				{
					PrintValues();
				}

				poly->Set(degree, this->iMainAd);
				poly->Set(degree - 1, mainAd_1);

				mpz_fdiv_q_2exp(halfM, mainM, 1); // m / 2

				if (ComputeRemainingCoeff(poly, degree, mainM, powerM, remainder, halfM, this->iN, aux1) == false)
				{
					// problem!!!
					continue;
				}

				gIFS = logM * sqrt(sqrt((double)degree)) * this->iIFSBound;

				// times
				SkewPolynomial(poly, skew, runningM, &integral, poly, mainM);

				if (integral < gIFS)
				{ 
					// times
					SieveForBestAlpha(&j0, &j1, &alfa, poly, runningM);

					if (alfa + integral < gIFS - this->iIFSImprovement)
					{
						poly->Inc(2, j1);

						mpz_mul_si(aux1, runningM, j1);
						poly->Dec(1, aux1);
						poly->Dec(1, j0);

						mpz_mul_si(aux1, runningM, j0);
						poly->Inc(0, aux1);

#ifdef DEBUG_POLY_SEL_PHASE_ASSERTION_MODE_ON
						if (Get_AssertionMode())
						{
							poly->Evaluate(control, runningM);
							mpz_mod(control, control, this->iN);
							if (mpz_cmp_ui(control, 0))
							{
								gmp_printf("Error - %Zd\n", control);
								poly->PrintToScreen();
								printf("\n");
							}
						}
#endif

						Skewness(skew, poly, skew);
						
						poly->Copy(this->iCandidatePolynomials->Poly[this->iFoundCandidatePolyCount]);
						mpz_set(this->iCandidatePolynomials->Root[this->iFoundCandidatePolyCount], runningM);
						mpf_set(this->iCandidatePolynomials->Skewness[this->iFoundCandidatePolyCount], skew);
						this->iFoundCandidatePolyCount++;

						if (this->iFoundCandidatePolyCount >= this->iCandidatesCount)
						{
							rc = ConstRC::Ok;
							goto generate_skewed_clear_gmp;
						}
					} 
#ifdef DEBUG_POLY_SEL_PHASE_INFO_STATS_ON
					else
					{
						this->iStats->Increase(EPSSRejectedAlpha);
					}
#endif

				}
			}
#ifdef DEBUG_POLY_SEL_PHASE_INFO_STATS_ON
			else 
			{
				this->iStats->Increase(EPSSRejectedCoeff);
			}
#endif

		} // end for cycle - trying leading coeff

#ifdef DEBUG_POLY_SEL_PHASE_TIME_MESSAGE_ON
		this->iTimes->Start(EPSTCompC);
#endif

	} 
	while (Utils::NextCombination(this->iCPrimes, allCPrimesEND, this->iWorkingCPrimes, cprimesEND)); 

	rc = ConstRC::Ok;

generate_skewed_clear_gmp:

	if (Get_InfoMode())
	{
		PrintValues();
		printf("\n");
	}

	mpz_clear(maxAd);
	mpz_clear(minAd);
	mpz_clear(mainAd_1);
	mpz_clear(runningM);
	mpz_clear(mainM);
	mpz_clear(powerM);
	mpz_clear(aux1);
	mpz_clear(remainder);
	mpz_clear(halfM);
	mpf_clear(skew);

#ifdef DEBUG_POLY_SEL_PHASE_ASSERTION_MODE_ON
	mpz_clear(control);
#endif

	if (poly != NULL)
	{
		delete poly;
		poly = NULL;
	}

	return rc;
}
bool CPolySelectionPhase::ComputeRemainingCoeff(Polynomial * aPoly, int aDegree, mpz_t aM, mpz_t aPowerM, mpz_t aRemainder, mpz_t aHalfM, mpz_t aCoeffBound, mpz_t aAux)
{
	for (int i = aDegree - 2; i > 0; i--)
	{
		mpz_divexact(aPowerM, aPowerM, aM);                 // lower m power
		mpz_fdiv_qr(aAux, aRemainder, aRemainder, aPowerM); // aux1 is quotient = new coeff
		aPoly->Set(i, aAux);                                // aux is new coef
		if ((mpz_cmp(aAux, aHalfM) > 0))                    // if is bigger than m / 2
		{
			// shift coeff
			aPoly->Dec(i, aM);
			aPoly->Inc(i + 1, 1);
		}

		mpz_abs(aAux, *aPoly->Get2(i));       // get abs value
		if (mpz_cmp(aAux, aCoeffBound) >= 0)  // if is too big for coeff
		{
			return false;
		}
	} // end for cycle - computing poly coeff 

	// last coeff
	aPoly->Set(0, aRemainder);
	if (mpz_cmp(aRemainder, aHalfM) > 0) // if is bigger than m / 2
	{
		// shift coeff
		aPoly->Dec(0, aM);
		aPoly->Inc(1, 1);
	}

	mpz_abs(aAux, *aPoly->Get2(0));       // get abs value
	if (mpz_cmp(aAux, aCoeffBound) >= 0)  // if is too big for coeff
	{
		return false;
	}

	return true;
}
void CPolySelectionPhase::PrintHeader()
{
	printf("\n\n");
	printf(" ________________________________________________________________________");
	printf("\n");
	printf("|        c        |          a_{d}       |   alpha |  found  |  percent  |");
	printf("\n");
}
void CPolySelectionPhase::PrintValues()
{
	printf("\r|");
	gmp_printf(" %15Zd |", this->iC);
	gmp_printf(" %20Zd |", this->iMainAd);	
	printf(" %7.3f |", this->iAlphaBound);				
	printf(" %7ld |", this->iFoundCandidatePolyCount);
	printf(" %7.3f %% |", 100 * (float) this->iFoundCandidatePolyCount / (float) this->iCandidatesCount);
	fflush(stdout);	
}
void CPolySelectionPhase::PrintEffectiveness()
{

}
int CPolySelectionPhase::PrepareRunningSerialization()
{
	this->iStats->ConfirmAll();
	this->iTimes->ConfirmAll();

	this->iRSerialFoundCandPolyCount = this->iFoundCandidatePolyCount;
	this->iRSerialRatedCandPolyCount = this->iRatedCandidatePolyCount;

	mpz_set(this->iRSerialMonicM, this->iWorkingMonicM);

	if (this->iRSerialCPrimes != NULL && this->iWorkingCPrimes != NULL)
	{
		for (int i = 0; i < this->iRSerialCPrimesCount; i++)
		{
			this->iRSerialCPrimes[i] = this->iWorkingCPrimes[i];
		}
	}

	return ConstRC::Ok;
}
/*!
	TODO: If call from more threads then could be a problem with writing candidates - same files...
*/
int CPolySelectionPhase::Serialize(const CBaseParameters &aParam) const
{
	int rc = MSerializable::Serialize(aParam);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	CBaseParameters paramb;
	if (Running())
	{
		AFactorAlgParameters * param = Get_Parameters();

		// for deserialization at new start
		param->SetPhaseParameter(ConstPN::DESERIALIZE, Utils::ConvertToString(true), Get_PhaseNbr());
		param->SetPhaseParameter(ConstPN::PHASE_FULL_FILENAME, 
			MSerializable::CreateFileNameAuto(aParam, ConstFN::POLY_PHASE), Get_PhaseNbr());

		paramb.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, Get_ParametersFullFileName()); 
		rc = param->Serialize(paramb);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	if (this->iCandidatePolynomials != NULL)
	{
		// we want serialize only assigned polynomials
		string auxStr = Utils::ConvertToString(this->iRSerialFoundCandPolyCount);
		paramb.SetParameter(ConstPN::MAX_ELEMENT_COUNT, auxStr);
		// we want serialize rating only for rated polynomials
		auxStr = Utils::ConvertToString(this->iRSerialRatedCandPolyCount);
		paramb.SetParameter(ConstPN::MAX_RATED_ELEMENT_COUNT, auxStr);
		paramb.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iCandidatePolyFullFileName);
		rc = this->iCandidatePolynomials->Serialize(paramb);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	return ConstRC::Ok; 
}
int CPolySelectionPhase::Serialize(const CBaseParameters &aParam, xmlTextWriterPtr &aWriter) const
{
	string mainTagName;	
	aParam.GetStringParameter(ConstPN::SERIALIZATION_ELEMENT_NAME, XML_SERIAL_POLY_SEL_PHASE_NAME, mainTagName);

	int rc = xmlTextWriterStartElement(aWriter, BAD_CAST mainTagName.c_str());
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","Serialize",rc,0,"Could not write the parameter tag.");
		return ConstRC::GeneralError;
	}
	rc = CXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_INNER_STATE_NAME, Get_InnerState());
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedLong(aWriter, XML_SERIAL_FOUND_CAND_POLY_NAME, this->iRSerialFoundCandPolyCount);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeSignedLong(aWriter, XML_SERIAL_RATED_CAND_POLY_NAME, this->iRSerialRatedCandPolyCount);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = CXMLHelper::SerializeMpz(aWriter, XML_SERIAL_MONIC_M_NAME, this->iRSerialMonicM);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	if (this->iRSerialCPrimes != NULL)
	{
		rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_C_PRIMES_COUNT_NAME, this->iRSerialCPrimesCount);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}

		rc = xmlTextWriterStartElement(aWriter, BAD_CAST XML_SERIAL_C_PRIMES_NAME);
		if (rc < 0) 
		{
			CLog::LogError(WHEREARG,"CPolySelectionPhase","Serialize",rc,0,"Could not write the parameter tag.");
			return ConstRC::GeneralError;
		}
		rc = CXMLHelper::NewLine(aWriter, FALSE);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}

		for (int i = 0; i < this->iRSerialCPrimesCount; i++)
		{
			rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_ELEMENT_NAME, this->iRSerialCPrimes[i]);
			if (rc != ConstRC::Ok)
			{
				return rc;
			}
		}

		rc = xmlTextWriterEndElement(aWriter); // XML_SERIAL_C_PRIMES_NAME
		if (rc < 0) 
		{
			CLog::LogError(WHEREARG,"CPolySelectionPhase","Serialize",rc,0,"Could not end the parameter tag.");
			return ConstRC::GeneralError;
		}
		rc = CXMLHelper::NewLine(aWriter,FALSE);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	CBaseParameters param;
	rc = this->iStats->Serialize(param, aWriter);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = this->iTimes->Serialize(param, aWriter);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = xmlTextWriterEndElement(aWriter); // mainTagName
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","Serialize",rc,0,"Could not end the parameter tag.");
		return ConstRC::GeneralError;
	}
	rc = CXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	return ConstRC::Ok; 
}
/*!
	Shouldn't be call from outside!!!
*/
int CPolySelectionPhase::Deserialize(const CBaseParameters &aParam)
{
	// we can deserialize only during initialization
	if (Get_InnerState() != EPSPhaseInitialization)
	{
		return ConstRC::CannotBeDone;
	}

	if (Get_InfoMode())
	{
		Utils::NormalMessage(" Deserializing Polynomial Selection Phase...");
	}

	int rc = MSerializable::Deserialize(aParam);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	if (MSerializable::ExistFile(this->iCandidatePolyFullFileName))
	{
		if (Get_InfoMode())
		{
			Utils::NormalMessage("\tDeserializing found polynomials...");
		}

		CBaseParameters param;
		param.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iCandidatePolyFullFileName);
		this->iCandidatePolynomials = new CandidateClass(this->iCandidatesCount);
		rc = this->iCandidatePolynomials->Deserialize(param);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	return ConstRC::Ok;
}
/*!
	Shouldn't be call from outside!!!
*/
int CPolySelectionPhase::Deserialize(const CBaseParameters &aParam, xmlTextReaderPtr &aReader)
{
	// we can deserialize only during initialization
	if (Get_InnerState() != EPSPhaseInitialization)
	{
		return ConstRC::CannotBeDone;
	}

	string mainTagName;	
	aParam.GetStringParameter(ConstPN::SERIALIZATION_ELEMENT_NAME, XML_SERIAL_POLY_SEL_PHASE_NAME, mainTagName);

	int rc;
	xmlChar* name = xmlTextReaderName(aReader);

	if (name == NULL)
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","Deserialize",0,0,"Deserialization main element name not found.");
		return ConstRC::GeneralError;  
	}

	if (xmlStrEqual(name,BAD_CAST mainTagName.c_str()) == 0)
	{
		CLog::LogError(WHEREARG,"CPolySelectionPhase","Deserialize",0,0,"Main element name is wrong.");
		return ConstRC::GeneralError; 
	}

	int auxInt = 0;
	int depth = xmlTextReaderDepth(aReader);
	CBaseParameters param;

	while ((rc = CXMLHelper::MoveToNextNode(aReader, depth)) == ConstRC::Ok)
	{
		name = xmlTextReaderName(aReader);
	
		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_INNER_STATE_NAME))
		{
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize inner state...", auxInt, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			Set_InnerState(auxInt);
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_C_PRIMES_COUNT_NAME))
		{
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize c primes count...", this->iRSerialCPrimesCount, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			
			this->iRSerialCPrimes = (int *)Utils::AllocationMachine((void**)&this->iRSerialCPrimes, this->iRSerialCPrimesCount * sizeof(int));
			if (this->iRSerialCPrimes == NULL)
			{
				CLog::LogError(WHEREARG,"CPolySelectionPhase","Deserialize",0,0,"Not enough memory for c primes.");
				rc = ConstRC::NotEnoughMemory;
				break;
			}

			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_C_PRIMES_NAME))
		{
			if (this->iRSerialCPrimes == NULL)
			{
				CLog::LogError(WHEREARG,"CPolySelectionPhase","Deserialize",0,0,"C primes are not allocated.");
				rc = ConstRC::GeneralError;
				break;
			}

			int index = 0;
			int elementsDepth = xmlTextReaderDepth(aReader);
			while (CXMLHelper::MoveToNextNode(aReader, elementsDepth) == ConstRC::Ok 
				&& index < this->iRSerialCPrimesCount) // element
			{
				name = xmlTextReaderName(aReader);

				if (xmlStrEqual(name, BAD_CAST XML_SERIAL_ELEMENT_NAME))
				{
					rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize c prime...", this->iRSerialCPrimes[index], FALSE);
					if (rc != ConstRC::Ok)
					{
						rc = ConstRC::GeneralError;
						break;
					}
					index++;
				}
			}

			if (rc != ConstRC::Ok)
			{
				break;
			}
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_FOUND_CAND_POLY_NAME))
		{
			rc = CXMLHelper::DeserializeSignedLong(aReader, "Deserialize found candidate poly count...", this->iFoundCandidatePolyCount, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			this->iRSerialFoundCandPolyCount = this->iFoundCandidatePolyCount;
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_RATED_CAND_POLY_NAME))
		{
			rc = CXMLHelper::DeserializeSignedLong(aReader, "Deserialize rated candidate poly count...", this->iRatedCandidatePolyCount, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			this->iRSerialRatedCandPolyCount = this->iRatedCandidatePolyCount;
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_MONIC_M_NAME))
		{
			rc = CXMLHelper::DeserializeMpz(aReader, "Deserialize monic root m...", this->iWorkingMonicM, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			mpz_set(this->iRSerialMonicM, this->iWorkingMonicM);
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_STATISTIC_CONTAINER_NAME))
		{
			rc = this->iStats->Deserialize(param, aReader);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_TIME_CONTAINER_NAME))
		{
			rc = this->iTimes->Deserialize(param, aReader);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			continue;
		}
	}

	if (rc == ConstRC::GeneralError || rc == ConstRC::NotEnoughMemory)
	{
		return rc;
	}

	return ConstRC::Ok;
}
