#ifndef __POLYNOMIAL_IMPROVEMENT_H__
#define __POLYNOMIAL_IMPROVEMENT_H__

#include "multivariable_polynomial.h"
#include "const_poly_selection_phase.h"

///Class for improvement of polynomial from poly selection phase 
class CPolynomialImprovement
{
private:
	///The small primes
	static const int	iCoefficientPrimes[ConstPSPhase::SMALL_PRIME_COUNT];
	///An array used when sieving for the best alpha
	unsigned char		jValuations[ConstPSPhase::MAX_J_PRIME][ConstPSPhase::MAX_J_PRIME];
	///An array used when sieving for the best alpha
	double				jAlphaP[ConstPSPhase::MAX_J_PRIME];
	///An array used when sieving for the best alpha
	double				jAlpha[2 * ConstPSPhase::J0 + 1];
	///The steepest descent minimum accuracy
	long				iSteepestDescentLevel;

	mp_exp_t iT;

	mpz_t iAuxMPZ;
	mpz_t iAuxMPZ2;
	mpz_t iSumMPZ2;
	mpz_t iFC[MULT_VAR_VARIABLES - 1];

	mpf_t iAuxMPF;
	mpf_t iDeriv;
	mpf_t iAux;
	mpf_t iVal;
	mpf_t iPow;
	mpf_t iSq;
	mpf_t iValP;
	mpf_t iValP1;
	mpf_t iAux1;
	mpf_t iAux2;

	mpf_t iAuxMPFMatrix[T_LOC][T_LOC + 1];
	mpf_t iGrad[MULT_VAR_VARIABLES];
	mpf_t iP[MULT_VAR_VARIABLES];
	mpf_t iP1[MULT_VAR_VARIABLES];

	mpf_t*	iIntegral;
	mpf_t*	iSPowers;
	mpf_t**	iEvaluatedPowers; 

	int iDegree;

	MultVarPolynomial*	iTr;
	MultVarPolynomial*	iRot;
	MultVarPolynomial*	iF;
	MultVarPolynomial*	iAuxF;
	MultVarPolynomial*	iInteg;

	MultVarPolynomial*	iDerivMVP[MULT_VAR_VARIABLES];

	char * iChars;

	double *	iIFSBoundBuffer; ///<this field contains best IFS bounds
	double		iIFSBound; ///<this is iBufferSize-th best IFS bound - this is used for comparison
	double *	iAlphaBoundBuffer; ///<this field contains best alphas
	double		iAlphaBound; ///<this is iBufferSize-th best alpha - this is used for comparison
	int			iBufferSize; ///<size of buffer for determination of suitable bounds for IFS and alpha



public:
	CPolynomialImprovement(int aBufferSize);
	~CPolynomialImprovement();

	int Init(int aDegree);
	int Reset();

	int Improve(Polynomial * aResultPoly, mpz_t aResultM, mpf_t aSkewness, mpz_t aM, mpz_t aP);

public:
	// getters and setters

	int Get_BufferSize() { return this->iBufferSize; }
	void Set_BufferSize(int aBufferSize) { if (this->iIFSBoundBuffer == NULL && this->iAlphaBoundBuffer == NULL) this->iBufferSize = aBufferSize; }

private:

	int DisposeGMP();

	int SieveForBestAlpha(int *aJ0, int *aJ1, double *aAlpha, Polynomial *aPolynomial, mpz_t aM, mpz_t aP);
	int EvalIntInSkewness(mpf_t aResult, mpf_t * aInt, mpf_t aSkewness, int d);
	int SkewnessKleinjung(mpf_t aResult, Polynomial *aPolynomial, mpf_t aStartingSkewness);
	int SkewPolynomial(Polynomial *aResult, mpf_t aSkewness, mpz_t aNewRoot, double *aIntegral, Polynomial *aPolynomial, mpz_t aOldRoot, mpz_t aP);
	int DoubleIntegralAcrossSkewedRegionKleinjung(mpf_t *aResult, Polynomial *aOperand);
	int DoubleIntegralAcrossSkewedRegionKleinjung(MultVarPolynomial *aResult, MultVarPolynomial *aOperand, unsigned aDegree);
	int CMinimumKleinjung(mpf_t *aResult, MultVarPolynomial *aOperand, mpf_t **aPowerTable, mpf_t *aSPowerTable, unsigned aDegree);

};

#endif // __POLYNOMIAL_IMPROVEMENT_H__
