#include "nfs_parameters.h"
#include "number_field_sieve.h"
#include "const_parameter_names.h"
# include "center.h"
# include "node.h"
#include "test_suite.h"
#include "log.h"


/*! \file 
 *  \brief main() function
 *
*/
// NFS++ version 0.8, 24.5.2008

/*!
 * \brief Just takes care of parameters, constructs and calls instance of NFS.
 * @param argc - number of arguments
 * @param argv - array of arguments
 */
int main(int argc, char *argv[])
{
	int rc;

	//CTestSuite::TestPrimes();
	//CTestSuite::Test(argc, argv);
	//CTestSuite::TestRelations();
	//CTestSuite::TestFactorBaseHelper();
	//CTestSuite::TestTimes();
	//CTestSuite::TestSieving(argc, argv);
	//CTestSuite::TestWriter();
	//CTestSuite::TestRewrite();
	//CTestSuite::TestMPF();
	//CTestSuite::TestPolyMod();
	//CTestSuite::TestCandidates();

	//return ConstRC::Ok;

	// for using libxml2 in more threads
	xmlInitParser();

	// parameters specialized for NFS
	CNFSParameters param;
	param.ParseInputParameters(argc, argv);

	// choose NFS type
	int type = DEFAULT_DISTRIBUTION_MACHINE_TYPE;
	param.GetIntegerPhaseParameter(DISTRIBUTION_MACHINE_TYPE, type, (int)EMain);

	// machine name
	string machine_name = "";
	param.GetStringPhaseParameter(MACHINE_NAME, machine_name, (int)EMain);

	// keep messages
	bool keep_messages = false;
	param.GetBoolPhaseParameter(KEEP_MESSAGES, keep_messages, (int)EMain);

	rc = ConstRC::Ok;

	switch ((distribution)type)
	{
		case ECenter:
		{
			fprintf(stdout, "\nThis machine is NFS center.\n");
			fprintf(stdout, "This machine name is \"%s\".\n", machine_name.c_str());
			if (keep_messages) fprintf(stdout, "We will keep old messages.\n\n");
			else fprintf(stdout, "We will delete old messages.\n\n");
			CNumberFieldSieve *alg = new CNumberFieldSieve();
			Center *center = new Center(*alg, ".");
			center->SetMachineName(machine_name);
			center->SetKeepMessages(keep_messages);
			// get connectors
			string infos = "";
			param.GetStringPhaseParameter(CONNECTOR_INFOS, infos, (int)EMain);
			if (center->ParseConnectors(infos.c_str(), ";") == ConstRC::Ok) {
				fprintf(stdout, "Available connectors:\n\n");
				center->PrintConnectors(stdout);
				center->Run();
			}
			else CLog::LogError(WHEREARG,"program","Main",0,0,"Bad connectors information: \""+infos+"\".");
			delete center;
			delete alg;
		}
			break;
		case ENode:
		{
			fprintf(stdout, "\nThis machine is NFS node.\n");
			fprintf(stdout, "This machine name is \"%s\".\n", machine_name.c_str());
			if (keep_messages) fprintf(stdout, "We will keep old messages.\n\n");
			else fprintf(stdout, "We will delete old messages.\n\n");
			CNumberFieldSieve *alg = new CNumberFieldSieve();
			Node *node = new Node(*alg);
			node->SetMachineName(machine_name);
			node->SetKeepMessages(keep_messages);
			// get centers
			string infos = "";
			param.GetStringPhaseParameter(CENTER_INFOS, infos, (int)EMain);
			if (node->ParseCenters(infos.c_str(), ";") == ConstRC::Ok) node->Run();
			else CLog::LogError(WHEREARG,"program","Main",0,0,"Bad centers information: \""+infos+"\".");
			delete node;
			delete alg;
		}
			break;
		case ENone:
		{
			fprintf(stdout, "\nThis machine is standalone NFS.\n\n");
			AFactorAlg *alg = new CNumberFieldSieve();
			rc = alg->RunFactorization(&param);
			delete alg;
#ifndef MEASUREMENT_ON
			// just for info
			//Utils::Wait();
#endif
		}
			break;
		default:
			CLog::LogError(WHEREARG,"program","Main",0,0,"Unknown distribution machine type.");
			rc = ConstRC::GeneralError;
			break;
	}

	CLog::CloseLog();

	xmlCleanupParser();	

	return rc;
}

