#include "rel_processing_phase.h"

#include "matrix_helper.h"
#include "const_rel_processing_phase.h"
#include "const_parameter_names.h"
#include "const_hashtables.h"
#include "lp_hashtable_type1.h"
#include "lp_hashtable_type2.h"
#include "cc_table.h"
#include <set>
#include "test_suite.h"
#include <cstring>
#include "log.h"

using namespace std;

CRelProcessingPhase::CRelProcessingPhase() : MSerializable(ConstFN::REL_PRO_PHASE)
{
	Set_AutoSerializable(ConstRPPhase::AUTO_SERIALIZABLE);

	for (int i = 0; i < CNFSRelation::DEFAULT_NFS_PARTS_COUNT; i++)
	{
		this->iSievingPoly[i] = NULL;
	}

	this->iRootsList.reserve(ConstRPPhase::ROOTS_LIST);

	Set_InnerState((int)ERPPhaseNotStarted);
}

CRelProcessingPhase::~CRelProcessingPhase()
{
	Reset();
	DisposeFunctorField();
	DisposeMutexes();
	DisposeAcceptedReturnCodes();
}
int CRelProcessingPhase::Reset()
{
	Set_RelationMatrix(NULL);

	for (int i = 0; i < CNFSRelation::DEFAULT_NFS_PARTS_COUNT; i++)
	{
		if (this->iSievingPoly[i] != NULL)
		{
			delete this->iSievingPoly[i];
			this->iSievingPoly[i] = NULL;
		}

		this->iFBs[i].Reset();
	}

	this->iRootsList.clear();

	ARelProcessingPhase::Reset();

	Set_InnerState((int)ERPPhaseNotStarted);

	return ConstRC::Ok;
}
int CRelProcessingPhase::FillFunctorField()
{
	if (Get_FunctorsInit() == false)
	{
		Set_FunctorsInit(true);
		this->iPhaseFunctors.reserve(ConstRPPhase::INNER_STATE_COUNT);
		this->iPhaseFunctors.clear();
		// ERPPhaseNotStarted
		this->iPhaseFunctors.push_back(NULL);
		// ERPPhaseInitialization
		this->iPhaseFunctors.push_back(NULL);
		// ERPPhaseRemovingLargeSingletons
		this->iPhaseFunctors.push_back(new CPhaseFunctor<CRelProcessingPhase>(this, &CRelProcessingPhase::RemoveLargeSingletons));
		// ERPPhaseConstructingCycles
		this->iPhaseFunctors.push_back(new CPhaseFunctor<CRelProcessingPhase>(this, &CRelProcessingPhase::ConstructCycles));
		// ERPPhaseRemovingSmallSingletons
		this->iPhaseFunctors.push_back(new CPhaseFunctor<CRelProcessingPhase>(this, &CRelProcessingPhase::RemoveSmallSingletons));
		// ERPPhaseCreatingRelMatrix
		this->iPhaseFunctors.push_back(new CPhaseFunctor<CRelProcessingPhase>(this, &CRelProcessingPhase::CreateRelationMatrix));
		// ERPPhaseSavingResult
		this->iPhaseFunctors.push_back(new CPhaseFunctor<CRelProcessingPhase>(this, &CRelProcessingPhase::SaveResult));
		// ERPPhaseFinished
		this->iPhaseFunctors.push_back(NULL);
	}

	return ConstRC::Ok;
}

int CRelProcessingPhase::CleanUp(AFactorAlgParameters* aParameters, int aPhaseNBR, cleaning_types aType, bool aError)
{
	if (aParameters == NULL)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","CleanUp",0,0,"Parameters are NULL.");
		return ConstRC::BadArgument;
	}

	int phaseNBR;
	if (aParameters->GetIntegerPhaseParameter(ConstPN::REL_PROCESSING_PHASE_NUMBER, phaseNBR, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","CleanUp",0,0,"Didn't find phase number.");
    	return ConstRC::GeneralError;
	}

	string identifier;
	string directory = "";
	string file;
	string auxStr;

	if (aParameters->GetStringPhaseParameter(ConstPN::SERIALIZATION_IDENTIFIER, identifier, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","CleanUp",0,0,"Didn't find serial identifier.");
		return ConstRC::GeneralError;
	}
	aParameters->GetStringPhaseParameter(ConstPN::SERIALIZATION_DIRECTORY, directory, (int)EMain);

	switch (aType)
	{
		case ECleanAllIfNotError:
			if (aError)
			{
				break;
			}
		case ECleanAll:
		case ECleanAllExcCF:
		case ECleanAllExcCFRF:
			// All partial relation full filename
			if (aParameters->ContainsPhaseParameter(ConstPN::ALL_PARTIAL_REL_FULL_FILENAME, phaseNBR))
			{
				aParameters->GetStringPhaseParameter(ConstPN::ALL_PARTIAL_REL_FULL_FILENAME, file, phaseNBR);
				MSerializable::AdaptPathSeparator(file);
			}
			else
			{
				aParameters->CreateFullFileName(file, ConstPN::ALL_PARTIAL_REL_FILENAME, 
					ConstPN::ALL_PARTIAL_REL_DIRECTORY, ConstPN::ALL_PARTIAL_REL_COMPRESSION, directory,
					ConstFN::ALL_PARTIAL_REL, identifier, phaseNBR, TXT_FILE_EXTENSION);
			}
			remove(file.c_str());

			// Temp file full filename
			if (aParameters->ContainsPhaseParameter(ConstPN::TEMP_FILE_FULL_FILENAME, phaseNBR))
			{
				aParameters->GetStringPhaseParameter(ConstPN::TEMP_FILE_FULL_FILENAME, file, phaseNBR);
				MSerializable::AdaptPathSeparator(file);
			}
			else
			{
				aParameters->CreateFullFileName(file, ConstPN::TEMP_FILE_FILENAME, 
					ConstPN::TEMP_FILE_DIRECTORY, ConstPN::TEMP_FILE_COMPRESSION, directory,
					ConstFN::TEMP_FILE, identifier, phaseNBR, TXT_FILE_EXTENSION);
			}
			remove(file.c_str());

			// Usable partial relation full filename
			if (aParameters->ContainsPhaseParameter(ConstPN::USABLE_PART_REL_FULL_FILENAME, phaseNBR))
			{
				aParameters->GetStringPhaseParameter(ConstPN::USABLE_PART_REL_FULL_FILENAME, file, phaseNBR);
				MSerializable::AdaptPathSeparator(file);
			}
			else
			{
				aParameters->CreateFullFileName(file, ConstPN::USABLE_PART_REL_FILENAME, 
					ConstPN::USABLE_PART_REL_DIRECTORY, ConstPN::USABLE_PART_REL_COMPRESSION, directory,
					ConstFN::USABLE_PART_REL, identifier, phaseNBR, TXT_FILE_EXTENSION);
			}
			remove(file.c_str());

			// All smooth relation full filename
			if (aParameters->ContainsPhaseParameter(ConstPN::ALL_SMOOTH_REL_FULL_FILENAME, phaseNBR))
			{
				aParameters->GetStringPhaseParameter(ConstPN::ALL_SMOOTH_REL_FULL_FILENAME, file, phaseNBR);
				MSerializable::AdaptPathSeparator(file);
			}
			else
			{
				aParameters->CreateFullFileName(file, ConstPN::ALL_SMOOTH_REL_FILENAME, 
					ConstPN::ALL_SMOOTH_REL_DIRECTORY, ConstPN::ALL_SMOOTH_REL_COMPRESSION, directory,
					ConstFN::ALL_SMOOTH_REL, identifier, phaseNBR, TXT_FILE_EXTENSION);
			}
			remove(file.c_str());

			// Quadratic characters full filenames
			for (int i = 0; i < CNFSRelation::DEFAULT_NFS_PARTS_COUNT; i++)
			{
				auxStr = Utils::ConvertToString(i + 1);
				if (aParameters->ContainsPhaseParameter(ConstPN::QUAD_CHARS_FULL_FILENAME + auxStr, phaseNBR))
				{
					aParameters->GetStringPhaseParameter(ConstPN::QUAD_CHARS_FULL_FILENAME + auxStr, file, phaseNBR);
					MSerializable::AdaptPathSeparator(file);
				}
				else
				{
					aParameters->CreateFullFileName(file, ConstPN::QUAD_CHARS_FILENAME + auxStr, 
						ConstPN::QUAD_CHARS_DIRECTORY + auxStr, ConstPN::QUAD_CHARS_COMPRESSION + auxStr, directory,
						ConstFN::QUAD_CHARS + auxStr, identifier, phaseNBR);
				}
				remove(file.c_str());
			}
			break;
		default:
			break;
	}

	return ARelProcessingPhase::CleanUp(aParameters, phaseNBR, aType, aError);
}
int CRelProcessingPhase::InitParameters()
{
	Set_InnerState((int)ERPPhaseInitialization);

	AFactorAlgParameters * param = Get_Parameters();
	if (param == NULL)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","InitParameters",0,0,"Parameters aren't set.");
    	return ConstRC::GeneralError;
	}

	// generally we don't know order of the square root phase
	int auxInt;
	if (param->GetIntegerPhaseParameter(ConstPN::REL_PROCESSING_PHASE_NUMBER, auxInt, (int)EMain) != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","InitParameters",0,0,"Didn't find phase number.");
    	return ConstRC::GeneralError;
	}
	Set_PhaseNbr(auxInt);

	int rc = ARelProcessingPhase::InitParameters();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	// we support only relations with 2 parts
	if (ConstSPhase::ELEMENT_MASK_COUNT != CNFSRelation::DEFAULT_NFS_PARTS_COUNT)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","InitParameters",0,0,"This phase support only relations with two parts.");
		return ConstRC::BadArgument;
	}
	
	int i;
	string auxStr;
	for (i = 0; i < CNFSRelation::DEFAULT_NFS_PARTS_COUNT; i++)
	{
		auxStr = Utils::ConvertToString(i + 1);
		rc = param->GetPolynomialPhaseParameter(ConstPN::SIEVING_POLY + auxStr, this->iSievingPoly[i], (int)EMain);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"CRelProcessingPhase","InitParameters",0,0,"Didn't find the sieving polynomial.");
			return rc;
		}
	}
	rc = param->GetLongPhaseParameter(ConstPN::SMOOTH_REL_COUNT, this->iSmoothRelationsCount, Get_PhaseNbr());
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","InitParameters",0,0,"Didn't find the smooth relation count.");
		return rc;
	}
	rc = param->GetIntegerPhaseParameter(ConstPN::PARTIAL_REL_GRAPH_VERTICES, this->iVertices, Get_PhaseNbr());
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","InitParameters",0,0,"Didn't find the graph vertices.");
		return rc;
	}
	rc = param->GetIntegerPhaseParameter(ConstPN::PARTIAL_REL_GRAPH_COMPONENTS, this->iComponents, Get_PhaseNbr());
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","InitParameters",0,0,"Didn't find the graph components.");
		return rc;
	}
	rc = param->GetIntegerPhaseParameter(ConstPN::PARTIAL_REL_GRAPH_EDGES, this->iEdges, Get_PhaseNbr());
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","InitParameters",0,0,"Didn't find the graph edges.");
		return rc;
	}

	// now we create all filenames which we need - some of then should be in parameters, 
	// others names have to be assembled
	for (i = 0; i < CNFSRelation::DEFAULT_NFS_PARTS_COUNT; i++)
	{
		auxStr = Utils::ConvertToString(i + 1);
		// Quadratic characters full filename
		if (param->ContainsPhaseParameter(ConstPN::QUAD_CHARS_FULL_FILENAME + auxStr, Get_PhaseNbr()))
		{
			param->GetStringPhaseParameter(ConstPN::QUAD_CHARS_FULL_FILENAME + auxStr, this->iQuadCharsFullFileName[i], Get_PhaseNbr());
			MSerializable::AdaptPathSeparator(this->iQuadCharsFullFileName[i]);
		}
		else
		{
			param->CreateFullFileName(this->iQuadCharsFullFileName[i], ConstPN::QUAD_CHARS_FILENAME + auxStr, 
				ConstPN::QUAD_CHARS_DIRECTORY + auxStr, ConstPN::QUAD_CHARS_COMPRESSION + auxStr, Get_SerialMainDirectory(),
				ConstFN::QUAD_CHARS + auxStr, Get_SerialIdentifier(), Get_PhaseNbr());
		}
		// FB full filename
		if (param->ContainsPhaseParameter(ConstPN::FACTOR_BASE_FULL_FILENAME + auxStr, Get_PhaseNbr()))
		{
			param->GetStringPhaseParameter(ConstPN::FACTOR_BASE_FULL_FILENAME + auxStr, this->iFBs[i].FBFullFileName, Get_PhaseNbr());
			MSerializable::AdaptPathSeparator(this->iFBs[i].FBFullFileName);
		}
		else
		{
			param->CreateFullFileName(this->iFBs[i].FBFullFileName, ConstPN::FACTOR_BASE_FILENAME + auxStr, 
				ConstPN::FACTOR_BASE_DIRECTORY + auxStr, ConstPN::FACTOR_BASE_COMPRESSION + auxStr, Get_SerialMainDirectory(),
				ConstFN::FACTOR_BASE + auxStr, Get_SerialIdentifier(), Get_PhaseNbr());
		}
	}
	// All partial relation full filename
	if (param->ContainsPhaseParameter(ConstPN::ALL_PARTIAL_REL_FULL_FILENAME, Get_PhaseNbr()))
	{
		param->GetStringPhaseParameter(ConstPN::ALL_PARTIAL_REL_FULL_FILENAME, this->iAllPartialRelFullFileName, Get_PhaseNbr());
		MSerializable::AdaptPathSeparator(this->iAllPartialRelFullFileName);
	}
	else
	{
		param->CreateFullFileName(this->iAllPartialRelFullFileName, ConstPN::ALL_PARTIAL_REL_FILENAME, 
			ConstPN::ALL_PARTIAL_REL_DIRECTORY, ConstPN::ALL_PARTIAL_REL_COMPRESSION, Get_SerialMainDirectory(),
			ConstFN::ALL_PARTIAL_REL, Get_SerialIdentifier(), Get_PhaseNbr(), TXT_FILE_EXTENSION);
	}
	// Partial relation full filename
	if (param->ContainsPhaseParameter(ConstPN::PARTIAL_REL_FULL_FILENAME, Get_PhaseNbr()))
	{
		param->GetStringPhaseParameter(ConstPN::PARTIAL_REL_FULL_FILENAME, this->iPartialRelFullFileName, Get_PhaseNbr());
		MSerializable::AdaptPathSeparator(this->iPartialRelFullFileName);
	}
	else
	{
		param->CreateFullFileName(this->iPartialRelFullFileName, ConstPN::PARTIAL_REL_FILENAME, 
			ConstPN::PARTIAL_REL_DIRECTORY, ConstPN::PARTIAL_REL_COMPRESSION, Get_SerialMainDirectory(),
			ConstFN::PARTIAL_REL, Get_SerialIdentifier(), Get_PhaseNbr(), TXT_FILE_EXTENSION);
	}
	// Temp file full filename
	if (param->ContainsPhaseParameter(ConstPN::TEMP_FILE_FULL_FILENAME, Get_PhaseNbr()))
	{
		param->GetStringPhaseParameter(ConstPN::TEMP_FILE_FULL_FILENAME, this->iTempFileFullFileName, Get_PhaseNbr());
		MSerializable::AdaptPathSeparator(this->iTempFileFullFileName);
	}
	else
	{
		param->CreateFullFileName(this->iTempFileFullFileName, ConstPN::TEMP_FILE_FILENAME, 
			ConstPN::TEMP_FILE_DIRECTORY, ConstPN::TEMP_FILE_COMPRESSION, Get_SerialMainDirectory(),
			ConstFN::TEMP_FILE, Get_SerialIdentifier(), Get_PhaseNbr(), TXT_FILE_EXTENSION);
	}
	// Usable partial relation full filename
	if (param->ContainsPhaseParameter(ConstPN::USABLE_PART_REL_FULL_FILENAME, Get_PhaseNbr()))
	{
		param->GetStringPhaseParameter(ConstPN::USABLE_PART_REL_FULL_FILENAME, this->iUsablePartialRelFullFileName, Get_PhaseNbr());
		MSerializable::AdaptPathSeparator(this->iUsablePartialRelFullFileName);
	}
	else
	{
		param->CreateFullFileName(this->iUsablePartialRelFullFileName, ConstPN::USABLE_PART_REL_FILENAME, 
			ConstPN::USABLE_PART_REL_DIRECTORY, ConstPN::USABLE_PART_REL_COMPRESSION, Get_SerialMainDirectory(),
			ConstFN::USABLE_PART_REL, Get_SerialIdentifier(), Get_PhaseNbr(), TXT_FILE_EXTENSION);
	}
	// Smooth relation full filename
	if (param->ContainsPhaseParameter(ConstPN::SMOOTH_RELATIONS_FULL_FILENAME, Get_PhaseNbr()))
	{
		param->GetStringPhaseParameter(ConstPN::SMOOTH_RELATIONS_FULL_FILENAME, this->iSmoothRelFullFileName, Get_PhaseNbr());
		MSerializable::AdaptPathSeparator(this->iSmoothRelFullFileName);
	}
	else
	{
		param->CreateFullFileName(this->iSmoothRelFullFileName, ConstPN::SMOOTH_RELATIONS_FILENAME, 
			ConstPN::SMOOTH_RELATIONS_DIRECTORY, ConstPN::SMOOTH_RELATIONS_COMPRESSION, Get_SerialMainDirectory(),
			ConstFN::SMOOTH_RELATIONS, Get_SerialIdentifier(), Get_PhaseNbr(), TXT_FILE_EXTENSION);
	}
	// All smooth relation full filename
	if (param->ContainsPhaseParameter(ConstPN::ALL_SMOOTH_REL_FULL_FILENAME, Get_PhaseNbr()))
	{
		param->GetStringPhaseParameter(ConstPN::ALL_SMOOTH_REL_FULL_FILENAME, this->iAllSmoothRelFullFileName, Get_PhaseNbr());
		MSerializable::AdaptPathSeparator(this->iAllSmoothRelFullFileName);
	}
	else
	{
		param->CreateFullFileName(this->iAllSmoothRelFullFileName, ConstPN::ALL_SMOOTH_REL_FILENAME, 
			ConstPN::ALL_SMOOTH_REL_DIRECTORY, ConstPN::ALL_SMOOTH_REL_COMPRESSION, Get_SerialMainDirectory(),
			ConstFN::ALL_SMOOTH_REL, Get_SerialIdentifier(), Get_PhaseNbr(), TXT_FILE_EXTENSION);
	}

	// should we deserialize or serialize?
	bool deserialize = false;
	param->GetBoolPhaseParameter(ConstPN::DESERIALIZE, false, deserialize, Get_PhaseNbr());
	CBaseParameters bparam;
	bparam.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, Get_SerialPhaseFullFileName());

	if (Get_InfoMode())
	{
		Utils::NormalMessage("Relation Processing Phase is running ...");
    }

	if (deserialize)
	{
		rc = Deserialize(bparam);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}
	else
	{
		rc = Serialize(bparam);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	return ConstRC::Ok;
}
int CRelProcessingPhase::SaveResult()
{
	Set_InnerState((int)ERPPhaseSavingResult);

	if (Get_InfoMode())
	{
		Utils::NormalMessage("   Writing relation matrix ...");
    }

	AbstractMatrix* matrix = Get_RelationMatrix();
	if (matrix == NULL)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","SaveResult",0,0,"Relation matrix doesn't exist.");
		return ConstRC::GeneralError;
	}

	CBaseParameters bparam;
	bparam.AddParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iRelationMatrixFullFileName);

	int rc = matrix->Serialize(bparam);
	if (rc == ConstRC::Ok)
	{
		AFactorAlgParameters * param = Get_Parameters();
		// serialization is ConstRC::Ok so we save file name for other phases
		param->SetParameterForOthers(ConstPN::RELATION_MATRIX_FULL_FILENAME, this->iRelationMatrixFullFileName, Get_PhaseNbr());
		string aux = Utils::ConvertToString((int)this->iRelationMatrixType);
		param->SetParameterForOthers(ConstPN::RELATION_MATRIX_TYPE, aux, Get_PhaseNbr());
		param->SetParameterForOthers(ConstPN::RELATIONS_RESULT_FULL_FILENAME, this->iRelationsResultFullFileName, Get_PhaseNbr());

		Set_InnerState((int)ERPPhaseFinished);
	}

	return rc;
}
int CRelProcessingPhase::RunPhase(AFactorAlgParameters *aParameters)
{
	return APhase::RunPhase(aParameters);
}
/*!
	This process is iterative - removal of the first batch of singletons will almost
	always create a new one.
	Empirically, the number of iterations, and the number of singletons removed, are
	strikingly similar to the Fibonacci sequence, but the exact reason for such 
	a similarity has not yet been found.

	The methods allocates a large chunk of memory for the sake of singleton detection;
	this allocation may fail in case of a large number of relations (several million).
*/
int CRelProcessingPhase::RemoveLargeSingletons()
{
	Set_InnerState((int)ERPPhaseRemovingLargeSingletons);

	if (this->iVariationsInfo[0].Type == EDoNotUseVariations && this->iVariationsInfo[1].Type == EDoNotUseVariations)
	{
		// we don't need use this method
		return ConstRC::Ok;
	}

	if (MSerializable::ExistFile(this->iPartialRelFullFileName) == false)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","RemoveLargeSingletons",0,0,"Partial relation file doesn't exist.");
		return ConstRC::GeneralError;
	}

	// we allocate a temporary array of bytes, with a single entry per each found x-partial relation.	
	// when we found singleton primeideal, we indicate its relation as singleton in this array,
	// so when we iterate throught partial relations we can quickly decide if it has singleton primeideal.
	detection_type * detectionField = NULL;
	
	//	int maxDetectionField = this->iEdges - 1;
	//	maxDetectionField is indexed by Hashtable indexes, so it needs to be of at least same size
	//	TODO: exact bound computation
	int maxDetectionField = (unsigned ((this->iEdges)/ConstHT::MAX_FULLNESS))<<3;

	detectionField = (detection_type*) Utils::AllocationMachine((void**) &detectionField,(maxDetectionField + 1) * sizeof(detection_type));
	if (detectionField == NULL) 
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","RemoveLargeSingletons",this->iEdges,(maxDetectionField+1)*sizeof(detection_type),"Not enough memory to allocate data space.");
		return ConstRC::NotEnoughMemory;
	}
	memset(detectionField, ConstHT::NONSINGLETON, (maxDetectionField + 1) * sizeof(detection_type));
	
	int rc;

	this->iIndependentCycles = this->iEdges + this->iComponents - this->iVertices;
	this->iExpectedVertices = this->iVertices;
	int currentVertices = this->iVertices;
	int currentEdges    = this->iEdges;
	int currentlyRemovedVertices = 0;
	int currentlyRemovedEdges    = 0;
	int iter = 0;
	int total = 0;
	int totalEdges = 0;
	if (Get_InfoMode()) 
	{
		Utils::NormalMessage("\tDeleting singletons from x-partials ... ");
	}

	rc = MSerializable::MCopyFile(this->iPartialRelFullFileName, this->iAllPartialRelFullFileName);
	if (rc != ConstRC::Ok)
	{
		free(detectionField);
		detectionField = NULL;
		return rc;
	}
	rc = MSerializable::MCopyFile(this->iSmoothRelFullFileName, this->iAllSmoothRelFullFileName);
	if (rc != ConstRC::Ok)
	{
		free(detectionField);
		detectionField = NULL;
		return rc;
	}

	// Allocate hashtable of size suitable for the remaining large primes
	CLPType1Hashtable lpTable;
	rc = lpTable.SetupHashtable(currentVertices);
	if (rc != ConstRC::Ok)
	{
		free(detectionField);
		detectionField = NULL;
		return rc;
	}

	// we support only relations with 2 parts!!!
	CNFSRelation candidate(ETrippleLargePrimeV, ConstSPhase::ELEMENT_MASK_COUNT);

	FILE * partialsFILE = NULL;
	FILE * tempFILE = NULL;

	int i;

	// This cycle will be broken if the number of singletons removed drops to 0.
	// Such situation must always occur, for the number of useful relations always
	// drops when singletons are found - and the natural limit of this number is 0.
	// Of course, 0 singletons can be removed from 0 relations, therefore ending
	// the cycle.
	while (true) 
	{
		currentlyRemovedVertices = 0;
		currentlyRemovedEdges    = 0;	
		rc = lpTable.ClearHashtable();
		if (rc != ConstRC::Ok)
		{
			free(detectionField);
			detectionField = NULL;
			return rc;
		}

		// Now, we read the relations from file "partials" one-by-one and put corresponding
		// values into the hashtable
		partialsFILE = fopen(this->iPartialRelFullFileName.c_str(), FILE_MODE_READ);
		if (partialsFILE == NULL) 
		{
			CLog::LogError(WHEREARG,"CRelProcessingPhase","RemoveLargeSingletons",0,0,"Could not open file with partial relations.");
			free(detectionField);
			detectionField = NULL;
			return ConstRC::CouldNotOpenFile;
		}

		while (candidate.ReadFromFile(partialsFILE) == ConstRC::Ok) 
		{
			// We will check the presence of lprimes in the hashtable
			for (i = 0; i < CNFSRelation::DEFAULT_NFS_PARTS_COUNT; i++)
			{
				for (int j = 0; j <= candidate.Get_Parts(i)->Large.MaxElementIndex; j++)
				{
					rc = lpTable.AddToTable(candidate.Get_Parts(i)->Large.Elements[j].prime, candidate.Get_Parts(i)->Large.Elements[j].root, ConstSPhase::ELEMENT_MASK[i], candidate.Index);
					if (rc != ConstRC::Ok)
					{
						CLog::LogError(WHEREARG,"CRelProcessingPhase","RemoveLargeSingletons",0,0,"Add to table failed.");
						free(detectionField);
						detectionField = NULL;
						fclose(partialsFILE);
						partialsFILE = NULL;
						return rc;
					}
				}
			}
		}

		fclose(partialsFILE);
		partialsFILE = NULL;

		rc = lpTable.FindSingletonRelations(detectionField, maxDetectionField, currentlyRemovedVertices);
		if (rc != ConstRC::Ok)
		{
			free(detectionField);
			detectionField = NULL;
			return rc;
		}

		//printf("maxDetectionField == %d.\n", maxDetectionField);
		for (int i = 0; i <= maxDetectionField; i++) 
		{
			if (detectionField[i] == ConstHT::FRESH_SINGLETON) 
			{
				currentlyRemovedEdges++;
				detectionField[i] = ConstHT::SINGLETON;
			}
		}

		//int nonsingletons = 0;
		//int singletons = 0;
		//int other = 0;
		//for (int i = 0; i <= maxDetectionField; i++) 
		//{
		//	if (detectionField[i] == ConstHT::NONSINGLETON) 
		//	{
		//		nonsingletons++;
		//	} 
		//	else if (detectionField[i] == ConstHT::SINGLETON) 
		//	{
		//		singletons++;
		//	} 
		//	else 
		//	{
		//		other++;
		//	}
		//}

		//printf(" SUMMARY\n");
		//printf("Nonsingletons: %d\n",nonsingletons);
		//printf("Singletons: %d\n",singletons);
		//printf("Others??: %d\n",other);

		// Now, we open the file "relations" for reading and copy only those relations
		// which are not marked as singletons
		//int read = 0;
		//int written = 0;

		partialsFILE = fopen(this->iPartialRelFullFileName.c_str(), FILE_MODE_READ);
		if (partialsFILE == NULL) 
		{
			CLog::LogError(WHEREARG,"CRelProcessingPhase","RemoveLargeSingletons",0,0,"Could not open file with partial relations.");
			free(detectionField);
			detectionField = NULL;
			return ConstRC::CouldNotOpenFile;
		}
		tempFILE = fopen(this->iTempFileFullFileName.c_str(), FILE_MODE_WRITE);
		if (tempFILE == NULL) 
		{
			CLog::LogError(WHEREARG,"CRelProcessingPhase","RemoveLargeSingletons",0,0,"Could not open temp file.");
			free(detectionField);
			detectionField = NULL;
			fclose(partialsFILE);
			partialsFILE = NULL;
			return ConstRC::CouldNotOpenFile;
		}

		while (candidate.ReadFromFile(partialsFILE) == ConstRC::Ok) 
		{
			//read++;
			//printf("     Nactena relace s indexem %d.\n",candidate.Index);
			if (detectionField[candidate.Index] == ConstHT::NONSINGLETON) 
			{
				//printf("  !!!!!!Relace s indexem %d zapsana jako nesingleton.\n",candidate.Index);
				rc = candidate.PrintToFile(tempFILE);
				if (rc != ConstRC::Ok)
				{
					free(detectionField);
					detectionField = NULL;
					fclose(partialsFILE);
					partialsFILE = NULL;
					fclose(tempFILE);
					tempFILE = NULL;
					return rc;
				}
				//written++;
			}
		}

		fclose(partialsFILE);
		partialsFILE = NULL;
		fclose(tempFILE);
		tempFILE = NULL;

		rc = MSerializable::RenameFile(this->iTempFileFullFileName,this->iPartialRelFullFileName);
		if (rc != ConstRC::Ok)
		{
			free(detectionField);
			detectionField = NULL;
			return rc;
		}

		if (Get_InfoMode()) 
		{
			printf("\r\t\t| %8d V | %8d E | \n", currentlyRemovedVertices, currentlyRemovedEdges);	
			fflush(stdout);		
		}
		total += currentlyRemovedVertices;
		totalEdges += currentlyRemovedEdges;
		currentVertices -= currentlyRemovedVertices;
		currentEdges    -= currentlyRemovedEdges;		
		iter++;
		
		if (!(currentlyRemovedVertices && currentlyRemovedEdges)) 
		{
			break; 
		}
	}

	if (Get_InfoMode())
	{
		printf("\r\t\t%d V and %d E removed in %d iterations)\n", total, totalEdges, iter);
		printf("\t\tCurrent count: %d V and %d E\n", currentVertices, currentEdges);

	}
	this->iExpectedVertices -= total;
	
	// The graph of remaining partial relations now consists of cycles only (no
	// leaves). However, it may have more than 1 component of connectivity.
	// We must create the list of roots of all components etc.
	
	// This is not done if all the cycles have already been constructed, generated and 	appended
	// into the file of smooth relations.
	if (Get_InfoMode())
	{
		printf("\n\t\tCalling count roots ... independent cycles %d.", this->iIndependentCycles);
	}
	rc = lpTable.CountRoots(this->iRootsList);	

	free(detectionField);
	detectionField = NULL;

	return rc;
}
/*!
 	This method constructs the independent cycles in the graph of partial relations.

	It assumes that:

	- large singletons have been already removed from the graph
	- an exact number of the independent cycles is known
	
	It will always attempt to construct the exact number of the independent cycles
	that is known to the system. If it runs out of available partial values, it
	stops and complains about this fact; this should not happen and it indicates
 	an error in the previous run of NFS, probably a result of some subtle software
 	bug.

	The resulting cycles, which are effectively smooth relations, are appended to
 	the end of the smooths file. From now on, they are treated as regular smooths.

	The method can print out the sizes of the cycles. These values can be interested
	for anyone studying the properties of the graph of partial relations.

	The method also performs "RemoveDuplicitePaths" action, which cleans the cycles
	of pathological paths. This was a strange, hard-to-detect bug appearing only
	in some rare instances of NFS. Please consult the comment at this method for details.

	The algorithm for construction of the cycles has been taken from Riele, Lenstra's 
	article on using 2-partial relations in MPQS, with some adjustions to the
	NFS.

*/
int CRelProcessingPhase::ConstructCycles()
{
	Set_InnerState((int)ERPPhaseConstructingCycles);

	if (this->iVariationsInfo[0].Type == EDoNotUseVariations && this->iVariationsInfo[1].Type == EDoNotUseVariations)
	{
		// we don't need use this method
		return ConstRC::Ok;
	}

	if (Get_InfoMode()) 
	{
		Utils::NormalMessage("\tConstructCycles: ");
	}

	int rc = ConstRC::Ok;

	// We create new hashtable in order to insert values again.
	CLPType2Hashtable lpTable;
	rc = lpTable.SetupHashtable(this->iExpectedVertices);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	long index[ConstSPhase::ELEMENT_MASK_COUNT];
	bool found[ConstSPhase::ELEMENT_MASK_COUNT];
	bool depth[ConstSPhase::ELEMENT_MASK_COUNT];
	main_sieving_type searched[ConstSPhase::ELEMENT_MASK_COUNT];
	main_sieving_type root_fingerprint[ConstSPhase::ELEMENT_MASK_COUNT];
	int used = 0;
	int deferred = 0;
	// int ancestor;
	int lastIterConstructed = 0;
	CNFS_HashEntry2* last_entry = NULL;
	bool print = false;
	unsigned long relation;

	// We put all roots found before into the hashtable
	int max = this->iRootsList.size();
	int i;
	long j;
	for (i = 0; i < max; i++) 
	{
		bool enlarged;
		rc = lpTable.FindPrimeInHashtable(this->iRootsList[i].GetValue(), this->iRootsList[i].GetRootFingerprint(), print, j, enlarged);	
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
		lpTable.Hashtable[j].SetValue(this->iRootsList[i].GetValue());
		lpTable.Hashtable[j].SetRootFingerprint(this->iRootsList[i].GetRootFingerprint());
		lpTable.Hashtable[j].SetAncestor(j);
		lpTable.Hashtable[j].SetRelation(ConstHT::ROOT_FLAG);
		lpTable.Added();
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}
	print = false; // DEBUG

	CCCTable cycleTable;
	rc = cycleTable.SetupTable(this->iIndependentCycles);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	rc = MSerializable::MCopyFile(this->iPartialRelFullFileName, this->iUsablePartialRelFullFileName);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	// Now, the fundamental iterative part begins. 
	// At each pass, entries are read from file and possibly added to the graph	
	unsigned int iteration = 1; // we consider putting roots into hashtable as the iteration #0
	//printf("Independent cycles: %d.\n",this->iIndependentCycles);

	// we support only relations with 2 parts!!!
	CNFSRelation candidate(ETrippleLargePrimeV, ConstSPhase::ELEMENT_MASK_COUNT);

	FILE * partialsFILE = NULL;
	FILE * tempFILE = NULL;

	int counter = 0;
	int now_read = 0;
	int constructedCycles = 0;	
	while (constructedCycles != this->iIndependentCycles) 
	{
		// If the cycles have been already constructed (reloaded instance), we do not repeat the construction.		
		
		if (Get_InfoMode())
		{
			printf("\r\t\tConstructing cycles: iter %d, so far constructed cycles %d ...", iteration, constructedCycles); 
			fflush(stdout);
		}

		used = 0;
		deferred = 0;
		lastIterConstructed = 0;
		counter = 0;
		now_read = 0;

		partialsFILE = fopen(this->iPartialRelFullFileName.c_str(), FILE_MODE_READ);
		if (partialsFILE == NULL) 
		{
			CLog::LogError(WHEREARG,"CRelProcessingPhase","ConstructCycles",0,0,"Could not open file with partial relations.");
			return ConstRC::CouldNotOpenFile;
		}
		tempFILE = fopen(this->iTempFileFullFileName.c_str(), FILE_MODE_WRITE);
		if (tempFILE == NULL) 
		{
			CLog::LogError(WHEREARG,"CRelProcessingPhase","ConstructCycles",0,0,"Could not open temp file.");
			fclose(partialsFILE);
			return ConstRC::CouldNotOpenFile;
		}

		while (candidate.ReadFromFile(partialsFILE) == ConstRC::Ok) 
		{
			counter++;
			now_read++;
			if (print)
			{
				printf("\r%8d ",now_read); fflush(stdout);
			}
			// we read each relation, put the corresponding primes into the hashtable, etc.
			// We will check whether any of the corresponding primes is in the hashtable
			// at previous depth			
			bool NoneEnlarged = true;
			for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
			{
				bool enlarged;
				depth[i] = false;
				if (candidate.Get_Parts(i)->Large.MaxElementIndex >= 0) 
				{
					searched[i] = candidate.Get_Parts(i)->Large.Elements[0].prime;
					root_fingerprint[i] = candidate.Get_Parts(i)->Large.Elements[0].root;
					root_fingerprint[i] |= ConstSPhase::ELEMENT_MASK[i];
				} 
				else 
				{
					searched[i] = 1;
					root_fingerprint[i] = 0;
				}

				if (print) 
				{
					printf("Calling FPH %d: %d...", i, searched[i]); fflush(stdout);
				}

				rc = lpTable.FindPrimeInHashtable(searched[i], root_fingerprint[i], found[i], index[i], enlarged);
				if (enlarged) NoneEnlarged = false;
				if (rc != ConstRC::Ok)
				{
					fclose(partialsFILE);
					partialsFILE = NULL;
					fclose(tempFILE);
					tempFILE = NULL;
					return rc;
				}

				if (print) 
				{
					printf("%d.\n", index[i]);  fflush(stdout);
				}

				if (found[i]) 
				{
					if ((lpTable.Hashtable[index[i]].GetRelation() & (ConstHT::PARITY_MASK_NFS)) == (((iteration-1)%2) << 31)) 
					{
						depth[i] = true;
					}  
				}
			}
			// During previous computation the hashtable has been
			// enlarged. We need to renew index[i] while
			// preserving found[i]
			if (! NoneEnlarged) {
				for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++) {
					bool temp_found;
					bool enlarged;
					rc = lpTable.FindPrimeInHashtable(searched[i], root_fingerprint[i], temp_found, index[i], enlarged);
				};
			};

			// If the currently read relation has neither vertex in the current graph at suitable
			// depth, we defer that edge
			if (((!found[0]) && (!found[1])) || ((!depth[0]) && (!depth[1]))) 
			{
				// either both are out          
				// or some of them is in, but one or both at wrong depth
			   	if (print) 
				{
					printf("Deferred edge\n"); fflush(stdout);
				}
				
				rc = candidate.PrintToFile(tempFILE);
				if (rc != ConstRC::Ok)
				{
					fclose(partialsFILE);
					partialsFILE = NULL;
					fclose(tempFILE);
					tempFILE = NULL;
					return rc;
				}

				deferred++;
				// go to next edge
				continue;
			}

			if (print) 
			{
				printf("Used edge\n"); fflush(stdout);
			}

			used++;
			// precisely one of the values is present in the table
			if (found[0] ^ found[1]) 
			{
				for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++) 
				{
					if (found[i]) 
					{		
						j = ConstSPhase::ELEMENT_MASK_COUNT - i - 1;
						if (print) 
						{
							printf("Adding edge to table %d (#%ld).\n", i, candidate.Index); fflush(stdout);
						}
						relation = candidate.Index;
						if (iteration % 2) 
						{
							relation |= ConstHT::PARITY_MASK_NFS;
						}
						lpTable.Hashtable[index[j]].SetValue(searched[j]);
						lpTable.Hashtable[index[j]].SetRootFingerprint(root_fingerprint[j]);
						lpTable.Hashtable[index[j]].SetAncestor(index[i]);
						lpTable.Hashtable[index[j]].SetRelation(relation);
						lpTable.Added();
					}
				}
			} 
			else 
			{
				// we have found a cycle
				// This is the edge which finishes the cycle
				cycleTable.PutToConstructionEntry(cycleTable.Table + constructedCycles, candidate.Index);
				// We construct the way back to the root (each root is marked with ROOT_FLAG as relation) from d_1 and d_2		if 	*/
				if (print) 
				{
					printf("An edge finishing the cycle.\n"); fflush(stdout);
				}
				for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++) 
				{
					last_entry = &lpTable.Hashtable[index[i]];
					while ((last_entry->GetRelation()& ConstHT::ROOT_FLAG) != ConstHT::ROOT_FLAG) 
					{
						// by (& ROOT_FLAG) bit operation we mask out the parity bit
						cycleTable.PutToConstructionEntry(cycleTable.Table + constructedCycles, last_entry->GetRelation() & ConstHT::ROOT_FLAG);					
						last_entry = &lpTable.Hashtable[last_entry->GetAncestor()];
					}
				}

				constructedCycles++;				
				lastIterConstructed++;
			}
			//printf("End of while cycle iter...\n");
		}

		fclose(partialsFILE);
		partialsFILE = NULL;
		fclose(tempFILE);
		tempFILE = NULL;


		rc = MSerializable::RenameFile(this->iTempFileFullFileName,this->iPartialRelFullFileName);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}

		iteration++;
		if (!used && !deferred) 
		{
			Utils::NormalMessage("Strange, we have run out of edges!");
			cout << "used:"<<used<<" deferred:"<< deferred<< " lic:"<< lastIterConstructed<< " cc:"<<constructedCycles;
			// abort();
			break;
		}
		if (!lastIterConstructed) 
		{
			Utils::NormalMessage("Strange, we were not able to construct any cycle in this iteration!");
			break;
		}
	} 

	lpTable.DeleteHashtable();

	if (Get_InfoMode()) 
	{
		printf("\n\t\tIterations: %d", iteration);
	}

	rc = cycleTable.RemoveDuplicitePaths();
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	if (Get_InfoMode()) 
	{
		cycleTable.PrintCycleSizes();
	}

	// Now, we actually construct the cycles according to the information gathered in previous code
	// At first, we create space for them
	ARelation ** partialCycles = NULL;
	partialCycles = (ARelation**) Utils::AllocationMachine((void**) &partialCycles, cycleTable.AllocatedSize * sizeof(ARelation*));
	if (partialCycles == NULL) 
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","ConstructCycles",0,0,"Not enough memory to allocate nfs relations.");
		return ConstRC::NotEnoughMemory;
	}	
	long partialCyclesMaxIndex = cycleTable.AllocatedSize - 1;

	for (j = 0; j <= partialCyclesMaxIndex; j++) 
	{
		partialCycles[j] = new CNFSRelation(CNFSRelation::MODE_CYCLE_CONSTRUCTION);
		partialCycles[j]->Index = j;
		cycleTable.Table[j].running_index = 0;
	}

	partialsFILE = fopen(this->iUsablePartialRelFullFileName.c_str(), FILE_MODE_READ);
	if (partialsFILE == NULL) 
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","ConstructCycles",0,0,"Could not open file with partial relations.");
		DeleteRelationsArray(partialCycles, partialCyclesMaxIndex);
		return ConstRC::CouldNotOpenFile;
	}

	Polynomial * thetaPoly[ConstSPhase::ELEMENT_MASK_COUNT];
	for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
	{
		thetaPoly[i] = NULL;
		if (this->iSievingPoly[i]->GetDegree() > 1)
		{
			rc = IntegralDomain::ComputeThetaPolynomial(this->iSievingPoly[i], thetaPoly[i]);
			if (rc != ConstRC::Ok)
			{
				DeleteRelationsArray(partialCycles, partialCyclesMaxIndex);
				fclose(partialsFILE);
				partialsFILE = NULL;
				for (j = 0; j <= i; j++)
				{
					if (thetaPoly[j] != NULL)
					{
						delete thetaPoly[j];
						thetaPoly[j] = NULL;
					}
				}
				return rc;
			}
		}
	}

	if (Get_InfoMode()) 
	{
		printf("\n\t\tConstruction of smooth relations\n");
	}

	int currentRelation = 0;
	int currentIndex    = 0;
	while (candidate.ReadFromFile(partialsFILE) == ConstRC::Ok) 
	{
		currentRelation = candidate.Index;
		for (j = 0; j <= partialCyclesMaxIndex; j++) 
		{
			if (cycleTable.Table[j].running_index>cycleTable.Table[j].current_max_index) continue;
			if ((currentIndex = cycleTable.Table[j].running_index) != (signed) ConstHT::FINISHED) 
			{
				if (currentRelation == cycleTable.Table[j].array[currentIndex]) 
				{
					// We add the relation to the corresponding partial cycle*/
					//printf("Adding relation to partial cycle %d.\n",i);
					rc = partialCycles[j]->CombineWithRelation(&candidate, ConstSPhase::ELEMENT_MASK_COUNT, thetaPoly);
					if (rc != ConstRC::Ok)
					{
						DeleteRelationsArray(partialCycles, partialCyclesMaxIndex);
						fclose(partialsFILE);
						partialsFILE = NULL;
						for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
						{
							if (thetaPoly[i] != NULL)
							{
								delete thetaPoly[i];
								thetaPoly[i] = NULL;
							}
						}
						return rc;
					}
					if ((++(cycleTable.Table[j].running_index)) > cycleTable.Table[j].current_max_index) 
					{
						cycleTable.Table[j].running_index = ConstHT::FINISHED;
					}
				}
			}
		}
	}

	fclose(partialsFILE);
	partialsFILE = NULL;

	for (j = 0; j < ConstSPhase::ELEMENT_MASK_COUNT; j++)
	{
		if (thetaPoly[j] != NULL)
		{
			delete thetaPoly[j];
			thetaPoly[j] = NULL;
		}
	}

	cycleTable.DeleteTable();

	FILE * smoothsFILE = fopen(this->iSmoothRelFullFileName.c_str(), FILE_MODE_APPEND);
	if (smoothsFILE == NULL) 
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","ConstructCycles",0,0,"Could not open file with smooth relations.");
		DeleteRelationsArray(partialCycles, partialCyclesMaxIndex);
		return ConstRC::CouldNotOpenFile;
	}

	// Now, we add the cycles to the smooths file.
	for (int i = 0; i < constructedCycles; i++)
	{	
		partialCycles[i]->Index = this->iSmoothRelationsCount;
		this->iSmoothRelationsCount++;
		rc = partialCycles[i]->PrintToFile(smoothsFILE);
		if (rc != ConstRC::Ok)
		{
			DeleteRelationsArray(partialCycles, partialCyclesMaxIndex);
			fclose(smoothsFILE);
			smoothsFILE = NULL;
			return rc;
		}
		//partialCycles[i]->PrintToScreen();
	}

	fclose(smoothsFILE);
	smoothsFILE = NULL;

	DeleteRelationsArray(partialCycles, partialCyclesMaxIndex);

	string auxStr = Utils::ConvertToString(this->iSmoothRelationsCount);
	Get_Parameters()->SetPhaseParameter(ConstPN::SMOOTH_REL_COUNT, auxStr, Get_PhaseNbr());
	Get_Parameters()->SetParameterForOthers(ConstPN::SMOOTH_REL_COUNT, auxStr, Get_PhaseNbr());

	return ConstRC::Ok;
}
/*! 
	Here we list all the "definite" smooth relations and check them for
    occurrence of unique small prime ideals - small singletons.
    This is, of course, an iterative process.

	We have a "detection field" for each of the factor bases.
	
	It has three possible values for each member of the factor base:
	NOT_YET ... -10 ... means that this prime ideal has not occurred anywhere (cannot be 0 since a prime can occur in relation nr. 0)
	
	TWICE   ... 0xffffffff ... means that this prime ideal has occurred twice or more times
	
	(something in between) ... number of the only relation containing this prime ideal so far

	At each iteration we do the following:

	* reset the detection fields to contain NOT_YET values everywhere
	* read remaining relations one-by-one
	* for each of the remaining relations:
		** iterate through all the prime ideal divisors in both FB
	        ** put each of them into the detection field with the following logic:
		   *** if there is NOT_YET at the given index of the detection field, change
		       it into index of this relation
        	   *** if there is index of a relation, change it into TWICE
		   *** if there is TWICE, do nothing

	* After all of the relations have been read, check which relations 
	  contain singletons, by reading all the indices of relations in the detection fields.
	* Sort their numbers into an auxiliary block of longs.
	* Read the smooths again, and do not write back those whose numbers are
	  contained in the field of "bad" (singleton-containing) relations.

	* Iterate as long as there is any singleton-containing relation left.
	* At the last iteration, each of the small prime ideals
	  should be contained in at least two relations, or in no relation at all. 
	  Therefore, the indices where NOT_YET is contained are precisely those
          indices, at which redundant elements of factor bases are present.
*/
int CRelProcessingPhase::RemoveSmallSingletons()
{
	Set_InnerState((int)ERPPhaseRemovingSmallSingletons);

	int i;
	long j;
	int rc;
	if (this->iVariationsInfo[0].Type == EDoNotUseVariations && this->iVariationsInfo[1].Type == EDoNotUseVariations)
	{
		rc = MSerializable::MCopyFile(this->iSmoothRelFullFileName, this->iAllSmoothRelFullFileName);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}
	
	if (Get_InfoMode())
	{
		Utils::NormalMessage("\tRemoving small singletons ...");
		Utils::NormalMessage("\t\tLoading FBs");
    }

#ifdef DEBUG_REL_PRO_PHASE_ASSERTION_MODE_ON
	if (Get_AssertionMode())
	{
		CTestSuite::ControlSmoothAlgRelationsCount(this->iSmoothRelFullFileName, this->iSmoothRelationsCount);
	}
#endif

	CBaseParameters bparam;
	bparam.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, Get_SerialPhaseFullFileName());
	rc = Serialize(bparam);
	if (rc != ConstRC::Ok) 
	{
		return rc;
	}

	// read factor bases
	for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
	{
		bparam.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iFBs[i].FBFullFileName);
		rc = this->iFBs[i].Deserialize(bparam);
		if (rc != ConstRC::Ok) 
		{
			return rc;
		}
	}

	long* detectionField[ConstSPhase::ELEMENT_MASK_COUNT];
	for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
	{
		detectionField[i] = NULL;
		detectionField[i] = (long*) Utils::AllocationMachine((void**) &detectionField[i], sizeof(long)*(this->iFBs[i].FBMaxIndex + 1));
		if (detectionField[i] == NULL) 
		{
			CLog::LogError(WHEREARG,"CRelProcessingPhase","RemoveSmallSingletons",0,0,"Not enough memory to allocate data space.");
			for (j = 0; j < i; j++)
			{
				free(detectionField[j]);
				detectionField[j] = NULL;
			}
			return ConstRC::NotEnoughMemory;
		}
	}

	long* listForDeletion = NULL;
	int   listAllocated = 0;
	int   listMaxIndex = -1;

	listForDeletion = (long*) Utils::AllocationMachine((void**) &listForDeletion, sizeof(long) * ConstHT::IMPLICIT_LIST_FOR_DELETION_SIZE);
	if (listForDeletion == NULL) 
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","RemoveSmallSingletons",0,0,"Not enough memory to allocate data space.");
		for (j = 0; j < ConstSPhase::ELEMENT_MASK_COUNT; j++)
		{
			free(detectionField[j]);
			detectionField[j] = NULL;
		}
		return ConstRC::NotEnoughMemory;
	}	
	listAllocated = ConstHT::IMPLICIT_LIST_FOR_DELETION_SIZE;
	listMaxIndex = -1;

	int totalSingletons = 0;
	int singletons = 0;
	int removedRelations = 0;
	long relationIndex;
	long newRelationIndex;
	int read;

	CNFSRelation candidate(CNFSRelation::MODE_CYCLE_CONSTRUCTION);

	FILE * smoothsFILE = NULL;
	FILE * tempFILE = NULL;

	do 
	{
		singletons = 0;
		removedRelations = 0;
		read = 0;

		for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
		{
			for (j = 0; j <= this->iFBs[i].FBMaxIndex; j++) 
			{
				detectionField[i][j] = ConstHT::NOT_YET;
			}
		}

		smoothsFILE = fopen(this->iSmoothRelFullFileName.c_str(), FILE_MODE_READ);
		if (smoothsFILE == NULL) 
		{
			CLog::LogError(WHEREARG,"CRelProcessingPhase","RemoveSmallSingletons",0,0,"Could not open file with smooth relations.");
			return ConstRC::CouldNotOpenFile;
		}

		while (candidate.ReadFromFile(smoothsFILE) == ConstRC::Ok) 
		{
			read++;

			for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
			{
				rc = ControlSingletons(candidate.Get_Parts(i)->Main, candidate.Index, this->iFBs[i], detectionField[i], singletons);
				if (rc != ConstRC::Ok)
				{
					for (j = 0; j < ConstSPhase::ELEMENT_MASK_COUNT; j++) 
					{
						free(detectionField[j]);
						detectionField[j] = NULL;
					}
					free(listForDeletion);
					listForDeletion = NULL;
					fclose(smoothsFILE);
					smoothsFILE = NULL;
					return ConstRC::GeneralError;
				}
			}
		}

		fclose(smoothsFILE);
		smoothsFILE = NULL;

		// Now we must detect the relations which contain singletons. For this, 
		// we'll make use of another 
		listMaxIndex = -1;
		for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
		{
			rc = DetectSingletons(this->iFBs[i].FBMaxIndex, detectionField[i], listMaxIndex, listAllocated, listForDeletion);
			if (rc != ConstRC::Ok)
			{
				for (j = 0; j < ConstSPhase::ELEMENT_MASK_COUNT; j++) 
				{
					free(detectionField[j]);
					detectionField[j] = NULL;
				}
				free(listForDeletion);
				listForDeletion = NULL;
				return ConstRC::GeneralError;
			}
		}

		// The list of relations for deletion must be sorted, to access
		// its elements as fast as possible.
		qsort(listForDeletion, listMaxIndex + 1, sizeof(long), &Utils::CompareLongs);

		//printf("List for deletion has %d entries.\n", listMaxIndex + 1);
		//for (j = 0; j <= listMaxIndex; j++) 
		//{
		//	printf("List for deletion: %ld.\n", listForDeletion[j]);
		//}

		// Now we copy only non-singleton relations


		smoothsFILE = fopen(this->iSmoothRelFullFileName.c_str(), FILE_MODE_READ);
		if (smoothsFILE == NULL) 
		{
			CLog::LogError(WHEREARG,"CRelProcessingPhase","RemoveSmallSingletons",0,0,"Could not open file with smooth relations.");
			return ConstRC::CouldNotOpenFile;
		}
		tempFILE = fopen(this->iTempFileFullFileName.c_str(), FILE_MODE_WRITE);
		if (tempFILE == NULL) 
		{
			CLog::LogError(WHEREARG,"CRelProcessingPhase","RemoveSmallSingletons",0,0,"Could not open temp file.");
			fclose(smoothsFILE);
			return ConstRC::CouldNotOpenFile;
		}
		newRelationIndex = -1;

		while ((rc = candidate.ReadFromFile(smoothsFILE)) == ConstRC::Ok) 
		{
			relationIndex = candidate.Index;
			if (!bsearch(&relationIndex, listForDeletion, listMaxIndex + 1, sizeof(long), &Utils::CompareLongs)) 
			{				
				candidate.Index = ++newRelationIndex;
				rc = candidate.PrintToFile(tempFILE);
				if (rc != ConstRC::Ok)
				{
					for (j = 0; j < ConstSPhase::ELEMENT_MASK_COUNT; j++) 
					{
						free(detectionField[j]);
						detectionField[j] = NULL;
					}
					free(listForDeletion);
					listForDeletion = NULL;
					fclose(smoothsFILE);
					smoothsFILE = NULL;
					fclose(tempFILE);
					tempFILE = NULL;
					return rc;
				}
			} 
			else 
			{
				removedRelations++;
				this->iSmoothRelationsCount--;
			}
		}

		fclose(smoothsFILE);
		smoothsFILE = NULL;
		fclose(tempFILE);
		tempFILE = NULL;

		rc = MSerializable::RenameFile(this->iTempFileFullFileName,this->iSmoothRelFullFileName);
		if (rc != ConstRC::Ok)
		{
			for (j = 0; j < ConstSPhase::ELEMENT_MASK_COUNT; j++) 
			{
				free(detectionField[j]);
				detectionField[j] = NULL;
			}
			free(listForDeletion);
			listForDeletion = NULL;
			return rc;
		}

#ifdef DEBUG_REL_PRO_PHASE_ASSERTION_MODE_ON
		if (Get_AssertionMode() && this->iSmoothRelationsCount != newRelationIndex + 1)
		{
			printf("Strange: the count of relations is wrong!!! SmoothCount %ld, RelationIndex %ld \n", this->iSmoothRelationsCount, newRelationIndex + 1);
		}
#endif
		
		if (Get_InfoMode())
		{
			printf("\t\tSingletons: %d (Remove rel: %d; Read: %d)\n", singletons, removedRelations, read); 
			fflush(stdout);
		}
		totalSingletons += singletons;		
	} 
	while (removedRelations > 0);

	free(listForDeletion);
	listForDeletion = NULL;

	// Now we have only smooth values not containing small singletons
	// We can kick out the useless elements (not present in any relation) from the FBs.
	for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
	{
		rc = this->iFBs[i].CleanFB(detectionField[i]);
		if (rc != ConstRC::Ok)
		{
			break;
		}
	}
	for (j = 0; j < ConstSPhase::ELEMENT_MASK_COUNT; j++) 
	{
		free(detectionField[j]);
		detectionField[j] = NULL;
	}
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	// We can serialize FBs for later use
	for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
	{
		bparam.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iFBs[i].FBFullFileName);
		rc = this->iFBs[i].Serialize(bparam);
		if (rc != ConstRC::Ok) 
		{
			return rc;
		}
	}

	// save new smooth relations count
	string auxStr = Utils::ConvertToString(this->iSmoothRelationsCount);
	Get_Parameters()->SetPhaseParameter(ConstPN::SMOOTH_REL_COUNT, auxStr, Get_PhaseNbr());
	Get_Parameters()->SetParameterForOthers(ConstPN::SMOOTH_REL_COUNT, auxStr, Get_PhaseNbr());

	return ConstRC::Ok;
}
/*!
	This method will convert the current list of smooth values to a bit
	matrix, which will then be used for the linear algebra phase.

	Legendre symbols will be added to the matrix as extra columns in the end.
*/
int CRelProcessingPhase::CreateRelationMatrix() 
{
	Set_InnerState((int)ERPPhaseCreatingRelMatrix);

	if (Get_InfoMode())
	{
		Utils::NormalMessage("\tConverting to bit matrix representation ... ");
    }

#ifdef DEBUG_REL_PRO_PHASE_ASSERTION_MODE_ON
	if (Get_AssertionMode())
	{
		CTestSuite::ControlSmoothNFSRelationsCount(this->iSmoothRelFullFileName, this->iSmoothRelationsCount);
	}
#endif

	CBaseParameters bparam;
	bparam.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, Get_SerialPhaseFullFileName());
	int rc = Serialize(bparam);
	if (rc != ConstRC::Ok) 
	{
		return rc;
	}

	// At first we copy smooth relations to relations result file - this file is used in next phases
	rc = MSerializable::MCopyFile(this->iSmoothRelFullFileName, this->iRelationsResultFullFileName);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	// We need to know how many quadratic characters we will use
	int quadCharsCount = 0;
	Get_Parameters()->GetIntegerPhaseParameter(ConstPN::QUAD_CHARS_COUNT, ConstRPPhase::QUAD_CHARS_COUNT, quadCharsCount, Get_PhaseNbr());
	long maxRow = this->iSmoothRelationsCount - 1;
	// maxColumn = -1 because of max index
	long maxColumn = -1; 
	long i, j, k;
	int nbrOfAlgParts = 0;
	bool columnOfOnes = false;
	// maxColumn += count of FB elements for all FBs
	for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
	{
		maxColumn += this->iFBs[i].FBMaxIndex + 1;
		if (this->iSievingPoly[i]->GetDegree() > 1)
		{
			nbrOfAlgParts++;
		}
		if (mpz_cmp_ui(*this->iSievingPoly[i]->Leading2(), 1) != 0)
		{
			columnOfOnes = true;
		}
	}
	// maxColumn += nbrOfAlgParts * count of characters 
	maxColumn += nbrOfAlgParts * quadCharsCount;
	// maxColumn += 1 as column of 1's 
	if (columnOfOnes)
	{
		maxColumn++;
	}

	// offset setup
	long fbOffsets[ConstSPhase::ELEMENT_MASK_COUNT];
	long legendreOffsets[ConstSPhase::ELEMENT_MASK_COUNT];
	long offset = 0;
	for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
	{
		fbOffsets[i] = offset;
		offset += this->iFBs[i].FBMaxIndex + 1;
	}
	for (i = 0; i < nbrOfAlgParts; i++)
	{
		legendreOffsets[i] = offset;
		offset += quadCharsCount;
	}

	if (maxRow < maxColumn) 
	{ 
		CLog::LogError(WHEREARG,"CRelProcessingPhase","CreateRelationMatrix",0,0,"Columns are more than rows.");
		printf("\t\tMaximal row is %d, maximal column is %d.\n", maxRow, maxColumn);
		return ConstRC::NotEnoughSmooths; 
	}
	
	// Now, we will read entries from smooths file (which includes cycles constructed from partial values) and
	// put them one-by-one to the (sparse bit) matrix.
	// Indexing in matrix: we will use index 
	if (Get_InfoMode())
	{
		printf("\t\tMatrix: row = %ld, column %ld", maxRow + 1, maxColumn + 1);
    }

	CNFSRelation candidate(CNFSRelation::MODE_CYCLE_CONSTRUCTION);

	Polynomial * thetaPoly = NULL;
	prime_ideal_for_legendre * quadChars[ConstSPhase::ELEMENT_MASK_COUNT];
	for (i = 0, j = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
	{
		if (this->iSievingPoly[i]->GetDegree() > 1)
		{
			rc = IntegralDomain::ComputeThetaPolynomial(this->iSievingPoly[i], thetaPoly);
			if (rc != ConstRC::Ok)
			{
				for (k = 0; k < j; k++)
				{
					free(quadChars[k]);
					quadChars[k] = NULL;
				}
				return rc;
			}

			quadChars[j] = NULL;
			rc = PrepareQuadraticCharacters(quadChars[j], quadCharsCount, &candidate, i, thetaPoly, 
				this->iFBs[i].GetMaxPrime(), this->iSmoothRelFullFileName);
			if (rc != ConstRC::Ok) 
			{
				CLog::LogError(WHEREARG,"CRelProcessingPhase","CreateRelationMatrix",0,0,"Problem in filling of the quadratic characters");
				delete thetaPoly;
				thetaPoly = NULL;
				for (k = 0; k <= j; k++)
				{
					if (quadChars[k] != NULL)
					{
						free(quadChars[k]);
						quadChars[k] = NULL;
					}
				}
				return rc;
			}

			delete thetaPoly;
			thetaPoly = NULL;
			j++;
		}
	}

	
#ifdef DEBUG_REL_PRO_PHASE_SAVE_QUAD_CHARACTERS

	// now we serialize the first quadratic characters - for square root phase (debug)
	CBaseParameters paramChars;
	string auxStrCH;
	for (i = 0, j = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
	{
		if (this->iSievingPoly[i]->GetDegree() > 1)
		{
			auxStrCH = Utils::ConvertToString(i + 1);
			paramChars.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iQuadCharsFullFileName[i]);

			rc = ARelProcessingPhase::SerializeQuadCharacters(paramChars, quadChars[j], quadCharsCount);
			if (rc != ConstRC::Ok) 
			{
				for (k = 0; k < nbrOfAlgParts; k++)
				{
					if (quadChars[k] != NULL)
					{
						free(quadChars[k]);
						quadChars[k] = NULL;
					}
				}
				return rc;
			}
			// we also save this file name to NFSParameters for other phases
			Get_Parameters()->SetParameterForOthers(ConstPN::QUAD_CHARS_FULL_FILENAME + auxStrCH, this->iQuadCharsFullFileName[i], Get_PhaseNbr());
			j++;
		}
	}

#endif

	if (Get_InfoMode())
	{
		Utils::NormalMessage("\t\tFQCh finished");
	}

#ifdef DEBUG_REL_PRO_PHASE_PRINT_FB
	
	for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
	{
		this->iFBs[i].PrintToScreen();
	}

#endif	

	AbstractMatrix * matrix = CMatrixHelper::CreateMatrix(this->iRelationMatrixType);
	if (matrix == NULL)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","CreateRelationMatrix",0,0,"Cannot create relation matrix.");
		for (j = 0; j < nbrOfAlgParts; j++)
		{
			if (quadChars[j] != NULL)
			{
				free(quadChars[j]);
				quadChars[j] = NULL;
			}
		}
		return ConstRC::GeneralError;
	}
	matrix->SetMaximalColumnIndex(maxColumn);
	matrix->SetMaximalRowIndex(maxRow);

	FILE * relationsResultFILE = fopen(this->iRelationsResultFullFileName.c_str(), FILE_MODE_READ);
	if (relationsResultFILE == NULL) 
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","CreateRelationMatrix",0,0,"Could not open file with result relations.");
		for (j = 0; j < nbrOfAlgParts; j++)
		{
			if (quadChars[j] != NULL)
			{
				free(quadChars[j]);
				quadChars[j] = NULL;
			}
		}
		delete matrix;
		matrix = NULL;
		return ConstRC::CouldNotOpenFile;
	}

	if (Get_InfoMode())
	{
		printf("\n\t\tConverting ...");
	}

	long count = 0;
	long max;
	long columnIndex;
	nfs_sieving_element * element;
	int iter = 0;
	long row = 0;
	while (candidate.ReadFromFile(relationsResultFILE) == ConstRC::Ok) 
	{
		// We will walk the elements one-by-one and put 1's in corresponding locations.
		// At first, the first algebraic divisors are iterated.
		iter++;
		if (Get_InfoMode() && !(iter % 10))
		{
			printf("\r\t\tConverting ... %10d       / %10ld      ", iter, this->iSmoothRelationsCount);
			fflush(stdout);
		}
		for (i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
		{
			max = candidate.Get_Parts(i)->Main.MaxElementIndex;
			element = candidate.Get_Parts(i)->Main.Elements;
			for (k = 0; k <= max; k++, element++) 
			{	
				if (element->exponent % 2) 
				{
					count++;
					columnIndex = this->iFBs[i].FindIndexInFB(element);
					if (columnIndex == -1) 
					{
						CLog::LogError(WHEREARG,"CRelProcessingPhase","CreateRelationMatrix",0,0,"PrimeIdeal was not found in the FB.");
						for (j = 0; j < nbrOfAlgParts; j++)
						{
							if (quadChars[j] != NULL)
							{
								free(quadChars[j]);
								quadChars[j] = NULL;
							}
						}
						delete matrix;
						matrix = NULL;
						fclose(relationsResultFILE);
						relationsResultFILE = NULL;
						return ConstRC::GeneralError;
					}
					columnIndex += fbOffsets[i];
					matrix->PutOne(row, columnIndex);
				}
			}
		}

		// Now, generalized Legendre symbols are appended to the matrix.
		for (i = 0, k = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
		{
			if (this->iSievingPoly[i]->GetDegree() > 1)
			{
				rc = ComputeLegendreSymbols(quadChars[k], quadCharsCount, candidate.Get_Parts(i)->Poly, matrix, row, legendreOffsets[k]);
				if (rc != ConstRC::Ok)
				{
					for (j = 0; j < nbrOfAlgParts; j++)
					{
						if (quadChars[j] != NULL)
						{
							free(quadChars[j]);
							quadChars[j] = NULL;
						}
					}
					delete matrix;
					matrix = NULL;
					fclose(relationsResultFILE);
					relationsResultFILE = NULL;
					return rc;
				}
				k++;
			}
		}

		// last column of 1's
		// more precisely: we have to add number of relations mod 2
		// if we use large prime variations then some relations are combined from more relations
		if (columnOfOnes && candidate.Get_NumberOfRelations() % 2)
		{
			matrix->PutOne(row, maxColumn);
		}

		row++;
	}

	// matrix is now ready
	Set_RelationMatrix(matrix);
	matrix = NULL;

	if (Get_InfoMode())
	{
		printf("\r\t\tConverting ... %10d       / %10ld      ", iter, this->iSmoothRelationsCount);
		printf("\n\t\tNonzero bits in the matrix: %ld\n", count);
	}

	fclose(relationsResultFILE);
	relationsResultFILE = NULL;
	
	for (j = 0; j < nbrOfAlgParts; j++)
	{
		if (quadChars[j] != NULL)
		{
			free(quadChars[j]);
			quadChars[j] = NULL;
		}
	}

	return ConstRC::Ok;
}
int CRelProcessingPhase::Serialize(const CBaseParameters &aParam) const
{
	int rc = MSerializable::Serialize(aParam);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	if (Running())
	{
		AFactorAlgParameters * param = Get_Parameters();

		// for deserialization at new start
		param->SetPhaseParameter(ConstPN::DESERIALIZE, Utils::ConvertToString(true), Get_PhaseNbr());
		param->SetPhaseParameter(ConstPN::PHASE_FULL_FILENAME, 
			MSerializable::CreateFileNameAuto(aParam, ConstFN::REL_PRO_PHASE), Get_PhaseNbr());

		CBaseParameters paramb;
		paramb.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, Get_ParametersFullFileName()); 
		rc = param->Serialize(paramb);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	return ConstRC::Ok; 
}
int CRelProcessingPhase::Serialize(const CBaseParameters &aParam, xmlTextWriterPtr &aWriter) const
{
	string mainTagName;	
	aParam.GetStringParameter(ConstPN::SERIALIZATION_ELEMENT_NAME, XML_SERIAL_REL_PRO_PHASE_NAME, mainTagName);

	int rc = xmlTextWriterStartElement(aWriter, BAD_CAST mainTagName.c_str());
    if (rc < 0) 
	{
        CLog::LogError(WHEREARG,"CRelProcessingPhase","Serialize",rc,0,"Could not write the parameter tag.");
		return ConstRC::GeneralError;
    }
	rc = CXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	int state = Get_InnerState();
	rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_INNER_STATE_NAME, state);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	if (state >= ERPPhaseConstructingCycles)
	{
		rc = xmlTextWriterStartElement(aWriter, BAD_CAST XML_SERIAL_ROOT_LIST_NAME);
		if (rc < 0) 
		{
			CLog::LogError(WHEREARG,"CRelProcessingPhase","Serialize",rc,0,"Could not write the start element.");
    		return ConstRC::GeneralError;
		}
		rc = CXMLHelper::NewLine(aWriter,FALSE);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}

		int max = this->iRootsList.size();
		for (int i = 0; i < max; i++)
		{
			rc = CXMLHelper::SerializeHashtableEntryType1(aWriter, XML_SERIAL_ELEMENT_NAME, this->iRootsList[i]);
			if (rc != ConstRC::Ok)
			{
				return rc;
			}
		}

		rc = xmlTextWriterEndElement(aWriter); // XML_SERIAL_ROOT_LIST_NAME
		if (rc < 0) 
		{
			CLog::LogError(WHEREARG,"CRelProcessingPhase","Serialize",rc,0,"Could not end the start element.");
			return ConstRC::GeneralError;
		}
		rc = CXMLHelper::NewLine(aWriter,FALSE);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}

		rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_INDEPENDENT_CYCLE_NAME, this->iIndependentCycles);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}

		rc = CXMLHelper::SerializeSignedInt(aWriter, XML_SERIAL_EXPECTED_VERTICES_NAME, this->iExpectedVertices);
		if (rc != ConstRC::Ok)
		{
			return rc;
		}
	}

	rc = xmlTextWriterEndElement(aWriter); // mainTagName
    if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","Serialize",rc,0,"Could not end the parameter tag.");
        return ConstRC::GeneralError;
    }
	rc = CXMLHelper::NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	return ConstRC::Ok; 
}
int CRelProcessingPhase::Deserialize(const CBaseParameters &aParam)
{
	int rc = MSerializable::Deserialize(aParam);
	if (rc != ConstRC::Ok)
	{
		return rc;
	}

	if (Get_InnerState() == ERPPhaseCreatingRelMatrix)
	{
		if (Get_InfoMode())
		{
			Utils::NormalMessage("		Deserializing FBs ... ");
		}

		CBaseParameters paramFB;
		// read factor bases
		for (int i = 0; i < ConstSPhase::ELEMENT_MASK_COUNT; i++)
		{
			paramFB.SetParameter(ConstPN::SERIALIZATION_FULL_FILENAME, this->iFBs[i].FBFullFileName);
			rc = this->iFBs[i].Deserialize(paramFB);
			if (rc != ConstRC::Ok) 
			{
				return rc;
			}
		}
	}

	return ConstRC::Ok;
}
int CRelProcessingPhase::Deserialize(const CBaseParameters &aParam, xmlTextReaderPtr &aReader)
{
	// we can deserialize only during initialization
	if (Get_InnerState() != ERPPhaseInitialization)
	{
		return ConstRC::CannotBeDone;
	}

	string mainTagName;	
	aParam.GetStringParameter(ConstPN::SERIALIZATION_ELEMENT_NAME, XML_SERIAL_REL_PRO_PHASE_NAME, mainTagName);

	int rc;
	xmlChar* name = xmlTextReaderName(aReader);

	if (name == NULL)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","Deserialize",0,0,"Deserialization main element name not found.");
		return ConstRC::GeneralError;  
	}

	if (xmlStrEqual(name,BAD_CAST mainTagName.c_str()) == 0)
	{
		CLog::LogError(WHEREARG,"CRelProcessingPhase","Deserialize",0,0,"Main element name is wrong.");
		return ConstRC::GeneralError; 
	}

	int auxInt = 0;
	int depth = xmlTextReaderDepth(aReader);

	while ((rc = CXMLHelper::MoveToNextNode(aReader, depth)) == ConstRC::Ok)
	{
		name = xmlTextReaderName(aReader);
	
		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_INNER_STATE_NAME))
		{
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize inner state...", auxInt, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			Set_InnerState(auxInt);
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_INDEPENDENT_CYCLE_NAME))
		{
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize independent cycle...", this->iIndependentCycles, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_EXPECTED_VERTICES_NAME))
		{
			rc = CXMLHelper::DeserializeSignedInt(aReader, "Deserialize expected vertices...", this->iExpectedVertices, FALSE);
			if (rc != ConstRC::Ok)
			{
				rc = ConstRC::GeneralError;
				break;
			}
			continue;
		}

		if (xmlStrEqual(name, BAD_CAST XML_SERIAL_ROOT_LIST_NAME))
		{
			this->iRootsList.clear();
			int elementsDepth = xmlTextReaderDepth(aReader);
			CNFS_HashEntry1 current;
			while ((rc = CXMLHelper::MoveToNextNode(aReader, elementsDepth)) == ConstRC::Ok)
			{
				name = xmlTextReaderName(aReader);

				if (xmlStrEqual(name, BAD_CAST XML_SERIAL_ELEMENT_NAME))
				{
					rc = CXMLHelper::DeserializeHashtableEntryType1(aReader, "Deserialize root...", current, FALSE);
					if (rc != ConstRC::Ok)
					{
						rc = ConstRC::GeneralError;
						break;
					}
					this->iRootsList.push_back(current);
					continue;
				}
			}

			if (rc == ConstRC::GeneralError || rc == ConstRC::NotEnoughMemory)
			{
				break;
			}
			continue;
		}
	}

	if (rc == ConstRC::GeneralError || rc == ConstRC::NotEnoughMemory)
	{
		return rc;
	}

	return ConstRC::Ok;
}
