#ifndef __RELATION_PART_H__
#define __RELATION_PART_H__

#include "divisors.h"

/*!
 * \brief Relation part of NFS relation
 */
class CRelationPart : public virtual MSerializable
{
private:
	std::string iDescription;
	std::string iAuxStr;

public:
	///Divisors of a norm of this relation part
	CDivisors Main;
	CDivisors Large;

	///Polynomial representation in an integral domain - c_{d}a + b\bar{theta}
	Polynomial* Poly;

	///Norm of this part in an integral domain
	mpz_t Norm;
	///Remaining of the norm after trial division - in the end it should be one
	mpz_t NormRemaining;

public:
// CONSTANTS
	///Relation part type - combined result is in ARelation.Type
	static const unsigned int SMOOTH		= 0x1;
	static const unsigned int LARGE			= 0x2;
	static const unsigned int DOUBLE_LARGE  = 0x4;
	static const unsigned int TRIPPLE_LARGE = 0x8;
	static const unsigned int TOO_LARGE		= 0x80;
	static const unsigned int PARTIAL_MASK	= 0xe;

public:

	CRelationPart(const std::string aDescription = "");
	CRelationPart(const CRelationPart& aOperand);
	~CRelationPart();

	int CombineWithPart(const CRelationPart &aOperand, Polynomial * aThetaPoly);
	int CombineWithPart(const CRelationPart &aOperand);
	int Copy2(CRelationPart& aOperand);
	int Reset();
	bool Equals(const CRelationPart& aOperand) const;

	void PrintToScreen();
	int PrintToFile(FILE* aFw);
	int ReadFromFile(FILE * aFr, char * aBuffer, int aBufferSize);

	void Set_Description(std::string aDescription) { this->iDescription = aDescription; }
	std::string Get_Description() const { return this->iDescription; }

	// De/Serialization methods
	int Serialize(const CBaseParameters& aParam, xmlTextWriterPtr& aWriter) const;

	int Deserialize(const CBaseParameters& aParam, xmlTextReaderPtr& aReader);

};


#endif // __RELATION_PART_H__
