#ifndef __RELATION_READER_H_
#define __RELATION_READER_H_

#include "abstract_relation.h"

/*!
 * \brief Class for reading relations from the file
*/
class CRelationReader
{
private:

	relation_types iType;
	int iCurrentIndex;
	int iCount;
	ARelation * iCurrentRelation;
	xmlTextReaderPtr iReader;

	const CBaseParameters * iParameters;

public:

	///Constructor - aParameters are not deleted
	CRelationReader(const CBaseParameters * aParameters);
	~CRelationReader();

	/*!
		Read next relation from the file.
		
		This class has a pointer to last readed relation.
		
		If ConstRC::All is returned then there isn't next relation in the file
		and the file is closed (xmlreader is disposed).
	*/
	int ReadRelation(ARelation * aRelation);

	int Close();
	int Reset();

private:
	CRelationReader(const CRelationReader& aOperand) {}

	int PrepareForReading();
	int FinishReading();


	// Getters and Setters
public:
	relation_types Get_Type() const { return this->iType; }
	int Get_CurrentIndex() const { return this->iCurrentIndex; }
	int Get_Count() const { return this->iCount; }
	ARelation * Get_CurrentRelation() const { return this->iCurrentRelation; }
	const CBaseParameters * Get_Parameters() const { return this->iParameters; }
	void Set_Parameters(const CBaseParameters * aParameters) { this->iParameters = aParameters; }

};

#endif // __RELATION_READER_H_
