#ifndef __RELATION_WRITER_H_
#define __RELATION_WRITER_H_

#include "abstract_relation.h"

/*!
 * \brief Class for writing relations to the file 
*/
class CRelationWriter
{
private:

	relation_types iType;
	int iCurrentIndex;
	int iCount;
	xmlTextWriterPtr iWriter;

	const CBaseParameters * iParameters;

public:

	///Constructor - aParameters are not deleted
	CRelationWriter(const CBaseParameters * aParameters, int aCount = RELATION_UNDEFINED_COUNT);
	~CRelationWriter();

	/*!
		Write aRelation to file.
		
		If aRelation is first - create new file according to iParameters.
		
		For closing the file is necessary to call the method Close().
		File is closed also when this class is dispose.
	*/
	int WriteRelation(ARelation * aRelation);

	///Copy all relations from file in aParam to this working file
	int CopyRelationsFrom(const CBaseParameters& aParam);

	int Close();

private:
	CRelationWriter(const CRelationWriter& aOperand) {}

	int PrepareForWriting();
	int FinishWriting();


	// Getters and Setters
public:
	relation_types Get_Type() const { return this->iType; }
	void Set_Type(relation_types aType) { this->iType = aType; }
	int  Get_CurrentIndex() const { return this->iCurrentIndex; }
	int  Get_Count() const { return this->iCount; }
	void Set_Count(int aCount) { this->iCount = aCount; }
	const CBaseParameters * Get_Parameters() const { return this->iParameters; }
	void Set_Parameters(const CBaseParameters * aParameters) { this->iParameters = aParameters; }

};

#endif // __RELATION_READER_H_
