#ifndef __SQUARE_ROOT_PHASE_H__
#define __SQUARE_ROOT_PHASE_H__

#include "abstract_square_root_phase.h"
#include "nfs_relation.h"

///Class for square root phases - two algebraic bases or one algebraic and one integral base
class CSquareRootPhase : public ASquareRootPhase
{
protected:

	/*!
		Field of the sieving polynomials.

		This polynomials can be monic or non-monic.
	*/
	Polynomial*		iSievingPoly[CNFSRelation::DEFAULT_NFS_PARTS_COUNT];
	/*!
		Field of monic polynomials corresponding to the sieving polynomials.

		\bar{f}(x) = f(x/c_{d}) * c_{d}^{d - 1}

		where f is the sieving polynomial, c_{d} is the leading coefficient of f and d the degree of f.

		Obviously if the sieving polynomial is monic then this polynomial is same.
	*/
	Polynomial *	iThetaPoly[CNFSRelation::DEFAULT_NFS_PARTS_COUNT];
	///Algebraic elements as polynomials - for calculating square root
	Polynomial **	iAlgElements[CNFSRelation::DEFAULT_NFS_PARTS_COUNT];
	long			iAlgElementsCount;
	long			iAlgElementsAllocated[CNFSRelation::DEFAULT_NFS_PARTS_COUNT];
	///Index of the first algebraic part - degree of the sieving poly is bigger than 1
	int				iStartOfAlg;
	///Type of RootFinder used for algebraic square
	root_finder_types iRootFinderType;
	///Full filename with quadratic characters
	std::string iQuadCharsFullFileName[CNFSRelation::DEFAULT_NFS_PARTS_COUNT];
	///Given max bound of FBs
	long iMaxFBBound;
	///Aggregate relation - of all read relations
	ARelation * iRelation;


protected:

	///Print info about dependency
	void PrintDetails();

	/*!
		Fill the iPhaseFunctors with correct function pointers.
		This method is called from InitParameters in APhase.
	*/
	int FillFunctorField();

	///Dispose all resources which was used and prepare for new start. Also set inner state.
	int Reset();

	///Try k-th dependency for factorization 
	int TryKthDependence();

	///Try all dependencies for factorization 
	int TryDependencies();

	///Save phase's result
	int SaveResult();

	///Init all parameters necessary for calculation - get from iParameters
	int InitParameters();

public:

	CSquareRootPhase();
	~CSquareRootPhase();
	
	///Delete all used files according to enum cleaning_types
	int CleanUp(AFactorAlgParameters* aParameters, int aPhaseNBR, cleaning_types aType, bool aError);

	///Start method for linear phase
	int RunPhase(AFactorAlgParameters*  aParameters);

	// De/Serialization methods
	int Serialize(const CBaseParameters& aParam) const;
	int Serialize(const CBaseParameters& aParam, xmlTextWriterPtr& aWriter) const;

	int Deserialize(const CBaseParameters& aParam);
	int Deserialize(const CBaseParameters& aParam, xmlTextReaderPtr& aReader);

};


#endif // __SQUARE_ROOT_PHASE_H__
