#ifndef __STATISTIC_CONTAINER_H__
#define __STATISTIC_CONTAINER_H__

#include "mserializable.h"
#include <vector>
#include <pthread.h>
#include "xml_helper.h"

///Statistic container - for saving informations - thread safe
class CStatContainer : public virtual MSerializable
{
private:
	std::vector<int> iConfirmTotals;
	std::vector<int> iTotals;
	std::vector<int> iIntermediates;
	mutable std::vector<pthread_mutex_t> iMutexes;

	int iMaxSize;

private:
	CStatContainer(const CStatContainer& aOperand) {}

	bool Control(int aIndex) const { return (0 <= aIndex) && (aIndex < this->iMaxSize); }

public:

	CStatContainer(int aMaxSize);
	virtual ~CStatContainer();

	int Increase(int aIndex);
	int Increase(int aIndex, int aValue);
	int Decrease(int aIndex);
	int Decrease(int aIndex, int aValue);
	int Reset(int aIndex);
	int ResetAll();
	int ResetIntermediate(int aIndex);
	int ResetAllIntermediate();
	int Confirm(int aIndex);
	int ConfirmAll();

	// Getters and Setters

	int Get_MaxSize() const { return this->iMaxSize; }
	int Get_ConfirmTotal(int aIndex) const;
	int Get_Total(int aIndex) const;
	int Get_Intermediate(int aIndex) const;

	// De/Serialization methods
	int Serialize(const CBaseParameters& aParam, xmlTextWriterPtr& aWriter) const;
	int Deserialize(const CBaseParameters& aParam, xmlTextReaderPtr& aReader);

};

#endif // __STATISTIC_CONTAINER_H__
