#include "supporting_factor_alg_creator.h"

#include "const_return_codes.h"
#include "nfs_utils.h"

#include "pollard_p_m_1.h"
#include "pollard_rho.h"
#include "squfof.h"
#include "cfrac_class.h"
#include "log.h"
//#include "ecm_old.h"

#ifdef HAVE_CL_CL_H
#include "ECM/ECM.h"
#endif


int CFactorAlgCreator::CreateFactorAlg(supporting_factor_alg_types aType, ASupportingFactorAlg *& aAlg)
{
	if (aAlg != NULL)
	{
		CLog::LogError(WHEREARG,"CFactorAlgCreator","CreateFactorAlg",0,0,"Algorithm is not NULL.");
		return ConstRC::GeneralError;
	}

	switch (aType) 
	{
		case EFAPollardRho:	
			aAlg = new PollardRho();
			return ConstRC::Ok;	
		case EFASQUFOF:
			aAlg = new SQUFOF();
			return ConstRC::Ok;
		case EFACFRAC:		
			aAlg = new CFRAC(DEFAULT_FB_PRIME_BOUND, DEFAULT_CFRAC_BASE_SIZE);
			return ConstRC::Ok;	
		case EFAPMinus1:		
			aAlg = new PollardPMinus1(DEFAULT_FB_PRIME_BOUND);
			return ConstRC::Ok;	
		#ifdef HAVE_CL_CL_H
		case EFAEllipticCurve:
			aAlg = new ECM(4000, 256);
			return ConstRC::Ok;
		#endif
		default:		
			CLog::LogError(WHEREARG,"CFactorAlgCreator","CreateFactorAlg",0,0,"Number of factoring alg. is wrong.");
			return ConstRC::BadArgument;					
	}

	CLog::LogError(WHEREARG,"CFactorAlgCreator","CreateFactorAlg",0,0,"Number of factoring alg. is wrong.");
	return ConstRC::BadArgument;
}
