#ifndef __THREAD_POOL_H__
#define __THREAD_POOL_H__

#include <pthread.h>
#include "abstract_phase.h"
#include "abstract_factor_alg_parameters.h"


/*!
 * \brief Pool of threads for inner parallelization
 */

class CThreadPool
{
private:
// Private structure for thread work - specialized for factoring algorithms
	struct thread_work
	{
		APhase * phase;	
		AFactorAlgParameters * param;
		thread_work * next;
	};

private:
     /* pool characteristics */	
     int iThreadsCount;	
     int iMaxQueueSize;	
     bool iDoNotBlockWhenFull;	
     
	 /* pool state */	
     pthread_t* iThreads;	
     int iCurQueueSize;	
     thread_work* iQueueHead;	
     thread_work* iQueueTail;	
     bool iQueueClosed;	
     bool iShutdown;
     bool iDestroyed;
     mutable pthread_mutex_t iDestroyedMutex;
     bool iInnerError;
     pthread_mutex_t iQueueMutex;	
     pthread_cond_t  iQueueNotEmptyCond;	
     pthread_cond_t  iQueueNotFullCond;	
     pthread_cond_t  iQueueEmptyCond;	

public:

	CThreadPool(const int& aThreadsCount, const int& aMaxQueueSize, const bool& aDoNotBlockWhenFull);
	~CThreadPool();

	int AddWork(APhase * aPhase, AFactorAlgParameters * aParam);
	int Destroy(bool aWaitForFinish);

	// GETTERS and SETTERS
	int Get_ThreadsCount() const { return this->iThreadsCount; }
	int Get_MaxQueueSize() const { return this->iMaxQueueSize; }
	bool Get_Destroyed() const;
	bool Get_InnerError() const { return this->iInnerError; }

	bool Get_DoNotBlockWhenFull() const { return this->iDoNotBlockWhenFull; }
	void Set_DoNotBlockWhenFull(const bool& aDoNotBlockWhenFull) { this->iDoNotBlockWhenFull = aDoNotBlockWhenFull; }

private:
	static void * WorkerThreadStart(void * aArg);
	
	CThreadPool(const CThreadPool& aOperand);

	void InitDestroyed(bool aValue);

};

#endif // __THREAD_POOL_H__
