#ifndef __TIME_CONTAINER_H__
#define __TIME_CONTAINER_H__

#include "mserializable.h"
#include <vector>
#include <pthread.h>
#include "xml_helper.h"

///Time container - for saving time informations - thread safe
class CTimeContainer : public virtual MSerializable
{
private:
	std::vector<long> iIntermediateTimes;
	std::vector<long> iConfirmTotalTimes;
	std::vector<long> iTotalTimes;
	std::vector<long> iStartTimes;
	mutable std::vector<pthread_mutex_t> iMutexes;

	int iMaxSize;

	long iStorageRatio;
	static const long STANDARD_CLOCKS_PER_SEC	= 1000;

private:
	CTimeContainer(const CTimeContainer& aOperand) {}

	bool  Control(int aIndex) const { return (0 <= aIndex) && (aIndex < this->iMaxSize); }
	float GetSeconds(long aClocks) const { return aClocks / (float)CTimeContainer::STANDARD_CLOCKS_PER_SEC; } 
	long  GetStoreValue(long aClocks) const { return iStorageRatio != 0 ? aClocks / iStorageRatio : aClocks; }

public:

	CTimeContainer(int aMaxSize);
	virtual ~CTimeContainer();

	int Start(int aIndex);
	int StartAll();
	int End(int aIndex);
	int EndAll();
	int Reset(int aIndex);
	int ResetAll();
	int ResetIntermediate(int aIndex);
	int ResetAllIntermediate();
	int Confirm(int aIndex);
	int ConfirmAll();

	// Getters and Setters

	int  Get_MaxSize() const { return this->iMaxSize; }
	float Get_TotalTime(int aIndex) const;
	float Get_ConfirmTotalTime(int aIndex) const;
	float Get_IntermediateTime(int aIndex) const;

	// De/Serialization methods
	int Serialize(const CBaseParameters& aParam, xmlTextWriterPtr& aWriter) const;
	int Deserialize(const CBaseParameters& aParam, xmlTextReaderPtr& aReader);

};

#endif // __TIME_CONTAINER_H__
