#include <string>
#include <cstring>
#include "const_return_codes.h"
#include "xml_helper.h"
#include "log.h"



int CXMLHelper::SerializeMpf(xmlTextWriterPtr& aWriter, const char* aTagName, const mpf_t aValue)
{
	if (!aTagName)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Mpf",0,0,"Insufficient data.");
			return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Mpf",rc,0,"Could not write the serializator tag.");
		return ConstRC::GeneralError;
	}

	char* mpfChar = NULL;
	int base = 10;
	mp_exp_t exp;
	mpfChar = mpf_get_str(mpfChar, &exp, base, 20, aValue);
	int len = strlen(mpfChar);
	if (len > 0 && mpfChar[0] == '-')
	{
		len--;
	}
	exp -= len;

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_VALUE_ATTR,"%s",BAD_CAST mpfChar);
	free(mpfChar);
	mpfChar = NULL;
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Mpf",rc,0,"Could not write the value.");
		return ConstRC::GeneralError;
	}	
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_EXPONENT_ATTR,"%ld",exp);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Mpf",rc,0,"Could not write the exp.");
		return ConstRC::GeneralError;
	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_BASE_ATTR,"%d",base);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Mpf",rc,0,"Could not write the base.");
		return ConstRC::GeneralError;
	}	
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_TYPE_ATTR,"%s",BAD_CAST XML_SERIAL_MPF_TYPE);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Mpf",rc,0,"Could not write the type.");
		return ConstRC::GeneralError;
	}
	rc = xmlTextWriterEndElement(aWriter);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Mpf",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int CXMLHelper::DeserializeMpf(xmlTextReaderPtr& aReader, const char* aMessage, mpf_t aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpf",0,0,"No attributes of mpf, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_TYPE_ATTR);
	if (attr == NULL)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpf",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!xmlStrEqual(attr, BAD_CAST XML_SERIAL_MPF_TYPE))
	{
		xmlFree(attr);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpf",0,0,"Data type not mpf, aborting");
		return ConstRC::GeneralError;
	}
	xmlFree(attr);

	int base = 10;
	attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_BASE_ATTR);  
	if (attr == NULL)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpf",0,0,"Base not found, aborting");			
		return ConstRC::GeneralError;
	}
	
	int rc = ParseDecimalInt(attr, base);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpf",0,0,"Could not parse base int");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);

	if (base == 10)
	{
		attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_VALUE_ATTR);  
		if (attr == NULL)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpf",0,0,"Value not found, aborting");			
			return ConstRC::GeneralError;	
		}

		std::string auxStr((char*)attr);
		auxStr.append("e");

		attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_EXPONENT_ATTR);  
		if (attr == NULL)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpf",0,0,"Exp not found, aborting");			
			return ConstRC::GeneralError;
		}

		auxStr.append((char*)attr);		
		xmlFree(attr);
		attr = NULL;

		mpf_set_str(aTarget, auxStr.c_str(), base);
	} 
	else 
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpf",0,0,"Cannot parse nondecimal mpf's yet");			
		return ConstRC::GeneralError;	
	}

	if (aInfoMode)
	{
		const xmlChar* name = xmlTextReaderConstName(aReader);
		gmp_printf("%s: Value of %s is %.*Ff.\n", aMessage, name, 10, aTarget);
	}

	return ConstRC::Ok;
}


int CXMLHelper::SerializeText(xmlTextWriterPtr& aWriter, const char* aTagName, const char* aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Text ",0,0,"Insufficient data.");
			return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
		if (rc < 0) {
			CLog::LogError(WHEREARG,"XML Helper","Serialize Text ",rc,0,"Could not write the serializator tag.");
			return ConstRC::GeneralError;
		}	
	rc = xmlTextWriterWriteString(aWriter, BAD_CAST aValue);
		if (rc < 0) {
			CLog::LogError(WHEREARG,"XML Helper","Serialize Text ",rc,0,"Could not write the string value.");
			return ConstRC::GeneralError;
		}	
	rc = xmlTextWriterEndElement(aWriter);
		if (rc < 0) {
			CLog::LogError(WHEREARG,"XML Helper","Serialize Text",rc,0,"Could not end the tag.");
			return ConstRC::GeneralError;
		}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}
int CXMLHelper::SerializeText(xmlTextWriterPtr& aWriter, const char* aTagName, const std::string& aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Text ",0,0,"Insufficient data.");
		return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Text ",rc,0,"Could not write the serializator tag.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteString(aWriter, BAD_CAST aValue.c_str());
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Text ",rc,0,"Could not write the string value.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterEndElement(aWriter);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Text",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}

	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}
int CXMLHelper::DeserializeText(xmlTextReaderPtr& aReader, const char* aMessage, char*& aTarget, bool aInfoMode)
{
	// Makes a new read
	int ret = xmlTextReaderRead(aReader);		
	if (ret != 1)
	{
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderReadString(aReader);  
	if (value)
	{
		// really dangerous!!!!!!
		strcpy(aTarget,(char*) value);
	}
	if (aInfoMode)
	{
		const xmlChar* name = xmlTextReaderConstName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
	}	
	xmlFree(value);	
	return ConstRC::Ok;
}


int CXMLHelper::DeserializeText(xmlTextReaderPtr& aReader, const char* aMessage, std::string& aTarget, bool aInfoMode)
{
	// Makes a new read
	int ret = xmlTextReaderRead(aReader);		
	if (ret != 1)
	{
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderReadString(aReader);  
	if (value)
	{
		aTarget.assign((char*) value);
	} 
	else
	{
		aTarget.assign("");
	}
	if (aInfoMode)
	{
		const xmlChar* name = xmlTextReaderConstName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
	}	
	xmlFree(value);	
	return ConstRC::Ok;
}








int CXMLHelper::SerializeBoolean(xmlTextWriterPtr &aWriter, const char *aTagName, const bool &aValue)
{
	if (!aTagName)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Boolean",0,0,"Insufficient data.");
		return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Boolean",rc,0,"Could not write the serializator tag.");
		return ConstRC::GeneralError;
	}	

	const char * value = aValue ? TRUE_STRING : FALSE_STRING;
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%s",value);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Boolean",rc,0,"Could not write the value.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST BOOLEAN_XML);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Boolean",rc,0,"Could not write the type.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterEndElement(aWriter);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Boolean",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}
int CXMLHelper::DeserializeBoolean(xmlTextReaderPtr &aReader, const char *aMessage, bool &aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Boolean",0,0,"No attributes of unsigned long, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Boolean",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!xmlStrEqual(type,BAD_CAST BOOLEAN_XML))
	{
		xmlFree(type);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Boolean",0,0,"Data type not boolean, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);  
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Boolean",0,0,"Value not found, aborting");			
		xmlFree(type);
		return ConstRC::GeneralError;	
	}
	
	aTarget = xmlStrEqual(value, BAD_CAST TRUE_STRING) == 1;

	if (aInfoMode)
	{
		const xmlChar* name = xmlTextReaderConstName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
	}
	xmlFree(type);
	xmlFree(value);
	return ConstRC::Ok;
}




int CXMLHelper::SerializeSievingElement(xmlTextWriterPtr &aWriter, const char *aTagName, const nfs_sieving_element &aValue)
{
	if (!aTagName)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize SievingElement",0,0,"Insufficient data.");
		return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize SievingElement",rc,0,"Could not write the serializator tag.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST SIEVING_ELEMENT);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize SievingElement",rc,0,"Could not write the type.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_PRIME_ATTR,"%d",aValue.prime);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize SievingElement",rc,0,"Could not write the prime.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_EXPONENT_ATTR,"%u",aValue.exponent);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize SievingElement",rc,0,"Could not write the exponent.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_ROOT_ATTR,"%d",aValue.root);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize SievingElement",rc,0,"Could not write the root.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_FLAGS_ATTR,"%u",aValue.flags);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize SievingElement",rc,0,"Could not write the flags.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterEndElement(aWriter);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize SievingElement",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int CXMLHelper::DeserializeSievingElement(xmlTextReaderPtr &aReader, const char *aMessage, nfs_sieving_element &aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize SievingElement",0,0,"No attributes of sieving element, aborting");
		return ConstRC::GeneralError;
	}

	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize SievingElement",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!xmlStrEqual(type, BAD_CAST SIEVING_ELEMENT))
	{
		xmlFree(type);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize SievingElement",0,0,"Data type not sieving element, aborting");
		return ConstRC::GeneralError;
	}
	xmlFree(type);
	type = NULL;
	int rc;

	xmlChar* attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_PRIME_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize SievingElement",0,0,"Prime not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalInt(attr, aTarget.prime);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize SievingElement",0,0,"Could not parse int");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);

	attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_EXPONENT_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize SievingElement",0,0,"Exponent not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalUnsignedIntOrLong(attr, aTarget.exponent);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize SievingElement",0,0,"Could not parse uint");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);

	attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_ROOT_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize SievingElement",0,0,"Root not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalInt(attr, aTarget.root);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize SievingElement",0,0,"Could not parse int");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);

	attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_FLAGS_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize SievingElement",0,0,"Flags not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalUnsignedIntOrLong(attr, aTarget.flags);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize SievingElement",0,0,"Could not parse uint");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);
	attr = NULL;

	if (aInfoMode)
	{
		const xmlChar* name = xmlTextReaderConstName(aReader);
		printf("%s: %s - prime: %d, exponent: %u, root: %d, flags: %u.\n",aMessage,name,aTarget.prime, aTarget.exponent, aTarget.root, aTarget.flags);
	}

	return ConstRC::Ok;
}
int CXMLHelper::SerializeFBElement(xmlTextWriterPtr &aWriter, const char *aTagName, const fb_element &aValue)
{
	if (!aTagName)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize FBElement",0,0,"Insufficient data.");
		return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize FBElement",rc,0,"Could not write the serializator tag.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST FB_ELEMENT);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize FBElement",rc,0,"Could not write the type.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_PRIME_ATTR,"%ld",aValue.prime);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize FBElement",rc,0,"Could not write the prime.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_ROOT_ATTR,"%ld",aValue.root);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize FBElement",rc,0,"Could not write the root.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_INVERSION_OF_ROOT_ATTR,"%ld",aValue.inversion_of_root);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize FBElement",rc,0,"Could not write the inversion of root.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_LOG_ATTR,"%u",aValue.log);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize FBElement",rc,0,"Could not write the log.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_FLAGS_ATTR,"%u",aValue.flags);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize FBElement",rc,0,"Could not write the flags.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterEndElement(aWriter);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize FBElement",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}
int CXMLHelper::DeserializeFBElement(xmlTextReaderPtr &aReader, const char *aMessage, fb_element &aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize FBElement",0,0,"No attributes of fb element, aborting");
		return ConstRC::GeneralError;
	}

	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize FBElement",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!xmlStrEqual(type, BAD_CAST FB_ELEMENT))
	{
		xmlFree(type);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize FBElement",0,0,"Data type not fb element, aborting");
		return ConstRC::GeneralError;
	}
	xmlFree(type);
	type = NULL;
	int rc;
	long auxLong;

	xmlChar* attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_PRIME_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize FBElement",0,0,"Prime not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalLong(attr, auxLong);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize FBElement",0,0,"Could not parse long");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);
	aTarget.prime = (main_sieving_type)auxLong;

	attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_ROOT_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize FBElement",0,0,"Root not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalLong(attr, auxLong);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize FBElement",0,0,"Could not parse long");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);
	aTarget.root = (main_sieving_type)auxLong;

	attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_INVERSION_OF_ROOT_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize FBElement",0,0,"Inversion of root not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalLong(attr, auxLong);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize FBElement",0,0,"Could not parse long");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);
	aTarget.inversion_of_root = (main_sieving_type)auxLong;

	attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_LOG_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize FBElement",0,0,"Log not found, aborting");			
		return ConstRC::GeneralError;	
	}
	unsigned int temp;
	rc = ParseDecimalUnsignedIntOrLong(attr, temp);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize FBElement",0,0,"Could not parse uint");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	aTarget.log = (log_type)temp;
	xmlFree(attr);
	attr = NULL;

	attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_FLAGS_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize FBElement",0,0,"Flags not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalUnsignedIntOrLong(attr, aTarget.flags);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize FBElement",0,0,"Could not parse uint");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);
	attr = NULL;

	if (aInfoMode)
	{
		const xmlChar* name = xmlTextReaderConstName(aReader);
		printf("%s: %s - prime: %d, root: %d, inversion of root: %d, log: %u, flags: %u.\n",
			aMessage,name,aTarget.prime, aTarget.root, aTarget.inversion_of_root, aTarget.log, aTarget.flags);
	}

	return ConstRC::Ok;
}
int CXMLHelper::SerializePrimeIdealForLegendre(xmlTextWriterPtr &aWriter, const char *aTagName, const prime_ideal_for_legendre &aValue)
{
	if (!aTagName)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize PrimeIdealForLegendre",0,0,"Insufficient data.");
		return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize PrimeIdealForLegendre",rc,0,"Could not write the serializator tag.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST PRIME_IDEAL_FOR_LEGENDRE);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize PrimeIdealForLegendre",rc,0,"Could not write the type.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_PRIME_ATTR,"%d",aValue.prime);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize PrimeIdealForLegendre",rc,0,"Could not write the prime.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_C_P_ATTR,"%d",aValue.c_p);	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize PrimeIdealForLegendre",rc,0,"Could not write the c_p.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterEndElement(aWriter);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize PrimeIdealForLegendre",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}
int CXMLHelper::DeserializePrimeIdealForLegendre(xmlTextReaderPtr &aReader, const char *aMessage, prime_ideal_for_legendre &aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize PrimeIdealForLegendre",0,0,"No attributes of prime ideal for legendre, aborting");
		return ConstRC::GeneralError;
	}

	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize PrimeIdealForLegendre",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!xmlStrEqual(type, BAD_CAST PRIME_IDEAL_FOR_LEGENDRE))
	{
		xmlFree(type);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize PrimeIdealForLegendre",0,0,"Data type not prime ideal for legendre, aborting");
		return ConstRC::GeneralError;
	}
	xmlFree(type);
	type = NULL;
	int rc;

	xmlChar* attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_PRIME_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize PrimeIdealForLegendre",0,0,"Prime not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalInt(attr, aTarget.prime);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize PrimeIdealForLegendre",0,0,"Could not parse int");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);

	attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_C_P_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize PrimeIdealForLegendre",0,0,"C_p not found, aborting");			
		return ConstRC::GeneralError;	
	}
	rc = ParseDecimalInt(attr, aTarget.c_p);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize PrimeIdealForLegendre",0,0,"Could not parse int");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);
	attr = NULL;

	if (aInfoMode)
	{
		const xmlChar* name = xmlTextReaderConstName(aReader);
		printf("%s: %s - prime: %d, c_p: %d\n",	aMessage, name, aTarget.prime, aTarget.c_p);
	}

	return ConstRC::Ok;
}
int CXMLHelper::SerializeHashtableEntryType1(xmlTextWriterPtr &aWriter, const char *aTagName, const CNFS_HashEntry1 & aValue)
{
	if (!aTagName)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize HashtableEntryType1",0,0,"Insufficient data.");
		return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize HashtableEntryType1",rc,0,"Could not write the serializator tag.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST XML_SERIAL_HASHTABLE_ENTRY_TYPE1_TYPE);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize HashtableEntryType1",rc,0,"Could not write the type.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_VALUE_ATTR,"%d",aValue.GetValue());	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize HashtableEntryType1",rc,0,"Could not write the value.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_ROOT_ATTR,"%d",aValue.GetRootFingerprint());	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize HashtableEntryType1",rc,0,"Could not write the root fingerprint.");
		return ConstRC::GeneralError;
	}

	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST XML_SERIAL_ANCESTOR_ATTR,"%d",aValue.GetAncestor());	
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize HashtableEntryType1",rc,0,"Could not write the ancestor.");
		return ConstRC::GeneralError;
	}	

	rc = xmlTextWriterEndElement(aWriter);
	if (rc < 0) 
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize HashtableEntryType1",rc,0,"Could not end the tag.");
		return ConstRC::GeneralError;
	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}
int CXMLHelper::DeserializeHashtableEntryType1(xmlTextReaderPtr &aReader, const char *aMessage, CNFS_HashEntry1 &aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize HashtableEntryType1",0,0,"No attributes of prime ideal for legendre, aborting");
		return ConstRC::GeneralError;
	}

	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize HashtableEntryType1",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!xmlStrEqual(type, BAD_CAST XML_SERIAL_HASHTABLE_ENTRY_TYPE1_TYPE))
	{
		xmlFree(type);
		CLog::LogError(WHEREARG,"XML Helper","Deserialize HashtableEntryType1",0,0,"Data type not hashtable entry type 1, aborting");
		return ConstRC::GeneralError;
	}
	xmlFree(type);
	type = NULL;
	int rc;

	xmlChar* attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_VALUE_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize HashtableEntryType1",0,0,"Value not found, aborting");			
		return ConstRC::GeneralError;	
	}
	large_prime_type value;
	rc = ParseDecimalUnsignedIntOrLong(attr, value);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize HashtableEntryType1",0,0,"Could not parse uint");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);
	aTarget.SetValue(value);

	attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_ROOT_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize HashtableEntryType1",0,0,"Root not found, aborting");			
		return ConstRC::GeneralError;	
	}
	large_prime_type root_fingerprint;
	rc = ParseDecimalUnsignedIntOrLong(attr, root_fingerprint);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize HashtableEntryType1",0,0,"Could not parse uint");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	aTarget.SetRootFingerprint(root_fingerprint);
	xmlFree(attr);

	attr = xmlTextReaderGetAttribute(aReader, BAD_CAST XML_SERIAL_ANCESTOR_ATTR);  
	if (!attr)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize HashtableEntryType1",0,0,"Ancestor not found, aborting");			
		return ConstRC::GeneralError;	
	}
	unsigned int ancestor;
	rc = ParseDecimalUnsignedIntOrLong(attr, ancestor);
	aTarget.SetAncestor(ancestor);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize HashtableEntryType1",0,0,"Could not parse uint");			
		xmlFree(attr);
		return ConstRC::GeneralError;	
	}
	xmlFree(attr);

	attr = NULL;

	if (aInfoMode)
	{
		const xmlChar* name = xmlTextReaderConstName(aReader);
		printf("%s: %s - value: %d, c_p: %d, ancestor: %d\n",	aMessage, name, aTarget.GetValue(), aTarget.GetRootFingerprint(), aTarget.GetAncestor());
	}

	return ConstRC::Ok;
}


