#ifndef __XML_HELPER_H__
#define __XML_HELPER_H__

#include <libxml/encoding.h>
#include <libxml/xmlwriter.h>
#include <libxml/xmlreader.h>
#include <libxml/chvalid.h>
#include "nfs_definitions.h"
#include "xml_definitions.h"
#include "common_utils.h"
#include <gmp.h>
#include <time.h>
#include "enums.h"
#include "structures.h"
#include "common_xml_helper.h"
#include "xml_helper.h"
#include "lp_hashtable_type1.h"

/*!
 * \brief Serializator/Deserializator for NFS
 */
class CXMLHelper : public CommonXMLHelper 
{

public:

	static int SerializeMpf(xmlTextWriterPtr& aWriter, const char* aTagName, const mpf_t aValue);
	static int DeserializeMpf(xmlTextReaderPtr& aReader, const char* aMessage, mpf_t aTarget, bool aInfoMode);

	static int SerializeText(xmlTextWriterPtr& aWriter, const char* aTagName, const char* aValue);
	static int SerializeText(xmlTextWriterPtr& aWriter, const char* aTagName, const std::string& aValue);
	static int DeserializeText(xmlTextReaderPtr& aReader, const char* aMessage, char*& aTarget, bool aInfoMode);
	static int DeserializeText(xmlTextReaderPtr& aReader, const char* aMessage, std::string& aTarget, bool aInfoMode);


	static int SerializeBoolean(xmlTextWriterPtr& aWriter, const char* aTagName, const bool& aValue);
	static int DeserializeBoolean(xmlTextReaderPtr& aReader, const char* aMessage, bool& aTarget, bool aInfoMode);

	static int SerializeSievingElement(xmlTextWriterPtr& aWriter, const char* aTagName, const nfs_sieving_element& aValue);
	static int DeserializeSievingElement(xmlTextReaderPtr& aReader, const char* aMessage, nfs_sieving_element& aTarget, bool aInfoMode);

	static int SerializeFBElement(xmlTextWriterPtr& aWriter, const char* aTagName, const fb_element& aValue);
	static int DeserializeFBElement(xmlTextReaderPtr& aReader, const char* aMessage, fb_element& aTarget, bool aInfoMode);

	static int SerializePrimeIdealForLegendre(xmlTextWriterPtr& aWriter, const char* aTagName, const prime_ideal_for_legendre& aValue);
	static int DeserializePrimeIdealForLegendre(xmlTextReaderPtr& aReader, const char* aMessage, prime_ideal_for_legendre& aTarget, bool aInfoMode);

	static int SerializeHashtableEntryType1(xmlTextWriterPtr& aWriter, const char* aTagName, const CNFS_HashEntry1& aValue);
	static int DeserializeHashtableEntryType1(xmlTextReaderPtr& aReader, const char* aMessage, CNFS_HashEntry1& aTarget, bool aInfoMode);

};

#endif // __XML_HELPER_H__

