#include "xml_helper_ks.h"
#include "constants.h"
#include "const_return_codes.h"
#include "log.h"
#include "xml_nfs_definitions.h"

int ParseDecimalDouble(xmlChar* aString, double& aTarget)
{
	if (aString == NULL) return ConstRC::GeneralError;
	int len = xmlStrlen(aString);
	if (!len) return ConstRC::GeneralError;
	aTarget = 0;
	bool negative = (aString[0] == '-');
	int i,start = 0;
	if (negative)
	{
		start = 1;
	}
	xmlChar char1;
	//printf("XML Helper, parse decimal float. Input string is %s of length %d, start is %d.\n",aString,len,start);
	int decimal_point_position = -1;
	for (i=start; i< len; i++)
	{
		//printf("For cycle, i = %d.\n",i);
		char1 = aString[i];
		//printf("At index %d is char %c.\n",i,char1);
		if (char1 == '.')
		{
			decimal_point_position = (len-1) - i;
			//printf("Found decimal point, used pos is %d.\n",decimal_point_position);
			continue;
		}
		if (! xmlIsDigit_ch(char1))
		{
			printf("Parse decimal double: char %c on index %d of %s is not a digit!\n",char1,i,aString);
			return ConstRC::GeneralError;
		}
		int diff = char1 - '0';
		aTarget *= 10;
		aTarget += diff;
		//printf("Current intermediate result is %f.\n",aTarget);
	}
	if (negative)
	{
		aTarget *= -1;
	}
	if (decimal_point_position != -1)
	{
		for (i=0; i< decimal_point_position; i++)
		{
			aTarget /= 10;
		}
	}
	//printf("Returning %f.\n",aTarget);
	return ConstRC::Ok;
}

int ParseDecimalMpz(xmlChar* aString, mpz_t aTarget)
{
	if (aString == NULL) return ConstRC::GeneralError;
	int len = xmlStrlen(aString);
	if (!len) return ConstRC::GeneralError;
	mpz_set_ui(aTarget,0);
	bool negative = (aString[0] == '-');
	int i,start = 0 + 1*(negative == TRUE);
	xmlChar char1;
	//printf("XML Helper, parse decimal mpz. Input string is %s of length %d.\n",aString,len);
	for (i=start; i< len;i++)
	{
		char1 = aString[i];
		//printf("At index %d is char %c.\n",i,char1);
		if (! xmlIsDigit_ch(char1))
		{
			printf("Parse decimal mpz: char %c on index %d of %s is not a digit!\n",char1,i,aString);
			return ConstRC::GeneralError;
		}
		int diff = char1 - '0';
		mpz_mul_ui(aTarget,aTarget,10);
		mpz_add_ui(aTarget,aTarget,diff);
		//gmp_printf("Current intermediate result is %Zd.\n",aTarget);
	}
	if (negative)
	{
		mpz_mul_si(aTarget,aTarget,-1);
	}
	//gmp_printf("Returning %Zd.\n",aTarget);
	return ConstRC::Ok;
}

int ParseDecimalInt(xmlChar* aString, int& aTarget)
{
	if (aString == NULL) return ConstRC::GeneralError;
	int len = xmlStrlen(aString);
	if (!len) return ConstRC::GeneralError;
	aTarget = 0;
	bool negative = (aString[0] == '-');
	int i,start = 0 + 1*(negative == TRUE);
	xmlChar char1;
// 	printf("XML Helper, parse decimal int. Input string is %s of length %d.\n",aString,len);
	for (i=start; i< len; i++)
	{
		char1 = aString[i];
// 		printf("At index %d is char %c.\n",i,char1);
		if (! xmlIsDigit_ch(char1))
		{
			printf("Parse decimal int: char %c on index %d of %s is not a digit!\n",char1,i,aString);
			return ConstRC::GeneralError;
		}
		int diff = char1 - '0';
		aTarget *= 10;
		aTarget += diff;
// 		printf("Current intermediate result is %d.\n",aTarget);
	}
	if (negative)
	{
		aTarget *= -1;
	}
// 	printf("Returning %d.\n",aTarget);
	return ConstRC::Ok;
}

int ParseDecimalUnsignedInt(xmlChar* aString, unsigned int& aTarget)
{
	if (aString == NULL) return ConstRC::GeneralError;
	int len = xmlStrlen(aString);
	if (!len) return ConstRC::GeneralError;
	aTarget = 0;
	int i,start = 0;
	xmlChar char1;
// 	printf("XML Helper, parse decimal int. Input string is %s of length %d.\n",aString,len);
	for (i=start; i< len; i++)
	{
		char1 = aString[i];
// 		printf("At index %d is char %c.\n",i,char1);
		if (! xmlIsDigit_ch(char1))
		{
			printf("Parse decimal unsigned int: char %c on index %d of %s is not a digit!\n",char1,i,aString);
			return ConstRC::GeneralError;
		}
		int diff = char1 - '0';
		aTarget *= 10;
		aTarget += diff;
// 		printf("Current intermediate result is %d.\n",aTarget);
	}
// 	printf("Returning %d.\n",aTarget);
	return ConstRC::Ok;
}

int ParseDecimalLong(xmlChar* aString, long& aTarget)
{
	if (aString == NULL) return ConstRC::GeneralError;
	int len = xmlStrlen(aString);
	if (!len) return ConstRC::GeneralError;
	aTarget = 0;
	bool negative = (aString[0] == '-');
	int i,start = 0 + 1*(negative == TRUE);
	xmlChar char1;
// 	printf("XML Helper, parse decimal long. Input string is %s of length %d.\n",aString,len);
	for (i=start; i< len; i++)
	{
		char1 = aString[i];
// 		printf("At index %d is char %c.\n",i,char1);
		if (! xmlIsDigit_ch(char1))
		{
			printf("Parse decimal long: char %c on index %d of %s is not a digit!\n",char1,i,aString);
			return ConstRC::GeneralError;
		}
		int diff = char1 - '0';
		aTarget *= 10;
		aTarget += diff;
// 		printf("Current intermediate result is %ld.\n",aTarget);
	}
	if (negative)
	{
		aTarget *= -1;
	}
// 	printf("Returning %ld.\n",aTarget);
	return ConstRC::Ok;
}

int NewLine(xmlTextWriterPtr& aWriter, bool aIndent)
{
	int rc =  xmlTextWriterWriteString(aWriter, BAD_CAST "\n");
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","New Line",rc,0,"Could not write new line.");
        	return ConstRC::GeneralError;
    	}
	return ConstRC::Ok;
}

int DeserializeTime(xmlTextReaderPtr& aReader, const char* aMessage, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize time",0,0,"No attributes of time, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* sec = xmlTextReaderGetAttribute(aReader, BAD_CAST SECONDS);
	if (!sec)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize time",0,0,"No seconds attribute, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* out = xmlTextReaderGetAttribute(aReader, BAD_CAST FORMATTED);
	if (!out)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize time",0,0,"No formatted string specified");
		int decimalTime;
		int res = ParseDecimalInt(sec,decimalTime);
		if (res == ConstRC::Ok)
		{
			time_t what = (time_t) decimalTime;
			out = BAD_CAST ctime(&what);
		}
	}
	if (aInfoMode)
	{
		printf("%s: Time %s.\n",aMessage,out);
	}
	xmlFree(sec);
	xmlFree(out);
	return ConstRC::Ok;
}

int SerializeTime(xmlTextWriterPtr& aWriter)
{
	return SerializeTime(aWriter, TIME_NAME, time(NULL));
}

int SerializeTime(xmlTextWriterPtr& aWriter, const char* aTag, time_t aTime)
{
	char*  formattedTime = ctime(&aTime);
	int len = strlen(formattedTime);
	// At some Unices, formatted time is ended by new line. We do not want this, it looks ugly.
	if (formattedTime[len-1] == '\n')
	{
		formattedTime[len-1] = 0;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTag);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Time",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST SECONDS,"%ld",aTime);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Time",rc,0,"Could not write the serializator name.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST FORMATTED,"%s",BAD_CAST formattedTime);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Time",rc,0,"Could not write the serializator version.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Time",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int DeserializeMpz(xmlTextReaderPtr& aReader, const char* aMessage, mpz_t aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpz",0,0,"No attributes of mpz, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpz",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!(type = xmlStrcat(type,BAD_CAST MPZ)))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpz",0,0,"Data type not mpz, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpz",0,0,"Value not found, aborting");
		xmlFree(type);
		return ConstRC::GeneralError;
	}
	int base = 10;
	xmlChar* baseChar = xmlTextReaderGetAttribute(aReader, BAD_CAST BASE);
	if (baseChar)
	{
		int rc = ParseDecimalInt(baseChar,base);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpz",0,0,"Could not parse base int");
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;
		}
	}
	if (base == 10)
	{
		int rc = ParseDecimalMpz(value,aTarget);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpz",0,0,"Could not parse value mpz");
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;
		}
	}
	else {
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Mpz",0,0,"Cannot parse nondecimal mpz's yet");
		xmlFree(type);
		xmlFree(value);
		xmlFree(baseChar);
		return ConstRC::GeneralError;
	}
	if (aInfoMode)
	{
		xmlChar* name = xmlTextReaderName(aReader);
		int decLen = mpz_sizeinbase(aTarget,10);
		int binLen = mpz_sizeinbase(aTarget,2);
		printf("%s: Value of %s is %s (%d dec, %d bin digits).\n",aMessage,name,value,decLen,binLen);
		xmlFree(name);
	}
	xmlFree(type);
	xmlFree(value);
	xmlFree(baseChar);
	return ConstRC::Ok;
}

int DeserializeSignedInt(xmlTextReaderPtr& aReader, const char* aMessage, int& aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Int",0,0,"No attributes of mpz, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Int",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!(type = xmlStrcat(type,BAD_CAST SIGNED_INT)))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Int",0,0,"Data type not mpz, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Int",0,0,"Value not found, aborting");
		return ConstRC::GeneralError;
	}
	int base = 10;
	xmlChar* baseChar = xmlTextReaderGetAttribute(aReader, BAD_CAST BASE);
	if (baseChar)
	{
		int rc = ParseDecimalInt(baseChar,base);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Int",0,0,"Could not parse base int");
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;
		}
	}
	if (base == 10)
	{
		int rc = ParseDecimalInt(value,aTarget);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Int",0,0,"Could not parse value");
			xmlFree(value);
			xmlFree(type);
			xmlFree(baseChar);
		return ConstRC::GeneralError;
		}
	}
	else {
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Int",0,0,"Cannot parse nondecimal integers yet");
		xmlFree(value);
		xmlFree(type);
		xmlFree(baseChar);
		return ConstRC::GeneralError;
	}
	if (aInfoMode)
	{
		xmlChar* name = xmlTextReaderName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
		xmlFree(name);
	}
	xmlFree(type);
	xmlFree(value);
	xmlFree(baseChar);
	return ConstRC::Ok;
}

int DeserializeUnsignedInt(xmlTextReaderPtr& aReader, const char* aMessage, unsigned int& aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Int",0,0,"No attributes of mpz, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Int",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!(type = xmlStrcat(type,BAD_CAST UNSIGNED_INT)))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Int",0,0,"Data type not mpz, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Int",0,0,"Value not found, aborting");
		xmlFree(type);
		return ConstRC::GeneralError;
	}
	int base = 10;
	xmlChar* baseChar = xmlTextReaderGetAttribute(aReader, BAD_CAST BASE);
	if (baseChar)
	{
		int rc = ParseDecimalInt(baseChar,base);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Int",0,0,"Could not parse base int");
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;
		}
	}
	if (base == 10)
	{
		int rc = ParseDecimalUnsignedInt(value,aTarget);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Int",0,0,"Could not parse value");
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;
		}
	}
	else {
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Unsigned Int",0,0,"Cannot parse nondecimal integers yet");
		xmlFree(type);
		xmlFree(value);
		xmlFree(baseChar);
		return ConstRC::GeneralError;
	}
	if (aInfoMode)
	{
		xmlChar* name = xmlTextReaderName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
	}
	xmlFree(type);
	xmlFree(value);
	xmlFree(baseChar);
	return ConstRC::Ok;
}

int DeserializeSignedLong(xmlTextReaderPtr& aReader, const char* aMessage, long& aTarget, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Long",0,0,"No attributes of mpz, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* type = xmlTextReaderGetAttribute(aReader, BAD_CAST TYPE);
	if (!type)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Long",0,0,"Data type not specified, aborting");
		return ConstRC::GeneralError;
	}
	if (!(type = xmlStrcat(type,BAD_CAST SIGNED_INT)))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Long",0,0,"Data type not mpz, aborting");
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Long",0,0,"Value not found, aborting");
		xmlFree(type);
		return ConstRC::GeneralError;
	}
	int base = 10;
	xmlChar* baseChar = xmlTextReaderGetAttribute(aReader, BAD_CAST BASE);
	if (baseChar)
	{
		int rc = ParseDecimalInt(baseChar,base);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Long",0,0,"Could not parse base int");
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;
		}
	}
	if (base == 10)
	{
		int rc = ParseDecimalLong(value,aTarget);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Long",0,0,"Could not parse value");
			xmlFree(type);
			xmlFree(value);
			xmlFree(baseChar);
			return ConstRC::GeneralError;
		}
	}
	else {
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Signed Long",0,0,"Cannot parse nondecimal longs yet");
		xmlFree(type);
		xmlFree(value);
		xmlFree(baseChar);
		return ConstRC::GeneralError;
	}
	if (aInfoMode)
	{
		xmlChar* name = xmlTextReaderName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
		xmlFree(name);
	}
	xmlFree(type);
	xmlFree(value);
	free(baseChar);
	return ConstRC::Ok;
}

int SerializeMpz(xmlTextWriterPtr& aWriter, const char* aTagName, /*const char* aAttributeName,*/ mpz_t aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Mpz",0,0,"Insufficient data.");
        	return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Mpz",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	/*rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST NAME ,"%s",BAD_CAST aAttributeName);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize mpz",rc,0,"Could not write the name.");
        	return ConstRC::GeneralError;
    	}*/
	char* mpzChar = NULL;
	int base = 10;
	mpzChar = mpz_get_str(mpzChar,base,aValue);
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%s",BAD_CAST mpzChar);
	free(mpzChar);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Mpz",rc,0,"Could not write the value.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST BASE,"%d", base);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Mpz",rc,0,"Could not write the base.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST MPZ);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Mpz",rc,0,"Could not write the type.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Mpz",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int SerializePolynomialCoefficient(xmlTextWriterPtr& aWriter, mpz_t aValue, int aDegree)
{
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST COEFFICIENT_NAME);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Polynomial Coefficent",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	int base = 10;
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST POWER,"%d",aDegree);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Polynomial Coefficient",rc,0,"Could not write the value.");
        	return ConstRC::GeneralError;
    	}
	char* mpzChar = NULL;
	mpzChar = mpz_get_str(mpzChar,base,aValue);
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%s",BAD_CAST mpzChar);
	free(mpzChar);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Polynomial Coefficient",rc,0,"Could not write the value.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Polynomial Coefficient",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int DeserializePolynomialCoefficient(xmlTextReaderPtr& aReader, const char* aMessage, mpz_t aTargetValue, int& aTargetDegree, bool aInfoMode)
{
	if (!xmlTextReaderHasAttributes(aReader))
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Polynomial Coefficient",0,0,"No attributes, aborting");
		return ConstRC::GeneralError;
	}
	int base = 10;
	xmlChar* degree = xmlTextReaderGetAttribute(aReader, BAD_CAST POWER);
	if (!degree)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Polynomial Coefficient",0,0,"Degree not specified, aborting");
		return ConstRC::GeneralError;
	}
	int result;
	int rc = ParseDecimalInt(degree,result);
	if (rc != ConstRC::Ok)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Polynomial Coefficient",0,0,"Could not degree int");
		return ConstRC::GeneralError;
	}
	aTargetDegree = result;
	xmlChar* value = xmlTextReaderGetAttribute(aReader, BAD_CAST VALUE);
	if (!value)
	{
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Polynomial Coefficient",0,0,"Value not found, aborting");
		return ConstRC::GeneralError;
	}
	if (base == 10)
	{
		int rc = ParseDecimalMpz(value,aTargetValue);
		if (rc != ConstRC::Ok)
		{
			CLog::LogError(WHEREARG,"XML Helper","Deserialize Polynomial Coefficient",0,0,"Could not parse value mpz");
			return ConstRC::GeneralError;
		}
	}
	else {
		CLog::LogError(WHEREARG,"XML Helper","Deserialize Polynomial Coefficient",0,0,"Cannot parse nondecimal mpz's yet");
		return ConstRC::GeneralError;
	}
	if (aInfoMode)
	{
		xmlChar* name = xmlTextReaderName(aReader);
		printf("%s: Value of poly coefficient at %d is %s.\n",aMessage,aTargetDegree,value);
		xmlFree(name);
	}
	xmlFree(degree);
	xmlFree(value);
	return ConstRC::Ok;
}

int SerializeSignedInt(xmlTextWriterPtr& aWriter, const char* aTagName, /*const char* aAttributeName,*/ int aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",0,0,"Insufficient data.");
        	return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	/*rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST NAME ,"%s",BAD_CAST aAttributeName);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize mpz",rc,0,"Could not write the name.");
        	return ConstRC::GeneralError;
    	}*/
	int base = 10;
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%d",aValue);

	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the value.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST BASE,"%d", base);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the base.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST SIGNED_INT);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the type.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Int",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int SerializeDouble(xmlTextWriterPtr& aWriter, const char* aTagName, /*const char* aAttributeName,*/ double aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",0,0,"Insufficient data.");
        	return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	/*rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST NAME ,"%s",BAD_CAST aAttributeName);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize mpz",rc,0,"Could not write the name.");
        	return ConstRC::GeneralError;
    	}*/
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%f",aValue);

	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the value.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST DOUBLE);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the type.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Int",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int SerializeUnsignedInt(xmlTextWriterPtr& aWriter, const char* aTagName, /*const char* aAttributeName,*/ unsigned int aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",0,0,"Insufficient data.");
        	return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	/*rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST NAME ,"%s",BAD_CAST aAttributeName);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize mpz",rc,0,"Could not write the name.");
        	return ConstRC::GeneralError;
    	}*/
	int base = 10;
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%u",aValue);

	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",rc,0,"Could not write the value.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST BASE,"%u", base);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",rc,0,"Could not write the base.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST UNSIGNED_INT);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Unsigned Int",rc,0,"Could not write the type.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Int",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int SerializeSignedLong(xmlTextWriterPtr& aWriter, const char* aTagName, /*const char* aAttributeName,*/ long aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",0,0,"Insufficient data.");
        	return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	/*rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST NAME ,"%s",BAD_CAST aAttributeName);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize mpz",rc,0,"Could not write the name.");
        	return ConstRC::GeneralError;
    	}*/
	int base = 10;
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VALUE,"%ld",aValue);

	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the value.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST BASE,"%d",base);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the base.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST TYPE,"%s",BAD_CAST SIGNED_LONG);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Signed Int",rc,0,"Could not write the type.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize Int",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int SerializeString(xmlTextWriterPtr& aWriter, const char* aTagName, const char* aValue)
{
	if ((!aTagName)/* || (!aAttributeName)*/)
	{
		CLog::LogError(WHEREARG,"XML Helper","Serialize String ",0,0,"Insufficient data.");
        	return ConstRC::GeneralError;
	}
	int rc =  xmlTextWriterStartElement(aWriter, BAD_CAST aTagName);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize String ",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteString(aWriter, BAD_CAST aValue);
		if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize String ",rc,0,"Could not write the string value.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Serialize String",rc,0,"Could not end the tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc != ConstRC::Ok)
	{
		return ConstRC::Ok;
	}
	return ConstRC::Ok;
}

int DeserializeString(xmlTextReaderPtr& aReader, const char* aMessage, char*& aTarget, bool aInfoMode)
{
	// Makes a new read
	int ret = xmlTextReaderRead(aReader);
	if (ret != 1)
	{
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderValue(aReader);
	if (value)
	{
		strcpy(aTarget,(char*) value);
	}
	if (aInfoMode)
	{
		xmlChar* name = xmlTextReaderName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
		xmlFree(name);
	}
	xmlFree(value);
	return ConstRC::Ok;
}

int DeserializeString(xmlTextReaderPtr& aReader, const char* aMessage, string& aTarget, bool aInfoMode)
{
	// Makes a new read
	int ret = xmlTextReaderRead(aReader);
	if (ret != 1)
	{
		return ConstRC::GeneralError;
	}
	xmlChar* value = xmlTextReaderValue(aReader);
	if (value)
	{
		aTarget.assign((char*) value);
	} else
	{
		aTarget.assign("");
	}
	if (aInfoMode)
	{
		xmlChar* name = xmlTextReaderName(aReader);
		printf("%s: Value of %s is %s.\n",aMessage,name,value);
		xmlFree(name);
	}
	xmlFree(value);
	return ConstRC::Ok;
}


int WriteSerializatorData(xmlTextWriterPtr& aWriter, const char* aSerializatorNameVal, const char* aSerializatorVersion)
{
	int rc;
	rc = xmlTextWriterStartElement(aWriter, BAD_CAST SERIALIZATOR_VAL);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Write Serializator Data",rc,0,"Could not write the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST NAME,"%s",BAD_CAST aSerializatorNameVal);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Write Serializator Data",rc,0,"Could not write the serializator name.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterWriteFormatAttribute(aWriter, BAD_CAST VERSION_NAME,"%s",BAD_CAST aSerializatorVersion);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Write Serializator Data",rc,0,"Could not write the serializator version.");
        	return ConstRC::GeneralError;
    	}
	rc = xmlTextWriterEndElement(aWriter);
    	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Write Serializator Data",rc,0,"Could not end the serializator tag.");
        	return ConstRC::GeneralError;
    	}
	rc = NewLine(aWriter,FALSE);
	if (rc < 0) {
	        CLog::LogError(WHEREARG,"XML Helper","Write Serializator Data",rc,0,"Could not write new line.");
        	return ConstRC::GeneralError;
    	}
	return ConstRC::Ok;
}
